@@ -29,6 +29,7 @@
 #include "nsThreadUtils.h"  // For nsRunnableMethod and nsRevocableEventPtr
 #include "nsTransitionManager.h"      // For CSSTransition
 #include "PendingAnimationTracker.h"  // For PendingAnimationTracker
+#include "ScrollTimelineAnimationTracker.h"
 
 namespace mozilla::dom {
 
@@ -82,7 +83,9 @@ class MOZ_RAII AutoMutationBatchForAnimation {
 // ---------------------------------------------------------------------------
 
 Animation::Animation(nsIGlobalObject* aGlobal)
-    : DOMEventTargetHelper(aGlobal), mAnimationIndex(sNextAnimationIndex++) {}
+    : DOMEventTargetHelper(aGlobal),
+      mAnimationIndex(sNextAnimationIndex++),
+      mRTPCallerType(aGlobal->GetRTPCallerType()) {}
 
 Animation::~Animation() = default;
 
@@ -90,21 +93,45 @@ Animation::~Animation() = default;
 already_AddRefed<Animation> Animation::ClonePausedAnimation(
     nsIGlobalObject* aGlobal, const Animation& aOther, AnimationEffect& aEffect,
     AnimationTimeline& aTimeline) {
-  RefPtr<Animation> animation = new Animation(aGlobal);
-  // Setup the timing.
-  animation->mTimeline = &aTimeline;
-  const Nullable<TimeDuration> timelineTime =
-      aTimeline.GetCurrentTimeAsDuration();
-  MOZ_ASSERT(!timelineTime.IsNull(), "Timeline not yet set");
-
-  const Nullable<TimeDuration> currentTime = aOther.GetCurrentTimeAsDuration();
-  animation->mHoldTime = currentTime;
-  if (!currentTime.IsNull()) {
-    animation->mPreviousCurrentTime = timelineTime;
+  // FIXME: Bug 1805950: Support printing for scroll-timeline once we resolve
+  // the spec issue.
+  if (aOther.UsingScrollTimeline()) {
+    return nullptr;
   }
 
+  RefPtr<Animation> animation = new Animation(aGlobal);
+
+  // Setup the timeline. We always use document-timeline of the new document,
+  // even if the timeline of |aOther| is null.
+  animation->mTimeline = &aTimeline;
+
+  // Setup the playback rate.
   animation->mPlaybackRate = aOther.mPlaybackRate;
 
+  // Setup the timing.
+  const Nullable<TimeDuration> currentTime = aOther.GetCurrentTimeAsDuration();
+  if (!aOther.GetTimeline()) {
+    // This simulates what we do in SetTimelineNoUpdate(). It's possible to
+    // preserve the progress if the previous timeline is a scroll-timeline.
+    // So for null timeline, it may have a progress and the non-null current
+    // time.
+    if (!currentTime.IsNull()) {
+      animation->SilentlySetCurrentTime(currentTime.Value());
+    }
+    animation->mPreviousCurrentTime = animation->GetCurrentTimeAsDuration();
+  } else {
+    animation->mHoldTime = currentTime;
+    if (!currentTime.IsNull()) {
+      // FIXME: Should we use |timelineTime| as previous current time here? It
+      // seems we should use animation->GetCurrentTimeAsDuration(), per
+      // UpdateFinishedState().
+      const Nullable<TimeDuration> timelineTime =
+          aTimeline.GetCurrentTimeAsDuration();
+      MOZ_ASSERT(!timelineTime.IsNull(), "Timeline not yet set");
+      animation->mPreviousCurrentTime = timelineTime;
+    }
+  }
+
   // Setup the effect's link to this.
   animation->mEffect = &aEffect;
   animation->mEffect->SetAnimation(animation);
@@ -139,14 +166,14 @@ already_AddRefed<Animation> Animation::Constructor(
     const GlobalObject& aGlobal, AnimationEffect* aEffect,
     const Optional<AnimationTimeline*>& aTimeline, ErrorResult& aRv) {
   nsCOMPtr<nsIGlobalObject> global = do_QueryInterface(aGlobal.GetAsSupports());
-  RefPtr<Animation> animation = new Animation(global);
 
   AnimationTimeline* timeline;
+  Document* document =
+      AnimationUtils::GetCurrentRealmDocument(aGlobal.Context());
+
   if (aTimeline.WasPassed()) {
     timeline = aTimeline.Value();
   } else {
-    Document* document =
-        AnimationUtils::GetCurrentRealmDocument(aGlobal.Context());
     if (!document) {
       aRv.Throw(NS_ERROR_FAILURE);
       return nullptr;
@@ -154,6 +181,7 @@ already_AddRefed<Animation> Animation::Constructor(
     timeline = document->Timeline();
   }
 
+  RefPtr<Animation> animation = new Animation(global);
   animation->SetTimelineNoUpdate(timeline);
   animation->SetEffectNoUpdate(aEffect);
 
@@ -236,10 +264,6 @@ void Animation::SetEffectNoUpdate(AnimationEffect* aEffect) {
 }
 
 void Animation::SetTimeline(AnimationTimeline* aTimeline) {
-#ifndef NIGHTLY_BUILD
-  MOZ_ASSERT_UNREACHABLE(
-      "Animation.timeline setter is supported only on nightly");
-#endif
   SetTimelineNoUpdate(aTimeline);
   PostUpdate();
 }
@@ -253,12 +277,68 @@ void Animation::SetTimelineNoUpdate(AnimationTimeline* aTimeline) {
   StickyTimeDuration activeTime =
       mEffect ? mEffect->GetComputedTiming().mActiveTime : StickyTimeDuration();
 
+  const AnimationPlayState previousPlayState = PlayState();
+  const Nullable<TimeDuration> previousCurrentTime = GetCurrentTimeAsDuration();
+  // FIXME: The definition of end time in web-animation-1 is different from that
+  // in web-animation-2, which includes the start time. We are still using the
+  // definition in web-animation-1 here for now.
+  const TimeDuration endTime = TimeDuration(EffectEnd());
+  double previousProgress = 0.0;
+  if (!previousCurrentTime.IsNull() && !endTime.IsZero()) {
+    previousProgress =
+        previousCurrentTime.Value().ToSeconds() / endTime.ToSeconds();
+  }
+
   RefPtr<AnimationTimeline> oldTimeline = mTimeline;
   if (oldTimeline) {
     oldTimeline->RemoveAnimation(this);
   }
 
   mTimeline = aTimeline;
+
+  mResetCurrentTimeOnResume = false;
+
+  if (mEffect) {
+    mEffect->UpdateNormalizedTiming();
+  }
+
+  if (mTimeline && !mTimeline->IsMonotonicallyIncreasing()) {
+    // If "to finite timeline" is true.
+
+    ApplyPendingPlaybackRate();
+    Nullable<TimeDuration> seekTime;
+    if (mPlaybackRate >= 0.0) {
+      seekTime.SetValue(TimeDuration());
+    } else {
+      seekTime.SetValue(TimeDuration(EffectEnd()));
+    }
+
+    switch (previousPlayState) {
+      case AnimationPlayState::Running:
+      case AnimationPlayState::Finished:
+        mStartTime = seekTime;
+        break;
+      case AnimationPlayState::Paused:
+        if (!previousCurrentTime.IsNull()) {
+          mResetCurrentTimeOnResume = true;
+          mStartTime.SetNull();
+          mHoldTime.SetValue(
+              TimeDuration(EffectEnd().MultDouble(previousProgress)));
+        } else {
+          mStartTime = seekTime;
+        }
+        break;
+      case AnimationPlayState::Idle:
+      default:
+        break;
+    }
+  } else if (oldTimeline && !oldTimeline->IsMonotonicallyIncreasing() &&
+             !previousCurrentTime.IsNull()) {
+    // If "from finite timeline" and previous progress is resolved.
+    SetCurrentTimeNoUpdate(
+        TimeDuration(EffectEnd().MultDouble(previousProgress)));
+  }
+
   if (!mStartTime.IsNull()) {
     mHoldTime.SetNull();
   }
@@ -266,7 +346,13 @@ void Animation::SetTimelineNoUpdate(AnimationTimeline* aTimeline) {
   if (!aTimeline) {
     MaybeQueueCancelEvent(activeTime);
   }
+
+  UpdatePendingAnimationTracker(oldTimeline, aTimeline);
+
   UpdateTiming(SeekFlag::NoSeek, SyncNotifyFlag::Async);
+
+  // FIXME: Bug 1799071: Check if we need to add
+  // MutationObservers::NotifyAnimationChanged(this) here.
 }
 
 // https://drafts.csswg.org/web-animations/#set-the-animation-start-time
@@ -297,6 +383,8 @@ void Animation::SetStartTime(const Nullable<TimeDuration>& aNewStartTime) {
   ApplyPendingPlaybackRate();
   mStartTime = aNewStartTime;
 
+  mResetCurrentTimeOnResume = false;
+
   if (!aNewStartTime.IsNull()) {
     if (mPlaybackRate != 0.0) {
       mHoldTime.SetNull();
@@ -351,6 +439,15 @@ void Animation::SetCurrentTime(const TimeDuration& aSeekTime) {
 
   AutoMutationBatchForAnimation mb(*this);
 
+  SetCurrentTimeNoUpdate(aSeekTime);
+
+  if (IsRelevant()) {
+    MutationObservers::NotifyAnimationChanged(this);
+  }
+  PostUpdate();
+}
+
+void Animation::SetCurrentTimeNoUpdate(const TimeDuration& aSeekTime) {
   SilentlySetCurrentTime(aSeekTime);
 
   if (mPendingState == PendingState::PausePending) {
@@ -367,10 +464,6 @@ void Animation::SetCurrentTime(const TimeDuration& aSeekTime) {
   }
 
   UpdateTiming(SeekFlag::DidSeek, SyncNotifyFlag::Async);
-  if (IsRelevant()) {
-    MutationObservers::NotifyAnimationChanged(this);
-  }
-  PostUpdate();
 }
 
 // https://drafts.csswg.org/web-animations/#set-the-playback-rate
@@ -554,7 +647,8 @@ void Animation::Cancel(PostRestyleMode aPostRestyle) {
 
     if (mFinished) {
       mFinished->MaybeReject(NS_ERROR_DOM_ABORT_ERR);
-      mFinished->SetSettledPromiseIsHandled();
+      // mFinished can already be resolved.
+      MOZ_ALWAYS_TRUE(mFinished->SetAnyPromiseIsHandled());
     }
     ResetFinishedPromise();
 
@@ -805,7 +899,7 @@ void Animation::CommitStyles(ErrorResult& aRv) {
 // ---------------------------------------------------------------------------
 
 Nullable<double> Animation::GetStartTimeAsDouble() const {
-  return AnimationUtils::TimeDurationToDouble(mStartTime);
+  return AnimationUtils::TimeDurationToDouble(mStartTime, mRTPCallerType);
 }
 
 void Animation::SetStartTimeAsDouble(const Nullable<double>& aStartTime) {
@@ -813,7 +907,8 @@ void Animation::SetStartTimeAsDouble(const Nullable<double>& aStartTime) {
 }
 
 Nullable<double> Animation::GetCurrentTimeAsDouble() const {
-  return AnimationUtils::TimeDurationToDouble(GetCurrentTimeAsDuration());
+  return AnimationUtils::TimeDurationToDouble(GetCurrentTimeAsDuration(),
+                                              mRTPCallerType);
 }
 
 void Animation::SetCurrentTimeAsDouble(const Nullable<double>& aCurrentTime,
@@ -902,12 +997,9 @@ void Animation::TriggerNow() {
   }
 
   // If we don't have an active timeline we can't trigger the animation.
-  // For non monotonically increasing timelines, we call this function in Play()
-  // and Pause() immediately,
-  //
-  // For monotonically increasing timelines, this is a test-only method that we
-  // don't expect to be used in conjunction with animations without an active
-  // timeline so generate a warning if we do find ourselves in that situation.
+  // However, this is a test-only method that we don't expect to be used in
+  // conjunction with animations without an active timeline so generate
+  // a warning if we do find ourselves in that situation.
   if (!mTimeline || mTimeline->GetCurrentTimeAsDuration().IsNull()) {
     NS_WARNING("Failed to trigger an animation with an active timeline");
     return;
@@ -916,6 +1008,31 @@ void Animation::TriggerNow() {
   FinishPendingAt(mTimeline->GetCurrentTimeAsDuration().Value());
 }
 
+bool Animation::TryTriggerNowForFiniteTimeline() {
+  // Normally we expect the play state to be pending but when an animation
+  // is cancelled and its rendered document can't be reached, we can end up
+  // with the animation still in a pending player tracker even after it is
+  // no longer pending.
+  if (!Pending()) {
+    return true;
+  }
+
+  MOZ_ASSERT(mTimeline && !mTimeline->IsMonotonicallyIncreasing());
+
+  // It's possible that the primary frame or the scrollable frame is not ready
+  // when setting up this animation. So we don't finish pending right now. In
+  // this case, the timeline is inactive so it is still pending. The caller
+  // should handle this case by trying this later once the scrollable frame is
+  // ready.
+  const auto currentTime = mTimeline->GetCurrentTimeAsDuration();
+  if (currentTime.IsNull()) {
+    return false;
+  }
+
+  FinishPendingAt(currentTime.Value());
+  return true;
+}
+
 Nullable<TimeDuration> Animation::GetCurrentOrPendingStartTime() const {
   Nullable<TimeDuration> result;
 
@@ -995,25 +1112,31 @@ TimeStamp Animation::AnimationTimeToTimeStamp(
 TimeStamp Animation::ElapsedTimeToTimeStamp(
     const StickyTimeDuration& aElapsedTime) const {
   TimeDuration delay =
-      mEffect ? mEffect->SpecifiedTiming().Delay() : TimeDuration();
+      mEffect ? mEffect->NormalizedTiming().Delay() : TimeDuration();
   return AnimationTimeToTimeStamp(aElapsedTime + delay);
 }
 
 // https://drafts.csswg.org/web-animations/#silently-set-the-current-time
 void Animation::SilentlySetCurrentTime(const TimeDuration& aSeekTime) {
+  // TODO: Bug 1762238: Introduce "valid seek time" after introducing
+  // CSSNumberish time values.
+  // https://drafts.csswg.org/web-animations-2/#silently-set-the-current-time
+
   if (!mHoldTime.IsNull() || mStartTime.IsNull() || !mTimeline ||
       mTimeline->GetCurrentTimeAsDuration().IsNull() || mPlaybackRate == 0.0) {
     mHoldTime.SetValue(aSeekTime);
-    if (!mTimeline || mTimeline->GetCurrentTimeAsDuration().IsNull()) {
-      mStartTime.SetNull();
-    }
   } else {
     mStartTime =
         StartTimeFromTimelineTime(mTimeline->GetCurrentTimeAsDuration().Value(),
                                   aSeekTime, mPlaybackRate);
   }
 
+  if (!mTimeline || mTimeline->GetCurrentTimeAsDuration().IsNull()) {
+    mStartTime.SetNull();
+  }
+
   mPreviousCurrentTime.SetNull();
+  mResetCurrentTimeOnResume = false;
 }
 
 bool Animation::ShouldBeSynchronizedWithMainThread(
@@ -1360,6 +1483,11 @@ void Animation::PlayNoUpdate(ErrorResult& aRv, LimitBehavior aLimitBehavior) {
   double effectivePlaybackRate = CurrentOrPendingPlaybackRate();
 
   Nullable<TimeDuration> currentTime = GetCurrentTimeAsDuration();
+  if (mResetCurrentTimeOnResume) {
+    currentTime.SetNull();
+    mResetCurrentTimeOnResume = false;
+  }
+
   Nullable<TimeDuration> seekTime;
   if (isAutoRewind) {
     if (effectivePlaybackRate >= 0.0 &&
@@ -1440,10 +1568,14 @@ void Animation::PlayNoUpdate(ErrorResult& aRv, LimitBehavior aLimitBehavior) {
   // animations if it applies.
   mSyncWithGeometricAnimations = false;
 
-  // If the animation use finite timeline, e.g. scroll timeline, we don't use
-  // pending animation tracker. Instead, we let it play immediately.
   if (HasFiniteTimeline()) {
-    TriggerNow();
+    // Always schedule a task even if we would like to let this animation
+    // immedidately ready, per spec.
+    // https://drafts.csswg.org/web-animations/#playing-an-animation-section
+    if (Document* doc = GetRenderedDocument()) {
+      doc->GetOrCreateScrollTimelineAnimationTracker()->AddPending(*this);
+    }  // else: we fail to track this animation, so let the scroll frame to
+       // trigger it when ticking.
   } else {
     if (Document* doc = GetRenderedDocument()) {
       PendingAnimationTracker* tracker =
@@ -1502,10 +1634,14 @@ void Animation::Pause(ErrorResult& aRv) {
 
   mPendingState = PendingState::PausePending;
 
-  // If the animation use finite timeline, e.g. scroll timeline, we don't use
-  // pending animation tracker. Instead, we let it pause immediately.
   if (HasFiniteTimeline()) {
-    TriggerNow();
+    // Always schedule a task even if we would like to let this animation
+    // immedidately ready, per spec.
+    // https://drafts.csswg.org/web-animations/#playing-an-animation-section
+    if (Document* doc = GetRenderedDocument()) {
+      doc->GetOrCreateScrollTimelineAnimationTracker()->AddPending(*this);
+    }  // else: we fail to track this animation, so let the scroll frame to
+       // trigger it when ticking.
   } else {
     if (Document* doc = GetRenderedDocument()) {
       PendingAnimationTracker* tracker =
@@ -1608,30 +1744,32 @@ void Animation::UpdateTiming(SeekFlag aSeekFlag,
 // https://drafts.csswg.org/web-animations/#update-an-animations-finished-state
 void Animation::UpdateFinishedState(SeekFlag aSeekFlag,
                                     SyncNotifyFlag aSyncNotifyFlag) {
-  Nullable<TimeDuration> currentTime = GetCurrentTimeAsDuration();
+  Nullable<TimeDuration> unconstrainedCurrentTime =
+      aSeekFlag == SeekFlag::NoSeek ? GetUnconstrainedCurrentTime()
+                                    : GetCurrentTimeAsDuration();
   TimeDuration effectEnd = TimeDuration(EffectEnd());
 
-  if (!mStartTime.IsNull() && mPendingState == PendingState::NotPending) {
-    if (mPlaybackRate > 0.0 && !currentTime.IsNull() &&
-        currentTime.Value() >= effectEnd) {
+  if (!unconstrainedCurrentTime.IsNull() && !mStartTime.IsNull() &&
+      mPendingState == PendingState::NotPending) {
+    if (mPlaybackRate > 0.0 && unconstrainedCurrentTime.Value() >= effectEnd) {
       if (aSeekFlag == SeekFlag::DidSeek) {
-        mHoldTime = currentTime;
+        mHoldTime = unconstrainedCurrentTime;
       } else if (!mPreviousCurrentTime.IsNull()) {
         mHoldTime.SetValue(std::max(mPreviousCurrentTime.Value(), effectEnd));
       } else {
         mHoldTime.SetValue(effectEnd);
       }
-    } else if (mPlaybackRate < 0.0 && !currentTime.IsNull() &&
-               currentTime.Value() <= TimeDuration()) {
+    } else if (mPlaybackRate < 0.0 &&
+               unconstrainedCurrentTime.Value() <= TimeDuration()) {
       if (aSeekFlag == SeekFlag::DidSeek) {
-        mHoldTime = currentTime;
+        mHoldTime = unconstrainedCurrentTime;
       } else if (!mPreviousCurrentTime.IsNull()) {
         mHoldTime.SetValue(
             std::min(mPreviousCurrentTime.Value(), TimeDuration(0)));
       } else {
         mHoldTime.SetValue(0);
       }
-    } else if (mPlaybackRate != 0.0 && !currentTime.IsNull() && mTimeline &&
+    } else if (mPlaybackRate != 0.0 && mTimeline &&
                !mTimeline->GetCurrentTimeAsDuration().IsNull()) {
       if (aSeekFlag == SeekFlag::DidSeek && !mHoldTime.IsNull()) {
         mStartTime = StartTimeFromTimelineTime(
@@ -1642,15 +1780,16 @@ void Animation::UpdateFinishedState(SeekFlag aSeekFlag,
     }
   }
 
+  // We must recalculate the current time to take account of any mHoldTime
+  // changes the code above made.
+  mPreviousCurrentTime = GetCurrentTimeAsDuration();
+
   bool currentFinishedState = PlayState() == AnimationPlayState::Finished;
   if (currentFinishedState && !mFinishedIsResolved) {
     DoFinishNotification(aSyncNotifyFlag);
   } else if (!currentFinishedState && mFinishedIsResolved) {
     ResetFinishedPromise();
   }
-  // We must recalculate the current time to take account of any mHoldTime
-  // changes the code above made.
-  mPreviousCurrentTime = GetCurrentTimeAsDuration();
 }
 
 void Animation::UpdateEffect(PostRestyleMode aPostRestyle) {
@@ -1714,7 +1853,7 @@ void Animation::ResetPendingTasks() {
 
   if (mReady) {
     mReady->MaybeReject(NS_ERROR_DOM_ABORT_ERR);
-    mReady->SetSettledPromiseIsHandled();
+    MOZ_ALWAYS_TRUE(mReady->SetAnyPromiseIsHandled());
     mReady = nullptr;
   }
 }
@@ -1739,6 +1878,53 @@ void Animation::ReschedulePendingTasks() {
   }
 }
 
+// https://drafts.csswg.org/web-animations-2/#at-progress-timeline-boundary
+/* static*/ Animation::ProgressTimelinePosition
+Animation::AtProgressTimelineBoundary(
+    const Nullable<TimeDuration>& aTimelineDuration,
+    const Nullable<TimeDuration>& aCurrentTime,
+    const TimeDuration& aEffectStartTime, const double aPlaybackRate) {
+  // Based on changed defined in: https://github.com/w3c/csswg-drafts/pull/6702
+  // 1.  If any of the following conditions are true:
+  //     * the associated animation's timeline is not a progress-based timeline,
+  //     or
+  //     * the associated animation's timeline duration is unresolved or zero,
+  //     or
+  //     * the animation's playback rate is zero
+  //     return false
+  // Note: We can detect a progress-based timeline by relying on the fact that
+  // monotonic timelines (i.e. non-progress-based timelines) have an unresolved
+  // timeline duration.
+  if (aTimelineDuration.IsNull() || aTimelineDuration.Value().IsZero() ||
+      aPlaybackRate == 0.0) {
+    return ProgressTimelinePosition::NotBoundary;
+  }
+
+  // 2.  Let effective start time be the animation's start time if resolved, or
+  // zero otherwise.
+  const TimeDuration& effectiveStartTime = aEffectStartTime;
+
+  // 3.  Let effective timeline time be (animation's current time / animation's
+  // playback rate) + effective start time.
+  // Note: we use zero if the current time is unresolved. See the spec issue:
+  // https://github.com/w3c/csswg-drafts/issues/7458
+  const TimeDuration effectiveTimelineTime =
+      (aCurrentTime.IsNull()
+           ? TimeDuration()
+           : aCurrentTime.Value().MultDouble(1.0 / aPlaybackRate)) +
+      effectiveStartTime;
+
+  // 4.  Let effective timeline progress be (effective timeline time / timeline
+  // duration)
+  // 5.  If effective timeline progress is 0 or 1, return true,
+  // We avoid the division here but it is effectively the same as 4 & 5 above.
+  return effectiveTimelineTime.IsZero() ||
+                 (AnimationUtils::IsWithinAnimationTimeTolerance(
+                     effectiveTimelineTime, aTimelineDuration.Value()))
+             ? ProgressTimelinePosition::Boundary
+             : ProgressTimelinePosition::NotBoundary;
+}
+
 bool Animation::IsPossiblyOrphanedPendingAnimation() const {
   // Check if we are pending but might never start because we are not being
   // tracked.
@@ -1752,10 +1938,13 @@ bool Animation::IsPossiblyOrphanedPendingAnimation() const {
   // * We started playing but our timeline became inactive.
   //   In this case the pending animation tracker will drop us from its hashmap
   //   when we have been painted.
-  // * When we started playing we couldn't find a PendingAnimationTracker to
-  //   register with (perhaps the effect had no document) so we simply
-  //   set mPendingState in PlayNoUpdate and relied on this method to catch us
-  //   on the next tick.
+  // * When we started playing we couldn't find a
+  //   PendingAnimationTracker/ScrollTimelineAnimationTracker to register with
+  //   (perhaps the effect had no document) so we may
+  //   1. simply set mPendingState in PlayNoUpdate and relied on this method to
+  //      catch us on the next tick, or
+  //   2. rely on the scroll frame to tick this animation and catch us in this
+  //      method.
 
   // If we're not pending we're ok.
   if (mPendingState == PendingState::NotPending) {
@@ -1795,7 +1984,7 @@ StickyTimeDuration Animation::EffectEnd() const {
     return StickyTimeDuration(0);
   }
 
-  return mEffect->SpecifiedTiming().EndTime();
+  return mEffect->NormalizedTiming().EndTime();
 }
 
 Document* Animation::GetRenderedDocument() const {
@@ -1810,6 +1999,50 @@ Document* Animation::GetTimelineDocument() const {
   return mTimeline ? mTimeline->GetDocument() : nullptr;
 }
 
+void Animation::UpdatePendingAnimationTracker(AnimationTimeline* aOldTimeline,
+                                              AnimationTimeline* aNewTimeline) {
+  // If we are still in pending, we may have to move this animation into the
+  // correct animation tracker.
+  Document* doc = GetRenderedDocument();
+  if (!doc || !Pending()) {
+    return;
+  }
+
+  const bool fromFiniteTimeline =
+      aOldTimeline && !aOldTimeline->IsMonotonicallyIncreasing();
+  const bool toFiniteTimeline =
+      aNewTimeline && !aNewTimeline->IsMonotonicallyIncreasing();
+  if (fromFiniteTimeline == toFiniteTimeline) {
+    return;
+  }
+
+  const bool isPlayPending = mPendingState == PendingState::PlayPending;
+  if (toFiniteTimeline) {
+    // From null/document-timeline to scroll-timeline
+    if (auto* tracker = doc->GetPendingAnimationTracker()) {
+      if (isPlayPending) {
+        tracker->RemovePlayPending(*this);
+      } else {
+        tracker->RemovePausePending(*this);
+      }
+    }
+
+    doc->GetOrCreateScrollTimelineAnimationTracker()->AddPending(*this);
+  } else {
+    // From scroll-timeline to null/document-timeline
+    if (auto* tracker = doc->GetScrollTimelineAnimationTracker()) {
+      tracker->RemovePending(*this);
+    }
+
+    auto* tracker = doc->GetOrCreatePendingAnimationTracker();
+    if (isPlayPending) {
+      tracker->AddPlayPending(*this);
+    } else {
+      tracker->AddPausePending(*this);
+    }
+  }
+}
+
 class AsyncFinishNotification : public MicroTaskRunnable {
  public:
   explicit AsyncFinishNotification(Animation* aAnimation)
@@ -1913,13 +2146,27 @@ bool Animation::IsInEffect() const {
   return GetEffect() && GetEffect()->IsInEffect();
 }
 
+void Animation::SetHiddenByContentVisibility(bool hidden) {
+  if (mHiddenByContentVisibility == hidden) {
+    return;
+  }
+
+  mHiddenByContentVisibility = hidden;
+
+  if (!GetTimeline()) {
+    return;
+  }
+
+  GetTimeline()->NotifyAnimationContentVisibilityChanged(this, !hidden);
+}
+
 StickyTimeDuration Animation::IntervalStartTime(
     const StickyTimeDuration& aActiveDuration) const {
   MOZ_ASSERT(AsCSSTransition() || AsCSSAnimation(),
              "Should be called for CSS animations or transitions");
   static constexpr StickyTimeDuration zeroDuration = StickyTimeDuration();
   return std::max(
-      std::min(StickyTimeDuration(-mEffect->SpecifiedTiming().Delay()),
+      std::min(StickyTimeDuration(-mEffect->NormalizedTiming().Delay()),
                aActiveDuration),
       zeroDuration);
 }
@@ -1935,7 +2182,22 @@ StickyTimeDuration Animation::IntervalEndTime(
              "Should be called for CSS animations or transitions");
 
   static constexpr StickyTimeDuration zeroDuration = StickyTimeDuration();
-  return std::max(std::min((EffectEnd() - mEffect->SpecifiedTiming().Delay()),
+  const StickyTimeDuration& effectEnd = EffectEnd();
+
+  // If both "associated effect end" and "start delay" are Infinity, we skip it
+  // because we will get NaN when computing "Infinity - Infinity", and
+  // using NaN in std::min or std::max is undefined.
+  if (MOZ_UNLIKELY(effectEnd == TimeDuration::Forever() &&
+                   effectEnd == mEffect->NormalizedTiming().Delay())) {
+    // Note: If we use TimeDuration::Forever(), within our animation event
+    // handling, we'd end up turning that into a null TimeStamp which can causes
+    // errors if we try to do any arithmetic with it. Given that we should never
+    // end up _using_ the interval end time. So returning zeroDuration here is
+    // probably fine.
+    return zeroDuration;
+  }
+
+  return std::max(std::min(effectEnd - mEffect->NormalizedTiming().Delay(),
                            aActiveDuration),
                   zeroDuration);
 }
