@@ -18,33 +18,17 @@
 
 struct JSContext;
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 class DocumentTimeline final : public AnimationTimeline,
                                public nsARefreshObserver,
                                public nsATimerAdjustmentObserver,
                                public LinkedListElement<DocumentTimeline> {
  public:
-  DocumentTimeline(Document* aDocument, const TimeDuration& aOriginTime)
-      : AnimationTimeline(aDocument->GetParentObject()),
-        mDocument(aDocument),
-        mIsObservingRefreshDriver(false),
-        mOriginTime(aOriginTime) {
-    if (mDocument) {
-      mDocument->Timelines().insertBack(this);
-    }
-  }
+  DocumentTimeline(Document* aDocument, const TimeDuration& aOriginTime);
 
  protected:
-  virtual ~DocumentTimeline() {
-    MOZ_ASSERT(!mIsObservingRefreshDriver,
-               "Timeline should have disassociated"
-               " from the refresh driver before being destroyed");
-    if (isInList()) {
-      remove();
-    }
-  }
+  virtual ~DocumentTimeline();
 
  public:
   NS_DECL_ISUPPORTS_INHERITED
@@ -72,6 +56,8 @@ class DocumentTimeline final : public AnimationTimeline,
   void NotifyAnimationUpdated(Animation& aAnimation) override;
 
   void RemoveAnimation(Animation* aAnimation) override;
+  void NotifyAnimationContentVisibilityChanged(Animation* aAnimation,
+                                               bool visible) override;
 
   // nsARefreshObserver methods
   void WillRefresh(TimeStamp aTime) override;
@@ -83,6 +69,10 @@ class DocumentTimeline final : public AnimationTimeline,
 
   Document* GetDocument() const override { return mDocument; }
 
+  void UpdateLastRefreshDriverTime();
+
+  bool IsMonotonicallyIncreasing() const override { return true; }
+
  protected:
   TimeStamp GetCurrentTimeStamp() const;
   nsRefreshDriver* GetRefreshDriver() const;
@@ -96,13 +86,12 @@ class DocumentTimeline final : public AnimationTimeline,
   // The most recently used refresh driver time. This is used in cases where
   // we don't have a refresh driver (e.g. because we are in a display:none
   // iframe).
-  mutable TimeStamp mLastRefreshDriverTime;
+  TimeStamp mLastRefreshDriverTime;
   bool mIsObservingRefreshDriver;
 
   TimeDuration mOriginTime;
 };
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
 
 #endif  // mozilla_dom_DocumentTimeline_h
