@@ -63,6 +63,7 @@ static const uint32_t kCCPurpleLimit = 200;
 
 // Actions performed by the GCRunner state machine.
 enum class GCRunnerAction {
+  MinorGC,        // Run a minor GC (nursery collection)
   WaitToMajorGC,  // We want to start a new major GC
   StartMajorGC,   // The parent says we may begin our major GC
   GCSlice,        // Run a single slice of a major GC
@@ -74,12 +75,24 @@ struct GCRunnerStep {
   JS::GCReason mReason;
 };
 
+// Actions that are output from the CCRunner state machine.
 enum class CCRunnerAction {
+  // Do nothing.
   None,
+
+  // We crossed an eager minor GC threshold in the middle of an incremental CC,
+  // and we have some idle time.
+  MinorGC,
+
+  // Various cleanup actions.
   ForgetSkippable,
   CleanupContentUnbinder,
   CleanupDeferred,
+
+  // Do the actual cycle collection (build the graph etc).
   CycleCollect,
+
+  // All done.
   StopRunning
 };
 
@@ -91,20 +104,30 @@ enum CCRunnerForgetSkippableRemoveChildless {
 };
 
 struct CCRunnerStep {
-  // The action to scheduler is instructing the caller to perform.
+  // The action the scheduler is instructing the caller to perform.
   CCRunnerAction mAction;
 
   // Whether to stop processing actions for this invocation of the timer
   // callback.
   CCRunnerYield mYield;
 
-  // If the action is ForgetSkippable, then whether to remove childless nodes
-  // or not. (ForgetSkippable is the only action requiring a parameter; if
-  // that changes, this will become a union.)
-  CCRunnerForgetSkippableRemoveChildless mRemoveChildless;
+  union ActionData {
+    // If the action is ForgetSkippable, then whether to remove childless nodes
+    // or not.
+    CCRunnerForgetSkippableRemoveChildless mRemoveChildless;
 
-  // If the action is CycleCollect, the reason for the collection.
-  CCReason mCCReason;
+    // If the action is CycleCollect, the reason for the collection.
+    CCReason mCCReason;
+
+    // If the action is MinorGC, the reason for the GC.
+    JS::GCReason mReason;
+
+    MOZ_IMPLICIT ActionData(CCRunnerForgetSkippableRemoveChildless v)
+        : mRemoveChildless(v) {}
+    MOZ_IMPLICIT ActionData(CCReason v) : mCCReason(v) {}
+    MOZ_IMPLICIT ActionData(JS::GCReason v) : mReason(v) {}
+    ActionData() = default;
+  } mParam;
 };
 
 class CCGCScheduler {
@@ -145,6 +168,7 @@ class CCGCScheduler {
   void PokeShrinkingGC();
   void PokeFullGC();
   void MaybePokeCC(TimeStamp aNow, uint32_t aSuspectedCCObjects);
+  void PokeMinorGC(JS::GCReason aReason);
 
   void UserIsInactive();
   void UserIsActive();
@@ -215,6 +239,12 @@ class CCGCScheduler {
     }
   }
 
+  void SetWantEagerMinorGC(JS::GCReason aReason) {
+    if (mEagerMinorGCReason == JS::GCReason::NO_REASON) {
+      mEagerMinorGCReason = aReason;
+    }
+  }
+
   // Ensure that the current runner does a cycle collection, and trigger a GC
   // after it finishes.
   void EnsureCCThenGC(CCReason aReason) {
@@ -235,7 +265,7 @@ class CCGCScheduler {
   }
 
   // Starting a major GC (incremental or non-incremental).
-  void NoteGCBegin();
+  void NoteGCBegin(JS::GCReason aReason);
 
   // Major GC completed.
   void NoteGCEnd();
@@ -243,30 +273,23 @@ class CCGCScheduler {
   // A timer fired, but then decided not to run a GC.
   void NoteWontGC();
 
+  void NoteMinorGCEnd() { mEagerMinorGCReason = JS::GCReason::NO_REASON; }
+
   // This is invoked when we reach the actual cycle collection portion of the
   // overall cycle collection.
-  void NoteCCBegin(CCReason aReason, TimeStamp aWhen);
+  void NoteCCBegin(CCReason aReason, TimeStamp aWhen,
+                   uint32_t aNumForgetSkippables, uint32_t aSuspected,
+                   uint32_t aRemovedPurples);
 
   // This is invoked when the whole process of collection is done -- i.e., CC
   // preparation (eg ForgetSkippables) in addition to the CC itself. There
   // really ought to be a separate name for the overall CC as opposed to the
   // actual cycle collection portion.
-  void NoteCCEnd(TimeStamp aWhen);
+  void NoteCCEnd(const CycleCollectorResults& aResults, TimeStamp aWhen,
+                 mozilla::TimeDuration aMaxSliceTime);
 
-  void NoteGCSliceEnd(TimeDuration aSliceDuration) {
-    if (mMajorGCReason == JS::GCReason::NO_REASON) {
-      // Internally-triggered GCs do not wait for the parent's permission to
-      // proceed. This flag won't be checked during an incremental GC anyway,
-      // but it better reflects reality.
-      mReadyForMajorGC = true;
-    }
-
-    // Subsequent slices should be INTER_SLICE_GC unless they are triggered by
-    // something else that provides its own reason.
-    mMajorGCReason = JS::GCReason::INTER_SLICE_GC;
-
-    mGCUnnotifiedTotalTime += aSliceDuration;
-  }
+  // A single slice has completed.
+  void NoteGCSliceEnd(TimeStamp aStart, TimeStamp aEnd);
 
   bool GCRunnerFired(TimeStamp aDeadline);
   bool GCRunnerFiredDoGC(TimeStamp aDeadline, const GCRunnerStep& aStep);
@@ -304,13 +327,6 @@ class CCGCScheduler {
     return aSuspectedBeforeForgetSkippable - aSuspectedCCObjects;
   }
 
-  // After collecting cycles, record the results that are used in scheduling
-  // decisions.
-  void NoteCycleCollected(const CycleCollectorResults& aResults) {
-    mCCollectedWaitingForGC += aResults.mFreedGCed;
-    mCCollectedZonesWaitingForGC += aResults.mFreedJSZones;
-  }
-
   // Test if we are in the NoteCCBegin .. NoteCCEnd interval.
   bool IsCollectingCycles() const { return mIsCollectingCycles; }
 
@@ -424,7 +440,13 @@ class CCGCScheduler {
     mCCReason = CCReason::NO_REASON;
   }
 
-  GCRunnerStep GetNextGCRunnerAction() const;
+  bool HasMoreIdleGCRunnerWork() const {
+    return mMajorGCReason != JS::GCReason::NO_REASON ||
+           mEagerMajorGCReason != JS::GCReason::NO_REASON ||
+           mEagerMinorGCReason != JS::GCReason::NO_REASON;
+  }
+
+  GCRunnerStep GetNextGCRunnerAction(TimeStamp aDeadline) const;
 
   CCRunnerStep AdvanceCCRunner(TimeStamp aDeadline, TimeStamp aNow,
                                uint32_t aSuspectedCCObjects);
@@ -487,7 +509,10 @@ class CCGCScheduler {
 
   uint32_t mCleanupsSinceLastGC = UINT32_MAX;
 
-  TimeDuration mGCUnnotifiedTotalTime;
+  // If the GC runner triggers a GC slice, this will be set to the idle deadline
+  // or the null timestamp if non-idle. It will be Nothing at the end of an
+  // internally-triggered slice.
+  mozilla::Maybe<TimeStamp> mTriggeredGCDeadline;
 
   RefPtr<IdleTaskRunner> mGCRunner;
   RefPtr<IdleTaskRunner> mCCRunner;
@@ -496,6 +521,8 @@ class CCGCScheduler {
 
   mozilla::CCReason mCCReason = mozilla::CCReason::NO_REASON;
   JS::GCReason mMajorGCReason = JS::GCReason::NO_REASON;
+  JS::GCReason mEagerMajorGCReason = JS::GCReason::NO_REASON;
+  JS::GCReason mEagerMinorGCReason = JS::GCReason::NO_REASON;
 
   bool mIsCompactingOnUserInactive = false;
   bool mIsCollectingCycles = false;
