@@ -6,21 +6,34 @@
 
 #include "mozilla/dom/DOMMatrix.h"
 
-#include "mozilla/dom/BindingUtils.h"
+#include <cmath>
+#include <cstdint>
+#include <new>
+#include "ErrorList.h"
+#include "js/Conversions.h"
+#include "js/Equality.h"
+#include "js/StructuredClone.h"
+#include "js/Value.h"
+#include "mozilla/Casting.h"
+#include "mozilla/ErrorResult.h"
+#include "mozilla/FloatingPoint.h"
+#include "mozilla/MacroForEach.h"
+#include "mozilla/RefPtr.h"
+#include "mozilla/ServoCSSParser.h"
+#include "mozilla/dom/BindingDeclarations.h"
 #include "mozilla/dom/DOMMatrixBinding.h"
 #include "mozilla/dom/DOMPoint.h"
 #include "mozilla/dom/DOMPointBinding.h"
-#include "mozilla/dom/BindingDeclarations.h"
 #include "mozilla/dom/ToJSValue.h"
-#include "mozilla/ServoCSSParser.h"
-#include "nsGlobalWindowInner.h"
-#include "nsStyleTransformMatrix.h"
-#include "nsGlobalWindowInner.h"
-
-#include <math.h>
-
-#include "js/Conversions.h"  // JS::NumberToString
-#include "js/Equality.h"     // JS::SameValueZero
+#include "mozilla/gfx/BasePoint.h"
+#include "mozilla/gfx/BasePoint4D.h"
+#include "mozilla/gfx/Point.h"
+#include "nsIGlobalObject.h"
+#include "nsPIDOMWindow.h"
+#include "nsString.h"
+#include "nsStringFlags.h"
+#include "nsTArray.h"
+#include "nsTLiteralString.h"
 
 namespace mozilla::dom {
 
@@ -32,9 +45,6 @@ static const double radPerDegree = 2.0 * M_PI / 360.0;
 
 NS_IMPL_CYCLE_COLLECTION_WRAPPERCACHE(DOMMatrixReadOnly, mParent)
 
-NS_IMPL_CYCLE_COLLECTION_ROOT_NATIVE(DOMMatrixReadOnly, AddRef)
-NS_IMPL_CYCLE_COLLECTION_UNROOT_NATIVE(DOMMatrixReadOnly, Release)
-
 JSObject* DOMMatrixReadOnly::WrapObject(JSContext* aCx,
                                         JS::Handle<JSObject*> aGivenProto) {
   return DOMMatrixReadOnly_Binding::Wrap(aCx, this, aGivenProto);
@@ -545,10 +555,6 @@ void DOMMatrixReadOnly::Stringify(nsAString& aResult, ErrorResult& aRv) {
 // https://drafts.fxtf.org/geometry/#structured-serialization
 bool DOMMatrixReadOnly::WriteStructuredClone(
     JSContext* aCx, JSStructuredCloneWriter* aWriter) const {
-#define WriteDouble(d)                                                       \
-  JS_WriteUint32Pair(aWriter, (BitwiseCast<uint64_t>(d) >> 32) & 0xffffffff, \
-                     BitwiseCast<uint64_t>(d) & 0xffffffff)
-
   const uint8_t is2D = Is2D();
 
   if (!JS_WriteBytes(aWriter, &is2D, 1)) {
@@ -556,63 +562,61 @@ bool DOMMatrixReadOnly::WriteStructuredClone(
   }
 
   if (is2D == 1) {
-    return WriteDouble(mMatrix2D->_11) && WriteDouble(mMatrix2D->_12) &&
-           WriteDouble(mMatrix2D->_21) && WriteDouble(mMatrix2D->_22) &&
-           WriteDouble(mMatrix2D->_31) && WriteDouble(mMatrix2D->_32);
+    return JS_WriteDouble(aWriter, mMatrix2D->_11) &&
+           JS_WriteDouble(aWriter, mMatrix2D->_12) &&
+           JS_WriteDouble(aWriter, mMatrix2D->_21) &&
+           JS_WriteDouble(aWriter, mMatrix2D->_22) &&
+           JS_WriteDouble(aWriter, mMatrix2D->_31) &&
+           JS_WriteDouble(aWriter, mMatrix2D->_32);
   }
 
-  return WriteDouble(mMatrix3D->_11) && WriteDouble(mMatrix3D->_12) &&
-         WriteDouble(mMatrix3D->_13) && WriteDouble(mMatrix3D->_14) &&
-         WriteDouble(mMatrix3D->_21) && WriteDouble(mMatrix3D->_22) &&
-         WriteDouble(mMatrix3D->_23) && WriteDouble(mMatrix3D->_24) &&
-         WriteDouble(mMatrix3D->_31) && WriteDouble(mMatrix3D->_32) &&
-         WriteDouble(mMatrix3D->_33) && WriteDouble(mMatrix3D->_34) &&
-         WriteDouble(mMatrix3D->_41) && WriteDouble(mMatrix3D->_42) &&
-         WriteDouble(mMatrix3D->_43) && WriteDouble(mMatrix3D->_44);
-
-#undef WriteDouble
+  return JS_WriteDouble(aWriter, mMatrix3D->_11) &&
+         JS_WriteDouble(aWriter, mMatrix3D->_12) &&
+         JS_WriteDouble(aWriter, mMatrix3D->_13) &&
+         JS_WriteDouble(aWriter, mMatrix3D->_14) &&
+         JS_WriteDouble(aWriter, mMatrix3D->_21) &&
+         JS_WriteDouble(aWriter, mMatrix3D->_22) &&
+         JS_WriteDouble(aWriter, mMatrix3D->_23) &&
+         JS_WriteDouble(aWriter, mMatrix3D->_24) &&
+         JS_WriteDouble(aWriter, mMatrix3D->_31) &&
+         JS_WriteDouble(aWriter, mMatrix3D->_32) &&
+         JS_WriteDouble(aWriter, mMatrix3D->_33) &&
+         JS_WriteDouble(aWriter, mMatrix3D->_34) &&
+         JS_WriteDouble(aWriter, mMatrix3D->_41) &&
+         JS_WriteDouble(aWriter, mMatrix3D->_42) &&
+         JS_WriteDouble(aWriter, mMatrix3D->_43) &&
+         JS_WriteDouble(aWriter, mMatrix3D->_44);
 }
 
 bool DOMMatrixReadOnly::ReadStructuredCloneElements(
     JSStructuredCloneReader* aReader, DOMMatrixReadOnly* matrix) {
-  uint32_t high;
-  uint32_t low;
-
-#define ReadDouble(d)                             \
-  if (!JS_ReadUint32Pair(aReader, &high, &low)) { \
-    return false;                                 \
-  }                                               \
-  (*(d) = BitwiseCast<double>(static_cast<uint64_t>(high) << 32 | low))
-
   if (matrix->Is2D() == 1) {
-    ReadDouble(&(matrix->mMatrix2D->_11));
-    ReadDouble(&(matrix->mMatrix2D->_12));
-    ReadDouble(&(matrix->mMatrix2D->_21));
-    ReadDouble(&(matrix->mMatrix2D->_22));
-    ReadDouble(&(matrix->mMatrix2D->_31));
-    ReadDouble(&(matrix->mMatrix2D->_32));
+    JS_ReadDouble(aReader, &(matrix->mMatrix2D->_11));
+    JS_ReadDouble(aReader, &(matrix->mMatrix2D->_12));
+    JS_ReadDouble(aReader, &(matrix->mMatrix2D->_21));
+    JS_ReadDouble(aReader, &(matrix->mMatrix2D->_22));
+    JS_ReadDouble(aReader, &(matrix->mMatrix2D->_31));
+    JS_ReadDouble(aReader, &(matrix->mMatrix2D->_32));
   } else {
-    ReadDouble(&(matrix->mMatrix3D->_11));
-    ReadDouble(&(matrix->mMatrix3D->_12));
-    ReadDouble(&(matrix->mMatrix3D->_13));
-    ReadDouble(&(matrix->mMatrix3D->_14));
-    ReadDouble(&(matrix->mMatrix3D->_21));
-    ReadDouble(&(matrix->mMatrix3D->_22));
-    ReadDouble(&(matrix->mMatrix3D->_23));
-    ReadDouble(&(matrix->mMatrix3D->_24));
-    ReadDouble(&(matrix->mMatrix3D->_31));
-    ReadDouble(&(matrix->mMatrix3D->_32));
-    ReadDouble(&(matrix->mMatrix3D->_33));
-    ReadDouble(&(matrix->mMatrix3D->_34));
-    ReadDouble(&(matrix->mMatrix3D->_41));
-    ReadDouble(&(matrix->mMatrix3D->_42));
-    ReadDouble(&(matrix->mMatrix3D->_43));
-    ReadDouble(&(matrix->mMatrix3D->_44));
+    JS_ReadDouble(aReader, &(matrix->mMatrix3D->_11));
+    JS_ReadDouble(aReader, &(matrix->mMatrix3D->_12));
+    JS_ReadDouble(aReader, &(matrix->mMatrix3D->_13));
+    JS_ReadDouble(aReader, &(matrix->mMatrix3D->_14));
+    JS_ReadDouble(aReader, &(matrix->mMatrix3D->_21));
+    JS_ReadDouble(aReader, &(matrix->mMatrix3D->_22));
+    JS_ReadDouble(aReader, &(matrix->mMatrix3D->_23));
+    JS_ReadDouble(aReader, &(matrix->mMatrix3D->_24));
+    JS_ReadDouble(aReader, &(matrix->mMatrix3D->_31));
+    JS_ReadDouble(aReader, &(matrix->mMatrix3D->_32));
+    JS_ReadDouble(aReader, &(matrix->mMatrix3D->_33));
+    JS_ReadDouble(aReader, &(matrix->mMatrix3D->_34));
+    JS_ReadDouble(aReader, &(matrix->mMatrix3D->_41));
+    JS_ReadDouble(aReader, &(matrix->mMatrix3D->_42));
+    JS_ReadDouble(aReader, &(matrix->mMatrix3D->_43));
+    JS_ReadDouble(aReader, &(matrix->mMatrix3D->_44));
   }
 
   return true;
-
-#undef ReadDouble
 }
 
 already_AddRefed<DOMMatrix> DOMMatrix::FromMatrix(
