@@ -20,7 +20,6 @@
 #include "mozilla/EffectSet.h"
 #include "mozilla/EventDispatcher.h"
 #include "mozilla/EventListenerManager.h"
-#include "mozilla/EventStates.h"
 #include "mozilla/HTMLEditor.h"
 #include "mozilla/PresShell.h"
 #include "mozilla/RestyleManager.h"
@@ -34,8 +33,10 @@
 #include "mozilla/dom/Event.h"
 #include "mozilla/dom/ScriptLoader.h"
 #include "mozilla/dom/TouchEvent.h"
+#include "mozilla/dom/CustomElementRegistry.h"
 #include "mozilla/dom/Document.h"
 #include "mozilla/dom/DocumentInlines.h"
+#include "nsIControllers.h"
 #include "nsIDocumentEncoder.h"
 #include "nsFocusManager.h"
 #include "nsIScriptGlobalObject.h"
@@ -58,9 +59,6 @@
 #include "mozilla/MouseEvents.h"
 #include "nsAttrValueOrString.h"
 #include "nsQueryObject.h"
-#ifdef MOZ_XUL
-#  include "nsXULElement.h"
-#endif /* MOZ_XUL */
 #include "nsFrameSelection.h"
 #ifdef DEBUG
 #  include "nsRange.h"
@@ -196,8 +194,8 @@ nsIContent::IMEState nsIContent::GetDesiredIMEState() {
     // Check for the special case where we're dealing with elements which don't
     // have the editable flag set, but are readwrite (such as text controls).
     if (!IsElement() ||
-        !AsElement()->State().HasState(NS_EVENT_STATE_READWRITE)) {
-      return IMEState(IMEState::DISABLED);
+        !AsElement()->State().HasState(ElementState::READWRITE)) {
+      return IMEState(IMEEnabled::Disabled);
     }
   }
   // NOTE: The content for independent editors (e.g., input[type=text],
@@ -211,15 +209,15 @@ nsIContent::IMEState nsIContent::GetDesiredIMEState() {
   }
   Document* doc = GetComposedDoc();
   if (!doc) {
-    return IMEState(IMEState::DISABLED);
+    return IMEState(IMEEnabled::Disabled);
   }
   nsPresContext* pc = doc->GetPresContext();
   if (!pc) {
-    return IMEState(IMEState::DISABLED);
+    return IMEState(IMEEnabled::Disabled);
   }
   HTMLEditor* htmlEditor = nsContentUtils::GetHTMLEditor(pc);
   if (!htmlEditor) {
-    return IMEState(IMEState::DISABLED);
+    return IMEState(IMEEnabled::Disabled);
   }
   IMEState state;
   htmlEditor->GetPreferredIMEState(&state);
@@ -243,17 +241,18 @@ dom::Element* nsIContent::GetEditingHost() {
   }
 
   // If this is in designMode, we should return <body>
-  if (doc->HasFlag(NODE_IS_EDITABLE) && !IsInShadowTree()) {
+  if (IsInDesignMode() && !IsInShadowTree()) {
     return doc->GetBodyElement();
   }
 
-  nsIContent* content = this;
+  dom::Element* editableParentElement = nullptr;
   for (dom::Element* parent = GetParentElement();
        parent && parent->HasFlag(NODE_IS_EDITABLE);
-       parent = content->GetParentElement()) {
-    content = parent;
+       parent = editableParentElement->GetParentElement()) {
+    editableParentElement = parent;
   }
-  return content->AsElement();
+  return editableParentElement ? editableParentElement
+                               : dom::Element::FromNode(this);
 }
 
 nsresult nsIContent::LookupNamespaceURIInternal(
@@ -412,7 +411,7 @@ nsIContent* nsAttrChildContentList::Item(uint32_t aIndex) {
 
 int32_t nsAttrChildContentList::IndexOf(nsIContent* aContent) {
   if (mNode) {
-    return mNode->ComputeIndexOf(aContent);
+    return mNode->ComputeIndexOf_Deprecated(aContent);
   }
 
   return -1;
@@ -535,7 +534,9 @@ void nsIContent::nsExtendedContentSlots::TraverseExtendedSlots(
 
 nsIContent::nsExtendedContentSlots::nsExtendedContentSlots() = default;
 
-nsIContent::nsExtendedContentSlots::~nsExtendedContentSlots() = default;
+nsIContent::nsExtendedContentSlots::~nsExtendedContentSlots() {
+  MOZ_ASSERT(!mManualSlotAssignment);
+}
 
 size_t nsIContent::nsExtendedContentSlots::SizeOfExcludingThis(
     MallocSizeOf aMallocSizeOf) const {
@@ -735,7 +736,7 @@ static nsINode* FindChromeAccessOnlySubtreeOwner(nsINode* aNode) {
 
 already_AddRefed<nsINode> FindChromeAccessOnlySubtreeOwner(
     EventTarget* aTarget) {
-  nsCOMPtr<nsINode> node = do_QueryInterface(aTarget);
+  nsCOMPtr<nsINode> node = nsINode::FromEventTargetOrNull(aTarget);
   if (!node || !node->ChromeOnlyAccess()) {
     return node.forget();
   }
@@ -767,8 +768,8 @@ void nsIContent::GetEventTargetParent(EventChainPreVisitor& aVisitor) {
       // a shadow root to a shadow root host.
       ((this == aVisitor.mEvent->mOriginalTarget && !ChromeOnlyAccess()) ||
        isAnonForEvents)) {
-    nsCOMPtr<nsIContent> relatedTarget =
-        do_QueryInterface(aVisitor.mEvent->AsMouseEvent()->mRelatedTarget);
+    nsCOMPtr<nsIContent> relatedTarget = nsIContent::FromEventTargetOrNull(
+        aVisitor.mEvent->AsMouseEvent()->mRelatedTarget);
     if (relatedTarget && relatedTarget->OwnerDoc() == OwnerDoc()) {
       // If current target is anonymous for events or we know that related
       // target is descendant of an element which is anonymous for events,
@@ -796,7 +797,8 @@ void nsIContent::GetEventTargetParent(EventChainPreVisitor& aVisitor) {
             if (anonOwner == anonOwnerRelated) {
 #ifdef DEBUG_smaug
               nsCOMPtr<nsIContent> originalTarget =
-                  do_QueryInterface(aVisitor.mEvent->mOriginalTarget);
+                  nsIContent::FromEventTargetOrNull(
+                      aVisitor.mEvent->mOriginalTarget);
               nsAutoString ot, ct, rt;
               if (originalTarget) {
                 originalTarget->NodeInfo()->NameAtom()->ToString(ot);
@@ -848,7 +850,7 @@ void nsIContent::GetEventTargetParent(EventChainPreVisitor& aVisitor) {
     // If a DOM event is explicitly dispatched using node.dispatchEvent(), then
     // all the events are allowed even in the native anonymous content..
     nsCOMPtr<nsIContent> t =
-        do_QueryInterface(aVisitor.mEvent->mOriginalTarget);
+        nsIContent::FromEventTargetOrNull(aVisitor.mEvent->mOriginalTarget);
     NS_ASSERTION(!t || !t->ChromeOnlyAccess() ||
                      aVisitor.mEvent->mClass != eMutationEventClass ||
                      aVisitor.mDOMEvent,
@@ -856,7 +858,8 @@ void nsIContent::GetEventTargetParent(EventChainPreVisitor& aVisitor) {
 #endif
     aVisitor.mEventTargetAtParent = parent;
   } else if (parent && aVisitor.mOriginalTargetIsInAnon) {
-    nsCOMPtr<nsIContent> content(do_QueryInterface(aVisitor.mEvent->mTarget));
+    nsCOMPtr<nsIContent> content(
+        nsIContent::FromEventTargetOrNull(aVisitor.mEvent->mTarget));
     if (content &&
         content->GetClosestNativeAnonymousSubtreeRootParent() == parent) {
       aVisitor.mEventTargetAtParent = parent;
@@ -905,7 +908,7 @@ void nsIContent::GetEventTargetParent(EventChainPreVisitor& aVisitor) {
                 aVisitor.mEvent->mOriginalRelatedTarget);
         if (!originalTargetAsNode) {
           originalTargetAsNode =
-              do_QueryInterface(aVisitor.mEvent->mOriginalTarget);
+              nsINode::FromEventTargetOrNull(aVisitor.mEvent->mOriginalTarget);
         }
 
         if (relatedTargetAsNode && originalTargetAsNode) {
@@ -991,7 +994,8 @@ void nsIContent::GetEventTargetParent(EventChainPreVisitor& aVisitor) {
         Touch* touch = touches[i];
         EventTarget* originalTarget = touch->mOriginalTarget;
         EventTarget* touchTarget = originalTarget;
-        nsCOMPtr<nsINode> targetAsNode = do_QueryInterface(originalTarget);
+        nsCOMPtr<nsINode> targetAsNode =
+            nsINode::FromEventTargetOrNull(originalTarget);
         if (targetAsNode) {
           EventTarget* retargeted =
               nsContentUtils::Retarget(targetAsNode, this);
@@ -1023,6 +1027,68 @@ bool nsIContent::IsFocusable(int32_t* aTabIndex, bool aWithMouse) {
   return false;
 }
 
+Element* nsIContent::GetFocusDelegate(bool aWithMouse) const {
+  const nsIContent* whereToLook = this;
+  if (ShadowRoot* root = GetShadowRoot()) {
+    if (!root->DelegatesFocus()) {
+      // 1. If focusTarget is a shadow host and its shadow root 's delegates
+      // focus is false, then return null.
+      return nullptr;
+    }
+    whereToLook = root;
+  }
+
+  auto IsFocusable = [&](Element* aElement) {
+    nsIFrame* frame = aElement->GetPrimaryFrame();
+    return frame && frame->IsFocusable(aWithMouse);
+  };
+
+  Element* potentialFocus = nullptr;
+  for (nsINode* node = whereToLook->GetFirstChild(); node;
+       node = node->GetNextNode(whereToLook)) {
+    auto* el = Element::FromNode(*node);
+    if (!el) {
+      continue;
+    }
+
+    const bool autofocus = el->GetBoolAttr(nsGkAtoms::autofocus);
+
+    if (autofocus) {
+      if (IsFocusable(el)) {
+        // Found an autofocus candidate.
+        return el;
+      }
+    } else if (!potentialFocus && IsFocusable(el)) {
+      // This element could be the one if we can't find an
+      // autofocus candidate which has the precedence.
+      potentialFocus = el;
+    }
+
+    if (!autofocus && potentialFocus) {
+      // Nothing else to do, we are not looking for more focusable elements
+      // here.
+      continue;
+    }
+
+    if (auto* shadow = el->GetShadowRoot()) {
+      if (shadow->DelegatesFocus()) {
+        if (Element* delegatedFocus = shadow->GetFocusDelegate(aWithMouse)) {
+          if (autofocus) {
+            // This element has autofocus and we found an focus delegates
+            // in its descendants, so use the focus delegates
+            return delegatedFocus;
+          }
+          if (!potentialFocus) {
+            potentialFocus = delegatedFocus;
+          }
+        }
+      }
+    }
+  }
+
+  return potentialFocus;
+}
+
 bool nsIContent::IsFocusableInternal(int32_t* aTabIndex, bool aWithMouse) {
   if (aTabIndex) {
     *aTabIndex = -1;  // Default, not tabbable
@@ -1030,16 +1096,50 @@ bool nsIContent::IsFocusableInternal(int32_t* aTabIndex, bool aWithMouse) {
   return false;
 }
 
-bool FragmentOrElement::IsLink(nsIURI** aURI) const {
-  *aURI = nullptr;
-  return false;
-}
-
 void nsIContent::SetAssignedSlot(HTMLSlotElement* aSlot) {
   MOZ_ASSERT(aSlot || GetExistingExtendedContentSlots());
   ExtendedContentSlots()->mAssignedSlot = aSlot;
 }
 
+static Maybe<uint32_t> DoComputeFlatTreeIndexOf(FlattenedChildIterator& aIter,
+                                                const nsINode* aPossibleChild) {
+  if (aPossibleChild->GetFlattenedTreeParentNode() != aIter.Parent()) {
+    return Nothing();
+  }
+
+  uint32_t index = 0u;
+  for (nsIContent* child = aIter.GetNextChild(); child;
+       child = aIter.GetNextChild()) {
+    if (child == aPossibleChild) {
+      return Some(index);
+    }
+
+    ++index;
+  }
+
+  return Nothing();
+}
+
+Maybe<uint32_t> nsIContent::ComputeFlatTreeIndexOf(
+    const nsINode* aPossibleChild) const {
+  if (!aPossibleChild) {
+    return Nothing();
+  }
+
+  FlattenedChildIterator iter(this);
+  if (!iter.ShadowDOMInvolved()) {
+    auto index = ComputeIndexOf(aPossibleChild);
+    MOZ_ASSERT(DoComputeFlatTreeIndexOf(iter, aPossibleChild) == index);
+    return index;
+  }
+
+  return DoComputeFlatTreeIndexOf(iter, aPossibleChild);
+}
+
+#ifdef MOZ_DOM_LIST
+void nsIContent::Dump() { List(); }
+#endif
+
 void FragmentOrElement::GetTextContentInternal(nsAString& aTextContent,
                                                OOMReporter& aError) {
   if (!nsContentUtils::GetNodeTextContent(this, true, aTextContent, fallible)) {
@@ -1213,7 +1313,9 @@ ContentUnbinder* ContentUnbinder::sContentUnbinder = nullptr;
 
 void FragmentOrElement::ClearContentUnbinder() { ContentUnbinder::UnbindAll(); }
 
-NS_IMPL_CYCLE_COLLECTION_CLASS(FragmentOrElement)
+// Note, _INHERITED macro isn't used here since nsINode implementations are
+// rather special.
+NS_IMPL_CYCLE_COLLECTION_WRAPPERCACHE_CLASS(FragmentOrElement)
 
 // We purposefully don't UNLINK_BEGIN_INHERITED here.
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(FragmentOrElement)
@@ -1271,8 +1373,6 @@ NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(FragmentOrElement)
 
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
-NS_IMPL_CYCLE_COLLECTION_TRACE_WRAPPERCACHE(FragmentOrElement)
-
 void FragmentOrElement::MarkNodeChildren(nsINode* aNode) {
   JSObject* o = GetJSObjectChild(aNode);
   if (o) {
@@ -1300,15 +1400,13 @@ nsINode* FindOptimizableSubtreeRoot(nsINode* aNode) {
   return aNode;
 }
 
-StaticAutoPtr<nsTHashtable<nsPtrHashKey<nsINode>>> gCCBlackMarkedNodes;
+StaticAutoPtr<nsTHashSet<nsINode*>> gCCBlackMarkedNodes;
 
 static void ClearBlackMarkedNodes() {
   if (!gCCBlackMarkedNodes) {
     return;
   }
-  for (auto iter = gCCBlackMarkedNodes->ConstIter(); !iter.Done();
-       iter.Next()) {
-    nsINode* n = iter.Get()->GetKey();
+  for (nsINode* n : *gCCBlackMarkedNodes) {
     n->SetCCMarkedRoot(false);
     n->SetInCCBlackTree(false);
   }
@@ -1320,7 +1418,7 @@ void FragmentOrElement::RemoveBlackMarkedNode(nsINode* aNode) {
   if (!gCCBlackMarkedNodes) {
     return;
   }
-  gCCBlackMarkedNodes->RemoveEntry(aNode);
+  gCCBlackMarkedNodes->Remove(aNode);
 }
 
 static bool IsCertainlyAliveNode(nsINode* aNode, Document* aDoc) {
@@ -1366,7 +1464,7 @@ bool FragmentOrElement::CanSkipInCC(nsINode* aNode) {
   }
 
   if (!gCCBlackMarkedNodes) {
-    gCCBlackMarkedNodes = new nsTHashtable<nsPtrHashKey<nsINode>>(1020);
+    gCCBlackMarkedNodes = new nsTHashSet<nsINode*>(1020);
   }
 
   // nodesToUnpurple contains nodes which will be removed
@@ -1410,7 +1508,7 @@ bool FragmentOrElement::CanSkipInCC(nsINode* aNode) {
 
   root->SetCCMarkedRoot(true);
   root->SetInCCBlackTree(foundLiveWrapper);
-  gCCBlackMarkedNodes->PutEntry(root);
+  gCCBlackMarkedNodes->Insert(root);
 
   if (!foundLiveWrapper) {
     return false;
@@ -1425,7 +1523,7 @@ bool FragmentOrElement::CanSkipInCC(nsINode* aNode) {
     for (uint32_t i = 0; i < grayNodes.Length(); ++i) {
       nsINode* node = grayNodes[i];
       node->SetInCCBlackTree(true);
-      gCCBlackMarkedNodes->PutEntry(node);
+      gCCBlackMarkedNodes->Insert(node);
     }
   }
 
@@ -1719,8 +1817,7 @@ NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN_INTERNAL(FragmentOrElement)
           static_cast<IntersectionObserverList*>(
               elem->GetProperty(nsGkAtoms::intersectionobserverlist));
       if (observers) {
-        for (auto iter = observers->Iter(); !iter.Done(); iter.Next()) {
-          DOMIntersectionObserver* observer = iter.Key();
+        for (DOMIntersectionObserver* observer : observers->Keys()) {
           cb.NoteXPCOMChild(observer);
         }
       }
@@ -1791,7 +1888,7 @@ static inline bool IsVoidTag(nsAtom* aTag) {
       nsGkAtoms::link,    nsGkAtoms::meta,  nsGkAtoms::param,
       nsGkAtoms::source,  nsGkAtoms::track, nsGkAtoms::wbr};
 
-  static mozilla::BloomFilter<12, nsAtom> sFilter;
+  static mozilla::BitBloomFilter<12, nsAtom> sFilter;
   static bool sInitialized = false;
   if (!sInitialized) {
     sInitialized = true;
@@ -1972,21 +2069,6 @@ void FragmentOrElement::SetInnerHTMLInternal(const nsAString& aInnerHTML,
   }
 }
 
-void FragmentOrElement::FireNodeRemovedForChildren() {
-  Document* doc = OwnerDoc();
-  // Optimize the common case
-  if (!nsContentUtils::HasMutationListeners(
-          doc, NS_EVENT_BITS_MUTATION_NODEREMOVED)) {
-    return;
-  }
-
-  nsCOMPtr<nsINode> child;
-  for (child = GetFirstChild(); child && child->GetParentNode() == this;
-       child = child->GetNextSibling()) {
-    nsContentUtils::MaybeFireNodeRemoved(child, this);
-  }
-}
-
 void FragmentOrElement::AddSizeOfExcludingThis(nsWindowSizes& aSizes,
                                                size_t* aNodeSize) const {
   nsIContent::AddSizeOfExcludingThis(aSizes, aNodeSize);
