@@ -6,9 +6,10 @@
 
 #include "Link.h"
 
-#include "mozilla/EventStates.h"
 #include "mozilla/MemoryReporting.h"
 #include "mozilla/dom/Element.h"
+#include "mozilla/dom/SVGAElement.h"
+#include "mozilla/dom/HTMLDNSPrefetch.h"
 #include "mozilla/IHistory.h"
 #include "mozilla/StaticPrefs_layout.h"
 #include "nsLayoutUtils.h"
@@ -18,11 +19,10 @@
 
 #include "nsEscape.h"
 #include "nsGkAtoms.h"
-#include "nsHTMLDNSPrefetch.h"
 #include "nsString.h"
 #include "mozAutoDocUpdate.h"
 
-#include "mozilla/Services.h"
+#include "mozilla/Components.h"
 #include "nsAttrValueInlines.h"
 #include "HTMLLinkElement.h"
 
@@ -30,81 +30,55 @@ namespace mozilla::dom {
 
 Link::Link(Element* aElement)
     : mElement(aElement),
-      mLinkState(eLinkState_NotLink),
+      mState(State::NotLink),
       mNeedsRegistration(false),
       mRegistered(false),
       mHasPendingLinkUpdate(false),
-      mInDNSPrefetch(false),
       mHistory(true) {
   MOZ_ASSERT(mElement, "Must have an element");
 }
 
 Link::Link()
     : mElement(nullptr),
-      mLinkState(eLinkState_NotLink),
+      mState(State::NotLink),
       mNeedsRegistration(false),
       mRegistered(false),
       mHasPendingLinkUpdate(false),
-      mInDNSPrefetch(false),
       mHistory(false) {}
 
 Link::~Link() {
   // !mElement is for mock_Link.
   MOZ_ASSERT(!mElement || !mElement->IsInComposedDoc());
-  if (IsInDNSPrefetch()) {
-    nsHTMLDNSPrefetch::LinkDestroyed(this);
-  }
   UnregisterFromHistory();
 }
 
 bool Link::ElementHasHref() const {
-  return mElement->HasAttr(kNameSpaceID_None, nsGkAtoms::href) ||
-         (!mElement->IsHTMLElement() &&
-          mElement->HasAttr(kNameSpaceID_XLink, nsGkAtoms::href));
-}
-
-void Link::TryDNSPrefetch() {
-  MOZ_ASSERT(mElement->IsInComposedDoc());
-  if (ElementHasHref() && nsHTMLDNSPrefetch::IsAllowed(mElement->OwnerDoc())) {
-    nsHTMLDNSPrefetch::PrefetchLow(this);
+  if (mElement->HasAttr(nsGkAtoms::href)) {
+    return true;
   }
-}
-
-void Link::CancelDNSPrefetch(nsWrapperCache::FlagsType aDeferredFlag,
-                             nsWrapperCache::FlagsType aRequestedFlag) {
-  // If prefetch was deferred, clear flag and move on
-  if (mElement->HasFlag(aDeferredFlag)) {
-    mElement->UnsetFlags(aDeferredFlag);
-    // Else if prefetch was requested, clear flag and send cancellation
-  } else if (mElement->HasFlag(aRequestedFlag)) {
-    mElement->UnsetFlags(aRequestedFlag);
-    // Possible that hostname could have changed since binding, but since this
-    // covers common cases, most DNS prefetch requests will be canceled
-    nsHTMLDNSPrefetch::CancelPrefetchLow(this, NS_ERROR_ABORT);
+  if (const auto* svg = SVGAElement::FromNode(*mElement)) {
+    // This can be a HasAttr(kNameSpaceID_XLink, nsGkAtoms::href) check once
+    // SMIL is fixed to actually mutate DOM attributes rather than faking it.
+    return svg->HasHref();
   }
+  MOZ_ASSERT(!mElement->IsSVGElement(),
+             "What other SVG element inherits from Link?");
+  return false;
 }
 
 void Link::VisitedQueryFinished(bool aVisited) {
   MOZ_ASSERT(mRegistered, "Setting the link state of an unregistered Link!");
-  MOZ_ASSERT(mLinkState == eLinkState_Unvisited,
-             "Why would we want to know our visited state otherwise?");
 
-  auto newState = aVisited ? eLinkState_Visited : eLinkState_Unvisited;
+  auto newState = aVisited ? State::Visited : State::Unvisited;
 
   // Set our current state as appropriate.
-  mLinkState = newState;
+  mState = newState;
 
-  // We will be no longer registered if we're visited, as it'd be pointless, we
-  // never transition from visited -> unvisited.
-  if (aVisited) {
-    mRegistered = false;
-  }
-
-  MOZ_ASSERT(LinkState() == NS_EVENT_STATE_VISITED ||
-                 LinkState() == NS_EVENT_STATE_UNVISITED,
+  MOZ_ASSERT(LinkState() == ElementState::VISITED ||
+                 LinkState() == ElementState::UNVISITED,
              "Unexpected state obtained from LinkState()!");
 
-  // Tell the element to update its visited state
+  // Tell the element to update its visited state.
   mElement->UpdateState(true);
 
   if (StaticPrefs::layout_css_always_repaint_on_unvisited()) {
@@ -115,7 +89,7 @@ void Link::VisitedQueryFinished(bool aVisited) {
   }
 }
 
-EventStates Link::LinkState() const {
+ElementState Link::LinkState() const {
   // We are a constant method, but we are just lazily doing things and have to
   // track that state.  Cast away that constness!
   //
@@ -134,12 +108,12 @@ EventStates Link::LinkState() const {
     nsCOMPtr<nsIURI> hrefURI(GetURI());
 
     // Assume that we are not visited until we are told otherwise.
-    self->mLinkState = eLinkState_Unvisited;
+    self->mState = State::Unvisited;
 
     // Make sure the href attribute has a valid link (bug 23209).
     // If we have a good href, register with History if available.
     if (mHistory && hrefURI) {
-      if (nsCOMPtr<IHistory> history = services::GetHistory()) {
+      if (nsCOMPtr<IHistory> history = components::History::Service()) {
         self->mRegistered = true;
         history->RegisterVisitedCallback(hrefURI, self);
         // And make sure we are in the document's link map.
@@ -149,15 +123,15 @@ EventStates Link::LinkState() const {
   }
 
   // Otherwise, return our known state.
-  if (mLinkState == eLinkState_Visited) {
-    return NS_EVENT_STATE_VISITED;
+  if (mState == State::Visited) {
+    return ElementState::VISITED;
   }
 
-  if (mLinkState == eLinkState_Unvisited) {
-    return NS_EVENT_STATE_UNVISITED;
+  if (mState == State::Unvisited) {
+    return ElementState::UNVISITED;
   }
 
-  return EventStates();
+  return ElementState();
 }
 
 nsIURI* Link::GetURI() const {
@@ -349,9 +323,7 @@ void Link::GetOrigin(nsAString& aOrigin) {
 
 void Link::GetProtocol(nsAString& _protocol) {
   nsCOMPtr<nsIURI> uri(GetURI());
-  if (!uri) {
-    _protocol.AssignLiteral("http");
-  } else {
+  if (uri) {
     nsAutoCString scheme;
     (void)uri->GetScheme(scheme);
     CopyASCIItoUTF16(scheme, _protocol);
@@ -489,40 +461,31 @@ void Link::GetHash(nsAString& _hash) {
 }
 
 void Link::ResetLinkState(bool aNotify, bool aHasHref) {
-  nsLinkState defaultState;
-
-  // The default state for links with an href is unvisited.
-  if (aHasHref) {
-    defaultState = eLinkState_Unvisited;
-  } else {
-    defaultState = eLinkState_NotLink;
-  }
-
   // If !mNeedsRegstration, then either we've never registered, or we're
   // currently registered; in either case, we should remove ourself
   // from the doc and the history.
-  if (!mNeedsRegistration && mLinkState != eLinkState_NotLink) {
+  if (!mNeedsRegistration && mState != State::NotLink) {
     Document* doc = mElement->GetComposedDoc();
-    if (doc && (mRegistered || mLinkState == eLinkState_Visited)) {
+    if (doc && (mRegistered || mState == State::Visited)) {
       // Tell the document to forget about this link if we've registered
       // with it before.
       doc->ForgetLink(this);
     }
   }
 
-  // If we have an href, and we're not a <link>, we should register with the
-  // history.
+  // If we have an href, we should register with the history.
   //
   // FIXME(emilio): Do we really want to allow all MathML elements to be
   // :visited? That seems not great.
-  mNeedsRegistration = aHasHref && !mElement->IsHTMLElement(nsGkAtoms::link);
+  mNeedsRegistration = aHasHref;
 
   // If we've cached the URI, reset always invalidates it.
   UnregisterFromHistory();
   mCachedURI = nullptr;
 
-  // Update our state back to the default.
-  mLinkState = defaultState;
+  // Update our state back to the default; the default state for links with an
+  // href is unvisited.
+  mState = aHasHref ? State::Unvisited : State::NotLink;
 
   // We have to be very careful here: if aNotify is false we do NOT
   // want to call UpdateState, because that will call into LinkState()
@@ -535,10 +498,10 @@ void Link::ResetLinkState(bool aNotify, bool aHasHref) {
   if (aNotify) {
     mElement->UpdateState(aNotify);
   } else {
-    if (mLinkState == eLinkState_Unvisited) {
-      mElement->UpdateLinkState(NS_EVENT_STATE_UNVISITED);
+    if (mState == State::Unvisited) {
+      mElement->UpdateLinkState(ElementState::UNVISITED);
     } else {
-      mElement->UpdateLinkState(EventStates());
+      mElement->UpdateLinkState(ElementState());
     }
   }
 }
@@ -551,7 +514,7 @@ void Link::UnregisterFromHistory() {
 
   // And tell History to stop tracking us.
   if (mHistory && mCachedURI) {
-    if (nsCOMPtr<IHistory> history = services::GetHistory()) {
+    if (nsCOMPtr<IHistory> history = components::History::Service()) {
       history->UnregisterVisitedCallback(mCachedURI, this);
       mRegistered = false;
     }
@@ -573,11 +536,8 @@ void Link::SetHrefAttribute(nsIURI* aURI) {
 size_t Link::SizeOfExcludingThis(mozilla::SizeOfState& aState) const {
   size_t n = 0;
 
-  if (mCachedURI) {
-    nsCOMPtr<nsISizeOf> iface = do_QueryInterface(mCachedURI);
-    if (iface) {
-      n += iface->SizeOfIncludingThis(aState.mMallocSizeOf);
-    }
+  if (nsCOMPtr<nsISizeOf> iface = do_QueryInterface(mCachedURI)) {
+    n += iface->SizeOfIncludingThis(aState.mMallocSizeOf);
   }
 
   // The following members don't need to be measured:
