@@ -8,37 +8,41 @@
 
 #include "js/SavedFrameAPI.h"
 #include "mozilla/dom/WorkerPrivate.h"
+#include "mozilla/dom/ScriptSettings.h"
+#include "mozilla/Services.h"
+#include "nsJSPrincipals.h"
+#include "nsIObserverService.h"
+#include "xpcpublic.h"
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 SerializedStackHolder::SerializedStackHolder()
-  : mHolder(StructuredCloneHolder::CloningSupported,
-            StructuredCloneHolder::TransferringNotSupported,
-            StructuredCloneHolder::StructuredCloneScope::SameProcessDifferentThread) {}
+    : mHolder(StructuredCloneHolder::CloningSupported,
+              StructuredCloneHolder::TransferringNotSupported,
+              StructuredCloneHolder::StructuredCloneScope::SameProcess) {}
 
 void SerializedStackHolder::WriteStack(JSContext* aCx,
-                                       JS::HandleObject aStack) {
-  JS::RootedValue stackValue(aCx, JS::ObjectValue(*aStack));
+                                       JS::Handle<JSObject*> aStack) {
+  JS::Rooted<JS::Value> stackValue(aCx, JS::ObjectValue(*aStack));
   mHolder.Write(aCx, stackValue, IgnoreErrors());
 
   // StructuredCloneHolder::Write can leave a pending exception on the context.
   JS_ClearPendingException(aCx);
 }
 
-void SerializedStackHolder::SerializeMainThreadStack(JSContext* aCx,
-                                                     JS::HandleObject aStack) {
-  MOZ_ASSERT(NS_IsMainThread());
+void SerializedStackHolder::SerializeMainThreadOrWorkletStack(
+    JSContext* aCx, JS::Handle<JSObject*> aStack) {
+  MOZ_ASSERT(!IsCurrentThreadRunningWorker());
   WriteStack(aCx, aStack);
 }
 
 void SerializedStackHolder::SerializeWorkerStack(JSContext* aCx,
                                                  WorkerPrivate* aWorkerPrivate,
-                                                 JS::HandleObject aStack) {
+                                                 JS::Handle<JSObject*> aStack) {
   MOZ_ASSERT(aWorkerPrivate->IsOnCurrentThread());
 
-  RefPtr<StrongWorkerRef> workerRef = StrongWorkerRef::Create(
-      aWorkerPrivate, "WorkerErrorReport");
+  RefPtr<StrongWorkerRef> workerRef =
+      StrongWorkerRef::Create(aWorkerPrivate, "WorkerErrorReport");
   if (workerRef) {
     mWorkerRef = new ThreadSafeWorkerRef(workerRef);
   } else {
@@ -50,7 +54,7 @@ void SerializedStackHolder::SerializeWorkerStack(JSContext* aCx,
 }
 
 void SerializedStackHolder::SerializeCurrentStack(JSContext* aCx) {
-  JS::RootedObject stack(aCx);
+  JS::Rooted<JSObject*> stack(aCx);
   if (JS::CurrentGlobalOrNull(aCx) && !JS::CaptureCurrentStack(aCx, &stack)) {
     JS_ClearPendingException(aCx);
     return;
@@ -58,7 +62,7 @@ void SerializedStackHolder::SerializeCurrentStack(JSContext* aCx) {
 
   if (stack) {
     if (NS_IsMainThread()) {
-      SerializeMainThreadStack(aCx, stack);
+      SerializeMainThreadOrWorkletStack(aCx, stack);
     } else {
       WorkerPrivate* currentWorker = GetCurrentThreadWorkerPrivate();
       SerializeWorkerStack(aCx, currentWorker, stack);
@@ -72,28 +76,28 @@ JSObject* SerializedStackHolder::ReadStack(JSContext* aCx) {
     return nullptr;
   }
 
-  Maybe<nsJSPrincipals::AutoSetActiveWorkerPrincipal> set;
-  if (mWorkerRef) {
-    set.emplace(mWorkerRef->Private()->GetPrincipal());
-  }
+  JS::Rooted<JS::Value> stackValue(aCx);
+
+  mHolder.Read(xpc::CurrentNativeGlobal(aCx), aCx, &stackValue, IgnoreErrors());
 
-  JS::RootedValue stackValue(aCx);
-  mHolder.Read(xpc::CurrentNativeGlobal(aCx), aCx, &stackValue,
-               IgnoreErrors());
   return stackValue.isObject() ? &stackValue.toObject() : nullptr;
 }
 
 UniquePtr<SerializedStackHolder> GetCurrentStackForNetMonitor(JSContext* aCx) {
   MOZ_ASSERT_IF(!NS_IsMainThread(),
-                GetCurrentThreadWorkerPrivate()->IsWatchedByDevtools());
+                GetCurrentThreadWorkerPrivate()->IsWatchedByDevTools());
 
+  return GetCurrentStack(aCx);
+}
+
+UniquePtr<SerializedStackHolder> GetCurrentStack(JSContext* aCx) {
   UniquePtr<SerializedStackHolder> stack = MakeUnique<SerializedStackHolder>();
   stack->SerializeCurrentStack(aCx);
   return stack;
 }
 
-void NotifyNetworkMonitorAlternateStack(nsISupports* aChannel,
-                                        UniquePtr<SerializedStackHolder> aStackHolder) {
+void NotifyNetworkMonitorAlternateStack(
+    nsISupports* aChannel, UniquePtr<SerializedStackHolder> aStackHolder) {
   if (!aStackHolder) {
     return;
   }
@@ -115,20 +119,20 @@ void ConvertSerializedStackToJSON(UniquePtr<SerializedStackHolder> aStackHolder,
   DebugOnly<bool> ok = jsapi.Init(xpc::PrivilegedJunkScope());
   JSContext* cx = jsapi.cx();
 
-  JS::RootedObject savedFrame(cx, aStackHolder->ReadStack(cx));
+  JS::Rooted<JSObject*> savedFrame(cx, aStackHolder->ReadStack(cx));
   if (!savedFrame) {
     return;
   }
 
-  JS::RootedObject converted(cx);
-  converted = JS::ConvertSavedFrameToPlainObject(cx, savedFrame,
-                                                 JS::SavedFrameSelfHosted::Exclude);
+  JS::Rooted<JSObject*> converted(cx);
+  converted = JS::ConvertSavedFrameToPlainObject(
+      cx, savedFrame, JS::SavedFrameSelfHosted::Exclude);
   if (!converted) {
     JS_ClearPendingException(cx);
     return;
   }
 
-  JS::RootedValue convertedValue(cx, JS::ObjectValue(*converted));
+  JS::Rooted<JS::Value> convertedValue(cx, JS::ObjectValue(*converted));
   if (!nsContentUtils::StringifyJSON(cx, &convertedValue, aStackString)) {
     JS_ClearPendingException(cx);
     return;
@@ -146,5 +150,4 @@ void NotifyNetworkMonitorAlternateStack(nsISupports* aChannel,
                               PromiseFlatString(aStackJSON).get());
 }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
