@@ -4,24 +4,41 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "StructuredCloneHolder.h"
+#include "mozilla/dom/StructuredCloneHolder.h"
 
-#include "ImageContainer.h"
+#include <new>
+#include "ErrorList.h"
+#include "MainThreadUtils.h"
+#include "js/CallArgs.h"
+#include "js/Value.h"
+#include "js/WasmModule.h"
+#include "js/Wrapper.h"
+#include "jsapi.h"
+#include "mozilla/AlreadyAddRefed.h"
 #include "mozilla/AutoRestore.h"
+#include "mozilla/ErrorResult.h"
+#include "mozilla/OwningNonNull.h"
+#include "mozilla/RefPtr.h"
+#include "mozilla/ScopeExit.h"
+#include "mozilla/StaticPrefs_dom.h"
+#include "mozilla/dom/BindingDeclarations.h"
 #include "mozilla/dom/BindingUtils.h"
+#include "mozilla/dom/Blob.h"
 #include "mozilla/dom/BlobBinding.h"
+#include "mozilla/dom/BlobImpl.h"
 #include "mozilla/dom/BrowsingContext.h"
-#include "mozilla/dom/BrowsingContextBinding.h"
 #include "mozilla/dom/ClonedErrorHolder.h"
 #include "mozilla/dom/ClonedErrorHolderBinding.h"
-#include "mozilla/dom/StructuredCloneBlob.h"
-#include "mozilla/dom/DocGroup.h"
-#include "mozilla/dom/Directory.h"
 #include "mozilla/dom/DirectoryBinding.h"
+#include "mozilla/dom/DOMJSClass.h"
+#include "mozilla/dom/DOMTypes.h"
+#include "mozilla/dom/Directory.h"
+#include "mozilla/dom/DocGroup.h"
 #include "mozilla/dom/File.h"
 #include "mozilla/dom/FileList.h"
 #include "mozilla/dom/FileListBinding.h"
 #include "mozilla/dom/FormData.h"
+#include "mozilla/dom/FormDataBinding.h"
 #include "mozilla/dom/ImageBitmap.h"
 #include "mozilla/dom/ImageBitmapBinding.h"
 #include "mozilla/dom/JSExecutionManager.h"
@@ -29,18 +46,40 @@
 #include "mozilla/dom/MessagePortBinding.h"
 #include "mozilla/dom/OffscreenCanvas.h"
 #include "mozilla/dom/OffscreenCanvasBinding.h"
-#include "mozilla/dom/PMessagePort.h"
+#include "mozilla/dom/ReadableStream.h"
+#include "mozilla/dom/ReadableStreamBinding.h"
 #include "mozilla/dom/ScriptSettings.h"
+#include "mozilla/dom/StructuredCloneBlob.h"
+#include "mozilla/dom/StructuredCloneHolderBinding.h"
 #include "mozilla/dom/StructuredCloneTags.h"
 #include "mozilla/dom/ToJSValue.h"
+#include "mozilla/dom/TransformStream.h"
+#include "mozilla/dom/TransformStreamBinding.h"
+#include "mozilla/dom/VideoFrame.h"
+#include "mozilla/dom/VideoFrameBinding.h"
 #include "mozilla/dom/WebIDLSerializable.h"
+#include "mozilla/dom/WritableStream.h"
+#include "mozilla/dom/WritableStreamBinding.h"
+#include "mozilla/dom/WorkerCommon.h"
 #include "mozilla/dom/WorkerPrivate.h"
+#include "mozilla/fallible.h"
 #include "mozilla/gfx/2D.h"
-#include "mozilla/ipc/BackgroundChild.h"
-#include "mozilla/ipc/BackgroundUtils.h"
-#include "mozilla/ipc/PBackgroundSharedTypes.h"
-#include "MultipartBlobImpl.h"
-#include "nsQueryObject.h"
+#include "nsContentUtils.h"
+#include "nsDebug.h"
+#include "nsError.h"
+#include "nsID.h"
+#include "nsIEventTarget.h"
+#include "nsIFile.h"
+#include "nsIGlobalObject.h"
+#include "nsIInputStream.h"
+#include "nsIPrincipal.h"
+#include "nsISupports.h"
+#include "nsJSPrincipals.h"
+#include "nsPIDOMWindow.h"
+#include "nsString.h"
+#include "nsThreadUtils.h"
+#include "nsXPCOM.h"
+#include "xpcpublic.h"
 
 using namespace mozilla::ipc;
 
@@ -74,7 +113,7 @@ bool StructuredCloneCallbacksWrite(JSContext* aCx,
 bool StructuredCloneCallbacksReadTransfer(
     JSContext* aCx, JSStructuredCloneReader* aReader, uint32_t aTag,
     void* aContent, uint64_t aExtraData, void* aClosure,
-    JS::MutableHandleObject aReturnObject) {
+    JS::MutableHandle<JSObject*> aReturnObject) {
   StructuredCloneHolderBase* holder =
       static_cast<StructuredCloneHolderBase*>(aClosure);
   MOZ_ASSERT(holder);
@@ -173,7 +212,10 @@ void AssertTagValues() {
                     SCTAG_DOM_DOMMATRIX == 0xffff8013 &&
                     SCTAG_DOM_URLSEARCHPARAMS == 0xffff8014 &&
                     SCTAG_DOM_DOMMATRIXREADONLY == 0xffff8015 &&
-                    SCTAG_DOM_STRUCTUREDCLONETESTER == 0xffff8018,
+                    SCTAG_DOM_STRUCTUREDCLONETESTER == 0xffff8018 &&
+                    SCTAG_DOM_FILESYSTEMHANDLE == 0xffff8019 &&
+                    SCTAG_DOM_FILESYSTEMFILEHANDLE == 0xffff801a &&
+                    SCTAG_DOM_FILESYSTEMDIRECTORYHANDLE == 0xffff801b,
                 "Something has changed the sctag values. This is wrong!");
 }
 
@@ -263,7 +305,7 @@ bool StructuredCloneHolderBase::Read(
 bool StructuredCloneHolderBase::CustomReadTransferHandler(
     JSContext* aCx, JSStructuredCloneReader* aReader, uint32_t aTag,
     void* aContent, uint64_t aExtraData,
-    JS::MutableHandleObject aReturnObject) {
+    JS::MutableHandle<JSObject*> aReturnObject) {
   MOZ_CRASH("Nothing to read.");
   return false;
 }
@@ -353,6 +395,7 @@ void StructuredCloneHolder::Read(nsIGlobalObject* aGlobal, JSContext* aCx,
     mWasmModuleArray.Clear();
     mClonedSurfaces.Clear();
     mInputStreamArray.Clear();
+    mVideoFrameImages.Clear();
     Clear();
   }
 }
@@ -407,7 +450,7 @@ JSObject* StructuredCloneHolder::ReadFullySerializableObjects(
       return nullptr;
     }
 
-    JS::RootedValue result(aCx);
+    JS::Rooted<JS::Value> result(aCx);
     {
       // nsJSPrincipals::ReadKnownPrincipalType addrefs for us, but because of
       // the casting between JSPrincipals* and nsIPrincipal* we can't use
@@ -459,7 +502,11 @@ bool StructuredCloneHolder::WriteFullySerializableObjects(
   }
 
   // Don't know what this is
-  xpc::Throw(aCx, NS_ERROR_DOM_DATA_CLONE_ERR);
+  ErrorResult rv;
+  const char* className = JS::GetClass(obj)->name;
+  rv.ThrowDataCloneError(nsDependentCString(className) +
+                         " object could not be cloned."_ns);
+  MOZ_ALWAYS_TRUE(rv.MaybeSetPendingException(aCx));
   return false;
 }
 
@@ -620,12 +667,10 @@ JSObject* ReadFileList(JSContext* aCx, JSStructuredCloneReader* aReader,
 
     uint32_t zero, index;
     // |index| is the index of the first blobImpl.
-    if (!JS_ReadUint32Pair(aReader, &zero, &index)) {
+    if (!JS_ReadUint32Pair(aReader, &zero, &index) || zero != 0) {
       return nullptr;
     }
 
-    MOZ_ASSERT(zero == 0);
-
     // |aCount| is the number of BlobImpls to use from the |index|.
     for (uint32_t i = 0; i < aCount; ++i) {
       uint32_t pos = index + i;
@@ -741,7 +786,9 @@ JSObject* ReadFormData(JSContext* aCx, JSStructuredCloneReader* aReader,
         formData->Append(name, directory);
 
       } else {
-        MOZ_ASSERT(tag == 0);
+        if (NS_WARN_IF(tag != 0)) {
+          return nullptr;
+        }
 
         nsAutoString value;
         if (NS_WARN_IF(!value.SetLength(indexOrLengthOfString, fallible))) {
@@ -884,7 +931,7 @@ JSObject* ReadInputStream(JSContext* aCx, uint32_t aIndex,
   }
 #endif
   MOZ_ASSERT(aIndex < aHolder->InputStreams().Length());
-  JS::RootedValue result(aCx);
+  JS::Rooted<JS::Value> result(aCx);
   {
     nsCOMPtr<nsIInputStream> inputStream = aHolder->InputStreams()[aIndex];
 
@@ -943,7 +990,7 @@ JSObject* StructuredCloneHolder::CustomReadHandler(
       CloneScope() == StructuredCloneScope::SameProcess) {
     // Get the current global object.
     // This can be null.
-    JS::RootedObject result(aCx);
+    JS::Rooted<JSObject*> result(aCx);
     {
       // aIndex is the index of the cloned image.
       result = ImageBitmap::ReadStructuredClone(aCx, aReader, mGlobal,
@@ -974,6 +1021,13 @@ JSObject* StructuredCloneHolder::CustomReadHandler(
     return ClonedErrorHolder::ReadStructuredClone(aCx, aReader, this);
   }
 
+  if (StaticPrefs::dom_media_webcodecs_enabled() &&
+      aTag == SCTAG_DOM_VIDEOFRAME &&
+      CloneScope() == StructuredCloneScope::SameProcess) {
+    return VideoFrame::ReadStructuredClone(aCx, mGlobal, aReader,
+                                           VideoFrameImages()[aIndex]);
+  }
+
   return ReadFullySerializableObjects(aCx, aReader, aTag);
 }
 
@@ -1022,15 +1076,13 @@ bool StructuredCloneHolder::CustomWriteHandler(
   {
     ImageBitmap* imageBitmap = nullptr;
     if (NS_SUCCEEDED(UNWRAP_OBJECT(ImageBitmap, &obj, imageBitmap))) {
-      if (imageBitmap->IsWriteOnly()) {
-        return false;
-      }
-
       SameProcessScopeRequired(aSameProcessScopeRequired);
 
       if (CloneScope() == StructuredCloneScope::SameProcess) {
-        return ImageBitmap::WriteStructuredClone(aWriter, GetSurfaces(),
-                                                 imageBitmap);
+        ErrorResult rv;
+        ImageBitmap::WriteStructuredClone(aWriter, GetSurfaces(), imageBitmap,
+                                          rv);
+        return !rv.MaybeSetPendingException(aCx);
       }
       return false;
     }
@@ -1072,6 +1124,17 @@ bool StructuredCloneHolder::CustomWriteHandler(
     return false;
   }
 
+  // See if this is a VideoFrame object.
+  if (StaticPrefs::dom_media_webcodecs_enabled()) {
+    VideoFrame* videoFrame = nullptr;
+    if (NS_SUCCEEDED(UNWRAP_OBJECT(VideoFrame, &obj, videoFrame))) {
+      SameProcessScopeRequired(aSameProcessScopeRequired);
+      return CloneScope() == StructuredCloneScope::SameProcess
+                 ? videoFrame->WriteStructuredClone(aWriter, this)
+                 : false;
+    }
+  }
+
   {
     // We only care about streams, so ReflectorToISupportsStatic is fine.
     nsCOMPtr<nsISupports> base = xpc::ReflectorToISupportsStatic(aObj);
@@ -1084,10 +1147,29 @@ bool StructuredCloneHolder::CustomWriteHandler(
   return WriteFullySerializableObjects(aCx, aWriter, aObj);
 }
 
-bool StructuredCloneHolder::CustomReadTransferHandler(
+already_AddRefed<MessagePort> StructuredCloneHolder::ReceiveMessagePort(
+    uint64_t aIndex) {
+  if (NS_WARN_IF(aIndex >= mPortIdentifiers.Length())) {
+    return nullptr;
+  }
+  UniqueMessagePortId portId(mPortIdentifiers[aIndex]);
+
+  ErrorResult rv;
+  RefPtr<MessagePort> port = MessagePort::Create(mGlobal, portId, rv);
+  if (NS_WARN_IF(rv.Failed())) {
+    rv.SuppressException();
+    return nullptr;
+  }
+
+  return port.forget();
+}
+
+// TODO: Convert this to MOZ_CAN_RUN_SCRIPT (bug 1415230)
+MOZ_CAN_RUN_SCRIPT_BOUNDARY bool
+StructuredCloneHolder::CustomReadTransferHandler(
     JSContext* aCx, JSStructuredCloneReader* aReader, uint32_t aTag,
     void* aContent, uint64_t aExtraData,
-    JS::MutableHandleObject aReturnObject) {
+    JS::MutableHandle<JSObject*> aReturnObject) {
   MOZ_ASSERT(mSupportsTransferring);
 
   if (aTag == SCTAG_DOM_MAP_MESSAGEPORT) {
@@ -1096,16 +1178,10 @@ bool StructuredCloneHolder::CustomReadTransferHandler(
       return false;
     }
 #endif
-    MOZ_ASSERT(aExtraData < mPortIdentifiers.Length());
-    UniqueMessagePortId portIdentifier(mPortIdentifiers[aExtraData]);
-
-    ErrorResult rv;
-    RefPtr<MessagePort> port = MessagePort::Create(mGlobal, portIdentifier, rv);
-    if (NS_WARN_IF(rv.Failed())) {
-      rv.SuppressException();
+    RefPtr<MessagePort> port = ReceiveMessagePort(aExtraData);
+    if (!port) {
       return false;
     }
-
     mTransferredPorts.AppendElement(port);
 
     JS::Rooted<JS::Value> value(aCx);
@@ -1155,10 +1231,56 @@ bool StructuredCloneHolder::CustomReadTransferHandler(
     return true;
   }
 
+  if (aTag == SCTAG_DOM_READABLESTREAM) {
+#ifdef FUZZING
+    if (aExtraData >= mPortIdentifiers.Length()) {
+      return false;
+    }
+#endif
+    RefPtr<MessagePort> port = ReceiveMessagePort(aExtraData);
+    if (!port) {
+      return false;
+    }
+    nsCOMPtr<nsIGlobalObject> global = mGlobal;
+    return ReadableStream::ReceiveTransfer(aCx, global, *port, aReturnObject);
+  }
+
+  if (aTag == SCTAG_DOM_WRITABLESTREAM) {
+#ifdef FUZZING
+    if (aExtraData >= mPortIdentifiers.Length()) {
+      return false;
+    }
+#endif
+    RefPtr<MessagePort> port = ReceiveMessagePort(aExtraData);
+    if (!port) {
+      return false;
+    }
+    nsCOMPtr<nsIGlobalObject> global = mGlobal;
+    return WritableStream::ReceiveTransfer(aCx, global, *port, aReturnObject);
+  }
+
+  if (aTag == SCTAG_DOM_TRANSFORMSTREAM) {
+#ifdef FUZZING
+    if (aExtraData + 1 >= mPortIdentifiers.Length()) {
+      return false;
+    }
+#endif
+    RefPtr<MessagePort> port1 = ReceiveMessagePort(aExtraData);
+    RefPtr<MessagePort> port2 = ReceiveMessagePort(aExtraData + 1);
+    if (!port1 || !port2) {
+      return false;
+    }
+    nsCOMPtr<nsIGlobalObject> global = mGlobal;
+    return TransformStream::ReceiveTransfer(aCx, global, *port1, *port2,
+                                            aReturnObject);
+  }
+
   return false;
 }
 
-bool StructuredCloneHolder::CustomWriteTransferHandler(
+// TODO: Convert this to MOZ_CAN_RUN_SCRIPT (bug 1415230)
+MOZ_CAN_RUN_SCRIPT_BOUNDARY bool
+StructuredCloneHolder::CustomWriteTransferHandler(
     JSContext* aCx, JS::Handle<JSObject*> aObj, uint32_t* aTag,
     JS::TransferableOwnership* aOwnership, void** aContent,
     uint64_t* aExtraData) {
@@ -1196,7 +1318,7 @@ bool StructuredCloneHolder::CustomWriteTransferHandler(
       if (NS_SUCCEEDED(rv)) {
         MOZ_ASSERT(canvas);
 
-        if (canvas->IsNeutered()) {
+        if (!canvas->MayNeuter()) {
           return false;
         }
 
@@ -1232,6 +1354,68 @@ bool StructuredCloneHolder::CustomWriteTransferHandler(
         return true;
       }
     }
+
+    {
+      RefPtr<ReadableStream> stream;
+      rv = UNWRAP_OBJECT(ReadableStream, &obj, stream);
+      if (NS_SUCCEEDED(rv)) {
+        MOZ_ASSERT(stream);
+
+        *aTag = SCTAG_DOM_READABLESTREAM;
+        *aOwnership = JS::SCTAG_TMO_CUSTOM;
+        *aContent = nullptr;
+
+        UniqueMessagePortId id;
+        if (!stream->Transfer(aCx, id)) {
+          return false;
+        }
+        *aExtraData = mPortIdentifiers.Length();
+        mPortIdentifiers.AppendElement(id.release());
+        return true;
+      }
+    }
+
+    {
+      RefPtr<WritableStream> stream;
+      rv = UNWRAP_OBJECT(WritableStream, &obj, stream);
+      if (NS_SUCCEEDED(rv)) {
+        MOZ_ASSERT(stream);
+
+        *aTag = SCTAG_DOM_WRITABLESTREAM;
+        *aOwnership = JS::SCTAG_TMO_CUSTOM;
+        *aContent = nullptr;
+
+        UniqueMessagePortId id;
+        if (!stream->Transfer(aCx, id)) {
+          return false;
+        }
+        *aExtraData = mPortIdentifiers.Length();
+        mPortIdentifiers.AppendElement(id.release());
+        return true;
+      }
+    }
+
+    {
+      RefPtr<TransformStream> stream;
+      rv = UNWRAP_OBJECT(TransformStream, &obj, stream);
+      if (NS_SUCCEEDED(rv)) {
+        MOZ_ASSERT(stream);
+
+        *aTag = SCTAG_DOM_TRANSFORMSTREAM;
+        *aOwnership = JS::SCTAG_TMO_CUSTOM;
+        *aContent = nullptr;
+
+        UniqueMessagePortId id1;
+        UniqueMessagePortId id2;
+        if (!stream->Transfer(aCx, id1, id2)) {
+          return false;
+        }
+        *aExtraData = mPortIdentifiers.Length();
+        mPortIdentifiers.AppendElement(id1.release());
+        mPortIdentifiers.AppendElement(id2.release());
+        return true;
+      }
+    }
   }
 
   return false;
@@ -1270,6 +1454,31 @@ void StructuredCloneHolder::CustomFreeTransferHandler(
     delete data;
     return;
   }
+
+  if (aTag == SCTAG_DOM_READABLESTREAM || aTag == SCTAG_DOM_WRITABLESTREAM) {
+    MOZ_ASSERT(!aContent);
+#ifdef FUZZING
+    if (aExtraData >= mPortIdentifiers.Length()) {
+      return;
+    }
+#endif
+    MOZ_ASSERT(aExtraData < mPortIdentifiers.Length());
+    MessagePort::ForceClose(mPortIdentifiers[aExtraData]);
+    return;
+  }
+
+  if (aTag == SCTAG_DOM_TRANSFORMSTREAM) {
+    MOZ_ASSERT(!aContent);
+#ifdef FUZZING
+    if (aExtraData + 1 >= mPortIdentifiers.Length()) {
+      return;
+    }
+#endif
+    MOZ_ASSERT(aExtraData + 1 < mPortIdentifiers.Length());
+    MessagePort::ForceClose(mPortIdentifiers[aExtraData]);
+    MessagePort::ForceClose(mPortIdentifiers[aExtraData + 1]);
+    return;
+  }
 }
 
 bool StructuredCloneHolder::CustomCanTransferHandler(
@@ -1311,6 +1520,40 @@ bool StructuredCloneHolder::CustomCanTransferHandler(
     }
   }
 
+  {
+    ReadableStream* stream = nullptr;
+    nsresult rv = UNWRAP_OBJECT(ReadableStream, &obj, stream);
+    if (NS_SUCCEEDED(rv)) {
+      // https://streams.spec.whatwg.org/#ref-for-transfer-steps
+      // Step 1: If ! IsReadableStreamLocked(value) is true, throw a
+      // "DataCloneError" DOMException.
+      return !IsReadableStreamLocked(stream);
+    }
+  }
+
+  {
+    WritableStream* stream = nullptr;
+    nsresult rv = UNWRAP_OBJECT(WritableStream, &obj, stream);
+    if (NS_SUCCEEDED(rv)) {
+      // https://streams.spec.whatwg.org/#ref-for-transfer-steps①
+      // Step 1: If ! IsWritableStreamLocked(value) is true, throw a
+      // "DataCloneError" DOMException.
+      return !IsWritableStreamLocked(stream);
+    }
+  }
+
+  {
+    TransformStream* stream = nullptr;
+    nsresult rv = UNWRAP_OBJECT(TransformStream, &obj, stream);
+    if (NS_SUCCEEDED(rv)) {
+      // https://streams.spec.whatwg.org/#ref-for-transfer-steps②
+      // Step 3 + 4: If ! Is{Readable,Writable}StreamLocked(value) is true,
+      // throw a "DataCloneError" DOMException.
+      return !IsReadableStreamLocked(stream->Readable()) &&
+             !IsWritableStreamLocked(stream->Writable());
+    }
+  }
+
   return false;
 }
 
