@@ -89,7 +89,7 @@ inline const char* NS_CP_ResponseName(int16_t response) {
  * @param contentType the content type code
  * @return the name of the given content type code
  */
-inline const char* NS_CP_ContentTypeName(uint32_t contentType) {
+inline const char* NS_CP_ContentTypeName(nsContentPolicyType contentType) {
   switch (contentType) {
     CASE_RETURN(TYPE_OTHER);
     CASE_RETURN(TYPE_SCRIPT);
@@ -98,7 +98,6 @@ inline const char* NS_CP_ContentTypeName(uint32_t contentType) {
     CASE_RETURN(TYPE_OBJECT);
     CASE_RETURN(TYPE_DOCUMENT);
     CASE_RETURN(TYPE_SUBDOCUMENT);
-    CASE_RETURN(TYPE_REFRESH);
     CASE_RETURN(TYPE_PING);
     CASE_RETURN(TYPE_XMLHTTPREQUEST);
     CASE_RETURN(TYPE_OBJECT_SUBREQUEST);
@@ -144,13 +143,22 @@ inline const char* NS_CP_ContentTypeName(uint32_t contentType) {
     CASE_RETURN(TYPE_INTERNAL_CHROMEUTILS_COMPILED_SCRIPT);
     CASE_RETURN(TYPE_INTERNAL_FRAME_MESSAGEMANAGER_SCRIPT);
     CASE_RETURN(TYPE_INTERNAL_FETCH_PRELOAD);
-    default:
-      return "<Unknown Type>";
+    CASE_RETURN(TYPE_UA_FONT);
+    CASE_RETURN(TYPE_PROXIED_WEBRTC_MEDIA);
+    CASE_RETURN(TYPE_WEB_IDENTITY);
+    case nsIContentPolicy::TYPE_INVALID:
+      break;
+      // Do not add default: so that compilers can catch the missing case.
   }
+  return "<Unknown Type>";
 }
 
 #undef CASE_RETURN
 
+inline const char* NS_CP_ContentTypeName(ExtContentPolicyType contentType) {
+  return NS_CP_ContentTypeName(static_cast<nsContentPolicyType>(contentType));
+}
+
 /* Passes on parameters from its "caller"'s context. */
 #define CHECK_CONTENT_POLICY(action)                                    \
   PR_BEGIN_MACRO                                                        \
@@ -185,7 +193,8 @@ inline const char* NS_CP_ContentTypeName(uint32_t contentType) {
     if (NS_CP_REJECTED(*decision)) {                                           \
       return NS_OK;                                                            \
     }                                                                          \
-    if (contentType != nsIContentPolicy::TYPE_DOCUMENT) {                      \
+    if (contentType != nsIContentPolicy::TYPE_DOCUMENT &&                      \
+        contentType != nsIContentPolicy::TYPE_UA_FONT) {                       \
       *decision = nsIContentPolicy::ACCEPT;                                    \
       nsCOMPtr<nsINode> n = do_QueryInterface(context);                        \
       if (!n) {                                                                \
