@@ -19,6 +19,8 @@ DEPRECATED_OPERATION(UseOfReleaseEvents)
 DEPRECATED_OPERATION(SyncXMLHttpRequest)
 DEPRECATED_OPERATION(Window_Cc_ontrollers)
 DEPRECATED_OPERATION(ImportXULIntoContent)
+DEPRECATED_OPERATION(InstallTriggerDeprecated)
+DEPRECATED_OPERATION(InstallTriggerInstallDeprecated)
 DEPRECATED_OPERATION(NavigatorGetUserMedia)
 DEPRECATED_OPERATION(WebrtcDeprecatedPrefix)
 DEPRECATED_OPERATION(RTCPeerConnectionGetStreams)
@@ -38,21 +40,29 @@ DEPRECATED_OPERATION(DeprecatedTestingInterface)
 DEPRECATED_OPERATION(DeprecatedTestingMethod)
 DEPRECATED_OPERATION(DeprecatedTestingAttribute)
 DEPRECATED_OPERATION(CreateImageBitmapCanvasRenderingContext2D)
+DEPRECATED_OPERATION(DrawWindowCanvasRenderingContext2D)
+DEPRECATED_OPERATION(MozTextStyle)
 DEPRECATED_OPERATION(MozRequestFullScreenDeprecatedPrefix)
 DEPRECATED_OPERATION(MozfullscreenchangeDeprecatedPrefix)
 DEPRECATED_OPERATION(MozfullscreenerrorDeprecatedPrefix)
 DEPRECATED_OPERATION(External_AddSearchProvider)
 DEPRECATED_OPERATION(MouseEvent_MozPressure)
-DEPRECATED_OPERATION(MathML_DeprecatedAlignmentAttributes)
-DEPRECATED_OPERATION(MathML_DeprecatedBevelledAttribute)
-DEPRECATED_OPERATION(MathML_DeprecatedLineThicknessValue)
 DEPRECATED_OPERATION(MathML_DeprecatedMathSizeValue)
 DEPRECATED_OPERATION(MathML_DeprecatedMathSpaceValue)
-DEPRECATED_OPERATION(MathML_DeprecatedMencloseNotationRadical)
 DEPRECATED_OPERATION(MathML_DeprecatedMfencedElement)
-DEPRECATED_OPERATION(MathML_DeprecatedScriptShiftAttributes)
 DEPRECATED_OPERATION(MathML_DeprecatedStyleAttribute)
-DEPRECATED_OPERATION(MathML_DeprecatedXLinkAttribute)
 DEPRECATED_OPERATION(MathML_DeprecatedStixgeneralOperatorStretching)
 DEPRECATED_OPERATION(MathML_DeprecatedScriptminsizeAttribute)
 DEPRECATED_OPERATION(MathML_DeprecatedScriptsizemultiplierAttribute)
+DEPRECATED_OPERATION(FormSubmissionUntrustedEvent)
+DEPRECATED_OPERATION(ElementSetCapture)
+DEPRECATED_OPERATION(ElementReleaseCapture)
+DEPRECATED_OPERATION(DocumentReleaseCapture)
+DEPRECATED_OPERATION(OffscreenCanvasToBlob)
+DEPRECATED_OPERATION(IDBDatabaseCreateMutableFile)
+DEPRECATED_OPERATION(IDBMutableFileOpen)
+DEPRECATED_OPERATION(MozPreservesPitchDeprecatedPrefix)
+DEPRECATED_OPERATION(SVGNearestViewportElement)
+DEPRECATED_OPERATION(SVGFarthestViewportElement)
+DEPRECATED_OPERATION(U2FRegister)
+DEPRECATED_OPERATION(U2FSign)
