@@ -296,6 +296,9 @@ class nsIContent : public nsINode {
   bool IsFocusable(int32_t* aTabIndex = nullptr, bool aWithMouse = false);
   virtual bool IsFocusableInternal(int32_t* aTabIndex, bool aWithMouse);
 
+  // https://html.spec.whatwg.org/multipage/interaction.html#focus-delegate
+  mozilla::dom::Element* GetFocusDelegate(bool aWithMouse) const;
+
   /*
    * Get desired IME state for the content.
    *
@@ -372,6 +375,17 @@ class nsIContent : public nsINode {
    */
   inline nsIContent* GetFlattenedTreeParent() const;
 
+  /**
+   * Get the index of a child within this content's flat tree children.
+   *
+   * @param aPossibleChild the child to get the index of.
+   * @return the index of the child, or Nothing if not a child. Be aware that
+   *         anonymous children (e.g. a <div> child of an <input> element) will
+   *         result in Nothing.
+   */
+  mozilla::Maybe<uint32_t> ComputeFlatTreeIndexOf(
+      const nsINode* aPossibleChild) const;
+
  protected:
   // Handles getting inserted or removed directly under a <slot> element.
   // This is meant to only be called from the two functions below.
@@ -460,8 +474,7 @@ class nsIContent : public nsINode {
                                                  nsAtom* aName) {
     if (aNamespace == kNameSpaceID_XHTML &&
         (aName == nsGkAtoms::input || aName == nsGkAtoms::button ||
-         aName == nsGkAtoms::menuitem || aName == nsGkAtoms::audio ||
-         aName == nsGkAtoms::video)) {
+         aName == nsGkAtoms::audio || aName == nsGkAtoms::video)) {
       MOZ_ASSERT(
           !RequiresDoneAddingChildren(aNamespace, aName),
           "Both DoneCreatingElement and DoneAddingChildren on a same element "
