@@ -7,15 +7,16 @@
 #ifndef nsINode_h___
 #define nsINode_h___
 
+#include "mozilla/DoublyLinkedList.h"
 #include "mozilla/Likely.h"
 #include "mozilla/UniquePtr.h"
 #include "nsCOMPtr.h"              // for member, local
 #include "nsGkAtoms.h"             // for nsGkAtoms::baseURIProperty
 #include "mozilla/dom/NodeInfo.h"  // member (in nsCOMPtr)
 #include "nsIWeakReference.h"
+#include "nsIMutationObserver.h"
 #include "nsNodeInfoManager.h"  // for use in NodePrincipal()
 #include "nsPropertyTable.h"    // for typedefs
-#include "nsTObserverArray.h"   // for member
 #include "mozilla/ErrorResult.h"
 #include "mozilla/LinkedList.h"
 #include "mozilla/MemoryReporting.h"
@@ -44,7 +45,7 @@ class nsIContent;
 class nsIContentSecurityPolicy;
 class nsIFrame;
 class nsIHTMLCollection;
-class nsIMutationObserver;
+class nsMultiMutationObserver;
 class nsINode;
 class nsINodeList;
 class nsIPrincipal;
@@ -57,6 +58,8 @@ struct RawServoSelectorList;
 
 namespace mozilla {
 class EventListenerManager;
+template <typename T>
+class Maybe;
 class PresShell;
 class TextEditor;
 namespace dom {
@@ -250,7 +253,6 @@ class nsNodeWeakReference final : public nsIWeakReference {
 
   // nsIWeakReference
   NS_DECL_NSIWEAKREFERENCE
-  size_t SizeOfOnlyThis(mozilla::MallocSizeOf aMallocSizeOf) const override;
 
   void NoticeNodeDestruction() { mObject = nullptr; }
 
@@ -285,18 +287,18 @@ class nsINode : public mozilla::dom::EventTarget {
   void AssertInvariantsOnNodeInfoChange();
 #endif
  public:
-  typedef mozilla::dom::BoxQuadOptions BoxQuadOptions;
-  typedef mozilla::dom::ConvertCoordinateOptions ConvertCoordinateOptions;
-  typedef mozilla::dom::DocGroup DocGroup;
-  typedef mozilla::dom::Document Document;
-  typedef mozilla::dom::DOMPoint DOMPoint;
-  typedef mozilla::dom::DOMPointInit DOMPointInit;
-  typedef mozilla::dom::DOMQuad DOMQuad;
-  typedef mozilla::dom::DOMRectReadOnly DOMRectReadOnly;
-  typedef mozilla::dom::OwningNodeOrString OwningNodeOrString;
-  typedef mozilla::dom::TextOrElementOrDocument TextOrElementOrDocument;
-  typedef mozilla::dom::CallerType CallerType;
-  typedef mozilla::ErrorResult ErrorResult;
+  using BoxQuadOptions = mozilla::dom::BoxQuadOptions;
+  using ConvertCoordinateOptions = mozilla::dom::ConvertCoordinateOptions;
+  using DocGroup = mozilla::dom::DocGroup;
+  using Document = mozilla::dom::Document;
+  using DOMPoint = mozilla::dom::DOMPoint;
+  using DOMPointInit = mozilla::dom::DOMPointInit;
+  using DOMQuad = mozilla::dom::DOMQuad;
+  using DOMRectReadOnly = mozilla::dom::DOMRectReadOnly;
+  using OwningNodeOrString = mozilla::dom::OwningNodeOrString;
+  using TextOrElementOrDocument = mozilla::dom::TextOrElementOrDocument;
+  using CallerType = mozilla::dom::CallerType;
+  using ErrorResult = mozilla::ErrorResult;
 
   // XXXbz Maybe we should codegen a class holding these constants and
   // inherit from it...
@@ -445,6 +447,14 @@ class nsINode : public mozilla::dom::EventTarget {
    */
   bool IsShadowIncludingInclusiveDescendantOf(const nsINode* aNode) const;
 
+  /**
+   * Returns true if the given node is this node or one of its descendants
+   * in the "flat tree."
+   *
+   * @param aNode must not be nullptr.
+   */
+  bool IsInclusiveFlatTreeDescendantOf(const nsINode* aNode) const;
+
   /**
    * Return this node as a document fragment. Asserts IsDocumentFragment().
    *
@@ -500,6 +510,10 @@ class nsINode : public mozilla::dom::EventTarget {
 
   MOZ_CAN_RUN_SCRIPT mozilla::dom::Element* GetParentFlexElement();
 
+  bool IsNode() const final { return true; }
+
+  NS_IMPL_FROMEVENTTARGET_HELPER(nsINode, IsNode())
+
   /**
    * Return whether the node is an Element node. Faster than using `NodeType()`.
    */
@@ -612,6 +626,35 @@ class nsINode : public mozilla::dom::EventTarget {
    */
   nsIContent* GetChildAt_Deprecated(uint32_t aIndex) const;
 
+  /**
+   * Get the index of a child within this content.
+   *
+   * @param aPossibleChild the child to get the index of.
+   * @return the index of the child, or Nothing if not a child. Be aware that
+   *         anonymous children (e.g. a <div> child of an <input> element) will
+   *         result in Nothing.
+   *
+   * If the return value is Some, then calling GetChildAt_Deprecated() with
+   * that value will return aPossibleChild.
+   */
+  mozilla::Maybe<uint32_t> ComputeIndexOf(const nsINode* aPossibleChild) const;
+
+  /**
+   * Get the index of this within parent node (ComputeIndexInParentNode) or
+   * parent content (nsIContent) node (ComputeIndexInParentContent).
+   *
+   * @return the index of this node in the parent, or Nothing there is no
+   *         parent (content) node or the parent does not have this node anymore
+   *         (e.g., being removed from the parent). Be aware that anonymous
+   *         children (e.g. a <div> child of an <input> element) will result in
+   *         Nothing.
+   *
+   * If the return value is Some, then calling GetChildAt_Deprecated() with
+   * that value will return this.
+   */
+  mozilla::Maybe<uint32_t> ComputeIndexInParentNode() const;
+  mozilla::Maybe<uint32_t> ComputeIndexInParentContent() const;
+
   /**
    * Get the index of a child within this content.
    *
@@ -623,7 +666,7 @@ class nsINode : public mozilla::dom::EventTarget {
    * If the return value is not -1, then calling GetChildAt_Deprecated() with
    * that value will return aPossibleChild.
    */
-  virtual int32_t ComputeIndexOf(const nsINode* aPossibleChild) const;
+  int32_t ComputeIndexOf_Deprecated(const nsINode* aPossibleChild) const;
 
   /**
    * Returns the "node document" of this node.
@@ -825,17 +868,17 @@ class nsINode : public mozilla::dom::EventTarget {
    *        add aKid at the end.
    * @param aNotify whether to notify the document (current document for
    *        nsIContent, and |this| for Document) that the insert has occurred
-   *
-   * @throws NS_ERROR_DOM_HIERARCHY_REQUEST_ERR if one attempts to have more
-   * than one element node as a child of a document.  Doing this will also
-   * assert -- you shouldn't be doing it!  Check with Document::GetRootElement()
-   * first if you're not sure.  Apart from this one constraint, this doesn't do
-   * any checking on whether aKid is a valid child of |this|.
-   *
-   * @throws NS_ERROR_OUT_OF_MEMORY in some cases (from BindToTree).
+   * @param aRv The error, if any.
+   *        Throw NS_ERROR_DOM_HIERARCHY_REQUEST_ERR if one attempts to have
+   *        more than one element node as a child of a document.  Doing this
+   *        will also assert -- you shouldn't be doing it!  Check with
+   *        Document::GetRootElement() first if you're not sure.  Apart from
+   *        this one constraint, this doesn't do any checking on whether aKid is
+   *        a valid child of |this|.
+   *        Throw NS_ERROR_OUT_OF_MEMORY in some cases (from BindToTree).
    */
-  virtual nsresult InsertChildBefore(nsIContent* aKid, nsIContent* aBeforeThis,
-                                     bool aNotify);
+  virtual void InsertChildBefore(nsIContent* aKid, nsIContent* aBeforeThis,
+                                 bool aNotify, mozilla::ErrorResult& aRv);
 
   /**
    * Append a content node to the end of the child list.  This method handles
@@ -844,17 +887,18 @@ class nsINode : public mozilla::dom::EventTarget {
    * @param aKid the content to append
    * @param aNotify whether to notify the document (current document for
    *        nsIContent, and |this| for Document) that the append has occurred
-   *
-   * @throws NS_ERROR_DOM_HIERARCHY_REQUEST_ERR if one attempts to have more
-   * than one element node as a child of a document.  Doing this will also
-   * assert -- you shouldn't be doing it!  Check with Document::GetRootElement()
-   * first if you're not sure.  Apart from this one constraint, this doesn't do
-   * any checking on whether aKid is a valid child of |this|.
-   *
-   * @throws NS_ERROR_OUT_OF_MEMORY in some cases (from BindToTree).
+   * @param aRv The error, if any.
+   *        Throw NS_ERROR_DOM_HIERARCHY_REQUEST_ERR if one attempts to have
+   *        more than one element node as a child of a document.  Doing this
+   *        will also assert -- you shouldn't be doing it!  Check with
+   *        Document::GetRootElement() first if you're not sure.  Apart from
+   *        this one constraint, this doesn't do any checking on whether aKid is
+   *        a valid child of |this|.
+   *        Throw NS_ERROR_OUT_OF_MEMORY in some cases (from BindToTree).
    */
-  nsresult AppendChildTo(nsIContent* aKid, bool aNotify) {
-    return InsertChildBefore(aKid, nullptr, aNotify);
+  void AppendChildTo(nsIContent* aKid, bool aNotify,
+                     mozilla::ErrorResult& aRv) {
+    InsertChildBefore(aKid, nullptr, aNotify, aRv);
   }
 
   /**
@@ -1030,10 +1074,7 @@ class nsINode : public mozilla::dom::EventTarget {
   virtual mozilla::EventListenerManager* GetOrCreateListenerManager() override;
 
   mozilla::Maybe<mozilla::dom::EventCallbackDebuggerNotificationType>
-  GetDebuggerNotificationType() const override {
-    return mozilla::Some(
-        mozilla::dom::EventCallbackDebuggerNotificationType::Node);
-  }
+  GetDebuggerNotificationType() const override;
 
   bool ComputeDefaultWantsUntrusted(mozilla::ErrorResult& aRv) final;
 
@@ -1043,9 +1084,10 @@ class nsINode : public mozilla::dom::EventTarget {
   virtual nsIGlobalObject* GetOwnerGlobal() const override;
 
   using mozilla::dom::EventTarget::DispatchEvent;
-  bool DispatchEvent(mozilla::dom::Event& aEvent,
-                     mozilla::dom::CallerType aCallerType,
-                     mozilla::ErrorResult& aRv) override;
+  // TODO: Convert this to MOZ_CAN_RUN_SCRIPT (bug 1415230)
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY bool DispatchEvent(
+      mozilla::dom::Event& aEvent, mozilla::dom::CallerType aCallerType,
+      mozilla::ErrorResult& aRv) override;
 
   MOZ_CAN_RUN_SCRIPT
   nsresult PostHandleEvent(mozilla::EventChainPostVisitor& aVisitor) override;
@@ -1065,12 +1107,16 @@ class nsINode : public mozilla::dom::EventTarget {
    */
   void AddMutationObserver(nsIMutationObserver* aMutationObserver) {
     nsSlots* s = Slots();
-    NS_ASSERTION(s->mMutationObservers.IndexOf(aMutationObserver) ==
-                     nsTArray<int>::NoIndex,
-                 "Observer already in the list");
-    s->mMutationObservers.AppendElement(aMutationObserver);
+    if (aMutationObserver) {
+      NS_ASSERTION(!s->mMutationObservers.contains(aMutationObserver),
+                   "Observer already in the list");
+
+      s->mMutationObservers.pushBack(aMutationObserver);
+    }
   }
 
+  void AddMutationObserver(nsMultiMutationObserver* aMultiMutationObserver);
+
   /**
    * Same as above, but only adds the observer if its not observing
    * the node already.
@@ -1080,9 +1126,14 @@ class nsINode : public mozilla::dom::EventTarget {
    */
   void AddMutationObserverUnlessExists(nsIMutationObserver* aMutationObserver) {
     nsSlots* s = Slots();
-    s->mMutationObservers.AppendElementUnlessExists(aMutationObserver);
+    if (aMutationObserver &&
+        !s->mMutationObservers.contains(aMutationObserver)) {
+      s->mMutationObservers.pushBack(aMutationObserver);
+    }
   }
 
+  void AddMutationObserverUnlessExists(
+      nsMultiMutationObserver* aMultiMutationObserver);
   /**
    * Same as AddMutationObserver, but for nsIAnimationObservers.  This
    * additionally records on the document that animation observers have
@@ -1104,13 +1155,13 @@ class nsINode : public mozilla::dom::EventTarget {
   void RemoveMutationObserver(nsIMutationObserver* aMutationObserver) {
     nsSlots* s = GetExistingSlots();
     if (s) {
-      s->mMutationObservers.RemoveElement(aMutationObserver);
+      s->mMutationObservers.remove(aMutationObserver);
     }
   }
 
-  nsAutoTObserverArray<nsIMutationObserver*, 1>* GetMutationObservers() {
-    return HasSlots() ? &GetExistingSlots()->mMutationObservers : nullptr;
-  }
+  void RemoveMutationObserver(nsMultiMutationObserver* aMultiMutationObserver);
+
+  mozilla::SafeDoublyLinkedList<nsIMutationObserver>* GetMutationObservers();
 
   /**
    * Helper methods to access ancestor node(s) of type T.
@@ -1232,7 +1283,7 @@ class nsINode : public mozilla::dom::EventTarget {
     /**
      * A list of mutation observers
      */
-    nsAutoTObserverArray<nsIMutationObserver*, 1> mMutationObservers;
+    mozilla::SafeDoublyLinkedList<nsIMutationObserver> mMutationObservers;
 
     /**
      * An object implementing NodeList for this content (childNodes)
@@ -1294,6 +1345,18 @@ class nsINode : public mozilla::dom::EventTarget {
 
   inline bool IsEditable() const;
 
+  /**
+   * Check if this node is in design mode or not.  When this returns true and:
+   * - if this is a Document node, it's the design mode root.
+   * - if this is a content node, it's connected, it's not in a shadow tree
+   *   (except shadow tree for UI widget and native anonymous subtree) and its
+   *   uncomposed document is in design mode.
+   * Note that returning true does NOT mean the node or its children is
+   * editable.  E.g., when this node is in a shadow tree of a UA widget and its
+   * host is in design mode.
+   */
+  inline bool IsInDesignMode() const;
+
   /**
    * Returns true if |this| or any of its ancestors is native anonymous.
    */
@@ -1465,7 +1528,8 @@ class nsINode : public mozilla::dom::EventTarget {
    */
   Document* GetOwnerDocument() const;
 
-  void Normalize();
+  // TODO: Convert this to MOZ_CAN_RUN_SCRIPT (bug 1415230)
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY void Normalize();
 
   /**
    * Get the base URI for any relative URIs within this piece of
@@ -1512,9 +1576,9 @@ class nsINode : public mozilla::dom::EventTarget {
     SetTextContentInternal(aTextContent, nullptr, aError);
   }
 
-  mozilla::dom::Element* QuerySelector(const nsAString& aSelector,
+  mozilla::dom::Element* QuerySelector(const nsACString& aSelector,
                                        mozilla::ErrorResult& aResult);
-  already_AddRefed<nsINodeList> QuerySelectorAll(const nsAString& aSelector,
+  already_AddRefed<nsINodeList> QuerySelectorAll(const nsACString& aSelector,
                                                  mozilla::ErrorResult& aResult);
 
  protected:
@@ -1539,6 +1603,17 @@ class nsINode : public mozilla::dom::EventTarget {
   nsIContent* GetNextSibling() const { return mNextSibling; }
   nsIContent* GetPreviousSibling() const;
 
+  /**
+   * Return true if the node is being removed from the parent, it means that
+   * the node still knows the container which it's disconnected from, but the
+   * node has already been removed from the child node chain of the container.
+   * I.e., Return true between a call of DisconnectChild of the parent and
+   * a call of UnbindFromTree of the node.
+   */
+  bool IsBeingRemoved() const {
+    return mParent && !mNextSibling && !mPreviousOrLastSibling;
+  }
+
   /**
    * Get the next node in the pre-order tree traversal of the DOM.  If
    * aRoot is non-null, then it must be an ancestor of |this|
@@ -1570,6 +1645,12 @@ class nsINode : public mozilla::dom::EventTarget {
 
   bool UnoptimizableCCNode() const;
 
+  /**
+   * Fire a DOMNodeRemoved mutation event for all children of this node
+   * TODO: Convert this to MOZ_CAN_RUN_SCRIPT (bug 1415230)
+   */
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY void FireNodeRemovedForChildren();
+
  private:
   mozilla::dom::SVGUseElement* DoGetContainingSVGUseShadowHost() const;
 
@@ -1954,7 +2035,7 @@ class nsINode : public mozilla::dom::EventTarget {
     const nsString& nodeName = NodeName();
     aNodeName.SetKnownLiveString(nodeName);
   }
-  MOZ_MUST_USE nsresult GetBaseURI(nsAString& aBaseURI) const;
+  [[nodiscard]] nsresult GetBaseURI(nsAString& aBaseURI) const;
   // Return the base URI for the document.
   // The returned value may differ if the document is loaded via XHR, and
   // when accessed from chrome privileged script and
@@ -1964,9 +2045,9 @@ class nsINode : public mozilla::dom::EventTarget {
   bool HasChildNodes() const { return HasChildren(); }
 
   // See nsContentUtils::PositionIsBefore for aThisIndex and aOtherIndex usage.
-  uint16_t CompareDocumentPosition(nsINode& aOther,
-                                   int32_t* aThisIndex = nullptr,
-                                   int32_t* aOtherIndex = nullptr) const;
+  uint16_t CompareDocumentPosition(
+      nsINode& aOther, mozilla::Maybe<uint32_t>* aThisIndex = nullptr,
+      mozilla::Maybe<uint32_t>* aOtherIndex = nullptr) const;
   void GetNodeValue(nsAString& aNodeValue) { GetNodeValueInternal(aNodeValue); }
   void SetNodeValue(const nsAString& aNodeValue, mozilla::ErrorResult& aError) {
     SetNodeValueInternal(aNodeValue, aError);
@@ -1995,7 +2076,9 @@ class nsINode : public mozilla::dom::EventTarget {
                         mozilla::ErrorResult& aError) {
     return ReplaceOrInsertBefore(true, &aNode, &aChild, aError);
   }
-  nsINode* RemoveChild(nsINode& aChild, mozilla::ErrorResult& aError);
+  // TODO: Convert this to MOZ_CAN_RUN_SCRIPT (bug 1415230)
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY nsINode* RemoveChild(
+      nsINode& aChild, mozilla::ErrorResult& aError);
   already_AddRefed<nsINode> CloneNode(bool aDeep, mozilla::ErrorResult& aError);
   bool IsSameNode(nsINode* aNode);
   bool IsEqualNode(nsINode* aNode);
@@ -2053,6 +2136,7 @@ class nsINode : public mozilla::dom::EventTarget {
                                  ErrorResult& aRv);
   MOZ_CAN_RUN_SCRIPT void ReplaceChildren(
       const Sequence<OwningNodeOrString>& aNodes, ErrorResult& aRv);
+  MOZ_CAN_RUN_SCRIPT void ReplaceChildren(nsINode* aNode, ErrorResult& aRv);
 
   void GetBoxQuads(const BoxQuadOptions& aOptions,
                    nsTArray<RefPtr<DOMQuad>>& aResult, CallerType aCallerType,
@@ -2142,9 +2226,10 @@ class nsINode : public mozilla::dom::EventTarget {
   void EnsurePreInsertionValidity2(bool aReplace, nsINode& aNewChild,
                                    nsINode* aRefChild,
                                    mozilla::ErrorResult& aError);
-  nsINode* ReplaceOrInsertBefore(bool aReplace, nsINode* aNewChild,
-                                 nsINode* aRefChild,
-                                 mozilla::ErrorResult& aError);
+  // TODO: Convert this to MOZ_CAN_RUN_SCRIPT (bug 1415230)
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY nsINode* ReplaceOrInsertBefore(
+      bool aReplace, nsINode* aNewChild, nsINode* aRefChild,
+      mozilla::ErrorResult& aError);
 
   /**
    * Returns the Element that should be used for resolving namespaces
@@ -2163,7 +2248,7 @@ class nsINode : public mozilla::dom::EventTarget {
    * selector cache.
    */
   const RawServoSelectorList* ParseSelectorList(
-      const nsAString& aSelectorString, mozilla::ErrorResult&);
+      const nsACString& aSelectorString, mozilla::ErrorResult&);
 
  public:
   /* Event stuff that documents and elements share.
@@ -2228,6 +2313,24 @@ class nsINode : public mozilla::dom::EventTarget {
   nsSlots* mSlots;
 };
 
+inline nsINode* mozilla::dom::EventTarget::GetAsNode() {
+  return IsNode() ? AsNode() : nullptr;
+}
+
+inline const nsINode* mozilla::dom::EventTarget::GetAsNode() const {
+  return const_cast<mozilla::dom::EventTarget*>(this)->GetAsNode();
+}
+
+inline nsINode* mozilla::dom::EventTarget::AsNode() {
+  MOZ_DIAGNOSTIC_ASSERT(IsNode());
+  return static_cast<nsINode*>(this);
+}
+
+inline const nsINode* mozilla::dom::EventTarget::AsNode() const {
+  MOZ_DIAGNOSTIC_ASSERT(IsNode());
+  return static_cast<const nsINode*>(this);
+}
+
 // Useful inline function for getting a node given an nsIContent and a Document.
 // Returns the first argument cast to nsINode if it is non-null, otherwise
 // returns the second (which may be null).  We use type variables instead of
@@ -2258,23 +2361,51 @@ inline nsISupports* ToSupports(nsINode* aPointer) { return aPointer; }
   template <typename T>                                                  \
   static _const _class* FromNodeOrNull(_const T* aNode) {                \
     return aNode ? FromNode(*aNode) : nullptr;                           \
+  }                                                                      \
+  template <typename T>                                                  \
+  static auto FromEventTarget(_const T& aEventTarget)                    \
+      ->decltype(static_cast<_const _class*>(&aEventTarget)) {           \
+    return aEventTarget.IsNode() && aEventTarget.AsNode()->_check        \
+               ? static_cast<_const _class*>(&aEventTarget)              \
+               : nullptr;                                                \
+  }                                                                      \
+  template <typename T>                                                  \
+  static _const _class* FromEventTarget(_const T* aEventTarget) {        \
+    return FromEventTarget(*aEventTarget);                               \
+  }                                                                      \
+  template <typename T>                                                  \
+  static _const _class* FromEventTargetOrNull(_const T* aEventTarget) {  \
+    return aEventTarget ? FromEventTarget(*aEventTarget) : nullptr;      \
   }
 
-#define NS_IMPL_FROMNODE_HELPER(_class, _check)                               \
-  NS_IMPL_FROMNODE_GENERIC(_class, _check, )                                  \
-  NS_IMPL_FROMNODE_GENERIC(_class, _check, const)                             \
-                                                                              \
-  template <typename T>                                                       \
-  static _class* FromNode(T&& aNode) {                                        \
-    /* We need the double-cast in case aNode is a smartptr.  Those */         \
-    /* can cast to superclasses of the type they're templated on, */          \
-    /* but not directly to subclasses.  */                                    \
-    return aNode->_check ? static_cast<_class*>(static_cast<nsINode*>(aNode)) \
-                         : nullptr;                                           \
-  }                                                                           \
-  template <typename T>                                                       \
-  static _class* FromNodeOrNull(T&& aNode) {                                  \
-    return aNode ? FromNode(aNode) : nullptr;                                 \
+#define NS_IMPL_FROMNODE_HELPER(_class, _check)                                \
+  NS_IMPL_FROMNODE_GENERIC(_class, _check, )                                   \
+  NS_IMPL_FROMNODE_GENERIC(_class, _check, const)                              \
+                                                                               \
+  template <typename T>                                                        \
+  static _class* FromNode(T&& aNode) {                                         \
+    /* We need the double-cast in case aNode is a smartptr.  Those */          \
+    /* can cast to superclasses of the type they're templated on, */           \
+    /* but not directly to subclasses.  */                                     \
+    return aNode->_check ? static_cast<_class*>(static_cast<nsINode*>(aNode))  \
+                         : nullptr;                                            \
+  }                                                                            \
+  template <typename T>                                                        \
+  static _class* FromNodeOrNull(T&& aNode) {                                   \
+    return aNode ? FromNode(aNode) : nullptr;                                  \
+  }                                                                            \
+  template <typename T>                                                        \
+  static _class* FromEventTarget(T&& aEventTarget) {                           \
+    /* We need the double-cast in case aEventTarget is a smartptr.  Those */   \
+    /* can cast to superclasses of the type they're templated on, */           \
+    /* but not directly to subclasses.  */                                     \
+    return aEventTarget->IsNode() && aEventTarget->AsNode()->_check            \
+               ? static_cast<_class*>(static_cast<EventTarget*>(aEventTarget)) \
+               : nullptr;                                                      \
+  }                                                                            \
+  template <typename T>                                                        \
+  static _class* FromEventTargetOrNull(T&& aEventTarget) {                     \
+    return aEventTarget ? FromEventTarget(aEventTarget) : nullptr;             \
   }
 
 #define NS_IMPL_FROMNODE(_class, _nsid) \
