@@ -5,16 +5,35 @@
 #ifndef nsTreeSanitizer_h_
 #define nsTreeSanitizer_h_
 
+#include "nsAtom.h"
+#include "nsHashKeys.h"
+#include "nsHashtablesFwd.h"
 #include "nsIPrincipal.h"
-#include "mozilla/dom/Element.h"
+#include "nsTArray.h"
+#include "nsTHashSet.h"
+#include "mozilla/UniquePtr.h"
+#include "mozilla/dom/NameSpaceConstants.h"
+#include "mozilla/dom/SanitizerBinding.h"
 
 class nsIContent;
+class nsIGlobalObject;
+class nsINode;
+
+namespace mozilla {
+class DeclarationBlock;
+enum class StyleSanitizationKind : uint8_t;
+}  // namespace mozilla
+
+namespace mozilla::dom {
+class DocumentFragment;
+class Element;
+}  // namespace mozilla::dom
 
 /**
  * See the documentation of nsIParserUtils::sanitize for documentation
  * about the default behavior and the configuration options of this sanitizer.
  */
-class MOZ_STACK_CLASS nsTreeSanitizer {
+class nsTreeSanitizer {
  public:
   /**
    * The constructor.
@@ -41,6 +60,18 @@ class MOZ_STACK_CLASS nsTreeSanitizer {
    */
   void Sanitize(mozilla::dom::Document* aDocument);
 
+  /**
+   * Provides additional options for usage from the Web Sanitizer API
+   * which allows modifying the allow-list from above
+   */
+  void WithWebSanitizerOptions(nsIGlobalObject* aGlobal,
+                               const mozilla::dom::SanitizerConfig& aOptions);
+
+  /**
+   * Removes conditional CSS from this subtree.
+   */
+  static void RemoveConditionalCSSFromSubtree(nsINode* aRoot);
+
  private:
   /**
    * Whether <style> and style="" are allowed.
@@ -82,13 +113,16 @@ class MOZ_STACK_CLASS nsTreeSanitizer {
    */
   bool mLogRemovals;
 
+  // WindowID used for logging removals.
+  uint64_t mInnerWindowID = 0;
+
   /**
    * We have various tables of static atoms for elements and attributes.
    */
-  class AtomsTable : public nsTHashtable<nsPtrHashKey<const nsStaticAtom>> {
+  class AtomsTable : public nsTHashSet<const nsStaticAtom*> {
    public:
     explicit AtomsTable(uint32_t aLength)
-        : nsTHashtable<nsPtrHashKey<const nsStaticAtom>>(aLength) {}
+        : nsTHashSet<const nsStaticAtom*>(aLength) {}
 
     bool Contains(nsAtom* aAtom) {
       // Because this table only contains static atoms, if aAtom isn't
@@ -97,6 +131,44 @@ class MOZ_STACK_CLASS nsTreeSanitizer {
     }
   };
 
+  // The name of an element combined with its namespace.
+  class ElementName : public PLDHashEntryHdr {
+   public:
+    using KeyType = const ElementName&;
+    using KeyTypePointer = const ElementName*;
+
+    explicit ElementName(KeyTypePointer aKey)
+        : mNamespaceID(aKey->mNamespaceID), mLocalName(aKey->mLocalName) {}
+    ElementName(int32_t aNamespaceID, RefPtr<nsAtom> aLocalName)
+        : mNamespaceID(aNamespaceID), mLocalName(std::move(aLocalName)) {}
+    ElementName(ElementName&&) = default;
+    ~ElementName() = default;
+
+    bool KeyEquals(KeyTypePointer aKey) const {
+      return mNamespaceID == aKey->mNamespaceID &&
+             mLocalName == aKey->mLocalName;
+    }
+
+    static KeyTypePointer KeyToPointer(KeyType aKey) { return &aKey; }
+    static PLDHashNumber HashKey(KeyTypePointer aKey) {
+      if (!aKey) {
+        return 0;
+      }
+
+      return mozilla::HashGeneric(aKey->mNamespaceID, aKey->mLocalName.get());
+    }
+
+    enum { ALLOW_MEMMOVE = true };
+
+   private:
+    int32_t mNamespaceID = kNameSpaceID_None;
+    RefPtr<nsAtom> mLocalName;
+  };
+
+  using ElementNameSet = nsTHashSet<ElementName>;
+  using ElementToAttributeSetTable =
+      nsTHashMap<RefPtr<nsAtom>, mozilla::UniquePtr<ElementNameSet>>;
+
   void SanitizeChildren(nsINode* aRoot);
 
   /**
@@ -107,6 +179,7 @@ class MOZ_STACK_CLASS nsTreeSanitizer {
    *         false if the element is to be kept
    */
   bool MustFlatten(int32_t aNamespace, nsAtom* aLocal);
+  bool MustFlattenForSanitizerAPI(int32_t aNamespace, nsAtom* aLocal);
 
   /**
    * Queries if an element including its children must be removed.
@@ -118,6 +191,8 @@ class MOZ_STACK_CLASS nsTreeSanitizer {
    */
   bool MustPrune(int32_t aNamespace, nsAtom* aLocal,
                  mozilla::dom::Element* aElement);
+  bool MustPruneForSanitizerAPI(int32_t aNamespace, nsAtom* aLocal,
+                                mozilla::dom::Element* aElement);
 
   /**
    * Checks if a given local name (for an attribute) is on the given list
@@ -155,6 +230,11 @@ class MOZ_STACK_CLASS nsTreeSanitizer {
    */
   void SanitizeAttributes(mozilla::dom::Element* aElement,
                           AllowedAttributes aAllowed);
+  // Currently only used for the Sanitizer API.
+  bool MustDropAttribute(mozilla::dom::Element* aElement,
+                         int32_t aAttrNamespace, nsAtom* aAttrLocalName);
+  bool MustDropFunkyAttribute(mozilla::dom::Element* aElement,
+                              int32_t aAttrNamespace, nsAtom* aAttrLocalName);
 
   /**
    * Remove the named URL attribute from the element if the URL fails a
@@ -163,10 +243,11 @@ class MOZ_STACK_CLASS nsTreeSanitizer {
    * @param aElement the element whose attribute to possibly modify
    * @param aNamespace the namespace of the URL attribute
    * @param aLocalName the local name of the URL attribute
+   * @param aFragmentsOnly allows same-document references only
    * @return true if the attribute was removed and false otherwise
    */
   bool SanitizeURL(mozilla::dom::Element* aElement, int32_t aNamespace,
-                   nsAtom* aLocalName);
+                   nsAtom* aLocalName, bool aFragmentsOnly = false);
 
   /**
    * Checks a style rule for the presence of the 'binding' CSS property and
@@ -178,24 +259,33 @@ class MOZ_STACK_CLASS nsTreeSanitizer {
   bool SanitizeStyleDeclaration(mozilla::DeclarationBlock* aDeclaration);
 
   /**
-   * Parses a style sheet and reserializes it with the 'binding' property
-   * removed if it was present.
+   * Sanitizes an inline style element (an HTML or SVG <style>).
    *
-   * @param aOrigin the original style sheet source
-   * @param aSanitized the reserialization without 'binding'; only valid if
-   *                   this method return true
-   * @param aDocument the document the style sheet belongs to
-   * @param aBaseURI the base URI to use
-   * @return true if the 'binding' property was encountered and false
-   *              otherwise
+   * Returns whether the style has changed.
    */
-  bool SanitizeStyleSheet(const nsAString& aOriginal, nsAString& aSanitized,
-                          mozilla::dom::Document* aDocument, nsIURI* aBaseURI);
+  static bool SanitizeInlineStyle(mozilla::dom::Element*,
+                                  mozilla::StyleSanitizationKind);
 
   /**
    * Removes all attributes from an element node.
    */
-  void RemoveAllAttributes(mozilla::dom::Element* aElement);
+  static void RemoveAllAttributes(mozilla::dom::Element* aElement);
+
+  /**
+   * Removes all attributes from the descendants of an element but not from
+   * the element itself.
+   */
+  static void RemoveAllAttributesFromDescendants(mozilla::dom::Element*);
+
+  static bool MatchesElementName(ElementNameSet& aNames, int32_t aNamespace,
+                                 nsAtom* aLocalName);
+  static bool MatchesAttributeMatchList(ElementToAttributeSetTable& aMatchList,
+                                        mozilla::dom::Element& aElement,
+                                        int32_t aAttrNamespace,
+                                        nsAtom* aAttrLocalName);
+
+  static mozilla::UniquePtr<ElementNameSet> ConvertElementNames(
+      const mozilla::dom::Sequence<nsString>& aNames);
 
   /**
    * Log a Console Service message to indicate we removed something.
@@ -209,7 +299,8 @@ class MOZ_STACK_CLASS nsTreeSanitizer {
    * @param aAttribute optional, the attribute being removed or modified.
    */
   void LogMessage(const char* aMessage, mozilla::dom::Document* aDoc,
-                  Element* aElement = nullptr, nsAtom* aAttr = nullptr);
+                  mozilla::dom::Element* aElement = nullptr,
+                  nsAtom* aAttr = nullptr);
 
   /**
    * The whitelist of HTML elements.
@@ -246,10 +337,53 @@ class MOZ_STACK_CLASS nsTreeSanitizer {
    */
   static AtomsTable* sAttributesMathML;
 
+  /**
+   * The built-in baseline attribute allow list used by the Sanitizer API.
+   */
+  static AtomsTable* sBaselineAttributeAllowlist;
+
+  /**
+   * The built-in baseline element allow list used by the Sanitizer API.
+   */
+  static AtomsTable* sBaselineElementAllowlist;
+
+  /**
+   * The default configuration's attribute allow list used by the Sanitizer API.
+   */
+  static AtomsTable* sDefaultConfigurationAttributeAllowlist;
+
+  /**
+   * The default configuration's element allow list used by the Sanitizer API.
+   */
+  static AtomsTable* sDefaultConfigurationElementAllowlist;
+
   /**
    * Reusable null principal for URL checks.
    */
   static nsIPrincipal* sNullPrincipal;
+
+  // === Variables used to implement HTML Sanitizer API. ==
+
+  // This nsTreeSanitizer instance should behave like the Sanitizer API.
+  bool mIsForSanitizerAPI = false;
+
+  bool mAllowCustomElements = false;
+  bool mAllowUnknownMarkup = false;
+
+  // An allow-list of elements to keep.
+  mozilla::UniquePtr<ElementNameSet> mAllowElements;
+
+  // A deny-list of elements to block. (aka flatten)
+  mozilla::UniquePtr<ElementNameSet> mBlockElements;
+
+  // A deny-list of elements to drop. (aka prune)
+  mozilla::UniquePtr<ElementNameSet> mDropElements;
+
+  // An allow-list of attributes to keep.
+  mozilla::UniquePtr<ElementToAttributeSetTable> mAllowedAttributes;
+
+  // A deny-list of attributes to drop.
+  mozilla::UniquePtr<ElementToAttributeSetTable> mDroppedAttributes;
 };
 
 #endif  // nsTreeSanitizer_h_
