@@ -6,39 +6,45 @@
 
 #include "mozilla/dom/cache/CacheStorage.h"
 
+#include "mozilla/Preferences.h"
 #include "mozilla/Unused.h"
+#include "mozilla/dom/CacheBinding.h"
 #include "mozilla/dom/CacheStorageBinding.h"
+#include "mozilla/dom/InternalRequest.h"
 #include "mozilla/dom/Promise.h"
 #include "mozilla/dom/Response.h"
 #include "mozilla/dom/cache/AutoUtils.h"
 #include "mozilla/dom/cache/Cache.h"
 #include "mozilla/dom/cache/CacheChild.h"
+#include "mozilla/dom/cache/CacheCommon.h"
 #include "mozilla/dom/cache/CacheStorageChild.h"
-#include "mozilla/dom/cache/CacheWorkerHolder.h"
+#include "mozilla/dom/cache/CacheWorkerRef.h"
 #include "mozilla/dom/cache/PCacheChild.h"
 #include "mozilla/dom/cache/ReadStream.h"
 #include "mozilla/dom/cache/TypeUtils.h"
+#include "mozilla/dom/quota/QuotaManager.h"
+#include "mozilla/dom/quota/ResultExtensions.h"
+#include "mozilla/dom/WorkerPrivate.h"
 #include "mozilla/ipc/BackgroundChild.h"
 #include "mozilla/ipc/BackgroundUtils.h"
 #include "mozilla/ipc/PBackgroundChild.h"
 #include "mozilla/ipc/PBackgroundSharedTypes.h"
+#include "mozilla/StaticPrefs_dom.h"
+#include "mozilla/StaticPrefs_extensions.h"
 #include "nsContentUtils.h"
-#include "nsIDocument.h"
+#include "mozilla/dom/Document.h"
 #include "nsIGlobalObject.h"
-#include "nsIScriptSecurityManager.h"
+#include "nsMixedContentBlocker.h"
 #include "nsURLParsers.h"
-#include "WorkerPrivate.h"
+#include "js/Object.h"              // JS::GetClass
+#include "js/PropertyAndElement.h"  // JS_DefineProperty
 
-namespace mozilla {
-namespace dom {
-namespace cache {
+namespace mozilla::dom::cache {
 
-using mozilla::Unused;
 using mozilla::ErrorResult;
-using mozilla::dom::workers::WorkerPrivate;
+using mozilla::dom::quota::QuotaManager;
 using mozilla::ipc::BackgroundChild;
 using mozilla::ipc::PBackgroundChild;
-using mozilla::ipc::IProtocol;
 using mozilla::ipc::PrincipalInfo;
 using mozilla::ipc::PrincipalToPrincipalInfo;
 
@@ -50,25 +56,21 @@ NS_IMPL_CYCLE_COLLECTION_WRAPPERCACHE(mozilla::dom::cache::CacheStorage,
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(CacheStorage)
   NS_WRAPPERCACHE_INTERFACE_MAP_ENTRY
   NS_INTERFACE_MAP_ENTRY(nsISupports)
-  NS_INTERFACE_MAP_ENTRY(nsIIPCBackgroundChildCreateCallback)
 NS_INTERFACE_MAP_END
 
 // We cannot reference IPC types in a webidl binding implementation header.  So
-// define this in the .cpp and use heap storage in the mPendingRequests list.
-struct CacheStorage::Entry final
-{
+// define this in the .cpp.
+struct CacheStorage::Entry final {
   RefPtr<Promise> mPromise;
   CacheOpArgs mArgs;
   // We cannot add the requests until after the actor is present.  So store
   // the request data separately for now.
-  RefPtr<InternalRequest> mRequest;
+  SafeRefPtr<InternalRequest> mRequest;
 };
 
 namespace {
 
-bool
-IsTrusted(const PrincipalInfo& aPrincipalInfo, bool aTestingPrefEnabled)
-{
+bool IsTrusted(const PrincipalInfo& aPrincipalInfo, bool aTestingPrefEnabled) {
   // Can happen on main thread or worker thread
 
   if (aPrincipalInfo.type() == PrincipalInfo::TSystemPrincipalInfo) {
@@ -76,11 +78,10 @@ IsTrusted(const PrincipalInfo& aPrincipalInfo, bool aTestingPrefEnabled)
   }
 
   // Require a ContentPrincipal to avoid null principal, etc.
-  if (NS_WARN_IF(aPrincipalInfo.type() != PrincipalInfo::TContentPrincipalInfo)) {
-    return false;
-  }
+  QM_TRY(OkIf(aPrincipalInfo.type() == PrincipalInfo::TContentPrincipalInfo),
+         false);
 
-  // If we're in testing mode, then don't do any more work to determing if
+  // If we're in testing mode, then don't do any more work to determine if
   // the origin is trusted.  We have to run some tests as http.
   if (aTestingPrefEnabled) {
     return true;
@@ -95,72 +96,65 @@ IsTrusted(const PrincipalInfo& aPrincipalInfo, bool aTestingPrefEnabled)
   // TODO: Implement full secure setting algorithm. (bug 1177856)
 
   const nsCString& flatURL = aPrincipalInfo.get_ContentPrincipalInfo().spec();
-  const char* url = flatURL.get();
+  const char* const url = flatURL.get();
 
   // off the main thread URL parsing using nsStdURLParser.
-  nsCOMPtr<nsIURLParser> urlParser = new nsStdURLParser();
+  const nsCOMPtr<nsIURLParser> urlParser = new nsStdURLParser();
 
   uint32_t schemePos;
   int32_t schemeLen;
   uint32_t authPos;
   int32_t authLen;
-  nsresult rv = urlParser->ParseURL(url, flatURL.Length(),
-                                    &schemePos, &schemeLen,
-                                    &authPos, &authLen,
-                                    nullptr, nullptr);      // ignore path
-  if (NS_WARN_IF(NS_FAILED(rv))) { return false; }
+  QM_TRY(MOZ_TO_RESULT(urlParser->ParseURL(url, flatURL.Length(), &schemePos,
+                                           &schemeLen, &authPos, &authLen,
+                                           nullptr, nullptr)),  // ignore path
+         false);
 
-  nsAutoCString scheme(Substring(flatURL, schemePos, schemeLen));
+  const nsAutoCString scheme(Substring(flatURL, schemePos, schemeLen));
   if (scheme.LowerCaseEqualsLiteral("https") ||
-      scheme.LowerCaseEqualsLiteral("file")) {
+      scheme.LowerCaseEqualsLiteral("file") ||
+      scheme.LowerCaseEqualsLiteral("moz-extension")) {
     return true;
   }
 
   uint32_t hostPos;
   int32_t hostLen;
+  QM_TRY(MOZ_TO_RESULT(
+             urlParser->ParseAuthority(url + authPos, authLen, nullptr,
+                                       nullptr,           // ignore username
+                                       nullptr, nullptr,  // ignore password
+                                       &hostPos, &hostLen,
+                                       nullptr)),  // ignore port
+         false);
 
-  rv = urlParser->ParseAuthority(url + authPos, authLen,
-                                 nullptr, nullptr,          // ignore username
-                                 nullptr, nullptr,          // ignore password
-                                 &hostPos, &hostLen,
-                                 nullptr);                  // ignore port
-  if (NS_WARN_IF(NS_FAILED(rv))) { return false; }
-
-  nsDependentCSubstring hostname(url + authPos + hostPos, hostLen);
-
-  return hostname.EqualsLiteral("localhost") ||
-         hostname.EqualsLiteral("127.0.0.1") ||
-         hostname.EqualsLiteral("::1");
+  return nsMixedContentBlocker::IsPotentiallyTrustworthyLoopbackHost(
+      nsDependentCSubstring(url + authPos + hostPos, hostLen));
 }
 
-} // namespace
+}  // namespace
 
 // static
-already_AddRefed<CacheStorage>
-CacheStorage::CreateOnMainThread(Namespace aNamespace, nsIGlobalObject* aGlobal,
-                                 nsIPrincipal* aPrincipal, bool aStorageDisabled,
-                                 bool aForceTrustedOrigin, ErrorResult& aRv)
-{
+already_AddRefed<CacheStorage> CacheStorage::CreateOnMainThread(
+    Namespace aNamespace, nsIGlobalObject* aGlobal, nsIPrincipal* aPrincipal,
+    bool aForceTrustedOrigin, ErrorResult& aRv) {
   MOZ_DIAGNOSTIC_ASSERT(aGlobal);
   MOZ_DIAGNOSTIC_ASSERT(aPrincipal);
   MOZ_ASSERT(NS_IsMainThread());
 
-  if (aStorageDisabled) {
-    NS_WARNING("CacheStorage has been disabled.");
-    RefPtr<CacheStorage> ref = new CacheStorage(NS_ERROR_DOM_SECURITY_ERR);
-    return ref.forget();
-  }
-
   PrincipalInfo principalInfo;
-  nsresult rv = PrincipalToPrincipalInfo(aPrincipal, &principalInfo);
-  if (NS_WARN_IF(NS_FAILED(rv))) {
-    aRv.Throw(rv);
-    return nullptr;
-  }
+  QM_TRY(MOZ_TO_RESULT(PrincipalToPrincipalInfo(aPrincipal, &principalInfo)),
+         nullptr, [&aRv](const nsresult rv) { aRv.Throw(rv); });
 
-  bool testingEnabled = aForceTrustedOrigin ||
-    Preferences::GetBool("dom.caches.testing.enabled", false) ||
-    Preferences::GetBool("dom.serviceWorkers.testing.enabled", false);
+  QM_TRY(OkIf(QuotaManager::IsPrincipalInfoValid(principalInfo)),
+         RefPtr{new CacheStorage(NS_ERROR_DOM_SECURITY_ERR)}.forget(),
+         [](const auto) {
+           NS_WARNING("CacheStorage not supported on invalid origins.");
+         });
+
+  const bool testingEnabled =
+      aForceTrustedOrigin ||
+      Preferences::GetBool("dom.caches.testing.enabled", false) ||
+      StaticPrefs::dom_serviceWorkers_testing_enabled();
 
   if (!IsTrusted(principalInfo, testingEnabled)) {
     NS_WARNING("CacheStorage not supported on untrusted origins.");
@@ -168,42 +162,38 @@ CacheStorage::CreateOnMainThread(Namespace aNamespace, nsIGlobalObject* aGlobal,
     return ref.forget();
   }
 
-  RefPtr<CacheStorage> ref = new CacheStorage(aNamespace, aGlobal,
-                                                principalInfo, nullptr);
+  RefPtr<CacheStorage> ref =
+      new CacheStorage(aNamespace, aGlobal, principalInfo, nullptr);
   return ref.forget();
 }
 
 // static
-already_AddRefed<CacheStorage>
-CacheStorage::CreateOnWorker(Namespace aNamespace, nsIGlobalObject* aGlobal,
-                             WorkerPrivate* aWorkerPrivate, ErrorResult& aRv)
-{
+already_AddRefed<CacheStorage> CacheStorage::CreateOnWorker(
+    Namespace aNamespace, nsIGlobalObject* aGlobal,
+    WorkerPrivate* aWorkerPrivate, ErrorResult& aRv) {
   MOZ_DIAGNOSTIC_ASSERT(aGlobal);
   MOZ_DIAGNOSTIC_ASSERT(aWorkerPrivate);
   aWorkerPrivate->AssertIsOnWorkerThread();
 
-  if (!aWorkerPrivate->IsStorageAllowed()) {
-    NS_WARNING("CacheStorage is not allowed.");
-    RefPtr<CacheStorage> ref = new CacheStorage(NS_ERROR_DOM_SECURITY_ERR);
-    return ref.forget();
-  }
-
   if (aWorkerPrivate->GetOriginAttributes().mPrivateBrowsingId > 0) {
     NS_WARNING("CacheStorage not supported during private browsing.");
     RefPtr<CacheStorage> ref = new CacheStorage(NS_ERROR_DOM_SECURITY_ERR);
     return ref.forget();
   }
 
-  RefPtr<CacheWorkerHolder> workerHolder =
-    CacheWorkerHolder::Create(aWorkerPrivate,
-                              CacheWorkerHolder::AllowIdleShutdownStart);
-  if (!workerHolder) {
+  SafeRefPtr<CacheWorkerRef> workerRef =
+      CacheWorkerRef::Create(aWorkerPrivate, CacheWorkerRef::eIPCWorkerRef);
+  if (!workerRef) {
     NS_WARNING("Worker thread is shutting down.");
     aRv.Throw(NS_ERROR_FAILURE);
     return nullptr;
   }
 
-  const PrincipalInfo& principalInfo = aWorkerPrivate->GetPrincipalInfo();
+  const PrincipalInfo& principalInfo =
+      aWorkerPrivate->GetEffectiveStoragePrincipalInfo();
+
+  QM_TRY(OkIf(QuotaManager::IsPrincipalInfoValid(principalInfo)), nullptr,
+         [&aRv](const auto) { aRv.Throw(NS_ERROR_FAILURE); });
 
   // We have a number of cases where we want to skip the https scheme
   // validation:
@@ -218,8 +208,8 @@ CacheStorage::CreateOnWorker(Namespace aNamespace, nsIGlobalObject* aGlobal,
   //    origin checks.  The ServiceWorker has its own trusted origin checks
   //    that are better than ours.  In addition, we don't have information
   //    about the window any more, so we can't do our own checks.
-  bool testingEnabled = aWorkerPrivate->DOMCachesTestingEnabled() ||
-                        aWorkerPrivate->ServiceWorkersTestingEnabled() ||
+  bool testingEnabled = StaticPrefs::dom_caches_testing_enabled() ||
+                        StaticPrefs::dom_serviceWorkers_testing_enabled() ||
                         aWorkerPrivate->ServiceWorkersTestingInWindow() ||
                         aWorkerPrivate->IsServiceWorker();
 
@@ -229,22 +219,20 @@ CacheStorage::CreateOnWorker(Namespace aNamespace, nsIGlobalObject* aGlobal,
     return ref.forget();
   }
 
-  RefPtr<CacheStorage> ref = new CacheStorage(aNamespace, aGlobal,
-                                              principalInfo, workerHolder);
+  RefPtr<CacheStorage> ref = new CacheStorage(
+      aNamespace, aGlobal, principalInfo, std::move(workerRef));
   return ref.forget();
 }
 
 // static
-bool
-CacheStorage::DefineCaches(JSContext* aCx, JS::Handle<JSObject*> aGlobal)
-{
+bool CacheStorage::DefineCaches(JSContext* aCx, JS::Handle<JSObject*> aGlobal) {
   MOZ_ASSERT(NS_IsMainThread());
-  MOZ_DIAGNOSTIC_ASSERT(js::GetObjectClass(aGlobal)->flags & JSCLASS_DOM_GLOBAL,
-                                           "Passed object is not a global object!");
+  MOZ_DIAGNOSTIC_ASSERT(JS::GetClass(aGlobal)->flags & JSCLASS_DOM_GLOBAL,
+                        "Passed object is not a global object!");
   js::AssertSameCompartment(aCx, aGlobal);
 
-  if (NS_WARN_IF(!CacheStorageBinding::GetConstructorObject(aCx) ||
-                 !CacheBinding::GetConstructorObject(aCx))) {
+  if (NS_WARN_IF(!CacheStorage_Binding::GetConstructorObject(aCx) ||
+                 !Cache_Binding::GetConstructorObject(aCx))) {
     return false;
   }
 
@@ -253,10 +241,9 @@ CacheStorage::DefineCaches(JSContext* aCx, JS::Handle<JSObject*> aGlobal)
 
   ErrorResult rv;
   RefPtr<CacheStorage> storage =
-    CreateOnMainThread(DEFAULT_NAMESPACE, xpc::NativeGlobal(aGlobal), principal,
-                       false, /* private browsing */
-                       true,  /* force trusted */
-                       rv);
+      CreateOnMainThread(DEFAULT_NAMESPACE, xpc::NativeGlobal(aGlobal),
+                         principal, true, /* force trusted */
+                         rv);
   if (NS_WARN_IF(rv.MaybeSetPendingException(aCx))) {
     return false;
   }
@@ -271,54 +258,62 @@ CacheStorage::DefineCaches(JSContext* aCx, JS::Handle<JSObject*> aGlobal)
 
 CacheStorage::CacheStorage(Namespace aNamespace, nsIGlobalObject* aGlobal,
                            const PrincipalInfo& aPrincipalInfo,
-                           CacheWorkerHolder* aWorkerHolder)
-  : mNamespace(aNamespace)
-  , mGlobal(aGlobal)
-  , mPrincipalInfo(MakeUnique<PrincipalInfo>(aPrincipalInfo))
-  , mWorkerHolder(aWorkerHolder)
-  , mActor(nullptr)
-  , mStatus(NS_OK)
-{
+                           SafeRefPtr<CacheWorkerRef> aWorkerRef)
+    : mNamespace(aNamespace),
+      mGlobal(aGlobal),
+      mPrincipalInfo(MakeUnique<PrincipalInfo>(aPrincipalInfo)),
+      mActor(nullptr),
+      mStatus(NS_OK) {
   MOZ_DIAGNOSTIC_ASSERT(mGlobal);
 
   // If the PBackground actor is already initialized then we can
   // immediately use it
-  PBackgroundChild* actor = BackgroundChild::GetForCurrentThread();
-  if (actor) {
-    ActorCreated(actor);
+  PBackgroundChild* actor = BackgroundChild::GetOrCreateForCurrentThread();
+  if (NS_WARN_IF(!actor)) {
+    mStatus = NS_ERROR_UNEXPECTED;
     return;
   }
 
-  // Otherwise we must begin the PBackground initialization process and
-  // wait for the async ActorCreated() callback.
-  MOZ_ASSERT(NS_IsMainThread());
-  bool ok = BackgroundChild::GetOrCreateForCurrentThread(this);
-  if (NS_WARN_IF(!ok)) {
-    ActorFailed();
+  // WorkerRef ownership is passed to the CacheStorageChild actor and any
+  // actors it may create.  The WorkerRef will keep the worker thread alive
+  // until the actors can gracefully shutdown.
+  CacheStorageChild* newActor =
+      new CacheStorageChild(this, std::move(aWorkerRef));
+  PCacheStorageChild* constructedActor = actor->SendPCacheStorageConstructor(
+      newActor, mNamespace, *mPrincipalInfo);
+
+  if (NS_WARN_IF(!constructedActor)) {
+    mStatus = NS_ERROR_UNEXPECTED;
+    return;
   }
+
+  MOZ_DIAGNOSTIC_ASSERT(constructedActor == newActor);
+  mActor = newActor;
 }
 
 CacheStorage::CacheStorage(nsresult aFailureResult)
-  : mNamespace(INVALID_NAMESPACE)
-  , mActor(nullptr)
-  , mStatus(aFailureResult)
-{
+    : mNamespace(INVALID_NAMESPACE), mActor(nullptr), mStatus(aFailureResult) {
   MOZ_DIAGNOSTIC_ASSERT(NS_FAILED(mStatus));
 }
 
-already_AddRefed<Promise>
-CacheStorage::Match(JSContext* aCx, const RequestOrUSVString& aRequest,
-                    const CacheQueryOptions& aOptions, ErrorResult& aRv)
-{
+already_AddRefed<Promise> CacheStorage::Match(
+    JSContext* aCx, const RequestOrUSVString& aRequest,
+    const MultiCacheQueryOptions& aOptions, ErrorResult& aRv) {
   NS_ASSERT_OWNINGTHREAD(CacheStorage);
 
+  if (!HasStorageAccess(eUseCounter_custom_PrivateBrowsingCachesMatch,
+                        UseCounterWorker::Custom_PrivateBrowsingCachesMatch)) {
+    aRv.Throw(NS_ERROR_DOM_SECURITY_ERR);
+    return nullptr;
+  }
+
   if (NS_WARN_IF(NS_FAILED(mStatus))) {
     aRv.Throw(mStatus);
     return nullptr;
   }
 
-  RefPtr<InternalRequest> request =
-    ToInternalRequest(aCx, aRequest, IgnoreBody, aRv);
+  SafeRefPtr<InternalRequest> request =
+      ToInternalRequest(aCx, aRequest, IgnoreBody, aRv);
   if (NS_WARN_IF(aRv.Failed())) {
     return nullptr;
   }
@@ -331,22 +326,26 @@ CacheStorage::Match(JSContext* aCx, const RequestOrUSVString& aRequest,
   CacheQueryParams params;
   ToCacheQueryParams(params, aOptions);
 
-  nsAutoPtr<Entry> entry(new Entry());
+  auto entry = MakeUnique<Entry>();
   entry->mPromise = promise;
   entry->mArgs = StorageMatchArgs(CacheRequest(), params, GetOpenMode());
-  entry->mRequest = request;
+  entry->mRequest = std::move(request);
 
-  mPendingRequests.AppendElement(entry.forget());
-  MaybeRunPendingRequests();
+  RunRequest(std::move(entry));
 
   return promise.forget();
 }
 
-already_AddRefed<Promise>
-CacheStorage::Has(const nsAString& aKey, ErrorResult& aRv)
-{
+already_AddRefed<Promise> CacheStorage::Has(const nsAString& aKey,
+                                            ErrorResult& aRv) {
   NS_ASSERT_OWNINGTHREAD(CacheStorage);
 
+  if (!HasStorageAccess(eUseCounter_custom_PrivateBrowsingCachesHas,
+                        UseCounterWorker::Custom_PrivateBrowsingCachesHas)) {
+    aRv.Throw(NS_ERROR_DOM_SECURITY_ERR);
+    return nullptr;
+  }
+
   if (NS_WARN_IF(NS_FAILED(mStatus))) {
     aRv.Throw(mStatus);
     return nullptr;
@@ -357,21 +356,25 @@ CacheStorage::Has(const nsAString& aKey, ErrorResult& aRv)
     return nullptr;
   }
 
-  nsAutoPtr<Entry> entry(new Entry());
+  auto entry = MakeUnique<Entry>();
   entry->mPromise = promise;
   entry->mArgs = StorageHasArgs(nsString(aKey));
 
-  mPendingRequests.AppendElement(entry.forget());
-  MaybeRunPendingRequests();
+  RunRequest(std::move(entry));
 
   return promise.forget();
 }
 
-already_AddRefed<Promise>
-CacheStorage::Open(const nsAString& aKey, ErrorResult& aRv)
-{
+already_AddRefed<Promise> CacheStorage::Open(const nsAString& aKey,
+                                             ErrorResult& aRv) {
   NS_ASSERT_OWNINGTHREAD(CacheStorage);
 
+  if (!HasStorageAccess(eUseCounter_custom_PrivateBrowsingCachesOpen,
+                        UseCounterWorker::Custom_PrivateBrowsingCachesOpen)) {
+    aRv.Throw(NS_ERROR_DOM_SECURITY_ERR);
+    return nullptr;
+  }
+
   if (NS_WARN_IF(NS_FAILED(mStatus))) {
     aRv.Throw(mStatus);
     return nullptr;
@@ -382,21 +385,25 @@ CacheStorage::Open(const nsAString& aKey, ErrorResult& aRv)
     return nullptr;
   }
 
-  nsAutoPtr<Entry> entry(new Entry());
+  auto entry = MakeUnique<Entry>();
   entry->mPromise = promise;
   entry->mArgs = StorageOpenArgs(nsString(aKey));
 
-  mPendingRequests.AppendElement(entry.forget());
-  MaybeRunPendingRequests();
+  RunRequest(std::move(entry));
 
   return promise.forget();
 }
 
-already_AddRefed<Promise>
-CacheStorage::Delete(const nsAString& aKey, ErrorResult& aRv)
-{
+already_AddRefed<Promise> CacheStorage::Delete(const nsAString& aKey,
+                                               ErrorResult& aRv) {
   NS_ASSERT_OWNINGTHREAD(CacheStorage);
 
+  if (!HasStorageAccess(eUseCounter_custom_PrivateBrowsingCachesDelete,
+                        UseCounterWorker::Custom_PrivateBrowsingCachesDelete)) {
+    aRv.Throw(NS_ERROR_DOM_SECURITY_ERR);
+    return nullptr;
+  }
+
   if (NS_WARN_IF(NS_FAILED(mStatus))) {
     aRv.Throw(mStatus);
     return nullptr;
@@ -407,21 +414,24 @@ CacheStorage::Delete(const nsAString& aKey, ErrorResult& aRv)
     return nullptr;
   }
 
-  nsAutoPtr<Entry> entry(new Entry());
+  auto entry = MakeUnique<Entry>();
   entry->mPromise = promise;
   entry->mArgs = StorageDeleteArgs(nsString(aKey));
 
-  mPendingRequests.AppendElement(entry.forget());
-  MaybeRunPendingRequests();
+  RunRequest(std::move(entry));
 
   return promise.forget();
 }
 
-already_AddRefed<Promise>
-CacheStorage::Keys(ErrorResult& aRv)
-{
+already_AddRefed<Promise> CacheStorage::Keys(ErrorResult& aRv) {
   NS_ASSERT_OWNINGTHREAD(CacheStorage);
 
+  if (!HasStorageAccess(eUseCounter_custom_PrivateBrowsingCachesKeys,
+                        UseCounterWorker::Custom_PrivateBrowsingCachesKeys)) {
+    aRv.Throw(NS_ERROR_DOM_SECURITY_ERR);
+    return nullptr;
+  }
+
   if (NS_WARN_IF(NS_FAILED(mStatus))) {
     aRv.Throw(mStatus);
     return nullptr;
@@ -432,29 +442,19 @@ CacheStorage::Keys(ErrorResult& aRv)
     return nullptr;
   }
 
-  nsAutoPtr<Entry> entry(new Entry());
+  auto entry = MakeUnique<Entry>();
   entry->mPromise = promise;
   entry->mArgs = StorageKeysArgs();
 
-  mPendingRequests.AppendElement(entry.forget());
-  MaybeRunPendingRequests();
+  RunRequest(std::move(entry));
 
   return promise.forget();
 }
 
 // static
-bool
-CacheStorage::PrefEnabled(JSContext* aCx, JSObject* aObj)
-{
-  return Cache::PrefEnabled(aCx, aObj);
-}
-
-// static
-already_AddRefed<CacheStorage>
-CacheStorage::Constructor(const GlobalObject& aGlobal,
-                          CacheStorageNamespace aNamespace,
-                          nsIPrincipal* aPrincipal, ErrorResult& aRv)
-{
+already_AddRefed<CacheStorage> CacheStorage::Constructor(
+    const GlobalObject& aGlobal, CacheStorageNamespace aNamespace,
+    nsIPrincipal* aPrincipal, ErrorResult& aRv) {
   if (NS_WARN_IF(!NS_IsMainThread())) {
     aRv.Throw(NS_ERROR_FAILURE);
     return nullptr;
@@ -463,128 +463,71 @@ CacheStorage::Constructor(const GlobalObject& aGlobal,
   // TODO: remove Namespace in favor of CacheStorageNamespace
   static_assert(DEFAULT_NAMESPACE == (uint32_t)CacheStorageNamespace::Content,
                 "Default namespace should match webidl Content enum");
-  static_assert(CHROME_ONLY_NAMESPACE == (uint32_t)CacheStorageNamespace::Chrome,
-                "Chrome namespace should match webidl Chrome enum");
-  static_assert(NUMBER_OF_NAMESPACES == (uint32_t)CacheStorageNamespace::EndGuard_,
-                "Number of namespace should match webidl endguard enum");
+  static_assert(
+      CHROME_ONLY_NAMESPACE == (uint32_t)CacheStorageNamespace::Chrome,
+      "Chrome namespace should match webidl Chrome enum");
+  static_assert(NUMBER_OF_NAMESPACES == CacheStorageNamespaceValues::Count,
+                "Number of namespace should match webidl count");
 
   Namespace ns = static_cast<Namespace>(aNamespace);
   nsCOMPtr<nsIGlobalObject> global = do_QueryInterface(aGlobal.GetAsSupports());
 
   bool privateBrowsing = false;
   if (nsCOMPtr<nsPIDOMWindowInner> window = do_QueryInterface(global)) {
-    nsCOMPtr<nsIDocument> doc = window->GetExtantDoc();
+    RefPtr<Document> doc = window->GetExtantDoc();
     if (doc) {
       nsCOMPtr<nsILoadContext> loadContext = doc->GetLoadContext();
       privateBrowsing = loadContext && loadContext->UsePrivateBrowsing();
     }
   }
 
+  if (privateBrowsing) {
+    RefPtr<CacheStorage> ref = new CacheStorage(NS_ERROR_DOM_SECURITY_ERR);
+    return ref.forget();
+  }
+
   // Create a CacheStorage object bypassing the trusted origin checks
   // since this is a chrome-only constructor.
-  return CreateOnMainThread(ns, global, aPrincipal, privateBrowsing,
+  return CreateOnMainThread(ns, global, aPrincipal,
                             true /* force trusted origin */, aRv);
 }
 
-nsISupports*
-CacheStorage::GetParentObject() const
-{
-  return mGlobal;
+nsISupports* CacheStorage::GetParentObject() const { return mGlobal; }
+
+JSObject* CacheStorage::WrapObject(JSContext* aContext,
+                                   JS::Handle<JSObject*> aGivenProto) {
+  return mozilla::dom::CacheStorage_Binding::Wrap(aContext, this, aGivenProto);
 }
 
-JSObject*
-CacheStorage::WrapObject(JSContext* aContext, JS::Handle<JSObject*> aGivenProto)
-{
-  return mozilla::dom::CacheStorageBinding::Wrap(aContext, this, aGivenProto);
-}
-
-void
-CacheStorage::ActorCreated(PBackgroundChild* aActor)
-{
-  NS_ASSERT_OWNINGTHREAD(CacheStorage);
-  MOZ_DIAGNOSTIC_ASSERT(aActor);
-
-  if (NS_WARN_IF(mWorkerHolder && mWorkerHolder->Notified())) {
-    ActorFailed();
-    return;
-  }
-
-  // WorkerHolder ownership is passed to the CacheStorageChild actor and any
-  // actors it may create.  The WorkerHolder will keep the worker thread alive
-  // until the actors can gracefully shutdown.
-  CacheStorageChild* newActor = new CacheStorageChild(this, mWorkerHolder);
-  PCacheStorageChild* constructedActor =
-    aActor->SendPCacheStorageConstructor(newActor, mNamespace, *mPrincipalInfo);
-
-  if (NS_WARN_IF(!constructedActor)) {
-    ActorFailed();
-    return;
-  }
-
-  mWorkerHolder = nullptr;
-
-  MOZ_DIAGNOSTIC_ASSERT(constructedActor == newActor);
-  mActor = newActor;
-
-  MaybeRunPendingRequests();
-  MOZ_DIAGNOSTIC_ASSERT(mPendingRequests.IsEmpty());
-}
-
-void
-CacheStorage::ActorFailed()
-{
-  NS_ASSERT_OWNINGTHREAD(CacheStorage);
-  MOZ_DIAGNOSTIC_ASSERT(!NS_FAILED(mStatus));
-
-  mStatus = NS_ERROR_UNEXPECTED;
-  mWorkerHolder = nullptr;
-
-  for (uint32_t i = 0; i < mPendingRequests.Length(); ++i) {
-    nsAutoPtr<Entry> entry(mPendingRequests[i].forget());
-    entry->mPromise->MaybeReject(NS_ERROR_UNEXPECTED);
-  }
-  mPendingRequests.Clear();
-}
-
-void
-CacheStorage::DestroyInternal(CacheStorageChild* aActor)
-{
+void CacheStorage::DestroyInternal(CacheStorageChild* aActor) {
   NS_ASSERT_OWNINGTHREAD(CacheStorage);
   MOZ_DIAGNOSTIC_ASSERT(mActor);
   MOZ_DIAGNOSTIC_ASSERT(mActor == aActor);
+  MOZ_DIAGNOSTIC_ASSERT(!NS_FAILED(mStatus));
   mActor->ClearListener();
   mActor = nullptr;
+  mStatus = NS_ERROR_UNEXPECTED;
 
   // Note that we will never get an actor again in case another request is
   // made before this object is destructed.
-  ActorFailed();
 }
 
-nsIGlobalObject*
-CacheStorage::GetGlobalObject() const
-{
-  return mGlobal;
-}
+nsIGlobalObject* CacheStorage::GetGlobalObject() const { return mGlobal; }
 
 #ifdef DEBUG
-void
-CacheStorage::AssertOwningThread() const
-{
+void CacheStorage::AssertOwningThread() const {
   NS_ASSERT_OWNINGTHREAD(CacheStorage);
 }
 #endif
 
-PBackgroundChild*
-CacheStorage::GetIPCManager()
-{
+PBackgroundChild* CacheStorage::GetIPCManager() {
   // This is true because CacheStorage always uses IgnoreBody for requests.
   // So we should never need to get the IPC manager during Request or
   // Response serialization.
   MOZ_CRASH("CacheStorage does not implement TypeUtils::GetIPCManager()");
 }
 
-CacheStorage::~CacheStorage()
-{
+CacheStorage::~CacheStorage() {
   NS_ASSERT_OWNINGTHREAD(CacheStorage);
   if (mActor) {
     mActor->StartDestroyFromListener();
@@ -594,35 +537,46 @@ CacheStorage::~CacheStorage()
   }
 }
 
-void
-CacheStorage::MaybeRunPendingRequests()
-{
-  if (!mActor) {
-    return;
+void CacheStorage::RunRequest(UniquePtr<Entry> aEntry) {
+  MOZ_ASSERT(mActor);
+
+  AutoChildOpArgs args(this, aEntry->mArgs, 1);
+
+  if (aEntry->mRequest) {
+    ErrorResult rv;
+    args.Add(*aEntry->mRequest, IgnoreBody, IgnoreInvalidScheme, rv);
+    if (NS_WARN_IF(rv.Failed())) {
+      aEntry->mPromise->MaybeReject(std::move(rv));
+      return;
+    }
   }
 
-  for (uint32_t i = 0; i < mPendingRequests.Length(); ++i) {
-    ErrorResult rv;
-    nsAutoPtr<Entry> entry(mPendingRequests[i].forget());
-    AutoChildOpArgs args(this, entry->mArgs, 1);
-    if (entry->mRequest) {
-      args.Add(entry->mRequest, IgnoreBody, IgnoreInvalidScheme, rv);
-    }
-    if (NS_WARN_IF(rv.Failed())) {
-      entry->mPromise->MaybeReject(rv);
-      continue;
-    }
-    mActor->ExecuteOp(mGlobal, entry->mPromise, this, args.SendAsOpArgs());
-  }
-  mPendingRequests.Clear();
+  mActor->ExecuteOp(mGlobal, aEntry->mPromise, this, args.SendAsOpArgs());
 }
 
-OpenMode
-CacheStorage::GetOpenMode() const
-{
+OpenMode CacheStorage::GetOpenMode() const {
   return mNamespace == CHROME_ONLY_NAMESPACE ? OpenMode::Eager : OpenMode::Lazy;
 }
 
-} // namespace cache
-} // namespace dom
-} // namespace mozilla
+bool CacheStorage::HasStorageAccess(UseCounter aLabel,
+                                    UseCounterWorker aLabelWorker) const {
+  NS_ASSERT_OWNINGTHREAD(CacheStorage);
+  if (NS_WARN_IF(!mGlobal)) {
+    return false;
+  }
+
+  StorageAccess access = mGlobal->GetStorageAccess();
+  if (access == StorageAccess::ePrivateBrowsing) {
+    if (NS_IsMainThread()) {
+      SetUseCounter(mGlobal->GetGlobalJSObject(), aLabel);
+    } else {
+      SetUseCounter(aLabelWorker);
+    }
+  }
+  return access > StorageAccess::ePrivateBrowsing ||
+         (StaticPrefs::
+              privacy_partition_always_partition_third_party_non_cookie_storage() &&
+          ShouldPartitionStorage(access));
+}
+
+}  // namespace mozilla::dom::cache
