@@ -17,25 +17,48 @@
 // when the returned promise is resolved.
 
 function runTests(testFile, order) {
-  function setupPrefs() {
+  async function setupPrefs() {
+    // Bug 1746646: Make mochitests work with TCP enabled (cookieBehavior = 5)
+    // Acquire storage access permission here so that the Cache API is avaialable
+    SpecialPowers.wrap(document).notifyUserGestureActivation();
+    await SpecialPowers.addPermission(
+      "storageAccessAPI",
+      true,
+      window.location.href
+    );
+    await SpecialPowers.wrap(document).requestStorageAccess();
     return SpecialPowers.pushPrefEnv({
-      "set": [["dom.caches.enabled", true],
-              ["dom.caches.testing.enabled", true],
-              ["dom.serviceWorkers.enabled", true],
-              ["dom.serviceWorkers.testing.enabled", true],
-              ["dom.serviceWorkers.exemptFromPerDomainMax", true]],
+      set: [
+        ["dom.caches.enabled", true],
+        ["dom.caches.testing.enabled", true],
+        ["dom.serviceWorkers.enabled", true],
+        ["dom.serviceWorkers.testing.enabled", true],
+        ["dom.serviceWorkers.exemptFromPerDomainMax", true],
+        [
+          "privacy.partition.always_partition_third_party_non_cookie_storage",
+          false,
+        ],
+      ],
     });
   }
 
   // adapted from dom/indexedDB/test/helpers.js
   function clearStorage() {
-    return new Promise(function(resolve, reject) {
+    var clearUnpartitionedStorage = new Promise(function(resolve, reject) {
       var qms = SpecialPowers.Services.qms;
       var principal = SpecialPowers.wrap(document).nodePrincipal;
       var request = qms.clearStoragesForPrincipal(principal);
       var cb = SpecialPowers.wrapCallback(resolve);
       request.callback = cb;
     });
+    var clearPartitionedStorage = new Promise(function(resolve, reject) {
+      var qms = SpecialPowers.Services.qms;
+      var principal = SpecialPowers.wrap(document).partitionedPrincipal;
+      var request = qms.clearStoragesForPrincipal(principal);
+      var cb = SpecialPowers.wrapCallback(resolve);
+      request.callback = cb;
+    });
+    return Promise.all([clearUnpartitionedStorage, clearPartitionedStorage]);
   }
 
   function loadScript(script) {
@@ -51,8 +74,10 @@ function runTests(testFile, order) {
   function importDrivers() {
     /* import-globals-from worker_driver.js */
     /* import-globals-from serviceworker_driver.js */
-    return Promise.all([loadScript("worker_driver.js"),
-                        loadScript("serviceworker_driver.js")]);
+    return Promise.all([
+      loadScript("worker_driver.js"),
+      loadScript("serviceworker_driver.js"),
+    ]);
   }
 
   function runWorkerTest() {
@@ -95,36 +120,38 @@ function runTests(testFile, order) {
     // TODO: Make this "both" again.
   }
 
-  ok(order == "parallel" || order == "sequential" || order == "both",
-     "order argument should be valid");
+  ok(
+    order == "parallel" || order == "sequential" || order == "both",
+    "order argument should be valid"
+  );
 
   if (order == "both") {
     info("Running tests in both modes; first: sequential");
-    return runTests(testFile, "sequential")
-        .then(function() {
-          info("Running tests in parallel mode");
-          return runTests(testFile, "parallel");
-        });
+    return runTests(testFile, "sequential").then(function() {
+      info("Running tests in parallel mode");
+      return runTests(testFile, "parallel");
+    });
   }
   if (order == "sequential") {
     return setupPrefs()
-        .then(importDrivers)
-        .then(runWorkerTest)
-        .then(clearStorage)
-        .then(runServiceWorkerTest)
-        .then(clearStorage)
-        .then(runFrameTest)
-        .then(clearStorage)
-        .catch(function(e) {
-          ok(false, "A promise was rejected during test execution: " + e);
-        });
-  }
-  return setupPrefs()
       .then(importDrivers)
-      .then(() => Promise.all([runWorkerTest(), runServiceWorkerTest(), runFrameTest()]))
+      .then(runWorkerTest)
+      .then(clearStorage)
+      .then(runServiceWorkerTest)
+      .then(clearStorage)
+      .then(runFrameTest)
       .then(clearStorage)
       .catch(function(e) {
         ok(false, "A promise was rejected during test execution: " + e);
       });
+  }
+  return setupPrefs()
+    .then(importDrivers)
+    .then(() =>
+      Promise.all([runWorkerTest(), runServiceWorkerTest(), runFrameTest()])
+    )
+    .then(clearStorage)
+    .catch(function(e) {
+      ok(false, "A promise was rejected during test execution: " + e);
+    });
 }
-
