@@ -31,6 +31,7 @@
 #include "nsIScriptContext.h"
 #include "mozilla/dom/Document.h"
 #include "nsIScriptGlobalObject.h"
+#include "nsQueryObject.h"
 #include "nsVariant.h"
 #include "mozilla/dom/ContentChild.h"
 #include "mozilla/dom/DataTransferBinding.h"
@@ -42,6 +43,7 @@
 #include "mozilla/dom/BindingUtils.h"
 #include "mozilla/dom/OSFileSystem.h"
 #include "mozilla/dom/Promise.h"
+#include "mozilla/dom/WindowContext.h"
 #include "mozilla/Unused.h"
 #include "nsComponentManagerUtils.h"
 #include "nsNetUtil.h"
@@ -49,7 +51,7 @@
 
 namespace mozilla::dom {
 
-NS_IMPL_CYCLE_COLLECTION_CLASS(DataTransfer)
+NS_IMPL_CYCLE_COLLECTION_WRAPPERCACHE_CLASS(DataTransfer)
 
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(DataTransfer)
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mParent)
@@ -64,7 +66,6 @@ NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(DataTransfer)
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mDragTarget)
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mDragImage)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
-NS_IMPL_CYCLE_COLLECTION_TRACE_WRAPPERCACHE(DataTransfer)
 
 NS_IMPL_CYCLE_COLLECTING_ADDREF(DataTransfer)
 NS_IMPL_CYCLE_COLLECTING_RELEASE(DataTransfer)
@@ -438,6 +439,17 @@ already_AddRefed<nsINode> DataTransfer::GetMozSourceNode() {
   return sourceNode.forget();
 }
 
+already_AddRefed<WindowContext> DataTransfer::GetSourceWindowContext() {
+  nsCOMPtr<nsIDragSession> dragSession = nsContentUtils::GetDragSession();
+  if (!dragSession) {
+    return nullptr;
+  }
+
+  RefPtr<WindowContext> sourceWindowContext;
+  dragSession->GetSourceWindowContext(getter_AddRefs(sourceWindowContext));
+  return sourceWindowContext.forget();
+}
+
 already_AddRefed<DOMStringList> DataTransfer::MozTypesAt(
     uint32_t aIndex, ErrorResult& aRv) const {
   // Only the first item is valid for clipboard events
@@ -609,8 +621,8 @@ already_AddRefed<DataTransfer> DataTransfer::MozCloneForEvent(
 // The order of the types matters. `kFileMime` needs to be one of the first two
 // types.
 static const char* kNonPlainTextExternalFormats[] = {
-    kCustomTypesMime, kFileMime,    kHTMLMime,    kRTFMime,
-    kURLMime,         kURLDataMime, kUnicodeMime, kPNGImageMime};
+    kCustomTypesMime, kFileMime,    kHTMLMime,     kRTFMime,  kURLMime,
+    kURLDataMime,     kUnicodeMime, kPNGImageMime, kPDFJSMime};
 
 /* static */
 void DataTransfer::GetExternalClipboardFormats(const int32_t& aWhichClipboard,
@@ -889,28 +901,6 @@ already_AddRefed<nsITransferable> DataTransfer::GetTransferable(
   const uint32_t baseLength = sizeof(uint32_t) + 1;
   uint32_t totalCustomLength = baseLength;
 
-  const char* knownFormats[] = {kTextMime,
-                                kHTMLMime,
-                                kNativeHTMLMime,
-                                kRTFMime,
-                                kURLMime,
-                                kURLDataMime,
-                                kURLDescriptionMime,
-                                kURLPrivateMime,
-                                kPNGImageMime,
-                                kJPEGImageMime,
-                                kGIFImageMime,
-                                kNativeImageMime,
-                                kFileMime,
-                                kFilePromiseMime,
-                                kFilePromiseURLMime,
-                                kFilePromiseDestFilename,
-                                kFilePromiseDirectoryMime,
-                                kMozTextInternal,
-                                kHTMLContext,
-                                kHTMLInfo,
-                                kImageRequestMime};
-
   /*
    * Two passes are made here to iterate over all of the types. First, look for
    * any types that are not in the list of known types. For this pass,
@@ -946,8 +936,8 @@ already_AddRefed<nsITransferable> DataTransfer::GetTransferable(
 
       // If the data is of one of the well-known formats, use it directly.
       bool isCustomFormat = true;
-      for (uint32_t f = 0; f < ArrayLength(knownFormats); f++) {
-        if (type.EqualsASCII(knownFormats[f])) {
+      for (const char* format : kKnownFormats) {
+        if (type.EqualsASCII(format)) {
           isCustomFormat = false;
           break;
         }
@@ -1147,16 +1137,22 @@ bool DataTransfer::ConvertFromVariant(nsIVariant* aVariant,
       return false;
     }
 
-    nsCOMPtr<nsIFlavorDataProvider> fdp = do_QueryInterface(data);
-    if (fdp) {
-      // For flavour data providers, use 0 as the length.
+    // For flavour data providers, use 0 as the length.
+    if (nsCOMPtr<nsIFlavorDataProvider> fdp = do_QueryInterface(data)) {
       fdp.forget(aSupports);
       *aLength = 0;
-    } else {
-      data.forget(aSupports);
-      *aLength = sizeof(nsISupports*);
+      return true;
     }
 
+    // Only use the underlying BlobImpl for transferables.
+    if (RefPtr<Blob> blob = do_QueryObject(data)) {
+      RefPtr<BlobImpl> blobImpl = blob->Impl();
+      blobImpl.forget(aSupports);
+    } else {
+      data.forget(aSupports);
+    }
+
+    *aLength = sizeof(nsISupports*);
     return true;
   }
 
@@ -1379,7 +1375,8 @@ void DataTransfer::CacheExternalData(const nsTArray<nsCString>& aTypes,
   for (const nsCString& type : aTypes) {
     if (type.EqualsLiteral(kCustomTypesMime)) {
       FillInExternalCustomTypes(0, aPrincipal);
-    } else if (type.EqualsLiteral(kFileMime) && XRE_IsContentProcess()) {
+    } else if (type.EqualsLiteral(kFileMime) && XRE_IsContentProcess() &&
+               !StaticPrefs::dom_events_dataTransfer_mozFile_enabled()) {
       // We will be ignoring any application/x-moz-file files found in the paste
       // datatransfer within e10s, as they will fail top be sent over IPC.
       // Because of that, we will unset hasFileData, whether or not it would
@@ -1390,9 +1387,10 @@ void DataTransfer::CacheExternalData(const nsTArray<nsCString>& aTypes,
       // We expect that if kFileMime is supported, then it will be the either at
       // index 0 or at index 1 in the aTypes returned by
       // GetExternalClipboardFormats
-      if (type.EqualsLiteral(kFileMime) && !XRE_IsContentProcess()) {
+      if (type.EqualsLiteral(kFileMime)) {
         hasFileData = true;
       }
+
       // If we aren't the file data, and we have file data, we want to be hidden
       CacheExternalData(
           type.get(), 0, aPrincipal,
@@ -1492,25 +1490,4 @@ void DataTransfer::SetMode(DataTransfer::Mode aMode) {
   }
 }
 
-/* static */
-void DataTransfer::IPCDataTransferTextItemsToDataTransfer(
-    const IPCDataTransfer& aIpcDataTransfer, const bool aHidden,
-    DataTransfer& aDataTransfer) {
-  MOZ_ASSERT(XRE_IsContentProcess());
-  MOZ_ASSERT(aDataTransfer.Items()->Length() == 0);
-
-  uint32_t i = 0;
-  for (const IPCDataTransferItem& item : aIpcDataTransfer.items()) {
-    if (item.data().type() == IPCDataTransferData::TnsString) {
-      RefPtr<nsVariantCC> variant = new nsVariantCC();
-      const nsString& data = item.data().get_nsString();
-      variant->SetAsAString(data);
-
-      aDataTransfer.SetDataWithPrincipalFromOtherProcess(
-          NS_ConvertUTF8toUTF16(item.flavor()), variant, i,
-          nsContentUtils::GetSystemPrincipal(), aHidden);
-    }
-  }
-}
-
 }  // namespace mozilla::dom
