@@ -8,20 +8,29 @@
 #include "DataTransferItemList.h"
 
 #include "mozilla/Attributes.h"
+#include "mozilla/BasePrincipal.h"
 #include "mozilla/ContentEvents.h"
 #include "mozilla/EventForwards.h"
+#include "mozilla/StaticPrefs_dom.h"
+#include "mozilla/dom/BlobImpl.h"
 #include "mozilla/dom/DataTransferItemBinding.h"
 #include "mozilla/dom/Directory.h"
 #include "mozilla/dom/Event.h"
 #include "mozilla/dom/FileSystem.h"
 #include "mozilla/dom/FileSystemDirectoryEntry.h"
 #include "mozilla/dom/FileSystemFileEntry.h"
+#include "imgIContainer.h"
+#include "imgITools.h"
+#include "nsComponentManagerUtils.h"
 #include "nsIClipboard.h"
+#include "nsIFile.h"
+#include "nsIInputStream.h"
 #include "nsISupportsPrimitives.h"
 #include "nsIScriptObjectPrincipal.h"
 #include "nsNetUtil.h"
 #include "nsQueryObject.h"
 #include "nsContentUtils.h"
+#include "nsThreadUtils.h"
 #include "nsVariant.h"
 
 namespace {
@@ -38,8 +47,7 @@ FileMimeNameData kFileMimeNameMap[] = {
 
 }  // anonymous namespace
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 NS_IMPL_CYCLE_COLLECTION_WRAPPERCACHE(DataTransferItem, mData, mPrincipal,
                                       mDataTransfer, mCachedFile)
@@ -81,6 +89,8 @@ void DataTransferItem::SetData(nsIVariant* aData) {
     // These are provided by the system, and have guaranteed properties about
     // their kind based on their type.
     MOZ_ASSERT(!mType.IsEmpty());
+    // This type should not be provided by the OS.
+    MOZ_ASSERT(!mType.EqualsASCII(kNativeImageMime));
 
     mKind = KIND_STRING;
     for (uint32_t i = 0; i < ArrayLength(kFileMimeNameMap); ++i) {
@@ -109,6 +119,14 @@ void DataTransferItem::SetData(nsIVariant* aData) {
         nsCOMPtr<nsIFile>(do_QueryInterface(supports))) {
       return KIND_FILE;
     }
+
+    if (StaticPrefs::dom_events_dataTransfer_imageAsFile_enabled()) {
+      // Firefox internally uses imgIContainer to represent images being
+      // copied/dragged. These need to be encoded to PNG files.
+      if (nsCOMPtr<imgIContainer>(do_QueryInterface(supports))) {
+        return KIND_FILE;
+      }
+    }
   }
 
   nsAutoString string;
@@ -196,7 +214,6 @@ void DataTransferItem::FillInExternalData() {
       }
       data = do_QueryObject(file);
     }
-
     variant->SetAsISupports(data);
   } else {
     // We have an external piece of string data. Extract it and store it in the
@@ -286,14 +303,43 @@ already_AddRefed<File> DataTransferItem::GetAsFile(
 
     if (RefPtr<Blob> blob = do_QueryObject(supports)) {
       mCachedFile = blob->ToFile();
-    } else if (nsCOMPtr<BlobImpl> blobImpl = do_QueryInterface(supports)) {
-      MOZ_ASSERT(blobImpl->IsFile());
-      mCachedFile = File::Create(mDataTransfer, blobImpl);
-    } else if (nsCOMPtr<nsIFile> ifile = do_QueryInterface(supports)) {
-      mCachedFile = File::CreateFromFile(mDataTransfer, ifile);
     } else {
-      MOZ_ASSERT(false, "One of the above code paths should be taken");
-      return nullptr;
+      nsCOMPtr<nsIGlobalObject> global = GetGlobalFromDataTransfer();
+      if (NS_WARN_IF(!global)) {
+        return nullptr;
+      }
+
+      if (nsCOMPtr<BlobImpl> blobImpl = do_QueryInterface(supports)) {
+        MOZ_ASSERT(blobImpl->IsFile());
+        mCachedFile = File::Create(global, blobImpl);
+        if (NS_WARN_IF(!mCachedFile)) {
+          return nullptr;
+        }
+      } else if (nsCOMPtr<nsIFile> ifile = do_QueryInterface(supports)) {
+        mCachedFile = File::CreateFromFile(global, ifile);
+        if (NS_WARN_IF(!mCachedFile)) {
+          return nullptr;
+        }
+      } else if (nsCOMPtr<imgIContainer> img = do_QueryInterface(supports)) {
+        nsCOMPtr<imgITools> imgTools =
+            do_CreateInstance("@mozilla.org/image/tools;1");
+
+        nsCOMPtr<nsIInputStream> inputStream;
+        nsresult rv = imgTools->EncodeImage(img, "image/png"_ns, u""_ns,
+                                            getter_AddRefs(inputStream));
+        if (NS_WARN_IF(NS_FAILED(rv))) {
+          return nullptr;
+        }
+
+        mCachedFile = CreateFileFromInputStream(
+            inputStream, "GenericImageNamePNG", u"image/png"_ns);
+        if (NS_WARN_IF(!mCachedFile)) {
+          return nullptr;
+        }
+      } else {
+        MOZ_ASSERT(false, "One of the above code paths should be taken");
+        return nullptr;
+      }
     }
   }
 
@@ -308,20 +354,8 @@ already_AddRefed<FileSystemEntry> DataTransferItem::GetAsEntry(
     return nullptr;
   }
 
-  nsCOMPtr<nsIGlobalObject> global;
-  // This is annoying, but DataTransfer may have various things as parent.
-  nsCOMPtr<EventTarget> target =
-      do_QueryInterface(mDataTransfer->GetParentObject());
-  if (target) {
-    global = target->GetOwnerGlobal();
-  } else {
-    RefPtr<Event> event = do_QueryObject(mDataTransfer->GetParentObject());
-    if (event) {
-      global = event->GetParentObject();
-    }
-  }
-
-  if (!global) {
+  nsCOMPtr<nsIGlobalObject> global = GetGlobalFromDataTransfer();
+  if (NS_WARN_IF(!global)) {
     return nullptr;
   }
 
@@ -376,9 +410,15 @@ already_AddRefed<File> DataTransferItem::CreateFileFromInputStream(
     key = "GenericFileName";
   }
 
+  return CreateFileFromInputStream(aStream, key, mType);
+}
+
+already_AddRefed<File> DataTransferItem::CreateFileFromInputStream(
+    nsIInputStream* aStream, const char* aFileNameKey,
+    const nsAString& aContentType) {
   nsAutoString fileName;
   nsresult rv = nsContentUtils::GetLocalizedString(
-      nsContentUtils::eDOM_PROPERTIES, key, fileName);
+      nsContentUtils::eDOM_PROPERTIES, aFileNameKey, fileName);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return nullptr;
   }
@@ -390,8 +430,13 @@ already_AddRefed<File> DataTransferItem::CreateFileFromInputStream(
     return nullptr;
   }
 
-  return File::CreateMemoryFile(mDataTransfer, data, available, fileName, mType,
-                                PR_Now());
+  nsCOMPtr<nsIGlobalObject> global = GetGlobalFromDataTransfer();
+  if (NS_WARN_IF(!global)) {
+    return nullptr;
+  }
+
+  return File::CreateMemoryFileWithLastModifiedNow(global, data, available,
+                                                   fileName, aContentType);
 }
 
 void DataTransferItem::GetAsString(FunctionStringCallback* aCallback,
@@ -485,7 +530,7 @@ already_AddRefed<nsIVariant> DataTransferItem::Data(nsIPrincipal* aPrincipal,
 
   // If the inbound principal is system, we can skip the below checks, as
   // they will trivially succeed.
-  if (nsContentUtils::IsSystemPrincipal(aPrincipal)) {
+  if (aPrincipal->IsSystemPrincipal()) {
     return DataNoSecurityCheck();
   }
 
@@ -551,5 +596,22 @@ already_AddRefed<nsIVariant> DataTransferItem::Data(nsIPrincipal* aPrincipal,
   return variant.forget();
 }
 
-}  // namespace dom
-}  // namespace mozilla
+already_AddRefed<nsIGlobalObject>
+DataTransferItem::GetGlobalFromDataTransfer() {
+  nsCOMPtr<nsIGlobalObject> global;
+  // This is annoying, but DataTransfer may have various things as parent.
+  nsCOMPtr<EventTarget> target =
+      do_QueryInterface(mDataTransfer->GetParentObject());
+  if (target) {
+    global = target->GetOwnerGlobal();
+  } else {
+    RefPtr<Event> event = do_QueryObject(mDataTransfer->GetParentObject());
+    if (event) {
+      global = event->GetParentObject();
+    }
+  }
+
+  return global.forget();
+}
+
+}  // namespace mozilla::dom
