@@ -721,16 +721,15 @@ bool RootWindowGlobalKeyListener::IsHTMLEditorFocused() {
   nsINode* focusedNode = fm->GetFocusedElement();
   if (focusedNode && focusedNode->IsElement()) {
     // If there is a focused element, make sure it's in the active editing host.
-    // Note that GetActiveEditingHost finds the current editing host based on
+    // Note that ComputeEditingHost finds the current editing host based on
     // the document's selection.  Even though the document selection is usually
     // collapsed to where the focus is, but the page may modify the selection
     // without our knowledge, in which case this check will do something useful.
-    nsCOMPtr<dom::Element> activeEditingHost =
-        htmlEditor->GetActiveEditingHost();
-    if (!activeEditingHost) {
+    dom::Element* editingHost = htmlEditor->ComputeEditingHost();
+    if (!editingHost) {
       return false;
     }
-    return focusedNode->IsInclusiveDescendantOf(activeEditingHost);
+    return focusedNode->IsInclusiveDescendantOf(editingHost);
   }
 
   return false;
