@@ -4,17 +4,17 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#ifndef mozilla_IMEContentObserver_h_
-#define mozilla_IMEContentObserver_h_
+#ifndef mozilla_IMEContentObserver_h
+#define mozilla_IMEContentObserver_h
 
 #include "mozilla/Attributes.h"
+#include "mozilla/EditorBase.h"
+#include "mozilla/dom/Element.h"
+#include "mozilla/dom/Selection.h"
 #include "nsCOMPtr.h"
 #include "nsCycleCollectionParticipant.h"
-#include "nsIDocShell.h" // XXX Why does only this need to be included here?
-#include "nsIEditor.h"
-#include "nsIEditorObserver.h"
+#include "nsIDocShell.h"  // XXX Why does only this need to be included here?
 #include "nsIReflowObserver.h"
-#include "nsISelectionListener.h"
 #include "nsIScrollObserver.h"
 #include "nsIWidget.h"
 #include "nsStubDocumentObserver.h"
@@ -24,7 +24,6 @@
 
 class nsIContent;
 class nsINode;
-class nsISelection;
 class nsPresContext;
 
 namespace mozilla {
@@ -32,47 +31,48 @@ namespace mozilla {
 class EventStateManager;
 class TextComposition;
 
+namespace dom {
+class Selection;
+}  // namespace dom
+
 // IMEContentObserver notifies widget of any text and selection changes
 // in the currently focused editor
-class IMEContentObserver final : public nsISelectionListener
-                               , public nsStubMutationObserver
-                               , public nsIReflowObserver
-                               , public nsIScrollObserver
-                               , public nsSupportsWeakReference
-                               , public nsIEditorObserver
-{
-public:
-  typedef ContentEventHandler::NodePosition NodePosition;
-  typedef ContentEventHandler::NodePositionBefore NodePositionBefore;
-  typedef widget::IMENotification::SelectionChangeData SelectionChangeData;
-  typedef widget::IMENotification::TextChangeData TextChangeData;
-  typedef widget::IMENotification::TextChangeDataBase TextChangeDataBase;
-  typedef widget::IMENotificationRequests IMENotificationRequests;
-  typedef widget::IMEMessage IMEMessage;
+class IMEContentObserver final : public nsStubMutationObserver,
+                                 public nsIReflowObserver,
+                                 public nsIScrollObserver,
+                                 public nsSupportsWeakReference {
+ public:
+  using SelectionChangeData = widget::IMENotification::SelectionChangeData;
+  using TextChangeData = widget::IMENotification::TextChangeData;
+  using TextChangeDataBase = widget::IMENotification::TextChangeDataBase;
+  using IMENotificationRequests = widget::IMENotificationRequests;
+  using IMEMessage = widget::IMEMessage;
 
   IMEContentObserver();
 
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
   NS_DECL_CYCLE_COLLECTION_CLASS_AMBIGUOUS(IMEContentObserver,
-                                           nsISelectionListener)
-  NS_DECL_NSIEDITOROBSERVER
-  NS_DECL_NSISELECTIONLISTENER
+                                           nsIReflowObserver)
   NS_DECL_NSIMUTATIONOBSERVER_CHARACTERDATAWILLCHANGE
   NS_DECL_NSIMUTATIONOBSERVER_CHARACTERDATACHANGED
   NS_DECL_NSIMUTATIONOBSERVER_CONTENTAPPENDED
   NS_DECL_NSIMUTATIONOBSERVER_CONTENTINSERTED
   NS_DECL_NSIMUTATIONOBSERVER_CONTENTREMOVED
-  NS_DECL_NSIMUTATIONOBSERVER_ATTRIBUTEWILLCHANGE
-  NS_DECL_NSIMUTATIONOBSERVER_ATTRIBUTECHANGED
   NS_DECL_NSIREFLOWOBSERVER
 
   // nsIScrollObserver
   virtual void ScrollPositionChanged() override;
 
-  bool OnMouseButtonEvent(nsPresContext* aPresContext,
-                          WidgetMouseEvent* aMouseEvent);
+  /**
+   * OnSelectionChange() is called when selection is changed in the editor.
+   */
+  void OnSelectionChange(dom::Selection& aSelection);
 
-  nsresult HandleQueryContentEvent(WidgetQueryContentEvent* aEvent);
+  MOZ_CAN_RUN_SCRIPT bool OnMouseButtonEvent(nsPresContext& aPresContext,
+                                             WidgetMouseEvent& aMouseEvent);
+
+  MOZ_CAN_RUN_SCRIPT nsresult
+  HandleQueryContentEvent(WidgetQueryContentEvent* aEvent);
 
   /**
    * Init() initializes the instance, i.e., retrieving necessary objects and
@@ -82,18 +82,12 @@ public:
    *
    * @param aWidget         The widget which can access native IME.
    * @param aPresContext    The PresContext which has aContent.
-   * @param aContent        An editable element or a plugin host element which
-   *                        user may use IME in.
-   *                        Or nullptr if this will observe design mode
-   *                        document.
-   * @param aEditor         When aContent is an editable element or nullptr,
-   *                        non-nullptr referring an editor instance which
-   *                        manages aContent.
-   *                        Otherwise, i.e., this will observe a plugin content,
-   *                        should be nullptr.
+   * @param aElement        An editable element or nullptr if this will observe
+   *                        design mode document.
+   * @param aEditorBase     The editor which is associated with aContent.
    */
-  void Init(nsIWidget* aWidget, nsPresContext* aPresContext,
-            nsIContent* aContent, nsIEditor* aEditor);
+  MOZ_CAN_RUN_SCRIPT void Init(nsIWidget& aWidget, nsPresContext& aPresContext,
+                               dom::Element* aElement, EditorBase& aEditorBase);
 
   /**
    * Destroy() finalizes the instance, i.e., stops observing contents and
@@ -126,33 +120,38 @@ public:
    * @return            Returns true if the instance is managing the content.
    *                    Otherwise, false.
    */
-  bool MaybeReinitialize(nsIWidget* aWidget,
-                         nsPresContext* aPresContext,
-                         nsIContent* aContent,
-                         nsIEditor* aEditor);
+  MOZ_CAN_RUN_SCRIPT bool MaybeReinitialize(nsIWidget& aWidget,
+                                            nsPresContext& aPresContext,
+                                            dom::Element* aElement,
+                                            EditorBase& aEditorBase);
 
-  bool IsManaging(nsPresContext* aPresContext, nsIContent* aContent) const;
-  bool IsManaging(const TextComposition* aTextComposition) const;
-  bool WasInitializedWithPlugin() const;
+  bool IsManaging(const nsPresContext& aPresContext,
+                  const dom::Element* aElement) const;
+  bool IsBeingInitializedFor(const nsPresContext& aPresContext,
+                             const dom::Element* aElement) const;
+  bool IsManaging(const TextComposition& aTextComposition) const;
+  bool WasInitializedWith(const EditorBase& aEditorBase) const {
+    return mEditorBase == &aEditorBase;
+  }
   bool IsEditorHandlingEventForComposition() const;
-  bool KeepAliveDuringDeactive() const
-  {
+  bool KeepAliveDuringDeactive() const {
     return mIMENotificationRequests &&
            mIMENotificationRequests->WantDuringDeactive();
   }
   nsIWidget* GetWidget() const { return mWidget; }
-  nsIEditor* GetEditor() const { return mEditor; }
   void SuppressNotifyingIME();
   void UnsuppressNotifyingIME();
   nsPresContext* GetPresContext() const;
-  nsresult GetSelectionAndRoot(nsISelection** aSelection,
-                               nsIContent** aRoot) const;
+  nsresult GetSelectionAndRoot(dom::Selection** aSelection,
+                               dom::Element** aRootElement) const;
 
   /**
    * TryToFlushPendingNotifications() should be called when pending events
    * should be flushed.  This tries to run the queued IMENotificationSender.
+   * Doesn't do anything in child processes where flushing happens
+   * asynchronously unless aAllowAsync is false.
    */
-  void TryToFlushPendingNotifications();
+  void TryToFlushPendingNotifications(bool aAllowAsync);
 
   /**
    * MaybeNotifyCompositionEventHandled() posts composition event handled
@@ -160,8 +159,22 @@ public:
    */
   void MaybeNotifyCompositionEventHandled();
 
-private:
-  ~IMEContentObserver() {}
+  /**
+   * Following methods are called when the editor:
+   *   - an edit action handled.
+   *   - before handling an edit action.
+   *   - canceled handling an edit action after calling BeforeEditAction().
+   */
+  void OnEditActionHandled();
+  void BeforeEditAction();
+  void CancelEditAction();
+
+  dom::Element* GetObservingElement() const {
+    return mIsObserving ? mRootElement.get() : nullptr;
+  }
+
+ private:
+  ~IMEContentObserver() = default;
 
   enum State {
     eState_NotObserving,
@@ -170,23 +183,16 @@ private:
     eState_Observing
   };
   State GetState() const;
-  bool InitWithEditor(nsPresContext* aPresContext, nsIContent* aContent,
-                      nsIEditor* aEditor);
-  bool InitWithPlugin(nsPresContext* aPresContext, nsIContent* aContent);
-  bool IsInitializedWithPlugin() const { return !mEditor; }
+  MOZ_CAN_RUN_SCRIPT bool InitWithEditor(nsPresContext& aPresContext,
+                                         dom::Element* aElement,
+                                         EditorBase& aEditorBase);
   void OnIMEReceivedFocus();
   void Clear();
-  bool IsObservingContent(nsPresContext* aPresContext,
-                          nsIContent* aContent) const;
-  bool IsReflowLocked() const;
-  bool IsSafeToNotifyIME() const;
-  bool IsEditorComposing() const;
-
-  /**
-   * nsINode::GetChildAt() is slow.  So, this avoids to use it if it's
-   * first child or last child of aParent.
-   */
-  static nsIContent* GetChildNode(nsINode* aParent, int32_t aOffset);
+  [[nodiscard]] bool IsObservingContent(const nsPresContext& aPresContext,
+                                        const dom::Element* aElement) const;
+  [[nodiscard]] bool IsReflowLocked() const;
+  [[nodiscard]] bool IsSafeToNotifyIME() const;
+  [[nodiscard]] bool IsEditorComposing() const;
 
   // Following methods are called by DocumentObserver when
   // beginning to update the contents and ending updating the contents.
@@ -197,9 +203,9 @@ private:
 
   /**
    * MaybeNotifyIMEOfAddedTextDuringDocumentChange() may send text change
-   * notification caused by the nodes added between mFirstAddedNodeOffset in
-   * mFirstAddedNodeContainer and mLastAddedNodeOffset in
-   * mLastAddedNodeContainer and forgets the range.
+   * notification caused by the nodes added between mFirstAddedContent in
+   * mFirstAddedContainer and mLastAddedContent in
+   * mLastAddedContainer and forgets the range.
    */
   void MaybeNotifyIMEOfAddedTextDuringDocumentChange();
 
@@ -209,8 +215,7 @@ private:
    * insertAdjacentHTML().  This returns false when user types something in
    * the focused editor editor.
    */
-  bool IsInDocumentChange() const
-  {
+  bool IsInDocumentChange() const {
     return mDocumentObserver && mDocumentObserver->IsUpdating();
   }
 
@@ -225,17 +230,15 @@ private:
    * have not been sent to IME.  Note that this should always return false when
    * IsInDocumentChange() returns false.
    */
-  bool HasAddedNodesDuringDocumentChange() const
-  {
-    return mFirstAddedNodeContainer && mLastAddedNodeContainer;
+  bool HasAddedNodesDuringDocumentChange() const {
+    return mFirstAddedContainer && mLastAddedContainer;
   }
 
   /**
-   * Returns true if the node at aOffset in aParent is next node of the node at
-   * mLastAddedNodeOffset in mLastAddedNodeContainer in pre-order tree
-   * traversal of the DOM.
+   * Returns true if the passed-in node in aParent is the next node of
+   * mLastAddedContent in pre-order tree traversal of the DOM.
    */
-  bool IsNextNodeOfLastAddedNode(nsINode* aParent, int32_t aOffset) const;
+  bool IsNextNodeOfLastAddedNode(nsINode* aParent, nsIContent* aChild) const;
 
   void PostFocusSetNotification();
   void MaybeNotifyIMEOfFocusSet();
@@ -251,7 +254,8 @@ private:
   void CancelNotifyingIMEOfPositionChange();
   void PostCompositionEventHandledNotification();
 
-  void NotifyContentAdded(nsINode* aContainer, int32_t aStart, int32_t aEnd);
+  void NotifyContentAdded(nsINode* aContainer, nsIContent* aFirstContent,
+                          nsIContent* aLastContent);
   void ObserveEditableNode();
   /**
    *  NotifyIMEOfBlur() notifies IME of blur.
@@ -262,18 +266,15 @@ private:
    */
   void UnregisterObservers();
   void FlushMergeableNotifications();
-  bool NeedsTextChangeNotification() const
-  {
+  bool NeedsTextChangeNotification() const {
     return mIMENotificationRequests &&
            mIMENotificationRequests->WantTextChange();
   }
-  bool NeedsPositionChangeNotification() const
-  {
+  bool NeedsPositionChangeNotification() const {
     return mIMENotificationRequests &&
            mIMENotificationRequests->WantPositionChanged();
   }
-  void ClearPendingNotifications()
-  {
+  void ClearPendingNotifications() {
     mNeedsToNotifyIMEOfFocusSet = false;
     mNeedsToNotifyIMEOfTextChange = false;
     mNeedsToNotifyIMEOfSelectionChange = false;
@@ -281,10 +282,8 @@ private:
     mNeedsToNotifyIMEOfCompositionEventHandled = false;
     mTextChangeData.Clear();
   }
-  bool NeedsToNotifyIMEOfSomething() const
-  {
-    return mNeedsToNotifyIMEOfFocusSet ||
-           mNeedsToNotifyIMEOfTextChange ||
+  bool NeedsToNotifyIMEOfSomething() const {
+    return mNeedsToNotifyIMEOfFocusSet || mNeedsToNotifyIMEOfTextChange ||
            mNeedsToNotifyIMEOfSelectionChange ||
            mNeedsToNotifyIMEOfPositionChange ||
            mNeedsToNotifyIMEOfCompositionEventHandled;
@@ -293,31 +292,27 @@ private:
   /**
    * UpdateSelectionCache() updates mSelectionData with the latest selection.
    * This should be called only when IsSafeToNotifyIME() returns true.
-   *
-   * Note that this does nothing if WasInitializedWithPlugin() returns true.
    */
-  bool UpdateSelectionCache();
+  MOZ_CAN_RUN_SCRIPT bool UpdateSelectionCache(bool aRequireFlush = true);
 
   nsCOMPtr<nsIWidget> mWidget;
   // mFocusedWidget has the editor observed by the instance.  E.g., if the
   // focused editor is in XUL panel, this should be the widget of the panel.
   // On the other hand, mWidget is its parent which handles IME.
   nsCOMPtr<nsIWidget> mFocusedWidget;
-  nsCOMPtr<nsISelection> mSelection;
-  nsCOMPtr<nsIContent> mRootContent;
+  RefPtr<dom::Selection> mSelection;
+  RefPtr<dom::Element> mRootElement;
   nsCOMPtr<nsINode> mEditableNode;
   nsCOMPtr<nsIDocShell> mDocShell;
-  nsCOMPtr<nsIEditor> mEditor;
+  RefPtr<EditorBase> mEditorBase;
 
   /**
    * Helper classes to notify IME.
    */
 
-  class AChangeEvent: public Runnable
-  {
-  protected:
-    enum ChangeEventType
-    {
+  class AChangeEvent : public Runnable {
+   protected:
+    enum ChangeEventType {
       eChangeEventType_Focus,
       eChangeEventType_Selection,
       eChangeEventType_Text,
@@ -327,13 +322,19 @@ private:
 
     explicit AChangeEvent(const char* aName,
                           IMEContentObserver* aIMEContentObserver)
-      : Runnable(aName)
-      , mIMEContentObserver(aIMEContentObserver)
-    {
-      MOZ_ASSERT(mIMEContentObserver);
+        : Runnable(aName),
+          mIMEContentObserver(do_GetWeakReference(
+              static_cast<nsIReflowObserver*>(aIMEContentObserver))) {
+      MOZ_ASSERT(aIMEContentObserver);
     }
 
-    RefPtr<IMEContentObserver> mIMEContentObserver;
+    already_AddRefed<IMEContentObserver> GetObserver() const {
+      nsCOMPtr<nsIReflowObserver> observer =
+          do_QueryReferent(mIMEContentObserver);
+      return observer.forget().downcast<IMEContentObserver>();
+    }
+
+    nsWeakPtr mIMEContentObserver;
 
     /**
      * CanNotifyIME() checks if mIMEContentObserver can and should notify IME.
@@ -346,19 +347,18 @@ private:
     bool IsSafeToNotifyIME(ChangeEventType aChangeEventType) const;
   };
 
-  class IMENotificationSender: public AChangeEvent
-  {
-  public:
+  class IMENotificationSender : public AChangeEvent {
+   public:
     explicit IMENotificationSender(IMEContentObserver* aIMEContentObserver)
-      : AChangeEvent("IMENotificationSender", aIMEContentObserver)
-      , mIsRunning(false)
-    {
-    }
-    NS_IMETHOD Run() override;
+        : AChangeEvent("IMENotificationSender", aIMEContentObserver),
+          mIsRunning(false) {}
+    MOZ_CAN_RUN_SCRIPT_BOUNDARY NS_IMETHOD Run() override;
 
-  private:
-    void SendFocusSet();
-    void SendSelectionChange();
+    void Dispatch(nsIDocShell* aDocShell);
+
+   private:
+    MOZ_CAN_RUN_SCRIPT void SendFocusSet();
+    MOZ_CAN_RUN_SCRIPT void SendSelectionChange();
     void SendTextChange();
     void SendPositionChange();
     void SendCompositionEventHandled();
@@ -378,21 +378,17 @@ private:
    * methods need to check if the changing node is in mRootContent but it's
    * too expensive.
    */
-  class DocumentObserver final : public nsStubDocumentObserver
-  {
-  public:
+  class DocumentObserver final : public nsStubDocumentObserver {
+   public:
     explicit DocumentObserver(IMEContentObserver& aIMEContentObserver)
-      : mIMEContentObserver(&aIMEContentObserver)
-      , mDocumentUpdating(0)
-    {
-    }
+        : mIMEContentObserver(&aIMEContentObserver), mDocumentUpdating(0) {}
 
     NS_DECL_CYCLE_COLLECTION_CLASS(DocumentObserver)
     NS_DECL_CYCLE_COLLECTING_ISUPPORTS
     NS_DECL_NSIDOCUMENTOBSERVER_BEGINUPDATE
     NS_DECL_NSIDOCUMENTOBSERVER_ENDUPDATE
 
-    void Observe(nsIDocument* aDocument);
+    void Observe(dom::Document*);
     void StopObserving();
     void Destroy();
 
@@ -400,12 +396,12 @@ private:
     bool IsObserving() const { return mDocument != nullptr; }
     bool IsUpdating() const { return mDocumentUpdating != 0; }
 
-  private:
+   private:
     DocumentObserver() = delete;
     virtual ~DocumentObserver() { Destroy(); }
 
     RefPtr<IMEContentObserver> mIMEContentObserver;
-    nsCOMPtr<nsIDocument> mDocument;
+    RefPtr<dom::Document> mDocument;
     uint32_t mDocumentUpdating;
   };
   RefPtr<DocumentObserver> mDocumentObserver;
@@ -414,44 +410,37 @@ private:
    * FlatTextCache stores flat text length from start of the content to
    * mNodeOffset of mContainerNode.
    */
-  struct FlatTextCache
-  {
-    // mContainerNode and mNodeOffset represent a point in DOM tree.  E.g.,
-    // if mContainerNode is a div element, mNodeOffset is index of its child.
+  struct FlatTextCache {
+    // mContainerNode and mNode represent a point in DOM tree.  E.g.,
+    // if mContainerNode is a div element, mNode is a child.
     nsCOMPtr<nsINode> mContainerNode;
-    int32_t mNodeOffset;
+    // mNode points to the last child which participates in the current
+    // mFlatTextLength. If mNode is null, then that means that the end point for
+    // mFlatTextLength is immediately before the first child of mContainerNode.
+    nsCOMPtr<nsINode> mNode;
     // Length of flat text generated from contents between the start of content
     // and a child node whose index is mNodeOffset of mContainerNode.
     uint32_t mFlatTextLength;
 
-    FlatTextCache()
-      : mNodeOffset(0)
-      , mFlatTextLength(0)
-    {
-    }
+    FlatTextCache() : mFlatTextLength(0) {}
 
-    void Clear()
-    {
+    void Clear() {
       mContainerNode = nullptr;
-      mNodeOffset = 0;
+      mNode = nullptr;
       mFlatTextLength = 0;
     }
 
-    void Cache(nsINode* aContainer, int32_t aNodeOffset,
-               uint32_t aFlatTextLength)
-    {
+    void Cache(nsINode* aContainer, nsINode* aNode, uint32_t aFlatTextLength) {
       MOZ_ASSERT(aContainer, "aContainer must not be null");
-      MOZ_ASSERT(
-        aNodeOffset <= static_cast<int32_t>(aContainer->GetChildCount()),
-        "aNodeOffset must be same as or less than the count of children");
+      MOZ_ASSERT(!aNode || aNode->GetParentNode() == aContainer,
+                 "aNode must be either null or a child of aContainer");
       mContainerNode = aContainer;
-      mNodeOffset = aNodeOffset;
+      mNode = aNode;
       mFlatTextLength = aFlatTextLength;
     }
 
-    bool Match(nsINode* aContainer, int32_t aNodeOffset) const
-    {
-      return aContainer == mContainerNode && aNodeOffset == mNodeOffset;
+    bool Match(nsINode* aContainer, nsINode* aNode) const {
+      return aContainer == mContainerNode && aNode == mNode;
     }
   };
   // mEndOfAddedTextCache caches text length from the start of content to
@@ -464,26 +453,25 @@ private:
   // handled by the editor and no other mutation (e.g., adding node) occur.
   FlatTextCache mStartOfRemovingTextRangeCache;
 
-  // mFirstAddedNodeContainer is parent node of first added node in current
+  // mFirstAddedContainer is parent node of first added node in current
   // document change.  So, this is not nullptr only when a node was added
   // during a document change and the change has not been included into
   // mTextChangeData yet.
   // Note that this shouldn't be in cycle collection since this is not nullptr
   // only during a document change.
-  nsCOMPtr<nsINode> mFirstAddedNodeContainer;
-  // mLastAddedNodeContainer is parent node of last added node in current
+  nsCOMPtr<nsINode> mFirstAddedContainer;
+  // mLastAddedContainer is parent node of last added node in current
   // document change.  So, this is not nullptr only when a node was added
   // during a document change and the change has not been included into
   // mTextChangeData yet.
   // Note that this shouldn't be in cycle collection since this is not nullptr
   // only during a document change.
-  nsCOMPtr<nsINode> mLastAddedNodeContainer;
-  // mFirstAddedNodeOffset is offset of first added node in
-  // mFirstAddedNodeContainer.
-  int32_t mFirstAddedNodeOffset;
-  // mLastAddedNodeOffset is offset of *after* last added node in
-  // mLastAddedNodeContainer.  I.e., the index of last added node + 1.
-  int32_t mLastAddedNodeOffset;
+  nsCOMPtr<nsINode> mLastAddedContainer;
+
+  // mFirstAddedContent is the first node added in mFirstAddedContainer.
+  nsCOMPtr<nsIContent> mFirstAddedContent;
+  // mLastAddedContent is the last node added in mLastAddedContainer;
+  nsCOMPtr<nsIContent> mLastAddedContent;
 
   TextChangeData mTextChangeData;
 
@@ -495,7 +483,6 @@ private:
   EventStateManager* mESM;
 
   const IMENotificationRequests* mIMENotificationRequests;
-  uint32_t mPreAttrChangeLength;
   uint32_t mSuppressNotifications;
   int64_t mPreCharacterDataChangeLength;
 
@@ -516,6 +503,6 @@ private:
   bool mIsHandlingQueryContentEvent;
 };
 
-} // namespace mozilla
+}  // namespace mozilla
 
-#endif // mozilla_IMEContentObserver_h_
+#endif  // mozilla_IMEContentObserver_h
