@@ -6,21 +6,22 @@
 
 #include "mozilla/dom/InternalHeaders.h"
 
+#include "FetchUtil.h"
 #include "mozilla/dom/FetchTypes.h"
 #include "mozilla/ErrorResult.h"
 
 #include "nsCharSeparatedTokenizer.h"
 #include "nsContentUtils.h"
+#include "nsIHttpChannel.h"
 #include "nsIHttpHeaderVisitor.h"
 #include "nsNetUtil.h"
 #include "nsReadableUtils.h"
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
-InternalHeaders::InternalHeaders(const nsTArray<Entry>&& aHeaders,
+InternalHeaders::InternalHeaders(nsTArray<Entry>&& aHeaders,
                                  HeadersGuardEnum aGuard)
-    : mGuard(aGuard), mList(aHeaders), mListDirty(true) {}
+    : mGuard(aGuard), mList(std::move(aHeaders)), mListDirty(true) {}
 
 InternalHeaders::InternalHeaders(
     const nsTArray<HeadersEntry>& aHeadersEntryList, HeadersGuardEnum aGuard)
@@ -56,11 +57,11 @@ bool InternalHeaders::IsValidHeaderValue(const nsCString& aLowerName,
   }
 
   // Step 4
-  if (mGuard == HeadersGuardEnum::Request &&
-      IsForbiddenRequestHeader(aLowerName)) {
-    return false;
+  if (mGuard == HeadersGuardEnum::Request) {
+    if (IsForbiddenRequestHeader(aLowerName, aNormalizedValue)) {
+      return false;
+    }
   }
-
   // Step 5
   if (mGuard == HeadersGuardEnum::Request_no_cors) {
     nsAutoCString tempValue;
@@ -161,7 +162,9 @@ void InternalHeaders::Delete(const nsACString& aName, ErrorResult& aRv) {
   }
 
   // Step 3
-  if (IsForbiddenRequestHeader(lowerName)) {
+  nsAutoCString value;
+  GetInternal(lowerName, value, aRv);
+  if (IsForbiddenRequestHeader(lowerName, value)) {
     return;
   }
 
@@ -308,7 +311,7 @@ void InternalHeaders::SetGuard(HeadersGuardEnum aGuard, ErrorResult& aRv) {
   mGuard = aGuard;
 }
 
-InternalHeaders::~InternalHeaders() {}
+InternalHeaders::~InternalHeaders() = default;
 
 // static
 bool InternalHeaders::IsNoCorsSafelistedRequestHeaderName(
@@ -356,8 +359,7 @@ bool InternalHeaders::IsRevalidationHeader(const nsCString& aName) {
 // static
 bool InternalHeaders::IsInvalidName(const nsACString& aName, ErrorResult& aRv) {
   if (!NS_IsValidHTTPToken(aName)) {
-    NS_ConvertUTF8toUTF16 label(aName);
-    aRv.ThrowTypeError<MSG_INVALID_HEADER_NAME>(label);
+    aRv.ThrowTypeError<MSG_INVALID_HEADER_NAME>(aName);
     return true;
   }
 
@@ -368,8 +370,7 @@ bool InternalHeaders::IsInvalidName(const nsACString& aName, ErrorResult& aRv) {
 bool InternalHeaders::IsInvalidValue(const nsACString& aValue,
                                      ErrorResult& aRv) {
   if (!NS_IsReasonableHTTPHeaderValue(aValue)) {
-    NS_ConvertUTF8toUTF16 label(aValue);
-    aRv.ThrowTypeError<MSG_INVALID_HEADER_VALUE>(label);
+    aRv.ThrowTypeError<MSG_INVALID_HEADER_VALUE>(aValue);
     return true;
   }
   return false;
@@ -377,21 +378,22 @@ bool InternalHeaders::IsInvalidValue(const nsACString& aValue,
 
 bool InternalHeaders::IsImmutable(ErrorResult& aRv) const {
   if (mGuard == HeadersGuardEnum::Immutable) {
-    aRv.ThrowTypeError<MSG_HEADERS_IMMUTABLE>();
+    aRv.ThrowTypeError("Headers are immutable and cannot be modified.");
     return true;
   }
   return false;
 }
 
-bool InternalHeaders::IsForbiddenRequestHeader(const nsCString& aName) const {
+bool InternalHeaders::IsForbiddenRequestHeader(const nsCString& aName,
+                                               const nsACString& aValue) const {
   return mGuard == HeadersGuardEnum::Request &&
-         nsContentUtils::IsForbiddenRequestHeader(aName);
+         nsContentUtils::IsForbiddenRequestHeader(aName, aValue);
 }
 
 bool InternalHeaders::IsForbiddenRequestNoCorsHeader(
     const nsCString& aName) const {
   return mGuard == HeadersGuardEnum::Request_no_cors &&
-         !IsSimpleHeader(aName, EmptyCString());
+         !IsSimpleHeader(aName, ""_ns);
 }
 
 bool InternalHeaders::IsForbiddenRequestNoCorsHeader(
@@ -418,7 +420,9 @@ void InternalHeaders::Fill(const Sequence<Sequence<nsCString>>& aInit,
   for (uint32_t i = 0; i < aInit.Length() && !aRv.Failed(); ++i) {
     const Sequence<nsCString>& tuple = aInit[i];
     if (tuple.Length() != 2) {
-      aRv.ThrowTypeError<MSG_INVALID_HEADER_SEQUENCE>();
+      aRv.ThrowTypeError(
+          "Headers require name/value tuples when being initialized by a "
+          "sequence.");
       return;
     }
     Append(tuple[0], tuple[1], aRv);
@@ -501,28 +505,27 @@ already_AddRefed<InternalHeaders> InternalHeaders::BasicHeaders(
   ErrorResult result;
   // The Set-Cookie headers cannot be invalid mutable headers, so the Delete
   // must succeed.
-  basic->Delete(NS_LITERAL_CSTRING("Set-Cookie"), result);
+  basic->Delete("Set-Cookie"_ns, result);
   MOZ_ASSERT(!result.Failed());
-  basic->Delete(NS_LITERAL_CSTRING("Set-Cookie2"), result);
+  basic->Delete("Set-Cookie2"_ns, result);
   MOZ_ASSERT(!result.Failed());
   return basic.forget();
 }
 
 // static
 already_AddRefed<InternalHeaders> InternalHeaders::CORSHeaders(
-    InternalHeaders* aHeaders) {
+    InternalHeaders* aHeaders, RequestCredentials aCredentialsMode) {
   RefPtr<InternalHeaders> cors = new InternalHeaders(aHeaders->mGuard);
   ErrorResult result;
 
   nsAutoCString acExposedNames;
-  aHeaders->Get(NS_LITERAL_CSTRING("Access-Control-Expose-Headers"),
-                acExposedNames, result);
+  aHeaders->Get("Access-Control-Expose-Headers"_ns, acExposedNames, result);
   MOZ_ASSERT(!result.Failed());
 
+  bool allowAllHeaders = false;
   AutoTArray<nsCString, 5> exposeNamesArray;
-  nsCCharSeparatedTokenizer exposeTokens(acExposedNames, ',');
-  while (exposeTokens.hasMoreTokens()) {
-    const nsDependentCSubstring& token = exposeTokens.nextToken();
+  for (const nsACString& token :
+       nsCCharSeparatedTokenizer(acExposedNames, ',').ToRange()) {
     if (token.IsEmpty()) {
       continue;
     }
@@ -536,19 +539,28 @@ already_AddRefed<InternalHeaders> InternalHeaders::CORSHeaders(
       break;
     }
 
+    if (token.EqualsLiteral("*") &&
+        aCredentialsMode != RequestCredentials::Include) {
+      allowAllHeaders = true;
+    }
+
     exposeNamesArray.AppendElement(token);
   }
 
   nsCaseInsensitiveCStringArrayComparator comp;
   for (uint32_t i = 0; i < aHeaders->mList.Length(); ++i) {
     const Entry& entry = aHeaders->mList[i];
-    if (entry.mName.EqualsIgnoreCase("cache-control") ||
-        entry.mName.EqualsIgnoreCase("content-language") ||
-        entry.mName.EqualsIgnoreCase("content-type") ||
-        entry.mName.EqualsIgnoreCase("expires") ||
-        entry.mName.EqualsIgnoreCase("last-modified") ||
-        entry.mName.EqualsIgnoreCase("pragma") ||
-        exposeNamesArray.Contains(entry.mName, comp)) {
+    if (allowAllHeaders) {
+      cors->Append(entry.mName, entry.mValue, result);
+      MOZ_ASSERT(!result.Failed());
+    } else if (entry.mName.EqualsIgnoreCase("cache-control") ||
+               entry.mName.EqualsIgnoreCase("content-language") ||
+               entry.mName.EqualsIgnoreCase("content-type") ||
+               entry.mName.EqualsIgnoreCase("content-length") ||
+               entry.mName.EqualsIgnoreCase("expires") ||
+               entry.mName.EqualsIgnoreCase("last-modified") ||
+               entry.mName.EqualsIgnoreCase("pragma") ||
+               exposeNamesArray.Contains(entry.mName, comp)) {
       cors->Append(entry.mName, entry.mValue, result);
       MOZ_ASSERT(!result.Failed());
     }
@@ -627,5 +639,4 @@ void InternalHeaders::ReuseExistingNameIfExists(nsCString& aName) const {
   }
 }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
