@@ -7,59 +7,82 @@
 #ifndef mozilla_dom_InternalResponse_h
 #define mozilla_dom_InternalResponse_h
 
+#include "mozilla/dom/FetchTypes.h"
 #include "nsIInputStream.h"
 #include "nsICacheInfoChannel.h"
 #include "nsISupportsImpl.h"
 #include "nsProxyRelease.h"
 
 #include "mozilla/dom/InternalHeaders.h"
+#include "mozilla/dom/RequestBinding.h"
 #include "mozilla/dom/ResponseBinding.h"
 #include "mozilla/dom/ChannelInfo.h"
+#include "mozilla/dom/SafeRefPtr.h"
+#include "mozilla/NotNull.h"
 #include "mozilla/UniquePtr.h"
 
 namespace mozilla {
 namespace ipc {
+class PBackgroundChild;
+class PBackgroundParent;
 class PrincipalInfo;
-class AutoIPCStream;
 }  // namespace ipc
 
 namespace dom {
 
+class ChildToParentInternalResponse;
 class InternalHeaders;
+class ParentToChildInternalResponse;
+class ParentToParentInternalResponse;
 
-class InternalResponse final {
+class InternalResponse final : public AtomicSafeRefCounted<InternalResponse> {
   friend class FetchDriver;
 
  public:
-  NS_INLINE_DECL_THREADSAFE_REFCOUNTING(InternalResponse)
+  MOZ_DECLARE_REFCOUNTED_TYPENAME(InternalResponse)
 
-  InternalResponse(uint16_t aStatus, const nsACString& aStatusText);
+  InternalResponse(
+      uint16_t aStatus, const nsACString& aStatusText,
+      RequestCredentials aCredentialsMode = RequestCredentials::Omit);
+
+  static SafeRefPtr<InternalResponse> FromIPC(
+      const ParentToChildInternalResponse& aIPCResponse);
+
+  static SafeRefPtr<InternalResponse> FromIPC(
+      const ParentToParentInternalResponse& aIPCResponse);
+
+  void ToChildToParentInternalResponse(
+      ChildToParentInternalResponse* aIPCResponse,
+      mozilla::ipc::PBackgroundChild* aManager);
+
+  ParentToParentInternalResponse ToParentToParentInternalResponse();
 
   enum CloneType {
     eCloneInputStream,
     eDontCloneInputStream,
   };
 
-  already_AddRefed<InternalResponse> Clone(CloneType eCloneType);
+  SafeRefPtr<InternalResponse> Clone(CloneType aCloneType);
 
-  static already_AddRefed<InternalResponse> NetworkError(nsresult aRv) {
+  static SafeRefPtr<InternalResponse> NetworkError(nsresult aRv) {
     MOZ_DIAGNOSTIC_ASSERT(NS_FAILED(aRv));
-    RefPtr<InternalResponse> response = new InternalResponse(0, EmptyCString());
+    SafeRefPtr<InternalResponse> response =
+        MakeSafeRefPtr<InternalResponse>(0, ""_ns);
     ErrorResult result;
     response->Headers()->SetGuard(HeadersGuardEnum::Immutable, result);
     MOZ_ASSERT(!result.Failed());
     response->mType = ResponseType::Error;
     response->mErrorCode = aRv;
-    return response.forget();
+    return response;
   }
 
-  already_AddRefed<InternalResponse> OpaqueResponse();
+  SafeRefPtr<InternalResponse> OpaqueResponse();
 
-  already_AddRefed<InternalResponse> OpaqueRedirectResponse();
+  SafeRefPtr<InternalResponse> OpaqueRedirectResponse();
 
-  already_AddRefed<InternalResponse> BasicResponse();
+  SafeRefPtr<InternalResponse> BasicResponse();
 
-  already_AddRefed<InternalResponse> CORSResponse();
+  SafeRefPtr<InternalResponse> CORSResponse();
 
   ResponseType Type() const {
     MOZ_ASSERT_IF(mType == ResponseType::Error, !mWrappedResponse);
@@ -98,12 +121,18 @@ class InternalResponse final {
     return GetURLList(aURLList);
   }
 
+  nsTArray<nsCString> GetUnfilteredURLList() const {
+    nsTArray<nsCString> list;
+    GetUnfilteredURLList(list);
+    return list;
+  }
+
   void SetURLList(const nsTArray<nsCString>& aURLList) {
     mURLList.Assign(aURLList);
 
 #ifdef DEBUG
     for (uint32_t i = 0; i < mURLList.Length(); ++i) {
-      MOZ_ASSERT(mURLList[i].Find(NS_LITERAL_CSTRING("#")) == kNotFound);
+      MOZ_ASSERT(mURLList[i].Find("#"_ns) == kNotFound);
     }
 #endif
   }
@@ -208,6 +237,24 @@ class InternalResponse final {
 
   void SetPaddingSize(int64_t aPaddingSize);
 
+  void SetAlternativeDataType(const nsACString& aAltDataType) {
+    if (mWrappedResponse) {
+      return mWrappedResponse->SetAlternativeDataType(aAltDataType);
+    }
+
+    MOZ_DIAGNOSTIC_ASSERT(mAlternativeDataType.IsEmpty());
+
+    mAlternativeDataType.Assign(aAltDataType);
+  }
+
+  const nsCString& GetAlternativeDataType() {
+    if (mWrappedResponse) {
+      return mWrappedResponse->GetAlternativeDataType();
+    }
+
+    return mAlternativeDataType;
+  }
+
   void SetAlternativeBody(nsIInputStream* aAlternativeBody) {
     if (mWrappedResponse) {
       return mWrappedResponse->SetAlternativeBody(aAlternativeBody);
@@ -259,12 +306,14 @@ class InternalResponse final {
     return !!mCacheInfoChannel;
   }
 
+  bool HasBeenCloned() const { return mCloned; }
+
   void InitChannelInfo(nsIChannel* aChannel) {
     mChannelInfo.InitFromChannel(aChannel);
   }
 
-  void InitChannelInfo(const mozilla::ipc::IPCChannelInfo& aChannelInfo) {
-    mChannelInfo.InitFromIPCChannelInfo(aChannelInfo);
+  void InitChannelInfo(nsITransportSecurityInfo* aSecurityInfo) {
+    mChannelInfo.InitFromTransportSecurityInfo(aSecurityInfo);
   }
 
   void InitChannelInfo(const ChannelInfo& aChannelInfo) {
@@ -286,21 +335,25 @@ class InternalResponse final {
 
   LoadTainting GetTainting() const;
 
-  already_AddRefed<InternalResponse> Unfiltered();
+  SafeRefPtr<InternalResponse> Unfiltered();
 
- private:
   ~InternalResponse();
 
+ private:
   explicit InternalResponse(const InternalResponse& aOther) = delete;
   InternalResponse& operator=(const InternalResponse&) = delete;
 
   // Returns an instance of InternalResponse which is a copy of this
   // InternalResponse, except headers, body and wrapped response (if any) which
   // are left uninitialized. Used for cloning and filtering.
-  already_AddRefed<InternalResponse> CreateIncompleteCopy();
+  SafeRefPtr<InternalResponse> CreateIncompleteCopy();
+
+  template <typename T>
+  static SafeRefPtr<InternalResponse> FromIPCTemplate(const T& aIPCResponse);
+
+  InternalResponseMetadata GetMetadata();
 
   ResponseType mType;
-  nsCString mTerminationReason;
   // A response has an associated url list (a list of zero or more fetch URLs).
   // Unless stated otherwise, it is the empty list. The current url is the last
   // element in mURLlist
@@ -317,14 +370,17 @@ class InternalResponse final {
   Maybe<uint32_t> mPaddingInfo;
   int64_t mPaddingSize;
   nsresult mErrorCode;
+  RequestCredentials mCredentialsMode;
 
   // For alternative data such as JS Bytecode cached in the HTTP cache.
+  nsCString mAlternativeDataType;
   nsCOMPtr<nsIInputStream> mAlternativeBody;
   nsMainThreadPtrHandle<nsICacheInfoChannel> mCacheInfoChannel;
+  bool mCloned;
 
  public:
-  static const int64_t UNKNOWN_BODY_SIZE = -1;
-  static const int64_t UNKNOWN_PADDING_SIZE = -1;
+  static constexpr int64_t UNKNOWN_BODY_SIZE = -1;
+  static constexpr int64_t UNKNOWN_PADDING_SIZE = -1;
 
  private:
   ChannelInfo mChannelInfo;
@@ -334,9 +390,13 @@ class InternalResponse final {
   // Cache, and SW interception should always serialize/access the underlying
   // unfiltered headers and when deserializing, create an InternalResponse
   // with the unfiltered headers followed by wrapping it.
-  RefPtr<InternalResponse> mWrappedResponse;
+  SafeRefPtr<InternalResponse> mWrappedResponse;
 };
 
+ParentToChildInternalResponse ToParentToChild(
+    const ParentToParentInternalResponse& aResponse,
+    NotNull<mozilla::ipc::PBackgroundParent*> aBackgroundParent);
+
 }  // namespace dom
 }  // namespace mozilla
 
