@@ -10,6 +10,7 @@
 #include "mozilla/dom/HTMLUnknownElement.h"
 #include "mozilla/StaticPrefs_dom.h"
 
+#include "nsContentUtils.h"
 #include "nsFocusManager.h"
 #include "nsIFrame.h"
 
@@ -51,6 +52,18 @@ void HTMLDialogElement::Close(
 
   RemoveFromTopLayerIfNeeded();
 
+  RefPtr<Element> previouslyFocusedElement =
+      do_QueryReferent(mPreviouslyFocusedElement);
+
+  if (previouslyFocusedElement) {
+    mPreviouslyFocusedElement = nullptr;
+
+    FocusOptions options;
+    options.mPreventScroll = true;
+    previouslyFocusedElement->Focus(options, CallerType::NonSystem,
+                                    IgnoredErrorResult());
+  }
+
   RefPtr<AsyncEventDispatcher> eventDispatcher =
       new AsyncEventDispatcher(this, u"close"_ns, CanBubble::eNo);
   eventDispatcher->PostDOMEvent();
@@ -61,35 +74,39 @@ void HTMLDialogElement::Show() {
     return;
   }
   SetOpen(true, IgnoreErrors());
+
+  StorePreviouslyFocusedElement();
+
   FocusDialog();
 }
 
 bool HTMLDialogElement::IsInTopLayer() const {
-  return State().HasState(NS_EVENT_STATE_MODAL_DIALOG);
+  return State().HasState(ElementState::MODAL);
 }
 
 void HTMLDialogElement::AddToTopLayerIfNeeded() {
+  MOZ_ASSERT(IsInComposedDoc());
   if (IsInTopLayer()) {
     return;
   }
 
-  Document* doc = OwnerDoc();
-  doc->TopLayerPush(this);
-  doc->SetBlockedByModalDialog(*this);
-  AddStates(NS_EVENT_STATE_MODAL_DIALOG);
+  OwnerDoc()->AddModalDialog(*this);
 }
 
 void HTMLDialogElement::RemoveFromTopLayerIfNeeded() {
   if (!IsInTopLayer()) {
     return;
   }
-  auto predictFunc = [&](Element* element) { return element == this; };
+  OwnerDoc()->RemoveModalDialog(*this);
+}
 
-  Document* doc = OwnerDoc();
-  DebugOnly<Element*> removedElement = doc->TopLayerPop(predictFunc);
-  MOZ_ASSERT(removedElement == this);
-  RemoveStates(NS_EVENT_STATE_MODAL_DIALOG);
-  doc->UnsetBlockedByModalDialog(*this);
+void HTMLDialogElement::StorePreviouslyFocusedElement() {
+  if (Document* doc = GetComposedDoc()) {
+    if (nsIContent* unretargetedFocus = doc->GetUnretargetedFocusedContent()) {
+      mPreviouslyFocusedElement =
+          do_GetWeakReference(unretargetedFocus->AsElement());
+    }
+  }
 }
 
 void HTMLDialogElement::UnbindFromTree(bool aNullParent) {
@@ -99,20 +116,20 @@ void HTMLDialogElement::UnbindFromTree(bool aNullParent) {
 
 void HTMLDialogElement::ShowModal(ErrorResult& aError) {
   if (!IsInComposedDoc()) {
-    aError.ThrowInvalidStateError("Dialog element is not connected");
-    return;
+    return aError.ThrowInvalidStateError("Dialog element is not connected");
   }
 
   if (Open()) {
-    aError.ThrowInvalidStateError(
+    return aError.ThrowInvalidStateError(
         "Dialog element already has an 'open' attribute");
-    return;
   }
 
   AddToTopLayerIfNeeded();
 
   SetOpen(true, aError);
 
+  StorePreviouslyFocusedElement();
+
   FocusDialog();
 
   aError.SuppressException();
@@ -126,34 +143,15 @@ void HTMLDialogElement::FocusDialog() {
     doc->FlushPendingNotifications(FlushType::Frames);
   }
 
-  Element* control = nullptr;
-  nsIContent* child = GetFirstChild();
-  while (child) {
-    if (child->IsElement()) {
-      nsIFrame* frame = child->GetPrimaryFrame();
-      if (frame && frame->IsFocusable()) {
-        if (child->AsElement()->HasAttr(kNameSpaceID_None,
-                                        nsGkAtoms::autofocus)) {
-          // Find the first descendant of element of subject that this
-          // not inert and has autofucus attribute
-          // inert bug: https://bugzilla.mozilla.org/show_bug.cgi?id=921504
-          control = child->AsElement();
-          break;
-        }
-        // If there isn't one, then let control be the first non-inert
-        // descendant element of subject, in tree order.
-        if (!control) {
-          control = child->AsElement();
-        }
-      }
-    }
-    child = child->GetNextNode(this);
-  }
+  Element* controlCandidate = GetFocusDelegate(false /* aWithMouse */);
+
   // If there isn't one of those either, then let control be subject.
-  if (!control) {
-    control = this;
+  if (!controlCandidate) {
+    controlCandidate = this;
   }
 
+  RefPtr<Element> control = controlCandidate;
+
   // 3) Run the focusing steps for control.
   ErrorResult rv;
   nsIFrame* frame = control->GetPrimaryFrame();
@@ -162,11 +160,11 @@ void HTMLDialogElement::FocusDialog() {
     if (rv.Failed()) {
       return;
     }
-  } else {
-    nsFocusManager* fm = nsFocusManager::GetFocusManager();
-    if (fm) {
+  } else if (IsInTopLayer()) {
+    if (RefPtr<nsFocusManager> fm = nsFocusManager::GetFocusManager()) {
       // Clear the focus which ends up making the body gets focused
-      fm->ClearFocus(OwnerDoc()->GetWindow());
+      nsCOMPtr<nsPIDOMWindowOuter> outerWindow = OwnerDoc()->GetWindow();
+      fm->ClearFocus(outerWindow);
     }
   }
 
@@ -176,9 +174,8 @@ void HTMLDialogElement::FocusDialog() {
   // topDocument, then return.
   BrowsingContext* bc = control->OwnerDoc()->GetBrowsingContext();
   if (bc && bc->SameOriginWithTop()) {
-    nsCOMPtr<nsIDocShell> docShell = bc->Top()->GetDocShell();
-    if (docShell) {
-      if (Document* topDocument = docShell->GetDocument()) {
+    if (nsCOMPtr<nsIDocShell> docShell = bc->Top()->GetDocShell()) {
+      if (Document* topDocument = docShell->GetExtantDocument()) {
         // 6) Empty topDocument's autofocus candidates.
         // 7) Set topDocument's autofocus processed flag to true.
         topDocument->SetAutoFocusFired();
