@@ -6,7 +6,6 @@
 
 #include "mozilla/BasicEvents.h"
 #include "mozilla/EventDispatcher.h"
-#include "mozilla/EventStates.h"
 #include "mozilla/Maybe.h"
 #include "mozilla/dom/CustomElementRegistry.h"
 #include "mozilla/dom/HTMLFieldSetElement.h"
@@ -29,7 +28,7 @@ HTMLFieldSetElement::HTMLFieldSetElement(
   SetBarredFromConstraintValidation(true);
 
   // We start out enabled and valid.
-  AddStatesSilently(NS_EVENT_STATE_ENABLED | NS_EVENT_STATE_VALID);
+  AddStatesSilently(ElementState::ENABLED | ElementState::VALID);
 }
 
 HTMLFieldSetElement::~HTMLFieldSetElement() {
@@ -305,13 +304,13 @@ void HTMLFieldSetElement::UpdateValidity(bool aElementValidity) {
   }
 }
 
-EventStates HTMLFieldSetElement::IntrinsicState() const {
-  EventStates state = nsGenericHTMLFormControlElement::IntrinsicState();
+ElementState HTMLFieldSetElement::IntrinsicState() const {
+  ElementState state = nsGenericHTMLFormControlElement::IntrinsicState();
 
   if (mInvalidElementsCount) {
-    state |= NS_EVENT_STATE_INVALID;
+    state |= ElementState::INVALID;
   } else {
-    state |= NS_EVENT_STATE_VALID;
+    state |= ElementState::VALID;
   }
 
   return state;
