@@ -8,48 +8,49 @@
 #define mozilla_dom_HTMLFieldSetElement_h
 
 #include "mozilla/Attributes.h"
-#include "nsGenericHTMLElement.h"
-#include "nsIConstraintValidation.h"
-#include "mozilla/dom/HTMLFormElement.h"
+#include "mozilla/dom/ConstraintValidation.h"
 #include "mozilla/dom/ValidityState.h"
+#include "nsGenericHTMLElement.h"
 
 namespace mozilla {
+class ErrorResult;
 class EventChainPreVisitor;
 namespace dom {
+class FormData;
 
-class HTMLFieldSetElement final : public nsGenericHTMLFormElement,
-                                  public nsIConstraintValidation
-{
-public:
-  using nsGenericHTMLFormElement::GetForm;
-  using nsIConstraintValidation::GetValidationMessage;
-  using nsIConstraintValidation::SetCustomValidity;
+class HTMLFieldSetElement final : public nsGenericHTMLFormControlElement,
+                                  public ConstraintValidation {
+ public:
+  using ConstraintValidation::GetValidationMessage;
+  using ConstraintValidation::SetCustomValidity;
 
-  explicit HTMLFieldSetElement(already_AddRefed<mozilla::dom::NodeInfo>& aNodeInfo);
+  explicit HTMLFieldSetElement(
+      already_AddRefed<mozilla::dom::NodeInfo>&& aNodeInfo);
 
-  NS_IMPL_FROMCONTENT_HTML_WITH_TAG(HTMLFieldSetElement, fieldset)
+  NS_IMPL_FROMNODE_HTML_WITH_TAG(HTMLFieldSetElement, fieldset)
 
   // nsISupports
   NS_DECL_ISUPPORTS_INHERITED
 
   // nsIContent
-  virtual nsresult GetEventTargetParent(
-                     EventChainPreVisitor& aVisitor) override;
-  virtual nsresult AfterSetAttr(int32_t aNameSpaceID, nsIAtom* aName,
+  void GetEventTargetParent(EventChainPreVisitor& aVisitor) override;
+  virtual nsresult AfterSetAttr(int32_t aNameSpaceID, nsAtom* aName,
                                 const nsAttrValue* aValue,
                                 const nsAttrValue* aOldValue,
+                                nsIPrincipal* aSubjectPrincipal,
                                 bool aNotify) override;
 
-  virtual nsresult InsertChildAt(nsIContent* aChild, uint32_t aIndex,
-                                     bool aNotify) override;
-  virtual void RemoveChildAt(uint32_t aIndex, bool aNotify) override;
+  virtual void InsertChildBefore(nsIContent* aChild, nsIContent* aBeforeThis,
+                                 bool aNotify, ErrorResult& aRv) override;
+  virtual void RemoveChildNode(nsIContent* aKid, bool aNotify) override;
+
+  // nsGenericHTMLElement
+  virtual bool IsDisabledForEvents(WidgetEvent* aEvent) override;
 
   // nsIFormControl
   NS_IMETHOD Reset() override;
-  NS_IMETHOD SubmitNamesValues(HTMLFormSubmission* aFormSubmission) override;
-  virtual bool IsDisabledForEvents(EventMessage aMessage) override;
-  virtual nsresult Clone(mozilla::dom::NodeInfo *aNodeInfo, nsINode **aResult,
-                         bool aPreallocateChildren) const override;
+  NS_IMETHOD SubmitNamesValues(FormData* aFormData) override { return NS_OK; }
+  virtual nsresult Clone(dom::NodeInfo*, nsINode** aResult) const override;
 
   const nsIContent* GetFirstLegend() const { return mFirstLegend; }
 
@@ -58,29 +59,21 @@ public:
   void RemoveElement(nsGenericHTMLFormElement* aElement);
 
   NS_DECL_CYCLE_COLLECTION_CLASS_INHERITED(HTMLFieldSetElement,
-                                           nsGenericHTMLFormElement)
+                                           nsGenericHTMLFormControlElement)
 
   // WebIDL
-  bool Disabled() const
-  {
-    return GetBoolAttr(nsGkAtoms::disabled);
-  }
-  void SetDisabled(bool aValue, ErrorResult& aRv)
-  {
+  bool Disabled() const { return GetBoolAttr(nsGkAtoms::disabled); }
+  void SetDisabled(bool aValue, ErrorResult& aRv) {
     SetHTMLBoolAttr(nsGkAtoms::disabled, aValue, aRv);
   }
 
-  void GetName(nsAString& aValue)
-  {
-    GetHTMLAttr(nsGkAtoms::name, aValue);
-  }
+  void GetName(nsAString& aValue) { GetHTMLAttr(nsGkAtoms::name, aValue); }
 
-  void SetName(const nsAString& aValue, ErrorResult& aRv)
-  {
+  void SetName(const nsAString& aValue, ErrorResult& aRv) {
     SetHTMLAttr(nsGkAtoms::name, aValue, aRv);
   }
 
-  NS_IMETHOD GetType(nsAString & aType);
+  void GetType(nsAString& aType) const;
 
   nsIHTMLCollection* Elements();
 
@@ -94,13 +87,12 @@ public:
 
   // XPCOM SetCustomValidity is OK for us
 
-  virtual EventStates IntrinsicState() const override;
-
+  virtual ElementState IntrinsicState() const override;
 
   /*
    * This method will update the fieldset's validity.  This method has to be
-   * called by fieldset elements whenever their validity state or status regarding
-   * constraint validation changes.
+   * called by fieldset elements whenever their validity state or status
+   * regarding constraint validation changes.
    *
    * @note If an element becomes barred from constraint validation, it has to
    * be considered as valid.
@@ -109,13 +101,13 @@ public:
    */
   void UpdateValidity(bool aElementValidityState);
 
-protected:
+ protected:
   virtual ~HTMLFieldSetElement();
 
-  virtual JSObject* WrapNode(JSContext* aCx, JS::Handle<JSObject*> aGivenProto) override;
-
-private:
+  virtual JSObject* WrapNode(JSContext* aCx,
+                             JS::Handle<JSObject*> aGivenProto) override;
 
+ private:
   /**
    * Notify all elements (in mElements) that the first legend of the fieldset
    * has now changed.
@@ -124,7 +116,7 @@ private:
 
   // This function is used to generate the nsContentList (listed form elements).
   static bool MatchListedElements(Element* aElement, int32_t aNamespaceID,
-                                  nsIAtom* aAtom, void* aData);
+                                  nsAtom* aAtom, void* aData);
 
   // listed form controls elements.
   RefPtr<nsContentList> mElements;
@@ -143,7 +135,7 @@ private:
   int32_t mInvalidElementsCount;
 };
 
-} // namespace dom
-} // namespace mozilla
+}  // namespace dom
+}  // namespace mozilla
 
 #endif /* mozilla_dom_HTMLFieldSetElement_h */
