@@ -11,8 +11,9 @@
 #include "mozilla/Attributes.h"
 #include "mozilla/UniquePtr.h"
 #include "mozilla/dom/BrowsingContext.h"
+#include "mozilla/dom/PopupBlocker.h"
+#include "mozilla/dom/RadioGroupManager.h"
 #include "nsCOMPtr.h"
-#include "nsIForm.h"
 #include "nsIFormControl.h"
 #include "nsGenericHTMLElement.h"
 #include "nsIRadioGroupContainer.h"
@@ -20,7 +21,7 @@
 #include "nsThreadUtils.h"
 #include "nsInterfaceHashtable.h"
 #include "nsRefPtrHashtable.h"
-#include "nsDataHashtable.h"
+#include "nsTHashMap.h"
 #include "js/friend/DOMProxy.h"  // JS::ExpandoAndGeneration
 
 class nsIMutableArray;
@@ -30,13 +31,15 @@ namespace mozilla {
 class EventChainPostVisitor;
 class EventChainPreVisitor;
 namespace dom {
+class DialogFormSubmission;
 class HTMLFormControlsCollection;
+class HTMLFormSubmission;
 class HTMLImageElement;
 class FormData;
 
 class HTMLFormElement final : public nsGenericHTMLElement,
-                              public nsIForm,
-                              public nsIRadioGroupContainer {
+                              public nsIRadioGroupContainer,
+                              RadioGroupManager {
   friend class HTMLFormControlsCollection;
 
  public:
@@ -50,11 +53,8 @@ class HTMLFormElement final : public nsGenericHTMLElement,
   // nsISupports
   NS_DECL_ISUPPORTS_INHERITED
 
-  // nsIForm
-  NS_IMETHOD_(nsIFormControl*) GetElementAt(int32_t aIndex) const override;
-  NS_IMETHOD_(uint32_t) GetElementCount() const override;
-  NS_IMETHOD_(int32_t) IndexOfControl(nsIFormControl* aControl) override;
-  NS_IMETHOD_(nsIFormControl*) GetDefaultSubmitElement() const override;
+  int32_t IndexOfContent(nsIContent* aContent);
+  nsGenericHTMLFormElement* GetDefaultSubmitElement() const;
 
   // nsIRadioGroupContainer
   void SetCurrentRadioButton(const nsAString& aName,
@@ -63,8 +63,8 @@ class HTMLFormElement final : public nsGenericHTMLElement,
   NS_IMETHOD GetNextRadioButton(const nsAString& aName, const bool aPrevious,
                                 HTMLInputElement* aFocusedRadio,
                                 HTMLInputElement** aRadioOut) override;
-  NS_IMETHOD WalkRadioGroup(const nsAString& aName, nsIRadioVisitor* aVisitor,
-                            bool aFlushContent) override;
+  NS_IMETHOD WalkRadioGroup(const nsAString& aName,
+                            nsIRadioVisitor* aVisitor) override;
   void AddToRadioGroup(const nsAString& aName,
                        HTMLInputElement* aRadio) override;
   void RemoveFromRadioGroup(const nsAString& aName,
@@ -76,7 +76,7 @@ class HTMLFormElement final : public nsGenericHTMLElement,
   virtual void SetValueMissingState(const nsAString& aName,
                                     bool aValue) override;
 
-  virtual EventStates IntrinsicState() const override;
+  virtual ElementState IntrinsicState() const override;
 
   // EventTarget
   virtual void AsyncEventRunning(AsyncEventDispatcher* aEvent) override;
@@ -95,11 +95,6 @@ class HTMLFormElement final : public nsGenericHTMLElement,
   virtual nsresult BeforeSetAttr(int32_t aNamespaceID, nsAtom* aName,
                                  const nsAttrValueOrString* aValue,
                                  bool aNotify) override;
-  virtual nsresult AfterSetAttr(int32_t aNameSpaceID, nsAtom* aName,
-                                const nsAttrValue* aValue,
-                                const nsAttrValue* aOldValue,
-                                nsIPrincipal* aSubjectPrincipal,
-                                bool aNotify) override;
 
   /**
    * Forget all information about the current submission (and the fact that we
@@ -206,19 +201,19 @@ class HTMLFormElement final : public nsGenericHTMLElement,
   bool ImplicitSubmissionIsDisabled() const;
 
   /**
-   * Check whether a given nsIFormControl is the last single line input control
-   * that is not disabled. aControl is expected to not be null.
+   * Check whether a given nsGenericHTMLFormElement is the last single line
+   * input control that is not disabled. aElement is expected to not be null.
    */
-  bool IsLastActiveElement(const nsIFormControl* aControl) const;
+  bool IsLastActiveElement(const nsGenericHTMLFormElement* aElement) const;
 
   /**
-   * Check whether a given nsIFormControl is the default submit
+   * Check whether a given nsGenericHTMLFormElement is the default submit
    * element.  This is different from just comparing to
    * GetDefaultSubmitElement() in certain situations inside an update
-   * when GetDefaultSubmitElement() might not be up to date.  aControl
+   * when GetDefaultSubmitElement() might not be up to date. aElement
    * is expected to not be null.
    */
-  bool IsDefaultSubmitElement(const nsIFormControl* aControl) const;
+  bool IsDefaultSubmitElement(const nsGenericHTMLFormElement* aElement) const;
 
   /**
    * Flag the form to know that a button or image triggered scripted form
@@ -229,8 +224,8 @@ class HTMLFormElement final : public nsGenericHTMLElement,
   void OnSubmitClickEnd();
 
   /**
-   * This method will update the form validity so the submit controls states
-   * will be updated (for -moz-submit-invalid pseudo-class).
+   * This method will update the form validity.
+   *
    * This method has to be called by form elements whenever their validity state
    * or status regarding constraint validation changes.
    *
@@ -242,16 +237,6 @@ class HTMLFormElement final : public nsGenericHTMLElement,
    */
   void UpdateValidity(bool aElementValidityState);
 
-  /**
-   * Returns the form validity based on the last UpdateValidity() call.
-   *
-   * @return Whether the form was valid the last time UpdateValidity() was
-   * called.
-   *
-   * @note This method may not return the *current* validity state!
-   */
-  bool GetValidity() const { return !mInvalidElementsCount; }
-
   /**
    * This method check the form validity and make invalid form elements send
    * invalid event if needed.
@@ -272,7 +257,8 @@ class HTMLFormElement final : public nsGenericHTMLElement,
    *
    * @param aFormData the form data object
    */
-  nsresult ConstructEntryList(FormData* aFormData);
+  // TODO: Convert this to MOZ_CAN_RUN_SCRIPT (bug 1415230)
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY nsresult ConstructEntryList(FormData* aFormData);
 
   /**
    * Whether the submission of this form has been ever prevented because of
@@ -374,7 +360,7 @@ class HTMLFormElement final : public nsGenericHTMLElement,
   MOZ_CAN_RUN_SCRIPT void RequestSubmit(nsGenericHTMLElement* aSubmitter,
                                         ErrorResult& aRv);
 
-  void Reset();
+  MOZ_CAN_RUN_SCRIPT void Reset();
 
   bool CheckValidity() { return CheckFormValidity(nullptr); }
 
@@ -405,8 +391,10 @@ class HTMLFormElement final : public nsGenericHTMLElement,
                              JS::Handle<JSObject*> aGivenProto) override;
 
   void PostPasswordEvent();
+  void PostPossibleUsernameEvent();
 
   RefPtr<AsyncEventDispatcher> mFormPasswordEventDispatcher;
+  RefPtr<AsyncEventDispatcher> mFormPossibleUsernameEventDispatcher;
 
   class RemoveElementRunnable;
   friend class RemoveElementRunnable;
@@ -487,8 +475,7 @@ class HTMLFormElement final : public nsGenericHTMLElement,
    * Find form controls in this form with the correct value in the name
    * attribute.
    */
-  already_AddRefed<nsISupports> DoResolveName(const nsAString& aName,
-                                              bool aFlushContent);
+  already_AddRefed<nsISupports> DoResolveName(const nsAString& aName);
 
   /**
    * Check the form validity following this algorithm:
@@ -554,21 +541,9 @@ class HTMLFormElement final : public nsGenericHTMLElement,
   //
   /** The list of controls (form.elements as well as stuff not in elements) */
   RefPtr<HTMLFormControlsCollection> mControls;
-  /** The currently selected radio button of each group */
-  nsRefPtrHashtable<nsStringHashKey, HTMLInputElement> mSelectedRadioButtons;
-  /** The number of required radio button of each group */
-  nsDataHashtable<nsStringCaseInsensitiveHashKey, uint32_t>
-      mRequiredRadioButtonCounts;
-  /** The value missing state of each group */
-  nsDataHashtable<nsStringCaseInsensitiveHashKey, bool>
-      mValueMissingRadioGroups;
 
   /** The pending submission object */
   UniquePtr<HTMLFormSubmission> mPendingSubmission;
-  /** The request currently being submitted */
-  nsCOMPtr<nsIRequest> mSubmittingRequest;
-  /** The web progress object we are currently listening to */
-  nsWeakPtr mWebProgress;
 
   /** The target browsing context, if any. */
   RefPtr<BrowsingContext> mTargetContext;
@@ -610,7 +585,6 @@ class HTMLFormElement final : public nsGenericHTMLElement,
   /**
    * Number of invalid and candidate for constraint validation elements in the
    * form the last time UpdateValidity has been called.
-   * @note Should only be used by UpdateValidity() and GetValidity()!
    */
   int32_t mInvalidElementsCount;
 
@@ -641,6 +615,13 @@ class HTMLFormElement final : public nsGenericHTMLElement,
   bool IsSubmitting() const;
 
   NotNull<const Encoding*> GetSubmitEncoding();
+
+  /**
+   * Fire an event when the form is removed from the DOM tree. This is now only
+   * used by the password manager.
+   */
+  void MaybeFireFormRemoved();
+
   ~HTMLFormElement();
 };
 
