@@ -5,6 +5,8 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "mozilla/dom/HTMLIFrameElement.h"
+#include "mozilla/dom/ContentChild.h"
+#include "mozilla/dom/Document.h"
 #include "mozilla/dom/HTMLIFrameElementBinding.h"
 #include "mozilla/dom/FeaturePolicy.h"
 #include "mozilla/MappedDeclarations.h"
@@ -112,9 +114,9 @@ void HTMLIFrameElement::MapAttributesIntoRule(
   // else leave it as the value set in html.css
   const nsAttrValue* value = aAttributes->GetAttr(nsGkAtoms::frameborder);
   if (value && value->Type() == nsAttrValue::eEnum) {
-    int32_t frameborder = value->GetEnumValue();
-    if (NS_STYLE_FRAME_0 == frameborder || NS_STYLE_FRAME_NO == frameborder ||
-        NS_STYLE_FRAME_OFF == frameborder) {
+    auto frameborder = static_cast<FrameBorderProperty>(value->GetEnumValue());
+    if (FrameBorderProperty::No == frameborder ||
+        FrameBorderProperty::Zero == frameborder) {
       aDecls.SetPixelValueIfUnset(eCSSProperty_border_top_width, 0.0f);
       aDecls.SetPixelValueIfUnset(eCSSProperty_border_right_width, 0.0f);
       aDecls.SetPixelValueIfUnset(eCSSProperty_border_bottom_width, 0.0f);
@@ -233,24 +235,10 @@ void HTMLIFrameElement::MaybeStoreCrossOriginFeaturePolicy() {
     return;
   }
 
-  // If we are in subframe cross origin, store the featurePolicy to
-  // browsingContext
-  nsPIDOMWindowOuter* topWindow = browsingContext->Top()->GetDOMWindow();
-  if (NS_WARN_IF(!topWindow)) {
-    return;
+  if (ContentChild* cc = ContentChild::GetSingleton()) {
+    Unused << cc->SendSetContainerFeaturePolicy(browsingContext,
+                                                mFeaturePolicy);
   }
-
-  Document* topLevelDocument = topWindow->GetExtantDoc();
-  if (NS_WARN_IF(!topLevelDocument)) {
-    return;
-  }
-
-  if (!NS_SUCCEEDED(nsContentUtils::CheckSameOrigin(topLevelDocument, this))) {
-    return;
-  }
-
-  // Return value of setting synced field should be checked. See bug 1656492.
-  Unused << browsingContext->SetFeaturePolicy(mFeaturePolicy);
 }
 
 already_AddRefed<nsIPrincipal>
