@@ -10,7 +10,6 @@
 #include "mozilla/Attributes.h"
 #include "mozilla/Components.h"
 #include "mozilla/EventDispatcher.h"
-#include "mozilla/EventStates.h"
 #include "mozilla/MemoryReporting.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/StaticPrefs_dom.h"
@@ -19,6 +18,7 @@
 #include "mozilla/dom/DocumentInlines.h"
 #include "mozilla/dom/HTMLLinkElementBinding.h"
 #include "mozilla/dom/HTMLDNSPrefetch.h"
+#include "mozilla/dom/ScriptLoader.h"
 #include "nsContentUtils.h"
 #include "nsDOMTokenList.h"
 #include "nsGenericHTMLElement.h"
@@ -177,14 +177,15 @@ void HTMLLinkElement::CreateAndDispatchEvent(Document* aDoc,
   // this should never actually happen and the performance hit is minimal,
   // doing the "right" thing costs virtually nothing here, even if it doesn't
   // make much sense.
-  static Element::AttrValuesArray strings[] = {nsGkAtoms::_empty,
-                                               nsGkAtoms::stylesheet, nullptr};
+  static AttrArray::AttrValuesArray strings[] = {
+      nsGkAtoms::_empty, nsGkAtoms::stylesheet, nullptr};
 
   if (!nsContentUtils::HasNonEmptyAttr(this, kNameSpaceID_None,
                                        nsGkAtoms::rev) &&
       FindAttrValueIn(kNameSpaceID_None, nsGkAtoms::rel, strings,
-                      eIgnoreCase) != ATTR_VALUE_NO_MATCH)
+                      eIgnoreCase) != AttrArray::ATTR_VALUE_NO_MATCH) {
     return;
+  }
 
   RefPtr<AsyncEventDispatcher> asyncDispatcher = new AsyncEventDispatcher(
       this, aEventName, CanBubble::eYes, ChromeOnlyDispatch::eYes);
@@ -314,8 +315,9 @@ static const DOMTokenListSupportedToken sSupportedRelValues[] = {
     // Keep this and the one below in sync with ToLinkMask in
     // LinkStyle.cpp.
     // "preload" must come first because it can be disabled.
-    "preload",   "prefetch",   "dns-prefetch", "stylesheet", "next",
-    "alternate", "preconnect", "icon",         "search",     nullptr};
+    "preload", "prefetch",      "dns-prefetch", "stylesheet",
+    "next",    "alternate",     "preconnect",   "icon",
+    "search",  "modulepreload", nullptr};
 
 static const DOMTokenListSupportedToken sSupportedRelValuesWithManifest[] = {
     // Keep this in sync with ToLinkMask in LinkStyle.cpp.
@@ -487,6 +489,23 @@ void HTMLLinkElement::
     }
   }
 
+  if (linkTypes & eMODULE_PRELOAD) {
+    if (!OwnerDoc()->ScriptLoader()->GetModuleLoader()) {
+      // For the print preview documents, at this moment it doesn't have module
+      // loader yet, as the (print preview) document is not attached to the
+      // nsIContentViewer yet, so it doesn't have the GlobalObject.
+      // Also, the script elements won't be processed as they are also cloned
+      // from the original document.
+      // So we simply bail out if the module loader is null.
+      return;
+    }
+
+    // https://html.spec.whatwg.org/multipage/webappapis.html#fetch-a-modulepreload-module-script-graph
+    // Step 1. Disallow further import maps given settings object.
+    OwnerDoc()->ScriptLoader()->GetModuleLoader()->DisallowImportMaps();
+    return;
+  }
+
   if (linkTypes & ePRECONNECT) {
     if (nsCOMPtr<nsIURI> uri = GetURI()) {
       OwnerDoc()->MaybePreconnect(
