@@ -10,8 +10,7 @@
 #include "mozilla/Attributes.h"
 #include "nsGenericHTMLElement.h"
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 class HTMLMetaElement final : public nsGenericHTMLElement {
  public:
@@ -30,7 +29,7 @@ class HTMLMetaElement final : public nsGenericHTMLElement {
                                 nsIPrincipal* aSubjectPrincipal,
                                 bool aNotify) override;
 
-  void CreateAndDispatchEvent(Document* aDoc, const nsAString& aEventName);
+  void CreateAndDispatchEvent(Document&, const nsAString& aEventName);
 
   virtual nsresult Clone(dom::NodeInfo*, nsINode** aResult) const override;
 
@@ -54,20 +53,24 @@ class HTMLMetaElement final : public nsGenericHTMLElement {
   void SetScheme(const nsAString& aScheme, ErrorResult& aRv) {
     SetHTMLAttr(nsGkAtoms::scheme, aScheme, aRv);
   }
+  void GetMedia(nsAString& aValue) { GetHTMLAttr(nsGkAtoms::media, aValue); }
+  void SetMedia(const nsAString& aMedia, ErrorResult& aRv) {
+    SetHTMLAttr(nsGkAtoms::media, aMedia, aRv);
+  }
 
-  virtual JSObject* WrapNode(JSContext* aCx,
-                             JS::Handle<JSObject*> aGivenProto) override;
+  JSObject* WrapNode(JSContext*, JS::Handle<JSObject*> aGivenProto) override;
 
  protected:
   virtual ~HTMLMetaElement();
 
  private:
-  void SetMetaReferrer(Document* aDocument);
-  void ProcessViewportContent(Document* aDocument);
-  void DiscardViewportContent(Document* aDocument);
+  enum class ChangeKind : uint8_t { TreeChange, NameChange, ContentChange };
+  void MetaRemoved(Document& aDoc, const nsAttrValue& aName,
+                   ChangeKind aChangeKind);
+  void MetaAddedOrChanged(Document& aDoc, const nsAttrValue& aName,
+                          ChangeKind aChangeKind);
 };
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
 
 #endif  // mozilla_dom_HTMLMetaElement_h
