@@ -10,7 +10,6 @@
 #include "mozilla/Attributes.h"
 #include "mozilla/BasicEvents.h"
 #include "mozilla/EventDispatcher.h"
-#include "mozilla/EventStates.h"
 #include "mozilla/dom/Element.h"
 #include "mozilla/dom/FormData.h"
 #include "mozilla/dom/HTMLOptGroupElement.h"
@@ -21,6 +20,7 @@
 #include "mozilla/Maybe.h"
 #include "nsContentCreatorFunctions.h"
 #include "nsContentList.h"
+#include "nsContentUtils.h"
 #include "nsError.h"
 #include "nsGkAtoms.h"
 #include "nsComboboxControlFrame.h"
@@ -132,8 +132,8 @@ HTMLSelectElement::HTMLSelectElement(
   // otherwise it is
 
   // Set up our default state: enabled, optional, and valid.
-  AddStatesSilently(NS_EVENT_STATE_ENABLED | NS_EVENT_STATE_OPTIONAL |
-                    NS_EVENT_STATE_VALID);
+  AddStatesSilently(ElementState::ENABLED | ElementState::OPTIONAL_ |
+                    ElementState::VALID);
 }
 
 // ISupports
@@ -267,7 +267,7 @@ void HTMLSelectElement::InsertOptionsIntoList(nsIContent* aOptions,
         // This is sort of a hack ... we need to notify that the option was
         // set and change selectedIndex even though we didn't really change
         // its value.
-        OnOptionSelected(selectFrame, i, true, false, false);
+        OnOptionSelected(selectFrame, i, true, false, aNotify);
       }
     }
 
@@ -545,7 +545,7 @@ void HTMLSelectElement::Add(nsGenericHTMLElement& aElement,
   parent->InsertBefore(aElement, refNode, aError);
 }
 
-void HTMLSelectElement::Remove(int32_t aIndex) {
+void HTMLSelectElement::Remove(int32_t aIndex) const {
   if (aIndex < 0) {
     return;
   }
@@ -566,9 +566,9 @@ void HTMLSelectElement::GetType(nsAString& aType) {
   }
 }
 
-#define MAX_DYNAMIC_SELECT_LENGTH 10000
-
 void HTMLSelectElement::SetLength(uint32_t aLength, ErrorResult& aRv) {
+  constexpr uint32_t kMaxDynamicSelectLength = 100000;
+
   uint32_t curlen = Length();
 
   if (curlen > aLength) {  // Remove extra options
@@ -576,8 +576,17 @@ void HTMLSelectElement::SetLength(uint32_t aLength, ErrorResult& aRv) {
       Remove(i - 1);
     }
   } else if (aLength > curlen) {
-    if (aLength > MAX_DYNAMIC_SELECT_LENGTH) {
-      aRv.Throw(NS_ERROR_DOM_NOT_SUPPORTED_ERR);
+    if (aLength > kMaxDynamicSelectLength) {
+      nsAutoString strOptionsLength;
+      strOptionsLength.AppendInt(aLength);
+
+      nsAutoString strLimit;
+      strLimit.AppendInt(kMaxDynamicSelectLength);
+
+      nsContentUtils::ReportToConsole(
+          nsIScriptError::warningFlag, "DOM"_ns, OwnerDoc(),
+          nsContentUtils::eDOM_PROPERTIES,
+          "SelectOptionsLengthAssignmentWarning", {strOptionsLength, strLimit});
       return;
     }
 
@@ -638,7 +647,7 @@ void HTMLSelectElement::SetSelectedIndexInternal(int32_t aIndex, bool aNotify) {
   SetSelectionChanged(true, aNotify);
 }
 
-bool HTMLSelectElement::IsOptionSelectedByIndex(int32_t aIndex) {
+bool HTMLSelectElement::IsOptionSelectedByIndex(int32_t aIndex) const {
   HTMLOptionElement* option = Item(static_cast<uint32_t>(aIndex));
   return option && option->Selected();
 }
@@ -920,7 +929,7 @@ bool HTMLSelectElement::IsOptionDisabled(HTMLOptionElement* aOption) const {
   return false;
 }
 
-void HTMLSelectElement::GetValue(DOMString& aValue) {
+void HTMLSelectElement::GetValue(DOMString& aValue) const {
   int32_t selectedIndex = SelectedIndex();
   if (selectedIndex < 0) {
     return;
@@ -1208,8 +1217,8 @@ nsresult HTMLSelectElement::PostHandleEvent(EventChainPostVisitor& aVisitor) {
     // UI while focused.
     mCanShowValidUI = ShouldShowValidityUI();
 
-    // We don't have to update NS_EVENT_STATE_MOZ_UI_INVALID nor
-    // NS_EVENT_STATE_MOZ_UI_VALID given that the states should not change.
+    // We don't have to update ElementState::USER_INVALID nor
+    // ElementState::USER_VALID given that the states should not change.
   } else if (aVisitor.mEvent->mMessage == eBlur) {
     mCanShowInvalidUI = true;
     mCanShowValidUI = true;
@@ -1220,19 +1229,19 @@ nsresult HTMLSelectElement::PostHandleEvent(EventChainPostVisitor& aVisitor) {
   return nsGenericHTMLFormControlElementWithState::PostHandleEvent(aVisitor);
 }
 
-EventStates HTMLSelectElement::IntrinsicState() const {
-  EventStates state =
+ElementState HTMLSelectElement::IntrinsicState() const {
+  ElementState state =
       nsGenericHTMLFormControlElementWithState::IntrinsicState();
 
   if (IsCandidateForConstraintValidation()) {
     if (IsValid()) {
-      state |= NS_EVENT_STATE_VALID;
+      state |= ElementState::VALID;
     } else {
-      state |= NS_EVENT_STATE_INVALID;
+      state |= ElementState::INVALID;
 
       if (GetValidityState(VALIDITY_STATE_CUSTOM_ERROR) ||
           (mCanShowInvalidUI && ShouldShowValidityUI())) {
-        state |= NS_EVENT_STATE_MOZ_UI_INVALID;
+        state |= ElementState::USER_INVALID;
       }
     }
 
@@ -1244,9 +1253,9 @@ EventStates HTMLSelectElement::IntrinsicState() const {
     // 3. The element has already been modified or the user tried to submit the
     //    form owner while invalid.
     if (mCanShowValidUI && ShouldShowValidityUI() &&
-        (IsValid() || (state.HasState(NS_EVENT_STATE_MOZ_UI_INVALID) &&
-                       !mCanShowInvalidUI))) {
-      state |= NS_EVENT_STATE_MOZ_UI_VALID;
+        (IsValid() ||
+         (state.HasState(ElementState::USER_INVALID) && !mCanShowInvalidUI))) {
+      state |= ElementState::USER_VALID;
     }
   }
 
