@@ -115,10 +115,6 @@ class HTMLSelectElement final : public nsGenericHTMLFormControlElementWithState,
   virtual bool IsInteractiveHTMLContent() const override { return true; }
 
   // WebIdl HTMLSelectElement
-  bool Autofocus() const { return GetBoolAttr(nsGkAtoms::autofocus); }
-  void SetAutofocus(bool aVal, ErrorResult& aRv) {
-    SetHTMLBoolAttr(nsGkAtoms::autofocus, aVal, aRv);
-  }
   void GetAutocomplete(DOMString& aValue);
   void SetAutocomplete(const nsAString& aValue, ErrorResult& aRv) {
     SetHTMLAttr(nsGkAtoms::autocomplete, aValue, aRv);
@@ -139,7 +135,7 @@ class HTMLSelectElement final : public nsGenericHTMLFormControlElementWithState,
   void SetName(const nsAString& aName, ErrorResult& aRv) {
     SetHTMLAttr(nsGkAtoms::name, aName, aRv);
   }
-  bool Required() const { return State().HasState(NS_EVENT_STATE_REQUIRED); }
+  bool Required() const { return State().HasState(ElementState::REQUIRED); }
   void SetRequired(bool aVal, ErrorResult& aRv) {
     SetHTMLBoolAttr(nsGkAtoms::required, aVal, aRv);
   }
@@ -164,7 +160,7 @@ class HTMLSelectElement final : public nsGenericHTMLFormControlElementWithState,
   }
   void Add(const HTMLOptionElementOrHTMLOptGroupElement& aElement,
            const Nullable<HTMLElementOrLong>& aBefore, ErrorResult& aRv);
-  void Remove(int32_t aIndex);
+  void Remove(int32_t aIndex) const;
   void IndexedSetter(uint32_t aIndex, HTMLOptionElement* aOption,
                      ErrorResult& aRv) {
     mOptions->IndexedSetter(aIndex, aOption, aRv);
@@ -176,7 +172,7 @@ class HTMLSelectElement final : public nsGenericHTMLFormControlElementWithState,
 
   int32_t SelectedIndex() const { return mSelectedIndex; }
   void SetSelectedIndex(int32_t aIdx) { SetSelectedIndexInternal(aIdx, true); }
-  void GetValue(DOMString& aValue);
+  void GetValue(DOMString& aValue) const;
   void SetValue(const nsAString& aValue);
 
   // Override SetCustomValidity so we update our state properly when it's called
@@ -213,7 +209,7 @@ class HTMLSelectElement final : public nsGenericHTMLFormControlElementWithState,
 
   virtual void FieldSetDisabledChanged(bool aNotify) override;
 
-  EventStates IntrinsicState() const override;
+  ElementState IntrinsicState() const override;
 
   /**
    * To be called when stuff is added under a child of the select--but *before*
@@ -339,7 +335,7 @@ class HTMLSelectElement final : public nsGenericHTMLFormControlElementWithState,
    * @param aIndex the index
    * @return whether the option at the index is selected
    */
-  bool IsOptionSelectedByIndex(int32_t aIndex);
+  bool IsOptionSelectedByIndex(int32_t aIndex) const;
   /**
    * Starting with (and including) aStartIndex, find the first selected index
    * and set mSelectedIndex to it.
