@@ -7,7 +7,9 @@
 #include "mozilla/dom/TextTrackManager.h"
 #include "mozilla/ClearOnShutdown.h"
 #include "mozilla/CycleCollectedJSContext.h"
+#include "mozilla/Maybe.h"
 #include "mozilla/Telemetry.h"
+#include "mozilla/dom/Document.h"
 #include "mozilla/dom/Event.h"
 #include "mozilla/dom/HTMLMediaElement.h"
 #include "mozilla/dom/HTMLTrackElement.h"
@@ -43,11 +45,12 @@ CompareTextTracks::CompareTextTracks(HTMLMediaElement* aMediaElement) {
   mMediaElement = aMediaElement;
 }
 
-int32_t CompareTextTracks::TrackChildPosition(TextTrack* aTextTrack) const {
+Maybe<uint32_t> CompareTextTracks::TrackChildPosition(
+    TextTrack* aTextTrack) const {
   MOZ_DIAGNOSTIC_ASSERT(aTextTrack);
   HTMLTrackElement* trackElement = aTextTrack->GetTrackElement();
   if (!trackElement) {
-    return -1;
+    return Nothing();
   }
   return mMediaElement->ComputeIndexOf(trackElement);
 }
@@ -78,12 +81,13 @@ bool CompareTextTracks::LessThan(TextTrack* aOne, TextTrack* aTwo) const {
   }
   switch (sourceOne) {
     case TextTrackSource::Track: {
-      int32_t positionOne = TrackChildPosition(aOne);
-      int32_t positionTwo = TrackChildPosition(aTwo);
-      // If either position one or positiontwo are -1 then something has gone
-      // wrong. In this case we should just put them at the back of the list.
-      return positionOne != -1 && positionTwo != -1 &&
-             positionOne < positionTwo;
+      Maybe<uint32_t> positionOne = TrackChildPosition(aOne);
+      Maybe<uint32_t> positionTwo = TrackChildPosition(aTwo);
+      // If either position one or positiontwo are Nothing then something has
+      // gone wrong. In this case we should just put them at the back of the
+      // list.
+      return positionOne.isSome() && positionTwo.isSome() &&
+             *positionOne < *positionTwo;
     }
     case TextTrackSource::AddTextTrack:
       // For AddTextTrack sources the tracks will already be in the correct
@@ -159,7 +163,6 @@ already_AddRefed<TextTrack> TextTrackManager::AddTextTrack(
              NS_ConvertUTF16toUTF8(aLabel).get(),
              NS_ConvertUTF16toUTF8(aLanguage).get());
   AddCues(track);
-  ReportTelemetryForTrack(track);
 
   if (aTextTrackSource == TextTrackSource::Track) {
     RefPtr<nsIRunnable> task = NewRunnableMethod(
@@ -178,7 +181,6 @@ void TextTrackManager::AddTextTrack(TextTrack* aTextTrack) {
   WEBVTT_LOG("AddTextTrack TextTrack %p", aTextTrack);
   mTextTracks->AddTextTrack(aTextTrack, CompareTextTracks(mMediaElement));
   AddCues(aTextTrack);
-  ReportTelemetryForTrack(aTextTrack);
 
   if (aTextTrack->GetTextTrackSource() == TextTrackSource::Track) {
     RefPtr<nsIRunnable> task = NewRunnableMethod(
@@ -468,14 +470,14 @@ class SimpleTextTrackEvent : public Runnable {
 
 class CompareSimpleTextTrackEvents {
  private:
-  int32_t TrackChildPosition(SimpleTextTrackEvent* aEvent) const {
+  Maybe<uint32_t> TrackChildPosition(SimpleTextTrackEvent* aEvent) const {
     if (aEvent->mTrack) {
       HTMLTrackElement* trackElement = aEvent->mTrack->GetTrackElement();
       if (trackElement) {
         return mMediaElement->ComputeIndexOf(trackElement);
       }
     }
-    return -1;
+    return Nothing();
   }
   HTMLMediaElement* mMediaElement;
 
@@ -844,15 +846,6 @@ void TextTrackManager::NotifyReset() {
   UpdateCueDisplay();
 }
 
-void TextTrackManager::ReportTelemetryForTrack(TextTrack* aTextTrack) const {
-  MOZ_ASSERT(NS_IsMainThread());
-  MOZ_ASSERT(aTextTrack);
-  MOZ_ASSERT(mTextTracks->Length() > 0);
-
-  TextTrackKind kind = aTextTrack->Kind();
-  Telemetry::Accumulate(Telemetry::WEBVTT_TRACK_KINDS, uint32_t(kind));
-}
-
 bool TextTrackManager::IsLoaded() {
   return mTextTracks ? mTextTracks->AreTextTracksLoaded() : true;
 }
