@@ -483,7 +483,7 @@ RefPtr<IDBTransaction> IDBDatabase::Transaction(
 
   if ((aMode == IDBTransactionMode::Readwriteflush ||
        aMode == IDBTransactionMode::Cleanup) &&
-      !IndexedDatabaseManager::ExperimentalFeaturesEnabled()) {
+      !StaticPrefs::dom_indexedDB_experimental()) {
     // Pretend that this mode doesn't exist. We don't have a way to annotate
     // certain enum values as depending on preferences so we just duplicate the
     // normal exception generation here.
@@ -632,13 +632,6 @@ RefPtr<IDBTransaction> IDBDatabase::Transaction(
   return AsRefPtr(std::move(transaction));
 }
 
-StorageType IDBDatabase::Storage() const {
-  AssertIsOnOwningThread();
-  MOZ_ASSERT(mSpec);
-
-  return PersistenceTypeToStorageType(mSpec->metadata().persistenceType());
-}
-
 RefPtr<IDBRequest> IDBDatabase::CreateMutableFile(
     JSContext* aCx, const nsAString& aName, const Optional<nsAString>& aType,
     ErrorResult& aRv) {
@@ -790,12 +783,8 @@ PBackgroundIDBDatabaseFileChild* IDBDatabase::GetOrCreateFileActorForBlob(
     BlobImpl* blobImpl = aBlob.Impl();
     MOZ_ASSERT(blobImpl);
 
-    PBackgroundChild* backgroundManager =
-        mBackgroundActor->Manager()->Manager();
-    MOZ_ASSERT(backgroundManager);
-
     IPCBlob ipcBlob;
-    nsresult rv = IPCBlobUtils::Serialize(blobImpl, backgroundManager, ipcBlob);
+    nsresult rv = IPCBlobUtils::Serialize(blobImpl, ipcBlob);
     if (NS_WARN_IF(NS_FAILED(rv))) {
       return nullptr;
     }
@@ -1036,16 +1025,6 @@ void IDBDatabase::LastRelease() {
   }
 }
 
-nsresult IDBDatabase::PostHandleEvent(EventChainPostVisitor& aVisitor) {
-  nsresult rv =
-      IndexedDatabaseManager::CommonPostHandleEvent(aVisitor, *mFactory);
-  if (NS_WARN_IF(NS_FAILED(rv))) {
-    return rv;
-  }
-
-  return NS_OK;
-}
-
 JSObject* IDBDatabase::WrapObject(JSContext* aCx,
                                   JS::Handle<JSObject*> aGivenProto) {
   return IDBDatabase_Binding::Wrap(aCx, this, aGivenProto);
