@@ -65,8 +65,6 @@ class IndexedDatabaseManager final {
   }
 #endif
 
-  static bool InTestingMode();
-
   static bool FullSynchronous();
 
   static LoggingMode GetLoggingMode()
@@ -87,18 +85,10 @@ class IndexedDatabaseManager final {
   }
 #endif
 
-  static bool ExperimentalFeaturesEnabled();
-
-  static bool ExperimentalFeaturesEnabled(JSContext* aCx, JSObject* aGlobal);
-
-  static bool IsFileHandleEnabled();
-
   static uint32_t DataThreshold();
 
   static uint32_t MaxSerializedMsgSize();
 
-  static bool PreprocessingEnabled();
-
   // The maximum number of extra entries to preload in an Cursor::OpenOp or
   // Cursor::ContinueOp.
   static int32_t MaxPreloadExtraRecords();
@@ -133,10 +123,6 @@ class IndexedDatabaseManager final {
 
   static const nsCString& GetLocale();
 
-  // TODO: Convert this to MOZ_CAN_RUN_SCRIPT (bug 1415230)
-  MOZ_CAN_RUN_SCRIPT_BOUNDARY static nsresult CommonPostHandleEvent(
-      EventChainPostVisitor& aVisitor, const IDBFactory& aFactory);
-
   static bool ResolveSandboxBinding(JSContext* aCx);
 
   static bool DefineIndexedDB(JSContext* aCx, JS::Handle<JSObject*> aGlobal);
