@@ -5,10 +5,6 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifdef ACCESSIBILITY
-#  ifdef XP_WIN
-#    include "mozilla/a11y/ProxyAccessible.h"
-#    include "mozilla/a11y/ProxyWrappers.h"
-#  endif
 #  include "mozilla/a11y/DocAccessible.h"
 #  include "mozilla/a11y/DocManager.h"
 #  include "mozilla/a11y/OuterDocAccessible.h"
@@ -17,28 +13,29 @@
 #include "mozilla/dom/BrowserBridgeHost.h"
 #include "mozilla/dom/BrowsingContext.h"
 #include "mozilla/dom/MozFrameLoaderOwnerBinding.h"
+#include "mozilla/PresShell.h"
 #include "nsFocusManager.h"
 #include "nsFrameLoader.h"
 #include "nsFrameLoaderOwner.h"
+#include "nsObjectLoadingContent.h"
 #include "nsQueryObject.h"
 #include "nsSubDocumentFrame.h"
 #include "nsView.h"
 
 using namespace mozilla::ipc;
 
+mozilla::LazyLogModule gBrowserChildFocusLog("BrowserChildFocus");
+
+#define LOGBROWSERCHILDFOCUS(args) \
+  MOZ_LOG(gBrowserChildFocusLog, mozilla::LogLevel::Debug, args)
+
 namespace mozilla::dom {
 
 BrowserBridgeChild::BrowserBridgeChild(BrowsingContext* aBrowsingContext,
                                        TabId aId, const LayersId& aLayersId)
     : mId{aId}, mLayersId{aLayersId}, mBrowsingContext(aBrowsingContext) {}
 
-BrowserBridgeChild::~BrowserBridgeChild() {
-#if defined(ACCESSIBILITY) && defined(XP_WIN)
-  if (mEmbeddedDocAccessible) {
-    mEmbeddedDocAccessible->Shutdown();
-  }
-#endif
-}
+BrowserBridgeChild::~BrowserBridgeChild() {}
 
 already_AddRefed<BrowserBridgeHost> BrowserBridgeChild::FinishInit(
     nsFrameLoader* aFrameLoader) {
@@ -46,15 +43,12 @@ already_AddRefed<BrowserBridgeHost> BrowserBridgeChild::FinishInit(
   mFrameLoader = aFrameLoader;
 
   RefPtr<Element> owner = mFrameLoader->GetOwnerContent();
-  nsCOMPtr<nsIDocShell> docShell = do_GetInterface(owner->GetOwnerGlobal());
-  MOZ_DIAGNOSTIC_ASSERT(docShell);
-
-  nsDocShell::Cast(docShell)->OOPChildLoadStarted(this);
+  Document* doc = owner->OwnerDoc();
+  doc->OOPChildLoadStarted(this);
 
 #if defined(ACCESSIBILITY)
-  if (a11y::DocAccessible* docAcc =
-          a11y::GetExistingDocAccessible(owner->OwnerDoc())) {
-    if (a11y::Accessible* ownerAcc = docAcc->GetAccessible(owner)) {
+  if (a11y::DocAccessible* docAcc = a11y::GetExistingDocAccessible(doc)) {
+    if (a11y::LocalAccessible* ownerAcc = docAcc->GetAccessible(owner)) {
       if (a11y::OuterDocAccessible* outerAcc = ownerAcc->AsOuterDoc()) {
         outerAcc->SendEmbedderAccessible(this);
       }
@@ -74,10 +68,14 @@ void BrowserBridgeChild::NavigateByKey(bool aForward,
   Unused << SendNavigateByKey(aForward, aForDocumentNavigation);
 }
 
-void BrowserBridgeChild::Activate() { Unused << SendActivate(); }
+void BrowserBridgeChild::Activate(uint64_t aActionId) {
+  LOGBROWSERCHILDFOCUS(
+      ("BrowserBridgeChild::Activate actionid: %" PRIu64, aActionId));
+  Unused << SendActivate(aActionId);
+}
 
-void BrowserBridgeChild::Deactivate(bool aWindowLowering) {
-  Unused << SendDeactivate(aWindowLowering);
+void BrowserBridgeChild::Deactivate(bool aWindowLowering, uint64_t aActionId) {
+  Unused << SendDeactivate(aWindowLowering, aActionId);
 }
 
 void BrowserBridgeChild::SetIsUnderHiddenEmbedderElement(
@@ -147,12 +145,7 @@ BrowserBridgeChild::RecvSetEmbeddedDocAccessibleCOMProxy(
     const a11y::IDispatchHolder& aCOMProxy) {
 #if defined(ACCESSIBILITY) && defined(XP_WIN)
   MOZ_ASSERT(!aCOMProxy.IsNull());
-  if (mEmbeddedDocAccessible) {
-    mEmbeddedDocAccessible->Shutdown();
-  }
-  RefPtr<IDispatch> comProxy(aCOMProxy.Get());
-  mEmbeddedDocAccessible =
-      new a11y::RemoteIframeDocProxyAccessibleWrap(comProxy);
+  mEmbeddedDocAccessible = aCOMProxy.Get();
 #endif
   return IPC_OK();
 }
@@ -206,9 +199,8 @@ mozilla::ipc::IPCResult BrowserBridgeChild::RecvScrollRectIntoView(
       aRect.ScaleToOtherAppUnitsRoundOut(aAppUnitsPerDevPixel, parentAPD);
   rect += extraOffset;
   RefPtr<PresShell> presShell = frame->PresShell();
-  presShell->ScrollFrameRectIntoView(frame, rect, aVertical, aHorizontal,
-                                     aScrollFlags);
-
+  presShell->ScrollFrameIntoView(frame, Some(rect), aVertical, aHorizontal,
+                                 aScrollFlags);
   return IPC_OK();
 }
 
@@ -221,6 +213,10 @@ mozilla::ipc::IPCResult BrowserBridgeChild::RecvSubFrameCrashed() {
 }
 
 void BrowserBridgeChild::ActorDestroy(ActorDestroyReason aWhy) {
+  if (mFrameLoader) {
+    mFrameLoader->DestroyComplete();
+  }
+
   if (!mBrowsingContext) {
     // This BBC was never valid, skip teardown.
     return;
@@ -234,9 +230,9 @@ void BrowserBridgeChild::ActorDestroy(ActorDestroyReason aWhy) {
 void BrowserBridgeChild::UnblockOwnerDocsLoadEvent() {
   if (!mHadInitialLoad) {
     mHadInitialLoad = true;
-    if (auto* docShell =
-            nsDocShell::Cast(mBrowsingContext->GetParent()->GetDocShell())) {
-      docShell->OOPChildLoadDone(this);
+
+    if (Document* doc = mBrowsingContext->GetParent()->GetExtantDocument()) {
+      doc->OOPChildLoadDone(this);
     }
   }
 }
@@ -245,11 +241,21 @@ mozilla::ipc::IPCResult BrowserBridgeChild::RecvIntrinsicSizeOrRatioChanged(
     const Maybe<IntrinsicSize>& aIntrinsicSize,
     const Maybe<AspectRatio>& aIntrinsicRatio) {
   if (RefPtr<Element> owner = mFrameLoader->GetOwnerContent()) {
-    if (nsIFrame* f = owner->GetPrimaryFrame()) {
-      if (nsSubDocumentFrame* sdf = do_QueryFrame(f)) {
-        sdf->SubdocumentIntrinsicSizeOrRatioChanged(aIntrinsicSize,
-                                                    aIntrinsicRatio);
-      }
+    if (nsCOMPtr<nsIObjectLoadingContent> olc = do_QueryInterface(owner)) {
+      static_cast<nsObjectLoadingContent*>(olc.get())
+          ->SubdocumentIntrinsicSizeOrRatioChanged(aIntrinsicSize,
+                                                   aIntrinsicRatio);
+    }
+  }
+  return IPC_OK();
+}
+
+mozilla::ipc::IPCResult BrowserBridgeChild::RecvImageLoadComplete(
+    const nsresult& aResult) {
+  if (RefPtr<Element> owner = mFrameLoader->GetOwnerContent()) {
+    if (nsCOMPtr<nsIObjectLoadingContent> olc = do_QueryInterface(owner)) {
+      static_cast<nsObjectLoadingContent*>(olc.get())
+          ->SubdocumentImageLoadComplete(aResult);
     }
   }
   return IPC_OK();
