@@ -8,7 +8,6 @@
 #define mozilla_dom_BrowserBridgeParent_h
 
 #include "mozilla/dom/PBrowserBridgeParent.h"
-#include "mozilla/Tuple.h"
 #include "mozilla/dom/ipc/IdType.h"
 #include "mozilla/dom/WindowGlobalTypes.h"
 
@@ -18,6 +17,10 @@ namespace a11y {
 class DocAccessibleParent;
 }
 
+namespace embedding {
+class PrintData;
+}
+
 namespace dom {
 
 class BrowserParent;
@@ -46,14 +49,21 @@ class BrowserBridgeParent : public PBrowserBridgeParent {
 
 #if defined(ACCESSIBILITY)
   /**
-   * Get the accessible for this iframe's embedder OuterDocAccessible.
-   * This returns the actor for the containing document and the unique id of
-   * the embedder accessible within that document.
+   * Get the DocAccessibleParent which contains this iframe.
    */
-  Tuple<a11y::DocAccessibleParent*, uint64_t> GetEmbedderAccessible() {
-    return Tuple<a11y::DocAccessibleParent*, uint64_t>(mEmbedderAccessibleDoc,
-                                                       mEmbedderAccessibleID);
-  }
+  a11y::DocAccessibleParent* GetEmbedderAccessibleDoc();
+
+  /**
+   * Get the unique id of the OuterDocAccessible associated with this iframe.
+   * This is the id of the RemoteAccessible inside the document returned by
+   * GetEmbedderAccessibleDoc.
+   */
+  uint64_t GetEmbedderAccessibleId() { return mEmbedderAccessibleID; }
+
+  /**
+   * Get the DocAccessibleParent for the embedded document.
+   */
+  a11y::DocAccessibleParent* GetDocAccessibleParent();
 #endif  // defined(ACCESSIBILITY)
 
   // Tear down this BrowserBridgeParent.
@@ -69,24 +79,31 @@ class BrowserBridgeParent : public PBrowserBridgeParent {
   mozilla::ipc::IPCResult RecvUpdateDimensions(const nsIntRect& aRect,
                                                const ScreenIntSize& aSize);
   mozilla::ipc::IPCResult RecvUpdateEffects(const EffectsInfo& aEffects);
+  mozilla::ipc::IPCResult RecvUpdateRemotePrintSettings(
+      const embedding::PrintData&);
   mozilla::ipc::IPCResult RecvRenderLayers(const bool& aEnabled,
                                            const LayersObserverEpoch& aEpoch);
 
   mozilla::ipc::IPCResult RecvNavigateByKey(const bool& aForward,
                                             const bool& aForDocumentNavigation);
+  mozilla::ipc::IPCResult RecvBeginDestroy();
 
   mozilla::ipc::IPCResult RecvDispatchSynthesizedMouseEvent(
       const WidgetMouseEvent& aEvent);
 
   mozilla::ipc::IPCResult RecvWillChangeProcess();
 
-  mozilla::ipc::IPCResult RecvActivate();
+  mozilla::ipc::IPCResult RecvActivate(uint64_t aActionId);
 
-  mozilla::ipc::IPCResult RecvDeactivate(const bool& aWindowLowering);
+  mozilla::ipc::IPCResult RecvDeactivate(const bool& aWindowLowering,
+                                         uint64_t aActionId);
 
   mozilla::ipc::IPCResult RecvSetIsUnderHiddenEmbedderElement(
       const bool& aIsUnderHiddenEmbedderElement);
 
+  mozilla::ipc::IPCResult RecvUpdateRemoteStyle(
+      const StyleImageRendering& aImageRendering);
+
 #ifdef ACCESSIBILITY
   mozilla::ipc::IPCResult RecvSetEmbedderAccessible(PDocAccessibleParent* aDoc,
                                                     uint64_t aID);
