@@ -7,16 +7,16 @@
 #include "FilePickerParent.h"
 #include "nsComponentManagerUtils.h"
 #include "nsNetCID.h"
-#include "nsIDocument.h"
-#include "nsIDOMWindow.h"
+#include "mozilla/dom/Document.h"
 #include "nsIFile.h"
 #include "nsISimpleEnumerator.h"
-#include "mozilla/unused.h"
-#include "mozilla/dom/File.h"
+#include "mozilla/Unused.h"
+#include "mozilla/dom/FileBlobImpl.h"
+#include "mozilla/dom/FileSystemSecurity.h"
 #include "mozilla/dom/ContentParent.h"
 #include "mozilla/dom/Element.h"
-#include "mozilla/dom/TabParent.h"
-#include "mozilla/dom/ipc/BlobParent.h"
+#include "mozilla/dom/BrowserParent.h"
+#include "mozilla/dom/IPCBlobUtils.h"
 
 using mozilla::Unused;
 using namespace mozilla::dom;
@@ -25,47 +25,35 @@ NS_IMPL_ISUPPORTS(FilePickerParent::FilePickerShownCallback,
                   nsIFilePickerShownCallback);
 
 NS_IMETHODIMP
-FilePickerParent::FilePickerShownCallback::Done(int16_t aResult)
-{
+FilePickerParent::FilePickerShownCallback::Done(int16_t aResult) {
   if (mFilePickerParent) {
     mFilePickerParent->Done(aResult);
   }
   return NS_OK;
 }
 
-void
-FilePickerParent::FilePickerShownCallback::Destroy()
-{
+void FilePickerParent::FilePickerShownCallback::Destroy() {
   mFilePickerParent = nullptr;
 }
 
-FilePickerParent::~FilePickerParent()
-{
-}
+FilePickerParent::~FilePickerParent() = default;
 
-// Before sending a blob to the child, we need to get its size and modification
-// date. Otherwise it will be sent as a "mystery blob" by
-// GetOrCreateActorForBlob, which will cause problems for the child
-// process. This runnable stat()s the file off the main thread.
-//
 // We run code in three places:
 // 1. The main thread calls Dispatch() to start the runnable.
 // 2. The stream transport thread stat()s the file in Run() and then dispatches
 // the same runnable on the main thread.
 // 3. The main thread sends the results over IPC.
-FilePickerParent::IORunnable::IORunnable(FilePickerParent *aFPParent,
-                                         nsTArray<nsCOMPtr<nsIFile>>& aFiles,
+FilePickerParent::IORunnable::IORunnable(FilePickerParent* aFPParent,
+                                         nsTArray<nsCOMPtr<nsIFile>>&& aFiles,
                                          bool aIsDirectory)
- : mFilePickerParent(aFPParent)
- , mIsDirectory(aIsDirectory)
-{
-  mFiles.SwapElements(aFiles);
+    : mozilla::Runnable("dom::FilePickerParent::IORunnable"),
+      mFilePickerParent(aFPParent),
+      mFiles(std::move(aFiles)),
+      mIsDirectory(aIsDirectory) {
   MOZ_ASSERT_IF(aIsDirectory, mFiles.Length() == 1);
 }
 
-bool
-FilePickerParent::IORunnable::Dispatch()
-{
+bool FilePickerParent::IORunnable::Dispatch() {
   MOZ_ASSERT(NS_IsMainThread());
 
   mEventTarget = do_GetService(NS_STREAMTRANSPORTSERVICE_CONTRACTID);
@@ -78,8 +66,7 @@ FilePickerParent::IORunnable::Dispatch()
 }
 
 NS_IMETHODIMP
-FilePickerParent::IORunnable::Run()
-{
+FilePickerParent::IORunnable::Run() {
   // If we're on the main thread, then that means we're done. Just send the
   // results.
   if (NS_IsMainThread()) {
@@ -105,7 +92,7 @@ FilePickerParent::IORunnable::Run()
       continue;
     }
 
-    RefPtr<BlobImpl> blobImpl = new BlobImplFile(mFiles[i]);
+    RefPtr<BlobImpl> blobImpl = new FileBlobImpl(mFiles[i]);
 
     ErrorResult error;
     blobImpl->GetSize(error);
@@ -136,15 +123,12 @@ FilePickerParent::IORunnable::Run()
   return NS_OK;
 }
 
-void
-FilePickerParent::IORunnable::Destroy()
-{
-  mFilePickerParent = nullptr;
-}
+void FilePickerParent::IORunnable::Destroy() { mFilePickerParent = nullptr; }
+
+void FilePickerParent::SendFilesOrDirectories(
+    const nsTArray<BlobImplOrString>& aData) {
+  ContentParent* parent = BrowserParent::GetFrom(Manager())->Manager();
 
-void
-FilePickerParent::SendFilesOrDirectories(const nsTArray<BlobImplOrString>& aData)
-{
   if (mMode == nsIFilePicker::modeGetFolder) {
     MOZ_ASSERT(aData.Length() <= 1);
     if (aData.IsEmpty()) {
@@ -154,31 +138,39 @@ FilePickerParent::SendFilesOrDirectories(const nsTArray<BlobImplOrString>& aData
 
     MOZ_ASSERT(aData[0].mType == BlobImplOrString::eDirectoryPath);
 
+    // Let's inform the security singleton about the given access of this tab on
+    // this directory path.
+    RefPtr<FileSystemSecurity> fss = FileSystemSecurity::GetOrCreate();
+    fss->GrantAccessToContentProcess(parent->ChildID(),
+                                     aData[0].mDirectoryPath);
+
     InputDirectory input;
     input.directoryPath() = aData[0].mDirectoryPath;
     Unused << Send__delete__(this, input, mResult);
     return;
   }
 
-  nsIContentParent* parent = TabParent::GetFrom(Manager())->Manager();
-  InfallibleTArray<PBlobParent*> blobs;
+  nsTArray<IPCBlob> ipcBlobs;
 
   for (unsigned i = 0; i < aData.Length(); i++) {
+    IPCBlob ipcBlob;
+
     MOZ_ASSERT(aData[i].mType == BlobImplOrString::eBlobImpl);
-    BlobParent* blobParent = parent->GetOrCreateActorForBlobImpl(aData[i].mBlobImpl);
-    if (blobParent) {
-      blobs.AppendElement(blobParent);
+    nsresult rv = IPCBlobUtils::Serialize(aData[i].mBlobImpl, ipcBlob);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      break;
     }
+
+    ipcBlobs.AppendElement(ipcBlob);
   }
 
   InputBlobs inblobs;
-  inblobs.blobsParent().SwapElements(blobs);
+  inblobs.blobs() = std::move(ipcBlobs);
+
   Unused << Send__delete__(this, inblobs, mResult);
 }
 
-void
-FilePickerParent::Done(int16_t aResult)
-{
+void FilePickerParent::Done(int16_t aResult) {
   mResult = aResult;
 
   if (mResult != nsIFilePicker::returnOK) {
@@ -215,7 +207,8 @@ FilePickerParent::Done(int16_t aResult)
   }
 
   MOZ_ASSERT(!mRunnable);
-  mRunnable = new IORunnable(this, files, mMode == nsIFilePicker::modeGetFolder);
+  mRunnable = new IORunnable(this, std::move(files),
+                             mMode == nsIFilePicker::modeGetFolder);
 
   // Dispatch to background thread to do I/O:
   if (!mRunnable->Dispatch()) {
@@ -223,15 +216,13 @@ FilePickerParent::Done(int16_t aResult)
   }
 }
 
-bool
-FilePickerParent::CreateFilePicker()
-{
+bool FilePickerParent::CreateFilePicker() {
   mFilePicker = do_CreateInstance("@mozilla.org/filepicker;1");
   if (!mFilePicker) {
     return false;
   }
 
-  Element* element = TabParent::GetFrom(Manager())->GetOwnerElement();
+  Element* element = BrowserParent::GetFrom(Manager())->GetOwnerElement();
   if (!element) {
     return false;
   }
@@ -244,19 +235,16 @@ FilePickerParent::CreateFilePicker()
   return NS_SUCCEEDED(mFilePicker->Init(window, mTitle, mMode));
 }
 
-bool
-FilePickerParent::RecvOpen(const int16_t& aSelectedType,
-                           const bool& aAddToRecentDocs,
-                           const nsString& aDefaultFile,
-                           const nsString& aDefaultExtension,
-                           InfallibleTArray<nsString>&& aFilters,
-                           InfallibleTArray<nsString>&& aFilterNames,
-                           const nsString& aDisplayDirectory,
-                           const nsString& aOkButtonLabel)
-{
+mozilla::ipc::IPCResult FilePickerParent::RecvOpen(
+    const int16_t& aSelectedType, const bool& aAddToRecentDocs,
+    const nsString& aDefaultFile, const nsString& aDefaultExtension,
+    nsTArray<nsString>&& aFilters, nsTArray<nsString>&& aFilterNames,
+    nsTArray<nsString>&& aRawFilters, const nsString& aDisplayDirectory,
+    const nsString& aDisplaySpecialDirectory, const nsString& aOkButtonLabel,
+    const int16_t& aCapture) {
   if (!CreateFilePicker()) {
     Unused << Send__delete__(this, void_t(), nsIFilePicker::returnCancel);
-    return true;
+    return IPC_OK();
   }
 
   mFilePicker->SetAddToRecentDocs(aAddToRecentDocs);
@@ -265,10 +253,15 @@ FilePickerParent::RecvOpen(const int16_t& aSelectedType,
     mFilePicker->AppendFilter(aFilterNames[i], aFilters[i]);
   }
 
+  for (uint32_t i = 0; i < aRawFilters.Length(); ++i) {
+    mFilePicker->AppendRawFilter(aRawFilters[i]);
+  }
+
   mFilePicker->SetDefaultString(aDefaultFile);
   mFilePicker->SetDefaultExtension(aDefaultExtension);
   mFilePicker->SetFilterIndex(aSelectedType);
   mFilePicker->SetOkButtonLabel(aOkButtonLabel);
+  mFilePicker->SetCapture(aCapture);
 
   if (!aDisplayDirectory.IsEmpty()) {
     nsCOMPtr<nsIFile> localFile = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID);
@@ -276,17 +269,17 @@ FilePickerParent::RecvOpen(const int16_t& aSelectedType,
       localFile->InitWithPath(aDisplayDirectory);
       mFilePicker->SetDisplayDirectory(localFile);
     }
+  } else if (!aDisplaySpecialDirectory.IsEmpty()) {
+    mFilePicker->SetDisplaySpecialDirectory(aDisplaySpecialDirectory);
   }
 
   mCallback = new FilePickerShownCallback(this);
 
   mFilePicker->Open(mCallback);
-  return true;
+  return IPC_OK();
 }
 
-void
-FilePickerParent::ActorDestroy(ActorDestroyReason aWhy)
-{
+void FilePickerParent::ActorDestroy(ActorDestroyReason aWhy) {
   if (mCallback) {
     mCallback->Destroy();
     mCallback = nullptr;
