@@ -13,12 +13,18 @@
 #include "mozilla/dom/ipc/StructuredCloneData.h"
 #include "mozilla/dom/ScriptSettings.h"
 #include "mozilla/ErrorResult.h"
+#include "mozilla/RandomNum.h"
 #include "nsFrameMessageManager.h"
+#include "prenv.h"
 
 namespace mozilla::dom {
 
 LazyLogModule MMPrinter::sMMLog("MessageManager");
 
+// You can use
+// https://gist.github.com/tomrittervg/adb8688426a9a5340da96004e2c8af79 to parse
+// the output of the logs into logs more friendly to reading.
+
 /* static */
 void MMPrinter::PrintImpl(char const* aLocation, const nsAString& aMsg,
                           ClonedMessageData const& aData) {
@@ -40,9 +46,11 @@ void MMPrinter::PrintImpl(char const* aLocation, const nsAString& aMsg,
     return;
   }
 
+  uint64_t msg_id = RandomUint64OrDie();
+
   MOZ_LOG(MMPrinter::sMMLog, LogLevel::Debug,
-          ("%s Message: %s in process type: %s", aLocation, charMsg.get(),
-           XRE_GetProcessTypeString()));
+          ("%" PRIu64 " %s Message: %s in process type: %s", msg_id, aLocation,
+           charMsg.get(), XRE_GetProcessTypeString()));
 
   if (!MOZ_LOG_TEST(sMMLog, LogLevel::Verbose)) {
     return;
@@ -59,22 +67,26 @@ void MMPrinter::PrintImpl(char const* aLocation, const nsAString& aMsg,
   JSContext* cx = jsapi.cx();
 
   ipc::StructuredCloneData data;
-  ipc::UnpackClonedMessageDataForChild(aData, data);
+  ipc::UnpackClonedMessageData(aData, data);
 
   /* Read original StructuredCloneData. */
-  JS::RootedValue scdContent(cx);
+  JS::Rooted<JS::Value> scdContent(cx);
   data.Read(cx, &scdContent, rv);
-  if (NS_WARN_IF(rv.Failed())) {
+  if (rv.Failed()) {
+    // In testing, the only reason this would fail was if there was no data in
+    // the message; so it seems like this is safe-ish.
+    MOZ_LOG(MMPrinter::sMMLog, LogLevel::Verbose,
+            ("%" PRIu64 " (No Data)", msg_id));
     rv.SuppressException();
     return;
   }
 
-  JS::RootedString unevalObj(cx, JS_ValueToSource(cx, scdContent));
+  JS::Rooted<JSString*> unevalObj(cx, JS_ValueToSource(cx, scdContent));
   nsAutoJSString srcString;
   if (!srcString.init(cx, unevalObj)) return;
 
   MOZ_LOG(MMPrinter::sMMLog, LogLevel::Verbose,
-          ("   %s", NS_ConvertUTF16toUTF8(srcString).get()));
+          ("%" PRIu64 " %s", msg_id, NS_ConvertUTF16toUTF8(srcString).get()));
 }
 
 }  // namespace mozilla::dom
