@@ -22,6 +22,7 @@
 #include "mozilla/dom/BrowserChild.h"
 #include "mozilla/dom/BrowserParent.h"
 #include "mozilla/ipc/Endpoint.h"
+#include "mozilla/ipc/ProcessChild.h"
 #include "mozilla/ipc/TaskFactory.h"
 #include "mozilla/Monitor.h"
 #include "mozilla/Preferences.h"
@@ -110,6 +111,7 @@ class HangMonitorChild : public PProcessHangMonitorChild,
   void MaybeStartPaintWhileInterruptingJS();
 
   mozilla::ipc::IPCResult RecvTerminateScript() override;
+  mozilla::ipc::IPCResult RecvRequestContentJSInterrupt() override;
   mozilla::ipc::IPCResult RecvBeginStartingDebugger() override;
   mozilla::ipc::IPCResult RecvEndStartingDebugger() override;
 
@@ -154,21 +156,22 @@ class HangMonitorChild : public PProcessHangMonitorChild,
   bool mSentReport;
 
   // These fields must be accessed with mMonitor held.
-  bool mTerminateScript GUARDED_BY(mMonitor);
-  bool mStartDebugger GUARDED_BY(mMonitor);
-  bool mFinishedStartingDebugger GUARDED_BY(mMonitor);
-  bool mPaintWhileInterruptingJS GUARDED_BY(mMonitor);
-  TabId mPaintWhileInterruptingJSTab GUARDED_BY(mMonitor);
+  bool mTerminateScript MOZ_GUARDED_BY(mMonitor);
+  bool mStartDebugger MOZ_GUARDED_BY(mMonitor);
+  bool mFinishedStartingDebugger MOZ_GUARDED_BY(mMonitor);
+  bool mPaintWhileInterruptingJS MOZ_GUARDED_BY(mMonitor);
+  TabId mPaintWhileInterruptingJSTab MOZ_GUARDED_BY(mMonitor);
   MOZ_INIT_OUTSIDE_CTOR LayersObserverEpoch mPaintWhileInterruptingJSEpoch
-      GUARDED_BY(mMonitor);
-  bool mCancelContentJS GUARDED_BY(mMonitor);
-  TabId mCancelContentJSTab GUARDED_BY(mMonitor);
+      MOZ_GUARDED_BY(mMonitor);
+  bool mCancelContentJS MOZ_GUARDED_BY(mMonitor);
+  TabId mCancelContentJSTab MOZ_GUARDED_BY(mMonitor);
   nsIRemoteTab::NavigationType mCancelContentJSNavigationType
-      GUARDED_BY(mMonitor);
-  int32_t mCancelContentJSNavigationIndex GUARDED_BY(mMonitor);
-  mozilla::Maybe<nsCString> mCancelContentJSNavigationURI GUARDED_BY(mMonitor);
-  int32_t mCancelContentJSEpoch GUARDED_BY(mMonitor);
-  bool mShutdownDone GUARDED_BY(mMonitor);
+      MOZ_GUARDED_BY(mMonitor);
+  int32_t mCancelContentJSNavigationIndex MOZ_GUARDED_BY(mMonitor);
+  mozilla::Maybe<nsCString> mCancelContentJSNavigationURI
+      MOZ_GUARDED_BY(mMonitor);
+  int32_t mCancelContentJSEpoch MOZ_GUARDED_BY(mMonitor);
+  bool mShutdownDone MOZ_GUARDED_BY(mMonitor);
 
   JSContext* mContext;  // const after constructor
 
@@ -291,14 +294,14 @@ class HangMonitorParent : public PProcessHangMonitorParent,
   RefPtr<HangMonitoredProcess> mProcess;
 
   // Must be accessed with mMonitor held.
-  bool mShutdownDone GUARDED_BY(mMonitor);
+  bool mShutdownDone MOZ_GUARDED_BY(mMonitor);
   // Map from plugin ID to crash dump ID. Protected by
   // mBrowserCrashDumpHashLock.
   nsTHashMap<nsUint32HashKey, nsString> mBrowserCrashDumpIds
-      GUARDED_BY(mMonitor);
-  Mutex mBrowserCrashDumpHashLock GUARDED_BY(mMonitor);
+      MOZ_GUARDED_BY(mMonitor);
+  Mutex mBrowserCrashDumpHashLock MOZ_GUARDED_BY(mMonitor);
   mozilla::ipc::TaskFactory<HangMonitorParent> mMainThreadTaskFactory
-      GUARDED_BY(mMonitor);
+      MOZ_GUARDED_BY(mMonitor);
 };
 
 }  // namespace
@@ -344,6 +347,18 @@ HangMonitorChild::~HangMonitorChild() {
 bool HangMonitorChild::InterruptCallback() {
   MOZ_RELEASE_ASSERT(NS_IsMainThread());
 
+  if (StaticPrefs::dom_abort_script_on_child_shutdown() &&
+      mozilla::ipc::ProcessChild::ExpectingShutdown()) {
+    // We preserve chrome JS from cancel, but not extension content JS.
+    if (!nsContentUtils::IsCallerChrome()) {
+      NS_WARNING(
+          "HangMonitorChild::InterruptCallback: ExpectingShutdown, "
+          "canceling content JS execution.\n");
+      return false;
+    }
+    return true;
+  }
+
   // Don't start painting if we're not in a good place to run script. We run
   // chrome script during layout and such, and it wouldn't be good to interrupt
   // painting code from there.
@@ -375,7 +390,7 @@ bool HangMonitorChild::InterruptCallback() {
 
   // Only handle the interrupt for cancelling content JS if we have a
   // non-privileged script (i.e. not part of Gecko or an add-on).
-  JS::RootedObject global(mContext, JS::CurrentGlobalOrNull(mContext));
+  JS::Rooted<JSObject*> global(mContext, JS::CurrentGlobalOrNull(mContext));
   nsIPrincipal* principal = xpc::GetObjectPrincipal(global);
   if (principal && (principal->IsSystemPrincipal() ||
                     principal->GetIsAddonOrExpandedAddonPrincipal())) {
@@ -495,6 +510,24 @@ mozilla::ipc::IPCResult HangMonitorChild::RecvTerminateScript() {
   return IPC_OK();
 }
 
+mozilla::ipc::IPCResult HangMonitorChild::RecvRequestContentJSInterrupt() {
+  MOZ_RELEASE_ASSERT(IsOnThread());
+
+  // In order to cancel JS execution on shutdown, we expect that
+  // ProcessChild::NotifiedImpendingShutdown has been called before.
+  if (mozilla::ipc::ProcessChild::ExpectingShutdown()) {
+    CrashReporter::AppendToCrashReportAnnotation(
+        CrashReporter::Annotation::IPCShutdownState,
+        "HangMonitorChild::RecvRequestContentJSInterrupt (expected)"_ns);
+  } else {
+    CrashReporter::AppendToCrashReportAnnotation(
+        CrashReporter::Annotation::IPCShutdownState,
+        "HangMonitorChild::RecvRequestContentJSInterrupt (unexpected)"_ns);
+  }
+  JS_RequestInterruptCallback(mContext);
+  return IPC_OK();
+}
+
 mozilla::ipc::IPCResult HangMonitorChild::RecvBeginStartingDebugger() {
   MOZ_RELEASE_ASSERT(IsOnThread());
 
