@@ -8,9 +8,10 @@
 
 #include "AutoplayPolicy.h"
 #include "nsContentUtils.h"
-#include "mozJSComponentLoader.h"
+#include "mozilla/Components.h"
 #include "mozilla/ContentBlockingAllowList.h"
 #include "mozilla/Logging.h"
+#include "mozilla/dom/Document.h"
 #include "mozilla/dom/JSActorService.h"
 #include "mozilla/dom/JSWindowActorParent.h"
 #include "mozilla/dom/JSWindowActorChild.h"
@@ -21,6 +22,7 @@
 #include "mozilla/dom/WindowGlobalParent.h"
 
 #include "nsGlobalWindowInner.h"
+#include "nsNetUtil.h"
 
 namespace mozilla::dom {
 
@@ -53,6 +55,7 @@ WindowGlobalInit WindowGlobalActor::BaseInitializer(
   auto& fields = ctx.mFields;
   fields.mEmbedderPolicy = InheritedPolicy(aBrowsingContext);
   fields.mAutoplayPermission = nsIPermissionManager::UNKNOWN_ACTION;
+  fields.mAllowJavascript = true;
   return init;
 }
 
@@ -63,9 +66,9 @@ WindowGlobalInit WindowGlobalActor::AboutBlankInitializer(
                       nsContentUtils::GenerateWindowId());
 
   init.principal() = aPrincipal;
+  init.storagePrincipal() = aPrincipal;
   Unused << NS_NewURI(getter_AddRefs(init.documentURI()), "about:blank");
-  ContentBlockingAllowList::ComputePrincipal(
-      aPrincipal, getter_AddRefs(init.contentBlockingAllowListPrincipal()));
+  init.isInitialDocument() = true;
 
   return init;
 }
@@ -77,12 +80,12 @@ WindowGlobalInit WindowGlobalActor::WindowInitializer(
                       aWindow->GetOuterWindow()->WindowID());
 
   init.principal() = aWindow->GetPrincipal();
-  init.contentBlockingAllowListPrincipal() =
-      aWindow->GetDocumentContentBlockingAllowListPrincipal();
+  init.storagePrincipal() = aWindow->GetEffectiveStoragePrincipal();
   init.documentURI() = aWindow->GetDocumentURI();
 
   Document* doc = aWindow->GetDocument();
 
+  init.isInitialDocument() = doc->IsInitialDocument();
   init.blockAllMixedContent() = doc->GetBlockAllMixedContent(false);
   init.upgradeInsecureRequests() = doc->GetUpgradeInsecureRequests(false);
   init.sandboxFlags() = doc->GetSandboxFlags();
@@ -101,45 +104,47 @@ WindowGlobalInit WindowGlobalActor::WindowInitializer(
 
   // Initialze permission fields
   fields.mAutoplayPermission =
-      AutoplayPolicy::GetSiteAutoplayPermission(init.principal());
+      media::AutoplayPolicy::GetSiteAutoplayPermission(init.principal());
   fields.mPopupPermission = PopupBlocker::GetPopupPermission(init.principal());
 
   // Initialize top level permission fields
   if (aWindow->GetBrowsingContext()->IsTop()) {
+    fields.mAllowMixedContent = [&] {
+      uint32_t permit = nsIPermissionManager::UNKNOWN_ACTION;
+      nsCOMPtr<nsIPermissionManager> permissionManager =
+          components::PermissionManager::Service();
+
+      if (permissionManager) {
+        permissionManager->TestPermissionFromPrincipal(
+            init.principal(), "mixed-content"_ns, &permit);
+      }
+
+      return permit == nsIPermissionManager::ALLOW_ACTION;
+    }();
+
     fields.mShortcutsPermission =
         nsGlobalWindowInner::GetShortcutsPermission(init.principal());
   }
 
-  auto policy = doc->GetEmbedderPolicy();
-  if (policy.isSome()) {
+  if (auto policy = doc->GetEmbedderPolicy()) {
     fields.mEmbedderPolicy = *policy;
   }
 
   // Init Mixed Content Fields
   nsCOMPtr<nsIURI> innerDocURI = NS_GetInnermostURI(doc->GetDocumentURI());
-  if (innerDocURI) {
-    fields.mIsSecure = innerDocURI->SchemeIs("https");
-  }
-  nsCOMPtr<nsIChannel> mixedChannel;
-  aWindow->GetDocShell()->GetMixedContentChannel(getter_AddRefs(mixedChannel));
-  // A non null mixedContent channel on the docshell indicates,
-  // that the user has overriden mixed content to allow mixed
-  // content loads to happen.
-  if (mixedChannel && (mixedChannel == doc->GetChannel())) {
-    fields.mAllowMixedContent = true;
-  }
+  fields.mIsSecure = innerDocURI && innerDocURI->SchemeIs("https");
 
   nsCOMPtr<nsITransportSecurityInfo> securityInfo;
   if (nsCOMPtr<nsIChannel> channel = doc->GetChannel()) {
     nsCOMPtr<nsILoadInfo> loadInfo(channel->LoadInfo());
     fields.mIsOriginalFrameSource = loadInfo->GetOriginalFrameSrcLoad();
 
-    nsCOMPtr<nsISupports> securityInfoSupports;
-    channel->GetSecurityInfo(getter_AddRefs(securityInfoSupports));
-    securityInfo = do_QueryInterface(securityInfoSupports);
+    channel->GetSecurityInfo(getter_AddRefs(securityInfo));
   }
   init.securityInfo() = securityInfo;
 
+  fields.mIsLocalIP = init.principal()->GetIsLocalIpAddress();
+
   // Most data here is specific to the Document, which can change without
   // creating a new WindowGlobal. Anything new added here which fits that
   // description should also be synchronized in
@@ -157,10 +162,12 @@ already_AddRefed<JSActorProtocol> WindowGlobalActor::MatchingJSActorProtocol(
     return nullptr;
   }
 
-  if (!proto->Matches(BrowsingContext(), GetDocumentURI(), GetRemoteType())) {
-    aRv.Throw(NS_ERROR_NOT_AVAILABLE);
+  if (!proto->Matches(BrowsingContext(), GetDocumentURI(), GetRemoteType(),
+                      aRv)) {
+    MOZ_ASSERT(aRv.Failed());
     return nullptr;
   }
+  MOZ_ASSERT(!aRv.Failed());
   return proto.forget();
 }
 
