@@ -312,10 +312,24 @@ add_task(async function test_normal_background_tab() {
       let origtabID = browsingContextChildID(
         originalTab.linkedBrowser.browsingContext
       );
+
       Assert.equal(
         originalTab.linkedBrowser.frameLoader.remoteTab.priorityHint,
         false,
-        "PriorityHint of the original tab should be false on default"
+        "PriorityHint of the original tab should be false by default"
+      );
+
+      // Changing renderLayers doesn't change priority of the background tab.
+      originalTab.linkedBrowser.preserveLayers(true);
+      originalTab.linkedBrowser.renderLayers = true;
+      await new Promise(resolve =>
+        // eslint-disable-next-line mozilla/no-arbitrary-setTimeout
+        setTimeout(resolve, WAIT_FOR_CHANGE_TIME_MS)
+      );
+      Assert.equal(
+        gTabPriorityWatcher.currentPriority(origtabID),
+        PROCESS_PRIORITY_BACKGROUND,
+        "Tab didn't get prioritized only due to renderLayers"
       );
 
       // Test when priorityHint is true, the original tab priority
@@ -341,11 +355,6 @@ add_task(async function test_normal_background_tab() {
       );
 
       let tabID = browsingContextChildID(tab.linkedBrowser.browsingContext);
-      Assert.equal(
-        tab.linkedBrowser.frameLoader.remoteTab.priorityHint,
-        false,
-        "PriorityHint of the active tab should be false on default"
-      );
 
       // Test when priorityHint is true, the process priority of the
       // active tab remains PROCESS_PRIORITY_FOREGROUND.
@@ -364,6 +373,9 @@ add_task(async function test_normal_background_tab() {
         PROCESS_PRIORITY_FOREGROUND,
         "Setting priorityHint to false should maintain the new tab priority as foreground"
       );
+
+      originalTab.linkedBrowser.preserveLayers(false);
+      originalTab.linkedBrowser.renderLayers = false;
     }
   );
 });
