@@ -8,8 +8,6 @@
 
 const NS_ERROR_DOM_QUOTA_EXCEEDED_ERR = 22;
 
-var {Services} = ChromeUtils.import("resource://gre/modules/Services.jsm");
-
 function is(a, b, msg) {
   Assert.equal(a, b, msg);
 }
@@ -18,30 +16,15 @@ function ok(cond, msg) {
   Assert.ok(!!cond, msg);
 }
 
-function run_test() {
-  runTest();
-}
+add_setup(function() {
+  do_get_profile();
 
-if (!this.runTest) {
-  this.runTest = function() {
-    do_get_profile();
+  enableTesting();
 
-    enableTesting();
+  Cu.importGlobalProperties(["crypto"]);
 
-    Assert.ok(typeof testSteps === "function",
-              "There should be a testSteps function");
-    Assert.ok(testSteps.constructor.name === "AsyncFunction",
-              "testSteps should be an async function");
-
-    registerCleanupFunction(resetTesting);
-
-    add_task(testSteps);
-
-    // Since we defined run_test, we must invoke run_next_test() to start the
-    // async test.
-    run_next_test();
-  };
-}
+  registerCleanupFunction(resetTesting);
+});
 
 function returnToEventLoop() {
   return new Promise(function(resolve) {
@@ -50,6 +33,7 @@ function returnToEventLoop() {
 }
 
 function enableTesting() {
+  Services.prefs.setBoolPref("dom.simpleDB.enabled", true);
   Services.prefs.setBoolPref("dom.storage.testing", true);
 
   // xpcshell globals don't have associated clients in the Clients API sense, so
@@ -64,11 +48,14 @@ function resetTesting() {
   Services.prefs.clearUserPref("dom.quotaManager.testing");
   Services.prefs.clearUserPref("dom.storage.client_validation");
   Services.prefs.clearUserPref("dom.storage.testing");
+  Services.prefs.clearUserPref("dom.simpleDB.enabled");
 }
 
 function setGlobalLimit(globalLimit) {
-  Services.prefs.setIntPref("dom.quotaManager.temporaryStorage.fixedLimit",
-                            globalLimit);
+  Services.prefs.setIntPref(
+    "dom.quotaManager.temporaryStorage.fixedLimit",
+    globalLimit
+  );
 }
 
 function resetGlobalLimit() {
@@ -86,29 +73,41 @@ function resetOriginLimit() {
 function setTimeout(callback, timeout) {
   let timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
 
-  timer.initWithCallback({
-    notify() {
-      callback();
+  timer.initWithCallback(
+    {
+      notify() {
+        callback();
+      },
     },
-  }, timeout, Ci.nsITimer.TYPE_ONE_SHOT);
+    timeout,
+    Ci.nsITimer.TYPE_ONE_SHOT
+  );
 
   return timer;
 }
 
-function init() {
-  let request = Services.qms.init();
-
-  return request;
+function initStorage() {
+  return Services.qms.init();
 }
 
-function initOrigin(principal, persistence) {
-  let request = Services.qms.initStoragesForPrincipal(principal, persistence);
-
-  return request;
+function initTemporaryStorage() {
+  return Services.qms.initTemporaryStorage();
 }
 
-function getOriginUsage(principal) {
-  let request = Services.qms.getUsageForPrincipal(principal, function() { });
+function initPersistentOrigin(principal) {
+  return Services.qms.initializePersistentOrigin(principal);
+}
+
+function initTemporaryOrigin(persistence, principal) {
+  return Services.qms.initializeTemporaryOrigin(persistence, principal);
+}
+
+function getOriginUsage(principal, fromMemory = false) {
+  let request = Services.qms.getUsageForPrincipal(
+    principal,
+    function() {},
+    fromMemory
+  );
 
   return request;
 }
@@ -126,8 +125,12 @@ function clearOriginsByPattern(pattern) {
 }
 
 function clearOriginsByPrefix(principal, persistence) {
-  let request =
-    Services.qms.clearStoragesForPrincipal(principal, persistence, null, true);
+  let request = Services.qms.clearStoragesForPrincipal(
+    principal,
+    persistence,
+    null,
+    true
+  );
 
   return request;
 }
@@ -145,8 +148,11 @@ function reset() {
 }
 
 function resetOrigin(principal) {
-  let request =
-    Services.qms.resetStoragesForPrincipal(principal, "default", "ls");
+  let request = Services.qms.resetStoragesForPrincipal(
+    principal,
+    "default",
+    "ls"
+  );
 
   return request;
 }
@@ -157,8 +163,9 @@ function installPackage(packageName) {
   let packageFile = currentDir.clone();
   packageFile.append(packageName + ".zip");
 
-  let zipReader = Cc["@mozilla.org/libjar/zip-reader;1"]
-                  .createInstance(Ci.nsIZipReader);
+  let zipReader = Cc["@mozilla.org/libjar/zip-reader;1"].createInstance(
+    Ci.nsIZipReader
+  );
   zipReader.open(packageFile);
 
   let entryNames = [];
@@ -179,12 +186,14 @@ function installPackage(packageName) {
     } else {
       let istream = zipReader.getInputStream(entryName);
 
-      var ostream = Cc["@mozilla.org/network/file-output-stream;1"]
-                    .createInstance(Ci.nsIFileOutputStream);
+      var ostream = Cc[
+        "@mozilla.org/network/file-output-stream;1"
+      ].createInstance(Ci.nsIFileOutputStream);
       ostream.init(file, -1, parseInt("0644", 8), 0);
 
-      let bostream = Cc["@mozilla.org/network/buffered-output-stream;1"]
-                     .createInstance(Ci.nsIBufferedOutputStream);
+      let bostream = Cc[
+        "@mozilla.org/network/buffered-output-stream;1"
+      ].createInstance(Ci.nsIBufferedOutputStream);
       bostream.init(ostream, 32768);
 
       bostream.writeFrom(istream, istream.available());
@@ -239,31 +248,65 @@ function getPrincipal(url, attrs) {
   if (!attrs) {
     attrs = {};
   }
-  return Services.scriptSecurityManager.createCodebasePrincipal(uri, attrs);
+  return Services.scriptSecurityManager.createContentPrincipal(uri, attrs);
 }
 
 function getCurrentPrincipal() {
   return Cc["@mozilla.org/systemprincipal;1"].createInstance(Ci.nsIPrincipal);
 }
 
-function getLocalStorage(principal) {
-  if (!principal) {
-    principal = getCurrentPrincipal();
-  }
-
-  return Services.domStorageManager.createStorage(null, principal, principal, "");
+function getDefaultPrincipal() {
+  return getPrincipal("http://example.com");
 }
 
-function requestFinished(request) {
-  return new Promise(function(resolve, reject) {
-    request.callback = function(requestInner) {
-      if (requestInner.resultCode == Cr.NS_OK) {
-        resolve(requestInner.result);
-      } else {
-        reject(requestInner.resultCode);
-      }
+function getSimpleDatabase(principal, persistence) {
+  let connection = Cc["@mozilla.org/dom/sdb-connection;1"].createInstance(
+    Ci.nsISDBConnection
+  );
+
+  if (!principal) {
+    principal = getDefaultPrincipal();
+  }
+
+  connection.init(principal, persistence);
+
+  return connection;
+}
+
+function getLocalStorage(principal) {
+  if (!principal) {
+    principal = getDefaultPrincipal();
+  }
+
+  return Services.domStorageManager.createStorage(
+    null,
+    principal,
+    principal,
+    ""
+  );
+}
+
+class RequestError extends Error {
+  constructor(resultCode, resultName) {
+    super(`Request failed (code: ${resultCode}, name: ${resultName})`);
+    this.name = "RequestError";
+    this.resultCode = resultCode;
+    this.resultName = resultName;
+  }
+}
+
+async function requestFinished(request) {
+  await new Promise(function(resolve) {
+    request.callback = function() {
+      resolve();
     };
   });
+
+  if (request.resultCode !== Cr.NS_OK) {
+    throw new RequestError(request.resultCode, request.resultName);
+  }
+
+  return request.result;
 }
 
 function loadSubscript(path) {
@@ -271,3 +314,19 @@ function loadSubscript(path) {
   let uri = Services.io.newFileURI(file);
   Services.scriptloader.loadSubScript(uri.spec);
 }
+
+async function readUsageFromUsageFile(usageFile) {
+  let file = await File.createFromNsIFile(usageFile);
+
+  let buffer = await new Promise(resolve => {
+    let reader = new FileReader();
+    reader.onloadend = () => resolve(reader.result);
+    reader.readAsArrayBuffer(file);
+  });
+
+  // Manually getting the lower 32-bits because of the lack of support for
+  // 64-bit values currently from DataView/JS (other than the BigInt support
+  // that's currently behind a flag).
+  let view = new DataView(buffer, 8, 4);
+  return view.getUint32();
+}
