@@ -5,67 +5,69 @@
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "mozilla/dom/MediaKeys.h"
+
+#include "ChromiumCDMProxy.h"
 #include "GMPCrashHelper.h"
+#include "mozilla/EMEUtils.h"
+#include "mozilla/JSONStringWriteFuncs.h"
+#include "mozilla/Telemetry.h"
+#include "mozilla/dom/DOMException.h"
+#include "mozilla/dom/Document.h"
 #include "mozilla/dom/HTMLMediaElement.h"
-#include "mozilla/dom/MediaKeysBinding.h"
-#include "mozilla/dom/MediaKeyMessageEvent.h"
 #include "mozilla/dom/MediaKeyError.h"
+#include "mozilla/dom/MediaKeyMessageEvent.h"
 #include "mozilla/dom/MediaKeySession.h"
 #include "mozilla/dom/MediaKeyStatusMap.h"
-#include "mozilla/dom/DOMException.h"
+#include "mozilla/dom/MediaKeySystemAccess.h"
+#include "mozilla/dom/MediaKeysBinding.h"
 #include "mozilla/dom/UnionTypes.h"
-#include "mozilla/Telemetry.h"
+#include "mozilla/dom/WindowContext.h"
+#include "mozilla/dom/WindowGlobalChild.h"
+#include "nsContentCID.h"
+#include "nsContentTypeParser.h"
+#include "nsContentUtils.h"
+#include "nsIScriptObjectPrincipal.h"
+#include "nsPrintfCString.h"
+#include "nsServiceManagerUtils.h"
+
 #ifdef MOZ_WIDGET_ANDROID
 #  include "mozilla/MediaDrmCDMProxy.h"
 #endif
-#include "mozilla/EMEUtils.h"
-#include "nsContentUtils.h"
-#include "nsIScriptObjectPrincipal.h"
-#include "nsContentTypeParser.h"
 #ifdef XP_WIN
 #  include "mozilla/WindowsVersion.h"
 #endif
-#include "nsContentCID.h"
-#include "nsServiceManagerUtils.h"
-#include "mozilla/dom/MediaKeySystemAccess.h"
-#include "nsPrintfCString.h"
-#include "ChromiumCDMProxy.h"
 
-namespace mozilla {
-
-namespace dom {
+namespace mozilla::dom {
 
 // We don't use NS_IMPL_CYCLE_COLLECTION_WRAPPERCACHE because we need to
-// unregister our MediaKeys from mDocument's activity listeners. If we don't do
-// this then cycle collection can null mDocument before our dtor runs and the
-// observer ptr held by mDocument will dangle.
-NS_IMPL_CYCLE_COLLECTION_CLASS(MediaKeys)
+// disconnect our MediaKeys instances from the inner window (mparent) before
+// we unlink it.
+NS_IMPL_CYCLE_COLLECTION_WRAPPERCACHE_CLASS(MediaKeys)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(MediaKeys)
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mElement)
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mParent)
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mKeySessions)
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mPromises)
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mPendingSessions)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mDocument)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
-NS_IMPL_CYCLE_COLLECTION_TRACE_WRAPPERCACHE(MediaKeys)
 
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(MediaKeys)
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mElement)
+  tmp->DisconnectInnerWindow();
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mParent)
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mKeySessions)
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mPromises)
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mPendingSessions)
-  tmp->UnregisterActivityObserver();
-  NS_IMPL_CYCLE_COLLECTION_UNLINK(mDocument)
   NS_IMPL_CYCLE_COLLECTION_UNLINK_PRESERVED_WRAPPER
+  NS_IMPL_CYCLE_COLLECTION_UNLINK_WEAK_PTR
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
 NS_IMPL_CYCLE_COLLECTING_ADDREF(MediaKeys)
 NS_IMPL_CYCLE_COLLECTING_RELEASE(MediaKeys)
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(MediaKeys)
   NS_WRAPPERCACHE_INTERFACE_MAP_ENTRY
-  NS_INTERFACE_MAP_ENTRY(nsIDocumentActivity)
+  NS_INTERFACE_MAP_ENTRY(nsISupports)
+  NS_INTERFACE_MAP_ENTRY(nsIObserver)
 NS_INTERFACE_MAP_END
 
 MediaKeys::MediaKeys(nsPIDOMWindowInner* aParent, const nsAString& aKeySystem,
@@ -79,36 +81,85 @@ MediaKeys::MediaKeys(nsPIDOMWindowInner* aParent, const nsAString& aKeySystem,
 }
 
 MediaKeys::~MediaKeys() {
-  UnregisterActivityObserver();
-  mDocument = nullptr;
+  MOZ_ASSERT(NS_IsMainThread());
+
+  DisconnectInnerWindow();
   Shutdown();
   EME_LOG("MediaKeys[%p] destroyed", this);
 }
 
-void MediaKeys::RegisterActivityObserver() {
-  MOZ_ASSERT(mDocument);
-  if (mDocument) {
-    mDocument->RegisterActivityObserver(this);
+NS_IMETHODIMP MediaKeys::Observe(nsISupports* aSubject, const char* aTopic,
+                                 const char16_t* aData) {
+  MOZ_ASSERT(NS_IsMainThread());
+  MOZ_ASSERT(!strcmp(aTopic, kMediaKeysResponseTopic),
+             "Should only listen for responses to MediaKey requests");
+  EME_LOG("MediaKeys[%p] observing message with aTopic=%s aData=%s", this,
+          aTopic, NS_ConvertUTF16toUTF8(aData).get());
+  if (!strcmp(aTopic, kMediaKeysResponseTopic)) {
+    if (!mProxy) {
+      // This may happen if we're notified during shutdown or startup. If this
+      // is happening outside of those scenarios there's a bug.
+      EME_LOG(
+          "MediaKeys[%p] can't notify CDM of observed message as mProxy is "
+          "unset",
+          this);
+      return NS_OK;
+    }
+
+    if (u"capture-possible"_ns.Equals(aData)) {
+      mProxy->NotifyOutputProtectionStatus(
+          CDMProxy::OutputProtectionCheckStatus::CheckSuccessful,
+          CDMProxy::OutputProtectionCaptureStatus::CapturePossilbe);
+    } else if (u"capture-not-possible"_ns.Equals(aData)) {
+      mProxy->NotifyOutputProtectionStatus(
+          CDMProxy::OutputProtectionCheckStatus::CheckSuccessful,
+          CDMProxy::OutputProtectionCaptureStatus::CaptureNotPossible);
+    } else {
+      MOZ_ASSERT_UNREACHABLE("No code paths should lead to the failure case");
+      // This should be unreachable, but gracefully handle in case.
+      mProxy->NotifyOutputProtectionStatus(
+          CDMProxy::OutputProtectionCheckStatus::CheckFailed,
+          CDMProxy::OutputProtectionCaptureStatus::Unused);
+    }
   }
+  return NS_OK;
 }
 
-void MediaKeys::UnregisterActivityObserver() {
-  if (mDocument) {
-    mDocument->UnregisterActivityObserver(this);
-  }
+void MediaKeys::ConnectInnerWindow() {
+  MOZ_ASSERT(NS_IsMainThread());
+
+  nsCOMPtr<nsPIDOMWindowInner> innerWindowParent = GetParentObject();
+  MOZ_ASSERT(innerWindowParent,
+             "We should only be connecting when we have an inner window!");
+  innerWindowParent->AddMediaKeysInstance(this);
 }
 
-// NS_DECL_NSIDOCUMENTACTIVITY
-void MediaKeys::NotifyOwnerDocumentActivityChanged() {
-  EME_LOG("MediaKeys[%p] NotifyOwnerDocumentActivityChanged()", this);
-  // If our owning document is no longer active we should shutdown.
-  if (!mDocument->IsCurrentActiveDocument()) {
-    EME_LOG(
-        "MediaKeys[%p] NotifyOwnerDocumentActivityChanged() owning document is "
-        "not active, shutting down!",
-        this);
-    Shutdown();
+void MediaKeys::DisconnectInnerWindow() {
+  MOZ_ASSERT(NS_IsMainThread());
+
+  if (!GetParentObject()) {
+    // We don't have a parent. We've been cycle collected, or the window
+    // already notified us of its destruction and we cleared the ref.
+    return;
   }
+
+  GetParentObject()->RemoveMediaKeysInstance(this);
+}
+
+void MediaKeys::OnInnerWindowDestroy() {
+  MOZ_ASSERT(NS_IsMainThread());
+
+  EME_LOG("MediaKeys[%p] OnInnerWindowDestroy()", this);
+
+  // The InnerWindow should clear its reference to this object after this call,
+  // so we don't need to explicitly call DisconnectInnerWindow before nulling.
+  mParent = nullptr;
+
+  // Don't call shutdown directly because (at time of writing) mProxy can
+  // spin the event loop when it's shutdown. This can change the world state
+  // in the middle of window destruction, which we do not want.
+  GetMainThreadEventTarget()->Dispatch(
+      NewRunnableMethod("MediaKeys::Shutdown", this, &MediaKeys::Shutdown));
 }
 
 void MediaKeys::Terminated() {
@@ -116,12 +167,11 @@ void MediaKeys::Terminated() {
 
   KeySessionHashMap keySessions;
   // Remove entries during iteration will screw it. Make a copy first.
-  for (auto iter = mKeySessions.Iter(); !iter.Done(); iter.Next()) {
-    RefPtr<MediaKeySession>& session = iter.Data();
-    keySessions.Put(session->GetSessionId(), session);
+  for (const RefPtr<MediaKeySession>& session : mKeySessions.Values()) {
+    // XXX Could the RefPtr still be moved here?
+    keySessions.InsertOrUpdate(session->GetSessionId(), RefPtr{session});
   }
-  for (auto iter = keySessions.Iter(); !iter.Done(); iter.Next()) {
-    RefPtr<MediaKeySession>& session = iter.Data();
+  for (const RefPtr<MediaKeySession>& session : keySessions.Values()) {
     session->OnClosed();
   }
   keySessions.Clear();
@@ -142,13 +192,20 @@ void MediaKeys::Shutdown() {
     mProxy = nullptr;
   }
 
-  RefPtr<MediaKeys> kungFuDeathGrip = this;
+  nsCOMPtr<nsIObserverService> observerService =
+      mozilla::services::GetObserverService();
+  if (observerService && mObserverAdded) {
+    observerService->RemoveObserver(this, kMediaKeysResponseTopic);
+  }
 
-  for (auto iter = mPromises.Iter(); !iter.Done(); iter.Next()) {
-    RefPtr<dom::DetailedPromise>& promise = iter.Data();
-    promise->MaybeReject(
-        NS_ERROR_DOM_INVALID_STATE_ERR,
-        NS_LITERAL_CSTRING("Promise still outstanding at MediaKeys shutdown"));
+  // Hold a self reference to keep us alive after we clear the self reference
+  // for each promise. This ensures we stay alive until we're done shutting
+  // down.
+  RefPtr<MediaKeys> selfReference = this;
+
+  for (const RefPtr<dom::DetailedPromise>& promise : mPromises.Values()) {
+    promise->MaybeRejectWithInvalidStateError(
+        "Promise still outstanding at MediaKeys shutdown");
     Release();
   }
   mPromises.Clear();
@@ -168,26 +225,23 @@ void MediaKeys::GetKeySystem(nsString& aOutKeySystem) const {
 already_AddRefed<DetailedPromise> MediaKeys::SetServerCertificate(
     const ArrayBufferViewOrArrayBuffer& aCert, ErrorResult& aRv) {
   RefPtr<DetailedPromise> promise(
-      MakePromise(aRv, NS_LITERAL_CSTRING("MediaKeys.setServerCertificate")));
+      MakePromise(aRv, "MediaKeys.setServerCertificate"_ns));
   if (aRv.Failed()) {
     return nullptr;
   }
 
   if (!mProxy) {
     NS_WARNING("Tried to use a MediaKeys without a CDM");
-    promise->MaybeReject(
-        NS_ERROR_DOM_INVALID_STATE_ERR,
-        NS_LITERAL_CSTRING("Null CDM in MediaKeys.setServerCertificate()"));
+    promise->MaybeRejectWithInvalidStateError(
+        "Null CDM in MediaKeys.setServerCertificate()");
     return promise.forget();
   }
 
   nsTArray<uint8_t> data;
   CopyArrayBufferViewOrArrayBufferData(aCert, data);
   if (data.IsEmpty()) {
-    promise->MaybeReject(
-        NS_ERROR_DOM_TYPE_ERR,
-        NS_LITERAL_CSTRING(
-            "Empty certificate passed to MediaKeys.setServerCertificate()"));
+    promise->MaybeRejectWithTypeError(
+        "Empty certificate passed to MediaKeys.setServerCertificate()");
     return promise.forget();
   }
 
@@ -220,19 +274,19 @@ PromiseId MediaKeys::StorePromise(DetailedPromise* aPromise) {
 
 #ifdef DEBUG
   // We should not have already stored this promise!
-  for (auto iter = mPromises.ConstIter(); !iter.Done(); iter.Next()) {
-    MOZ_ASSERT(iter.Data() != aPromise);
+  for (const RefPtr<dom::DetailedPromise>& promise : mPromises.Values()) {
+    MOZ_ASSERT(promise != aPromise);
   }
 #endif
 
-  mPromises.Put(id, aPromise);
+  mPromises.InsertOrUpdate(id, RefPtr{aPromise});
   return id;
 }
 
 void MediaKeys::ConnectPendingPromiseIdWithToken(PromiseId aId,
                                                  uint32_t aToken) {
   // Should only be called from MediaKeySession::GenerateRequest.
-  mPromiseIdToken.Put(aId, aToken);
+  mPromiseIdToken.InsertOrUpdate(aId, aToken);
   EME_LOG(
       "MediaKeys[%p]::ConnectPendingPromiseIdWithToken() id=%u => token(%u)",
       this, aId, aToken);
@@ -257,16 +311,17 @@ already_AddRefed<DetailedPromise> MediaKeys::RetrievePromise(PromiseId aId) {
   return promise.forget();
 }
 
-void MediaKeys::RejectPromise(PromiseId aId, nsresult aExceptionCode,
+void MediaKeys::RejectPromise(PromiseId aId, ErrorResult&& aException,
                               const nsCString& aReason) {
+  uint32_t errorCodeAsInt = aException.ErrorCodeAsInt();
   EME_LOG("MediaKeys[%p]::RejectPromise(%" PRIu32 ", 0x%" PRIx32 ")", this, aId,
-          static_cast<uint32_t>(aExceptionCode));
+          errorCodeAsInt);
 
   RefPtr<DetailedPromise> promise(RetrievePromise(aId));
   if (!promise) {
     EME_LOG("MediaKeys[%p]::RejectPromise(%" PRIu32 ", 0x%" PRIx32
             ") couldn't retrieve promise! Bailing!",
-            this, aId, static_cast<uint32_t>(aExceptionCode));
+            this, aId, errorCodeAsInt);
     return;
   }
 
@@ -281,14 +336,14 @@ void MediaKeys::RejectPromise(PromiseId aId, nsresult aExceptionCode,
     mPromiseIdToken.Remove(aId);
   }
 
-  MOZ_ASSERT(NS_FAILED(aExceptionCode));
-  promise->MaybeReject(aExceptionCode, aReason);
+  MOZ_ASSERT(aException.Failed());
+  promise->MaybeReject(std::move(aException), aReason);
 
   if (mCreatePromiseId == aId) {
     // Note: This will probably destroy the MediaKeys object!
     EME_LOG("MediaKeys[%p]::RejectPromise(%" PRIu32 ", 0x%" PRIx32
             ") calling Release()",
-            this, aId, static_cast<uint32_t>(aExceptionCode));
+            this, aId, errorCodeAsInt);
     Release();
   }
 }
@@ -312,7 +367,7 @@ void MediaKeys::OnSessionIdReady(MediaKeySession* aSession) {
         "MediaKeySession with invalid sessionId passed to OnSessionIdReady()");
     return;
   }
-  mKeySessions.Put(aSession->GetSessionId(), aSession);
+  mKeySessions.InsertOrUpdate(aSession->GetSessionId(), RefPtr{aSession});
 }
 
 void MediaKeys::ResolvePromise(PromiseId aId) {
@@ -343,13 +398,11 @@ void MediaKeys::ResolvePromise(PromiseId aId) {
   mPendingSessions.Remove(token, getter_AddRefs(session));
   if (!session || session->GetSessionId().IsEmpty()) {
     NS_WARNING("Received activation for non-existent session!");
-    promise->MaybeReject(
-        NS_ERROR_DOM_INVALID_ACCESS_ERR,
-        NS_LITERAL_CSTRING("CDM LoadSession() returned a different session ID "
-                           "than requested"));
+    promise->MaybeRejectWithInvalidAccessError(
+        "CDM LoadSession() returned a different session ID than requested");
     return;
   }
-  mKeySessions.Put(session->GetSessionId(), session);
+  mKeySessions.InsertOrUpdate(session->GetSessionId(), RefPtr{session});
   promise->MaybeResolve(session);
 }
 
@@ -371,8 +424,7 @@ class MediaKeysGMPCrashHelper : public GMPCrashHelper {
   WeakPtr<MediaKeys> mMediaKeys;
 };
 
-already_AddRefed<CDMProxy> MediaKeys::CreateCDMProxy(
-    nsISerialEventTarget* aMainThread) {
+already_AddRefed<CDMProxy> MediaKeys::CreateCDMProxy() {
   EME_LOG("MediaKeys[%p]::CreateCDMProxy()", this);
   RefPtr<CDMProxy> proxy;
 #ifdef MOZ_WIDGET_ANDROID
@@ -380,24 +432,21 @@ already_AddRefed<CDMProxy> MediaKeys::CreateCDMProxy(
     proxy = new MediaDrmCDMProxy(
         this, mKeySystem,
         mConfig.mDistinctiveIdentifier == MediaKeysRequirement::Required,
-        mConfig.mPersistentState == MediaKeysRequirement::Required,
-        aMainThread);
+        mConfig.mPersistentState == MediaKeysRequirement::Required);
   } else
 #endif
   {
     proxy = new ChromiumCDMProxy(
         this, mKeySystem, new MediaKeysGMPCrashHelper(this),
         mConfig.mDistinctiveIdentifier == MediaKeysRequirement::Required,
-        mConfig.mPersistentState == MediaKeysRequirement::Required,
-        aMainThread);
+        mConfig.mPersistentState == MediaKeysRequirement::Required);
   }
   return proxy.forget();
 }
 
 already_AddRefed<DetailedPromise> MediaKeys::Init(ErrorResult& aRv) {
   EME_LOG("MediaKeys[%p]::Init()", this);
-  RefPtr<DetailedPromise> promise(
-      MakePromise(aRv, NS_LITERAL_CSTRING("MediaKeys::Init()")));
+  RefPtr<DetailedPromise> promise(MakePromise(aRv, "MediaKeys::Init()"_ns));
   if (aRv.Failed()) {
     return nullptr;
   }
@@ -405,66 +454,125 @@ already_AddRefed<DetailedPromise> MediaKeys::Init(ErrorResult& aRv) {
   // Determine principal (at creation time) of the MediaKeys object.
   nsCOMPtr<nsIScriptObjectPrincipal> sop = do_QueryInterface(GetParentObject());
   if (!sop) {
-    promise->MaybeReject(
-        NS_ERROR_DOM_INVALID_STATE_ERR,
-        NS_LITERAL_CSTRING("Couldn't get script principal in MediaKeys::Init"));
+    promise->MaybeRejectWithInvalidStateError(
+        "Couldn't get script principal in MediaKeys::Init");
     return promise.forget();
   }
   mPrincipal = sop->GetPrincipal();
 
-  // Determine principal of the "top-level" window; the principal of the
-  // page that will display in the URL bar.
+  // Begin figuring out the top level principal.
   nsCOMPtr<nsPIDOMWindowInner> window = GetParentObject();
-  if (!window) {
-    promise->MaybeReject(
-        NS_ERROR_DOM_INVALID_STATE_ERR,
-        NS_LITERAL_CSTRING("Couldn't get top-level window in MediaKeys::Init"));
-    return promise.forget();
-  }
-  nsCOMPtr<nsPIDOMWindowOuter> top = window->GetOuterWindow()->GetTop();
-  if (!top || !top->GetExtantDoc()) {
-    promise->MaybeReject(
-        NS_ERROR_DOM_INVALID_STATE_ERR,
-        NS_LITERAL_CSTRING("Couldn't get document in MediaKeys::Init"));
+
+  // If we're in a top level document, getting the top level principal is easy.
+  // However, we're not in a top level doc this becomes more complicated. If
+  // we're not top level we need to get the top level principal, this can be
+  // done by reading the principal of the load info, which we can get of a
+  // document's channel.
+  //
+  // There is an edge case we need to watch out for here where this code can be
+  // run in an about:blank document before it has done its async load. In this
+  // case the document will not yet have a load info. We address this below by
+  // walking up a level in the window context chain. See
+  // https://bugzilla.mozilla.org/show_bug.cgi?id=1675360
+  // for more info.
+  Document* document = window->GetExtantDoc();
+  if (!document) {
+    NS_WARNING("Failed to get document when creating MediaKeys");
+    promise->MaybeRejectWithInvalidStateError(
+        "Couldn't get document in MediaKeys::Init");
     return promise.forget();
   }
 
-  mDocument = top->GetExtantDoc();
+  WindowGlobalChild* windowGlobalChild = window->GetWindowGlobalChild();
+  if (!windowGlobalChild) {
+    NS_WARNING("Failed to get window global child when creating MediaKeys");
+    promise->MaybeRejectWithInvalidStateError(
+        "Couldn't get window global child in MediaKeys::Init");
+    return promise.forget();
+  }
 
-  mTopLevelPrincipal = mDocument->NodePrincipal();
+  if (windowGlobalChild->SameOriginWithTop()) {
+    // We're in the same origin as the top window context, so our principal
+    // is also the top principal.
+    mTopLevelPrincipal = mPrincipal;
+  } else {
+    // We have a different origin than the top doc, try and find the top level
+    // principal by looking it up via load info, which we read off a channel.
+    nsIChannel* channel = document->GetChannel();
 
+    WindowContext* windowContext = document->GetWindowContext();
+    if (!windowContext) {
+      NS_WARNING("Failed to get window context when creating MediaKeys");
+      promise->MaybeRejectWithInvalidStateError(
+          "Couldn't get window context in MediaKeys::Init");
+      return promise.forget();
+    }
+    while (!channel) {
+      // We don't have a channel, this can happen if we're in an about:blank
+      // page that hasn't yet had its async load performed. Try and get
+      // the channel from our parent doc. We should be able to do this because
+      // an about:blank is considered the same origin as its parent. We do this
+      // recursively to cover pages do silly things like nesting blank iframes
+      // and not waiting for loads.
+
+      // Move our window context up a level.
+      windowContext = windowContext->GetParentWindowContext();
+      if (!windowContext || !windowContext->GetExtantDoc()) {
+        NS_WARNING(
+            "Failed to get parent window context's document when creating "
+            "MediaKeys");
+        promise->MaybeRejectWithInvalidStateError(
+            "Couldn't get parent window context's document in "
+            "MediaKeys::Init (likely due to an nested about about:blank frame "
+            "that hasn't loaded yet)");
+        return promise.forget();
+      }
+
+      Document* parentDoc = windowContext->GetExtantDoc();
+      channel = parentDoc->GetChannel();
+    }
+
+    MOZ_RELEASE_ASSERT(
+        channel, "Should either have a channel or should have returned by now");
+
+    nsCOMPtr<nsILoadInfo> loadInfo = channel->LoadInfo();
+    MOZ_RELEASE_ASSERT(loadInfo, "Channels should always have LoadInfo");
+    mTopLevelPrincipal = loadInfo->GetTopLevelPrincipal();
+    if (!mTopLevelPrincipal) {
+      NS_WARNING("Failed to get top level principal when creating MediaKeys");
+      promise->MaybeRejectWithInvalidStateError(
+          "Couldn't get top level principal in MediaKeys::Init");
+      return promise.forget();
+    }
+  }
+
+  // We should have figured out our top level principal.
   if (!mPrincipal || !mTopLevelPrincipal) {
     NS_WARNING("Failed to get principals when creating MediaKeys");
-    promise->MaybeReject(
-        NS_ERROR_DOM_INVALID_STATE_ERR,
-        NS_LITERAL_CSTRING("Couldn't get principal(s) in MediaKeys::Init"));
+    promise->MaybeRejectWithInvalidStateError(
+        "Couldn't get principal(s) in MediaKeys::Init");
     return promise.forget();
   }
 
   nsAutoCString origin;
   nsresult rv = mPrincipal->GetOrigin(origin);
   if (NS_FAILED(rv)) {
-    promise->MaybeReject(
-        NS_ERROR_DOM_INVALID_STATE_ERR,
-        NS_LITERAL_CSTRING(
-            "Couldn't get principal origin string in MediaKeys::Init"));
+    promise->MaybeRejectWithInvalidStateError(
+        "Couldn't get principal origin string in MediaKeys::Init");
     return promise.forget();
   }
   nsAutoCString topLevelOrigin;
   rv = mTopLevelPrincipal->GetOrigin(topLevelOrigin);
   if (NS_FAILED(rv)) {
-    promise->MaybeReject(
-        NS_ERROR_DOM_INVALID_STATE_ERR,
-        NS_LITERAL_CSTRING("Couldn't get top-level principal origin string in "
-                           "MediaKeys::Init"));
+    promise->MaybeRejectWithInvalidStateError(
+        "Couldn't get top-level principal origin string in MediaKeys::Init");
     return promise.forget();
   }
 
   EME_LOG("MediaKeys[%p]::Create() (%s, %s)", this, origin.get(),
           topLevelOrigin.get());
 
-  mProxy =
-      CreateCDMProxy(top->GetExtantDoc()->EventTargetFor(TaskCategory::Other));
+  mProxy = CreateCDMProxy();
 
   // The CDMProxy's initialization is asynchronous. The MediaKeys is
   // refcounted, and its instance is returned to JS by promise once
@@ -482,7 +590,7 @@ already_AddRefed<DetailedPromise> MediaKeys::Init(ErrorResult& aRv) {
                NS_ConvertUTF8toUTF16(topLevelOrigin),
                KeySystemToGMPName(mKeySystem));
 
-  RegisterActivityObserver();
+  ConnectInnerWindow();
 
   return promise.forget();
 }
@@ -522,9 +630,9 @@ static bool IsSessionTypeSupported(const MediaKeySessionType aSessionType,
 }
 
 already_AddRefed<MediaKeySession> MediaKeys::CreateSession(
-    JSContext* aCx, MediaKeySessionType aSessionType, ErrorResult& aRv) {
-  EME_LOG("MediaKeys[%p]::CreateSession(aCx=%p, aSessionType=%" PRIu8 ")", this,
-          aCx, static_cast<uint8_t>(aSessionType));
+    MediaKeySessionType aSessionType, ErrorResult& aRv) {
+  EME_LOG("MediaKeys[%p]::CreateSession(aSessionType=%" PRIu8 ")", this,
+          static_cast<uint8_t>(aSessionType));
   if (!IsSessionTypeSupported(aSessionType, mConfig)) {
     EME_LOG("MediaKeys[%p]::CreateSession() failed, unsupported session type",
             this);
@@ -541,7 +649,7 @@ already_AddRefed<MediaKeySession> MediaKeys::CreateSession(
   EME_LOG("MediaKeys[%p] Creating session", this);
 
   RefPtr<MediaKeySession> session = new MediaKeySession(
-      aCx, GetParentObject(), this, mKeySystem, aSessionType, aRv);
+      GetParentObject(), this, mKeySystem, aSessionType, aRv);
 
   if (aRv.Failed()) {
     return nullptr;
@@ -549,10 +657,10 @@ already_AddRefed<MediaKeySession> MediaKeys::CreateSession(
   DDLINKCHILD("session", session.get());
 
   // Add session to the set of sessions awaiting their sessionId being ready.
-  EME_LOG("MediaKeys[%p]::CreateSession(aCx=%p, aSessionType=%" PRIu8
+  EME_LOG("MediaKeys[%p]::CreateSession(aSessionType=%" PRIu8
           ") putting session with token=%" PRIu32 " into mPendingSessions",
-          this, aCx, static_cast<uint8_t>(aSessionType), session->Token());
-  mPendingSessions.Put(session->Token(), session);
+          this, static_cast<uint8_t>(aSessionType), session->Token());
+  mPendingSessions.InsertOrUpdate(session->Token(), RefPtr{session});
 
   return session.forget();
 }
@@ -564,16 +672,6 @@ void MediaKeys::OnSessionLoaded(PromiseId aId, bool aSuccess) {
   ResolvePromiseWithResult(aId, aSuccess);
 }
 
-template <typename T>
-void MediaKeys::ResolvePromiseWithResult(PromiseId aId, const T& aResult) {
-  RefPtr<DetailedPromise> promise(RetrievePromise(aId));
-  if (!promise) {
-    return;
-  }
-
-  promise->MaybeResolve(aResult);
-}
-
 void MediaKeys::OnSessionClosed(MediaKeySession* aSession) {
   nsAutoString id;
   aSession->GetSessionId(id);
@@ -617,10 +715,59 @@ void MediaKeys::Unbind() {
   mElement = nullptr;
 }
 
+void MediaKeys::CheckIsElementCapturePossible() {
+  MOZ_ASSERT(NS_IsMainThread());
+  EME_LOG("MediaKeys[%p]::IsElementCapturePossible()", this);
+  // Note, HTMLMediaElement prevents capture of its content via Capture APIs
+  // on the element if it has a media keys attached (see bug 1071482). So we
+  // don't need to check those cases here (they are covered by tests).
+
+  nsCOMPtr<nsIObserverService> observerService =
+      mozilla::services::GetObserverService();
+
+  if (!observerService) {
+    // This can happen if we're in shutdown which means we may be going away
+    // soon anyway, but respond saying capture is possible since we can't
+    // forward the check further.
+    if (mProxy) {
+      mProxy->NotifyOutputProtectionStatus(
+          CDMProxy::OutputProtectionCheckStatus::CheckFailed,
+          CDMProxy::OutputProtectionCaptureStatus::Unused);
+    }
+    return;
+  }
+  if (!mObserverAdded) {
+    nsresult rv =
+        observerService->AddObserver(this, kMediaKeysResponseTopic, false);
+    if (NS_FAILED(rv)) {
+      if (mProxy) {
+        mProxy->NotifyOutputProtectionStatus(
+            CDMProxy::OutputProtectionCheckStatus::CheckFailed,
+            CDMProxy::OutputProtectionCaptureStatus::Unused);
+      }
+      return;
+    }
+    mObserverAdded = true;
+  }
+
+  if (mCaptureCheckRequestJson.IsEmpty()) {
+    // Lazily populate the JSON the first time we need it.
+    JSONStringWriteFunc<nsAutoCString> json;
+    JSONWriter jw{json};
+    jw.Start();
+    jw.StringProperty("status", "is-capture-possible");
+    jw.StringProperty("keySystem", NS_ConvertUTF16toUTF8(mKeySystem));
+    jw.End();
+    mCaptureCheckRequestJson = NS_ConvertUTF8toUTF16(json.StringCRef());
+  }
+
+  MOZ_DIAGNOSTIC_ASSERT(!mCaptureCheckRequestJson.IsEmpty());
+  observerService->NotifyObservers(mParent.get(), kMediaKeysRequestTopic,
+                                   mCaptureCheckRequestJson.get());
+}
+
 void MediaKeys::GetSessionsInfo(nsString& sessionsInfo) {
-  for (KeySessionHashMap::Iterator it = mKeySessions.Iter(); !it.Done();
-       it.Next()) {
-    MediaKeySession* keySession = it.Data();
+  for (const auto& keySession : mKeySessions.Values()) {
     nsString sessionID;
     keySession->GetSessionId(sessionID);
     sessionsInfo.AppendLiteral("(sid=");
@@ -630,12 +777,9 @@ void MediaKeys::GetSessionsInfo(nsString& sessionsInfo) {
       nsString keyID = keyStatusMap->GetKeyIDAsHexString(i);
       sessionsInfo.AppendLiteral("(kid=");
       sessionsInfo.Append(keyID);
-      using IntegerType = typename std::underlying_type<MediaKeyStatus>::type;
-      auto idx = static_cast<IntegerType>(keyStatusMap->GetValueAtIndex(i));
-      const char* keyStatus = MediaKeyStatusValues::strings[idx].value;
       sessionsInfo.AppendLiteral(" status=");
-      sessionsInfo.Append(
-          NS_ConvertUTF8toUTF16((nsDependentCString(keyStatus))));
+      sessionsInfo.AppendASCII(
+          MediaKeyStatusValues::GetString(keyStatusMap->GetValueAtIndex(i)));
       sessionsInfo.AppendLiteral(")");
     }
     sessionsInfo.AppendLiteral(")");
@@ -645,7 +789,7 @@ void MediaKeys::GetSessionsInfo(nsString& sessionsInfo) {
 already_AddRefed<Promise> MediaKeys::GetStatusForPolicy(
     const MediaKeysPolicy& aPolicy, ErrorResult& aRv) {
   RefPtr<DetailedPromise> promise(
-      MakePromise(aRv, NS_LITERAL_CSTRING("MediaKeys::GetStatusForPolicy()")));
+      MakePromise(aRv, "MediaKeys::GetStatusForPolicy()"_ns));
   if (aRv.Failed()) {
     return nullptr;
   }
@@ -657,17 +801,15 @@ already_AddRefed<Promise> MediaKeys::GetStatusForPolicy(
         "keysystem ",
         this);
     NS_WARNING("Tried to query without a CDM");
-    promise->MaybeReject(
-        NS_ERROR_DOM_NOT_SUPPORTED_ERR,
-        NS_LITERAL_CSTRING("HDCP policy check on unsupported keysystem"));
+    promise->MaybeRejectWithNotSupportedError(
+        "HDCP policy check on unsupported keysystem");
     return promise.forget();
   }
 
   if (!mProxy) {
     NS_WARNING("Tried to use a MediaKeys without a CDM");
-    promise->MaybeReject(
-        NS_ERROR_DOM_INVALID_STATE_ERR,
-        NS_LITERAL_CSTRING("Null CDM in MediaKeys.GetStatusForPolicy()"));
+    promise->MaybeRejectWithInvalidStateError(
+        "Null CDM in MediaKeys.GetStatusForPolicy()");
     return promise.forget();
   }
 
@@ -691,5 +833,4 @@ void MediaKeys::ResolvePromiseWithKeyStatus(PromiseId aId,
   promise->MaybeResolve(aMediaKeyStatus);
 }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
