@@ -13,6 +13,7 @@
 #include "GMPTimerParent.h"
 #include "GMPStorageParent.h"
 #include "mozilla/gmp/PGMPParent.h"
+#include "mozilla/ipc/CrashReporterHelper.h"
 #include "nsCOMPtr.h"
 #include "nscore.h"
 #include "nsISupports.h"
@@ -21,28 +22,25 @@
 #include "nsIFile.h"
 #include "mozilla/MozPromise.h"
 
-namespace mozilla {
-namespace ipc {
-class CrashReporterHost;
-}  // namespace ipc
-namespace gmp {
+namespace mozilla::gmp {
 
 class GMPCapability {
  public:
-  explicit GMPCapability() {}
+  explicit GMPCapability() = default;
   GMPCapability(GMPCapability&& aOther)
       : mAPIName(std::move(aOther.mAPIName)),
         mAPITags(std::move(aOther.mAPITags)) {}
-  explicit GMPCapability(const nsCString& aAPIName) : mAPIName(aAPIName) {}
+  explicit GMPCapability(const nsACString& aAPIName) : mAPIName(aAPIName) {}
   explicit GMPCapability(const GMPCapability& aOther) = default;
   nsCString mAPIName;
-  nsTArray<nsCString> mAPITags;
+  CopyableTArray<nsCString> mAPITags;
 
   static bool Supports(const nsTArray<GMPCapability>& aCapabilities,
-                       const nsCString& aAPI, const nsTArray<nsCString>& aTags);
+                       const nsACString& aAPI,
+                       const nsTArray<nsCString>& aTags);
 
   static bool Supports(const nsTArray<GMPCapability>& aCapabilities,
-                       const nsCString& aAPI, const nsCString& aTag);
+                       const nsACString& aAPI, const nsCString& aTag);
 };
 
 enum GMPState {
@@ -54,17 +52,19 @@ enum GMPState {
 
 class GMPContentParent;
 
-class GMPParent final : public PGMPParent {
+class GMPParent final
+    : public PGMPParent,
+      public ipc::CrashReporterHelper<GeckoProcessType_GMPlugin> {
   friend class PGMPParent;
 
  public:
   NS_INLINE_DECL_THREADSAFE_REFCOUNTING(GMPParent)
 
-  explicit GMPParent(AbstractThread* aMainThread);
+  GMPParent();
 
   RefPtr<GenericPromise> Init(GeckoMediaPluginServiceParent* aService,
                               nsIFile* aPluginDir);
-  nsresult CloneFrom(const GMPParent* aOther);
+  void CloneFrom(const GMPParent* aOther);
 
   void Crash();
 
@@ -150,16 +150,13 @@ class GMPParent final : public PGMPParent {
   RefPtr<GenericPromise> ReadGMPMetaData();
   RefPtr<GenericPromise> ReadGMPInfoFile(nsIFile* aFile);
   RefPtr<GenericPromise> ParseChromiumManifest(
-      const nsAString& aJSON);  // Main thread.
+      const nsAString& aJSON);  // Worker thread.
   RefPtr<GenericPromise> ReadChromiumManifestFile(
       nsIFile* aFile);  // GMP thread.
-  void WriteExtraDataForMinidump();
-  bool GetCrashID(nsString& aResult);
+  void AddCrashAnnotations();
+  void GetCrashID(nsString& aResult);
   void ActorDestroy(ActorDestroyReason aWhy) override;
 
-  mozilla::ipc::IPCResult RecvInitCrashReporter(
-      Shmem&& shmem, const NativeThreadId& aThreadId);
-
   mozilla::ipc::IPCResult RecvPGMPStorageConstructor(
       PGMPStorageParent* actor) override;
   PGMPStorageParent* AllocPGMPStorageParent();
@@ -171,6 +168,9 @@ class GMPParent final : public PGMPParent {
   bool DeallocPGMPTimerParent(PGMPTimerParent* aActor);
 
   mozilla::ipc::IPCResult RecvPGMPContentChildDestroyed();
+
+  mozilla::ipc::IPCResult RecvFOGData(ByteBuf&& aBuf);
+
   bool IsUsed() {
     return mGMPContentChildCount > 0 || !mGetContentParentPromises.IsEmpty();
   }
@@ -178,17 +178,34 @@ class GMPParent final : public PGMPParent {
   void ResolveGetContentParentPromises();
   void RejectGetContentParentPromises();
 
+#if defined(XP_MACOSX) && defined(__aarch64__)
+  // We pre-translate XUL and our plugin file to avoid x64 child process
+  // startup delays caused by translation for instances when the child
+  // process binary translations have not already been cached. i.e., the
+  // first time we launch an x64 child process after installation or
+  // update. Measured by binary size of a recent XUL and Widevine plugin,
+  // this makes up 94% of the translation needed. Re-translating the
+  // same binary does not cause translation to occur again.
+  void PreTranslateBins();
+  void PreTranslateBinsWorker();
+#endif
+
+#if defined(XP_MACOSX)
+  nsresult GetPluginFileArch(nsIFile* aPluginDir, nsAutoString& aLeafName,
+                             uint32_t& aArchSet);
+#endif
+
   GMPState mState;
   nsCOMPtr<nsIFile> mDirectory;  // plugin directory on disk
   nsString mName;  // base name of plugin on disk, UTF-16 because used for paths
   nsCString mDisplayName;  // name of plugin displayed to users
   nsCString mDescription;  // description of plugin for display to users
   nsCString mVersion;
-#ifdef XP_WIN
+#if defined(XP_WIN) || defined(XP_LINUX)
   nsCString mLibs;
 #endif
   nsString mAdapter;
-  uint32_t mPluginId;
+  const uint32_t mPluginId;
   nsTArray<GMPCapability> mCapabilities;
   GMPProcessParent* mProcess;
   bool mDeleteProcessOnlyOnUnload;
@@ -217,12 +234,15 @@ class GMPParent final : public PGMPParent {
   // to terminate gracefully.
   bool mHoldingSelfRef;
 
-  UniquePtr<ipc::CrashReporterHost> mCrashReporter;
+#if defined(XP_MACOSX) && defined(__aarch64__)
+  // The child process architecture to use.
+  uint32_t mChildLaunchArch;
+  nsCString mPluginFilePath;
+#endif
 
-  const RefPtr<AbstractThread> mMainThread;
+  const nsCOMPtr<nsISerialEventTarget> mMainThread;
 };
 
-}  // namespace gmp
-}  // namespace mozilla
+}  // namespace mozilla::gmp
 
 #endif  // GMPParent_h_
