@@ -5,21 +5,23 @@
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "GMPUtils.h"
-#include "nsDirectoryServiceDefs.h"
-#include "nsIFile.h"
-#include "nsCOMPtr.h"
-#include "nsLiteralString.h"
-#include "nsCRTGlue.h"
+
+#include "GMPService.h"
+#include "VideoLimits.h"
+#include "mozIGeckoMediaPluginService.h"
 #include "mozilla/Base64.h"
-#include "nsISimpleEnumerator.h"
+#include "nsCOMPtr.h"
+#include "nsCRTGlue.h"
+#include "nsDirectoryServiceDefs.h"
+#include "nsIConsoleService.h"
+#include "nsIFile.h"
+#include "nsLiteralString.h"
+#include "prio.h"
 
 namespace mozilla {
 
-void
-SplitAt(const char* aDelims,
-        const nsACString& aInput,
-        nsTArray<nsCString>& aOutTokens)
-{
+void SplitAt(const char* aDelims, const nsACString& aInput,
+             nsTArray<nsCString>& aOutTokens) {
   nsAutoCString str(aInput);
   char* end = str.BeginWriting();
   const char* start = nullptr;
@@ -28,15 +30,9 @@ SplitAt(const char* aDelims,
   }
 }
 
-nsCString
-ToHexString(const uint8_t * aBytes, uint32_t aLength)
-{
-  static const char hex[] = {
-    '0', '1', '2', '3',
-    '4', '5', '6', '7',
-    '8', '9', 'a', 'b',
-    'c', 'd', 'e', 'f'
-  };
+nsCString ToHexString(const uint8_t* aBytes, uint32_t aLength) {
+  static const char hex[] = {'0', '1', '2', '3', '4', '5', '6', '7',
+                             '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};
   nsCString str;
   for (uint32_t i = 0; i < aLength; i++) {
     char buf[3];
@@ -48,28 +44,21 @@ ToHexString(const uint8_t * aBytes, uint32_t aLength)
   return str;
 }
 
-nsCString
-ToHexString(const nsTArray<uint8_t>& aBytes)
-{
+nsCString ToHexString(const nsTArray<uint8_t>& aBytes) {
   return ToHexString(aBytes.Elements(), aBytes.Length());
 }
 
-bool
-FileExists(nsIFile* aFile)
-{
+bool FileExists(nsIFile* aFile) {
   bool exists = false;
   return aFile && NS_SUCCEEDED(aFile->Exists(&exists)) && exists;
 }
 
 DirectoryEnumerator::DirectoryEnumerator(nsIFile* aPath, Mode aMode)
-  : mMode(aMode)
-{
+    : mMode(aMode) {
   aPath->GetDirectoryEntries(getter_AddRefs(mIter));
 }
 
-already_AddRefed<nsIFile>
-DirectoryEnumerator::Next()
-{
+already_AddRefed<nsIFile> DirectoryEnumerator::Next() {
   if (!mIter) {
     return nullptr;
   }
@@ -98,18 +87,15 @@ DirectoryEnumerator::Next()
   return nullptr;
 }
 
-bool
-ReadIntoArray(nsIFile* aFile,
-              nsTArray<uint8_t>& aOutDst,
-              size_t aMaxLength)
-{
+bool ReadIntoArray(nsIFile* aFile, nsTArray<uint8_t>& aOutDst,
+                   size_t aMaxLength) {
   if (!FileExists(aFile)) {
     return false;
   }
 
   PRFileDesc* fd = nullptr;
   nsresult rv = aFile->OpenNSPRFileDesc(PR_RDONLY, 0, &fd);
-  if (NS_FAILED(rv)) {
+  if (NS_WARN_IF(NS_FAILED(rv))) {
     return false;
   }
 
@@ -127,23 +113,17 @@ ReadIntoArray(nsIFile* aFile,
   return (bytesRead == length);
 }
 
-bool
-ReadIntoString(nsIFile* aFile,
-               nsCString& aOutDst,
-               size_t aMaxLength)
-{
+bool ReadIntoString(nsIFile* aFile, nsCString& aOutDst, size_t aMaxLength) {
   nsTArray<uint8_t> buf;
   bool rv = ReadIntoArray(aFile, buf, aMaxLength);
   if (rv) {
-    buf.AppendElement(0); // Append null terminator, required by nsC*String.
+    buf.AppendElement(0);  // Append null terminator, required by nsC*String.
     aOutDst = nsDependentCString((const char*)buf.Elements(), buf.Length() - 1);
   }
   return rv;
 }
 
-bool
-GMPInfoFileParser::Init(nsIFile* aInfoFile)
-{
+bool GMPInfoFileParser::Init(nsIFile* aInfoFile) {
   nsTArray<nsCString> lines;
   static const size_t MAX_GMP_INFO_FILE_LENGTH = 5 * 1024;
 
@@ -170,23 +150,23 @@ GMPInfoFileParser::Init(nsIFile* aInfoFile)
     ToLowerCase(key);
     key.Trim(" ");
 
-    nsCString* value = new nsCString(Substring(line, colon + 1));
+    auto value = MakeUnique<nsCString>(Substring(line, colon + 1));
     value->Trim(" ");
-    mValues.Put(key, value); // Hashtable assumes ownership of value.
+    mValues.InsertOrUpdate(
+        key,
+        std::move(value));  // Hashtable assumes ownership of value.
   }
 
   return true;
 }
 
-bool
-GMPInfoFileParser::Contains(const nsCString& aKey) const {
+bool GMPInfoFileParser::Contains(const nsCString& aKey) const {
   nsCString key(aKey);
   ToLowerCase(key);
   return mValues.Contains(key);
 }
 
-nsCString
-GMPInfoFileParser::Get(const nsCString& aKey) const {
+nsCString GMPInfoFileParser::Get(const nsCString& aKey) const {
   MOZ_ASSERT(Contains(aKey));
   nsCString key(aKey);
   ToLowerCase(key);
@@ -194,15 +174,12 @@ GMPInfoFileParser::Get(const nsCString& aKey) const {
   if (mValues.Get(key, &p)) {
     return nsCString(*p);
   }
-  return EmptyCString();
+  return ""_ns;
 }
 
-bool
-HaveGMPFor(const nsCString& aAPI,
-           nsTArray<nsCString>&& aTags)
-{
+bool HaveGMPFor(const nsCString& aAPI, nsTArray<nsCString>&& aTags) {
   nsCOMPtr<mozIGeckoMediaPluginService> mps =
-    do_GetService("@mozilla.org/gecko-media-plugin-service;1");
+      do_GetService("@mozilla.org/gecko-media-plugin-service;1");
   if (NS_WARN_IF(!mps)) {
     return false;
   }
@@ -214,5 +191,38 @@ HaveGMPFor(const nsCString& aAPI,
   return hasPlugin;
 }
 
+void LogToConsole(const nsAString& aMsg) {
+  nsCOMPtr<nsIConsoleService> console(
+      do_GetService("@mozilla.org/consoleservice;1"));
+  if (!console) {
+    NS_WARNING("Failed to log message to console.");
+    return;
+  }
+  nsAutoString msg(aMsg);
+  console->LogStringMessage(msg.get());
+}
 
-} // namespace mozilla
+already_AddRefed<nsISerialEventTarget> GetGMPThread() {
+  RefPtr<gmp::GeckoMediaPluginService> service =
+      gmp::GeckoMediaPluginService::GetGeckoMediaPluginService();
+  nsCOMPtr<nsISerialEventTarget> thread =
+      service ? service->GetGMPThread() : nullptr;
+  return thread.forget();
+}
+
+static size_t Align16(size_t aNumber) {
+  const size_t mask = 15;  // Alignment - 1.
+  return (aNumber + mask) & ~mask;
+}
+
+size_t I420FrameBufferSizePadded(int32_t aWidth, int32_t aHeight) {
+  if (aWidth <= 0 || aHeight <= 0 || aWidth > MAX_VIDEO_WIDTH ||
+      aHeight > MAX_VIDEO_HEIGHT) {
+    return 0;
+  }
+
+  size_t ySize = Align16(aWidth) * Align16(aHeight);
+  return ySize + (ySize / 4) * 2;
+}
+
+}  // namespace mozilla
