@@ -23,14 +23,11 @@
 #  include "MP4Interval.h"
 #  include "ByteStream.h"
 #endif
-#include "nsAutoPtr.h"
 #include "SourceBufferResource.h"
 #include <algorithm>
 
 extern mozilla::LogModule* GetMediaSourceSamplesLog();
 
-#define STRINGIFY(x) #x
-#define TOSTRING(x) STRINGIFY(x)
 #define MSE_DEBUG(arg, ...)                                            \
   DDMOZ_LOG(GetMediaSourceSamplesLog(), mozilla::LogLevel::Debug,      \
             "(%s)::%s: " arg, mType.OriginalString().Data(), __func__, \
@@ -51,25 +48,23 @@ ContainerParser::ContainerParser(const MediaContainerType& aType)
 
 ContainerParser::~ContainerParser() = default;
 
-MediaResult ContainerParser::IsInitSegmentPresent(MediaByteBuffer* aData) {
-  MSE_DEBUG("aLength=%zu [%x%x%x%x]", aData->Length(),
-            aData->Length() > 0 ? (*aData)[0] : 0,
-            aData->Length() > 1 ? (*aData)[1] : 0,
-            aData->Length() > 2 ? (*aData)[2] : 0,
-            aData->Length() > 3 ? (*aData)[3] : 0);
+MediaResult ContainerParser::IsInitSegmentPresent(const MediaSpan& aData) {
+  MSE_DEBUG(
+      "aLength=%zu [%x%x%x%x]", aData.Length(),
+      aData.Length() > 0 ? aData[0] : 0, aData.Length() > 1 ? aData[1] : 0,
+      aData.Length() > 2 ? aData[2] : 0, aData.Length() > 3 ? aData[3] : 0);
   return NS_ERROR_NOT_AVAILABLE;
 }
 
-MediaResult ContainerParser::IsMediaSegmentPresent(MediaByteBuffer* aData) {
-  MSE_DEBUG("aLength=%zu [%x%x%x%x]", aData->Length(),
-            aData->Length() > 0 ? (*aData)[0] : 0,
-            aData->Length() > 1 ? (*aData)[1] : 0,
-            aData->Length() > 2 ? (*aData)[2] : 0,
-            aData->Length() > 3 ? (*aData)[3] : 0);
+MediaResult ContainerParser::IsMediaSegmentPresent(const MediaSpan& aData) {
+  MSE_DEBUG(
+      "aLength=%zu [%x%x%x%x]", aData.Length(),
+      aData.Length() > 0 ? aData[0] : 0, aData.Length() > 1 ? aData[1] : 0,
+      aData.Length() > 2 ? aData[2] : 0, aData.Length() > 3 ? aData[3] : 0);
   return NS_ERROR_NOT_AVAILABLE;
 }
 
-MediaResult ContainerParser::ParseStartAndEndTimestamps(MediaByteBuffer* aData,
+MediaResult ContainerParser::ParseStartAndEndTimestamps(const MediaSpan& aData,
                                                         int64_t& aStart,
                                                         int64_t& aEnd) {
   return NS_ERROR_NOT_AVAILABLE;
@@ -113,44 +108,38 @@ class WebMContainerParser
 
   static const unsigned NS_PER_USEC = 1000;
 
-  MediaResult IsInitSegmentPresent(MediaByteBuffer* aData) override {
+  MediaResult IsInitSegmentPresent(const MediaSpan& aData) override {
     ContainerParser::IsInitSegmentPresent(aData);
-    if (aData->Length() < 4) {
+    if (aData.Length() < 4) {
       return NS_ERROR_NOT_AVAILABLE;
     }
 
     WebMBufferedParser parser(0);
     nsTArray<WebMTimeDataOffset> mapping;
-    ReentrantMonitor dummy("dummy");
-    bool result =
-        parser.Append(aData->Elements(), aData->Length(), mapping, dummy);
-    if (!result) {
-      return MediaResult(NS_ERROR_FAILURE,
-                         RESULT_DETAIL("Invalid webm content"));
+    if (auto result = parser.Append(aData.Elements(), aData.Length(), mapping);
+        NS_FAILED(result)) {
+      return result;
     }
     return parser.mInitEndOffset > 0 ? NS_OK : NS_ERROR_NOT_AVAILABLE;
   }
 
-  MediaResult IsMediaSegmentPresent(MediaByteBuffer* aData) override {
+  MediaResult IsMediaSegmentPresent(const MediaSpan& aData) override {
     ContainerParser::IsMediaSegmentPresent(aData);
-    if (aData->Length() < 4) {
+    if (aData.Length() < 4) {
       return NS_ERROR_NOT_AVAILABLE;
     }
 
     WebMBufferedParser parser(0);
     nsTArray<WebMTimeDataOffset> mapping;
-    ReentrantMonitor dummy("dummy");
     parser.AppendMediaSegmentOnly();
-    bool result =
-        parser.Append(aData->Elements(), aData->Length(), mapping, dummy);
-    if (!result) {
-      return MediaResult(NS_ERROR_FAILURE,
-                         RESULT_DETAIL("Invalid webm content"));
+    if (auto result = parser.Append(aData.Elements(), aData.Length(), mapping);
+        NS_FAILED(result)) {
+      return result;
     }
     return parser.GetClusterOffset() >= 0 ? NS_OK : NS_ERROR_NOT_AVAILABLE;
   }
 
-  MediaResult ParseStartAndEndTimestamps(MediaByteBuffer* aData,
+  MediaResult ParseStartAndEndTimestamps(const MediaSpan& aData,
                                          int64_t& aStart,
                                          int64_t& aEnd) override {
     bool initSegment = NS_SUCCEEDED(IsInitSegmentPresent(aData));
@@ -187,8 +176,10 @@ class WebMContainerParser
     nsTArray<WebMTimeDataOffset> mapping;
     mapping.AppendElements(mOverlappedMapping);
     mOverlappedMapping.Clear();
-    ReentrantMonitor dummy("dummy");
-    mParser.Append(aData->Elements(), aData->Length(), mapping, dummy);
+    if (auto result = mParser.Append(aData.Elements(), aData.Length(), mapping);
+        NS_FAILED(result)) {
+      return result;
+    }
     if (mResource) {
       mResource->AppendData(aData);
     }
@@ -198,7 +189,8 @@ class WebMContainerParser
     // We should be more precise.
     if (initSegment || !HasCompleteInitData()) {
       if (mParser.mInitEndOffset > 0) {
-        MOZ_ASSERT(mParser.mInitEndOffset <= mResource->GetLength());
+        MOZ_DIAGNOSTIC_ASSERT(mInitData && mResource &&
+                              mParser.mInitEndOffset <= mResource->GetLength());
         if (!mInitData->SetLength(mParser.mInitEndOffset, fallible)) {
           // Super unlikely OOM
           return NS_ERROR_OUT_OF_MEMORY;
@@ -214,8 +206,8 @@ class WebMContainerParser
       }
       mHasInitData = true;
     }
-    mOffset += aData->Length();
-    mTotalParsed += aData->Length();
+    mOffset += aData.Length();
+    mTotalParsed += aData.Length();
 
     if (mapping.IsEmpty()) {
       return NS_ERROR_NOT_AVAILABLE;
@@ -329,6 +321,7 @@ class MP4Stream : public ByteStream, public DecoderDoctorLifeLogger<MP4Stream> {
   bool CachedReadAt(int64_t aOffset, void* aBuffer, size_t aCount,
                     size_t* aBytesRead) override;
   bool Length(int64_t* aSize) override;
+  const uint8_t* GetContiguousAccess(int64_t aOffset, size_t aSize) override;
 
  private:
   RefPtr<SourceBufferResource> mResource;
@@ -359,6 +352,10 @@ bool MP4Stream::CachedReadAt(int64_t aOffset, void* aBuffer, size_t aCount,
   return true;
 }
 
+const uint8_t* MP4Stream::GetContiguousAccess(int64_t aOffset, size_t aSize) {
+  return mResource->GetContiguousAccess(aOffset, aSize);
+}
+
 bool MP4Stream::Length(int64_t* aSize) {
   if (mResource->GetLength() < 0) return false;
   *aSize = mResource->GetLength();
@@ -373,12 +370,12 @@ class MP4ContainerParser : public ContainerParser,
   explicit MP4ContainerParser(const MediaContainerType& aType)
       : ContainerParser(aType) {}
 
-  MediaResult IsInitSegmentPresent(MediaByteBuffer* aData) override {
+  MediaResult IsInitSegmentPresent(const MediaSpan& aData) override {
     ContainerParser::IsInitSegmentPresent(aData);
     // Each MP4 atom has a chunk size and chunk type. The root chunk in an MP4
     // file is the 'ftyp' atom followed by a file type. We just check for a
     // vaguely valid 'ftyp' atom.
-    if (aData->Length() < 8) {
+    if (aData.Length() < 8) {
       return NS_ERROR_NOT_AVAILABLE;
     }
     AtomParser parser(*this, aData, AtomParser::StopAt::eInitSegment);
@@ -390,8 +387,8 @@ class MP4ContainerParser : public ContainerParser,
     return parser.StartWithInitSegment() ? NS_OK : NS_ERROR_NOT_AVAILABLE;
   }
 
-  MediaResult IsMediaSegmentPresent(MediaByteBuffer* aData) override {
-    if (aData->Length() < 8) {
+  MediaResult IsMediaSegmentPresent(const MediaSpan& aData) override {
+    if (aData.Length() < 8) {
       return NS_ERROR_NOT_AVAILABLE;
     }
     AtomParser parser(*this, aData, AtomParser::StopAt::eMediaSegment);
@@ -408,13 +405,13 @@ class MP4ContainerParser : public ContainerParser,
    public:
     enum class StopAt { eInitSegment, eMediaSegment, eEnd };
 
-    AtomParser(const MP4ContainerParser& aParser, const MediaByteBuffer* aData,
+    AtomParser(const MP4ContainerParser& aParser, const MediaSpan& aData,
                StopAt aStop = StopAt::eEnd) {
       mValid = Init(aParser, aData, aStop).isOk();
     }
 
     Result<Ok, nsresult> Init(const MP4ContainerParser& aParser,
-                              const MediaByteBuffer* aData, StopAt aStop) {
+                              const MediaSpan& aData, StopAt aStop) {
       const MediaContainerType mType(
           aParser.ContainerType());  // for logging macro.
       BufferReader reader(aData);
@@ -514,7 +511,7 @@ class MP4ContainerParser : public ContainerParser,
   };
 
  public:
-  MediaResult ParseStartAndEndTimestamps(MediaByteBuffer* aData,
+  MediaResult ParseStartAndEndTimestamps(const MediaSpan& aData,
                                          int64_t& aStart,
                                          int64_t& aEnd) override {
     bool initSegment = NS_SUCCEEDED(IsInitSegmentPresent(aData));
@@ -526,8 +523,8 @@ class MP4ContainerParser : public ContainerParser,
       // consumers of ParseStartAndEndTimestamps to add their timestamp offset
       // manually. This allows the ContainerParser to be shared across different
       // timestampOffsets.
-      mParser = new MoofParser(mStream, AsVariant(ParseAllTracks{}),
-                               /* aIsAudio = */ false);
+      mParser = MakeUnique<MoofParser>(mStream, AsVariant(ParseAllTracks{}),
+                                       /* aIsAudio = */ false);
       DDLINKCHILD("parser", mParser.get());
       mInitData = new MediaByteBuffer();
       mCompleteInitSegmentRange = MediaByteRange();
@@ -535,10 +532,13 @@ class MP4ContainerParser : public ContainerParser,
       mCompleteMediaSegmentRange = MediaByteRange();
       mGlobalOffset = mTotalParsed;
     } else if (!mStream || !mParser) {
-      mTotalParsed += aData->Length();
+      mTotalParsed += aData.Length();
       return NS_ERROR_NOT_AVAILABLE;
     }
 
+    MOZ_DIAGNOSTIC_ASSERT(mResource && mParser && mInitData,
+                          "Should have received an init segment first");
+
     mResource->AppendData(aData);
     MediaByteRangeSet byteRanges;
     byteRanges +=
@@ -561,7 +561,7 @@ class MP4ContainerParser : public ContainerParser,
       }
       mHasInitData = true;
     }
-    mTotalParsed += aData->Length();
+    mTotalParsed += aData.Length();
 
     MP4Interval<Microseconds> compositionRange =
         mParser->GetCompositionRange(byteRanges);
@@ -571,13 +571,8 @@ class MP4ContainerParser : public ContainerParser,
     mCompleteMediaSegmentRange =
         mParser->FirstCompleteMediaSegment() + mGlobalOffset;
 
-    ErrorResult rv;
     if (HasCompleteInitData()) {
-      mResource->EvictData(mParser->mOffset, mParser->mOffset, rv);
-    }
-    if (NS_WARN_IF(rv.Failed())) {
-      rv.SuppressException();
-      return NS_ERROR_OUT_OF_MEMORY;
+      mResource->EvictData(mParser->mOffset, mParser->mOffset);
     }
 
     if (compositionRange.IsNull()) {
@@ -595,7 +590,7 @@ class MP4ContainerParser : public ContainerParser,
 
  private:
   RefPtr<MP4Stream> mStream;
-  nsAutoPtr<MoofParser> mParser;
+  UniquePtr<MoofParser> mParser;
 };
 #endif  // MOZ_FMP4
 
@@ -620,35 +615,32 @@ class ADTSContainerParser
   /// Returns true if the header is parsed successfully.
   /// Returns false if the header is invalid or incomplete,
   /// without modifying the passed-in Header object.
-  bool Parse(MediaByteBuffer* aData, Header& header) {
-    MOZ_ASSERT(aData);
-
+  bool Parse(const MediaSpan& aData, Header& header) {
     // ADTS initialization segments are just the packet header.
-    if (aData->Length() < 7) {
+    if (aData.Length() < 7) {
       MSE_DEBUG("buffer too short for header.");
       return false;
     }
     // Check 0xfffx sync word plus layer 0.
-    if (((*aData)[0] != 0xff) || (((*aData)[1] & 0xf6) != 0xf0)) {
+    if ((aData[0] != 0xff) || ((aData[1] & 0xf6) != 0xf0)) {
       MSE_DEBUG("no syncword.");
       return false;
     }
-    bool have_crc = !((*aData)[1] & 0x01);
-    if (have_crc && aData->Length() < 9) {
+    bool have_crc = !(aData[1] & 0x01);
+    if (have_crc && aData.Length() < 9) {
       MSE_DEBUG("buffer too short for header with crc.");
       return false;
     }
-    uint8_t frequency_index = ((*aData)[2] & 0x3c) >> 2;
+    uint8_t frequency_index = (aData[2] & 0x3c) >> 2;
     MOZ_ASSERT(frequency_index < 16);
     if (frequency_index == 15) {
       MSE_DEBUG("explicit frequency disallowed.");
       return false;
     }
     size_t header_length = have_crc ? 9 : 7;
-    size_t data_length = (((*aData)[3] & 0x03) << 11) |
-                         (((*aData)[4] & 0xff) << 3) |
-                         (((*aData)[5] & 0xe0) >> 5);
-    uint8_t frames = ((*aData)[6] & 0x03) + 1;
+    size_t data_length = ((aData[3] & 0x03) << 11) | ((aData[4] & 0xff) << 3) |
+                         ((aData[5] & 0xe0) >> 5);
+    uint8_t frames = (aData[6] & 0x03) + 1;
     MOZ_ASSERT(frames > 0);
     MOZ_ASSERT(frames < 4);
 
@@ -660,7 +652,7 @@ class ADTSContainerParser
     return true;
   }
 
-  MediaResult IsInitSegmentPresent(MediaByteBuffer* aData) override {
+  MediaResult IsInitSegmentPresent(const MediaSpan& aData) override {
     // Call superclass for logging.
     ContainerParser::IsInitSegmentPresent(aData);
 
@@ -676,7 +668,7 @@ class ADTSContainerParser
     return NS_OK;
   }
 
-  MediaResult IsMediaSegmentPresent(MediaByteBuffer* aData) override {
+  MediaResult IsMediaSegmentPresent(const MediaSpan& aData) override {
     // Call superclass for logging.
     ContainerParser::IsMediaSegmentPresent(aData);
 
@@ -692,7 +684,7 @@ class ADTSContainerParser
     // We're supposed to return true as long as aData contains the
     // start of a media segment, whether or not it's complete. So
     // return true if we have any data beyond the header.
-    if (aData->Length() <= header.header_length) {
+    if (aData.Length() <= header.header_length) {
       return NS_ERROR_NOT_AVAILABLE;
     }
 
@@ -700,7 +692,7 @@ class ADTSContainerParser
     return NS_OK;
   }
 
-  MediaResult ParseStartAndEndTimestamps(MediaByteBuffer* aData,
+  MediaResult ParseStartAndEndTimestamps(const MediaSpan& aData,
                                          int64_t& aStart,
                                          int64_t& aEnd) override {
     // ADTS header.
@@ -714,15 +706,15 @@ class ADTSContainerParser
 
     // Cache raw header in case the caller wants a copy.
     mInitData = new MediaByteBuffer(header.header_length);
-    mInitData->AppendElements(aData->Elements(), header.header_length);
+    mInitData->AppendElements(aData.Elements(), header.header_length);
 
     // Check that we have enough data for the frame body.
-    if (aData->Length() < header.frame_length) {
+    if (aData.Length() < header.frame_length) {
       MSE_DEBUGV(
           "Not enough data for %llu byte frame"
           " in %llu byte buffer.",
           (unsigned long long)header.frame_length,
-          (unsigned long long)(aData->Length()));
+          (unsigned long long)(aData.Length()));
       return NS_ERROR_NOT_AVAILABLE;
     }
     mCompleteMediaSegmentRange =
@@ -744,24 +736,24 @@ class ADTSContainerParser
 #endif  // MOZ_FMP4
 
 /*static*/
-ContainerParser* ContainerParser::CreateForMIMEType(
+UniquePtr<ContainerParser> ContainerParser::CreateForMIMEType(
     const MediaContainerType& aType) {
   if (aType.Type() == MEDIAMIMETYPE(VIDEO_WEBM) ||
       aType.Type() == MEDIAMIMETYPE(AUDIO_WEBM)) {
-    return new WebMContainerParser(aType);
+    return MakeUnique<WebMContainerParser>(aType);
   }
 
 #ifdef MOZ_FMP4
   if (aType.Type() == MEDIAMIMETYPE(VIDEO_MP4) ||
       aType.Type() == MEDIAMIMETYPE(AUDIO_MP4)) {
-    return new MP4ContainerParser(aType);
+    return MakeUnique<MP4ContainerParser>(aType);
   }
   if (aType.Type() == MEDIAMIMETYPE("audio/aac")) {
-    return new ADTSContainerParser(aType);
+    return MakeUnique<ADTSContainerParser>(aType);
   }
 #endif
 
-  return new ContainerParser(aType);
+  return MakeUnique<ContainerParser>(aType);
 }
 
 #undef MSE_DEBUG
