@@ -21,9 +21,7 @@
 
 #include "base/thread.h"
 
-namespace mozilla {
-
-namespace camera {
+namespace mozilla::camera {
 
 class CamerasParent;
 
@@ -31,7 +29,10 @@ class CallbackHelper : public rtc::VideoSinkInterface<webrtc::VideoFrame> {
  public:
   CallbackHelper(CaptureEngine aCapEng, uint32_t aStreamId,
                  CamerasParent* aParent)
-      : mCapEngine(aCapEng), mStreamId(aStreamId), mParent(aParent){};
+      : mCapEngine(aCapEng),
+        mStreamId(aStreamId),
+        mTrackingId(CaptureEngineToTrackingSourceStr(aCapEng), aStreamId),
+        mParent(aParent){};
 
   // These callbacks end up running on the VideoCapture thread.
   // From  VideoCaptureCallback
@@ -40,9 +41,10 @@ class CallbackHelper : public rtc::VideoSinkInterface<webrtc::VideoFrame> {
   friend CamerasParent;
 
  private:
-  CaptureEngine mCapEngine;
-  uint32_t mStreamId;
-  CamerasParent* mParent;
+  const CaptureEngine mCapEngine;
+  const uint32_t mStreamId;
+  const TrackingId mTrackingId;
+  CamerasParent* const mParent;
 };
 
 class InputObserver : public webrtc::VideoInputFeedBack {
@@ -58,7 +60,7 @@ class InputObserver : public webrtc::VideoInputFeedBack {
  private:
   ~InputObserver() = default;
 
-  RefPtr<CamerasParent> mParent;
+  const RefPtr<CamerasParent> mParent;
 };
 
 class DeliverFrameRunnable;
@@ -75,16 +77,16 @@ class CamerasParent final : public PCamerasParent,
   // Messages received from the child. These run on the IPC/PBackground thread.
   mozilla::ipc::IPCResult RecvPCamerasConstructor();
   mozilla::ipc::IPCResult RecvAllocateCapture(
-      const CaptureEngine& aEngine, const nsCString& aUnique_idUTF8,
+      const CaptureEngine& aEngine, const nsACString& aUnique_idUTF8,
       const uint64_t& aWindowID) override;
   mozilla::ipc::IPCResult RecvReleaseCapture(const CaptureEngine&,
                                              const int&) override;
   mozilla::ipc::IPCResult RecvNumberOfCaptureDevices(
       const CaptureEngine&) override;
   mozilla::ipc::IPCResult RecvNumberOfCapabilities(const CaptureEngine&,
-                                                   const nsCString&) override;
+                                                   const nsACString&) override;
   mozilla::ipc::IPCResult RecvGetCaptureCapability(const CaptureEngine&,
-                                                   const nsCString&,
+                                                   const nsACString&,
                                                    const int&) override;
   mozilla::ipc::IPCResult RecvGetCaptureDevice(const CaptureEngine&,
                                                const int&) override;
@@ -110,8 +112,9 @@ class CamerasParent final : public PCamerasParent,
 
   // helper to forward to the PBackground thread
   int DeliverFrameOverIPC(CaptureEngine capEng, uint32_t aStreamId,
-                          ShmemBuffer buffer, unsigned char* altbuffer,
-                          VideoFrameProperties& aProps);
+                          const TrackingId& aTrackingId, ShmemBuffer buffer,
+                          unsigned char* altbuffer,
+                          const VideoFrameProperties& aProps);
 
   CamerasParent();
 
@@ -175,7 +178,6 @@ class CamerasParent final : public PCamerasParent,
 
 PCamerasParent* CreateCamerasParent();
 
-}  // namespace camera
-}  // namespace mozilla
+}  // namespace mozilla::camera
 
 #endif  // mozilla_CameraParent_h
