@@ -11,9 +11,14 @@
 #include "modules/video_capture/video_capture_impl.h"
 
 #include <stdlib.h>
+#include <memory>
 #include <string>
 
+#include "CamerasTypes.h"
+#include "PerformanceRecorder.h"
+
 #include "api/video/i420_buffer.h"
+#include "base/scoped_nsautorelease_pool.h"
 #include "common_video/libyuv/include/webrtc_libyuv.h"
 #include "libyuv.h"  // NOLINT
 #include "modules/include/module_common_types.h"
@@ -27,6 +32,10 @@
 #include "modules/desktop_capture/desktop_frame.h"
 #include "modules/desktop_capture/desktop_capture_options.h"
 #include "modules/video_capture/video_capture.h"
+#include "mozilla/Maybe.h"
+#include "mozilla/StaticPrefs_media.h"
+
+#include "PerformanceRecorder.h"
 
 #if defined(_WIN32)
 #  include "platform_uithread.h"
@@ -324,15 +333,48 @@ const char* DesktopCaptureImpl::CurrentDeviceName() const {
   return _deviceUniqueId.c_str();
 }
 
-int32_t DesktopCaptureImpl::Init() {
+static DesktopCaptureOptions CreateDesktopCaptureOptions() {
+  DesktopCaptureOptions options = DesktopCaptureOptions::CreateDefault();
+  // Leave desktop effects enabled during WebRTC captures.
+  options.set_disable_effects(false);
+
+#if defined(WEBRTC_WIN)
+  if (mozilla::StaticPrefs::media_webrtc_capture_allow_directx()) {
+    options.set_allow_directx_capturer(true);
+    options.set_allow_use_magnification_api(false);
+  } else {
+    options.set_allow_use_magnification_api(true);
+  }
+  options.set_allow_cropping_window_capturer(true);
+#  if defined(RTC_ENABLE_WIN_WGC)
+  if (mozilla::StaticPrefs::media_webrtc_capture_allow_wgc()) {
+    options.set_allow_wgc_capturer(true);
+  }
+#  endif
+#endif
+
+#if defined(WEBRTC_MAC)
+  if (mozilla::StaticPrefs::media_webrtc_capture_allow_iosurface()) {
+    options.set_allow_iosurface(true);
+  }
+#endif
+
+#if defined(WEBRTC_USE_PIPEWIRE)
+  if (mozilla::StaticPrefs::media_webrtc_capture_allow_pipewire()) {
+    options.set_allow_pipewire(true);
+  }
+#endif
+
+  return options;
+}
+
+int32_t DesktopCaptureImpl::LazyInitDesktopCapturer() {
   // Already initialized
   if (desktop_capturer_cursor_composer_) {
     return 0;
   }
 
-  DesktopCaptureOptions options = DesktopCaptureOptions::CreateDefault();
-  // Leave desktop effects enabled during WebRTC captures.
-  options.set_disable_effects(false);
+  DesktopCaptureOptions options = CreateDesktopCaptureOptions();
 
   if (_deviceType == CaptureDeviceType::Screen) {
     std::unique_ptr<DesktopCapturer> pScreenCapturer =
@@ -348,6 +390,9 @@ int32_t DesktopCaptureImpl::Init() {
         std::unique_ptr<DesktopAndCursorComposer>(
             new DesktopAndCursorComposer(std::move(pScreenCapturer), options));
   } else if (_deviceType == CaptureDeviceType::Window) {
+#if defined(RTC_ENABLE_WIN_WGC)
+    options.set_allow_wgc_capturer_fallback(true);
+#endif
     std::unique_ptr<DesktopCapturer> pWindowCapturer =
         DesktopCapturer::CreateWindowCapturer(options);
     if (!pWindowCapturer.get()) {
@@ -378,24 +423,27 @@ int32_t DesktopCaptureImpl::Init() {
 DesktopCaptureImpl::DesktopCaptureImpl(const int32_t id, const char* uniqueId,
                                        const CaptureDeviceType type)
     : _id(id),
+      _tracking_id(
+          mozilla::TrackingId(CaptureEngineToTrackingSourceStr([&] {
+                                switch (type) {
+                                  case CaptureDeviceType::Screen:
+                                    return CaptureEngine::ScreenEngine;
+                                  case CaptureDeviceType::Window:
+                                    return CaptureEngine::WinEngine;
+                                  case CaptureDeviceType::Browser:
+                                    return CaptureEngine::BrowserEngine;
+                                  default:
+                                    return CaptureEngine::InvalidEngine;
+                                }
+                              }()),
+                              id)),
       _deviceUniqueId(uniqueId),
       _deviceType(type),
       _requestedCapability(),
       _rotateFrame(kVideoRotation_0),
       last_capture_time_ms_(rtc::TimeMillis()),
       time_event_(EventWrapper::Create()),
-#if defined(_WIN32)
-      capturer_thread_(
-          new rtc::PlatformUIThread(Run, this, "ScreenCaptureThread")),
-#else
-#  if defined(WEBRTC_LINUX)
-      // We start the thread lazily for Linux
       capturer_thread_(nullptr),
-#  else
-      capturer_thread_(
-          new rtc::PlatformThread(Run, this, "ScreenCaptureThread")),
-#  endif
-#endif
       started_(false) {
   _requestedCapability.width = kDefaultWidth;
   _requestedCapability.height = kDefaultHeight;
@@ -408,7 +456,11 @@ DesktopCaptureImpl::DesktopCaptureImpl(const int32_t id, const char* uniqueId,
 DesktopCaptureImpl::~DesktopCaptureImpl() {
   time_event_->Set();
   if (capturer_thread_) {
+#if defined(_WIN32)
     capturer_thread_->Stop();
+#else
+    capturer_thread_->Finalize();
+#endif
   }
 }
 
@@ -481,6 +533,8 @@ int32_t DesktopCaptureImpl::IncomingFrame(
   // In Windows, the image starts bottom left, instead of top left.
   // Setting a negative source height, inverts the image (within LibYuv).
 
+  mozilla::PerformanceRecorder<mozilla::CopyVideoStage> rec(
+      "DesktopCaptureImpl::ConvertToI420"_ns, _tracking_id, width, abs(height));
   // TODO(nisse): Use a pool?
   rtc::scoped_refptr<I420Buffer> buffer =
       I420Buffer::Create(width, abs(height), stride_y, stride_uv, stride_uv);
@@ -497,6 +551,7 @@ int32_t DesktopCaptureImpl::IncomingFrame(
                       << static_cast<int>(frameInfo.videoType) << "to I420.";
     return -1;
   }
+  rec.Record();
 
   VideoFrame captureFrame(buffer, 0, rtc::TimeMillis(), kVideoRotation_0);
 
@@ -558,43 +613,51 @@ uint32_t DesktopCaptureImpl::CalculateFrameRate(int64_t now_ns) {
   return nrOfFrames;
 }
 
+void DesktopCaptureImpl::LazyInitCaptureThread() {
+  MOZ_ASSERT(desktop_capturer_cursor_composer_,
+             "DesktopCapturer must be initialized before the capture thread");
+  if (capturer_thread_) {
+    return;
+  }
+#if defined(_WIN32)
+  capturer_thread_ = std::make_unique<rtc::PlatformUIThread>(
+      std::function([self = (void*)this]() { Run(self); }),
+      "ScreenCaptureThread", rtc::ThreadAttributes{});
+  capturer_thread_->RequestCallbackTimer(_maxFPSNeeded);
+#else
+  auto self = rtc::scoped_refptr<DesktopCaptureImpl>(this);
+  capturer_thread_ =
+      std::make_unique<rtc::PlatformThread>(rtc::PlatformThread::SpawnJoinable(
+          [self] { self->process(); }, "ScreenCaptureThread"));
+#endif
+  started_ = true;
+}
+
 int32_t DesktopCaptureImpl::StartCapture(
     const VideoCaptureCapability& capability) {
   rtc::CritScope lock(&_apiCs);
+  // See Bug 1780884 for followup on understanding why multiple calls happen.
+  // MOZ_ASSERT(!started_, "Capture must be stopped before Start() can be
+  // called");
+  if (started_) {
+    return 0;
+  }
 
+  if (uint32_t err = LazyInitDesktopCapturer(); err) {
+    return err;
+  }
+  started_ = true;
   _requestedCapability = capability;
   _maxFPSNeeded = _requestedCapability.maxFPS > 0
                       ? 1000 / _requestedCapability.maxFPS
                       : 1000;
-#if defined(_WIN32)
-  capturer_thread_->RequestCallbackTimer(_maxFPSNeeded);
-#endif
-
-  if (started_) {
-    return 0;
-  }
-#if defined(WEBRTC_LINUX)
-  // Lazily init capturer_thread_
-  if (!capturer_thread_) {
-    capturer_thread_ = std::unique_ptr<rtc::PlatformThread>(
-        new rtc::PlatformThread(Run, this, "ScreenCaptureThread"));
-  }
-#endif
-
-  uint32_t err = Init();
-  if (err) {
-    return err;
-  }
-
-  capturer_thread_->Start();
-  started_ = true;
+  LazyInitCaptureThread();
 
   return 0;
 }
 
 bool DesktopCaptureImpl::FocusOnSelectedSource() {
-  uint32_t err = Init();
-  if (err) {
+  if (uint32_t err = LazyInitDesktopCapturer(); err) {
     return false;
   }
 
@@ -604,9 +667,18 @@ bool DesktopCaptureImpl::FocusOnSelectedSource() {
 int32_t DesktopCaptureImpl::StopCapture() {
   if (started_) {
     started_ = false;
+    MOZ_ASSERT(capturer_thread_, "Capturer thread should be initialized.");
+
+#if defined(_WIN32)
     capturer_thread_
         ->Stop();  // thread is guaranteed stopped before this returns
+#else
+    capturer_thread_
+        ->Finalize();  // thread is guaranteed stopped before this returns
+#endif
     desktop_capturer_cursor_composer_.reset();
+    cursor_composer_started_ = false;
+    capturer_thread_.reset();
     return 0;
   }
   return -1;
@@ -635,41 +707,54 @@ void DesktopCaptureImpl::OnCaptureResult(DesktopCapturer::Result result,
 
 void DesktopCaptureImpl::process() {
   // We need to call Start on the same thread we call CaptureFrame on.
-  desktop_capturer_cursor_composer_->Start(this);
-
-  // We should deliver at least one frame before stopping
+  if (!cursor_composer_started_) {
+    desktop_capturer_cursor_composer_->Start(this);
+    cursor_composer_started_ = true;
+  }
+#if defined(WEBRTC_WIN)
+  ProcessIter();
+#else
   do {
-#if !defined(_WIN32)
-    int64_t startProcessTime = rtc::TimeNanos();
+    ProcessIter();
+  } while (started_);
 #endif
+}
+
+void DesktopCaptureImpl::ProcessIter() {
+// We should deliver at least one frame before stopping
+#if !defined(_WIN32)
+  int64_t startProcessTime = rtc::TimeNanos();
+#endif
+
+  // Don't leak while we're looping
+  base::ScopedNSAutoreleasePool autoreleasepool;
 
 #if defined(WEBRTC_MAC)
-    // Give cycles to the RunLoop so frame callbacks can happen
-    CFRunLoopRunInMode(kCFRunLoopDefaultMode, 0.01, true);
+  // Give cycles to the RunLoop so frame callbacks can happen
+  CFRunLoopRunInMode(kCFRunLoopDefaultMode, 0.01, true);
 #endif
 
-    desktop_capturer_cursor_composer_->CaptureFrame();
+  desktop_capturer_cursor_composer_->CaptureFrame();
 
 #if !defined(_WIN32)
-    const uint32_t processTime =
-        ((uint32_t)(rtc::TimeNanos() - startProcessTime)) /
-        rtc::kNumNanosecsPerMillisec;
-    // Use at most x% CPU or limit framerate
-    const float sleepTimeFactor = (100.0f / kMaxDesktopCaptureCpuUsage) - 1.0f;
-    const uint32_t sleepTime = sleepTimeFactor * processTime;
-    time_event_->Wait(std::max<uint32_t>(_maxFPSNeeded, sleepTime));
+  const uint32_t processTime =
+      ((uint32_t)(rtc::TimeNanos() - startProcessTime)) /
+      rtc::kNumNanosecsPerMillisec;
+  // Use at most x% CPU or limit framerate
+  const float sleepTimeFactor = (100.0f / kMaxDesktopCaptureCpuUsage) - 1.0f;
+  const uint32_t sleepTime = sleepTimeFactor * processTime;
+  time_event_->Wait(std::max<uint32_t>(_maxFPSNeeded, sleepTime));
 #endif
 
 #if defined(WEBRTC_WIN)
-    // Alertable sleep to permit RaiseFlag to run and update |stop_|.
-    SleepEx(0, true);
+// Let the timer events in PlatformUIThread drive process,
+// don't sleep.
 #elif defined(WEBRTC_MAC)
-    sched_yield();
+  sched_yield();
 #else
-    static const struct timespec ts_null = {0};
-    nanosleep(&ts_null, nullptr);
+  static const struct timespec ts_null = {0};
+  nanosleep(&ts_null, nullptr);
 #endif
-  } while (started_);
 }
 
 }  // namespace webrtc
