@@ -9,38 +9,31 @@
 #include "AudioBuffer.h"
 #include "AudioDestinationNode.h"
 #include "AudioNodeEngine.h"
-#include "AudioNodeStream.h"
+#include "AudioNodeTrack.h"
 #include "AudioProcessingEvent.h"
 #include "WebAudioUtils.h"
 #include "mozilla/dom/ScriptSettings.h"
 #include "mozilla/Mutex.h"
 #include "mozilla/PodOperations.h"
-#include "nsAutoPtr.h"
 #include <deque>
+#include "Tracing.h"
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 // The maximum latency, in seconds, that we can live with before dropping
 // buffers.
 static const float MAX_LATENCY_S = 0.5;
 
-NS_IMPL_ISUPPORTS_INHERITED0(ScriptProcessorNode, AudioNode)
-
 // This class manages a queue of output buffers shared between
-// the main thread and the Media Stream Graph thread.
-class SharedBuffers final
-{
-private:
-  class OutputQueue final
-  {
-  public:
-    explicit OutputQueue(const char* aName)
-      : mMutex(aName)
-    {}
+// the main thread and the Media Track Graph thread.
+class SharedBuffers final {
+ private:
+  class OutputQueue final {
+   public:
+    explicit OutputQueue(const char* aName) : mMutex(aName) {}
 
     size_t SizeOfExcludingThis(MallocSizeOf aMallocSizeOf) const
-    {
+        MOZ_REQUIRES(mMutex) {
       mMutex.AssertCurrentThreadOwns();
 
       size_t amount = 0;
@@ -51,18 +44,18 @@ private:
       return amount;
     }
 
-    Mutex& Lock() const { return const_cast<OutputQueue*>(this)->mMutex; }
+    Mutex& Lock() const MOZ_RETURN_CAPABILITY(mMutex) {
+      return const_cast<OutputQueue*>(this)->mMutex;
+    }
 
-    size_t ReadyToConsume() const
-    {
+    size_t ReadyToConsume() const MOZ_REQUIRES(mMutex) {
       // Accessed on both main thread and media graph thread.
       mMutex.AssertCurrentThreadOwns();
       return mBufferList.size();
     }
 
     // Produce one buffer
-    AudioChunk& Produce()
-    {
+    AudioChunk& Produce() MOZ_REQUIRES(mMutex) {
       mMutex.AssertCurrentThreadOwns();
       MOZ_ASSERT(NS_IsMainThread());
       mBufferList.push_back(AudioChunk());
@@ -70,8 +63,7 @@ private:
     }
 
     // Consumes one buffer.
-    AudioChunk Consume()
-    {
+    AudioChunk Consume() MOZ_REQUIRES(mMutex) {
       mMutex.AssertCurrentThreadOwns();
       MOZ_ASSERT(!NS_IsMainThread());
       MOZ_ASSERT(ReadyToConsume() > 0);
@@ -81,35 +73,31 @@ private:
     }
 
     // Empties the buffer queue.
-    void Clear()
-    {
+    void Clear() MOZ_REQUIRES(mMutex) {
       mMutex.AssertCurrentThreadOwns();
       mBufferList.clear();
     }
 
-  private:
+   private:
     typedef std::deque<AudioChunk> BufferList;
 
     // Synchronizes access to mBufferList.  Note that it's the responsibility
     // of the callers to perform the required locking, and we assert that every
     // time we access mBufferList.
-    Mutex mMutex;
+    Mutex mMutex MOZ_UNANNOTATED;
     // The list representing the queue.
     BufferList mBufferList;
   };
 
-public:
+ public:
   explicit SharedBuffers(float aSampleRate)
-    : mOutputQueue("SharedBuffers::outputQueue")
-    , mDelaySoFar(STREAM_TIME_MAX)
-    , mSampleRate(aSampleRate)
-    , mLatency(0.0)
-    , mDroppingBuffers(false)
-  {
-  }
+      : mOutputQueue("SharedBuffers::outputQueue"),
+        mDelaySoFar(TRACK_TIME_MAX),
+        mSampleRate(aSampleRate),
+        mLatency(0.0),
+        mDroppingBuffers(false) {}
 
-  size_t SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const
-  {
+  size_t SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const {
     size_t amount = aMallocSizeOf(this);
 
     {
@@ -121,10 +109,31 @@ public:
   }
 
   // main thread
-  void FinishProducingOutputBuffer(ThreadSharedFloatArrayBufferList* aBuffer,
-                                   uint32_t aBufferSize)
-  {
+
+  // NotifyNodeIsConnected() may be called even when the state has not
+  // changed.
+  void NotifyNodeIsConnected(bool aIsConnected) {
     MOZ_ASSERT(NS_IsMainThread());
+    if (!aIsConnected) {
+      // Reset main thread state for FinishProducingOutputBuffer().
+      mLatency = 0.0f;
+      mLastEventTime = TimeStamp();
+      mDroppingBuffers = false;
+      // Don't flush the output buffer here because the graph thread may be
+      // using it now.  The graph thread will flush when it knows it is
+      // disconnected.
+    }
+    mNodeIsConnected = aIsConnected;
+  }
+
+  void FinishProducingOutputBuffer(const AudioChunk& aBuffer) {
+    MOZ_ASSERT(NS_IsMainThread());
+
+    if (!mNodeIsConnected) {
+      // The output buffer is not used, and mLastEventTime will not be
+      // initialized until the node is re-connected.
+      return;
+    }
 
     TimeStamp now = TimeStamp::Now();
 
@@ -142,7 +151,7 @@ public:
       // MAX_LATENCY_S, frame dropping starts again to maintain an as short
       // output queue as possible.
       float latency = (now - mLastEventTime).ToSeconds();
-      float bufferDuration = aBufferSize / mSampleRate;
+      float bufferDuration = aBuffer.mDuration / mSampleRate;
       mLatency += latency - bufferDuration;
       mLastEventTime = now;
       if (fabs(mLatency) > MAX_LATENCY_S) {
@@ -159,40 +168,31 @@ public:
       mLatency = 0;
     }
 
-    for (uint32_t offset = 0; offset < aBufferSize; offset += WEBAUDIO_BLOCK_SIZE) {
+    for (uint32_t offset = 0; offset < aBuffer.mDuration;
+         offset += WEBAUDIO_BLOCK_SIZE) {
       AudioChunk& chunk = mOutputQueue.Produce();
-      if (aBuffer) {
-        chunk.mDuration = WEBAUDIO_BLOCK_SIZE;
-        chunk.mBuffer = aBuffer;
-        chunk.mChannelData.SetLength(aBuffer->GetChannels());
-        for (uint32_t i = 0; i < aBuffer->GetChannels(); ++i) {
-          chunk.mChannelData[i] = aBuffer->GetData(i) + offset;
-        }
-        chunk.mVolume = 1.0f;
-        chunk.mBufferFormat = AUDIO_FORMAT_FLOAT32;
-      } else {
-        chunk.SetNull(WEBAUDIO_BLOCK_SIZE);
-      }
+      chunk = aBuffer;
+      chunk.SliceTo(offset, offset + WEBAUDIO_BLOCK_SIZE);
     }
   }
 
   // graph thread
-  AudioChunk GetOutputBuffer()
-  {
+
+  AudioChunk GetOutputBuffer() {
     MOZ_ASSERT(!NS_IsMainThread());
     AudioChunk buffer;
 
     {
       MutexAutoLock lock(mOutputQueue.Lock());
       if (mOutputQueue.ReadyToConsume() > 0) {
-        if (mDelaySoFar == STREAM_TIME_MAX) {
+        if (mDelaySoFar == TRACK_TIME_MAX) {
           mDelaySoFar = 0;
         }
         buffer = mOutputQueue.Consume();
       } else {
         // If we're out of buffers to consume, just output silence
         buffer.SetNull(WEBAUDIO_BLOCK_SIZE);
-        if (mDelaySoFar != STREAM_TIME_MAX) {
+        if (mDelaySoFar != TRACK_TIME_MAX) {
           // Remember the delay that we just hit
           mDelaySoFar += WEBAUDIO_BLOCK_SIZE;
         }
@@ -202,32 +202,30 @@ public:
     return buffer;
   }
 
-  StreamTime DelaySoFar() const
-  {
+  TrackTime DelaySoFar() const {
     MOZ_ASSERT(!NS_IsMainThread());
-    return mDelaySoFar == STREAM_TIME_MAX ? 0 : mDelaySoFar;
+    return mDelaySoFar == TRACK_TIME_MAX ? 0 : mDelaySoFar;
   }
 
-  void Reset()
-  {
+  void Flush() {
     MOZ_ASSERT(!NS_IsMainThread());
-    mDelaySoFar = STREAM_TIME_MAX;
-    mLatency = 0.0f;
+    mDelaySoFar = TRACK_TIME_MAX;
     {
       MutexAutoLock lock(mOutputQueue.Lock());
       mOutputQueue.Clear();
     }
-    mLastEventTime = TimeStamp();
   }
 
-private:
+ private:
   OutputQueue mOutputQueue;
   // How much delay we've seen so far.  This measures the amount of delay
   // caused by the main thread lagging behind in producing output buffers.
-  // STREAM_TIME_MAX means that we have not received our first buffer yet.
-  StreamTime mDelaySoFar;
+  // TRACK_TIME_MAX means that we have not received our first buffer yet.
+  // Graph thread only.
+  TrackTime mDelaySoFar;
   // The samplerate of the context.
-  float mSampleRate;
+  const float mSampleRate;
+  // The remaining members are main thread only.
   // This is the latency caused by the buffering. If this grows too high, we
   // will drop buffers until it is acceptable.
   float mLatency;
@@ -236,64 +234,58 @@ private:
   TimeStamp mLastEventTime;
   // True if we should be dropping buffers.
   bool mDroppingBuffers;
+  // True iff the AudioNode has at least one input or output connected.
+  bool mNodeIsConnected;
 };
 
-class ScriptProcessorNodeEngine final : public AudioNodeEngine
-{
-public:
+class ScriptProcessorNodeEngine final : public AudioNodeEngine {
+ public:
   ScriptProcessorNodeEngine(ScriptProcessorNode* aNode,
                             AudioDestinationNode* aDestination,
                             uint32_t aBufferSize,
                             uint32_t aNumberOfInputChannels)
-    : AudioNodeEngine(aNode)
-    , mDestination(aDestination->Stream())
-    , mSharedBuffers(new SharedBuffers(mDestination->SampleRate()))
-    , mBufferSize(aBufferSize)
-    , mInputChannelCount(aNumberOfInputChannels)
-    , mInputWriteIndex(0)
-  {
-  }
+      : AudioNodeEngine(aNode),
+        mDestination(aDestination->Track()),
+        mSharedBuffers(new SharedBuffers(mDestination->mSampleRate)),
+        mBufferSize(aBufferSize),
+        mInputChannelCount(aNumberOfInputChannels),
+        mInputWriteIndex(0) {}
 
-  SharedBuffers* GetSharedBuffers() const
-  {
-    return mSharedBuffers;
-  }
+  SharedBuffers* GetSharedBuffers() const { return mSharedBuffers.get(); }
 
   enum {
     IS_CONNECTED,
   };
 
-  void SetInt32Parameter(uint32_t aIndex, int32_t aParam) override
-  {
+  void SetInt32Parameter(uint32_t aIndex, int32_t aParam) override {
     switch (aIndex) {
       case IS_CONNECTED:
         mIsConnected = aParam;
         break;
       default:
         NS_ERROR("Bad Int32Parameter");
-    } // End index switch.
+    }  // End index switch.
   }
 
-  void ProcessBlock(AudioNodeStream* aStream,
-                    GraphTime aFrom,
-                    const AudioBlock& aInput,
-                    AudioBlock* aOutput,
-                    bool* aFinished) override
-  {
+  void ProcessBlock(AudioNodeTrack* aTrack, GraphTime aFrom,
+                    const AudioBlock& aInput, AudioBlock* aOutput,
+                    bool* aFinished) override {
+    TRACE("ScriptProcessorNodeEngine::ProcessBlock");
+
     // This node is not connected to anything. Per spec, we don't fire the
     // onaudioprocess event. We also want to clear out the input and output
     // buffer queue, and output a null buffer.
     if (!mIsConnected) {
       aOutput->SetNull(WEBAUDIO_BLOCK_SIZE);
-      mSharedBuffers->Reset();
+      mSharedBuffers->Flush();
       mInputWriteIndex = 0;
       return;
     }
 
     // The input buffer is allocated lazily when non-null input is received.
     if (!aInput.IsNull() && !mInputBuffer) {
-      mInputBuffer = ThreadSharedFloatArrayBufferList::
-        Create(mInputChannelCount, mBufferSize, fallible);
+      mInputBuffer = ThreadSharedFloatArrayBufferList::Create(
+          mInputChannelCount, mBufferSize, fallible);
       if (mInputBuffer && mInputWriteIndex) {
         // Zero leading for null chunks that were skipped.
         for (uint32_t i = 0; i < mInputChannelCount; ++i) {
@@ -312,8 +304,9 @@ public:
       } else {
         MOZ_ASSERT(aInput.GetDuration() == WEBAUDIO_BLOCK_SIZE, "sanity check");
         MOZ_ASSERT(aInput.ChannelCount() == inputChannelCount);
-        AudioBlockCopyChannelWithScale(static_cast<const float*>(aInput.mChannelData[i]),
-                                       aInput.mVolume, writeData);
+        AudioBlockCopyChannelWithScale(
+            static_cast<const float*>(aInput.mChannelData[i]), aInput.mVolume,
+            writeData);
       }
     }
     mInputWriteIndex += aInput.GetDuration();
@@ -324,21 +317,19 @@ public:
     *aOutput = mSharedBuffers->GetOutputBuffer();
 
     if (mInputWriteIndex >= mBufferSize) {
-      SendBuffersToMainThread(aStream, aFrom);
+      SendBuffersToMainThread(aTrack, aFrom);
       mInputWriteIndex -= mBufferSize;
     }
   }
 
-  bool IsActive() const override
-  {
+  bool IsActive() const override {
     // Could return false when !mIsConnected after all output chunks produced
     // by main thread events calling
     // SharedBuffers::FinishProducingOutputBuffer() have been processed.
     return true;
   }
 
-  size_t SizeOfExcludingThis(MallocSizeOf aMallocSizeOf) const override
-  {
+  size_t SizeOfExcludingThis(MallocSizeOf aMallocSizeOf) const override {
     // Not owned:
     // - mDestination (probably)
     size_t amount = AudioNodeEngine::SizeOfExcludingThis(aMallocSizeOf);
@@ -350,75 +341,67 @@ public:
     return amount;
   }
 
-  size_t SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const override
-  {
+  size_t SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const override {
     return aMallocSizeOf(this) + SizeOfExcludingThis(aMallocSizeOf);
   }
 
-private:
-  void SendBuffersToMainThread(AudioNodeStream* aStream, GraphTime aFrom)
-  {
+ private:
+  void SendBuffersToMainThread(AudioNodeTrack* aTrack, GraphTime aFrom) {
     MOZ_ASSERT(!NS_IsMainThread());
 
     // we now have a full input buffer ready to be sent to the main thread.
-    StreamTime playbackTick = mDestination->GraphTimeToStreamTime(aFrom);
+    TrackTime playbackTick = mDestination->GraphTimeToTrackTime(aFrom);
     // Add the duration of the current sample
     playbackTick += WEBAUDIO_BLOCK_SIZE;
     // Add the delay caused by the main thread
     playbackTick += mSharedBuffers->DelaySoFar();
     // Compute the playback time in the coordinate system of the destination
-    double playbackTime = mDestination->StreamTimeToSeconds(playbackTick);
+    double playbackTime = mDestination->TrackTimeToSeconds(playbackTick);
 
-    class Command final : public Runnable
-    {
-    public:
-      Command(AudioNodeStream* aStream,
+    class Command final : public Runnable {
+     public:
+      Command(AudioNodeTrack* aTrack,
               already_AddRefed<ThreadSharedFloatArrayBufferList> aInputBuffer,
               double aPlaybackTime)
-        : mStream(aStream)
-        , mInputBuffer(aInputBuffer)
-        , mPlaybackTime(aPlaybackTime)
-      {
-      }
+          : mozilla::Runnable("Command"),
+            mTrack(aTrack),
+            mInputBuffer(aInputBuffer),
+            mPlaybackTime(aPlaybackTime) {}
 
-      NS_IMETHOD Run() override
-      {
-        RefPtr<ThreadSharedFloatArrayBufferList> output;
-
-        auto engine =
-          static_cast<ScriptProcessorNodeEngine*>(mStream->Engine());
+      NS_IMETHOD Run() override {
+        auto engine = static_cast<ScriptProcessorNodeEngine*>(mTrack->Engine());
+        AudioChunk output;
+        output.SetNull(engine->mBufferSize);
         {
-          auto node = static_cast<ScriptProcessorNode*>
-            (engine->NodeMainThread());
+          auto node =
+              static_cast<ScriptProcessorNode*>(engine->NodeMainThread());
           if (!node) {
             return NS_OK;
           }
 
           if (node->HasListenersFor(nsGkAtoms::onaudioprocess)) {
-            output = DispatchAudioProcessEvent(node);
+            DispatchAudioProcessEvent(node, &output);
           }
           // The node may have been destroyed during event dispatch.
         }
 
         // Append it to our output buffer queue
-        engine->GetSharedBuffers()->
-          FinishProducingOutputBuffer(output, engine->mBufferSize);
+        engine->GetSharedBuffers()->FinishProducingOutputBuffer(output);
 
         return NS_OK;
       }
 
-      // Returns the output buffers if set in event handlers.
-      ThreadSharedFloatArrayBufferList*
-        DispatchAudioProcessEvent(ScriptProcessorNode* aNode)
-      {
+      // Sets up |output| iff buffers are set in event handlers.
+      void DispatchAudioProcessEvent(ScriptProcessorNode* aNode,
+                                     AudioChunk* aOutput) {
         AudioContext* context = aNode->Context();
         if (!context) {
-          return nullptr;
+          return;
         }
 
         AutoJSAPI jsapi;
         if (NS_WARN_IF(!jsapi.Init(aNode->GetOwner()))) {
-          return nullptr;
+          return;
         }
         JSContext* cx = jsapi.cx();
         uint32_t inputChannelCount = aNode->ChannelCount();
@@ -427,23 +410,23 @@ private:
         RefPtr<AudioBuffer> inputBuffer;
         if (mInputBuffer) {
           ErrorResult rv;
-          inputBuffer =
-            AudioBuffer::Create(context->GetOwner(), inputChannelCount,
-                                aNode->BufferSize(), context->SampleRate(),
-                                mInputBuffer.forget(), rv);
+          inputBuffer = AudioBuffer::Create(
+              context->GetOwner(), inputChannelCount, aNode->BufferSize(),
+              context->SampleRate(), mInputBuffer.forget(), rv);
           if (rv.Failed()) {
             rv.SuppressException();
-            return nullptr;
+            return;
           }
         }
 
         // Ask content to produce data in the output buffer
-        // Note that we always avoid creating the output buffer here, and we try to
-        // avoid creating the input buffer as well.  The AudioProcessingEvent class
-        // knows how to lazily create them if needed once the script tries to access
-        // them.  Otherwise, we may be able to get away without creating them!
+        // Note that we always avoid creating the output buffer here, and we try
+        // to avoid creating the input buffer as well.  The AudioProcessingEvent
+        // class knows how to lazily create them if needed once the script tries
+        // to access them.  Otherwise, we may be able to get away without
+        // creating them!
         RefPtr<AudioProcessingEvent> event =
-          new AudioProcessingEvent(aNode, nullptr, nullptr);
+            new AudioProcessingEvent(aNode, nullptr, nullptr);
         event->InitEvent(inputBuffer, inputChannelCount, mPlaybackTime);
         aNode->DispatchTrustedEvent(event);
 
@@ -457,25 +440,28 @@ private:
           // HasOutputBuffer() returning true means that GetOutputBuffer()
           // will not fail.
           MOZ_ASSERT(!rv.Failed());
-          return buffer->GetThreadSharedChannelsForRate(cx);
+          *aOutput = buffer->GetThreadSharedChannelsForRate(cx);
+          MOZ_ASSERT(aOutput->IsNull() ||
+                         aOutput->mBufferFormat == AUDIO_FORMAT_FLOAT32,
+                     "AudioBuffers initialized from JS have float data");
         }
-
-        return nullptr;
       }
-    private:
-      RefPtr<AudioNodeStream> mStream;
+
+     private:
+      RefPtr<AudioNodeTrack> mTrack;
       RefPtr<ThreadSharedFloatArrayBufferList> mInputBuffer;
       double mPlaybackTime;
     };
 
-    NS_DispatchToMainThread(new Command(aStream, mInputBuffer.forget(),
-                                        playbackTime));
+    RefPtr<Command> command =
+        new Command(aTrack, mInputBuffer.forget(), playbackTime);
+    mAbstractMainThread->Dispatch(command.forget());
   }
 
   friend class ScriptProcessorNode;
 
-  AudioNodeStream* mDestination;
-  nsAutoPtr<SharedBuffers> mSharedBuffers;
+  RefPtr<AudioNodeTrack> mDestination;
+  UniquePtr<SharedBuffers> mSharedBuffers;
   RefPtr<ThreadSharedFloatArrayBufferList> mInputBuffer;
   const uint32_t mBufferSize;
   const uint32_t mInputChannelCount;
@@ -488,86 +474,76 @@ ScriptProcessorNode::ScriptProcessorNode(AudioContext* aContext,
                                          uint32_t aBufferSize,
                                          uint32_t aNumberOfInputChannels,
                                          uint32_t aNumberOfOutputChannels)
-  : AudioNode(aContext,
-              aNumberOfInputChannels,
-              mozilla::dom::ChannelCountMode::Explicit,
-              mozilla::dom::ChannelInterpretation::Speakers)
-  , mBufferSize(aBufferSize ?
-                  aBufferSize : // respect what the web developer requested
-                  4096)         // choose our own buffer size -- 4KB for now
-  , mNumberOfOutputChannels(aNumberOfOutputChannels)
-{
+    : AudioNode(aContext, aNumberOfInputChannels,
+                mozilla::dom::ChannelCountMode::Explicit,
+                mozilla::dom::ChannelInterpretation::Speakers),
+      mBufferSize(aBufferSize ? aBufferSize
+                              :  // respect what the web developer requested
+                      4096)      // choose our own buffer size -- 4KB for now
+      ,
+      mNumberOfOutputChannels(aNumberOfOutputChannels) {
   MOZ_ASSERT(BufferSize() % WEBAUDIO_BLOCK_SIZE == 0, "Invalid buffer size");
-  ScriptProcessorNodeEngine* engine =
-    new ScriptProcessorNodeEngine(this,
-                                  aContext->Destination(),
-                                  BufferSize(),
-                                  aNumberOfInputChannels);
-  mStream = AudioNodeStream::Create(aContext, engine,
-                                    AudioNodeStream::NO_STREAM_FLAGS,
-                                    aContext->Graph());
+  ScriptProcessorNodeEngine* engine = new ScriptProcessorNodeEngine(
+      this, aContext->Destination(), BufferSize(), aNumberOfInputChannels);
+  mTrack = AudioNodeTrack::Create(
+      aContext, engine, AudioNodeTrack::NO_TRACK_FLAGS, aContext->Graph());
 }
 
-ScriptProcessorNode::~ScriptProcessorNode()
-{
-}
+ScriptProcessorNode::~ScriptProcessorNode() = default;
 
-size_t
-ScriptProcessorNode::SizeOfExcludingThis(MallocSizeOf aMallocSizeOf) const
-{
+size_t ScriptProcessorNode::SizeOfExcludingThis(
+    MallocSizeOf aMallocSizeOf) const {
   size_t amount = AudioNode::SizeOfExcludingThis(aMallocSizeOf);
   return amount;
 }
 
-size_t
-ScriptProcessorNode::SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const
-{
+size_t ScriptProcessorNode::SizeOfIncludingThis(
+    MallocSizeOf aMallocSizeOf) const {
   return aMallocSizeOf(this) + SizeOfExcludingThis(aMallocSizeOf);
 }
 
-void
-ScriptProcessorNode::EventListenerAdded(nsIAtom* aType)
-{
+void ScriptProcessorNode::EventListenerAdded(nsAtom* aType) {
   AudioNode::EventListenerAdded(aType);
   if (aType == nsGkAtoms::onaudioprocess) {
     UpdateConnectedStatus();
   }
 }
 
-void
-ScriptProcessorNode::EventListenerRemoved(nsIAtom* aType)
-{
+void ScriptProcessorNode::EventListenerRemoved(nsAtom* aType) {
   AudioNode::EventListenerRemoved(aType);
-  if (aType == nsGkAtoms::onaudioprocess) {
+  if (aType == nsGkAtoms::onaudioprocess && mTrack) {
     UpdateConnectedStatus();
   }
 }
 
-JSObject*
-ScriptProcessorNode::WrapObject(JSContext* aCx, JS::Handle<JSObject*> aGivenProto)
-{
-  return ScriptProcessorNodeBinding::Wrap(aCx, this, aGivenProto);
+JSObject* ScriptProcessorNode::WrapObject(JSContext* aCx,
+                                          JS::Handle<JSObject*> aGivenProto) {
+  return ScriptProcessorNode_Binding::Wrap(aCx, this, aGivenProto);
 }
 
-void
-ScriptProcessorNode::UpdateConnectedStatus()
-{
-  bool isConnected = mHasPhantomInput ||
-    !(OutputNodes().IsEmpty() && OutputParams().IsEmpty()
-      && InputNodes().IsEmpty());
+void ScriptProcessorNode::UpdateConnectedStatus() {
+  bool isConnected =
+      mHasPhantomInput || !(OutputNodes().IsEmpty() &&
+                            OutputParams().IsEmpty() && InputNodes().IsEmpty());
 
   // Events are queued even when there is no listener because a listener
   // may be added while events are in the queue.
-  SendInt32ParameterToStream(ScriptProcessorNodeEngine::IS_CONNECTED,
-                             isConnected);
+  SendInt32ParameterToTrack(ScriptProcessorNodeEngine::IS_CONNECTED,
+                            isConnected);
 
   if (isConnected && HasListenersFor(nsGkAtoms::onaudioprocess)) {
     MarkActive();
   } else {
     MarkInactive();
   }
+
+  // MarkInactive above might have released this node, check if it has a track.
+  if (!mTrack) {
+    return;
+  }
+
+  auto engine = static_cast<ScriptProcessorNodeEngine*>(mTrack->Engine());
+  engine->GetSharedBuffers()->NotifyNodeIsConnected(isConnected);
 }
 
-} // namespace dom
-} // namespace mozilla
-
+}  // namespace mozilla::dom
