@@ -4,40 +4,33 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 #if !defined(WebMBufferedParser_h_)
-#define WebMBufferedParser_h_
+#  define WebMBufferedParser_h_
 
-#include "nsISupportsImpl.h"
-#include "nsTArray.h"
-#include "mozilla/ReentrantMonitor.h"
-#include "MediaResource.h"
+#  include "nsISupportsImpl.h"
+#  include "nsTArray.h"
+#  include "mozilla/Mutex.h"
+#  include "MediaResource.h"
+#  include "MediaResult.h"
 
 namespace mozilla {
 
 // Stores a stream byte offset and the scaled timecode of the block at
 // that offset.
-struct WebMTimeDataOffset
-{
+struct WebMTimeDataOffset {
   WebMTimeDataOffset(int64_t aEndOffset, uint64_t aTimecode,
                      int64_t aInitOffset, int64_t aSyncOffset,
                      int64_t aClusterEndOffset)
-    : mEndOffset(aEndOffset)
-    , mInitOffset(aInitOffset)
-    , mSyncOffset(aSyncOffset)
-    , mClusterEndOffset(aClusterEndOffset)
-    , mTimecode(aTimecode)
-  {}
+      : mEndOffset(aEndOffset),
+        mInitOffset(aInitOffset),
+        mSyncOffset(aSyncOffset),
+        mClusterEndOffset(aClusterEndOffset),
+        mTimecode(aTimecode) {}
 
-  bool operator==(int64_t aEndOffset) const {
-    return mEndOffset == aEndOffset;
-  }
+  bool operator==(int64_t aEndOffset) const { return mEndOffset == aEndOffset; }
 
-  bool operator!=(int64_t aEndOffset) const {
-    return mEndOffset != aEndOffset;
-  }
+  bool operator!=(int64_t aEndOffset) const { return mEndOffset != aEndOffset; }
 
-  bool operator<(int64_t aEndOffset) const {
-    return mEndOffset < aEndOffset;
-  }
+  bool operator<(int64_t aEndOffset) const { return mEndOffset < aEndOffset; }
 
   int64_t mEndOffset;
   int64_t mInitOffset;
@@ -52,40 +45,17 @@ struct WebMTimeDataOffset
 // range.  Old parsers are destroyed when their range merges with a later
 // parser or an already parsed range.  The parser may start at any position
 // within the stream.
-struct WebMBufferedParser
-{
-  explicit WebMBufferedParser(int64_t aOffset)
-    : mStartOffset(aOffset)
-    , mCurrentOffset(aOffset)
-    , mInitEndOffset(-1)
-    , mBlockEndOffset(-1)
-    , mState(READ_ELEMENT_ID)
-    , mNextState(READ_ELEMENT_ID)
-    , mVIntRaw(false)
-    , mLastInitStartOffset(-1)
-    , mClusterSyncPos(0)
-    , mVIntLeft(0)
-    , mBlockSize(0)
-    , mClusterTimecode(0)
-    , mClusterOffset(0)
-    , mClusterEndOffset(-1)
-    , mBlockOffset(0)
-    , mBlockTimecode(0)
-    , mBlockTimecodeLength(0)
-    , mSkipBytes(0)
-    , mTimecodeScale(1000000)
-    , mGotTimecodeScale(false)
-  {
-    if (mStartOffset != 0) {
-      mState = FIND_CLUSTER_SYNC;
-    }
-  }
+struct WebMBufferedParser {
+  explicit WebMBufferedParser(int64_t aOffset);
 
   uint32_t GetTimecodeScale() {
     MOZ_ASSERT(mGotTimecodeScale);
     return mTimecodeScale;
   }
 
+  // Use this function when we would only feed media segment for the parser.
+  void AppendMediaSegmentOnly() { mGotTimecodeScale = true; }
+
   // If this parser is not expected to parse a segment info, it must be told
   // the appropriate timecode scale to use from elsewhere.
   void SetTimecodeScale(uint32_t aTimecodeScale) {
@@ -94,25 +64,24 @@ struct WebMBufferedParser
   }
 
   // Steps the parser through aLength bytes of data.  Always consumes
-  // aLength bytes.  Updates mCurrentOffset before returning.  Acquires
-  // aReentrantMonitor before using aMapping.
-  void Append(const unsigned char* aBuffer, uint32_t aLength,
-              nsTArray<WebMTimeDataOffset>& aMapping,
-              ReentrantMonitor& aReentrantMonitor);
+  // aLength bytes.  Updates mCurrentOffset before returning.
+  // Returns false if an error was encountered.
+  MediaResult Append(const unsigned char* aBuffer, uint32_t aLength,
+                     nsTArray<WebMTimeDataOffset>& aMapping);
 
-  bool operator==(int64_t aOffset) const {
-    return mCurrentOffset == aOffset;
-  }
+  bool operator==(int64_t aOffset) const { return mCurrentOffset == aOffset; }
 
-  bool operator<(int64_t aOffset) const {
-    return mCurrentOffset < aOffset;
-  }
+  bool operator<(int64_t aOffset) const { return mCurrentOffset < aOffset; }
 
   // Returns the start offset of the init (EBML) or media segment (Cluster)
-  // following the aOffset position. If none were found, returns mBlockEndOffset.
-  // This allows to determine the end of the interval containg aOffset.
+  // following the aOffset position. If none were found, returns
+  // mBlockEndOffset. This allows to determine the end of the interval containg
+  // aOffset.
   int64_t EndSegmentOffset(int64_t aOffset);
 
+  // Return the Cluster offset, return -1 if we can't find the Cluster.
+  int64_t GetClusterOffset() const;
+
   // The offset at which this parser started parsing.  Used to merge
   // adjacent parsers, in which case the later parser adopts the earlier
   // parser's mStartOffset.
@@ -130,7 +99,7 @@ struct WebMBufferedParser
   // Will only be set if a complete block has been parsed.
   int64_t mBlockEndOffset;
 
-private:
+ private:
   enum State {
     // Parser start state.  Expects to begin at a valid EBML element.  Move
     // to READ_VINT with mVIntRaw true, then return to READ_ELEMENT_SIZE.
@@ -175,6 +144,14 @@ private:
     // for this offset.
     READ_BLOCK_TIMECODE,
 
+    // mVInt holds the parsed EBMLMaxIdLength, store it in mEBMLMaxIdLength,
+    // then return to READ_ELEMENT_ID.
+    READ_EBML_MAX_ID_LENGTH,
+
+    // mVInt holds the parsed EBMLMaxSizeLength, store it in mEBMLMaxSizeLength,
+    // then return to READ_ELEMENT_ID.
+    READ_EBML_MAX_SIZE_LENGTH,
+
     // Will skip the current tracks element and set mInitEndOffset if an init
     // segment has been found.
     // Currently, only assumes it's the end of the tracks element.
@@ -213,6 +190,18 @@ private:
   // parsed. Will only be set if an EBML element has been found.
   int64_t mLastInitStartOffset;
 
+  // EBML element size. This indicates the size of the body of the last init
+  // segment parsed. Will only be set if an EBML element has been found.
+  uint32_t mLastInitSize;
+
+  // EBML max id length is the max number of bytes allowed for an element id
+  // vint.
+  uint8_t mEBMLMaxIdLength;
+
+  // EBML max size length is the max number of bytes allowed for an element size
+  // vint.
+  uint8_t mEBMLMaxSizeLength;
+
   // Current match position within CLUSTER_SYNC_ID.  Used to find sync
   // within arbitrary data.
   uint32_t mClusterSyncPos;
@@ -230,7 +219,7 @@ private:
 
   // Start offset of the cluster currently being parsed.  Used as the sync
   // point offset for the offset-to-time mapping as each block timecode is
-  // been parsed.
+  // been parsed. -1 if unknown.
   int64_t mClusterOffset;
 
   // End offset of the cluster currently being parsed. -1 if unknown.
@@ -259,29 +248,31 @@ private:
   // True if we read the timecode scale from the segment info or have
   // confirmed that the default value is to be used.
   bool mGotTimecodeScale;
+
+  // True if we've read the cluster time code.
+  bool mGotClusterTimecode;
 };
 
-class WebMBufferedState final
-{
+class WebMBufferedState final {
   NS_INLINE_DECL_THREADSAFE_REFCOUNTING(WebMBufferedState)
 
-public:
-  WebMBufferedState()
-    : mReentrantMonitor("WebMBufferedState")
-    , mLastBlockOffset(-1)
-  {
+ public:
+  WebMBufferedState() : mMutex("WebMBufferedState"), mLastBlockOffset(-1) {
     MOZ_COUNT_CTOR(WebMBufferedState);
   }
 
-  void NotifyDataArrived(const unsigned char* aBuffer, uint32_t aLength, int64_t aOffset);
+  void NotifyDataArrived(const unsigned char* aBuffer, uint32_t aLength,
+                         int64_t aOffset);
   void Reset();
   void UpdateIndex(const MediaByteRangeSet& aRanges, MediaResource* aResource);
   bool CalculateBufferedForRange(int64_t aStartOffset, int64_t aEndOffset,
                                  uint64_t* aStartTime, uint64_t* aEndTime);
 
-  // Returns true if aTime is is present in mTimeMapping and sets aOffset to
+  // Returns true if mTimeMapping is not empty and sets aOffset to
   // the latest offset for which decoding can resume without data
-  // dependencies to arrive at aTime.
+  // dependencies to arrive at aTime. aTime will be clamped to the start
+  // of mTimeMapping if it is earlier than the first element, and to the end
+  // if later than the last
   bool GetOffsetForTime(uint64_t aTime, int64_t* aOffset);
 
   // Returns end offset of init segment or -1 if none found.
@@ -290,30 +281,28 @@ public:
   int64_t GetLastBlockOffset();
 
   // Returns start time
-  bool GetStartTime(uint64_t *aTime);
+  bool GetStartTime(uint64_t* aTime);
 
   // Returns keyframe for time
   bool GetNextKeyframeTime(uint64_t aTime, uint64_t* aKeyframeTime);
 
-private:
+ private:
   // Private destructor, to discourage deletion outside of Release():
-  ~WebMBufferedState() {
-    MOZ_COUNT_DTOR(WebMBufferedState);
-  }
+  MOZ_COUNTED_DTOR(WebMBufferedState)
 
   // Synchronizes access to the mTimeMapping array and mLastBlockOffset.
-  ReentrantMonitor mReentrantMonitor;
+  Mutex mMutex;
 
   // Sorted (by offset) map of data offsets to timecodes.  Populated
   // on the main thread as data is received and parsed by WebMBufferedParsers.
-  nsTArray<WebMTimeDataOffset> mTimeMapping;
+  nsTArray<WebMTimeDataOffset> mTimeMapping MOZ_GUARDED_BY(mMutex);
   // The last complete block parsed. -1 if not set.
-  int64_t mLastBlockOffset;
+  int64_t mLastBlockOffset MOZ_GUARDED_BY(mMutex);
 
   // Sorted (by offset) live parser instances.  Main thread only.
   nsTArray<WebMBufferedParser> mRangeParsers;
 };
 
-} // namespace mozilla
+}  // namespace mozilla
 
 #endif
