@@ -1211,7 +1211,7 @@ void MediaTransportHandler::OnPacketReceived(const std::string& aTransportId,
     mCallbackThread->Dispatch(
         // This is being called from sigslot, which does not hold a strong ref.
         WrapRunnable(this, &MediaTransportHandler::OnPacketReceived,
-                     aTransportId, const_cast<MediaPacket&>(aPacket)),
+                     aTransportId, aPacket.Clone()),
         NS_DISPATCH_NORMAL);
     return;
   }
@@ -1225,7 +1225,7 @@ void MediaTransportHandler::OnEncryptedSending(const std::string& aTransportId,
     mCallbackThread->Dispatch(
         // This is being called from sigslot, which does not hold a strong ref.
         WrapRunnable(this, &MediaTransportHandler::OnEncryptedSending,
-                     aTransportId, const_cast<MediaPacket&>(aPacket)),
+                     aTransportId, aPacket.Clone()),
         NS_DISPATCH_NORMAL);
     return;
   }
@@ -1275,20 +1275,35 @@ RefPtr<dom::RTCStatsPromise> MediaTransportHandlerSTS::GetIceStats(
     const std::string& aTransportId, DOMHighResTimeStamp aNow) {
   MOZ_RELEASE_ASSERT(mInitPromise);
 
-  return mInitPromise->Then(
-      mStsThread, __func__,
-      [=, self = RefPtr<MediaTransportHandlerSTS>(this)]() {
-        UniquePtr<dom::RTCStatsCollection> stats(new dom::RTCStatsCollection);
-        if (mIceCtx) {
-          for (const auto& stream : mIceCtx->GetStreams()) {
-            if (aTransportId.empty() || aTransportId == stream->GetId()) {
-              GetIceStats(*stream, aNow, stats.get());
+  return mInitPromise
+      ->Then(
+          mStsThread, __func__,
+          [=, self = RefPtr<MediaTransportHandlerSTS>(this)]() {
+            UniquePtr<dom::RTCStatsCollection> stats(
+                new dom::RTCStatsCollection);
+            if (mIceCtx) {
+              for (const auto& stream : mIceCtx->GetStreams()) {
+                if (aTransportId.empty() || aTransportId == stream->GetId()) {
+                  GetIceStats(*stream, aNow, stats.get());
+                }
+              }
             }
-          }
-        }
-        return dom::RTCStatsPromise::CreateAndResolve(std::move(stats),
-                                                      __func__);
-      });
+            return dom::RTCStatsPromise::CreateAndResolve(std::move(stats),
+                                                          __func__);
+          })
+      ->Then(mStsThread, __func__,
+             [](dom::RTCStatsPromise::ResolveOrRejectValue&& aValue) {
+               // Eat errors! Caller is using MozPromise::All, and we don't
+               // want the whole thing to fail if this fails.
+               if (aValue.IsResolve()) {
+                 return dom::RTCStatsPromise::CreateAndResolve(
+                     std::move(aValue.ResolveValue()), __func__);
+               }
+               UniquePtr<dom::RTCStatsCollection> empty(
+                   new dom::RTCStatsCollection);
+               return dom::RTCStatsPromise::CreateAndResolve(std::move(empty),
+                                                             __func__);
+             });
 }
 
 RefPtr<MediaTransportHandler::IceLogPromise>
