@@ -25,12 +25,23 @@
 #include "RTCStatsReport.h"
 #include "mozilla/Preferences.h"
 #include "PeerConnectionCtx.h"
-#include "TransceiverImpl.h"
+#include "RTCRtpTransceiver.h"
 #include "libwebrtcglue/AudioConduit.h"
+#include "call/call.h"
 
 namespace mozilla::dom {
 
-NS_IMPL_CYCLE_COLLECTION_WRAPPERCACHE(RTCRtpReceiver, mWindow, mPc, mTrack)
+LazyLogModule gReceiverLog("RTCRtpReceiver");
+
+NS_IMPL_CYCLE_COLLECTION_WRAPPERCACHE_CLASS(RTCRtpReceiver)
+NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(RTCRtpReceiver)
+  // We do not do anything here, we wait for BreakCycles to be called
+  NS_IMPL_CYCLE_COLLECTION_UNLINK_PRESERVED_WRAPPER
+NS_IMPL_CYCLE_COLLECTION_UNLINK_END
+NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(RTCRtpReceiver)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mWindow, mPc, mTransceiver, mTrack)
+NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
+
 NS_IMPL_CYCLE_COLLECTING_ADDREF(RTCRtpReceiver)
 NS_IMPL_CYCLE_COLLECTING_RELEASE(RTCRtpReceiver)
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(RTCRtpReceiver)
@@ -38,8 +49,6 @@ NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(RTCRtpReceiver)
   NS_INTERFACE_MAP_ENTRY(nsISupports)
 NS_INTERFACE_MAP_END
 
-LazyLogModule gReceiverLog("RTCRtpReceiver");
-
 static PrincipalHandle GetPrincipalHandle(nsPIDOMWindowInner* aWindow,
                                           bool aPrivacyNeeded) {
   // Set the principal used for creating the tracks. This makes the track
@@ -57,7 +66,7 @@ static PrincipalHandle GetPrincipalHandle(nsPIDOMWindowInner* aWindow,
 
 static already_AddRefed<dom::MediaStreamTrack> CreateTrack(
     nsPIDOMWindowInner* aWindow, bool aAudio,
-    const nsCOMPtr<nsIPrincipal>& aPrincipal) {
+    const nsCOMPtr<nsIPrincipal>& aPrincipal, const TrackingId& aTrackingId) {
   MediaTrackGraph* graph = MediaTrackGraph::GetInstance(
       aAudio ? MediaTrackGraph::AUDIO_THREAD_DRIVER
              : MediaTrackGraph::SYSTEM_THREAD_DRIVER,
@@ -69,12 +78,14 @@ static already_AddRefed<dom::MediaStreamTrack> CreateTrack(
   if (aAudio) {
     RefPtr<SourceMediaTrack> source =
         graph->CreateSourceTrack(MediaSegment::AUDIO);
-    trackSource = new RemoteTrackSource(source, aPrincipal, u"remote audio"_ns);
+    trackSource = new RemoteTrackSource(source, aPrincipal, u"remote audio"_ns,
+                                        aTrackingId);
     track = new AudioStreamTrack(aWindow, source, trackSource);
   } else {
     RefPtr<SourceMediaTrack> source =
         graph->CreateSourceTrack(MediaSegment::VIDEO);
-    trackSource = new RemoteTrackSource(source, aPrincipal, u"remote video"_ns);
+    trackSource = new RemoteTrackSource(source, aPrincipal, u"remote video"_ns,
+                                        aTrackingId);
     track = new VideoStreamTrack(aWindow, source, trackSource);
   }
 
@@ -90,16 +101,15 @@ static already_AddRefed<dom::MediaStreamTrack> CreateTrack(
 
 RTCRtpReceiver::RTCRtpReceiver(
     nsPIDOMWindowInner* aWindow, bool aPrivacyNeeded, PeerConnectionImpl* aPc,
-    MediaTransportHandler* aTransportHandler, JsepTransceiver* aJsepTransceiver,
-    AbstractThread* aCallThread, nsISerialEventTarget* aStsThread,
-    MediaSessionConduit* aConduit, TransceiverImpl* aTransceiverImpl)
+    MediaTransportHandler* aTransportHandler, AbstractThread* aCallThread,
+    nsISerialEventTarget* aStsThread, MediaSessionConduit* aConduit,
+    RTCRtpTransceiver* aTransceiver, const TrackingId& aTrackingId)
     : mWindow(aWindow),
       mPc(aPc),
-      mJsepTransceiver(aJsepTransceiver),
       mCallThread(aCallThread),
       mStsThread(aStsThread),
       mTransportHandler(aTransportHandler),
-      mTransceiverImpl(aTransceiverImpl),
+      mTransceiver(aTransceiver),
       INIT_CANONICAL(mSsrc, 0),
       INIT_CANONICAL(mVideoRtxSsrc, 0),
       INIT_CANONICAL(mLocalRtpExtensions, RtpExtList()),
@@ -109,7 +119,7 @@ RTCRtpReceiver::RTCRtpReceiver(
       INIT_CANONICAL(mReceiving, false) {
   PrincipalHandle principalHandle = GetPrincipalHandle(aWindow, aPrivacyNeeded);
   mTrack = CreateTrack(aWindow, aConduit->type() == MediaSessionConduit::AUDIO,
-                       principalHandle.get());
+                       principalHandle.get(), aTrackingId);
   // Until Bug 1232234 is fixed, we'll get extra RTCP BYES during renegotiation,
   // so we'll disable muting on RTCP BYE and timeout for now.
   if (Preferences::GetBool("media.peerconnection.mute_on_bye_or_timeout",
@@ -140,22 +150,20 @@ JSObject* RTCRtpReceiver::WrapObject(JSContext* aCx,
 }
 
 RTCDtlsTransport* RTCRtpReceiver::GetTransport() const {
-  if (!mTransceiverImpl) {
+  if (!mTransceiver) {
     return nullptr;
   }
-  return mTransceiverImpl->GetDtlsTransport();
+  return mTransceiver->GetDtlsTransport();
 }
 
-already_AddRefed<Promise> RTCRtpReceiver::GetStats() {
+already_AddRefed<Promise> RTCRtpReceiver::GetStats(ErrorResult& aError) {
   nsCOMPtr<nsIGlobalObject> global = do_QueryInterface(mWindow);
-  ErrorResult rv;
-  RefPtr<Promise> promise = Promise::Create(global, rv);
-  if (NS_WARN_IF(rv.Failed())) {
-    rv.StealNSResult();
+  RefPtr<Promise> promise = Promise::Create(global, aError);
+  if (NS_WARN_IF(aError.Failed())) {
     return nullptr;
   }
 
-  if (NS_WARN_IF(!mTransceiverImpl)) {
+  if (NS_WARN_IF(!mTransceiver)) {
     // TODO(bug 1056433): When we stop nulling this out when the PC is closed
     // (or when the transceiver is stopped), we can remove this code. We
     // resolve instead of reject in order to make this eventual change in
@@ -164,12 +172,12 @@ already_AddRefed<Promise> RTCRtpReceiver::GetStats() {
     return promise.forget();
   }
 
-  mTransceiverImpl->ChainToDomPromiseWithCodecStats(GetStatsInternal(),
-                                                    promise);
+  mTransceiver->ChainToDomPromiseWithCodecStats(GetStatsInternal(), promise);
   return promise.forget();
 }
 
-nsTArray<RefPtr<RTCStatsPromise>> RTCRtpReceiver::GetStatsInternal() {
+nsTArray<RefPtr<RTCStatsPromise>> RTCRtpReceiver::GetStatsInternal(
+    bool aSkipIceStats) {
   MOZ_ASSERT(NS_IsMainThread());
   nsTArray<RefPtr<RTCStatsPromise>> promises(3);
 
@@ -274,7 +282,7 @@ nsTArray<RefPtr<RTCStatsPromise>> RTCRtpReceiver::GetStatsInternal() {
                 };
 
             asAudio.apply([&](auto& aConduit) {
-              Maybe<webrtc::AudioReceiveStream::Stats> audioStats =
+              Maybe<webrtc::AudioReceiveStreamInterface::Stats> audioStats =
                   aConduit->GetReceiverStats();
               if (audioStats.isNothing()) {
                 return;
@@ -317,8 +325,22 @@ nsTArray<RefPtr<RTCStatsPromise>> RTCRtpReceiver::GetStatsInternal() {
               local.mPacketsReceived.Construct(audioStats->packets_rcvd);
               local.mPacketsDiscarded.Construct(audioStats->packets_discarded);
               local.mBytesReceived.Construct(audioStats->payload_bytes_rcvd);
-              /*
-               * Potential new stats that are now available upstream.
+              // Always missing from libwebrtc stats
+              // if (audioStats->estimated_playout_ntp_timestamp_ms) {
+              //   local.mEstimatedPlayoutTimestamp.Construct(
+              //       aConduit->GetTimestampMaker().ConvertNtpToDomTime(
+              //           webrtc::Timestamp::Millis(
+              //               *audioStats->estimated_playout_ntp_timestamp_ms)));
+              // }
+              local.mJitterBufferDelay.Construct(
+                  audioStats->jitter_buffer_delay_seconds);
+              local.mJitterBufferEmittedCount.Construct(
+                  audioStats->jitter_buffer_emitted_count);
+              local.mTotalSamplesReceived.Construct(
+                  audioStats->total_samples_received);
+              local.mConcealedSamples.Construct(audioStats->concealed_samples);
+              local.mSilentConcealedSamples.Construct(
+                  audioStats->silent_concealed_samples);
               if (audioStats->last_packet_received_timestamp_ms) {
                 local.mLastPacketReceivedTimestamp.Construct(
                     aConduit->GetTimestampMaker().ConvertNtpToDomTime(
@@ -332,23 +354,9 @@ nsTArray<RefPtr<RTCStatsPromise>> RTCRtpReceiver::GetStatsInternal() {
                   audioStats->fec_packets_received);
               local.mFecPacketsDiscarded.Construct(
                   audioStats->fec_packets_discarded);
-              if (audioStats->estimated_playout_ntp_timestamp_ms) {
-                local.mEstimatedPlayoutTimestamp.Construct(
-                    aConduit->GetTimestampMaker().ConvertNtpToDomTime(
-                        webrtc::Timestamp::Millis(
-                            *audioStats->estimated_playout_ntp_timestamp_ms)));
-              }
-              local.mJitterBufferDelay.Construct(
-                  audioStats->jitter_buffer_delay_seconds);
-              local.mJitterBufferEmittedCount.Construct(
-                  audioStats->jitter_buffer_emitted_count);
-              local.mTotalSamplesReceived.Construct(
-                  audioStats->total_samples_received);
-              local.mConcealedSamples.Construct(audioStats->concealed_samples);
-              local.mSilentConcealedSamples.Construct(
-                  audioStats->silent_concealed_samples);
               local.mConcealmentEvents.Construct(
                   audioStats->concealment_events);
+
               local.mInsertedSamplesForDeceleration.Construct(
                   audioStats->inserted_samples_for_deceleration);
               local.mRemovedSamplesForAcceleration.Construct(
@@ -361,7 +369,7 @@ nsTArray<RefPtr<RTCStatsPromise>> RTCRtpReceiver::GetStatsInternal() {
                   audioStats->total_output_energy);
               local.mTotalSamplesDuration.Construct(
                   audioStats->total_output_duration);
-               */
+
               if (!report->mInboundRtpStreamStats.AppendElement(
                       std::move(local), fallible)) {
                 mozalloc_handle_oom(0);
@@ -369,7 +377,7 @@ nsTArray<RefPtr<RTCStatsPromise>> RTCRtpReceiver::GetStatsInternal() {
             });
 
             asVideo.apply([&](auto& aConduit) {
-              Maybe<webrtc::VideoReceiveStream::Stats> videoStats =
+              Maybe<webrtc::VideoReceiveStreamInterface::Stats> videoStats =
                   aConduit->GetReceiverStats();
               if (videoStats.isNothing()) {
                 return;
@@ -394,7 +402,7 @@ nsTArray<RefPtr<RTCStatsPromise>> RTCRtpReceiver::GetStatsInternal() {
                 remote.mBytesSent.Construct(
                     videoStats->rtcp_sender_octets_sent);
                 remote.mRemoteTimestamp.Construct(
-                    (webrtc::Timestamp::Millis(
+                    (webrtc::TimeDelta::Millis(
                          videoStats->rtcp_sender_remote_ntp_timestamp_ms) -
                      webrtc::TimeDelta::Seconds(webrtc::kNtpJan1970))
                         .ms());
@@ -433,22 +441,27 @@ nsTArray<RefPtr<RTCStatsPromise>> RTCRtpReceiver::GetStatsInternal() {
               local.mFramesPerSecond.Construct(videoStats->decode_frame_rate);
               local.mFrameWidth.Construct(videoStats->width);
               local.mFrameHeight.Construct(videoStats->height);
-              // XXX: key_frames + delta_frames may undercount frames because they were dropped in FrameBuffer::InsertFrame. (bug 1766553)
-              local.mFramesReceived.Construct(videoStats->frame_counts.key_frames + videoStats->frame_counts.delta_frames);
+              // XXX: key_frames + delta_frames may undercount frames because
+              // they were dropped in FrameBuffer::InsertFrame. (bug 1766553)
+              local.mFramesReceived.Construct(
+                  videoStats->frame_counts.key_frames +
+                  videoStats->frame_counts.delta_frames);
+              local.mJitterBufferDelay.Construct(
+                  videoStats->jitter_buffer_delay_seconds);
+              local.mJitterBufferEmittedCount.Construct(
+                  videoStats->jitter_buffer_emitted_count);
 
-              /*
-               * Potential new stats that are now available upstream.
               if (videoStats->qp_sum) {
-                local.mQpSum.Construct(*videoStats->qp_sum.value);
+                local.mQpSum.Construct(videoStats->qp_sum.value());
               }
               local.mTotalDecodeTime.Construct(
-                  double(videoStats->total_decode_time_ms) / 1000);
+                  double(videoStats->total_decode_time.ms()) / 1000);
               local.mTotalInterFrameDelay.Construct(
                   videoStats->total_inter_frame_delay);
               local.mTotalSquaredInterFrameDelay.Construct(
                   videoStats->total_squared_inter_frame_delay);
               if (videoStats->rtp_stats.last_packet_received_timestamp_ms) {
-                local.mLastPacketReceiveTimestamp.Construct(
+                local.mLastPacketReceivedTimestamp.Construct(
                     aConduit->GetTimestampMaker().ConvertNtpToDomTime(
                         webrtc::Timestamp::Millis(
                             *videoStats->rtp_stats
@@ -458,16 +471,16 @@ nsTArray<RefPtr<RTCStatsPromise>> RTCRtpReceiver::GetStatsInternal() {
               local.mHeaderBytesReceived.Construct(
                   videoStats->rtp_stats.packet_counter.header_bytes +
                   videoStats->rtp_stats.packet_counter.padding_bytes);
+              local.mTotalProcessingDelay.Construct(
+                  videoStats->total_processing_delay.seconds<double>());
+              /*
+               * Potential new stats that are now available upstream.
               if (videoStats->estimated_playout_ntp_timestamp_ms) {
                 local.mEstimatedPlayoutTimestamp.Construct(
                     aConduit->GetTimestampMaker().ConvertNtpToDomTime(
                         webrtc::Timestamp::Millis(
                             *videoStats->estimated_playout_ntp_timestamp_ms)));
               }
-              local.mJitterBufferDelay.Construct(
-                  videoStats->jitter_buffer_delay_seconds);
-              local.mJitterBufferEmittedCount.Construct(
-                  videoStats->jitter_buffer_emitted_count);
               local.mFramesReceived.Construct(
                   videoStats->frame_counts.key_frames +
                   videoStats->frame_counts.delta_frames);
@@ -502,9 +515,9 @@ nsTArray<RefPtr<RTCStatsPromise>> RTCRtpReceiver::GetStatsInternal() {
                                                         __func__);
               }));
 
-  if (mJsepTransceiver->mTransport.mComponents) {
+  if (!aSkipIceStats && GetJsepTransceiver().mTransport.mComponents) {
     promises.AppendElement(mTransportHandler->GetIceStats(
-        mJsepTransceiver->mTransport.mTransportId,
+        GetJsepTransceiver().mTransport.mTransportId,
         mPipeline->GetTimestampMaker().GetNow()));
   }
 
@@ -547,23 +560,28 @@ void RTCRtpReceiver::Shutdown() {
     mPipeline->Shutdown();
     mPipeline = nullptr;
   }
-  mTransceiverImpl = nullptr;
   mCallThread = nullptr;
   mRtcpByeListener.DisconnectIfExists();
   mRtcpTimeoutListener.DisconnectIfExists();
 }
 
+void RTCRtpReceiver::BreakCycles() {
+  mWindow = nullptr;
+  mPc = nullptr;
+  mTrack = nullptr;
+}
+
 void RTCRtpReceiver::UpdateTransport() {
   MOZ_ASSERT(NS_IsMainThread());
   if (!mHaveSetupTransport) {
-    mPipeline->SetLevel(mJsepTransceiver->GetLevel());
+    mPipeline->SetLevel(GetJsepTransceiver().GetLevel());
     mHaveSetupTransport = true;
   }
 
   UniquePtr<MediaPipelineFilter> filter;
 
-  auto const& details = mJsepTransceiver->mRecvTrack.GetNegotiatedDetails();
-  if (mJsepTransceiver->HasBundleLevel() && details) {
+  auto const& details = GetJsepTransceiver().mRecvTrack.GetNegotiatedDetails();
+  if (GetJsepTransceiver().HasBundleLevel() && details) {
     std::vector<webrtc::RtpExtension> extmaps;
     details->ForEachRTPHeaderExtension(
         [&extmaps](const SdpExtmapAttributeList::Extmap& extmap) {
@@ -573,10 +591,10 @@ void RTCRtpReceiver::UpdateTransport() {
 
     // Add remote SSRCs so we can distinguish which RTP packets actually
     // belong to this pipeline (also RTCP sender reports).
-    for (uint32_t ssrc : mJsepTransceiver->mRecvTrack.GetSsrcs()) {
+    for (uint32_t ssrc : GetJsepTransceiver().mRecvTrack.GetSsrcs()) {
       filter->AddRemoteSSRC(ssrc);
     }
-    for (uint32_t ssrc : mJsepTransceiver->mRecvTrack.GetRtxSsrcs()) {
+    for (uint32_t ssrc : GetJsepTransceiver().mRecvTrack.GetRtxSsrcs()) {
       filter->AddRemoteSSRC(ssrc);
     }
     auto mid = Maybe<std::string>();
@@ -586,14 +604,15 @@ void RTCRtpReceiver::UpdateTransport() {
     filter->SetRemoteMediaStreamId(mid);
 
     // Add unique payload types as a last-ditch fallback
-    auto uniquePts = mJsepTransceiver->mRecvTrack.GetNegotiatedDetails()
+    auto uniquePts = GetJsepTransceiver()
+                         .mRecvTrack.GetNegotiatedDetails()
                          ->GetUniquePayloadTypes();
     for (unsigned char& uniquePt : uniquePts) {
       filter->AddUniquePT(uniquePt);
     }
   }
 
-  mPipeline->UpdateTransport_m(mJsepTransceiver->mTransport.mTransportId,
+  mPipeline->UpdateTransport_m(GetJsepTransceiver().mTransport.mTransportId,
                                std::move(filter));
 }
 
@@ -607,7 +626,7 @@ void RTCRtpReceiver::UpdateConduit() {
     UpdateAudioConduit();
   }
 
-  if ((mReceiving = mJsepTransceiver->mRecvTrack.GetActive())) {
+  if ((mReceiving = GetJsepTransceiver().mRecvTrack.GetActive())) {
     Start();
   }
 }
@@ -620,24 +639,25 @@ void RTCRtpReceiver::UpdateVideoConduit() {
   // CreateVideoReceiveStream method of the Call API will assert (in debug)
   // and fail if a value is not provided for the remote_ssrc that will be used
   // by the far-end sender.
-  if (!mJsepTransceiver->mRecvTrack.GetSsrcs().empty()) {
+  if (!GetJsepTransceiver().mRecvTrack.GetSsrcs().empty()) {
     MOZ_LOG(gReceiverLog, LogLevel::Debug,
             ("%s[%s]: %s Setting remote SSRC %u", mPc->GetHandle().c_str(),
              GetMid().c_str(), __FUNCTION__,
-             mJsepTransceiver->mRecvTrack.GetSsrcs().front()));
-    uint32_t rtxSsrc = mJsepTransceiver->mRecvTrack.GetRtxSsrcs().empty()
-                           ? 0
-                           : mJsepTransceiver->mRecvTrack.GetRtxSsrcs().front();
-    mSsrc = mJsepTransceiver->mRecvTrack.GetSsrcs().front();
+             GetJsepTransceiver().mRecvTrack.GetSsrcs().front()));
+    uint32_t rtxSsrc =
+        GetJsepTransceiver().mRecvTrack.GetRtxSsrcs().empty()
+            ? 0
+            : GetJsepTransceiver().mRecvTrack.GetRtxSsrcs().front();
+    mSsrc = GetJsepTransceiver().mRecvTrack.GetSsrcs().front();
     mVideoRtxSsrc = rtxSsrc;
 
     // TODO (bug 1423041) once we pay attention to receiving MID's in RTP
     // packets (see bug 1405495) we could make this depending on the presence of
     // MID in the RTP packets instead of relying on the signaling.
     // In any case, do not disable SSRC changes if no SSRCs were negotiated
-    if (mJsepTransceiver->HasBundleLevel() &&
-        (!mJsepTransceiver->mRecvTrack.GetNegotiatedDetails() ||
-         !mJsepTransceiver->mRecvTrack.GetNegotiatedDetails()->GetExt(
+    if (GetJsepTransceiver().HasBundleLevel() &&
+        (!GetJsepTransceiver().mRecvTrack.GetNegotiatedDetails() ||
+         !GetJsepTransceiver().mRecvTrack.GetNegotiatedDetails()->GetExt(
              webrtc::RtpExtension::kMidUri))) {
       mCallThread->Dispatch(
           NewRunnableMethod("VideoSessionConduit::DisableSsrcChanges", conduit,
@@ -645,9 +665,10 @@ void RTCRtpReceiver::UpdateVideoConduit() {
     }
   }
 
-  if (mJsepTransceiver->mRecvTrack.GetNegotiatedDetails() &&
-      mJsepTransceiver->mRecvTrack.GetActive()) {
-    const auto& details(*mJsepTransceiver->mRecvTrack.GetNegotiatedDetails());
+  if (GetJsepTransceiver().mRecvTrack.GetNegotiatedDetails() &&
+      GetJsepTransceiver().mRecvTrack.GetActive()) {
+    const auto& details(
+        *GetJsepTransceiver().mRecvTrack.GetNegotiatedDetails());
 
     {
       std::vector<webrtc::RtpExtension> extmaps;
@@ -660,7 +681,7 @@ void RTCRtpReceiver::UpdateVideoConduit() {
     }
 
     std::vector<VideoCodecConfig> configs;
-    TransceiverImpl::NegotiatedDetailsToVideoCodecConfigs(details, &configs);
+    RTCRtpTransceiver::NegotiatedDetailsToVideoCodecConfigs(details, &configs);
     if (configs.empty()) {
       // TODO: Are we supposed to plumb this error back to JS? This does not
       // seem like a failure to set an answer, it just means that codec
@@ -681,20 +702,20 @@ void RTCRtpReceiver::UpdateAudioConduit() {
   RefPtr<AudioSessionConduit> conduit =
       *mPipeline->mConduit->AsAudioSessionConduit();
 
-  if (!mJsepTransceiver->mRecvTrack.GetSsrcs().empty()) {
+  if (!GetJsepTransceiver().mRecvTrack.GetSsrcs().empty()) {
     MOZ_LOG(gReceiverLog, LogLevel::Debug,
             ("%s[%s]: %s Setting remote SSRC %u", mPc->GetHandle().c_str(),
              GetMid().c_str(), __FUNCTION__,
-             mJsepTransceiver->mRecvTrack.GetSsrcs().front()));
-    mSsrc = mJsepTransceiver->mRecvTrack.GetSsrcs().front();
+             GetJsepTransceiver().mRecvTrack.GetSsrcs().front()));
+    mSsrc = GetJsepTransceiver().mRecvTrack.GetSsrcs().front();
 
     // TODO (bug 1423041) once we pay attention to receiving MID's in RTP
     // packets (see bug 1405495) we could make this depending on the presence of
     // MID in the RTP packets instead of relying on the signaling.
     // In any case, do not disable SSRC changes if no SSRCs were negotiated
-    if (mJsepTransceiver->HasBundleLevel() &&
-        (!mJsepTransceiver->mRecvTrack.GetNegotiatedDetails() ||
-         !mJsepTransceiver->mRecvTrack.GetNegotiatedDetails()->GetExt(
+    if (GetJsepTransceiver().HasBundleLevel() &&
+        (!GetJsepTransceiver().mRecvTrack.GetNegotiatedDetails() ||
+         !GetJsepTransceiver().mRecvTrack.GetNegotiatedDetails()->GetExt(
              webrtc::RtpExtension::kMidUri))) {
       mCallThread->Dispatch(
           NewRunnableMethod("AudioSessionConduit::DisableSsrcChanges", conduit,
@@ -702,11 +723,12 @@ void RTCRtpReceiver::UpdateAudioConduit() {
     }
   }
 
-  if (mJsepTransceiver->mRecvTrack.GetNegotiatedDetails() &&
-      mJsepTransceiver->mRecvTrack.GetActive()) {
-    const auto& details(*mJsepTransceiver->mRecvTrack.GetNegotiatedDetails());
+  if (GetJsepTransceiver().mRecvTrack.GetNegotiatedDetails() &&
+      GetJsepTransceiver().mRecvTrack.GetActive()) {
+    const auto& details(
+        *GetJsepTransceiver().mRecvTrack.GetNegotiatedDetails());
     std::vector<AudioCodecConfig> configs;
-    TransceiverImpl::NegotiatedDetailsToAudioCodecConfigs(details, &configs);
+    RTCRtpTransceiver::NegotiatedDetailsToAudioCodecConfigs(details, &configs);
     if (configs.empty()) {
       // TODO: Are we supposed to plumb this error back to JS? This does not
       // seem like a failure to set an answer, it just means that codec
@@ -748,13 +770,24 @@ bool RTCRtpReceiver::HasTrack(const dom::MediaStreamTrack* aTrack) const {
   return !aTrack || (mTrack == aTrack);
 }
 
+void RTCRtpReceiver::SyncFromJsep(const JsepTransceiver& aJsepTransceiver) {
+  // If a SRD has unset the receive bit, stop the receive pipeline so incoming
+  // RTP does not unmute the receive track.
+  if (!aJsepTransceiver.mRecvTrack.GetRemoteSetSendBit() ||
+      !aJsepTransceiver.mRecvTrack.GetActive()) {
+    Stop();
+  }
+}
+
+void RTCRtpReceiver::SyncToJsep(JsepTransceiver& aJsepTransceiver) const {}
+
 void RTCRtpReceiver::UpdateStreams(StreamAssociationChanges* aChanges) {
   // We don't sort and use set_difference, because we need to report the
   // added/removed streams in the order that they appear in the SDP.
   std::set<std::string> newIds(
-      mJsepTransceiver->mRecvTrack.GetStreamIds().begin(),
-      mJsepTransceiver->mRecvTrack.GetStreamIds().end());
-  MOZ_ASSERT(mJsepTransceiver->mRecvTrack.GetRemoteSetSendBit() ||
+      GetJsepTransceiver().mRecvTrack.GetStreamIds().begin(),
+      GetJsepTransceiver().mRecvTrack.GetStreamIds().end());
+  MOZ_ASSERT(GetJsepTransceiver().mRecvTrack.GetRemoteSetSendBit() ||
              newIds.empty());
   bool needsTrackEvent = false;
   for (const auto& id : mStreamIds) {
@@ -764,17 +797,18 @@ void RTCRtpReceiver::UpdateStreams(StreamAssociationChanges* aChanges) {
   }
 
   std::set<std::string> oldIds(mStreamIds.begin(), mStreamIds.end());
-  for (const auto& id : mJsepTransceiver->mRecvTrack.GetStreamIds()) {
+  for (const auto& id : GetJsepTransceiver().mRecvTrack.GetStreamIds()) {
     if (!oldIds.count(id)) {
       needsTrackEvent = true;
       aChanges->mStreamAssociationsAdded.push_back({mTrack, id});
     }
   }
 
-  mStreamIds = mJsepTransceiver->mRecvTrack.GetStreamIds();
+  mStreamIds = GetJsepTransceiver().mRecvTrack.GetStreamIds();
 
-  if (mRemoteSetSendBit != mJsepTransceiver->mRecvTrack.GetRemoteSetSendBit()) {
-    mRemoteSetSendBit = mJsepTransceiver->mRecvTrack.GetRemoteSetSendBit();
+  if (mRemoteSetSendBit !=
+      GetJsepTransceiver().mRecvTrack.GetRemoteSetSendBit()) {
+    mRemoteSetSendBit = GetJsepTransceiver().mRecvTrack.GetRemoteSetSendBit();
     if (mRemoteSetSendBit) {
       needsTrackEvent = true;
     } else {
@@ -810,10 +844,17 @@ void RTCRtpReceiver::SetReceiveTrackMuted(bool aMuted) {
 }
 
 std::string RTCRtpReceiver::GetMid() const {
-  if (mJsepTransceiver->IsAssociated()) {
-    return mJsepTransceiver->GetMid();
-  }
-  return std::string();
+  return mTransceiver->GetMidAscii();
+}
+
+JsepTransceiver& RTCRtpReceiver::GetJsepTransceiver() {
+  MOZ_ASSERT(mTransceiver);
+  return *mTransceiver->GetJsepTransceiver();
+}
+
+const JsepTransceiver& RTCRtpReceiver::GetJsepTransceiver() const {
+  MOZ_ASSERT(mTransceiver);
+  return *mTransceiver->GetJsepTransceiver();
 }
 
 }  // namespace mozilla::dom
