@@ -16,7 +16,6 @@
 #include "DOMMediaStream.h"
 #include "ImageContainer.h"
 #include "ImageTypes.h"
-#include "Layers.h"
 #include "MediaEngine.h"
 #include "MediaSegment.h"
 #include "MediaTrackGraphImpl.h"
@@ -51,6 +50,8 @@
 #include "libwebrtcglue/WebrtcImageBuffer.h"
 #include "common_video/include/video_frame_buffer.h"
 #include "modules/rtp_rtcp/include/rtp_rtcp.h"
+#include "modules/rtp_rtcp/include/rtp_header_extension_map.h"
+#include "modules/rtp_rtcp/source/rtp_packet_received.h"
 
 // Max size given stereo is 480*2*2 = 1920 (10ms of 16-bits stereo audio at
 // 48KHz)
@@ -78,7 +79,7 @@ class AudioProxyThread {
 
   explicit AudioProxyThread(RefPtr<AudioSessionConduit> aConduit)
       : mConduit(std::move(aConduit)),
-        mTaskQueue(new TaskQueue(
+        mTaskQueue(TaskQueue::Create(
             GetMediaThreadPool(MediaThreadType::WEBRTC_WORKER), "AudioProxy")),
         mAudioConverter(nullptr) {
     MOZ_ASSERT(mConduit);
@@ -259,7 +260,7 @@ MediaPipeline::MediaPipeline(const std::string& aPc,
       mRtpBytesReceived(0),
       mPc(aPc),
       mFilter(),
-      mRtpParser(webrtc::RtpHeaderParser::CreateForTest().release()),
+      mRtpHeaderExtensionMap(new webrtc::RtpHeaderExtensionMap()),
       mPacketDumper(PacketDumper::GetPacketDumper(mPc)) {}
 
 MediaPipeline::~MediaPipeline() {
@@ -338,7 +339,7 @@ void MediaPipeline::UpdateTransport_s(
 
   if (mFilter) {
     for (const auto& extension : mFilter->GetExtmap()) {
-      mRtpParser->DeregisterRtpHeaderExtension(extension);
+      mRtpHeaderExtensionMap->Deregister(extension.uri);
     }
   }
   if (mFilter && aFilter) {
@@ -350,7 +351,7 @@ void MediaPipeline::UpdateTransport_s(
   }
   if (mFilter) {
     for (const auto& extension : mFilter->GetExtmap()) {
-      mRtpParser->RegisterRtpHeaderExtension(extension);
+      mRtpHeaderExtensionMap->RegisterByUri(extension.id, extension.uri);
     }
   }
 }
@@ -538,9 +539,12 @@ void MediaPipeline::RtpPacketReceived(const MediaPacket& packet) {
   }
 
   webrtc::RTPHeader header;
-  if (!mRtpParser->Parse(packet.data(), packet.len(), &header, true)) {
+  rtc::CopyOnWriteBuffer packet_buffer(packet.data(), packet.len());
+  webrtc::RtpPacketReceived pktHeader(mRtpHeaderExtensionMap.get());
+  if (!pktHeader.Parse(packet_buffer)) {
     return;
   }
+  pktHeader.GetHeader(&header);
 
   if (mFilter && !mFilter->Filter(header)) {
     return;
@@ -599,7 +603,7 @@ void MediaPipeline::RtpPacketReceived(const MediaPacket& packet) {
   mPacketDumper->Dump(mLevel, dom::mozPacketDumpType::Rtp, false, packet.data(),
                       packet.len());
 
-  mRtpReceiveEvent.Notify(packet, header);
+  mRtpReceiveEvent.Notify(packet.Clone(), header);
 }
 
 void MediaPipeline::RtcpPacketReceived(const MediaPacket& packet) {
@@ -636,9 +640,9 @@ void MediaPipeline::RtcpPacketReceived(const MediaPacket& packet) {
   }
 
   if (mDirection == DirectionType::TRANSMIT) {
-    mSenderRtcpReceiveEvent.Notify(std::move(packet));
+    mSenderRtcpReceiveEvent.Notify(packet.Clone());
   } else {
-    mReceiverRtcpReceiveEvent.Notify(std::move(packet));
+    mReceiverRtcpReceiveEvent.Notify(packet.Clone());
   }
 }
 
@@ -879,8 +883,7 @@ void MediaPipelineTransmit::UpdateSendState() {
       mSendTrackOverride.Ref() && !mSendTrackOverride.Ref()->IsDestroyed();
   const bool mustRemoveSendTrack =
       haveLiveSendTrack && !mSendTrackOverride.Ref() &&
-      (!haveLiveDomTrack ||
-       mDomTrack.Ref()->GetTrack() != mSendPort->GetSource());
+      (!haveLiveDomTrack || mDomTrack.Ref()->GetTrack() != mSendPortSource);
 
   mTransmitting = mActive && (haveLiveDomTrack || haveLiveOverrideTrack) &&
                   !mustRemoveSendTrack;
@@ -917,7 +920,11 @@ void MediaPipelineTransmit::UpdateSendState() {
     } else {
       mSendTrack = mDomTrack.Ref()->Graph()->CreateForwardedInputTrack(
           mDomTrack.Ref()->GetTrack()->mType);
-      mSendPort = mSendTrack->AllocateInputPort(mDomTrack.Ref()->GetTrack());
+      mSendPortSource = mDomTrack.Ref()->GetTrack();
+      mSendPort = mSendTrack->AllocateInputPort(mSendPortSource.get());
+    }
+    if (mIsVideo) {
+      mConverter->SetTrackingId(mDomTrack.Ref()->GetSource().mTrackingId);
     }
     mSendTrack->QueueSetAutoend(false);
     if (mIsVideo) {
@@ -948,6 +955,7 @@ void MediaPipelineTransmit::UpdateSendState() {
       mSendTrack->Destroy();
       mSendPort->Destroy();
       mSendPort = nullptr;
+      mSendPortSource = nullptr;
     }
   }
 }
@@ -1167,6 +1175,7 @@ class GenericReceiveListener : public MediaTrackListener {
             "GenericReceiveListener::mTrackSource",
             &static_cast<RemoteTrackSource&>(aTrack->GetSource()))),
         mSource(mTrackSource->mStream),
+        mTrackingId(mTrackSource->mTrackingId),
         mIsAudio(aTrack->AsAudioStreamTrack()),
         mEnabled(false),
         mMaybeTrackNeedsUnmute(true) {
@@ -1223,6 +1232,7 @@ class GenericReceiveListener : public MediaTrackListener {
  protected:
   const nsMainThreadPtrHandle<RemoteTrackSource> mTrackSource;
   const RefPtr<SourceMediaTrack> mSource;
+  const TrackingId mTrackingId;
   const bool mIsAudio;
   // Main thread only.
   bool mEnabled;
@@ -1256,9 +1266,9 @@ class MediaPipelineReceiveAudio::PipelineListener
                       ->IsSamplingFreqSupported(mSource->Graph()->GraphRate())
                   ? mSource->Graph()->GraphRate()
                   : WEBRTC_MAX_SAMPLE_RATE),
-        mTaskQueue(
-            new TaskQueue(GetMediaThreadPool(MediaThreadType::WEBRTC_WORKER),
-                          "AudioPipelineListener")),
+        mTaskQueue(TaskQueue::Create(
+            GetMediaThreadPool(MediaThreadType::WEBRTC_WORKER),
+            "AudioPipelineListener")),
         mPlayedTicks(0),
         mAudioFrame(std::make_unique<webrtc::AudioFrame>()),
         mPrincipalHandle(aPrincipalHandle),
@@ -1516,11 +1526,15 @@ class MediaPipelineReceiveVideo::PipelineListener
       image = imageBuffer->GetNativeImage();
     } else {
       MOZ_ASSERT(aBuffer.type() == webrtc::VideoFrameBuffer::Type::kI420);
-      rtc::scoped_refptr<const webrtc::I420BufferInterface> i420 =
-          aBuffer.GetI420();
+      rtc::scoped_refptr<const webrtc::I420BufferInterface> i420(
+          aBuffer.GetI420());
 
       MOZ_ASSERT(i420->DataY());
       // Create a video frame using |buffer|.
+      PerformanceRecorder<CopyVideoStage> rec(
+          "MediaPipelineReceiveVideo::CopyToImage"_ns, mTrackingId,
+          i420->width(), i420->height());
+
       RefPtr<PlanarYCbCrImage> yuvImage =
           mImageContainer->CreatePlanarYCbCrImage();
 
@@ -1542,6 +1556,7 @@ class MediaPipelineReceiveVideo::PipelineListener
         MOZ_ASSERT(false);
         return;
       }
+      rec.Record();
 
       image = std::move(yuvImage);
     }
