@@ -8,50 +8,89 @@
 #include "nsServiceManagerUtils.h"
 #include "nsObjCExceptions.h"
 #include "nsCocoaUtils.h"
+#include "nsIThread.h"
 #include "nsThreadUtils.h"
+#include "nsXULAppAPI.h"
+#include "mozilla/ClearOnShutdown.h"
 #include "mozilla/dom/nsSynthVoiceRegistry.h"
 #include "mozilla/dom/nsSpeechTask.h"
 #include "mozilla/Preferences.h"
+#include "mozilla/StaticPrefs_media.h"
 #include "mozilla/Assertions.h"
 #include "OSXSpeechSynthesizerService.h"
 
 #import <Cocoa/Cocoa.h>
 
+@class SpeechDelegate;
+
+// We can escape the default delimiters ("[[" and "]]") by temporarily
+// changing the delimiters just before they appear, and changing them back
+// just after.
+#define DLIM_ESCAPE_START "[[dlim (( ))]]"
+#define DLIM_ESCAPE_END "((dlim [[ ]]))"
+
 using namespace mozilla;
 
-class SpeechTaskCallback final : public nsISpeechTaskCallback
-{
-public:
-  SpeechTaskCallback(nsISpeechTask* aTask, NSSpeechSynthesizer* aSynth)
-    : mTask(aTask)
-    , mSpeechSynthesizer(aSynth)
-  {
-    mStartingTime = TimeStamp::Now();
-  }
+class SpeechTaskCallback final : public nsISpeechTaskCallback {
+ public:
+  SpeechTaskCallback(nsISpeechTask* aTask, NSSpeechSynthesizer* aSynth,
+                     const nsTArray<size_t>& aOffsets);
 
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
   NS_DECL_CYCLE_COLLECTION_CLASS_AMBIGUOUS(SpeechTaskCallback, nsISpeechTaskCallback)
 
   NS_DECL_NSISPEECHTASKCALLBACK
 
-  void OnWillSpeakWord(uint32_t aIndex);
+  void OnWillSpeakWord(uint32_t aIndex, uint32_t aLength);
   void OnError(uint32_t aIndex);
   void OnDidFinishSpeaking();
 
-private:
-  virtual ~SpeechTaskCallback()
-  {
-    [mSpeechSynthesizer release];
-  }
+ private:
+  virtual ~SpeechTaskCallback();
 
   float GetTimeDurationFromStart();
 
   nsCOMPtr<nsISpeechTask> mTask;
   NSSpeechSynthesizer* mSpeechSynthesizer;
+  SpeechDelegate* mDelegate;
   TimeStamp mStartingTime;
   uint32_t mCurrentIndex;
+  nsTArray<size_t> mOffsets;
 };
 
+@interface SpeechDelegate : NSObject <NSSpeechSynthesizerDelegate> {
+ @private
+  SpeechTaskCallback* mCallback;
+}
+
+- (id)initWithCallback:(SpeechTaskCallback*)aCallback;
+@end
+
+@implementation SpeechDelegate
+- (id)initWithCallback:(SpeechTaskCallback*)aCallback {
+  [super init];
+  mCallback = aCallback;
+  return self;
+}
+
+- (void)speechSynthesizer:(NSSpeechSynthesizer*)aSender
+            willSpeakWord:(NSRange)aRange
+                 ofString:(NSString*)aString {
+  mCallback->OnWillSpeakWord(aRange.location, aRange.length);
+}
+
+- (void)speechSynthesizer:(NSSpeechSynthesizer*)aSender didFinishSpeaking:(BOOL)aFinishedSpeaking {
+  mCallback->OnDidFinishSpeaking();
+}
+
+- (void)speechSynthesizer:(NSSpeechSynthesizer*)aSender
+    didEncounterErrorAtIndex:(NSUInteger)aCharacterIndex
+                    ofString:(NSString*)aString
+                     message:(NSString*)aMessage {
+  mCallback->OnError(aCharacterIndex);
+}
+@end
+
 NS_IMPL_CYCLE_COLLECTION(SpeechTaskCallback, mTask);
 
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(SpeechTaskCallback)
@@ -62,21 +101,33 @@ NS_INTERFACE_MAP_END
 NS_IMPL_CYCLE_COLLECTING_ADDREF(SpeechTaskCallback)
 NS_IMPL_CYCLE_COLLECTING_RELEASE(SpeechTaskCallback)
 
+SpeechTaskCallback::SpeechTaskCallback(nsISpeechTask* aTask, NSSpeechSynthesizer* aSynth,
+                                       const nsTArray<size_t>& aOffsets)
+    : mTask(aTask), mSpeechSynthesizer(aSynth), mCurrentIndex(0), mOffsets(aOffsets.Clone()) {
+  mDelegate = [[SpeechDelegate alloc] initWithCallback:this];
+  [mSpeechSynthesizer setDelegate:mDelegate];
+  mStartingTime = TimeStamp::Now();
+}
+
+SpeechTaskCallback::~SpeechTaskCallback() {
+  [mSpeechSynthesizer setDelegate:nil];
+  [mDelegate release];
+  [mSpeechSynthesizer release];
+}
+
 NS_IMETHODIMP
-SpeechTaskCallback::OnCancel()
-{
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
+SpeechTaskCallback::OnCancel() {
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
   [mSpeechSynthesizer stopSpeaking];
   return NS_OK;
 
-  NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
+  NS_OBJC_END_TRY_BLOCK_RETURN(NS_ERROR_FAILURE);
 }
 
 NS_IMETHODIMP
-SpeechTaskCallback::OnPause()
-{
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
+SpeechTaskCallback::OnPause() {
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
   [mSpeechSynthesizer pauseSpeakingAtBoundary:NSSpeechImmediateBoundary];
   if (!mTask) {
@@ -87,13 +138,12 @@ SpeechTaskCallback::OnPause()
   mTask->DispatchPause(GetTimeDurationFromStart(), mCurrentIndex);
   return NS_OK;
 
-  NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
+  NS_OBJC_END_TRY_BLOCK_RETURN(NS_ERROR_FAILURE);
 }
 
 NS_IMETHODIMP
-SpeechTaskCallback::OnResume()
-{
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
+SpeechTaskCallback::OnResume() {
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
   [mSpeechSynthesizer continueSpeaking];
   if (!mTask) {
@@ -104,103 +154,53 @@ SpeechTaskCallback::OnResume()
   mTask->DispatchResume(GetTimeDurationFromStart(), mCurrentIndex);
   return NS_OK;
 
-  NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
+  NS_OBJC_END_TRY_BLOCK_RETURN(NS_ERROR_FAILURE);
 }
 
 NS_IMETHODIMP
-SpeechTaskCallback::OnVolumeChanged(float aVolume)
-{
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
+SpeechTaskCallback::OnVolumeChanged(float aVolume) {
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
   [mSpeechSynthesizer setObject:[NSNumber numberWithFloat:aVolume]
-                    forProperty:NSSpeechVolumeProperty error:nil];
+                    forProperty:NSSpeechVolumeProperty
+                          error:nil];
   return NS_OK;
 
-  NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
+  NS_OBJC_END_TRY_BLOCK_RETURN(NS_ERROR_FAILURE);
 }
 
-float
-SpeechTaskCallback::GetTimeDurationFromStart()
-{
+float SpeechTaskCallback::GetTimeDurationFromStart() {
   TimeDuration duration = TimeStamp::Now() - mStartingTime;
-  return duration.ToMilliseconds();
+  return duration.ToSeconds();
 }
 
-void
-SpeechTaskCallback::OnWillSpeakWord(uint32_t aIndex)
-{
-  mCurrentIndex = aIndex;
+void SpeechTaskCallback::OnWillSpeakWord(uint32_t aIndex, uint32_t aLength) {
+  mCurrentIndex = aIndex < mOffsets.Length() ? mOffsets[aIndex] : mCurrentIndex;
   if (!mTask) {
     return;
   }
-  mTask->DispatchBoundary(NS_LITERAL_STRING("word"),
-                          GetTimeDurationFromStart(), mCurrentIndex);
+  mTask->DispatchBoundary(u"word"_ns, GetTimeDurationFromStart(), mCurrentIndex, aLength, 1);
 }
 
-void
-SpeechTaskCallback::OnError(uint32_t aIndex)
-{
+void SpeechTaskCallback::OnError(uint32_t aIndex) {
   if (!mTask) {
     return;
   }
   mTask->DispatchError(GetTimeDurationFromStart(), aIndex);
 }
 
-void
-SpeechTaskCallback::OnDidFinishSpeaking()
-{
+void SpeechTaskCallback::OnDidFinishSpeaking() {
   mTask->DispatchEnd(GetTimeDurationFromStart(), mCurrentIndex);
   // no longer needed
   [mSpeechSynthesizer setDelegate:nil];
   mTask = nullptr;
 }
 
-@interface SpeechDelegate : NSObject<NSSpeechSynthesizerDelegate>
-{
-@private
-  SpeechTaskCallback* mCallback;
-}
-
-  - (id)initWithCallback:(SpeechTaskCallback*)aCallback;
-@end
-
-@implementation SpeechDelegate
-- (id)initWithCallback:(SpeechTaskCallback*)aCallback
-{
-  [super init];
-  mCallback = aCallback;
-  return self;
-}
-
-- (void)speechSynthesizer:(NSSpeechSynthesizer *)aSender
-            willSpeakWord:(NSRange)aRange ofString:(NSString*)aString
-{
-  mCallback->OnWillSpeakWord(aRange.location);
-}
-
-- (void)speechSynthesizer:(NSSpeechSynthesizer *)aSender
-        didFinishSpeaking:(BOOL)aFinishedSpeaking
-{
-  mCallback->OnDidFinishSpeaking();
-}
-
-- (void)speechSynthesizer:(NSSpeechSynthesizer*)aSender
- didEncounterErrorAtIndex:(NSUInteger)aCharacterIndex
-                 ofString:(NSString*)aString
-                  message:(NSString*)aMessage
-{
-  mCallback->OnError(aCharacterIndex);
-}
-@end
-
 namespace mozilla {
 namespace dom {
 
-struct OSXVoice
-{
-  OSXVoice() : mIsDefault(false)
-  {
-  }
+struct OSXVoice {
+  OSXVoice() : mIsDefault(false) {}
 
   nsString mUri;
   nsString mName;
@@ -208,22 +208,15 @@ struct OSXVoice
   bool mIsDefault;
 };
 
-class RegisterVoicesRunnable final : public nsRunnable
-{
-public:
-  RegisterVoicesRunnable(OSXSpeechSynthesizerService* aSpeechService,
-                         nsTArray<OSXVoice>& aList)
-    : mSpeechService(aSpeechService)
-    , mVoices(aList)
-  {
-  }
+class RegisterVoicesRunnable final : public Runnable {
+ public:
+  RegisterVoicesRunnable(OSXSpeechSynthesizerService* aSpeechService, nsTArray<OSXVoice>& aList)
+      : Runnable("RegisterVoicesRunnable"), mSpeechService(aSpeechService), mVoices(aList) {}
 
   NS_IMETHOD Run() override;
 
-private:
-  ~RegisterVoicesRunnable()
-  {
-  }
+ private:
+  ~RegisterVoicesRunnable() override = default;
 
   // This runnable always use sync mode.  It is unnecesarry to reference object
   OSXSpeechSynthesizerService* mSpeechService;
@@ -231,11 +224,9 @@ private:
 };
 
 NS_IMETHODIMP
-RegisterVoicesRunnable::Run()
-{
+RegisterVoicesRunnable::Run() {
   nsresult rv;
-  nsCOMPtr<nsISynthVoiceRegistry> registry =
-    do_GetService(NS_SYNTHVOICEREGISTRY_CONTRACTID, &rv);
+  nsCOMPtr<nsISynthVoiceRegistry> registry = do_GetService(NS_SYNTHVOICEREGISTRY_CONTRACTID, &rv);
   if (!registry) {
     return rv;
   }
@@ -256,28 +247,22 @@ RegisterVoicesRunnable::Run()
   return NS_OK;
 }
 
-class EnumVoicesRunnable final : public nsRunnable
-{
-public:
+class EnumVoicesRunnable final : public Runnable {
+ public:
   explicit EnumVoicesRunnable(OSXSpeechSynthesizerService* aSpeechService)
-    : mSpeechService(aSpeechService)
-  {
-  }
+      : Runnable("EnumVoicesRunnable"), mSpeechService(aSpeechService) {}
 
   NS_IMETHOD Run() override;
 
-private:
-  ~EnumVoicesRunnable()
-  {
-  }
+ private:
+  ~EnumVoicesRunnable() override = default;
 
   RefPtr<OSXSpeechSynthesizerService> mSpeechService;
 };
 
 NS_IMETHODIMP
-EnumVoicesRunnable::Run()
-{
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
+EnumVoicesRunnable::Run() {
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
   AutoTArray<OSXVoice, 64> list;
 
@@ -290,13 +275,11 @@ EnumVoicesRunnable::Run()
     NSDictionary* attr = [NSSpeechSynthesizer attributesForVoice:voice];
 
     nsAutoString identifier;
-    nsCocoaUtils::GetStringForNSString([attr objectForKey:NSVoiceIdentifier],
-                                       identifier);
+    nsCocoaUtils::GetStringForNSString([attr objectForKey:NSVoiceIdentifier], identifier);
 
     nsCocoaUtils::GetStringForNSString([attr objectForKey:NSVoiceName], item.mName);
 
-    nsCocoaUtils::GetStringForNSString(
-      [attr objectForKey:NSVoiceLocaleIdentifier], item.mLocale);
+    nsCocoaUtils::GetStringForNSString([attr objectForKey:NSVoiceLocaleIdentifier], item.mLocale);
     item.mLocale.ReplaceChar('_', '-');
 
     item.mUri.AssignLiteral("urn:moz-tts:osx:");
@@ -314,7 +297,7 @@ EnumVoicesRunnable::Run()
 
   return NS_OK;
 
-  NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
+  NS_OBJC_END_TRY_BLOCK_RETURN(NS_ERROR_FAILURE);
 }
 
 StaticRefPtr<OSXSpeechSynthesizerService> OSXSpeechSynthesizerService::sSingleton;
@@ -328,27 +311,18 @@ NS_INTERFACE_MAP_END
 NS_IMPL_ADDREF(OSXSpeechSynthesizerService)
 NS_IMPL_RELEASE(OSXSpeechSynthesizerService)
 
-OSXSpeechSynthesizerService::OSXSpeechSynthesizerService()
-  : mInitialized(false)
-{
-}
+OSXSpeechSynthesizerService::OSXSpeechSynthesizerService() : mInitialized(false) {}
 
-OSXSpeechSynthesizerService::~OSXSpeechSynthesizerService()
-{
-}
-
-bool
-OSXSpeechSynthesizerService::Init()
-{
+bool OSXSpeechSynthesizerService::Init() {
   if (Preferences::GetBool("media.webspeech.synth.test") ||
-      !Preferences::GetBool("media.webspeech.synth.enabled")) {
+      !StaticPrefs::media_webspeech_synth_enabled()) {
     // When test is enabled, we shouldn't add OS backend (Bug 1160844)
     return false;
   }
 
   nsCOMPtr<nsIThread> thread;
   if (NS_FAILED(NS_NewNamedThread("SpeechWorker", getter_AddRefs(thread)))) {
-  	return false;
+    return false;
   }
 
   // Get all the voices and register in the SynthVoiceRegistry
@@ -360,16 +334,11 @@ OSXSpeechSynthesizerService::Init()
 }
 
 NS_IMETHODIMP
-OSXSpeechSynthesizerService::Speak(const nsAString& aText,
-                                   const nsAString& aUri,
-                                   float aVolume,
-                                   float aRate,
-                                   float aPitch,
-                                   nsISpeechTask* aTask)
-{
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
+OSXSpeechSynthesizerService::Speak(const nsAString& aText, const nsAString& aUri, float aVolume,
+                                   float aRate, float aPitch, nsISpeechTask* aTask) {
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
-  MOZ_ASSERT(StringBeginsWith(aUri, NS_LITERAL_STRING("urn:moz-tts:osx:")),
+  MOZ_ASSERT(StringBeginsWith(aUri, u"urn:moz-tts:osx:"_ns),
              "OSXSpeechSynthesizerService doesn't allow this voice URI");
 
   NSSpeechSynthesizer* synth = [[NSSpeechSynthesizer alloc] init];
@@ -378,85 +347,84 @@ OSXSpeechSynthesizerService::Speak(const nsAString& aText,
   [synth setVoice:identifier];
 
   // default rate is 180-220
-  [synth setObject:[NSNumber numberWithInt:aRate * 200]
-         forProperty:NSSpeechRateProperty error:nil];
+  [synth setObject:[NSNumber numberWithInt:aRate * 200] forProperty:NSSpeechRateProperty error:nil];
   // volume allows 0.0-1.0
-  [synth setObject:[NSNumber numberWithFloat:aVolume]
-         forProperty:NSSpeechVolumeProperty error:nil];
+  [synth setObject:[NSNumber numberWithFloat:aVolume] forProperty:NSSpeechVolumeProperty error:nil];
   // Use default pitch value to calculate this
-  NSNumber* defaultPitch =
-    [synth objectForProperty:NSSpeechPitchBaseProperty error:nil];
+  NSNumber* defaultPitch = [synth objectForProperty:NSSpeechPitchBaseProperty error:nil];
   if (defaultPitch) {
     int newPitch = [defaultPitch intValue] * (aPitch / 2 + 0.5);
     [synth setObject:[NSNumber numberWithInt:newPitch]
-           forProperty:NSSpeechPitchBaseProperty error:nil];
+         forProperty:NSSpeechPitchBaseProperty
+               error:nil];
   }
 
-  RefPtr<SpeechTaskCallback> callback = new SpeechTaskCallback(aTask, synth);
-  nsresult rv = aTask->Setup(callback, 0, 0, 0);
+  nsAutoString escapedText;
+  // We need to map the the offsets from the given text to the escaped text.
+  // The index of the offsets array is the position in the escaped text,
+  // the element value is the position in the user-supplied text.
+  nsTArray<size_t> offsets;
+  offsets.SetCapacity(aText.Length());
+
+  // This loop looks for occurances of "[[" or "]]", escapes them, and
+  // populates the offsets array to supply a map to the original offsets.
+  for (size_t i = 0; i < aText.Length(); i++) {
+    if (aText.Length() > i + 1 &&
+        ((aText[i] == ']' && aText[i + 1] == ']') || (aText[i] == '[' && aText[i + 1] == '['))) {
+      escapedText.AppendLiteral(DLIM_ESCAPE_START);
+      offsets.AppendElements(strlen(DLIM_ESCAPE_START));
+      escapedText.Append(aText[i]);
+      offsets.AppendElement(i);
+      escapedText.Append(aText[++i]);
+      offsets.AppendElement(i);
+      escapedText.AppendLiteral(DLIM_ESCAPE_END);
+      offsets.AppendElements(strlen(DLIM_ESCAPE_END));
+    } else {
+      escapedText.Append(aText[i]);
+      offsets.AppendElement(i);
+    }
+  }
+
+  RefPtr<SpeechTaskCallback> callback = new SpeechTaskCallback(aTask, synth, offsets);
+  nsresult rv = aTask->Setup(callback);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  SpeechDelegate* delegate = [[SpeechDelegate alloc] initWithCallback:callback];
-  [synth setDelegate:delegate];
-  [delegate release ];
-
-  NSString* text = nsCocoaUtils::ToNSString(aText);
+  NSString* text = nsCocoaUtils::ToNSString(escapedText);
   BOOL success = [synth startSpeakingString:text];
   NS_ENSURE_TRUE(success, NS_ERROR_FAILURE);
 
   aTask->DispatchStart();
   return NS_OK;
 
-  NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
-}
-
-NS_IMETHODIMP
-OSXSpeechSynthesizerService::GetServiceType(SpeechServiceType* aServiceType)
-{
-  *aServiceType = nsISpeechService::SERVICETYPE_INDIRECT_AUDIO;
-  return NS_OK;
+  NS_OBJC_END_TRY_BLOCK_RETURN(NS_ERROR_FAILURE);
 }
 
 NS_IMETHODIMP
 OSXSpeechSynthesizerService::Observe(nsISupports* aSubject, const char* aTopic,
-                                     const char16_t* aData)
-{
+                                     const char16_t* aData) {
   return NS_OK;
 }
 
-OSXSpeechSynthesizerService*
-OSXSpeechSynthesizerService::GetInstance()
-{
+OSXSpeechSynthesizerService* OSXSpeechSynthesizerService::GetInstance() {
   MOZ_ASSERT(NS_IsMainThread());
   if (XRE_GetProcessType() != GeckoProcessType_Default) {
     return nullptr;
   }
 
   if (!sSingleton) {
-    RefPtr<OSXSpeechSynthesizerService> speechService =
-      new OSXSpeechSynthesizerService();
+    RefPtr<OSXSpeechSynthesizerService> speechService = new OSXSpeechSynthesizerService();
     if (speechService->Init()) {
       sSingleton = speechService;
+      ClearOnShutdown(&sSingleton);
     }
   }
   return sSingleton;
 }
 
-already_AddRefed<OSXSpeechSynthesizerService>
-OSXSpeechSynthesizerService::GetInstanceForService()
-{
+already_AddRefed<OSXSpeechSynthesizerService> OSXSpeechSynthesizerService::GetInstanceForService() {
   RefPtr<OSXSpeechSynthesizerService> speechService = GetInstance();
   return speechService.forget();
 }
 
-void
-OSXSpeechSynthesizerService::Shutdown()
-{
-  if (!sSingleton) {
-    return;
-  }
-  sSingleton = nullptr;
-}
-
-} // namespace dom
-} // namespace mozilla
+}  // namespace dom
+}  // namespace mozilla
