@@ -16,53 +16,52 @@
 #include "mozilla/dom/MessagePortBinding.h"
 #include "mozilla/dom/MessagePortChild.h"
 #include "mozilla/dom/PMessagePort.h"
+#include "mozilla/dom/RootedDictionary.h"
+#include "mozilla/dom/ScriptSettings.h"
 #include "mozilla/dom/StructuredCloneTags.h"
-#include "mozilla/dom/WorkerPrivate.h"
+#include "mozilla/dom/WorkerCommon.h"
+#include "mozilla/dom/WorkerRef.h"
 #include "mozilla/dom/WorkerScope.h"
 #include "mozilla/ipc/BackgroundChild.h"
+#include "mozilla/dom/RefMessageBodyService.h"
+#include "mozilla/dom/SharedMessageBody.h"
 #include "mozilla/ipc/PBackgroundChild.h"
 #include "mozilla/MessagePortTimelineMarker.h"
+#include "mozilla/ScopeExit.h"
 #include "mozilla/TimelineConsumers.h"
 #include "mozilla/TimelineMarker.h"
 #include "mozilla/Unused.h"
 #include "nsContentUtils.h"
 #include "nsGlobalWindow.h"
 #include "nsPresContext.h"
-#include "ScriptSettings.h"
-#include "SharedMessagePortMessage.h"
-
-#include "nsIBFCacheEntry.h"
-#include "nsIDocument.h"
-#include "nsIDOMFileList.h"
-#include "nsIPresShell.h"
-#include "nsISupportsPrimitives.h"
-#include "nsServiceManagerUtils.h"
 
 #ifdef XP_WIN
-#undef PostMessage
+#  undef PostMessage
 #endif
 
-using namespace mozilla::dom::workers;
+namespace mozilla::dom {
 
-namespace mozilla {
-namespace dom {
+void UniqueMessagePortId::ForceClose() {
+  if (!mIdentifier.neutered()) {
+    MessagePort::ForceClose(mIdentifier);
+    mIdentifier.neutered() = true;
+  }
+}
 
-class PostMessageRunnable final : public CancelableRunnable
-{
+class PostMessageRunnable final : public CancelableRunnable {
   friend class MessagePort;
 
-public:
-  PostMessageRunnable(MessagePort* aPort, SharedMessagePortMessage* aData)
-    : mPort(aPort)
-    , mData(aData)
-  {
+ public:
+  PostMessageRunnable(MessagePort* aPort, SharedMessageBody* aData)
+      : CancelableRunnable("dom::PostMessageRunnable"),
+        mPort(aPort),
+        mData(aData) {
     MOZ_ASSERT(aPort);
     MOZ_ASSERT(aData);
   }
 
   NS_IMETHOD
-  Run() override
-  {
+  Run() override {
     NS_ASSERT_OWNINGTHREAD(Runnable);
 
     // The port can be cycle collected while this runnable is pending in
@@ -73,7 +72,7 @@ public:
 
     MOZ_ASSERT(mPort->mPostMessageRunnable == this);
 
-    nsresult rv = DispatchMessage();
+    DispatchMessage();
 
     // We must check if we were waiting for this message in order to shutdown
     // the port.
@@ -82,12 +81,10 @@ public:
     mPort->mPostMessageRunnable = nullptr;
     mPort->Dispatch();
 
-    return rv;
+    return NS_OK;
   }
 
-  nsresult
-  Cancel() override
-  {
+  nsresult Cancel() override {
     NS_ASSERT_OWNINGTHREAD(Runnable);
 
     mPort = nullptr;
@@ -95,82 +92,79 @@ public:
     return NS_OK;
   }
 
-private:
-  nsresult
-  DispatchMessage() const
-  {
+ private:
+  void DispatchMessage() const {
     NS_ASSERT_OWNINGTHREAD(Runnable);
 
+    if (NS_FAILED(mPort->CheckCurrentGlobalCorrectness())) {
+      return;
+    }
+
     nsCOMPtr<nsIGlobalObject> globalObject = mPort->GetParentObject();
 
     AutoJSAPI jsapi;
     if (!globalObject || !jsapi.Init(globalObject)) {
       NS_WARNING("Failed to initialize AutoJSAPI object.");
-      return NS_ERROR_FAILURE;
+      return;
     }
 
     JSContext* cx = jsapi.cx();
 
-    ErrorResult rv;
+    IgnoredErrorResult rv;
     JS::Rooted<JS::Value> value(cx);
 
     UniquePtr<AbstractTimelineMarker> start;
     UniquePtr<AbstractTimelineMarker> end;
-    RefPtr<TimelineConsumers> timelines = TimelineConsumers::Get();
-    bool isTimelineRecording = timelines && !timelines->IsEmpty();
+    bool isTimelineRecording = !TimelineConsumers::IsEmpty();
 
     if (isTimelineRecording) {
       start = MakeUnique<MessagePortTimelineMarker>(
-        ProfileTimelineMessagePortOperationType::DeserializeData,
-        MarkerTracingType::START);
+          ProfileTimelineMessagePortOperationType::DeserializeData,
+          MarkerTracingType::START);
     }
 
-    mData->Read(cx, &value, rv);
+    mData->Read(cx, &value, mPort->mRefMessageBodyService,
+                SharedMessageBody::ReadMethod::StealRefMessageBody, rv);
 
     if (isTimelineRecording) {
       end = MakeUnique<MessagePortTimelineMarker>(
-        ProfileTimelineMessagePortOperationType::DeserializeData,
-        MarkerTracingType::END);
-      timelines->AddMarkerForAllObservedDocShells(start);
-      timelines->AddMarkerForAllObservedDocShells(end);
+          ProfileTimelineMessagePortOperationType::DeserializeData,
+          MarkerTracingType::END);
+      TimelineConsumers::AddMarkerForAllObservedDocShells(start);
+      TimelineConsumers::AddMarkerForAllObservedDocShells(end);
     }
 
     if (NS_WARN_IF(rv.Failed())) {
-      return rv.StealNSResult();
+      JS_ClearPendingException(cx);
+      mPort->DispatchError();
+      return;
     }
 
     // Create the event
     nsCOMPtr<mozilla::dom::EventTarget> eventTarget =
-      do_QueryInterface(mPort->GetOwner());
+        do_QueryInterface(mPort->GetOwner());
     RefPtr<MessageEvent> event =
-      new MessageEvent(eventTarget, nullptr, nullptr);
+        new MessageEvent(eventTarget, nullptr, nullptr);
 
     Sequence<OwningNonNull<MessagePort>> ports;
     if (!mData->TakeTransferredPortsAsSequence(ports)) {
-      return NS_ERROR_OUT_OF_MEMORY;
+      mPort->DispatchError();
+      return;
     }
 
-    Nullable<WindowProxyOrMessagePort> source;
-    source.SetValue().SetAsMessagePort() = mPort;
-
-    event->InitMessageEvent(nullptr, NS_LITERAL_STRING("message"),
-                            false /* non-bubbling */,
-                            false /* cancelable */, value, EmptyString(),
-                            EmptyString(), source, ports);
+    event->InitMessageEvent(nullptr, u"message"_ns, CanBubble::eNo,
+                            Cancelable::eNo, value, u""_ns, u""_ns, nullptr,
+                            ports);
     event->SetTrusted(true);
 
-    bool dummy;
-    mPort->DispatchEvent(static_cast<dom::Event*>(event.get()), &dummy);
-
-    return NS_OK;
+    mPort->DispatchEvent(*event);
   }
 
-private:
-  ~PostMessageRunnable()
-  {}
+ private:
+  ~PostMessageRunnable() = default;
 
   RefPtr<MessagePort> mPort;
-  RefPtr<SharedMessagePortMessage> mData;
+  RefPtr<SharedMessageBody> mData;
 };
 
 NS_IMPL_CYCLE_COLLECTION_CLASS(MessagePort)
@@ -181,9 +175,9 @@ NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN_INHERITED(MessagePort,
     NS_IMPL_CYCLE_COLLECTION_UNLINK(mPostMessageRunnable->mPort);
   }
 
-  NS_IMPL_CYCLE_COLLECTION_UNLINK(mMessages);
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mMessagesForTheOtherPort);
-  NS_IMPL_CYCLE_COLLECTION_UNLINK(mUnshippedEntangledPort);
+
+  tmp->CloseForced();
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN_INHERITED(MessagePort,
@@ -195,168 +189,84 @@ NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN_INHERITED(MessagePort,
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mUnshippedEntangledPort);
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
-NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION_INHERITED(MessagePort)
-  NS_INTERFACE_MAP_ENTRY(nsIIPCBackgroundChildCreateCallback)
-  NS_INTERFACE_MAP_ENTRY(nsIObserver)
+NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(MessagePort)
 NS_INTERFACE_MAP_END_INHERITING(DOMEventTargetHelper)
 
 NS_IMPL_ADDREF_INHERITED(MessagePort, DOMEventTargetHelper)
 NS_IMPL_RELEASE_INHERITED(MessagePort, DOMEventTargetHelper)
 
-namespace {
-
-class MessagePortWorkerHolder final : public workers::WorkerHolder
-{
-  MessagePort* mPort;
-
-public:
-  explicit MessagePortWorkerHolder(MessagePort* aPort)
-    : mPort(aPort)
-  {
-    MOZ_ASSERT(aPort);
-    MOZ_COUNT_CTOR(MessagePortWorkerHolder);
-  }
-
-  virtual bool Notify(workers::Status aStatus) override
-  {
-    if (aStatus > Running) {
-      // We cannot process messages anymore because we cannot dispatch new
-      // runnables. Let's force a Close().
-      mPort->CloseForced();
-    }
-
-    return true;
-  }
-
-private:
-  ~MessagePortWorkerHolder()
-  {
-    MOZ_COUNT_DTOR(MessagePortWorkerHolder);
-  }
-};
-
-class ForceCloseHelper final : public nsIIPCBackgroundChildCreateCallback
-{
-public:
-  NS_DECL_ISUPPORTS
-
-  static void ForceClose(const MessagePortIdentifier& aIdentifier)
-  {
-    PBackgroundChild* actor =
-      mozilla::ipc::BackgroundChild::GetForCurrentThread();
-    if (actor) {
-      Unused << actor->SendMessagePortForceClose(aIdentifier.uuid(),
-                                                 aIdentifier.destinationUuid(),
-                                                 aIdentifier.sequenceId());
-      return;
-    }
-
-    RefPtr<ForceCloseHelper> helper = new ForceCloseHelper(aIdentifier);
-    if (NS_WARN_IF(!mozilla::ipc::BackgroundChild::GetOrCreateForCurrentThread(helper))) {
-      MOZ_CRASH();
-    }
-  }
-
-private:
-  explicit ForceCloseHelper(const MessagePortIdentifier& aIdentifier)
-    : mIdentifier(aIdentifier)
-  {}
-
-  ~ForceCloseHelper() {}
-
-  void ActorFailed() override
-  {
-    MOZ_CRASH("Failed to create a PBackgroundChild actor!");
-  }
-
-  void ActorCreated(mozilla::ipc::PBackgroundChild* aActor) override
-  {
-    ForceClose(mIdentifier);
-  }
-
-  const MessagePortIdentifier mIdentifier;
-};
-
-NS_IMPL_ISUPPORTS(ForceCloseHelper, nsIIPCBackgroundChildCreateCallback)
-
-} // namespace
-
-MessagePort::MessagePort(nsIGlobalObject* aGlobal)
-  : DOMEventTargetHelper(aGlobal)
-  , mInnerID(0)
-  , mMessageQueueEnabled(false)
-  , mIsKeptAlive(false)
-{
+MessagePort::MessagePort(nsIGlobalObject* aGlobal, State aState)
+    : DOMEventTargetHelper(aGlobal),
+      mRefMessageBodyService(RefMessageBodyService::GetOrCreate()),
+      mState(aState),
+      mMessageQueueEnabled(false),
+      mIsKeptAlive(false),
+      mHasBeenTransferredOrClosed(false) {
   MOZ_ASSERT(aGlobal);
 
-  mIdentifier = new MessagePortIdentifier();
+  mIdentifier = MakeUnique<MessagePortIdentifier>();
   mIdentifier->neutered() = true;
   mIdentifier->sequenceId() = 0;
 }
 
-MessagePort::~MessagePort()
-{
+MessagePort::~MessagePort() {
   CloseForced();
-  MOZ_ASSERT(!mWorkerHolder);
+  MOZ_ASSERT(!mWorkerRef);
 }
 
-/* static */ already_AddRefed<MessagePort>
-MessagePort::Create(nsIGlobalObject* aGlobal, const nsID& aUUID,
-                    const nsID& aDestinationUUID, ErrorResult& aRv)
-{
+/* static */
+already_AddRefed<MessagePort> MessagePort::Create(nsIGlobalObject* aGlobal,
+                                                  const nsID& aUUID,
+                                                  const nsID& aDestinationUUID,
+                                                  ErrorResult& aRv) {
   MOZ_ASSERT(aGlobal);
 
-  RefPtr<MessagePort> mp = new MessagePort(aGlobal);
+  RefPtr<MessagePort> mp = new MessagePort(aGlobal, eStateUnshippedEntangled);
   mp->Initialize(aUUID, aDestinationUUID, 1 /* 0 is an invalid sequence ID */,
-                 false /* Neutered */, eStateUnshippedEntangled, aRv);
+                 false /* Neutered */, aRv);
   return mp.forget();
 }
 
-/* static */ already_AddRefed<MessagePort>
-MessagePort::Create(nsIGlobalObject* aGlobal,
-                    const MessagePortIdentifier& aIdentifier,
-                    ErrorResult& aRv)
-{
+/* static */
+already_AddRefed<MessagePort> MessagePort::Create(
+    nsIGlobalObject* aGlobal, UniqueMessagePortId& aIdentifier,
+    ErrorResult& aRv) {
   MOZ_ASSERT(aGlobal);
 
-  RefPtr<MessagePort> mp = new MessagePort(aGlobal);
+  RefPtr<MessagePort> mp = new MessagePort(aGlobal, eStateEntangling);
   mp->Initialize(aIdentifier.uuid(), aIdentifier.destinationUuid(),
-                 aIdentifier.sequenceId(), aIdentifier.neutered(),
-                 eStateEntangling, aRv);
+                 aIdentifier.sequenceId(), aIdentifier.neutered(), aRv);
+  aIdentifier.neutered() = true;
   return mp.forget();
 }
 
-void
-MessagePort::UnshippedEntangle(MessagePort* aEntangledPort)
-{
-  MOZ_ASSERT(aEntangledPort);
-  MOZ_ASSERT(!mUnshippedEntangledPort);
+void MessagePort::UnshippedEntangle(MessagePort* aEntangledPort) {
+  MOZ_DIAGNOSTIC_ASSERT(aEntangledPort);
+  MOZ_DIAGNOSTIC_ASSERT(!mUnshippedEntangledPort);
 
   mUnshippedEntangledPort = aEntangledPort;
 }
 
-void
-MessagePort::Initialize(const nsID& aUUID,
-                        const nsID& aDestinationUUID,
-                        uint32_t aSequenceID, bool mNeutered,
-                        State aState, ErrorResult& aRv)
-{
+void MessagePort::Initialize(const nsID& aUUID, const nsID& aDestinationUUID,
+                             uint32_t aSequenceID, bool aNeutered,
+                             ErrorResult& aRv) {
   MOZ_ASSERT(mIdentifier);
   mIdentifier->uuid() = aUUID;
   mIdentifier->destinationUuid() = aDestinationUUID;
   mIdentifier->sequenceId() = aSequenceID;
 
-  mState = aState;
-
-  if (mNeutered) {
+  if (aNeutered) {
     // If this port is neutered we don't want to keep it alive artificially nor
-    // we want to add listeners or workerWorkerHolders.
+    // we want to add listeners or WorkerRefs.
     mState = eStateDisentangled;
     return;
   }
 
   if (mState == eStateEntangling) {
-    ConnectToPBackground();
+    if (!ConnectToPBackground()) {
+      aRv.Throw(NS_ERROR_FAILURE);
+      return;
+    }
   } else {
     MOZ_ASSERT(mState == eStateUnshippedEntangled);
   }
@@ -364,104 +274,89 @@ MessagePort::Initialize(const nsID& aUUID,
   // The port has to keep itself alive until it's entangled.
   UpdateMustKeepAlive();
 
-  if (!NS_IsMainThread()) {
-    WorkerPrivate* workerPrivate = GetCurrentThreadWorkerPrivate();
-    MOZ_ASSERT(workerPrivate);
-    MOZ_ASSERT(!mWorkerHolder);
+  if (WorkerPrivate* workerPrivate = GetCurrentThreadWorkerPrivate()) {
+    RefPtr<MessagePort> self = this;
 
-    nsAutoPtr<WorkerHolder> workerHolder(new MessagePortWorkerHolder(this));
-    if (NS_WARN_IF(!workerHolder->HoldWorker(workerPrivate, Closing))) {
+    // When the callback is executed, we cannot process messages anymore because
+    // we cannot dispatch new runnables. Let's force a Close().
+    RefPtr<StrongWorkerRef> strongWorkerRef = StrongWorkerRef::Create(
+        workerPrivate, "MessagePort", [self]() { self->CloseForced(); });
+    if (NS_WARN_IF(!strongWorkerRef)) {
+      // The worker is shutting down.
+      mState = eStateDisentangled;
+      UpdateMustKeepAlive();
       aRv.Throw(NS_ERROR_FAILURE);
       return;
     }
 
-    mWorkerHolder = Move(workerHolder);
-  } else if (GetOwner()) {
-    MOZ_ASSERT(NS_IsMainThread());
-    MOZ_ASSERT(GetOwner()->IsInnerWindow());
-    mInnerID = GetOwner()->WindowID();
-
-    nsCOMPtr<nsIObserverService> obs = mozilla::services::GetObserverService();
-    if (obs) {
-      obs->AddObserver(this, "inner-window-destroyed", false);
-    }
+    MOZ_ASSERT(!mWorkerRef);
+    mWorkerRef = std::move(strongWorkerRef);
   }
 }
 
-JSObject*
-MessagePort::WrapObject(JSContext* aCx, JS::Handle<JSObject*> aGivenProto)
-{
-  return MessagePortBinding::Wrap(aCx, this, aGivenProto);
+JSObject* MessagePort::WrapObject(JSContext* aCx,
+                                  JS::Handle<JSObject*> aGivenProto) {
+  return MessagePort_Binding::Wrap(aCx, this, aGivenProto);
 }
 
-void
-MessagePort::PostMessage(JSContext* aCx, JS::Handle<JS::Value> aMessage,
-                         const Optional<Sequence<JS::Value>>& aTransferable,
-                         ErrorResult& aRv)
-{
+void MessagePort::PostMessage(JSContext* aCx, JS::Handle<JS::Value> aMessage,
+                              const Sequence<JSObject*>& aTransferable,
+                              ErrorResult& aRv) {
   // We *must* clone the data here, or the JS::Value could be modified
   // by script
 
-  JS::Rooted<JS::Value> transferable(aCx, JS::UndefinedValue());
-  if (aTransferable.WasPassed()) {
-    const Sequence<JS::Value>& realTransferable = aTransferable.Value();
-
-    // Here we want to check if the transerable object list contains
-    // this port. No other checks are done.
-    for (const JS::Value& value : realTransferable) {
-      if (!value.isObject()) {
-        continue;
-      }
-
-      MessagePort* port = nullptr;
-      nsresult rv = UNWRAP_OBJECT(MessagePort, &value.toObject(), port);
-      if (NS_FAILED(rv)) {
-        continue;
-      }
-
-      if (port == this) {
-        aRv.Throw(NS_ERROR_DOM_DATA_CLONE_ERR);
-        return;
-      }
+  // Here we want to check if the transerable object list contains
+  // this port.
+  for (uint32_t i = 0; i < aTransferable.Length(); ++i) {
+    JS::Rooted<JSObject*> object(aCx, aTransferable[i]);
+    if (!object) {
+      continue;
     }
 
-    // The input sequence only comes from the generated bindings code, which
-    // ensures it is rooted.
-    JS::HandleValueArray elements =
-      JS::HandleValueArray::fromMarkedLocation(realTransferable.Length(),
-                                               realTransferable.Elements());
-
-    JSObject* array =
-      JS_NewArrayObject(aCx, elements);
-    if (!array) {
-      aRv.Throw(NS_ERROR_OUT_OF_MEMORY);
+    MessagePort* port = nullptr;
+    nsresult rv = UNWRAP_OBJECT(MessagePort, &object, port);
+    if (NS_SUCCEEDED(rv) && port == this) {
+      aRv.Throw(NS_ERROR_DOM_DATA_CLONE_ERR);
       return;
     }
-
-    transferable.setObject(*array);
   }
 
-  RefPtr<SharedMessagePortMessage> data = new SharedMessagePortMessage();
+  JS::Rooted<JS::Value> transferable(aCx, JS::UndefinedValue());
+
+  aRv = nsContentUtils::CreateJSValueFromSequenceOfObject(aCx, aTransferable,
+                                                          &transferable);
+  if (NS_WARN_IF(aRv.Failed())) {
+    return;
+  }
+
+  Maybe<nsID> agentClusterId;
+  nsCOMPtr<nsIGlobalObject> global = GetOwnerGlobal();
+  if (global) {
+    agentClusterId = global->GetAgentClusterId();
+  }
+
+  RefPtr<SharedMessageBody> data = new SharedMessageBody(
+      StructuredCloneHolder::TransferringSupported, agentClusterId);
 
   UniquePtr<AbstractTimelineMarker> start;
   UniquePtr<AbstractTimelineMarker> end;
-  RefPtr<TimelineConsumers> timelines = TimelineConsumers::Get();
-  bool isTimelineRecording = timelines && !timelines->IsEmpty();
+  bool isTimelineRecording = !TimelineConsumers::IsEmpty();
 
   if (isTimelineRecording) {
     start = MakeUnique<MessagePortTimelineMarker>(
-      ProfileTimelineMessagePortOperationType::SerializeData,
-      MarkerTracingType::START);
+        ProfileTimelineMessagePortOperationType::SerializeData,
+        MarkerTracingType::START);
   }
 
-  data->Write(aCx, aMessage, transferable, aRv);
+  data->Write(aCx, aMessage, transferable, mIdentifier->uuid(),
+              mRefMessageBodyService, aRv);
 
   if (isTimelineRecording) {
     end = MakeUnique<MessagePortTimelineMarker>(
-      ProfileTimelineMessagePortOperationType::SerializeData,
-      MarkerTracingType::END);
-    timelines->AddMarkerForAllObservedDocShells(start);
-    timelines->AddMarkerForAllObservedDocShells(end);
+        ProfileTimelineMessagePortOperationType::SerializeData,
+        MarkerTracingType::END);
+    TimelineConsumers::AddMarkerForAllObservedDocShells(start);
+    TimelineConsumers::AddMarkerForAllObservedDocShells(end);
   }
 
   if (NS_WARN_IF(aRv.Failed())) {
@@ -475,7 +370,7 @@ MessagePort::PostMessage(JSContext* aCx, JS::Handle<JS::Value> aMessage,
 
   // If we are unshipped we are connected to the other port on the same thread.
   if (mState == eStateUnshippedEntangled) {
-    MOZ_ASSERT(mUnshippedEntangledPort);
+    MOZ_DIAGNOSTIC_ASSERT(mUnshippedEntangledPort);
     mUnshippedEntangledPort->mMessages.AppendElement(data);
     mUnshippedEntangledPort->Dispatch();
     return;
@@ -498,20 +393,25 @@ MessagePort::PostMessage(JSContext* aCx, JS::Handle<JS::Value> aMessage,
   MOZ_ASSERT(mActor);
   MOZ_ASSERT(mMessagesForTheOtherPort.IsEmpty());
 
-  AutoTArray<RefPtr<SharedMessagePortMessage>, 1> array;
+  AutoTArray<RefPtr<SharedMessageBody>, 1> array;
   array.AppendElement(data);
 
-  AutoTArray<ClonedMessageData, 1> messages;
+  AutoTArray<MessageData, 1> messages;
   // note: `messages` will borrow the underlying buffer, but this is okay
   // because reverse destruction order means `messages` will be destroyed prior
   // to `array`/`data`.
-  SharedMessagePortMessage::FromSharedToMessagesChild(mActor, array, messages);
+  SharedMessageBody::FromSharedToMessagesChild(mActor->Manager(), array,
+                                               messages);
   mActor->SendPostMessages(messages);
 }
 
-void
-MessagePort::Start()
-{
+void MessagePort::PostMessage(JSContext* aCx, JS::Handle<JS::Value> aMessage,
+                              const StructuredSerializeOptions& aOptions,
+                              ErrorResult& aRv) {
+  PostMessage(aCx, aMessage, aOptions.mTransfer, aRv);
+}
+
+void MessagePort::Start() {
   if (mMessageQueueEnabled) {
     return;
   }
@@ -520,9 +420,7 @@ MessagePort::Start()
   Dispatch();
 }
 
-void
-MessagePort::Dispatch()
-{
+void MessagePort::Dispatch() {
   if (!mMessageQueueEnabled || mMessages.IsEmpty() || mPostMessageRunnable) {
     return;
   }
@@ -572,47 +470,44 @@ MessagePort::Dispatch()
       break;
   }
 
-  RefPtr<SharedMessagePortMessage> data = mMessages.ElementAt(0);
+  RefPtr<SharedMessageBody> data = mMessages.ElementAt(0);
   mMessages.RemoveElementAt(0);
 
   mPostMessageRunnable = new PostMessageRunnable(this, data);
 
   nsCOMPtr<nsIGlobalObject> global = GetOwnerGlobal();
   if (NS_IsMainThread() && global) {
-    MOZ_ALWAYS_SUCCEEDS(global->Dispatch("MessagePortMessage", TaskCategory::Other, do_AddRef(mPostMessageRunnable)));
+    MOZ_ALWAYS_SUCCEEDS(
+        global->Dispatch(TaskCategory::Other, do_AddRef(mPostMessageRunnable)));
     return;
   }
 
   MOZ_ALWAYS_SUCCEEDS(NS_DispatchToCurrentThread(mPostMessageRunnable));
 }
 
-void
-MessagePort::Close()
-{
+void MessagePort::Close() {
+  mHasBeenTransferredOrClosed = true;
   CloseInternal(true /* aSoftly */);
 }
 
-void
-MessagePort::CloseForced()
-{
-  CloseInternal(false /* aSoftly */);
-}
+void MessagePort::CloseForced() { CloseInternal(false /* aSoftly */); }
 
-void
-MessagePort::CloseInternal(bool aSoftly)
-{
+void MessagePort::CloseInternal(bool aSoftly) {
   // If we have some messages to send but we don't want a 'soft' close, we have
   // to flush them now.
   if (!aSoftly) {
     mMessages.Clear();
   }
 
+  // Let's inform the RefMessageBodyService that any our shared messages are
+  // now invalid.
+  mRefMessageBodyService->ForgetPort(mIdentifier->uuid());
+
   if (mState == eStateUnshippedEntangled) {
-    MOZ_ASSERT(mUnshippedEntangledPort);
+    MOZ_DIAGNOSTIC_ASSERT(mUnshippedEntangledPort);
 
     // This avoids loops.
-    RefPtr<MessagePort> port = Move(mUnshippedEntangledPort);
-    MOZ_ASSERT(mUnshippedEntangledPort == nullptr);
+    RefPtr<MessagePort> port = std::move(mUnshippedEntangledPort);
 
     mState = eStateDisentangledForClose;
     port->CloseInternal(aSoftly);
@@ -657,23 +552,12 @@ MessagePort::CloseInternal(bool aSoftly)
   UpdateMustKeepAlive();
 }
 
-EventHandlerNonNull*
-MessagePort::GetOnmessage()
-{
-  if (NS_IsMainThread()) {
-    return GetEventHandler(nsGkAtoms::onmessage, EmptyString());
-  }
-  return GetEventHandler(nullptr, NS_LITERAL_STRING("message"));
+EventHandlerNonNull* MessagePort::GetOnmessage() {
+  return GetEventHandler(nsGkAtoms::onmessage);
 }
 
-void
-MessagePort::SetOnmessage(EventHandlerNonNull* aCallback)
-{
-  if (NS_IsMainThread()) {
-    SetEventHandler(nsGkAtoms::onmessage, EmptyString(), aCallback);
-  } else {
-    SetEventHandler(nullptr, NS_LITERAL_STRING("message"), aCallback);
-  }
+void MessagePort::SetOnmessage(EventHandlerNonNull* aCallback) {
+  SetEventHandler(nsGkAtoms::onmessage, aCallback);
 
   // When using onmessage, the call to start() is implied.
   Start();
@@ -683,9 +567,7 @@ MessagePort::SetOnmessage(EventHandlerNonNull* aCallback)
 // another actor. It receives a list of messages to be dispatch. It can be that
 // we were waiting for this entangling step in order to disentangle the port or
 // to close it.
-void
-MessagePort::Entangled(nsTArray<ClonedMessageData>& aMessages)
-{
+void MessagePort::Entangled(nsTArray<MessageData>& aMessages) {
   MOZ_ASSERT(mState == eStateEntangling ||
              mState == eStateEntanglingForDisentangle ||
              mState == eStateEntanglingForClose);
@@ -696,10 +578,9 @@ MessagePort::Entangled(nsTArray<ClonedMessageData>& aMessages)
   // If we have pending messages, these have to be sent.
   if (!mMessagesForTheOtherPort.IsEmpty()) {
     {
-      nsTArray<ClonedMessageData> messages;
-      SharedMessagePortMessage::FromSharedToMessagesChild(mActor,
-                                                          mMessagesForTheOtherPort,
-                                                          messages);
+      nsTArray<MessageData> messages;
+      SharedMessageBody::FromSharedToMessagesChild(
+          mActor->Manager(), mMessagesForTheOtherPort, messages);
       mActor->SendPostMessages(messages);
     }
     // Because `messages` borrow the underlying JSStructuredCloneData buffers,
@@ -707,11 +588,11 @@ MessagePort::Entangled(nsTArray<ClonedMessageData>& aMessages)
     mMessagesForTheOtherPort.Clear();
   }
 
-  // We must convert the messages into SharedMessagePortMessages to avoid leaks.
-  FallibleTArray<RefPtr<SharedMessagePortMessage>> data;
-  if (NS_WARN_IF(!SharedMessagePortMessage::FromMessagesToSharedChild(aMessages,
-                                                                      data))) {
-    // OOM, we cannot continue.
+  // We must convert the messages into SharedMessageBodys to avoid leaks.
+  FallibleTArray<RefPtr<SharedMessageBody>> data;
+  if (NS_WARN_IF(
+          !SharedMessageBody::FromMessagesToSharedChild(aMessages, data))) {
+    DispatchError();
     return;
   }
 
@@ -734,9 +615,7 @@ MessagePort::Entangled(nsTArray<ClonedMessageData>& aMessages)
   Dispatch();
 }
 
-void
-MessagePort::StartDisentangling()
-{
+void MessagePort::StartDisentangling() {
   MOZ_ASSERT(mActor);
   MOZ_ASSERT(mState == eStateEntangled);
 
@@ -749,11 +628,8 @@ MessagePort::StartDisentangling()
   mActor->SendStopSendingData();
 }
 
-void
-MessagePort::MessagesReceived(nsTArray<ClonedMessageData>& aMessages)
-{
-  MOZ_ASSERT(mState == eStateEntangled ||
-             mState == eStateDisentangling ||
+void MessagePort::MessagesReceived(nsTArray<MessageData>& aMessages) {
+  MOZ_ASSERT(mState == eStateEntangled || mState == eStateDisentangling ||
              // This last step can happen only if Close() has been called
              // manually. At this point SendClose() is sent but we can still
              // receive something until the Closing request is processed.
@@ -762,10 +638,10 @@ MessagePort::MessagesReceived(nsTArray<ClonedMessageData>& aMessages)
 
   RemoveDocFromBFCache();
 
-  FallibleTArray<RefPtr<SharedMessagePortMessage>> data;
-  if (NS_WARN_IF(!SharedMessagePortMessage::FromMessagesToSharedChild(aMessages,
-                                                                      data))) {
-    // OOM, We cannot continue.
+  FallibleTArray<RefPtr<SharedMessageBody>> data;
+  if (NS_WARN_IF(
+          !SharedMessageBody::FromMessagesToSharedChild(aMessages, data))) {
+    DispatchError();
     return;
   }
 
@@ -776,31 +652,32 @@ MessagePort::MessagesReceived(nsTArray<ClonedMessageData>& aMessages)
   }
 }
 
-void
-MessagePort::StopSendingDataConfirmed()
-{
+void MessagePort::StopSendingDataConfirmed() {
   MOZ_ASSERT(mState == eStateDisentangling);
   MOZ_ASSERT(mActor);
 
   Disentangle();
 }
 
-void
-MessagePort::Disentangle()
-{
+void MessagePort::Disentangle() {
   MOZ_ASSERT(mState == eStateDisentangling);
   MOZ_ASSERT(mActor);
 
   mState = eStateDisentangled;
 
   {
-    nsTArray<ClonedMessageData> messages;
-    SharedMessagePortMessage::FromSharedToMessagesChild(mActor, mMessages,
-                                                        messages);
+    nsTArray<MessageData> messages;
+    SharedMessageBody::FromSharedToMessagesChild(mActor->Manager(), mMessages,
+                                                 messages);
     mActor->SendDisentangle(messages);
   }
-  // Only clear mMessages after the ClonedMessageData instances have gone out of
-  // scope because they borrow mMessages' underlying JSStructuredCloneDatas.
+
+  // Let's inform the RefMessageBodyService that any our shared messages are
+  // now invalid.
+  mRefMessageBodyService->ForgetPort(mIdentifier->uuid());
+
+  // Only clear mMessages after the MessageData instances have gone out of scope
+  // because they borrow mMessages' underlying JSStructuredCloneDatas.
   mMessages.Clear();
 
   mActor->SetPort(nullptr);
@@ -809,10 +686,11 @@ MessagePort::Disentangle()
   UpdateMustKeepAlive();
 }
 
-void
-MessagePort::CloneAndDisentangle(MessagePortIdentifier& aIdentifier)
-{
+void MessagePort::CloneAndDisentangle(UniqueMessagePortId& aIdentifier) {
   MOZ_ASSERT(mIdentifier);
+  MOZ_ASSERT(!mHasBeenTransferredOrClosed);
+
+  mHasBeenTransferredOrClosed = true;
 
   // We can clone a port that has already been transfered. In this case, on the
   // otherside will have a neutered port. Here we set neutered to true so that
@@ -840,9 +718,15 @@ MessagePort::CloneAndDisentangle(MessagePortIdentifier& aIdentifier)
     MOZ_ASSERT(mUnshippedEntangledPort);
     MOZ_ASSERT(mMessagesForTheOtherPort.IsEmpty());
 
+    RefPtr<MessagePort> port = std::move(mUnshippedEntangledPort);
+
     // Disconnect the entangled port and connect it to PBackground.
-    mUnshippedEntangledPort->ConnectToPBackground();
-    mUnshippedEntangledPort = nullptr;
+    if (!port->ConnectToPBackground()) {
+      // We are probably shutting down. We cannot proceed.
+      mState = eStateDisentangled;
+      UpdateMustKeepAlive();
+      return;
+    }
 
     // In this case, we don't need to be connected to the PBackground service.
     if (mMessages.IsEmpty()) {
@@ -854,7 +738,10 @@ MessagePort::CloneAndDisentangle(MessagePortIdentifier& aIdentifier)
     }
 
     // Register this component to PBackground.
-    ConnectToPBackground();
+    if (!ConnectToPBackground()) {
+      // We are probably shutting down. We cannot proceed.
+      return;
+    }
 
     mState = eStateEntanglingForDisentangle;
     return;
@@ -870,9 +757,7 @@ MessagePort::CloneAndDisentangle(MessagePortIdentifier& aIdentifier)
   StartDisentangling();
 }
 
-void
-MessagePort::Closed()
-{
+void MessagePort::Closed() {
   if (mState >= eStateDisentangled) {
     return;
   }
@@ -887,68 +772,42 @@ MessagePort::Closed()
   UpdateMustKeepAlive();
 }
 
-void
-MessagePort::ConnectToPBackground()
-{
-  mState = eStateEntangling;
+bool MessagePort::ConnectToPBackground() {
+  RefPtr<MessagePort> self = this;
+  auto raii = MakeScopeExit([self] {
+    self->mState = eStateDisentangled;
+    self->UpdateMustKeepAlive();
+  });
 
-  PBackgroundChild* actor =
-    mozilla::ipc::BackgroundChild::GetForCurrentThread();
-  if (actor) {
-    ActorCreated(actor);
-  } else {
-    if (NS_WARN_IF(
-        !mozilla::ipc::BackgroundChild::GetOrCreateForCurrentThread(this))) {
-      MOZ_CRASH();
-    }
+  mozilla::ipc::PBackgroundChild* actorChild =
+      mozilla::ipc::BackgroundChild::GetOrCreateForCurrentThread();
+  if (NS_WARN_IF(!actorChild)) {
+    return false;
   }
-}
 
-void
-MessagePort::ActorFailed()
-{
-  MOZ_CRASH("Failed to create a PBackgroundChild actor!");
-}
-
-void
-MessagePort::ActorCreated(mozilla::ipc::PBackgroundChild* aActor)
-{
-  MOZ_ASSERT(aActor);
-  MOZ_ASSERT(!mActor);
-  MOZ_ASSERT(mIdentifier);
-  MOZ_ASSERT(mState == eStateEntangling ||
-             mState == eStateEntanglingForDisentangle ||
-             mState == eStateEntanglingForClose);
-
-  PMessagePortChild* actor =
-    aActor->SendPMessagePortConstructor(mIdentifier->uuid(),
-                                        mIdentifier->destinationUuid(),
-                                        mIdentifier->sequenceId());
+  PMessagePortChild* actor = actorChild->SendPMessagePortConstructor(
+      mIdentifier->uuid(), mIdentifier->destinationUuid(),
+      mIdentifier->sequenceId());
+  if (NS_WARN_IF(!actor)) {
+    return false;
+  }
 
   mActor = static_cast<MessagePortChild*>(actor);
   MOZ_ASSERT(mActor);
 
   mActor->SetPort(this);
+  mState = eStateEntangling;
+
+  raii.release();
+  return true;
 }
 
-void
-MessagePort::UpdateMustKeepAlive()
-{
-  if (mState >= eStateDisentangled &&
-      mMessages.IsEmpty() &&
-      mIsKeptAlive) {
+void MessagePort::UpdateMustKeepAlive() {
+  if (mState >= eStateDisentangled && mMessages.IsEmpty() && mIsKeptAlive) {
     mIsKeptAlive = false;
 
-    // The DTOR of this WorkerHolder will release the worker for us.
-    mWorkerHolder = nullptr;
-
-    if (NS_IsMainThread()) {
-      nsCOMPtr<nsIObserverService> obs =
-        do_GetService("@mozilla.org/observer-service;1");
-      if (obs) {
-        obs->RemoveObserver(this, "inner-window-destroyed");
-      }
-    }
+    // The DTOR of this WorkerRef will release the worker for us.
+    mWorkerRef = nullptr;
 
     Release();
     return;
@@ -960,66 +819,52 @@ MessagePort::UpdateMustKeepAlive()
   }
 }
 
-NS_IMETHODIMP
-MessagePort::Observe(nsISupports* aSubject, const char* aTopic,
-                     const char16_t* aData)
-{
-  MOZ_ASSERT(NS_IsMainThread());
-
-  if (strcmp(aTopic, "inner-window-destroyed")) {
-    return NS_OK;
-  }
-
-  // If the window id destroyed we have to release the reference that we are
-  // keeping.
-  if (!mIsKeptAlive) {
-    return NS_OK;
-  }
-
-  nsCOMPtr<nsISupportsPRUint64> wrapper = do_QueryInterface(aSubject);
-  NS_ENSURE_TRUE(wrapper, NS_ERROR_FAILURE);
-
-  uint64_t innerID;
-  nsresult rv = wrapper->GetData(&innerID);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  if (innerID == mInnerID) {
-    CloseForced();
-  }
-
-  return NS_OK;
+void MessagePort::DisconnectFromOwner() {
+  CloseForced();
+  DOMEventTargetHelper::DisconnectFromOwner();
 }
 
-void
-MessagePort::RemoveDocFromBFCache()
-{
+void MessagePort::RemoveDocFromBFCache() {
   if (!NS_IsMainThread()) {
     return;
   }
 
-  nsPIDOMWindowInner* window = GetOwner();
-  if (!window) {
-    return;
+  if (nsPIDOMWindowInner* window = GetOwner()) {
+    window->RemoveFromBFCacheSync();
   }
-
-  nsIDocument* doc = window->GetExtantDoc();
-  if (!doc) {
-    return;
-  }
-
-  nsCOMPtr<nsIBFCacheEntry> bfCacheEntry = doc->GetBFCacheEntry();
-  if (!bfCacheEntry) {
-    return;
-  }
-
-  bfCacheEntry->RemoveFromBFCacheSync();
 }
 
-/* static */ void
-MessagePort::ForceClose(const MessagePortIdentifier& aIdentifier)
-{
-  ForceCloseHelper::ForceClose(aIdentifier);
+/* static */
+void MessagePort::ForceClose(const MessagePortIdentifier& aIdentifier) {
+  mozilla::ipc::PBackgroundChild* actorChild =
+      mozilla::ipc::BackgroundChild::GetOrCreateForCurrentThread();
+  if (NS_WARN_IF(!actorChild)) {
+    MOZ_CRASH("Failed to create a PBackgroundChild actor!");
+  }
+
+  Unused << actorChild->SendMessagePortForceClose(aIdentifier.uuid(),
+                                                  aIdentifier.destinationUuid(),
+                                                  aIdentifier.sequenceId());
 }
 
-} // namespace dom
-} // namespace mozilla
+void MessagePort::DispatchError() {
+  nsCOMPtr<nsIGlobalObject> globalObject = GetParentObject();
+
+  AutoJSAPI jsapi;
+  if (!globalObject || !jsapi.Init(globalObject)) {
+    NS_WARNING("Failed to initialize AutoJSAPI object.");
+    return;
+  }
+
+  RootedDictionary<MessageEventInit> init(jsapi.cx());
+  init.mBubbles = false;
+  init.mCancelable = false;
+
+  RefPtr<Event> event =
+      MessageEvent::Constructor(this, u"messageerror"_ns, init);
+  event->SetTrusted(true);
+
+  DispatchEvent(*event);
+}
+
+}  // namespace mozilla::dom
