@@ -1,4 +1,5 @@
-/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- *//* vim: set ts=8 sts=2 et sw=2 tw=80: */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -9,75 +10,25 @@
 #include "mozilla/DOMEventTargetHelper.h"
 #include "mozilla/UniquePtr.h"
 #include "mozilla/dom/NotificationBinding.h"
-#include "mozilla/dom/workers/bindings/WorkerFeature.h"
+#include "mozilla/dom/WorkerPrivate.h"
 
 #include "nsIObserver.h"
+#include "nsISupports.h"
 
 #include "nsCycleCollectionParticipant.h"
 #include "nsHashKeys.h"
 #include "nsTHashtable.h"
 #include "nsWeakReference.h"
 
-#define NOTIFICATIONTELEMETRYSERVICE_CONTRACTID \
-  "@mozilla.org/notificationTelemetryService;1"
-
 class nsIPrincipal;
 class nsIVariant;
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 class NotificationRef;
 class WorkerNotificationObserver;
 class Promise;
-
-namespace workers {
-  class WorkerPrivate;
-} // namespace workers
-
-class Notification;
-class NotificationFeature final : public workers::WorkerFeature
-{
-  // Since the feature is strongly held by a Notification, it is ok to hold
-  // a raw pointer here.
-  Notification* mNotification;
-
-public:
-  explicit NotificationFeature(Notification* aNotification);
-
-  bool
-  Notify(JSContext* aCx, workers::Status aStatus) override;
-};
-
-// Records telemetry probes at application startup, when a notification is
-// shown, and when the notification permission is revoked for a site.
-class NotificationTelemetryService final : public nsIObserver
-{
-public:
-  NS_DECL_ISUPPORTS
-  NS_DECL_NSIOBSERVER
-
-  NotificationTelemetryService();
-
-  static already_AddRefed<NotificationTelemetryService> GetInstance();
-
-  nsresult Init();
-  void RecordDNDSupported();
-  void RecordPermissions();
-  nsresult RecordSender(nsIPrincipal* aPrincipal);
-
-private:
-  virtual ~NotificationTelemetryService();
-
-  nsresult AddPermissionChangeObserver();
-  nsresult RemovePermissionChangeObserver();
-
-  bool GetNotificationPermission(nsISupports* aSupports,
-                                 uint32_t* aCapability);
-
-  bool mDNDRecorded;
-  nsTHashtable<nsStringHashKey> mOrigins;
-};
+class StrongWorkerRef;
 
 /*
  * Notifications on workers introduce some lifetime issues. The property we
@@ -131,10 +82,9 @@ private:
  * dispatch a control runnable instead.
  *
  */
-class Notification : public DOMEventTargetHelper
-                   , public nsIObserver
-                   , public nsSupportsWeakReference
-{
+class Notification : public DOMEventTargetHelper,
+                     public nsIObserver,
+                     public nsSupportsWeakReference {
   friend class CloseNotificationRunnable;
   friend class NotificationTask;
   friend class NotificationPermissionRequest;
@@ -143,26 +93,25 @@ class Notification : public DOMEventTargetHelper
   friend class ServiceWorkerNotificationObserver;
   friend class WorkerGetRunnable;
   friend class WorkerNotificationObserver;
-  friend class NotificationTelemetryService;
 
-public:
+ public:
   IMPL_EVENT_HANDLER(click)
   IMPL_EVENT_HANDLER(show)
   IMPL_EVENT_HANDLER(error)
   IMPL_EVENT_HANDLER(close)
 
   NS_DECL_ISUPPORTS_INHERITED
-  NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS_INHERITED(Notification, DOMEventTargetHelper)
+  NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS_INHERITED(Notification,
+                                                         DOMEventTargetHelper)
   NS_DECL_NSIOBSERVER
 
   static bool PrefEnabled(JSContext* aCx, JSObject* aObj);
   // Returns if Notification.get() is allowed for the current global.
   static bool IsGetEnabled(JSContext* aCx, JSObject* aObj);
 
-  static already_AddRefed<Notification> Constructor(const GlobalObject& aGlobal,
-                                                    const nsAString& aTitle,
-                                                    const NotificationOptions& aOption,
-                                                    ErrorResult& aRv);
+  static already_AddRefed<Notification> Constructor(
+      const GlobalObject& aGlobal, const nsAString& aTitle,
+      const NotificationOptions& aOption, ErrorResult& aRv);
 
   /**
    * Used when dispatching the ServiceWorkerEvent.
@@ -174,88 +123,53 @@ public:
    * 2) The default binding requires main thread for parsing the JSON from the
    *    string behavior.
    */
-  static already_AddRefed<Notification>
-  ConstructFromFields(
-    nsIGlobalObject* aGlobal,
-    const nsAString& aID,
-    const nsAString& aTitle,
-    const nsAString& aDir,
-    const nsAString& aLang,
-    const nsAString& aBody,
-    const nsAString& aTag,
-    const nsAString& aIcon,
-    const nsAString& aData,
-    const nsAString& aServiceWorkerRegistrationID,
-    ErrorResult& aRv);
+  static already_AddRefed<Notification> ConstructFromFields(
+      nsIGlobalObject* aGlobal, const nsAString& aID, const nsAString& aTitle,
+      const nsAString& aDir, const nsAString& aLang, const nsAString& aBody,
+      const nsAString& aTag, const nsAString& aIcon, const nsAString& aData,
+      const nsAString& aServiceWorkerRegistrationScope, ErrorResult& aRv);
 
-  void GetID(nsAString& aRetval) {
-    aRetval = mID;
-  }
+  void GetID(nsAString& aRetval) { aRetval = mID; }
 
-  void GetTitle(nsAString& aRetval)
-  {
-    aRetval = mTitle;
-  }
+  void GetTitle(nsAString& aRetval) { aRetval = mTitle; }
 
-  NotificationDirection Dir()
-  {
-    return mDir;
-  }
+  NotificationDirection Dir() { return mDir; }
 
-  void GetLang(nsAString& aRetval)
-  {
-    aRetval = mLang;
-  }
+  void GetLang(nsAString& aRetval) { aRetval = mLang; }
 
-  void GetBody(nsAString& aRetval)
-  {
-    aRetval = mBody;
-  }
+  void GetBody(nsAString& aRetval) { aRetval = mBody; }
 
-  void GetTag(nsAString& aRetval)
-  {
-    aRetval = mTag;
-  }
+  void GetTag(nsAString& aRetval) { aRetval = mTag; }
 
-  void GetIcon(nsAString& aRetval)
-  {
-    aRetval = mIconUrl;
-  }
+  void GetIcon(nsAString& aRetval) { aRetval = mIconUrl; }
 
-  void SetStoredState(bool val)
-  {
-    mIsStored = val;
-  }
+  void SetStoredState(bool val) { mIsStored = val; }
 
-  bool IsStored()
-  {
-    return mIsStored;
-  }
+  bool IsStored() { return mIsStored; }
 
-  static bool RequestPermissionEnabledForScope(JSContext* aCx, JSObject* /* unused */);
+  static bool RequestPermissionEnabledForScope(JSContext* aCx,
+                                               JSObject* /* unused */);
 
-  static already_AddRefed<Promise>
-  RequestPermission(const GlobalObject& aGlobal,
-                    const Optional<OwningNonNull<NotificationPermissionCallback> >& aCallback,
-                    ErrorResult& aRv);
+  static already_AddRefed<Promise> RequestPermission(
+      const GlobalObject& aGlobal,
+      const Optional<OwningNonNull<NotificationPermissionCallback> >& aCallback,
+      ErrorResult& aRv);
 
   static NotificationPermission GetPermission(const GlobalObject& aGlobal,
                                               ErrorResult& aRv);
 
-  static already_AddRefed<Promise>
-  Get(nsPIDOMWindowInner* aWindow,
-      const GetNotificationOptions& aFilter,
-      const nsAString& aScope,
-      ErrorResult& aRv);
+  static already_AddRefed<Promise> Get(nsPIDOMWindowInner* aWindow,
+                                       const GetNotificationOptions& aFilter,
+                                       const nsAString& aScope,
+                                       ErrorResult& aRv);
 
   static already_AddRefed<Promise> Get(const GlobalObject& aGlobal,
                                        const GetNotificationOptions& aFilter,
                                        ErrorResult& aRv);
 
-  static already_AddRefed<Promise> WorkerGet(workers::WorkerPrivate* aWorkerPrivate,
-                                             const GetNotificationOptions& aFilter,
-                                             const nsAString& aScope,
-                                             ErrorResult& aRv);
+  static already_AddRefed<Promise> WorkerGet(
+      WorkerPrivate* aWorkerPrivate, const GetNotificationOptions& aFilter,
+      const nsAString& aScope, ErrorResult& aRv);
 
   // Notification implementation of
   // ServiceWorkerRegistration.showNotification.
@@ -263,37 +177,36 @@ public:
   //
   // Note that aCx may not be in the compartment of aGlobal, but aOptions will
   // have its JS things in the compartment of aCx.
-  static already_AddRefed<Promise>
-  ShowPersistentNotification(JSContext* aCx,
-                             nsIGlobalObject* aGlobal,
-                             const nsAString& aScope,
-                             const nsAString& aTitle,
-                             const NotificationOptions& aOptions,
-                             ErrorResult& aRv);
+  static already_AddRefed<Promise> ShowPersistentNotification(
+      JSContext* aCx, nsIGlobalObject* aGlobal, const nsAString& aScope,
+      const nsAString& aTitle, const NotificationOptions& aOptions,
+      const ServiceWorkerRegistrationDescriptor& aDescriptor, ErrorResult& aRv);
 
   void Close();
 
-  nsPIDOMWindowInner* GetParentObject()
-  {
-    return GetOwner();
-  }
+  nsPIDOMWindowInner* GetParentObject() { return GetOwner(); }
 
-  virtual JSObject* WrapObject(JSContext* aCx, JS::Handle<JSObject*> aGivenProto) override;
+  virtual JSObject* WrapObject(JSContext* aCx,
+                               JS::Handle<JSObject*> aGivenProto) override;
+
+  bool RequireInteraction() const;
+
+  bool Silent() const;
+
+  void GetVibrate(nsTArray<uint32_t>& aRetval) const;
 
   void GetData(JSContext* aCx, JS::MutableHandle<JS::Value> aRetval);
 
-  void InitFromJSVal(JSContext* aCx, JS::Handle<JS::Value> aData, ErrorResult& aRv);
+  void InitFromJSVal(JSContext* aCx, JS::Handle<JS::Value> aData,
+                     ErrorResult& aRv);
 
   void InitFromBase64(const nsAString& aData, ErrorResult& aRv);
 
-  void AssertIsOnTargetThread() const
-  {
-    MOZ_ASSERT(IsTargetThread());
-  }
+  void AssertIsOnTargetThread() const { MOZ_ASSERT(IsTargetThread()); }
 
   // Initialized on the worker thread, never unset, and always used in
   // a read-only capacity. Used on any thread.
-  workers::WorkerPrivate* mWorkerPrivate;
+  CheckedUnsafePtr<WorkerPrivate> mWorkerPrivate;
 
   // Main thread only.
   WorkerNotificationObserver* mObserver;
@@ -320,44 +233,45 @@ public:
   static NotificationPermission TestPermission(nsIPrincipal* aPrincipal);
 
   bool DispatchClickEvent();
-  bool DispatchNotificationClickEvent();
 
   static nsresult RemovePermission(nsIPrincipal* aPrincipal);
   static nsresult OpenSettings(nsIPrincipal* aPrincipal);
-protected:
+
+  nsresult DispatchToMainThread(already_AddRefed<nsIRunnable>&& aRunnable);
+
+ protected:
   Notification(nsIGlobalObject* aGlobal, const nsAString& aID,
                const nsAString& aTitle, const nsAString& aBody,
                NotificationDirection aDir, const nsAString& aLang,
                const nsAString& aTag, const nsAString& aIconUrl,
+               bool aRequireInteraction, bool aSilent,
+               nsTArray<uint32_t>&& aVibrate,
                const NotificationBehavior& aBehavior);
 
-  static already_AddRefed<Notification> CreateInternal(nsIGlobalObject* aGlobal,
-                                                       const nsAString& aID,
-                                                       const nsAString& aTitle,
-                                                       const NotificationOptions& aOptions);
+  static already_AddRefed<Notification> CreateInternal(
+      nsIGlobalObject* aGlobal, const nsAString& aID, const nsAString& aTitle,
+      const NotificationOptions& aOptions, ErrorResult& aRv);
 
   nsresult Init();
   bool IsInPrivateBrowsing();
   void ShowInternal();
-  void CloseInternal();
+  void CloseInternal(bool aContextClosed = false);
 
-  static NotificationPermission GetPermissionInternal(nsISupports* aGlobal,
-                                                      ErrorResult& rv);
+  static NotificationPermission GetPermissionInternal(
+      nsPIDOMWindowInner* aWindow, ErrorResult& rv);
 
-  static const nsString DirectionToString(NotificationDirection aDirection)
-  {
+  static const nsString DirectionToString(NotificationDirection aDirection) {
     switch (aDirection) {
-    case NotificationDirection::Ltr:
-      return NS_LITERAL_STRING("ltr");
-    case NotificationDirection::Rtl:
-      return NS_LITERAL_STRING("rtl");
-    default:
-      return NS_LITERAL_STRING("auto");
+      case NotificationDirection::Ltr:
+        return u"ltr"_ns;
+      case NotificationDirection::Rtl:
+        return u"rtl"_ns;
+      default:
+        return u"auto"_ns;
     }
   }
 
-  static NotificationDirection StringToDirection(const nsAString& aDirection)
-  {
+  static NotificationDirection StringToDirection(const nsAString& aDirection) {
     if (aDirection.EqualsLiteral("ltr")) {
       return NotificationDirection::Ltr;
     }
@@ -369,23 +283,17 @@ protected:
 
   static nsresult GetOrigin(nsIPrincipal* aPrincipal, nsString& aOrigin);
 
-  void GetAlertName(nsAString& aRetval)
-  {
-    workers::AssertIsOnMainThread();
+  void GetAlertName(nsAString& aRetval) {
+    AssertIsOnMainThread();
     if (mAlertName.IsEmpty()) {
       SetAlertName();
     }
     aRetval = mAlertName;
   }
 
-  void GetScope(nsAString& aScope)
-  {
-    aScope = mScope;
-  }
+  void GetScope(nsAString& aScope) { aScope = mScope; }
 
-  void
-  SetScope(const nsAString& aScope)
-  {
+  void SetScope(const nsAString& aScope) {
     MOZ_ASSERT(mScope.IsEmpty());
     mScope = aScope;
   }
@@ -397,6 +305,9 @@ protected:
   const nsString mLang;
   const nsString mTag;
   const nsString mIconUrl;
+  const bool mRequireInteraction;
+  const bool mSilent;
+  nsTArray<uint32_t> mVibrate;
   nsString mDataAsBase64;
   const NotificationBehavior mBehavior;
 
@@ -417,7 +328,7 @@ protected:
 
   static uint32_t sCount;
 
-private:
+ private:
   virtual ~Notification();
 
   // Creates a Notification and shows it. Returns a reference to the
@@ -427,40 +338,30 @@ private:
   //
   // Note that aCx may not be in the compartment of aGlobal, but aOptions will
   // have its JS things in the compartment of aCx.
-  static already_AddRefed<Notification>
-  CreateAndShow(JSContext* aCx,
-                nsIGlobalObject* aGlobal,
-                const nsAString& aTitle,
-                const NotificationOptions& aOptions,
-                const nsAString& aScope,
-                ErrorResult& aRv);
+  static already_AddRefed<Notification> CreateAndShow(
+      JSContext* aCx, nsIGlobalObject* aGlobal, const nsAString& aTitle,
+      const NotificationOptions& aOptions, const nsAString& aScope,
+      ErrorResult& aRv);
 
   nsIPrincipal* GetPrincipal();
 
   nsresult PersistNotification();
   void UnpersistNotification();
 
-  void
-  SetAlertName();
+  void SetAlertName();
 
-  bool IsTargetThread() const
-  {
-    return NS_IsMainThread() == !mWorkerPrivate;
-  }
+  bool IsTargetThread() const { return NS_IsMainThread() == !mWorkerPrivate; }
 
-  bool RegisterFeature();
-  void UnregisterFeature();
+  bool CreateWorkerRef();
 
   nsresult ResolveIconAndSoundURL(nsString&, nsString&);
 
   // Only used for Notifications on Workers, worker thread only.
-  UniquePtr<NotificationFeature> mFeature;
+  RefPtr<StrongWorkerRef> mWorkerRef;
   // Target thread only.
   uint32_t mTaskCount;
 };
 
-} // namespace dom
-} // namespace mozilla
-
-#endif // mozilla_dom_notification_h__
+}  // namespace mozilla::dom
 
+#endif  // mozilla_dom_notification_h__
