@@ -10,6 +10,7 @@
 #include "mozilla/Attributes.h"
 #include "mozilla/DOMEventTargetHelper.h"
 #include "nsCOMPtr.h"
+#include "nsContentUtils.h"
 #include "nsDOMNavigationTiming.h"
 #include "nsTObserverArray.h"
 
@@ -21,9 +22,13 @@ class ErrorResult;
 
 namespace dom {
 
+class OwningStringOrDouble;
+class StringOrPerformanceMeasureOptions;
 class PerformanceEntry;
 class PerformanceMark;
 struct PerformanceMarkOptions;
+struct PerformanceMeasureOptions;
+class PerformanceMeasure;
 class PerformanceNavigation;
 class PerformancePaintTiming;
 class PerformanceObserver;
@@ -84,8 +89,10 @@ class Performance : public DOMEventTargetHelper {
 
   void ClearMarks(const Optional<nsAString>& aName);
 
-  void Measure(const nsAString& aName, const Optional<nsAString>& aStartMark,
-               const Optional<nsAString>& aEndMark, ErrorResult& aRv);
+  already_AddRefed<PerformanceMeasure> Measure(
+      JSContext* aCx, const nsAString& aName,
+      const StringOrPerformanceMeasureOptions& aStartOrMeasureOptions,
+      const Optional<nsAString>& aEndMark, ErrorResult& aRv);
 
   void ClearMeasures(const Optional<nsAString>& aName);
 
@@ -113,7 +120,7 @@ class Performance : public DOMEventTargetHelper {
 
   virtual TimeStamp CreationTimeStamp() const = 0;
 
-  bool IsSystemPrincipal() const { return mSystemPrincipal; }
+  RTPCallerType GetRTPCallerType() const { return mRTPCallerType; }
 
   DOMHighResTimeStamp TimeStampToDOMHighResForRendering(TimeStamp) const;
 
@@ -146,13 +153,13 @@ class Performance : public DOMEventTargetHelper {
 
   void QueueNotificationObserversTask();
 
-  virtual bool IsPerformanceTimingAttribute(const nsAString& aName) {
-    return false;
-  }
+  bool IsPerformanceTimingAttribute(const nsAString& aName) const;
+
+  virtual bool IsGlobalObjectWindow() const { return false; };
 
  protected:
-  Performance(nsIGlobalObject* aGlobal, bool aSystemPrincipal);
-  Performance(nsPIDOMWindowInner* aWindow, bool aSystemPrincipal);
+  Performance(nsIGlobalObject* aGlobal);
+  Performance(nsPIDOMWindowInner* aWindow);
 
   virtual ~Performance();
 
@@ -161,9 +168,6 @@ class Performance : public DOMEventTargetHelper {
   void ClearUserEntries(const Optional<nsAString>& aEntryName,
                         const nsAString& aEntryType);
 
-  DOMHighResTimeStamp ResolveTimestampFromName(const nsAString& aName,
-                                               ErrorResult& aRv);
-
   virtual void DispatchBufferFullEvent() = 0;
 
   virtual DOMHighResTimeStamp CreationTime() const = 0;
@@ -175,7 +179,7 @@ class Performance : public DOMEventTargetHelper {
 
   void LogEntry(PerformanceEntry* aEntry, const nsACString& aOwner) const;
   void TimingNotification(PerformanceEntry* aEntry, const nsACString& aOwner,
-                          uint64_t epoch);
+                          const double aEpoch);
 
   void RunNotificationObserversTask();
   void QueueEntry(PerformanceEntry* aEntry);
@@ -201,11 +205,36 @@ class Performance : public DOMEventTargetHelper {
 
   RefPtr<PerformanceService> mPerformanceService;
 
-  bool mSystemPrincipal;
+  enum RTPCallerType mRTPCallerType;
 
  private:
   MOZ_ALWAYS_INLINE bool CanAddResourceTimingEntry();
   void BufferEvent();
+
+  // The attributes of a PerformanceMeasureOptions that we call
+  // ResolveTimestamp* on.
+  enum class ResolveTimestampAttribute;
+
+  DOMHighResTimeStamp ConvertMarkToTimestampWithString(const nsAString& aName,
+                                                       ErrorResult& aRv);
+  DOMHighResTimeStamp ConvertMarkToTimestampWithDOMHighResTimeStamp(
+      const ResolveTimestampAttribute aAttribute, const double aTimestamp,
+      ErrorResult& aRv);
+  DOMHighResTimeStamp ConvertMarkToTimestamp(
+      const ResolveTimestampAttribute aAttribute,
+      const OwningStringOrDouble& aMarkNameOrTimestamp, ErrorResult& aRv);
+
+  DOMHighResTimeStamp ConvertNameToTimestamp(const nsAString& aName,
+                                             ErrorResult& aRv);
+
+  DOMHighResTimeStamp ResolveEndTimeForMeasure(
+      const Optional<nsAString>& aEndMark,
+      const Maybe<const PerformanceMeasureOptions&>& aOptions,
+      ErrorResult& aRv);
+  DOMHighResTimeStamp ResolveStartTimeForMeasure(
+      const Maybe<const nsAString&>& aStartMark,
+      const Maybe<const PerformanceMeasureOptions&>& aOptions,
+      ErrorResult& aRv);
 };
 
 }  // namespace dom
