@@ -90,13 +90,13 @@ NS_INTERFACE_MAP_END_INHERITING(Performance)
 
 PerformanceMainThread::PerformanceMainThread(nsPIDOMWindowInner* aWindow,
                                              nsDOMNavigationTiming* aDOMTiming,
-                                             nsITimedChannel* aChannel,
-                                             bool aPrincipal)
-    : Performance(aWindow, aPrincipal),
+                                             nsITimedChannel* aChannel)
+    : Performance(aWindow),
       mDOMTiming(aDOMTiming),
       mChannel(aChannel),
       mCrossOriginIsolated(aWindow->AsGlobal()->CrossOriginIsolated()) {
   MOZ_ASSERT(aWindow, "Parent window object should be provided");
+  mRTPCallerType = aWindow->AsGlobal()->GetRTPCallerType();
   if (StaticPrefs::dom_enable_event_timing()) {
     mEventCounts = new class EventCounts(GetParentObject());
   }
@@ -289,42 +289,6 @@ void PerformanceMainThread::DispatchPendingEventTimingEntries() {
   }
 }
 
-// To be removed once bug 1124165 lands
-bool PerformanceMainThread::IsPerformanceTimingAttribute(
-    const nsAString& aName) {
-  // Note that toJSON is added to this list due to bug 1047848
-  static const char* attributes[] = {"navigationStart",
-                                     "unloadEventStart",
-                                     "unloadEventEnd",
-                                     "redirectStart",
-                                     "redirectEnd",
-                                     "fetchStart",
-                                     "domainLookupStart",
-                                     "domainLookupEnd",
-                                     "connectStart",
-                                     "secureConnectionStart",
-                                     "connectEnd",
-                                     "requestStart",
-                                     "responseStart",
-                                     "responseEnd",
-                                     "domLoading",
-                                     "domInteractive",
-                                     "domContentLoadedEventStart",
-                                     "domContentLoadedEventEnd",
-                                     "domComplete",
-                                     "loadEventStart",
-                                     "loadEventEnd",
-                                     nullptr};
-
-  for (uint32_t i = 0; attributes[i]; ++i) {
-    if (aName.EqualsASCII(attributes[i])) {
-      return true;
-    }
-  }
-
-  return false;
-}
-
 DOMHighResTimeStamp PerformanceMainThread::GetPerformanceTimingFromString(
     const nsAString& aProperty) {
   // ::Measure expects the values returned from this function to be passed
@@ -397,15 +361,14 @@ DOMHighResTimeStamp PerformanceMainThread::GetPerformanceTimingFromString(
         "of sync");
   }
   return nsRFPService::ReduceTimePrecisionAsMSecs(
-      retValue, GetRandomTimelineSeed(), /* aIsSystemPrinciapl */ false,
-      CrossOriginIsolated());
+      retValue, GetRandomTimelineSeed(), mRTPCallerType);
 }
 
 void PerformanceMainThread::InsertUserEntry(PerformanceEntry* aEntry) {
   MOZ_ASSERT(NS_IsMainThread());
 
   nsAutoCString uri;
-  uint64_t markCreationEpoch = 0;
+  double markCreationEpoch = 0;
 
   if (StaticPrefs::dom_performance_enable_user_timing_logging() ||
       StaticPrefs::dom_performance_enable_notify_performance_timing()) {
@@ -419,7 +382,11 @@ void PerformanceMainThread::InsertUserEntry(PerformanceEntry* aEntry) {
       // If we have no URI, just put in "none".
       uri.AssignLiteral("none");
     }
-    markCreationEpoch = static_cast<uint64_t>(PR_Now() / PR_USEC_PER_MSEC);
+
+    // PR_Now() returns a signed 64-bit integer. Since it represents a
+    // timestamp, only ~32-bits will represent the value which should safely fit
+    // into a double.
+    markCreationEpoch = static_cast<double>(PR_Now() / PR_USEC_PER_MSEC);
 
     if (StaticPrefs::dom_performance_enable_user_timing_logging()) {
       Performance::LogEntry(aEntry, uri);
