@@ -10,8 +10,7 @@
 #include "Performance.h"
 #include "mozilla/dom/WorkerPrivate.h"
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 class PerformanceWorker final : public Performance {
  public:
@@ -83,6 +82,8 @@ class PerformanceWorker final : public Performance {
 
   bool CrossOriginIsolated() const override;
 
+  void NoteShuttingDown();
+
  protected:
   ~PerformanceWorker();
 
@@ -96,7 +97,6 @@ class PerformanceWorker final : public Performance {
   CheckedUnsafePtr<WorkerPrivate> mWorkerPrivate;
 };
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
 
 #endif  // mozilla_dom_PerformanceWorker_h
