@@ -20,17 +20,20 @@ async function replacePushService(mockService) {
     } catch (error) {
       promise = Promise.reject(error);
     }
-    promise.then(result => {
-      chromeScript.sendAsyncMessage("service-response", {
-        id: msg.id,
-        result,
-      });
-    }, error => {
-      chromeScript.sendAsyncMessage("service-response", {
-        id: msg.id,
-        error,
-      });
-    });
+    promise.then(
+      result => {
+        chromeScript.sendAsyncMessage("service-response", {
+          id: msg.id,
+          result,
+        });
+      },
+      error => {
+        chromeScript.sendAsyncMessage("service-response", {
+          id: msg.id,
+          error,
+        });
+      }
+    );
   });
   await new Promise(resolve => {
     chromeScript.addMessageListener("service-replaced", function onReplaced() {
@@ -62,7 +65,7 @@ let currentMockSocket = null;
 function setupMockPushSocket(mockWebSocket) {
   currentMockSocket = mockWebSocket;
   currentMockSocket._isActive = true;
-  chromeScript.sendSyncMessage("socket-setup");
+  chromeScript.sendAsyncMessage("socket-setup");
   chromeScript.addMessageListener("socket-client-msg", function(msg) {
     mockWebSocket.handleMessage(msg);
   });
@@ -73,7 +76,7 @@ function teardownMockPushSocket() {
     return new Promise(resolve => {
       currentMockSocket._isActive = false;
       chromeScript.addMessageListener("socket-server-teardown", resolve);
-      chromeScript.sendSyncMessage("socket-teardown");
+      chromeScript.sendAsyncMessage("socket-teardown");
     });
   }
   return Promise.resolve();
@@ -96,30 +99,36 @@ class MockWebSocket {
   }
 
   onHello(request) {
-    this.serverSendMsg(JSON.stringify({
-      messageType: "hello",
-      uaid: this.userAgentID,
-      status: 200,
-      use_webpush: true,
-    }));
+    this.serverSendMsg(
+      JSON.stringify({
+        messageType: "hello",
+        uaid: this.userAgentID,
+        status: 200,
+        use_webpush: true,
+      })
+    );
   }
 
   onRegister(request) {
-    this.serverSendMsg(JSON.stringify({
-      messageType: "register",
-      uaid: this.userAgentID,
-      channelID: request.channelID,
-      status: 200,
-      pushEndpoint: "https://example.com/endpoint/" + this.registerCount++,
-    }));
+    this.serverSendMsg(
+      JSON.stringify({
+        messageType: "register",
+        uaid: this.userAgentID,
+        channelID: request.channelID,
+        status: 200,
+        pushEndpoint: "https://example.com/endpoint/" + this.registerCount++,
+      })
+    );
   }
 
   onUnregister(request) {
-    this.serverSendMsg(JSON.stringify({
-      messageType: "unregister",
-      channelID: request.channelID,
-      status: 200,
-    }));
+    this.serverSendMsg(
+      JSON.stringify({
+        messageType: "unregister",
+        channelID: request.channelID,
+        status: 200,
+      })
+    );
   }
 
   onAck(request) {
@@ -130,20 +139,20 @@ class MockWebSocket {
     let request = JSON.parse(msg);
     let messageType = request.messageType;
     switch (messageType) {
-    case "hello":
-      this.onHello(request);
-      break;
-    case "register":
-      this.onRegister(request);
-      break;
-    case "unregister":
-      this.onUnregister(request);
-      break;
-    case "ack":
-      this.onAck(request);
-      break;
-    default:
-      throw new Error("Unexpected message: " + messageType);
+      case "hello":
+        this.onHello(request);
+        break;
+      case "register":
+        this.onRegister(request);
+        break;
+      case "unregister":
+        this.onUnregister(request);
+        break;
+      case "ack":
+        this.onAck(request);
+        break;
+      default:
+        throw new Error("Unexpected message: " + messageType);
     }
   }
 
@@ -163,22 +172,43 @@ SimpleTest.registerCleanupFunction(async function() {
 });
 
 function setPushPermission(allow) {
-  return new Promise(resolve => {
-    SpecialPowers.pushPermissions([
-      { type: "desktop-notification", allow, context: document },
-      ], resolve);
-  });
+  let permissions = [
+    { type: "desktop-notification", allow, context: document },
+  ];
+
+  if (isXOrigin) {
+    // We need to add permission for the xorigin tests. In xorigin tests, the
+    // test page will be run under third-party context, so we need to use
+    // partitioned principal to add the permission.
+    let partitionedPrincipal = SpecialPowers.wrap(document)
+      .partitionedPrincipal;
+
+    permissions.push({
+      type: "desktop-notification",
+      allow,
+      context: {
+        url: partitionedPrincipal.originNoSuffix,
+        originAttributes: {
+          partitionKey: partitionedPrincipal.originAttributes.partitionKey,
+        },
+      },
+    });
+  }
+
+  return SpecialPowers.pushPermissions(permissions);
 }
 
 function setupPrefs() {
-  return SpecialPowers.pushPrefEnv({"set": [
-    ["dom.push.enabled", true],
-    ["dom.push.connection.enabled", true],
-    ["dom.push.maxRecentMessageIDsPerSubscription", 0],
-    ["dom.serviceWorkers.exemptFromPerDomainMax", true],
-    ["dom.serviceWorkers.enabled", true],
-    ["dom.serviceWorkers.testing.enabled", true],
-    ]});
+  return SpecialPowers.pushPrefEnv({
+    set: [
+      ["dom.push.enabled", true],
+      ["dom.push.connection.enabled", true],
+      ["dom.push.maxRecentMessageIDsPerSubscription", 0],
+      ["dom.serviceWorkers.exemptFromPerDomainMax", true],
+      ["dom.serviceWorkers.enabled", true],
+      ["dom.serviceWorkers.testing.enabled", true],
+    ],
+  });
 }
 
 async function setupPrefsAndReplaceService(mockService) {
@@ -202,12 +232,13 @@ function injectControlledFrame(target = document.body) {
         iframe = null;
       },
       waitOnWorkerMessage(type) {
-        return iframe ? iframe.contentWindow.waitOnWorkerMessage(type) :
-               Promise.reject(new Error("Frame removed from document"));
+        return iframe
+          ? iframe.contentWindow.waitOnWorkerMessage(type)
+          : Promise.reject(new Error("Frame removed from document"));
       },
       innerWindowId() {
-        var utils = SpecialPowers.getDOMWindowUtils(iframe.contentWindow);
-        return utils.currentInnerWindowID;
+        return SpecialPowers.wrap(iframe).browsingContext.currentWindowContext
+          .innerWindowId;
       },
     };
 
