@@ -3,8 +3,6 @@
 
 "use strict";
 
-const {PushDB, PushService, PushServiceHttp2} = serviceExports;
-
 function run_test() {
   do_get_profile();
   run_next_test();
@@ -12,7 +10,9 @@ function run_test() {
 
 add_task(async function test_registrations_error() {
   let db = PushServiceHttp2.newPushDB();
-  registerCleanupFunction(() => { return db.drop().then(_ => db.close()); });
+  registerCleanupFunction(() => {
+    return db.drop().then(_ => db.close());
+  });
 
   PushService.init({
     serverURI: "https://push.example.org/",
@@ -26,8 +26,9 @@ add_task(async function test_registrations_error() {
   await Assert.rejects(
     PushService.registration({
       scope: "https://example.net/1",
-      originAttributes: ChromeUtils.originAttributesToSuffix(
-        { inIsolatedMozBrowser: false }),
+      originAttributes: ChromeUtils.originAttributesToSuffix({
+        inIsolatedMozBrowser: false,
+      }),
     }),
     function(error) {
       return error == "Database error";
