@@ -10,12 +10,12 @@
 #include <cstdint>
 #include "mozilla/Assertions.h"
 #include "mozilla/Maybe.h"
-#include "mozilla/dom/StorageTypeBinding.h"
-#include "mozilla/dom/quota/QuotaCommon.h"
 #include "mozilla/fallible.h"
 #include "nsStringFwd.h"
 
-BEGIN_QUOTA_NAMESPACE
+class nsIFile;
+
+namespace mozilla::dom::quota {
 
 enum PersistenceType {
   PERSISTENCE_TYPE_PERSISTENT = 0,
@@ -26,15 +26,17 @@ enum PersistenceType {
   PERSISTENCE_TYPE_INVALID
 };
 
-static const PersistenceType kBestEffortPersistenceTypes[] = {
-    PERSISTENCE_TYPE_TEMPORARY, PERSISTENCE_TYPE_DEFAULT};
-
 static const PersistenceType kAllPersistenceTypes[] = {
     PERSISTENCE_TYPE_PERSISTENT, PERSISTENCE_TYPE_TEMPORARY,
     PERSISTENCE_TYPE_DEFAULT};
 
+static const PersistenceType kBestEffortPersistenceTypes[] = {
+    PERSISTENCE_TYPE_TEMPORARY, PERSISTENCE_TYPE_DEFAULT};
+
 bool IsValidPersistenceType(PersistenceType aPersistenceType);
 
+bool IsBestEffortPersistenceType(const PersistenceType aPersistenceType);
+
 nsLiteralCString PersistenceTypeToString(PersistenceType aPersistenceType);
 
 Maybe<PersistenceType> PersistenceTypeFromString(const nsACString& aString,
@@ -42,13 +44,14 @@ Maybe<PersistenceType> PersistenceTypeFromString(const nsACString& aString,
 
 PersistenceType PersistenceTypeFromString(const nsACString& aString);
 
-StorageType PersistenceTypeToStorageType(PersistenceType aPersistenceType);
-
-PersistenceType PersistenceTypeFromStorageType(StorageType aStorageType);
-
 Maybe<PersistenceType> PersistenceTypeFromInt32(int32_t aInt32,
                                                 const fallible_t&);
 
+// aFile is expected to be a repository directory (not some file or directory
+// within that).
+Maybe<PersistenceType> PersistenceTypeFromFile(nsIFile& aFile,
+                                               const fallible_t&);
+
 inline PersistenceType ComplementaryPersistenceType(
     const PersistenceType aPersistenceType) {
   MOZ_ASSERT(aPersistenceType == PERSISTENCE_TYPE_DEFAULT ||
@@ -61,6 +64,6 @@ inline PersistenceType ComplementaryPersistenceType(
   return PERSISTENCE_TYPE_DEFAULT;
 }
 
-END_QUOTA_NAMESPACE
+}  // namespace mozilla::dom::quota
 
 #endif  // mozilla_dom_quota_persistencetype_h__
