@@ -7,9 +7,12 @@
 #include "ScriptElement.h"
 #include "ScriptLoader.h"
 #include "mozilla/BasicEvents.h"
+#include "mozilla/CycleCollectedJSContext.h"
 #include "mozilla/EventDispatcher.h"
+#include "mozilla/dom/Document.h"
 #include "mozilla/dom/Element.h"
 #include "nsContentUtils.h"
+#include "nsThreadUtils.h"
 #include "nsPresContext.h"
 #include "nsIParser.h"
 #include "nsGkAtoms.h"
@@ -88,11 +91,22 @@ void ScriptElement::ContentInserted(nsIContent* aChild) {
 bool ScriptElement::MaybeProcessScript() {
   nsCOMPtr<nsIContent> cont = do_QueryInterface((nsIScriptElement*)this);
 
-  NS_ASSERTION(cont->DebugGetSlots()->mMutationObservers.Contains(this),
+  NS_ASSERTION(cont->DebugGetSlots()->mMutationObservers.contains(this),
                "You forgot to add self as observer");
 
   if (mAlreadyStarted || !mDoneAddingChildren || !cont->GetComposedDoc() ||
-      mMalformed || !HasScriptContent()) {
+      mMalformed) {
+    return false;
+  }
+
+  if (!HasScriptContent()) {
+    // In the case of an empty, non-external classic script, there is nothing
+    // to process. However, we must perform a microtask checkpoint afterwards,
+    // as per https://html.spec.whatwg.org/#clean-up-after-running-script
+    if (mKind == JS::loader::ScriptKind::eClassic && !mExternal) {
+      nsContentUtils::AddScriptRunner(NS_NewRunnableFunction(
+          "ScriptElement::MaybeProcessScript", []() { nsAutoMicroTask mt; }));
+    }
     return false;
   }
 
@@ -101,6 +115,9 @@ bool ScriptElement::MaybeProcessScript() {
 
   mAlreadyStarted = true;
 
+  nsContentUtils::AddScriptRunner(NS_NewRunnableFunction(
+      "ScriptElement::MaybeProcessScript", []() { nsAutoMicroTask mt; }));
+
   nsCOMPtr<nsIParser> parser = ((nsIScriptElement*)this)->GetCreatorParser();
   if (parser) {
     nsCOMPtr<nsIContentSink> sink = parser->GetContentSink();
