@@ -7,50 +7,100 @@
 #ifndef mozilla_dom_ScriptLoader_h
 #define mozilla_dom_ScriptLoader_h
 
+#include "js/TypeDecls.h"
+#include "js/loader/LoadedScript.h"
+#include "js/loader/ScriptKind.h"
+#include "js/loader/ScriptLoadRequest.h"
+#include "mozilla/dom/ScriptLoadContext.h"
 #include "nsCOMPtr.h"
 #include "nsRefPtrHashtable.h"
-#include "mozilla/Encoding.h"
 #include "nsIScriptElement.h"
 #include "nsCOMArray.h"
 #include "nsCycleCollectionParticipant.h"
 #include "nsTArray.h"
-#include "mozilla/dom/Document.h"
-#include "nsIIncrementalStreamLoader.h"
+#include "nsILoadInfo.h"  // nsSecurityFlags
+#include "nsINode.h"
+#include "nsIObserver.h"
+#include "nsIScriptLoaderObserver.h"
 #include "nsURIHashKey.h"
 #include "mozilla/CORSMode.h"
-#include "mozilla/dom/ScriptLoadRequest.h"
-#include "mozilla/dom/SRIMetadata.h"
-#include "mozilla/dom/SRICheck.h"
+#include "mozilla/dom/JSExecutionContext.h"  // JSExecutionContext
+#include "ModuleLoader.h"
 #include "mozilla/MaybeOneOf.h"
 #include "mozilla/MozPromise.h"
-#include "mozilla/Utf8.h"  // mozilla::Utf8Unit
-#include "mozilla/Vector.h"
-#include "ScriptKind.h"
 
+class nsCycleCollectionTraversalCallback;
+class nsIChannel;
+class nsIConsoleReportCollector;
+class nsIContent;
+class nsIIncrementalStreamLoader;
+class nsIPrincipal;
+class nsIScriptGlobalObject;
 class nsIURI;
 
 namespace JS {
 
+class CompileOptions;
+
 template <typename UnitT>
 class SourceText;
 
+namespace loader {
+
+class LoadedScript;
+class ScriptLoaderInterface;
+class ModuleLoadRequest;
+class ModuleScript;
+class ScriptLoadRequest;
+class ScriptLoadRequestList;
+
+}  // namespace loader
 }  // namespace JS
 
 namespace mozilla {
+
+class LazyLogModule;
+union Utf8Unit;
+
 namespace dom {
 
 class AutoJSAPI;
-class LoadedScript;
-class ModuleLoadRequest;
-class ModuleScript;
+class DocGroup;
+class Document;
+class ModuleLoader;
+class SRICheckDataVerifier;
+class SRIMetadata;
 class ScriptLoadHandler;
+class ScriptLoadContext;
+class ScriptLoader;
 class ScriptRequestProcessor;
 
+enum class ReferrerPolicy : uint8_t;
+
+class AsyncCompileShutdownObserver final : public nsIObserver {
+  ~AsyncCompileShutdownObserver() { Unregister(); }
+
+ public:
+  explicit AsyncCompileShutdownObserver(ScriptLoader* aLoader)
+      : mScriptLoader(aLoader) {}
+
+  void OnShutdown();
+  void Unregister();
+
+  NS_DECL_ISUPPORTS
+  NS_DECL_NSIOBSERVER
+
+ private:
+  // Defined during registration in ScriptLoader constructor, and
+  // cleared during destructor, ScriptLoader::Destroy() or Shutdown.
+  ScriptLoader* mScriptLoader;
+};
+
 //////////////////////////////////////////////////////////////
 // Script loader implementation
 //////////////////////////////////////////////////////////////
 
-class ScriptLoader final : public nsISupports {
+class ScriptLoader final : public JS::loader::ScriptLoaderInterface {
   class MOZ_STACK_CLASS AutoCurrentScriptUpdater {
    public:
     AutoCurrentScriptUpdater(ScriptLoader* aScriptLoader,
@@ -71,8 +121,9 @@ class ScriptLoader final : public nsISupports {
     ScriptLoader* mScriptLoader;
   };
 
-  friend class ModuleLoadRequest;
+  friend class JS::loader::ModuleLoadRequest;
   friend class ScriptRequestProcessor;
+  friend class ModuleLoader;
   friend class ScriptLoadHandler;
   friend class AutoCurrentScriptUpdater;
 
@@ -82,6 +133,12 @@ class ScriptLoader final : public nsISupports {
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
   NS_DECL_CYCLE_COLLECTION_CLASS(ScriptLoader)
 
+  /**
+   * Called when the document that owns this script loader changes global. The
+   * argument is null when the document is detached from a window.
+   */
+  void SetGlobalObject(nsIGlobalObject* aGlobalObject);
+
   /**
    * The loader maintains a weak reference to the document with
    * which it is initialized. This call forces the reference to
@@ -150,6 +207,11 @@ class ScriptLoader final : public nsISupports {
     mEnabled = aEnabled;
   }
 
+  ModuleLoader* GetModuleLoader() { return mModuleLoader; }
+
+  void RegisterContentScriptModuleLoader(ModuleLoader* aLoader);
+  void RegisterShadowRealmModuleLoader(ModuleLoader* aLoader);
+
   /**
    *  Check whether to speculatively OMT parse scripts as soon as
    *  they are fetched, even if not a parser blocking request.
@@ -214,12 +276,12 @@ class ScriptLoader final : public nsISupports {
                                  Document* aDocument, char16_t*& aBufOut,
                                  size_t& aLengthOut);
 
-  static inline nsresult ConvertToUTF16(nsIChannel* aChannel,
-                                        const uint8_t* aData, uint32_t aLength,
-                                        const nsAString& aHintCharset,
-                                        Document* aDocument,
-                                        JS::UniqueTwoByteChars& aBufOut,
-                                        size_t& aLengthOut) {
+  static nsresult ConvertToUTF16(nsIChannel* aChannel, const uint8_t* aData,
+                                 uint32_t aLength,
+                                 const nsAString& aHintCharset,
+                                 Document* aDocument,
+                                 JS::UniqueTwoByteChars& aBufOut,
+                                 size_t& aLengthOut) {
     char16_t* bufOut;
     nsresult rv = ConvertToUTF16(aChannel, aData, aLength, aHintCharset,
                                  aDocument, bufOut, aLengthOut);
@@ -254,6 +316,19 @@ class ScriptLoader final : public nsISupports {
                                 Document* aDocument, Utf8Unit*& aBufOut,
                                 size_t& aLengthOut);
 
+  static inline nsresult ConvertToUTF8(
+      nsIChannel* aChannel, const uint8_t* aData, uint32_t aLength,
+      const nsAString& aHintCharset, Document* aDocument,
+      UniquePtr<Utf8Unit[], JS::FreePolicy>& aBufOut, size_t& aLengthOut) {
+    Utf8Unit* bufOut;
+    nsresult rv = ConvertToUTF8(aChannel, aData, aLength, aHintCharset,
+                                aDocument, bufOut, aLengthOut);
+    if (NS_SUCCEEDED(rv)) {
+      aBufOut.reset(bufOut);
+    }
+    return rv;
+  };
+
   /**
    * Handle the completion of a stream.  This is called by the
    * ScriptLoadHandler object which observes the IncrementalStreamLoader
@@ -268,7 +343,12 @@ class ScriptLoader final : public nsISupports {
   /**
    * Returns wether any request is queued, and not executed yet.
    */
-  bool HasPendingRequests();
+  bool HasPendingRequests() const;
+
+  /**
+   * Returns wether there are any dynamic module import requests pending.
+   */
+  bool HasPendingDynamicImports() const;
 
   /**
    * Processes any pending requests that are ready for processing.
@@ -279,24 +359,7 @@ class ScriptLoader final : public nsISupports {
    * Starts deferring deferred scripts and puts them in the mDeferredRequests
    * queue instead.
    */
-  void BeginDeferringScripts() {
-    mDeferEnabled = true;
-    if (mDeferCheckpointReached) {
-      // We already completed a parse and were just waiting for some async
-      // scripts to load (and were already blocking the load event waiting for
-      // that to happen), when document.open() happened and now we're doing a
-      // new parse.  We shouldn't block the load event again, but _should_ reset
-      // mDeferCheckpointReached to false.  It'll get set to true again when the
-      // DeferCheckpointReached call that corresponds to this
-      // BeginDeferringScripts call happens (on document.close()), since we just
-      // set mDeferEnabled to true.
-      mDeferCheckpointReached = false;
-    } else {
-      if (mDocument) {
-        mDocument->BlockOnload();
-      }
-    }
-  }
+  void BeginDeferringScripts();
 
   /**
    * Notifies the script loader that parsing is done.  If aTerminated is true,
@@ -342,7 +405,8 @@ class ScriptLoader final : public nsISupports {
                           const nsAString& aIntegrity, bool aScriptFromHead,
                           bool aAsync, bool aDefer, bool aNoModule,
                           bool aLinkPreload,
-                          const ReferrerPolicy aReferrerPolicy);
+                          const ReferrerPolicy aReferrerPolicy,
+                          uint64_t aEarlyHintPreloaderId);
 
   /**
    * Process a request that was deferred so that the script could be compiled
@@ -357,9 +421,7 @@ class ScriptLoader final : public nsISupports {
     return true;
   }
 
-  mozilla::dom::DocGroup* GetDocGroup() const {
-    return mDocument->GetDocGroup();
-  }
+  mozilla::dom::DocGroup* GetDocGroup() const;
 
   /**
    * Register the fact that we saw the load event, and that we need to save the
@@ -373,49 +435,25 @@ class ScriptLoader final : public nsISupports {
    * any references to the JSScript or to the Request which might be used for
    * caching the encoded bytecode.
    */
-  void Destroy() { GiveUpBytecodeEncoding(); }
-
-  /**
-   * Implement the HostResolveImportedModule abstract operation.
-   *
-   * Resolve a module specifier string and look this up in the module
-   * map, returning the result. This is only called for previously
-   * loaded modules and always succeeds.
-   *
-   * @param aReferencingPrivate A JS::Value which is either undefined
-   *                            or contains a LoadedScript private pointer.
-   * @param aSpecifier The module specifier.
-   * @param aModuleOut This is set to the module found.
-   */
-  static void ResolveImportedModule(JSContext* aCx,
-                                    JS::Handle<JS::Value> aReferencingPrivate,
-                                    JS::Handle<JSString*> aSpecifier,
-                                    JS::MutableHandle<JSObject*> aModuleOut);
-
-  void StartDynamicImport(ModuleLoadRequest* aRequest);
-  void FinishDynamicImport(ModuleLoadRequest* aRequest, nsresult aResult);
-  void FinishDynamicImport(JSContext* aCx, ModuleLoadRequest* aRequest,
-                           nsresult aResult);
+  void Destroy();
 
   /*
    * Get the currently active script. This is used as the initiating script when
    * executing timeout handler scripts.
    */
-  static LoadedScript* GetActiveScript(JSContext* aCx);
+  static JS::loader::LoadedScript* GetActiveScript(JSContext* aCx);
 
   Document* GetDocument() const { return mDocument; }
 
+  nsIURI* GetBaseURI() const override;
+
  private:
-  virtual ~ScriptLoader();
+  ~ScriptLoader();
 
-  void EnsureModuleHooksInitialized();
-
-  ScriptLoadRequest* CreateLoadRequest(ScriptKind aKind, nsIURI* aURI,
-                                       nsIScriptElement* aElement,
-                                       nsIPrincipal* aTriggeringPrincipal,
-                                       mozilla::CORSMode aCORSMode,
-                                       const SRIMetadata& aIntegrity,
-                                       ReferrerPolicy aReferrerPolicy);
+  already_AddRefed<ScriptLoadRequest> CreateLoadRequest(
+      ScriptKind aKind, nsIURI* aURI, nsIScriptElement* aElement,
+      nsIPrincipal* aTriggeringPrincipal, mozilla::CORSMode aCORSMode,
+      const SRIMetadata& aIntegrity, ReferrerPolicy aReferrerPolicy);
 
   /**
    * Unblocks the creator parser of the parser-blocking scripts.
@@ -428,14 +466,14 @@ class ScriptLoader final : public nsISupports {
   void ContinueParserAsync(ScriptLoadRequest* aParserBlockingRequest);
 
   bool ProcessExternalScript(nsIScriptElement* aElement, ScriptKind aScriptKind,
-                             nsAutoString aTypeAttr,
+                             const nsAutoString& aTypeAttr,
                              nsIContent* aScriptContent);
 
   bool ProcessInlineScript(nsIScriptElement* aElement, ScriptKind aScriptKind);
 
-  ScriptLoadRequest* LookupPreloadRequest(nsIScriptElement* aElement,
-                                          ScriptKind aScriptKind,
-                                          const SRIMetadata& aSRIMetadata);
+  JS::loader::ScriptLoadRequest* LookupPreloadRequest(
+      nsIScriptElement* aElement, ScriptKind aScriptKind,
+      const SRIMetadata& aSRIMetadata);
 
   void GetSRIMetadata(const nsAString& aIntegrityAttr,
                       SRIMetadata* aMetadataOut);
@@ -460,12 +498,27 @@ class ScriptLoader final : public nsISupports {
    *   * the about: page is not linkable from content
    *     (e.g. the function will return false for about:blank or about:srcdoc)
    */
-  static bool IsAboutPageLoadingChromeURI(ScriptLoadRequest* aRequest);
+  static bool IsAboutPageLoadingChromeURI(ScriptLoadRequest* aRequest,
+                                          Document* aDocument);
 
   /**
    * Start a load for aRequest's URI.
    */
-  nsresult StartLoad(ScriptLoadRequest* aRequest);
+  nsresult StartLoad(ScriptLoadRequest* aRequest,
+                     uint64_t aEarlyHintPreloaderId);
+  /**
+   * Start a load for a classic script URI.
+   * Sets up the necessary security flags before calling StartLoadInternal.
+   */
+  nsresult StartClassicLoad(ScriptLoadRequest* aRequest,
+                            uint64_t aEarlyHintPreloaderId);
+
+  /**
+   * Start a load for a module script URI.
+   */
+  nsresult StartLoadInternal(ScriptLoadRequest* aRequest,
+                             nsSecurityFlags securityFlags,
+                             uint64_t aEarlyHintPreloaderId);
 
   /**
    * Abort the current stream, and re-start with a new load request from scratch
@@ -511,20 +564,66 @@ class ScriptLoader final : public nsISupports {
                      SRICheckDataVerifier* aSRIDataVerifier) const;
 
   nsresult SaveSRIHash(ScriptLoadRequest* aRequest,
-                       SRICheckDataVerifier* aSRIDataVerifier) const;
+                       SRICheckDataVerifier* aSRIDataVerifier,
+                       uint32_t* sriLength) const;
 
   void ReportErrorToConsole(ScriptLoadRequest* aRequest,
-                            nsresult aResult) const;
+                            nsresult aResult) const override;
+
+  void ReportWarningToConsole(
+      ScriptLoadRequest* aRequest, const char* aMessageName,
+      const nsTArray<nsString>& aParams = nsTArray<nsString>()) const override;
+
   void ReportPreloadErrorsToConsole(ScriptLoadRequest* aRequest);
 
   nsresult AttemptAsyncScriptCompile(ScriptLoadRequest* aRequest,
                                      bool* aCouldCompileOut);
   nsresult ProcessRequest(ScriptLoadRequest* aRequest);
-  void ProcessDynamicImport(ModuleLoadRequest* aRequest);
   nsresult CompileOffThreadOrProcessRequest(ScriptLoadRequest* aRequest);
   void FireScriptAvailable(nsresult aResult, ScriptLoadRequest* aRequest);
-  void FireScriptEvaluated(nsresult aResult, ScriptLoadRequest* aRequest);
-  nsresult EvaluateScript(ScriptLoadRequest* aRequest);
+  // TODO: Convert this to MOZ_CAN_RUN_SCRIPT (bug 1415230)
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY void FireScriptEvaluated(
+      nsresult aResult, ScriptLoadRequest* aRequest);
+
+  // Implements https://html.spec.whatwg.org/#execute-the-script-block
+  nsresult EvaluateScriptElement(ScriptLoadRequest* aRequest);
+
+  // Handles both bytecode and text source scripts; populates exec with a
+  // compiled script
+  nsresult CompileOrDecodeClassicScript(JSContext* aCx,
+                                        JSExecutionContext& aExec,
+                                        ScriptLoadRequest* aRequest);
+
+  static nsCString& BytecodeMimeTypeFor(ScriptLoadRequest* aRequest);
+
+  // Decide whether to encode bytecode for given script load request,
+  // and store the script into the request if necessary.
+  //
+  // This method must be called before executing the script.
+  void MaybePrepareForBytecodeEncodingBeforeExecute(
+      ScriptLoadRequest* aRequest, JS::Handle<JSScript*> aScript);
+
+  // Queue the script load request for bytecode encoding if we decided to
+  // encode, or cleanup the script load request fields otherwise.
+  //
+  // This method must be called after executing the script.
+  nsresult MaybePrepareForBytecodeEncodingAfterExecute(
+      ScriptLoadRequest* aRequest, nsresult aRv);
+
+  // Returns true if MaybePrepareForBytecodeEncodingAfterExecute is called
+  // for given script load request.
+  bool IsAlreadyHandledForBytecodeEncodingPreparation(
+      ScriptLoadRequest* aRequest);
+
+  void MaybePrepareModuleForBytecodeEncodingBeforeExecute(
+      JSContext* aCx, ModuleLoadRequest* aRequest) override;
+
+  nsresult MaybePrepareModuleForBytecodeEncodingAfterExecute(
+      ModuleLoadRequest* aRequest, nsresult aRv) override;
+
+  // Implements https://html.spec.whatwg.org/#run-a-classic-script
+  nsresult EvaluateScript(nsIGlobalObject* aGlobalObject,
+                          ScriptLoadRequest* aRequest);
 
   /**
    * Queue the current script load request to be saved, when the page
@@ -539,7 +638,7 @@ class ScriptLoader final : public nsISupports {
    * no more script have to be processed.  If all conditions are met, queue an
    * event to encode all the bytecode and save them on the cache.
    */
-  void MaybeTriggerBytecodeEncoding();
+  void MaybeTriggerBytecodeEncoding() override;
 
   /**
    * Iterate over all script load request and save the bytecode of executed
@@ -550,13 +649,20 @@ class ScriptLoader final : public nsISupports {
 
   void GiveUpBytecodeEncoding();
 
+  already_AddRefed<nsIGlobalObject> GetGlobalForRequest(
+      ScriptLoadRequest* aRequest);
+
   already_AddRefed<nsIScriptGlobalObject> GetScriptGlobalObject();
-  nsresult FillCompileOptionsForRequest(const mozilla::dom::AutoJSAPI& jsapi,
-                                        ScriptLoadRequest* aRequest,
-                                        JS::Handle<JSObject*> aScopeChain,
-                                        JS::CompileOptions* aOptions);
+
+  // Fill in CompileOptions, as well as produce the introducer script for
+  // subsequent calls to UpdateDebuggerMetadata
+  nsresult FillCompileOptionsForRequest(
+      JSContext* aCx, ScriptLoadRequest* aRequest, JS::CompileOptions* aOptions,
+      JS::MutableHandle<JSScript*> aIntroductionScript) override;
 
   uint32_t NumberOfProcessors();
+  int32_t PhysicalSizeOfMemoryInGB();
+
   nsresult PrepareLoadedRequest(ScriptLoadRequest* aRequest,
                                 nsIIncrementalStreamLoader* aLoader,
                                 nsresult aStatus);
@@ -565,6 +671,9 @@ class ScriptLoader final : public nsISupports {
   void AddAsyncRequest(ScriptLoadRequest* aRequest);
   bool MaybeRemovedDeferRequests();
 
+  bool ShouldApplyDelazifyStrategy(ScriptLoadRequest* aRequest);
+  void ApplyDelazifyStrategy(JS::CompileOptions* aOptions);
+
   bool ShouldCompileOffThread(ScriptLoadRequest* aRequest);
 
   void MaybeMoveToLoadedList(ScriptLoadRequest* aRequest);
@@ -572,57 +681,31 @@ class ScriptLoader final : public nsISupports {
   using MaybeSourceText =
       mozilla::MaybeOneOf<JS::SourceText<char16_t>, JS::SourceText<Utf8Unit>>;
 
-  // Get source text.  On success |aMaybeSource| will contain either UTF-8 or
-  // UTF-16 source; on failure it will remain in its initial state.
-  MOZ_MUST_USE nsresult GetScriptSource(JSContext* aCx,
-                                        ScriptLoadRequest* aRequest,
-                                        MaybeSourceText* aMaybeSource);
-
-  void SetModuleFetchStarted(ModuleLoadRequest* aRequest);
-  void SetModuleFetchFinishedAndResumeWaitingRequests(
-      ModuleLoadRequest* aRequest, nsresult aResult);
-
-  bool IsFetchingModule(ModuleLoadRequest* aRequest) const;
-
-  bool ModuleMapContainsURL(nsIURI* aURL) const;
-  RefPtr<mozilla::GenericNonExclusivePromise> WaitForModuleFetch(nsIURI* aURL);
-  ModuleScript* GetFetchedModule(nsIURI* aURL) const;
-
-  friend JSObject* HostResolveImportedModule(
-      JSContext* aCx, JS::Handle<JS::Value> aReferencingPrivate,
-      JS::Handle<JSString*> aSpecifier);
-
   // Returns wether we should save the bytecode of this script after the
   // execution of the script.
   static bool ShouldCacheBytecode(ScriptLoadRequest* aRequest);
 
-  nsresult CreateModuleScript(ModuleLoadRequest* aRequest);
-  nsresult ProcessFetchedModuleSource(ModuleLoadRequest* aRequest);
-  void CheckModuleDependenciesLoaded(ModuleLoadRequest* aRequest);
-  void ProcessLoadedModuleTree(ModuleLoadRequest* aRequest);
-  bool InstantiateModuleTree(ModuleLoadRequest* aRequest);
-  JS::Value FindFirstParseError(ModuleLoadRequest* aRequest);
-  void StartFetchingModuleDependencies(ModuleLoadRequest* aRequest);
-
-  RefPtr<mozilla::GenericPromise> StartFetchingModuleAndDependencies(
-      ModuleLoadRequest* aParent, nsIURI* aURI);
-
-  nsresult InitDebuggerDataForModuleTree(JSContext* aCx,
-                                         ModuleLoadRequest* aRequest);
-
   void RunScriptWhenSafe(ScriptLoadRequest* aRequest);
 
+  /**
+   *  Wait for any unused off thread compilations to finish and then
+   *  cancel them.
+   */
+  void CancelScriptLoadRequests();
+
   Document* mDocument;  // [WEAK]
   nsCOMArray<nsIScriptLoaderObserver> mObservers;
   ScriptLoadRequestList mNonAsyncExternalScriptInsertedRequests;
   // mLoadingAsyncRequests holds async requests while they're loading; when they
   // have been loaded they are moved to mLoadedAsyncRequests.
   ScriptLoadRequestList mLoadingAsyncRequests;
+  // mLoadedAsyncRequests holds async script requests and dynamic module import
+  // requests, which are processed in the same way.
   ScriptLoadRequestList mLoadedAsyncRequests;
   ScriptLoadRequestList mDeferRequests;
   ScriptLoadRequestList mXSLTRequests;
-  ScriptLoadRequestList mDynamicImportRequests;
   RefPtr<ScriptLoadRequest> mParserBlockingRequest;
+  ScriptLoadRequestList mOffThreadCompilingRequests;
 
   // List of script load request that are holding a buffer which has to be saved
   // on the cache.
@@ -658,6 +741,8 @@ class ScriptLoader final : public nsISupports {
   uint32_t mParserBlockingBlockerCount;
   uint32_t mBlockerCount;
   uint32_t mNumberOfProcessors;
+  uint32_t mTotalFullParseSize;
+  int32_t mPhysicalSizeOfMemory;
   bool mEnabled;
   bool mDeferEnabled;
   bool mSpeculativeOMTParsingEnabled;
@@ -666,13 +751,17 @@ class ScriptLoader final : public nsISupports {
   bool mLoadEventFired;
   bool mGiveUpEncoding;
 
-  // Module map
-  nsRefPtrHashtable<nsURIHashKey, mozilla::GenericNonExclusivePromise::Private>
-      mFetchingModules;
-  nsRefPtrHashtable<nsURIHashKey, ModuleScript> mFetchedModules;
+  TimeDuration mMainThreadParseTime;
 
   nsCOMPtr<nsIConsoleReportCollector> mReporter;
 
+  // ShutdownObserver for off thread compilations
+  RefPtr<AsyncCompileShutdownObserver> mShutdownObserver;
+
+  RefPtr<ModuleLoader> mModuleLoader;
+  nsTArray<RefPtr<ModuleLoader>> mWebExtModuleLoaders;
+  nsTArray<RefPtr<ModuleLoader>> mShadowRealmModuleLoaders;
+
   // Logging
  public:
   static LazyLogModule gCspPRLog;
@@ -681,19 +770,9 @@ class ScriptLoader final : public nsISupports {
 
 class nsAutoScriptLoaderDisabler {
  public:
-  explicit nsAutoScriptLoaderDisabler(Document* aDoc) {
-    mLoader = aDoc->ScriptLoader();
-    mWasEnabled = mLoader->GetEnabled();
-    if (mWasEnabled) {
-      mLoader->SetEnabled(false);
-    }
-  }
+  explicit nsAutoScriptLoaderDisabler(Document* aDoc);
 
-  ~nsAutoScriptLoaderDisabler() {
-    if (mWasEnabled) {
-      mLoader->SetEnabled(true);
-    }
-  }
+  ~nsAutoScriptLoaderDisabler();
 
   bool mWasEnabled;
   RefPtr<ScriptLoader> mLoader;
