@@ -9,14 +9,23 @@
 #include "nsContentUtils.h"
 #include "nsCSPUtils.h"
 #include "nsDebug.h"
+#include "nsCSPParser.h"
+#include "nsComponentManagerUtils.h"
 #include "nsIConsoleService.h"
+#include "nsIChannel.h"
 #include "nsICryptoHash.h"
 #include "nsIScriptError.h"
-#include "nsIServiceManager.h"
 #include "nsIStringBundle.h"
 #include "nsIURL.h"
+#include "nsNetUtil.h"
 #include "nsReadableUtils.h"
 #include "nsSandboxFlags.h"
+#include "nsServiceManagerUtils.h"
+
+#include "mozilla/Components.h"
+#include "mozilla/dom/CSPDictionariesBinding.h"
+#include "mozilla/dom/Document.h"
+#include "mozilla/StaticPrefs_security.h"
 
 #define DEFAULT_PORT -1
 
@@ -84,11 +93,73 @@ void CSP_PercentDecodeStr(const nsAString& aEncStr, nsAString& outDecStr) {
   }
 }
 
-void CSP_GetLocalizedStr(const char* aName, const char16_t** aParams,
-                         uint32_t aLength, nsAString& outResult) {
+// The Content Security Policy should be inherited for
+// local schemes like: "about", "blob", "data", or "filesystem".
+// see: https://w3c.github.io/webappsec-csp/#initialize-document-csp
+bool CSP_ShouldResponseInheritCSP(nsIChannel* aChannel) {
+  if (!aChannel) {
+    return false;
+  }
+
+  nsCOMPtr<nsIURI> uri;
+  nsresult rv = aChannel->GetURI(getter_AddRefs(uri));
+  NS_ENSURE_SUCCESS(rv, false);
+
+  bool isAbout = uri->SchemeIs("about");
+  if (isAbout) {
+    nsAutoCString aboutSpec;
+    rv = uri->GetSpec(aboutSpec);
+    NS_ENSURE_SUCCESS(rv, false);
+    // also allow about:blank#foo
+    if (StringBeginsWith(aboutSpec, "about:blank"_ns) ||
+        StringBeginsWith(aboutSpec, "about:srcdoc"_ns)) {
+      return true;
+    }
+  }
+
+  return uri->SchemeIs("blob") || uri->SchemeIs("data") ||
+         uri->SchemeIs("filesystem") || uri->SchemeIs("javascript");
+}
+
+void CSP_ApplyMetaCSPToDoc(mozilla::dom::Document& aDoc,
+                           const nsAString& aPolicyStr) {
+  if (aDoc.IsLoadedAsData()) {
+    return;
+  }
+
+  nsAutoString policyStr(
+      nsContentUtils::TrimWhitespace<nsContentUtils::IsHTMLWhitespace>(
+          aPolicyStr));
+
+  if (policyStr.IsEmpty()) {
+    return;
+  }
+
+  nsCOMPtr<nsIContentSecurityPolicy> csp = aDoc.GetCsp();
+  if (!csp) {
+    MOZ_ASSERT(false, "how come there is no CSP");
+    return;
+  }
+
+  // Multiple CSPs (delivered through either header of meta tag) need to
+  // be joined together, see:
+  // https://w3c.github.io/webappsec/specs/content-security-policy/#delivery-html-meta-element
+  nsresult rv =
+      csp->AppendPolicy(policyStr,
+                        false,  // csp via meta tag can not be report only
+                        true);  // delivered through the meta tag
+  NS_ENSURE_SUCCESS_VOID(rv);
+  if (nsPIDOMWindowInner* inner = aDoc.GetInnerWindow()) {
+    inner->SetCsp(csp);
+  }
+  aDoc.ApplySettingsFromCSP(false);
+}
+
+void CSP_GetLocalizedStr(const char* aName, const nsTArray<nsString>& aParams,
+                         nsAString& outResult) {
   nsCOMPtr<nsIStringBundle> keyStringBundle;
   nsCOMPtr<nsIStringBundleService> stringBundleService =
-      mozilla::services::GetStringBundleService();
+      mozilla::components::StringBundle::Service();
 
   NS_ASSERTION(stringBundleService, "String bundle service must be present!");
   stringBundleService->CreateBundle(
@@ -100,7 +171,7 @@ void CSP_GetLocalizedStr(const char* aName, const char16_t** aParams,
   if (!keyStringBundle) {
     return;
   }
-  keyStringBundle->FormatStringFromName(aName, aParams, aLength, outResult);
+  keyStringBundle->FormatStringFromName(aName, aParams, outResult);
 }
 
 void CSP_LogStrMessage(const nsAString& aMsg) {
@@ -139,10 +210,9 @@ void CSP_LogMessage(const nsAString& aMessage, const nsAString& aSourceName,
   // information contained within 'aSourceLine' can be really useful for devs.
   // E.g. 'aSourceLine' might be: 'onclick attribute on DIV element'.
   // In such cases we append 'aSourceLine' directly to the error message.
-  if (!aSourceLine.IsEmpty()) {
-    cspMsg.AppendLiteral(u" Source: ");
+  if (!aSourceLine.IsEmpty() && aLineNumber == 0) {
+    cspMsg.AppendLiteral(u"\nSource: ");
     cspMsg.Append(aSourceLine);
-    cspMsg.AppendLiteral(u".");
   }
 
   // Since we are leveraging csp errors as the category names which
@@ -159,7 +229,7 @@ void CSP_LogMessage(const nsAString& aMessage, const nsAString& aSourceName,
                                  aInnerWindowID);
   } else {
     rv = error->Init(cspMsg, aSourceName, aSourceLine, aLineNumber,
-                     aColumnNumber, aFlags, category.get(), aFromPrivateWindow,
+                     aColumnNumber, aFlags, category, aFromPrivateWindow,
                      true /* from chrome context */);
   }
   if (NS_FAILED(rv)) {
@@ -171,23 +241,30 @@ void CSP_LogMessage(const nsAString& aMessage, const nsAString& aSourceName,
 /**
  * Combines CSP_LogMessage and CSP_GetLocalizedStr into one call.
  */
-void CSP_LogLocalizedStr(const char* aName, const char16_t** aParams,
-                         uint32_t aLength, const nsAString& aSourceName,
+void CSP_LogLocalizedStr(const char* aName, const nsTArray<nsString>& aParams,
+                         const nsAString& aSourceName,
                          const nsAString& aSourceLine, uint32_t aLineNumber,
                          uint32_t aColumnNumber, uint32_t aFlags,
                          const nsACString& aCategory, uint64_t aInnerWindowID,
                          bool aFromPrivateWindow) {
   nsAutoString logMsg;
-  CSP_GetLocalizedStr(aName, aParams, aLength, logMsg);
+  CSP_GetLocalizedStr(aName, aParams, logMsg);
   CSP_LogMessage(logMsg, aSourceName, aSourceLine, aLineNumber, aColumnNumber,
                  aFlags, aCategory, aInnerWindowID, aFromPrivateWindow);
 }
 
 /* ===== Helpers ============================ */
+// This implements
+// https://w3c.github.io/webappsec-csp/#effective-directive-for-a-request.
+// However the spec doesn't currently cover all request destinations, which
+// we roughly represent using nsContentPolicyType.
 CSPDirective CSP_ContentTypeToDirective(nsContentPolicyType aType) {
   switch (aType) {
     case nsIContentPolicy::TYPE_IMAGE:
     case nsIContentPolicy::TYPE_IMAGESET:
+    case nsIContentPolicy::TYPE_INTERNAL_IMAGE:
+    case nsIContentPolicy::TYPE_INTERNAL_IMAGE_PRELOAD:
+    case nsIContentPolicy::TYPE_INTERNAL_IMAGE_FAVICON:
       return nsIContentSecurityPolicy::IMG_SRC_DIRECTIVE;
 
     // BLock XSLT as script, see bug 910139
@@ -198,15 +275,29 @@ CSPDirective CSP_ContentTypeToDirective(nsContentPolicyType aType) {
     case nsIContentPolicy::TYPE_INTERNAL_MODULE:
     case nsIContentPolicy::TYPE_INTERNAL_MODULE_PRELOAD:
     case nsIContentPolicy::TYPE_INTERNAL_WORKER_IMPORT_SCRIPTS:
-      return nsIContentSecurityPolicy::SCRIPT_SRC_DIRECTIVE;
+    case nsIContentPolicy::TYPE_INTERNAL_AUDIOWORKLET:
+    case nsIContentPolicy::TYPE_INTERNAL_PAINTWORKLET:
+    case nsIContentPolicy::TYPE_INTERNAL_CHROMEUTILS_COMPILED_SCRIPT:
+    case nsIContentPolicy::TYPE_INTERNAL_FRAME_MESSAGEMANAGER_SCRIPT:
+      // (https://github.com/w3c/webappsec-csp/issues/554)
+      // Some of these types are not explicitly defined in the spec.
+      //
+      // Chrome seems to use script-src-elem for worklet!
+      return nsIContentSecurityPolicy::SCRIPT_SRC_ELEM_DIRECTIVE;
 
     case nsIContentPolicy::TYPE_STYLESHEET:
-      return nsIContentSecurityPolicy::STYLE_SRC_DIRECTIVE;
+    case nsIContentPolicy::TYPE_INTERNAL_STYLESHEET:
+    case nsIContentPolicy::TYPE_INTERNAL_STYLESHEET_PRELOAD:
+      return nsIContentSecurityPolicy::STYLE_SRC_ELEM_DIRECTIVE;
 
     case nsIContentPolicy::TYPE_FONT:
+    case nsIContentPolicy::TYPE_INTERNAL_FONT_PRELOAD:
       return nsIContentSecurityPolicy::FONT_SRC_DIRECTIVE;
 
     case nsIContentPolicy::TYPE_MEDIA:
+    case nsIContentPolicy::TYPE_INTERNAL_AUDIO:
+    case nsIContentPolicy::TYPE_INTERNAL_VIDEO:
+    case nsIContentPolicy::TYPE_INTERNAL_TRACK:
       return nsIContentSecurityPolicy::MEDIA_SRC_DIRECTIVE;
 
     case nsIContentPolicy::TYPE_WEB_MANIFEST:
@@ -218,6 +309,8 @@ CSPDirective CSP_ContentTypeToDirective(nsContentPolicyType aType) {
       return nsIContentSecurityPolicy::WORKER_SRC_DIRECTIVE;
 
     case nsIContentPolicy::TYPE_SUBDOCUMENT:
+    case nsIContentPolicy::TYPE_INTERNAL_FRAME:
+    case nsIContentPolicy::TYPE_INTERNAL_IFRAME:
       return nsIContentSecurityPolicy::FRAME_SRC_DIRECTIVE;
 
     case nsIContentPolicy::TYPE_WEBSOCKET:
@@ -225,18 +318,27 @@ CSPDirective CSP_ContentTypeToDirective(nsContentPolicyType aType) {
     case nsIContentPolicy::TYPE_BEACON:
     case nsIContentPolicy::TYPE_PING:
     case nsIContentPolicy::TYPE_FETCH:
+    case nsIContentPolicy::TYPE_INTERNAL_XMLHTTPREQUEST:
+    case nsIContentPolicy::TYPE_INTERNAL_EVENTSOURCE:
+    case nsIContentPolicy::TYPE_INTERNAL_FETCH_PRELOAD:
+    case nsIContentPolicy::TYPE_WEB_IDENTITY:
       return nsIContentSecurityPolicy::CONNECT_SRC_DIRECTIVE;
 
     case nsIContentPolicy::TYPE_OBJECT:
     case nsIContentPolicy::TYPE_OBJECT_SUBREQUEST:
+    case nsIContentPolicy::TYPE_INTERNAL_EMBED:
+    case nsIContentPolicy::TYPE_INTERNAL_OBJECT:
       return nsIContentSecurityPolicy::OBJECT_SRC_DIRECTIVE;
 
-    case nsIContentPolicy::TYPE_XBL:
     case nsIContentPolicy::TYPE_DTD:
     case nsIContentPolicy::TYPE_OTHER:
     case nsIContentPolicy::TYPE_SPECULATIVE:
+    case nsIContentPolicy::TYPE_INTERNAL_DTD:
+    case nsIContentPolicy::TYPE_INTERNAL_FORCE_ALLOWED_DTD:
       return nsIContentSecurityPolicy::DEFAULT_SRC_DIRECTIVE;
 
+    // CSP does not apply to webrtc connections
+    case nsIContentPolicy::TYPE_PROXIED_WEBRTC_MEDIA:
     // csp shold not block top level loads, e.g. in case
     // of a redirect.
     case nsIContentPolicy::TYPE_DOCUMENT:
@@ -245,11 +347,14 @@ CSPDirective CSP_ContentTypeToDirective(nsContentPolicyType aType) {
       return nsIContentSecurityPolicy::NO_DIRECTIVE;
 
     case nsIContentPolicy::TYPE_SAVEAS_DOWNLOAD:
+    case nsIContentPolicy::TYPE_UA_FONT:
       return nsIContentSecurityPolicy::NO_DIRECTIVE;
 
     // Fall through to error for all other directives
-    default:
+    // Note that we should never end up here for navigate-to
+    case nsIContentPolicy::TYPE_INVALID:
       MOZ_ASSERT(false, "Can not map nsContentPolicyType to CSPDirective");
+      // Do not add default: so that compilers can catch the missing case.
   }
   return nsIContentSecurityPolicy::DEFAULT_SRC_DIRECTIVE;
 }
@@ -289,16 +394,6 @@ nsCSPHostSrc* CSP_CreateHostSrcFromSelfURI(nsIURI* aSelfURI) {
 bool CSP_IsEmptyDirective(const nsAString& aValue, const nsAString& aDir) {
   return (aDir.Length() == 0 && aValue.Length() == 0);
 }
-bool CSP_IsValidDirective(const nsAString& aDir) {
-  uint32_t numDirs = (sizeof(CSPStrDirectives) / sizeof(CSPStrDirectives[0]));
-
-  for (uint32_t i = 0; i < numDirs; i++) {
-    if (aDir.LowerCaseEqualsASCII(CSPStrDirectives[i])) {
-      return true;
-    }
-  }
-  return false;
-}
 bool CSP_IsDirective(const nsAString& aValue, CSPDirective aDir) {
   return aValue.LowerCaseEqualsASCII(CSP_CSPDirectiveToString(aDir));
 }
@@ -408,9 +503,8 @@ nsresult CSP_AppendCSPFromHeader(nsIContentSecurityPolicy* aCsp,
   // concatenated into one comma-separated list of policies.
   // See RFC2616 section 4.2 (last paragraph)
   nsresult rv = NS_OK;
-  nsCharSeparatedTokenizer tokenizer(aHeaderValue, ',');
-  while (tokenizer.hasMoreTokens()) {
-    const nsAString& policy = tokenizer.nextToken();
+  for (const nsAString& policy :
+       nsCharSeparatedTokenizer(aHeaderValue, ',').ToRange()) {
     rv = aCsp->AppendPolicy(policy, aReportOnly, false);
     NS_ENSURE_SUCCESS(rv, rv);
     {
@@ -425,7 +519,7 @@ nsresult CSP_AppendCSPFromHeader(nsIContentSecurityPolicy* aCsp,
 
 nsCSPBaseSrc::nsCSPBaseSrc() : mInvalidated(false) {}
 
-nsCSPBaseSrc::~nsCSPBaseSrc() {}
+nsCSPBaseSrc::~nsCSPBaseSrc() = default;
 
 // ::permits is only called for external load requests, therefore:
 // nsCSPKeywordSrc and nsCSPHashSource fall back to this base class
@@ -458,7 +552,7 @@ nsCSPSchemeSrc::nsCSPSchemeSrc(const nsAString& aScheme) : mScheme(aScheme) {
   ToLowerCase(mScheme);
 }
 
-nsCSPSchemeSrc::~nsCSPSchemeSrc() {}
+nsCSPSchemeSrc::~nsCSPSchemeSrc() = default;
 
 bool nsCSPSchemeSrc::permits(nsIURI* aUri, const nsAString& aNonce,
                              bool aWasRedirected, bool aReportOnly,
@@ -485,15 +579,11 @@ void nsCSPSchemeSrc::toString(nsAString& outStr) const {
 
 /* ===== nsCSPHostSrc ======================== */
 
-nsCSPHostSrc::nsCSPHostSrc(const nsAString& aHost)
-    : mHost(aHost),
-      mGeneratedFromSelfKeyword(false),
-      mIsUniqueOrigin(false),
-      mWithinFrameAncstorsDir(false) {
+nsCSPHostSrc::nsCSPHostSrc(const nsAString& aHost) : mHost(aHost) {
   ToLowerCase(mHost);
 }
 
-nsCSPHostSrc::~nsCSPHostSrc() {}
+nsCSPHostSrc::~nsCSPHostSrc() = default;
 
 /*
  * Checks whether the current directive permits a specific port.
@@ -514,8 +604,21 @@ bool permitsPort(const nsAString& aEnforcementScheme,
 
   int32_t resourcePort;
   nsresult rv = aResourceURI->GetPort(&resourcePort);
-  NS_ENSURE_SUCCESS(rv, false);
+  if (NS_FAILED(rv) && aEnforcementPort.IsEmpty()) {
+    // If we cannot get a Port (e.g. because of an Custom Protocol handler)
+    // We need to check if a default port is associated with the Scheme
+    if (aEnforcementScheme.IsEmpty()) {
+      return false;
+    }
+    int defaultPortforScheme =
+        NS_GetDefaultPort(NS_ConvertUTF16toUTF8(aEnforcementScheme).get());
 
+    // If there is no default port associated with the Scheme (
+    // defaultPortforScheme == -1) or it is an externally handled protocol (
+    // defaultPortforScheme == 0 ) and the csp does not enforce a port - we can
+    // allow not having a port
+    return (defaultPortforScheme == -1 || defaultPortforScheme == -0);
+  }
   // Avoid unnecessary string creation/manipulation and don't block the
   // load if the resource to be loaded uses the default port for that
   // scheme and there is no port to be enforced.
@@ -600,28 +703,6 @@ bool nsCSPHostSrc::permits(nsIURI* aUri, const nsAString& aNonce,
   // just a specific scheme, the parser should generate a nsCSPSchemeSource.
   NS_ASSERTION((!mHost.IsEmpty()), "host can not be the empty string");
 
-  // 2) host matching: Enforce a single *
-  if (mHost.EqualsASCII("*")) {
-    // The single ASTERISK character (*) does not match a URI's scheme of a type
-    // designating a globally unique identifier (such as blob:, data:, or
-    // filesystem:) At the moment firefox does not support filesystem; but for
-    // future compatibility we support it in CSP according to the spec,
-    // see: 4.2.2 Matching Source Expressions Note, that whitelisting any of
-    // these schemes would call nsCSPSchemeSrc::permits().
-    bool isBlobScheme =
-        (NS_SUCCEEDED(aUri->SchemeIs("blob", &isBlobScheme)) && isBlobScheme);
-    bool isDataScheme =
-        (NS_SUCCEEDED(aUri->SchemeIs("data", &isDataScheme)) && isDataScheme);
-    bool isFileScheme =
-        (NS_SUCCEEDED(aUri->SchemeIs("filesystem", &isFileScheme)) &&
-         isFileScheme);
-
-    if (isBlobScheme || isDataScheme || isFileScheme) {
-      return false;
-    }
-    return true;
-  }
-
   // Before we can check if the host matches, we have to
   // extract the host part from aUri.
   nsAutoCString uriHost;
@@ -631,8 +712,27 @@ bool nsCSPHostSrc::permits(nsIURI* aUri, const nsAString& aNonce,
   nsString decodedUriHost;
   CSP_PercentDecodeStr(NS_ConvertUTF8toUTF16(uriHost), decodedUriHost);
 
+  // 2) host matching: Enforce a single *
+  if (mHost.EqualsASCII("*")) {
+    // The single ASTERISK character (*) does not match a URI's scheme of a type
+    // designating a globally unique identifier (such as blob:, data:, or
+    // filesystem:) At the moment firefox does not support filesystem; but for
+    // future compatibility we support it in CSP according to the spec,
+    // see: 4.2.2 Matching Source Expressions Note, that allowlisting any of
+    // these schemes would call nsCSPSchemeSrc::permits().
+    if (aUri->SchemeIs("blob") || aUri->SchemeIs("data") ||
+        aUri->SchemeIs("filesystem")) {
+      return false;
+    }
+
+    // If no scheme is present there also wont be a port and folder to check
+    // which means we can return early
+    if (mScheme.IsEmpty()) {
+      return true;
+    }
+  }
   // 4.5) host matching: Check if the allowed host starts with a wilcard.
-  if (mHost.First() == '*') {
+  else if (mHost.First() == '*') {
     NS_ASSERTION(
         mHost[1] == '.',
         "Second character needs to be '.' whenever host starts with '*'");
@@ -687,8 +787,8 @@ bool nsCSPHostSrc::permits(nsIURI* aUri, const nsAString& aNonce,
         return false;
       }
     }
-    // otherwise mPath whitelists a specific file, and we have to
-    // check if the loading resource matches that whitelisted file.
+    // otherwise mPath refers to a specific file, and we have to
+    // check if the loading resource matches the file.
     else {
       if (!mPath.Equals(decodedUriPath)) {
         return false;
@@ -716,11 +816,13 @@ void nsCSPHostSrc::toString(nsAString& outStr) const {
     return;
   }
 
-  // append scheme
-  outStr.Append(mScheme);
+  // append scheme if it wasn't generated from the mSelfURI
+  if (!mGeneratedScheme) {
+    outStr.Append(mScheme);
+    outStr.AppendLiteral("://");
+  }
 
   // append host
-  outStr.AppendLiteral("://");
   outStr.Append(mHost);
 
   // append port
@@ -750,7 +852,7 @@ nsCSPKeywordSrc::nsCSPKeywordSrc(enum CSPKeyword aKeyword)
                "'self' should have been replaced in the parser");
 }
 
-nsCSPKeywordSrc::~nsCSPKeywordSrc() {}
+nsCSPKeywordSrc::~nsCSPKeywordSrc() = default;
 
 bool nsCSPKeywordSrc::permits(nsIURI* aUri, const nsAString& aNonce,
                               bool aWasRedirected, bool aReportOnly,
@@ -770,23 +872,24 @@ bool nsCSPKeywordSrc::allows(enum CSPKeyword aKeyword,
        "%s",
        CSP_EnumToUTF8Keyword(aKeyword),
        NS_ConvertUTF16toUTF8(aHashOrNonce).get(),
-       mInvalidated ? "yes" : "false"));
+       mInvalidated ? "true" : "false"));
 
   if (mInvalidated) {
-    // only 'self' and 'unsafe-inline' are keywords that can be ignored. Please
-    // note that the parser already translates 'self' into a uri (see assertion
-    // in constructor).
-    MOZ_ASSERT(mKeyword == CSP_UNSAFE_INLINE,
+    // only 'self', 'report-sample' and 'unsafe-inline' are keywords that can be
+    // ignored. Please note that the parser already translates 'self' into a uri
+    // (see assertion in constructor).
+    MOZ_ASSERT(mKeyword == CSP_UNSAFE_INLINE || mKeyword == CSP_REPORT_SAMPLE,
                "should only invalidate unsafe-inline");
     return false;
   }
   // either the keyword allows the load or the policy contains 'strict-dynamic',
   // in which case we have to make sure the script is not parser created before
-  // allowing the load and also eval should be blocked even if 'strict-dynamic'
-  // is present. Should be allowed only if 'unsafe-eval' is present.
+  // allowing the load and also eval & wasm-eval should be blocked even if
+  // 'strict-dynamic' is present. Should be allowed only if 'unsafe-eval' is
+  // present.
   return ((mKeyword == aKeyword) ||
           ((mKeyword == CSP_STRICT_DYNAMIC) && !aParserCreated &&
-           aKeyword != CSP_UNSAFE_EVAL));
+           aKeyword != CSP_UNSAFE_EVAL && aKeyword != CSP_WASM_UNSAFE_EVAL));
 }
 
 bool nsCSPKeywordSrc::visit(nsCSPSrcVisitor* aVisitor) const {
@@ -801,7 +904,7 @@ void nsCSPKeywordSrc::toString(nsAString& outStr) const {
 
 nsCSPNonceSrc::nsCSPNonceSrc(const nsAString& aNonce) : mNonce(aNonce) {}
 
-nsCSPNonceSrc::~nsCSPNonceSrc() {}
+nsCSPNonceSrc::~nsCSPNonceSrc() = default;
 
 bool nsCSPNonceSrc::permits(nsIURI* aUri, const nsAString& aNonce,
                             bool aWasRedirected, bool aReportOnly,
@@ -812,6 +915,25 @@ bool nsCSPNonceSrc::permits(nsIURI* aUri, const nsAString& aNonce,
                  NS_ConvertUTF16toUTF8(aNonce).get()));
   }
 
+  if (aReportOnly && aWasRedirected && aNonce.IsEmpty()) {
+    /* Fix for Bug 1505412
+     *  If we land here, we're currently handling a script-preload which got
+     *  redirected. Preloads do not have any info about the nonce assiociated.
+     *  Because of Report-Only the preload passes the 1st CSP-check so the
+     *  preload does not get retried with a nonce attached.
+     *  Currently we're relying on the script-manager to
+     *  provide a fake loadinfo to check the preloads against csp.
+     *  So during HTTPChannel->OnRedirect we cant check csp for this case.
+     *  But as the script-manager already checked the csp,
+     *  a report would already have been send,
+     *  if the nonce didnt match.
+     *  So we can pass the check here for Report-Only Cases.
+     */
+    MOZ_ASSERT(aParserCreated == false,
+               "Skipping nonce-check is only allowed for Preloads");
+    return true;
+  }
+
   // nonces can not be invalidated by strict-dynamic
   return mNonce.Equals(aNonce);
 }
@@ -847,9 +969,21 @@ nsCSPHashSrc::nsCSPHashSrc(const nsAString& aAlgo, const nsAString& aHash)
   // Only the algo should be rewritten to lowercase, the hash must remain the
   // same.
   ToLowerCase(mAlgorithm);
+  // Normalize the base64url encoding to base64 encoding:
+  char16_t* cur = mHash.BeginWriting();
+  char16_t* end = mHash.EndWriting();
+
+  for (; cur < end; ++cur) {
+    if (char16_t('-') == *cur) {
+      *cur = char16_t('+');
+    }
+    if (char16_t('_') == *cur) {
+      *cur = char16_t('/');
+    }
+  }
 }
 
-nsCSPHashSrc::~nsCSPHashSrc() {}
+nsCSPHashSrc::~nsCSPHashSrc() = default;
 
 bool nsCSPHashSrc::allows(enum CSPKeyword aKeyword,
                           const nsAString& aHashOrNonce,
@@ -867,12 +1001,9 @@ bool nsCSPHashSrc::allows(enum CSPKeyword aKeyword,
   // Convert aHashOrNonce to UTF-8
   NS_ConvertUTF16toUTF8 utf8_hash(aHashOrNonce);
 
-  nsresult rv;
   nsCOMPtr<nsICryptoHash> hasher;
-  hasher = do_CreateInstance("@mozilla.org/security/hash;1", &rv);
-  NS_ENSURE_SUCCESS(rv, false);
-
-  rv = hasher->InitWithString(NS_ConvertUTF16toUTF8(mAlgorithm));
+  nsresult rv = NS_NewCryptoHash(NS_ConvertUTF16toUTF8(mAlgorithm),
+                                 getter_AddRefs(hasher));
   NS_ENSURE_SUCCESS(rv, false);
 
   rv = hasher->Update((uint8_t*)utf8_hash.get(), utf8_hash.Length());
@@ -901,7 +1032,7 @@ void nsCSPHashSrc::toString(nsAString& outStr) const {
 
 nsCSPReportURI::nsCSPReportURI(nsIURI* aURI) : mReportURI(aURI) {}
 
-nsCSPReportURI::~nsCSPReportURI() {}
+nsCSPReportURI::~nsCSPReportURI() = default;
 
 bool nsCSPReportURI::visit(nsCSPSrcVisitor* aVisitor) const { return false; }
 
@@ -920,7 +1051,7 @@ nsCSPSandboxFlags::nsCSPSandboxFlags(const nsAString& aFlags) : mFlags(aFlags) {
   ToLowerCase(mFlags);
 }
 
-nsCSPSandboxFlags::~nsCSPSandboxFlags() {}
+nsCSPSandboxFlags::~nsCSPSandboxFlags() = default;
 
 bool nsCSPSandboxFlags::visit(nsCSPSrcVisitor* aVisitor) const { return false; }
 
@@ -978,13 +1109,10 @@ void nsCSPDirective::toString(nsAString& outStr) const {
   outStr.AppendLiteral(" ");
 
   // Append srcs
-  uint32_t length = mSrcs.Length();
-  for (uint32_t i = 0; i < length; i++) {
-    mSrcs[i]->toString(outStr);
-    if (i != (length - 1)) {
-      outStr.AppendLiteral(" ");
-    }
-  }
+  StringJoinAppend(outStr, u" "_ns, mSrcs,
+                   [](nsAString& dest, nsCSPBaseSrc* cspBaseSrc) {
+                     cspBaseSrc->toString(dest);
+                   });
 }
 
 void nsCSPDirective::toDomCSPStruct(mozilla::dom::CSP& outCSP) const {
@@ -993,7 +1121,12 @@ void nsCSPDirective::toDomCSPStruct(mozilla::dom::CSP& outCSP) const {
   for (uint32_t i = 0; i < mSrcs.Length(); i++) {
     src.Truncate();
     mSrcs[i]->toString(src);
-    srcs.AppendElement(src, mozilla::fallible);
+    if (!srcs.AppendElement(src, mozilla::fallible)) {
+      // XXX(Bug 1632090) Instead of extending the array 1-by-1 (which might
+      // involve multiple reallocations) and potentially crashing here,
+      // SetCapacity could be called outside the loop once.
+      mozalloc_handle_oom(0);
+    }
   }
 
   switch (mDirective) {
@@ -1093,20 +1226,21 @@ void nsCSPDirective::toDomCSPStruct(mozilla::dom::CSP& outCSP) const {
       outCSP.mWorker_src.Value() = std::move(srcs);
       return;
 
+    case nsIContentSecurityPolicy::SCRIPT_SRC_ELEM_DIRECTIVE:
+      outCSP.mScript_src_elem.Construct();
+      outCSP.mScript_src_elem.Value() = std::move(srcs);
+      return;
+
+    case nsIContentSecurityPolicy::SCRIPT_SRC_ATTR_DIRECTIVE:
+      outCSP.mScript_src_attr.Construct();
+      outCSP.mScript_src_attr.Value() = std::move(srcs);
+      return;
+
     default:
       NS_ASSERTION(false, "cannot find directive to convert CSP to JSON");
   }
 }
 
-bool nsCSPDirective::restrictsContentType(
-    nsContentPolicyType aContentType) const {
-  // make sure we do not check for the default src before any other sources
-  if (isDefaultDirective()) {
-    return false;
-  }
-  return mDirective == CSP_ContentTypeToDirective(aContentType);
-}
-
 void nsCSPDirective::getReportURIs(nsTArray<nsString>& outReportURIs) const {
   NS_ASSERTION((mDirective == nsIContentSecurityPolicy::REPORT_URI_DIRECTIVE),
                "not a report-uri directive");
@@ -1154,20 +1288,7 @@ nsCSPChildSrcDirective::nsCSPChildSrcDirective(CSPDirective aDirective)
       mRestrictFrames(false),
       mRestrictWorkers(false) {}
 
-nsCSPChildSrcDirective::~nsCSPChildSrcDirective() {}
-
-bool nsCSPChildSrcDirective::restrictsContentType(
-    nsContentPolicyType aContentType) const {
-  if (aContentType == nsIContentPolicy::TYPE_SUBDOCUMENT) {
-    return mRestrictFrames;
-  }
-  if (aContentType == nsIContentPolicy::TYPE_INTERNAL_WORKER ||
-      aContentType == nsIContentPolicy::TYPE_INTERNAL_SHARED_WORKER ||
-      aContentType == nsIContentPolicy::TYPE_INTERNAL_SERVICE_WORKER) {
-    return mRestrictWorkers;
-  }
-  return false;
-}
+nsCSPChildSrcDirective::~nsCSPChildSrcDirective() = default;
 
 bool nsCSPChildSrcDirective::equals(CSPDirective aDirective) const {
   if (aDirective == nsIContentSecurityPolicy::FRAME_SRC_DIRECTIVE) {
@@ -1182,25 +1303,38 @@ bool nsCSPChildSrcDirective::equals(CSPDirective aDirective) const {
 /* =============== nsCSPScriptSrcDirective ============= */
 
 nsCSPScriptSrcDirective::nsCSPScriptSrcDirective(CSPDirective aDirective)
-    : nsCSPDirective(aDirective), mRestrictWorkers(false) {}
+    : nsCSPDirective(aDirective) {}
 
-nsCSPScriptSrcDirective::~nsCSPScriptSrcDirective() {}
-
-bool nsCSPScriptSrcDirective::restrictsContentType(
-    nsContentPolicyType aContentType) const {
-  if (aContentType == nsIContentPolicy::TYPE_INTERNAL_WORKER ||
-      aContentType == nsIContentPolicy::TYPE_INTERNAL_SHARED_WORKER ||
-      aContentType == nsIContentPolicy::TYPE_INTERNAL_SERVICE_WORKER) {
-    return mRestrictWorkers;
-  }
-  return mDirective == CSP_ContentTypeToDirective(aContentType);
-}
+nsCSPScriptSrcDirective::~nsCSPScriptSrcDirective() = default;
 
 bool nsCSPScriptSrcDirective::equals(CSPDirective aDirective) const {
   if (aDirective == nsIContentSecurityPolicy::WORKER_SRC_DIRECTIVE) {
     return mRestrictWorkers;
   }
-  return (mDirective == aDirective);
+  if (aDirective == nsIContentSecurityPolicy::SCRIPT_SRC_ELEM_DIRECTIVE) {
+    return mRestrictScriptElem;
+  }
+  if (aDirective == nsIContentSecurityPolicy::SCRIPT_SRC_ATTR_DIRECTIVE) {
+    return mRestrictScriptAttr;
+  }
+  return mDirective == aDirective;
+}
+
+/* =============== nsCSPStyleSrcDirective ============= */
+
+nsCSPStyleSrcDirective::nsCSPStyleSrcDirective(CSPDirective aDirective)
+    : nsCSPDirective(aDirective) {}
+
+nsCSPStyleSrcDirective::~nsCSPStyleSrcDirective() = default;
+
+bool nsCSPStyleSrcDirective::equals(CSPDirective aDirective) const {
+  if (aDirective == nsIContentSecurityPolicy::STYLE_SRC_ELEM_DIRECTIVE) {
+    return mRestrictStyleElem;
+  }
+  if (aDirective == nsIContentSecurityPolicy::STYLE_SRC_ATTR_DIRECTIVE) {
+    return mRestrictStyleAttr;
+  }
+  return mDirective == aDirective;
 }
 
 /* =============== nsBlockAllMixedContentDirective ============= */
@@ -1209,7 +1343,7 @@ nsBlockAllMixedContentDirective::nsBlockAllMixedContentDirective(
     CSPDirective aDirective)
     : nsCSPDirective(aDirective) {}
 
-nsBlockAllMixedContentDirective::~nsBlockAllMixedContentDirective() {}
+nsBlockAllMixedContentDirective::~nsBlockAllMixedContentDirective() = default;
 
 void nsBlockAllMixedContentDirective::toString(nsAString& outStr) const {
   outStr.AppendASCII(CSP_CSPDirectiveToString(
@@ -1226,7 +1360,7 @@ void nsBlockAllMixedContentDirective::getDirName(nsAString& outStr) const {
 nsUpgradeInsecureDirective::nsUpgradeInsecureDirective(CSPDirective aDirective)
     : nsCSPDirective(aDirective) {}
 
-nsUpgradeInsecureDirective::~nsUpgradeInsecureDirective() {}
+nsUpgradeInsecureDirective::~nsUpgradeInsecureDirective() = default;
 
 void nsUpgradeInsecureDirective::toString(nsAString& outStr) const {
   outStr.AppendASCII(CSP_CSPDirectiveToString(
@@ -1255,13 +1389,6 @@ nsCSPPolicy::~nsCSPPolicy() {
   }
 }
 
-bool nsCSPPolicy::permits(CSPDirective aDir, nsIURI* aUri,
-                          bool aSpecific) const {
-  nsString outp;
-  return this->permits(aDir, aUri, EmptyString(), false, aSpecific, false,
-                       outp);
-}
-
 bool nsCSPPolicy::permits(CSPDirective aDir, nsIURI* aUri,
                           const nsAString& aNonce, bool aWasRedirected,
                           bool aSpecific, bool aParserCreated,
@@ -1310,8 +1437,7 @@ bool nsCSPPolicy::permits(CSPDirective aDir, nsIURI* aUri,
   return true;
 }
 
-bool nsCSPPolicy::allows(nsContentPolicyType aContentType,
-                         enum CSPKeyword aKeyword,
+bool nsCSPPolicy::allows(CSPDirective aDirective, enum CSPKeyword aKeyword,
                          const nsAString& aHashOrNonce,
                          bool aParserCreated) const {
   CSPUTILSLOG(("nsCSPPolicy::allows, aKeyWord: %s, a HashOrNonce: %s",
@@ -1322,25 +1448,16 @@ bool nsCSPPolicy::allows(nsContentPolicyType aContentType,
 
   // Try to find a matching directive
   for (uint32_t i = 0; i < mDirectives.Length(); i++) {
-    if (mDirectives[i]->restrictsContentType(aContentType)) {
+    if (mDirectives[i]->isDefaultDirective()) {
+      defaultDir = mDirectives[i];
+      continue;
+    }
+    if (mDirectives[i]->equals(aDirective)) {
       if (mDirectives[i]->allows(aKeyword, aHashOrNonce, aParserCreated)) {
         return true;
       }
       return false;
     }
-    if (mDirectives[i]->isDefaultDirective()) {
-      defaultDir = mDirectives[i];
-    }
-  }
-
-  // {nonce,hash}-source should not consult default-src:
-  //   * return false if default-src is specified
-  //   * but allow the load if default-src is *not* specified (Bug 1198422)
-  if (aKeyword == CSP_NONCE || aKeyword == CSP_HASH) {
-    if (!defaultDir) {
-      return true;
-    }
-    return false;
   }
 
   // If the above loop runs through, we haven't found a matching directive.
@@ -1357,19 +1474,11 @@ bool nsCSPPolicy::allows(nsContentPolicyType aContentType,
   return true;
 }
 
-bool nsCSPPolicy::allows(nsContentPolicyType aContentType,
-                         enum CSPKeyword aKeyword) const {
-  return allows(aContentType, aKeyword, NS_LITERAL_STRING(""), false);
-}
-
 void nsCSPPolicy::toString(nsAString& outStr) const {
-  uint32_t length = mDirectives.Length();
-  for (uint32_t i = 0; i < length; ++i) {
-    mDirectives[i]->toString(outStr);
-    if (i != (length - 1)) {
-      outStr.AppendLiteral("; ");
-    }
-  }
+  StringJoinAppend(outStr, u"; "_ns, mDirectives,
+                   [](nsAString& dest, nsCSPDirective* cspDirective) {
+                     cspDirective->toString(dest);
+                   });
 }
 
 void nsCSPPolicy::toDomCSPStruct(mozilla::dom::CSP& outCSP) const {
@@ -1389,6 +1498,30 @@ bool nsCSPPolicy::hasDirective(CSPDirective aDir) const {
   return false;
 }
 
+bool nsCSPPolicy::allowsNavigateTo(nsIURI* aURI, bool aWasRedirected,
+                                   bool aEnforceAllowlist) const {
+  bool allowsNavigateTo = true;
+
+  for (unsigned long i = 0; i < mDirectives.Length(); i++) {
+    if (mDirectives[i]->equals(
+            nsIContentSecurityPolicy::NAVIGATE_TO_DIRECTIVE)) {
+      // Early return if we can skip the allowlist AND 'unsafe-allow-redirects'
+      // is present.
+      if (!aEnforceAllowlist &&
+          mDirectives[i]->allows(CSP_UNSAFE_ALLOW_REDIRECTS, u""_ns, false)) {
+        return true;
+      }
+      // Otherwise, check against the allowlist.
+      if (!mDirectives[i]->permits(aURI, u""_ns, aWasRedirected, false, false,
+                                   false)) {
+        allowsNavigateTo = false;
+      }
+    }
+  }
+
+  return allowsNavigateTo;
+}
+
 /*
  * Use this function only after ::allows() returned 'false'. Most and
  * foremost it's used to get the violated directive before sending reports.
@@ -1396,21 +1529,22 @@ bool nsCSPPolicy::hasDirective(CSPDirective aDir) const {
  * for the ::permits() function family.
  */
 void nsCSPPolicy::getDirectiveStringAndReportSampleForContentType(
-    nsContentPolicyType aContentType, nsAString& outDirective,
+    CSPDirective aDirective, nsAString& outDirective,
     bool* aReportSample) const {
   MOZ_ASSERT(aReportSample);
   *aReportSample = false;
 
   nsCSPDirective* defaultDir = nullptr;
   for (uint32_t i = 0; i < mDirectives.Length(); i++) {
-    if (mDirectives[i]->restrictsContentType(aContentType)) {
+    if (mDirectives[i]->isDefaultDirective()) {
+      defaultDir = mDirectives[i];
+      continue;
+    }
+    if (mDirectives[i]->equals(aDirective)) {
       mDirectives[i]->getDirName(outDirective);
       *aReportSample = mDirectives[i]->hasReportSampleKeyword();
       return;
     }
-    if (mDirectives[i]->isDefaultDirective()) {
-      defaultDir = mDirectives[i];
-    }
   }
   // if we haven't found a matching directive yet,
   // the contentType must be restricted by the default directive
