@@ -40,10 +40,7 @@ using mozilla::dom::cache::Cache;
 using mozilla::dom::cache::CacheStorage;
 using mozilla::ipc::PrincipalInfo;
 
-namespace mozilla {
-namespace dom {
-
-namespace serviceWorkerScriptCache {
+namespace mozilla::dom::serviceWorkerScriptCache {
 
 namespace {
 
@@ -802,7 +799,7 @@ void CompareNetwork::Abort() {
     mState = Finished;
 
     MOZ_ASSERT(mChannel);
-    mChannel->Cancel(NS_BINDING_ABORTED);
+    mChannel->CancelWithReason(NS_BINDING_ABORTED, "CompareNetwork::Abort"_ns);
     mChannel = nullptr;
 
     if (mCC) {
@@ -1157,7 +1154,7 @@ void CompareCache::Abort() {
     mState = Finished;
 
     if (mPump) {
-      mPump->Cancel(NS_BINDING_ABORTED);
+      mPump->CancelWithReason(NS_BINDING_ABORTED, "CompareCache::Abort"_ns);
       mPump = nullptr;
     }
   }
@@ -1503,7 +1500,4 @@ nsresult Compare(ServiceWorkerRegistrationInfo* aRegistration,
   return NS_OK;
 }
 
-}  // namespace serviceWorkerScriptCache
-
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom::serviceWorkerScriptCache
