@@ -21,12 +21,12 @@
 
 // IMPORTANT: Do not change this list without review from
 //            a JavaScript Engine peer!
-var wasmGlobalEntry = {
+let wasmGlobalEntry = {
   name: "WebAssembly",
   insecureContext: true,
   disabled: !getJSTestingFunctions().wasmIsSupportedByHardware(),
 };
-var wasmGlobalInterfaces = [
+let wasmGlobalInterfaces = [
   { name: "Module", insecureContext: true },
   { name: "Instance", insecureContext: true },
   { name: "Memory", insecureContext: true },
@@ -35,23 +35,18 @@ var wasmGlobalInterfaces = [
   { name: "CompileError", insecureContext: true },
   { name: "LinkError", insecureContext: true },
   { name: "RuntimeError", insecureContext: true },
-  {
-    name: "Function",
-    insecureContext: true,
-    nightly: true,
-  },
-  {
-    name: "Exception",
-    insecureContext: true,
-  },
-  {
-    name: "Tag",
-    insecureContext: true,
-  },
+  { name: "Function", insecureContext: true, nightly: true },
+  { name: "Exception", insecureContext: true },
+  { name: "Tag", insecureContext: true },
+  { name: "compile", insecureContext: true },
+  { name: "compileStreaming", insecureContext: true },
+  { name: "instantiate", insecureContext: true },
+  { name: "instantiateStreaming", insecureContext: true },
+  { name: "validate", insecureContext: true },
 ];
 // IMPORTANT: Do not change this list without review from
 //            a JavaScript Engine peer!
-var ecmaGlobals = [
+let ecmaGlobals = [
   "AggregateError",
   "Array",
   "ArrayBuffer",
@@ -104,12 +99,25 @@ var ecmaGlobals = [
   "WeakRef",
   "WeakSet",
   wasmGlobalEntry,
+  "decodeURI",
+  "decodeURIComponent",
+  "encodeURI",
+  "encodeURIComponent",
+  "escape",
+  "eval",
+  "globalThis",
+  "isFinite",
+  "isNaN",
+  "parseFloat",
+  "parseInt",
+  "undefined",
+  "unescape",
 ];
 // IMPORTANT: Do not change the list above without review from
 //            a JavaScript Engine peer!
 
 // IMPORTANT: Do not change the list below without review from a DOM peer!
-var interfaceNamesInGlobalScope = [
+let interfaceNamesInGlobalScope = [
   // IMPORTANT: Do not change this list without review from a DOM peer!
   "AbortController",
   // IMPORTANT: Do not change this list without review from a DOM peer!
@@ -125,6 +133,10 @@ var interfaceNamesInGlobalScope = [
   // IMPORTANT: Do not change this list without review from a DOM peer!
   "CacheStorage",
   // IMPORTANT: Do not change this list without review from a DOM peer!
+  "CanvasGradient",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "CanvasPattern",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
   "Client",
   // IMPORTANT: Do not change this list without review from a DOM peer!
   "Clients",
@@ -177,6 +189,20 @@ var interfaceNamesInGlobalScope = [
   // IMPORTANT: Do not change this list without review from a DOM peer!
   "FileReader",
   // IMPORTANT: Do not change this list without review from a DOM peer!
+  { name: "FileSystemDirectoryHandle", nightly: true },
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  { name: "FileSystemFileHandle", nightly: true },
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  { name: "FileSystemHandle", nightly: true },
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  { name: "FileSystemWritableFileStream", nightly: true },
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "FontFace",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "FontFaceSet",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "FontFaceSetLoadEvent",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
   "FormData",
   // IMPORTANT: Do not change this list without review from a DOM peer!
   "Headers",
@@ -231,6 +257,12 @@ var interfaceNamesInGlobalScope = [
   // IMPORTANT: Do not change this list without review from a DOM peer!
   "NotificationEvent",
   // IMPORTANT: Do not change this list without review from a DOM peer!
+  "OffscreenCanvas",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "OffscreenCanvasRenderingContext2D",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "Path2D",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
   "Performance",
   // IMPORTANT: Do not change this list without review from a DOM peer!
   "PerformanceEntry",
@@ -261,18 +293,18 @@ var interfaceNamesInGlobalScope = [
   // IMPORTANT: Do not change this list without review from a DOM peer!
   { name: "PushSubscriptionOptions" },
   // IMPORTANT: Do not change this list without review from a DOM peer!
+  "ReadableByteStreamController",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
   "ReadableStream",
   // IMPORTANT: Do not change this list without review from a DOM peer!
+  "ReadableStreamBYOBReader",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "ReadableStreamBYOBRequest",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
   "ReadableStreamDefaultController",
   // IMPORTANT: Do not change this list without review from a DOM peer!
   "ReadableStreamDefaultReader",
   // IMPORTANT: Do not change this list without review from a DOM peer!
-  { name: "Report", nightly: true },
-  // IMPORTANT: Do not change this list without review from a DOM peer!
-  { name: "ReportBody", nightly: true },
-  // IMPORTANT: Do not change this list without review from a DOM peer!
-  { name: "ReportingObserver", nightly: true },
-  // IMPORTANT: Do not change this list without review from a DOM peer!
   "Request",
   // IMPORTANT: Do not change this list without review from a DOM peer!
   "Response",
@@ -297,11 +329,17 @@ var interfaceNamesInGlobalScope = [
   // IMPORTANT: Do not change this list without review from a DOM peer!
   "TextDecoder",
   // IMPORTANT: Do not change this list without review from a DOM peer!
+  "TextDecoderStream",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
   "TextEncoder",
   // IMPORTANT: Do not change this list without review from a DOM peer!
-  { name: "TransformStream", nightly: true },
+  "TextEncoderStream",
   // IMPORTANT: Do not change this list without review from a DOM peer!
-  { name: "TransformStreamDefaultController", nightly: true },
+  "TextMetrics",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "TransformStream",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "TransformStreamDefaultController",
   // IMPORTANT: Do not change this list without review from a DOM peer!
   "URL",
   // IMPORTANT: Do not change this list without review from a DOM peer!
@@ -309,8 +347,40 @@ var interfaceNamesInGlobalScope = [
   // IMPORTANT: Do not change this list without review from a DOM peer!
   "WebSocket",
   // IMPORTANT: Do not change this list without review from a DOM peer!
+  "WebGL2RenderingContext",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "WebGLActiveInfo",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "WebGLBuffer",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "WebGLContextEvent",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "WebGLFramebuffer",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "WebGLProgram",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
   "WebGLQuery",
   // IMPORTANT: Do not change this list without review from a DOM peer!
+  "WebGLRenderbuffer",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "WebGLRenderingContext",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "WebGLSampler",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "WebGLShader",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "WebGLShaderPrecisionFormat",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "WebGLSync",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "WebGLTexture",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "WebGLTransformFeedback",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "WebGLUniformLocation",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "WebGLVertexArrayObject",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
   "WindowClient",
   // IMPORTANT: Do not change this list without review from a DOM peer!
   "WorkerGlobalScope",
@@ -324,9 +394,50 @@ var interfaceNamesInGlobalScope = [
   "WritableStreamDefaultController",
   // IMPORTANT: Do not change this list without review from a DOM peer!
   "WritableStreamDefaultWriter",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "clients",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "console",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "onactivate",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "onfetch",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "oninstall",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "onmessage",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "onmessageerror",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "onnotificationclick",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "onnotificationclose",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "onpush",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "onpushsubscriptionchange",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "registration",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
+  "skipWaiting",
+  // IMPORTANT: Do not change this list without review from a DOM peer!
 ];
 // IMPORTANT: Do not change the list above without review from a DOM peer!
 
+// List of functions defined on the global by the test harness or this test
+// file.
+let testFunctions = [
+  "ok",
+  "is",
+  "workerTestArrayEquals",
+  "workerTestDone",
+  "workerTestGetHelperData",
+  "workerTestGetStorageManager",
+  "entryDisabled",
+  "createInterfaceMap",
+  "runTest",
+];
+
 function entryDisabled(
   entry,
   {
@@ -363,8 +474,10 @@ function createInterfaceMap(data, ...interfaceGroups) {
   function addInterfaces(interfaces) {
     for (var entry of interfaces) {
       if (typeof entry === "string") {
+        ok(!(entry in interfaceMap), "duplicate entry for " + entry);
         interfaceMap[entry] = true;
       } else {
+        ok(!(entry.name in interfaceMap), "duplicate entry for " + entry.name);
         ok(!("pref" in entry), "Bogus pref annotation for " + entry.name);
         if (entryDisabled(entry, data)) {
           interfaceMap[entry.name] = false;
@@ -387,8 +500,8 @@ function createInterfaceMap(data, ...interfaceGroups) {
 function runTest(parentName, parent, data, ...interfaceGroups) {
   var interfaceMap = createInterfaceMap(data, ...interfaceGroups);
   for (var name of Object.getOwnPropertyNames(parent)) {
-    // An interface name should start with an upper case character.
-    if (!/^[A-Z]/.test(name)) {
+    // Ignore functions on the global that are part of the test (harness).
+    if (parent === self && testFunctions.includes(name)) {
       continue;
     }
     ok(
