@@ -20,6 +20,7 @@
 #include "mozilla/Assertions.h"
 #include "mozilla/Atomics.h"
 #include "mozilla/DebugOnly.h"
+#include "mozilla/FixedBufferOutputStream.h"
 #include "mozilla/Maybe.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/RefPtr.h"
@@ -37,7 +38,6 @@
 #include "mozilla/dom/quota/ClientImpl.h"
 #include "mozilla/dom/quota/DirectoryLock.h"
 #include "mozilla/dom/quota/FileStreams.h"
-#include "mozilla/dom/quota/MemoryOutputStream.h"
 #include "mozilla/dom/quota/QuotaCommon.h"
 #include "mozilla/dom/quota/QuotaManager.h"
 #include "mozilla/dom/quota/ResultExtensions.h"
@@ -92,11 +92,12 @@ constexpr auto kSDBSuffix = u".sdb"_ns;
 
 class StreamHelper final : public Runnable {
   nsCOMPtr<nsIEventTarget> mOwningEventTarget;
-  nsCOMPtr<nsIFileStream> mFileStream;
+  nsCOMPtr<nsIFileRandomAccessStream> mFileRandomAccessStream;
   nsCOMPtr<nsIRunnable> mCallback;
 
  public:
-  StreamHelper(nsIFileStream* aFileStream, nsIRunnable* aCallback);
+  StreamHelper(nsIFileRandomAccessStream* aFileRandomAccessStream,
+               nsIRunnable* aCallback);
 
   void AsyncClose();
 
@@ -112,7 +113,7 @@ class StreamHelper final : public Runnable {
 
 class Connection final : public PBackgroundSDBConnectionParent {
   RefPtr<DirectoryLock> mDirectoryLock;
-  nsCOMPtr<nsIFileStream> mFileStream;
+  nsCOMPtr<nsIFileRandomAccessStream> mFileRandomAccessStream;
   const PrincipalInfo mPrincipalInfo;
   nsCString mOrigin;
   nsString mName;
@@ -135,10 +136,10 @@ class Connection final : public PBackgroundSDBConnectionParent {
     return ToMaybeRef(mDirectoryLock.get());
   }
 
-  nsIFileStream* GetFileStream() const {
+  nsIFileRandomAccessStream* GetFileRandomAccessStream() const {
     AssertIsOnIOThread();
 
-    return mFileStream;
+    return mFileRandomAccessStream;
   }
 
   PersistenceType GetPersistenceType() const { return mPersistenceType; }
@@ -167,9 +168,10 @@ class Connection final : public PBackgroundSDBConnectionParent {
 
   void OnRequestFinished();
 
-  void OnOpen(const nsACString& aOrigin, const nsAString& aName,
-              already_AddRefed<DirectoryLock> aDirectoryLock,
-              already_AddRefed<nsIFileStream> aFileStream);
+  void OnOpen(
+      const nsACString& aOrigin, const nsAString& aName,
+      already_AddRefed<DirectoryLock> aDirectoryLock,
+      already_AddRefed<nsIFileRandomAccessStream> aFileRandomAccessStream);
 
   void OnClose();
 
@@ -283,7 +285,8 @@ class ConnectionOperationBase : public Runnable,
   void DatabaseWork();
 
   // Methods that subclasses must implement.
-  virtual nsresult DoDatabaseWork(nsIFileStream* aFileStream) = 0;
+  virtual nsresult DoDatabaseWork(
+      nsIFileRandomAccessStream* aFileRandomAccessStream) = 0;
 
   // Subclasses use this override to set the IPDL response value.
   virtual void GetResponse(SDBRequestResponse& aResponse) = 0;
@@ -330,11 +333,11 @@ class OpenOp final : public ConnectionOperationBase,
 
   const SDBRequestOpenParams mParams;
   RefPtr<DirectoryLock> mDirectoryLock;
-  nsCOMPtr<nsIFileStream> mFileStream;
+  nsCOMPtr<nsIFileRandomAccessStream> mFileRandomAccessStream;
   // XXX Consider changing this to ClientMetadata.
   quota::OriginMetadata mOriginMetadata;
   State mState;
-  bool mFileStreamOpen;
+  bool mFileRandomAccessStreamOpen;
 
  public:
   OpenOp(Connection* aConnection, const SDBRequestParams& aParams);
@@ -355,7 +358,8 @@ class OpenOp final : public ConnectionOperationBase,
   void StreamClosedCallback();
 
   // ConnectionOperationBase overrides
-  nsresult DoDatabaseWork(nsIFileStream* aFileStream) override;
+  nsresult DoDatabaseWork(
+      nsIFileRandomAccessStream* aFileRandomAccessStream) override;
 
   void GetResponse(SDBRequestResponse& aResponse) override;
 
@@ -383,7 +387,8 @@ class SeekOp final : public ConnectionOperationBase {
  private:
   ~SeekOp() override = default;
 
-  nsresult DoDatabaseWork(nsIFileStream* aFileStream) override;
+  nsresult DoDatabaseWork(
+      nsIFileRandomAccessStream* aFileRandomAccessStream) override;
 
   void GetResponse(SDBRequestResponse& aResponse) override;
 };
@@ -391,7 +396,7 @@ class SeekOp final : public ConnectionOperationBase {
 class ReadOp final : public ConnectionOperationBase {
   const SDBRequestReadParams mParams;
 
-  RefPtr<MemoryOutputStream> mOutputStream;
+  RefPtr<FixedBufferOutputStream> mOutputStream;
 
  public:
   ReadOp(Connection* aConnection, const SDBRequestParams& aParams);
@@ -401,7 +406,8 @@ class ReadOp final : public ConnectionOperationBase {
  private:
   ~ReadOp() override = default;
 
-  nsresult DoDatabaseWork(nsIFileStream* aFileStream) override;
+  nsresult DoDatabaseWork(
+      nsIFileRandomAccessStream* aFileRandomAccessStream) override;
 
   void GetResponse(SDBRequestResponse& aResponse) override;
 };
@@ -421,7 +427,8 @@ class WriteOp final : public ConnectionOperationBase {
  private:
   ~WriteOp() override = default;
 
-  nsresult DoDatabaseWork(nsIFileStream* aFileStream) override;
+  nsresult DoDatabaseWork(
+      nsIFileRandomAccessStream* aFileRandomAccessStream) override;
 
   void GetResponse(SDBRequestResponse& aResponse) override;
 };
@@ -433,7 +440,8 @@ class CloseOp final : public ConnectionOperationBase {
  private:
   ~CloseOp() override = default;
 
-  nsresult DoDatabaseWork(nsIFileStream* aFileStream) override;
+  nsresult DoDatabaseWork(
+      nsIFileRandomAccessStream* aFileRandomAccessStream) override;
 
   void GetResponse(SDBRequestResponse& aResponse) override;
 
@@ -583,18 +591,19 @@ already_AddRefed<mozilla::dom::quota::Client> CreateQuotaClient() {
  * StreamHelper
  ******************************************************************************/
 
-StreamHelper::StreamHelper(nsIFileStream* aFileStream, nsIRunnable* aCallback)
+StreamHelper::StreamHelper(nsIFileRandomAccessStream* aFileRandomAccessStream,
+                           nsIRunnable* aCallback)
     : Runnable("dom::StreamHelper"),
       mOwningEventTarget(GetCurrentEventTarget()),
-      mFileStream(aFileStream),
+      mFileRandomAccessStream(aFileRandomAccessStream),
       mCallback(aCallback) {
   AssertIsOnBackgroundThread();
-  MOZ_ASSERT(aFileStream);
+  MOZ_ASSERT(aFileRandomAccessStream);
   MOZ_ASSERT(aCallback);
 }
 
 StreamHelper::~StreamHelper() {
-  MOZ_ASSERT(!mFileStream);
+  MOZ_ASSERT(!mFileRandomAccessStream);
   MOZ_ASSERT(!mCallback);
 }
 
@@ -611,8 +620,8 @@ void StreamHelper::AsyncClose() {
 void StreamHelper::RunOnBackgroundThread() {
   AssertIsOnBackgroundThread();
 
-  nsCOMPtr<nsIFileStream> fileStream;
-  mFileStream.swap(fileStream);
+  nsCOMPtr<nsIFileRandomAccessStream> fileRandomAccessStream;
+  mFileRandomAccessStream.swap(fileRandomAccessStream);
 
   nsCOMPtr<nsIRunnable> callback;
   mCallback.swap(callback);
@@ -622,9 +631,10 @@ void StreamHelper::RunOnBackgroundThread() {
 
 void StreamHelper::RunOnIOThread() {
   AssertIsOnIOThread();
-  MOZ_ASSERT(mFileStream);
+  MOZ_ASSERT(mFileRandomAccessStream);
 
-  nsCOMPtr<nsIInputStream> inputStream = do_QueryInterface(mFileStream);
+  nsCOMPtr<nsIInputStream> inputStream =
+      do_QueryInterface(mFileRandomAccessStream);
   MOZ_ASSERT(inputStream);
 
   nsresult rv = inputStream->Close();
@@ -684,22 +694,23 @@ void Connection::OnRequestFinished() {
   MaybeCloseStream();
 }
 
-void Connection::OnOpen(const nsACString& aOrigin, const nsAString& aName,
-                        already_AddRefed<DirectoryLock> aDirectoryLock,
-                        already_AddRefed<nsIFileStream> aFileStream) {
+void Connection::OnOpen(
+    const nsACString& aOrigin, const nsAString& aName,
+    already_AddRefed<DirectoryLock> aDirectoryLock,
+    already_AddRefed<nsIFileRandomAccessStream> aFileRandomAccessStream) {
   AssertIsOnBackgroundThread();
   MOZ_ASSERT(!aOrigin.IsEmpty());
   MOZ_ASSERT(!aName.IsEmpty());
   MOZ_ASSERT(mOrigin.IsEmpty());
   MOZ_ASSERT(mName.IsEmpty());
   MOZ_ASSERT(!mDirectoryLock);
-  MOZ_ASSERT(!mFileStream);
+  MOZ_ASSERT(!mFileRandomAccessStream);
   MOZ_ASSERT(!mOpen);
 
   mOrigin = aOrigin;
   mName = aName;
   mDirectoryLock = aDirectoryLock;
-  mFileStream = aFileStream;
+  mFileRandomAccessStream = aFileRandomAccessStream;
   mOpen = true;
 
   if (!gOpenConnections) {
@@ -713,13 +724,13 @@ void Connection::OnClose() {
   AssertIsOnBackgroundThread();
   MOZ_ASSERT(!mOrigin.IsEmpty());
   MOZ_ASSERT(mDirectoryLock);
-  MOZ_ASSERT(mFileStream);
+  MOZ_ASSERT(mFileRandomAccessStream);
   MOZ_ASSERT(mOpen);
 
   mOrigin.Truncate();
   mName.Truncate();
   mDirectoryLock = nullptr;
-  mFileStream = nullptr;
+  mFileRandomAccessStream = nullptr;
   mOpen = false;
 
   MOZ_ASSERT(gOpenConnections);
@@ -757,7 +768,8 @@ void Connection::MaybeCloseStream() {
     nsCOMPtr<nsIRunnable> callback = NewRunnableMethod(
         "dom::Connection::OnClose", this, &Connection::OnClose);
 
-    RefPtr<StreamHelper> helper = new StreamHelper(mFileStream, callback);
+    RefPtr<StreamHelper> helper =
+        new StreamHelper(mFileRandomAccessStream, callback);
     helper->AsyncClose();
   }
 }
@@ -999,10 +1011,11 @@ void ConnectionOperationBase::DatabaseWork() {
     // has crashed.
     mResultCode = NS_ERROR_ABORT;
   } else {
-    nsIFileStream* fileStream = mConnection->GetFileStream();
-    MOZ_ASSERT(fileStream);
+    nsIFileRandomAccessStream* fileRandomAccessStream =
+        mConnection->GetFileRandomAccessStream();
+    MOZ_ASSERT(fileRandomAccessStream);
 
-    nsresult rv = DoDatabaseWork(fileStream);
+    nsresult rv = DoDatabaseWork(fileRandomAccessStream);
     if (NS_FAILED(rv)) {
       mResultCode = rv;
     }
@@ -1035,14 +1048,14 @@ OpenOp::OpenOp(Connection* aConnection, const SDBRequestParams& aParams)
     : ConnectionOperationBase(aConnection),
       mParams(aParams.get_SDBRequestOpenParams()),
       mState(State::Initial),
-      mFileStreamOpen(false) {
+      mFileRandomAccessStreamOpen(false) {
   MOZ_ASSERT(aParams.type() == SDBRequestParams::TSDBRequestOpenParams);
 }
 
 OpenOp::~OpenOp() {
   MOZ_ASSERT(!mDirectoryLock);
-  MOZ_ASSERT(!mFileStream);
-  MOZ_ASSERT(!mFileStreamOpen);
+  MOZ_ASSERT(!mFileRandomAccessStream);
+  MOZ_ASSERT(!mFileRandomAccessStreamOpen);
   MOZ_ASSERT_IF(OperationMayProceed(),
                 mState == State::Initial || mState == State::Completed);
 }
@@ -1136,9 +1149,9 @@ nsresult OpenOp::SendToIOThread() {
     return NS_ERROR_ABORT;
   }
 
-  mFileStream =
-      new FileStream(GetConnection()->GetPersistenceType(), mOriginMetadata,
-                     mozilla::dom::quota::Client::SDB);
+  mFileRandomAccessStream = new FileRandomAccessStream(
+      GetConnection()->GetPersistenceType(), mOriginMetadata,
+      mozilla::dom::quota::Client::SDB);
 
   QuotaManager* quotaManager = QuotaManager::Get();
   MOZ_ASSERT(quotaManager);
@@ -1157,8 +1170,8 @@ nsresult OpenOp::SendToIOThread() {
 nsresult OpenOp::DatabaseWork() {
   AssertIsOnIOThread();
   MOZ_ASSERT(mState == State::DatabaseWorkOpen);
-  MOZ_ASSERT(mFileStream);
-  MOZ_ASSERT(!mFileStreamOpen);
+  MOZ_ASSERT(mFileRandomAccessStream);
+  MOZ_ASSERT(!mFileRandomAccessStreamOpen);
 
   if (NS_WARN_IF(QuotaClient::IsShuttingDownOnNonBackgroundThread()) ||
       !OperationMayProceed()) {
@@ -1230,14 +1243,14 @@ nsresult OpenOp::DatabaseWork() {
     return rv;
   }
 
-  rv = mFileStream->Init(dbFile, PR_RDWR | PR_CREATE_FILE, 0644, 0);
+  rv = mFileRandomAccessStream->Init(dbFile, PR_RDWR | PR_CREATE_FILE, 0644, 0);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
 
-  mFileStreamOpen = true;
+  mFileRandomAccessStreamOpen = true;
 
-  rv = DoDatabaseWork(mFileStream);
+  rv = DoDatabaseWork(mFileRandomAccessStream);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
@@ -1258,15 +1271,16 @@ void OpenOp::StreamClosedCallback() {
   AssertIsOnOwningThread();
   MOZ_ASSERT(NS_FAILED(ResultCode()));
   MOZ_ASSERT(mDirectoryLock);
-  MOZ_ASSERT(mFileStream);
-  MOZ_ASSERT(mFileStreamOpen);
+  MOZ_ASSERT(mFileRandomAccessStream);
+  MOZ_ASSERT(mFileRandomAccessStreamOpen);
 
   mDirectoryLock = nullptr;
-  mFileStream = nullptr;
-  mFileStreamOpen = false;
+  mFileRandomAccessStream = nullptr;
+  mFileRandomAccessStreamOpen = false;
 }
 
-nsresult OpenOp::DoDatabaseWork(nsIFileStream* aFileStream) {
+nsresult OpenOp::DoDatabaseWork(
+    nsIFileRandomAccessStream* aFileRandomAccessStream) {
   AssertIsOnIOThread();
 
   return NS_OK;
@@ -1283,25 +1297,26 @@ void OpenOp::OnSuccess() {
   MOZ_ASSERT(NS_SUCCEEDED(ResultCode()));
   MOZ_ASSERT(!mOriginMetadata.mOrigin.IsEmpty());
   MOZ_ASSERT(mDirectoryLock);
-  MOZ_ASSERT(mFileStream);
-  MOZ_ASSERT(mFileStreamOpen);
+  MOZ_ASSERT(mFileRandomAccessStream);
+  MOZ_ASSERT(mFileRandomAccessStreamOpen);
 
   RefPtr<DirectoryLock> directoryLock;
-  nsCOMPtr<nsIFileStream> fileStream;
+  nsCOMPtr<nsIFileRandomAccessStream> fileRandomAccessStream;
 
   mDirectoryLock.swap(directoryLock);
-  mFileStream.swap(fileStream);
-  mFileStreamOpen = false;
+  mFileRandomAccessStream.swap(fileRandomAccessStream);
+  mFileRandomAccessStreamOpen = false;
 
   GetConnection()->OnOpen(mOriginMetadata.mOrigin, mParams.name(),
-                          directoryLock.forget(), fileStream.forget());
+                          directoryLock.forget(),
+                          fileRandomAccessStream.forget());
 }
 
 void OpenOp::Cleanup() {
   AssertIsOnOwningThread();
-  MOZ_ASSERT_IF(mFileStreamOpen, mFileStream);
+  MOZ_ASSERT_IF(mFileRandomAccessStreamOpen, mFileRandomAccessStream);
 
-  if (mFileStream && mFileStreamOpen) {
+  if (mFileRandomAccessStream && mFileRandomAccessStreamOpen) {
     // If we have an initialized file stream then the operation must have failed
     // and there must be a directory lock too.
     MOZ_ASSERT(NS_FAILED(ResultCode()));
@@ -1313,13 +1328,14 @@ void OpenOp::Cleanup() {
         NewRunnableMethod("dom::OpenOp::StreamClosedCallback", this,
                           &OpenOp::StreamClosedCallback);
 
-    RefPtr<StreamHelper> helper = new StreamHelper(mFileStream, callback);
+    RefPtr<StreamHelper> helper =
+        new StreamHelper(mFileRandomAccessStream, callback);
     helper->AsyncClose();
   } else {
-    MOZ_ASSERT(!mFileStreamOpen);
+    MOZ_ASSERT(!mFileRandomAccessStreamOpen);
 
     mDirectoryLock = nullptr;
-    mFileStream = nullptr;
+    mFileRandomAccessStream = nullptr;
   }
 
   ConnectionOperationBase::Cleanup();
@@ -1411,15 +1427,13 @@ SeekOp::SeekOp(Connection* aConnection, const SDBRequestParams& aParams)
   MOZ_ASSERT(aParams.type() == SDBRequestParams::TSDBRequestSeekParams);
 }
 
-nsresult SeekOp::DoDatabaseWork(nsIFileStream* aFileStream) {
+nsresult SeekOp::DoDatabaseWork(
+    nsIFileRandomAccessStream* aFileRandomAccessStream) {
   AssertIsOnIOThread();
-  MOZ_ASSERT(aFileStream);
+  MOZ_ASSERT(aFileRandomAccessStream);
 
-  nsCOMPtr<nsISeekableStream> seekableStream = do_QueryInterface(aFileStream);
-  MOZ_ASSERT(seekableStream);
-
-  nsresult rv =
-      seekableStream->Seek(nsISeekableStream::NS_SEEK_SET, mParams.offset());
+  nsresult rv = aFileRandomAccessStream->Seek(nsISeekableStream::NS_SEEK_SET,
+                                              mParams.offset());
 
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
@@ -1445,7 +1459,11 @@ bool ReadOp::Init() {
     return false;
   }
 
-  mOutputStream = MemoryOutputStream::Create(mParams.size());
+  if (NS_WARN_IF(mParams.size() > std::numeric_limits<std::size_t>::max())) {
+    return false;
+  }
+
+  mOutputStream = FixedBufferOutputStream::Create(mParams.size(), fallible);
   if (NS_WARN_IF(!mOutputStream)) {
     return false;
   }
@@ -1453,11 +1471,13 @@ bool ReadOp::Init() {
   return true;
 }
 
-nsresult ReadOp::DoDatabaseWork(nsIFileStream* aFileStream) {
+nsresult ReadOp::DoDatabaseWork(
+    nsIFileRandomAccessStream* aFileRandomAccessStream) {
   AssertIsOnIOThread();
-  MOZ_ASSERT(aFileStream);
+  MOZ_ASSERT(aFileRandomAccessStream);
 
-  nsCOMPtr<nsIInputStream> inputStream = do_QueryInterface(aFileStream);
+  nsCOMPtr<nsIInputStream> inputStream =
+      do_QueryInterface(aFileRandomAccessStream);
   MOZ_ASSERT(inputStream);
 
   nsresult rv;
@@ -1508,7 +1528,7 @@ nsresult ReadOp::DoDatabaseWork(nsIFileStream* aFileStream) {
 }
 
 void ReadOp::GetResponse(SDBRequestResponse& aResponse) {
-  aResponse = SDBRequestReadResponse(mOutputStream->Data());
+  aResponse = SDBRequestReadResponse(nsCString(mOutputStream->WrittenData()));
 }
 
 WriteOp::WriteOp(Connection* aConnection, const SDBRequestParams& aParams)
@@ -1539,11 +1559,13 @@ bool WriteOp::Init() {
   return true;
 }
 
-nsresult WriteOp::DoDatabaseWork(nsIFileStream* aFileStream) {
+nsresult WriteOp::DoDatabaseWork(
+    nsIFileRandomAccessStream* aFileRandomAccessStream) {
   AssertIsOnIOThread();
-  MOZ_ASSERT(aFileStream);
+  MOZ_ASSERT(aFileRandomAccessStream);
 
-  nsCOMPtr<nsIOutputStream> outputStream = do_QueryInterface(aFileStream);
+  nsCOMPtr<nsIOutputStream> outputStream =
+      do_QueryInterface(aFileRandomAccessStream);
   MOZ_ASSERT(outputStream);
 
   nsresult rv;
@@ -1584,11 +1606,13 @@ void WriteOp::GetResponse(SDBRequestResponse& aResponse) {
 CloseOp::CloseOp(Connection* aConnection)
     : ConnectionOperationBase(aConnection) {}
 
-nsresult CloseOp::DoDatabaseWork(nsIFileStream* aFileStream) {
+nsresult CloseOp::DoDatabaseWork(
+    nsIFileRandomAccessStream* aFileRandomAccessStream) {
   AssertIsOnIOThread();
-  MOZ_ASSERT(aFileStream);
+  MOZ_ASSERT(aFileRandomAccessStream);
 
-  nsCOMPtr<nsIInputStream> inputStream = do_QueryInterface(aFileStream);
+  nsCOMPtr<nsIInputStream> inputStream =
+      do_QueryInterface(aFileRandomAccessStream);
   MOZ_ASSERT(inputStream);
 
   nsresult rv = inputStream->Close();
