@@ -65,7 +65,7 @@ bool CanTransferArrayBuffer(JSContext* aCx, JS::Handle<JSObject*> aObject,
 }
 
 // https://streams.spec.whatwg.org/#abstract-opdef-cloneasuint8array
-JSObject* CloneAsUint8Array(JSContext* aCx, JS::HandleObject aObject) {
+JSObject* CloneAsUint8Array(JSContext* aCx, JS::Handle<JSObject*> aObject) {
   // Step 1. Assert: Type(O) is Object. Implicit.
   // Step 2. Assert: O has an [[ViewedArrayBuffer]] internal slot.
   MOZ_ASSERT(JS_IsArrayBufferViewObject(aObject));
@@ -110,8 +110,8 @@ class BodyStreamUnderlyingSourceAlgorithms final
                                            UnderlyingSourceAlgorithmsBase)
 
   explicit BodyStreamUnderlyingSourceAlgorithms(
-      BodyStreamHolder* underlyingSource)
-      : mUnderlyingSource(underlyingSource) {}
+      nsIGlobalObject& aGlobal, BodyStreamHolder& aUnderlyingSource)
+      : mGlobal(&aGlobal), mUnderlyingSource(&aUnderlyingSource) {}
 
   MOZ_CAN_RUN_SCRIPT void StartCallback(JSContext* aCx,
                                         ReadableStreamController& aController,
@@ -130,24 +130,30 @@ class BodyStreamUnderlyingSourceAlgorithms final
   MOZ_CAN_RUN_SCRIPT already_AddRefed<Promise> CancelCallback(
       JSContext* aCx, const Optional<JS::Handle<JS::Value>>& aReason,
       ErrorResult& aRv) override {
-    RefPtr<BodyStream> bodyStream = mUnderlyingSource->GetBodyStream();
-    return bodyStream->CancelCallback(aCx, aReason, aRv);
+    return Promise::CreateResolvedWithUndefined(mGlobal, aRv);
   }
 
-  void ErrorCallback() override {
-    RefPtr<BodyStream> bodyStream = mUnderlyingSource->GetBodyStream();
-    bodyStream->ErrorCallback();
+  void ReleaseObjects() override {
+    RefPtr<BodyStreamHolder> holder = mUnderlyingSource.forget();
+    // BodyStream may not be available if this cleanup happened first from
+    // ByteStream side.
+    if (RefPtr<BodyStream> bodyStream = holder->GetBodyStream()) {
+      bodyStream->CloseInputAndReleaseObjects();
+    }
   }
 
+  BodyStreamHolder* GetBodyStreamHolder() override { return mUnderlyingSource; }
+
  protected:
   ~BodyStreamUnderlyingSourceAlgorithms() override = default;
 
  private:
+  nsCOMPtr<nsIGlobalObject> mGlobal;
   RefPtr<BodyStreamHolder> mUnderlyingSource;
 };
 
 NS_IMPL_CYCLE_COLLECTION_INHERITED(BodyStreamUnderlyingSourceAlgorithms,
-                                   UnderlyingSourceAlgorithmsBase,
+                                   UnderlyingSourceAlgorithmsBase, mGlobal,
                                    mUnderlyingSource)
 NS_IMPL_ADDREF_INHERITED(BodyStreamUnderlyingSourceAlgorithms,
                          UnderlyingSourceAlgorithmsBase)
@@ -170,8 +176,8 @@ void SetUpReadableByteStreamControllerFromBodyStreamUnderlyingSource(
       MakeRefPtr<ReadableByteStreamController>(aStream->GetParentObject());
 
   // Step 2 - 7.
-  auto algorithms =
-      MakeRefPtr<BodyStreamUnderlyingSourceAlgorithms>(aUnderlyingSource);
+  auto algorithms = MakeRefPtr<BodyStreamUnderlyingSourceAlgorithms>(
+      *aStream->GetParentObject(), *aUnderlyingSource);
 
   // Step 8
   Maybe<uint64_t> autoAllocateChunkSize = mozilla::Nothing();
