@@ -112,11 +112,6 @@ class ReadableByteStreamController final : public ReadableStreamController,
     mCloseRequested = aCloseRequested;
   }
 
-  UnderlyingSourceAlgorithmsBase* GetAlgorithms() { return mAlgorithms; }
-  void SetAlgorithms(UnderlyingSourceAlgorithmsBase* aAlgorithms) {
-    mAlgorithms = aAlgorithms;
-  }
-
   LinkedList<RefPtr<ReadableByteStreamQueueEntry>>& Queue() { return mQueue; }
   void ClearQueue();
 
@@ -158,9 +153,6 @@ class ReadableByteStreamController final : public ReadableStreamController,
   // request, or null if there are no pending requests
   RefPtr<ReadableStreamBYOBRequest> mByobRequest;
 
-  // The algorithms for the underlying byte source
-  RefPtr<UnderlyingSourceAlgorithmsBase> mAlgorithms;
-
   // A list of pull-into descriptors
   LinkedList<RefPtr<PullIntoDescriptor>> mPendingPullIntos;
 
@@ -336,16 +328,16 @@ MOZ_CAN_RUN_SCRIPT void ReadableByteStreamControllerRespondWithNewView(
 
 MOZ_CAN_RUN_SCRIPT void ReadableByteStreamControllerPullInto(
     JSContext* aCx, ReadableByteStreamController* aController,
-    JS::HandleObject aView, ReadIntoRequest* aReadIntoRequest,
+    JS::Handle<JSObject*> aView, ReadIntoRequest* aReadIntoRequest,
     ErrorResult& aRv);
 
 void ReadableByteStreamControllerError(
-    ReadableByteStreamController* aController, JS::HandleValue aValue,
+    ReadableByteStreamController* aController, JS::Handle<JS::Value> aValue,
     ErrorResult& aRv);
 
 MOZ_CAN_RUN_SCRIPT void ReadableByteStreamControllerEnqueue(
     JSContext* aCx, ReadableByteStreamController* aController,
-    JS::HandleObject aChunk, ErrorResult& aRv);
+    JS::Handle<JSObject*> aChunk, ErrorResult& aRv);
 
 already_AddRefed<ReadableStreamBYOBRequest>
 ReadableByteStreamControllerGetBYOBRequest(
@@ -367,13 +359,11 @@ MOZ_CAN_RUN_SCRIPT void ReadableByteStreamControllerCallPullIfNeeded(
     ErrorResult& aRv);
 
 MOZ_CAN_RUN_SCRIPT void SetUpReadableByteStreamControllerFromUnderlyingSource(
-    JSContext* aCx, ReadableStream* aStream, JS::HandleObject aUnderlyingSource,
+    JSContext* aCx, ReadableStream* aStream,
+    JS::Handle<JSObject*> aUnderlyingSource,
     UnderlyingSource& aUnderlyingSourceDict, double aHighWaterMark,
     ErrorResult& aRv);
 
-void ReadableByteStreamControllerClearAlgorithms(
-    ReadableByteStreamController* aController);
-
 }  // namespace mozilla::dom
 
 #endif
