@@ -12,6 +12,7 @@
 #include "mozilla/Attributes.h"
 #include "mozilla/ErrorResult.h"
 #include "mozilla/dom/BindingDeclarations.h"
+#include "mozilla/dom/IterableIterator.h"
 #include "mozilla/dom/QueuingStrategyBinding.h"
 #include "mozilla/dom/ReadableStreamController.h"
 #include "mozilla/dom/ReadableStreamDefaultController.h"
@@ -26,6 +27,7 @@ class ReadableStreamGenericReader;
 class ReadableStreamDefaultReader;
 class ReadableStreamGenericReader;
 struct ReadableStreamGetReaderOptions;
+struct ReadableStreamIteratorOptions;
 struct ReadIntoRequest;
 class WritableStream;
 struct ReadableWritablePair;
@@ -37,14 +39,16 @@ using OwningReadableStreamReader =
     OwningReadableStreamDefaultReaderOrReadableStreamBYOBReader;
 class NativeUnderlyingSource;
 class BodyStreamHolder;
+class UniqueMessagePortId;
+class MessagePort;
 
-class ReadableStream final : public nsISupports, public nsWrapperCache {
+class ReadableStream : public nsISupports, public nsWrapperCache {
  public:
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
   NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS(ReadableStream)
 
  protected:
-  ~ReadableStream();
+  virtual ~ReadableStream();
 
   nsCOMPtr<nsIGlobalObject> mGlobal;
 
@@ -78,24 +82,42 @@ class ReadableStream final : public nsISupports, public nsWrapperCache {
   void SetState(const ReaderState& aState) { mState = aState; }
 
   JS::Value StoredError() const { return mStoredError; }
-  void SetStoredError(JS::HandleValue aStoredError) {
+  void SetStoredError(JS::Handle<JS::Value> aStoredError) {
     mStoredError = aStoredError;
   }
 
-  UnderlyingSourceAlgorithmsBase* GetAlgorithms() const { return mAlgorithms; }
-  void SetErrorAlgorithm(UnderlyingSourceAlgorithmsBase* aAlgorithms) {
-    mAlgorithms = aAlgorithms;
+  BodyStreamHolder* GetBodyStreamHolder() {
+    if (UnderlyingSourceAlgorithmsBase* algorithms =
+            Controller()->GetAlgorithms()) {
+      return algorithms->GetBodyStreamHolder();
+    }
+    return nullptr;
   }
 
-  void SetNativeUnderlyingSource(BodyStreamHolder* aUnderlyingSource);
-  BodyStreamHolder* GetNativeUnderlyingSource() {
-    return mNativeUnderlyingSource;
-  }
-  bool HasNativeUnderlyingSource() { return mNativeUnderlyingSource; }
+  // [Transferable]
+  // https://html.spec.whatwg.org/multipage/structured-data.html#transfer-steps
+  MOZ_CAN_RUN_SCRIPT bool Transfer(JSContext* aCx,
+                                   UniqueMessagePortId& aPortId);
+  // https://html.spec.whatwg.org/multipage/structured-data.html#transfer-receiving-steps
+  static MOZ_CAN_RUN_SCRIPT bool ReceiveTransfer(
+      JSContext* aCx, nsIGlobalObject* aGlobal, MessagePort& aPort,
+      JS::MutableHandle<JSObject*> aReturnObject);
 
-  void ReleaseObjects();
+  // Public functions to implement other specs
+
+  // https://streams.spec.whatwg.org/#other-specs-rs-create
+  // The following algorithms must only be used on ReadableStream instances
+  // initialized via the above set up or set up with byte reading support
+  // algorithms (not, e.g., on web-developer-created instances):
+
+  MOZ_CAN_RUN_SCRIPT void CloseNative(JSContext* aCx, ErrorResult& aRv);
+
+  MOZ_CAN_RUN_SCRIPT void EnqueueNative(JSContext* aCx,
+                                        JS::Handle<JS::Value> aChunk,
+                                        ErrorResult& aRv);
+
+  // IDL layer functions
 
- public:
   nsIGlobalObject* GetParentObject() const { return mGlobal; }
 
   JSObject* WrapObject(JSContext* aCx,
@@ -132,31 +154,32 @@ class ReadableStream final : public nsISupports, public nsWrapperCache {
                               nsTArray<RefPtr<ReadableStream>>& aResult,
                               ErrorResult& aRv);
 
+  struct IteratorData {
+    void Traverse(nsCycleCollectionTraversalCallback& cb);
+    void Unlink();
+
+    RefPtr<ReadableStreamDefaultReader> mReader;
+    bool mPreventCancel;
+  };
+
+  using Iterator = AsyncIterableIterator<ReadableStream>;
+
+  void InitAsyncIteratorData(IteratorData& aData, Iterator::IteratorType aType,
+                             const ReadableStreamIteratorOptions& aOptions,
+                             ErrorResult& aRv);
+  MOZ_CAN_RUN_SCRIPT already_AddRefed<Promise> GetNextIterationResult(
+      Iterator* aIterator, ErrorResult& aRv);
+  MOZ_CAN_RUN_SCRIPT already_AddRefed<Promise> IteratorReturn(
+      JSContext* aCx, Iterator* aIterator, JS::Handle<JS::Value> aValue,
+      ErrorResult& aRv);
+
   // Internal Slots:
  private:
-  MOZ_KNOWN_LIVE RefPtr<ReadableStreamController> mController;
+  RefPtr<ReadableStreamController> mController;
   bool mDisturbed = false;
   RefPtr<ReadableStreamGenericReader> mReader;
   ReaderState mState = ReaderState::Readable;
   JS::Heap<JS::Value> mStoredError;
-
-  // Optional Callback for erroring a stream.
-  RefPtr<UnderlyingSourceAlgorithmsBase> mAlgorithms;
-
-  // Optional strong reference to an Underlying Source; This
-  // exists because NativeUnderlyingSource callbacks don't hold
-  // a strong reference to the underlying source: So we need
-  // something else to hold onto that. As well, some of the integration
-  // desires the ability to extract the underlying source from the
-  // ReadableStream.
-  //
-  // While theoretically this ought to be some base class type to support
-  // multiple native underlying source types, I'm not sure what base class
-  // makes any sense for BodyStream, and given there's only body streams
-  // as the underlying source right now, I'm going to punt that problem to
-  // the future where we need to provide other native underlying sources
-  // (i.e. perhaps WebTransport.)
-  RefPtr<BodyStreamHolder> mNativeUnderlyingSource;
 };
 
 bool IsReadableStreamLocked(ReadableStream* aStream);
