@@ -12,7 +12,6 @@
 #include "mozilla/HoldDropJSObjects.h"
 #include "mozilla/dom/Promise.h"
 #include "mozilla/dom/Promise-inl.h"
-#include "mozilla/dom/PromiseNativeHandler.h"
 #include "mozilla/dom/ReadableStream.h"
 #include "mozilla/dom/ReadableStreamController.h"
 #include "mozilla/dom/ReadableStreamDefaultController.h"
@@ -25,7 +24,7 @@
 
 namespace mozilla::dom {
 
-NS_IMPL_CYCLE_COLLECTION(ReadableStreamController, mGlobal)
+NS_IMPL_CYCLE_COLLECTION(ReadableStreamController, mGlobal, mAlgorithms)
 NS_IMPL_CYCLE_COLLECTING_ADDREF(ReadableStreamController)
 NS_IMPL_CYCLE_COLLECTING_RELEASE(ReadableStreamController)
 
@@ -39,15 +38,14 @@ NS_INTERFACE_MAP_END
 NS_IMPL_CYCLE_COLLECTION_CLASS(ReadableStreamDefaultController)
 
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(ReadableStreamDefaultController)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK(mAlgorithms, mStrategySizeAlgorithm, mStream)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mStrategySizeAlgorithm, mStream)
   tmp->mQueue.clear();
   NS_IMPL_CYCLE_COLLECTION_UNLINK_PRESERVED_WRAPPER
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN_INHERITED(
     ReadableStreamDefaultController, ReadableStreamController)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mAlgorithms, mStrategySizeAlgorithm,
-                                    mStream)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mStrategySizeAlgorithm, mStream)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 NS_IMPL_CYCLE_COLLECTION_TRACE_BEGIN_INHERITED(ReadableStreamDefaultController,
@@ -187,7 +185,7 @@ void ReadableStreamDefaultControllerClearAlgorithms(
     ReadableStreamDefaultController* aController) {
   // Step 1.
   // Step 2.
-  aController->SetAlgorithms(nullptr);
+  aController->ClearAlgorithms();
 
   // Step 3.
   aController->setStrategySizeAlgorithm(nullptr);
@@ -276,7 +274,7 @@ void ReadableStreamDefaultControllerEnqueue(
     // Step 4.2:
     if (aRv.MaybeSetPendingException(
             aCx, "ReadableStreamDefaultController.enqueue")) {
-      JS::RootedValue errorValue(aCx);
+      JS::Rooted<JS::Value> errorValue(aCx);
 
       JS_GetPendingException(aCx, &errorValue);
 
@@ -303,7 +301,7 @@ void ReadableStreamDefaultControllerEnqueue(
     // and there are tests to ensure the identity of these errors are the same.
     if (aRv.MaybeSetPendingException(
             aCx, "ReadableStreamDefaultController.enqueue")) {
-      JS::RootedValue errorValue(aCx);
+      JS::Rooted<JS::Value> errorValue(aCx);
 
       JS_GetPendingException(aCx, &errorValue);
 
@@ -488,14 +486,14 @@ void SetUpReadableStreamDefaultController(
 
   // Step 6.
   // Step 7.
-  aController->SetAlgorithms(aAlgorithms);
+  aController->SetAlgorithms(*aAlgorithms);
 
   // Step 8.
   aStream->SetController(*aController);
 
   // Step 9. Default algorithm returns undefined. See Step 2 of
   // https://streams.spec.whatwg.org/#set-up-readable-stream-default-controller
-  JS::RootedValue startResult(aCx, JS::UndefinedValue());
+  JS::Rooted<JS::Value> startResult(aCx, JS::UndefinedValue());
   RefPtr<ReadableStreamDefaultController> controller = aController;
   aAlgorithms->StartCallback(aCx, *controller, &startResult, aRv);
   if (aRv.Failed()) {
@@ -503,7 +501,8 @@ void SetUpReadableStreamDefaultController(
   }
 
   // Step 10.
-  RefPtr<Promise> startPromise = Promise::Create(GetIncumbentGlobal(), aRv);
+  RefPtr<Promise> startPromise =
+      Promise::Create(aStream->GetParentObject(), aRv);
   if (aRv.Failed()) {
     return;
   }
@@ -540,7 +539,8 @@ void SetUpReadableStreamDefaultController(
 
 // https://streams.spec.whatwg.org/#set-up-readable-stream-default-controller-from-underlying-source
 void SetupReadableStreamDefaultControllerFromUnderlyingSource(
-    JSContext* aCx, ReadableStream* aStream, JS::HandleObject aUnderlyingSource,
+    JSContext* aCx, ReadableStream* aStream,
+    JS::Handle<JSObject*> aUnderlyingSource,
     UnderlyingSource& aUnderlyingSourceDict, double aHighWaterMark,
     QueuingStrategySize* aSizeAlgorithm, ErrorResult& aRv) {
   // Step 1.
@@ -588,7 +588,7 @@ void ReadableStreamDefaultController::PullSteps(JSContext* aCx,
   // Step 2.
   if (!mQueue.isEmpty()) {
     // Step 2.1
-    JS::RootedValue chunk(aCx);
+    JS::Rooted<JS::Value> chunk(aCx);
     DequeueValue(this, &chunk);
 
     // Step 2.2
