@@ -5,6 +5,7 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "mozilla/dom/ReadableStreamDefaultReader.h"
+#include "mozilla/dom/AutoEntryScript.h"
 #include "mozilla/dom/ReadableStream.h"
 #include "mozilla/dom/RootedDictionary.h"
 #include "js/PropertyAndElement.h"
@@ -88,7 +89,7 @@ bool ReadableStreamReaderGenericInitialize(ReadableStreamGenericReader* aReader,
       // Step 5.1 Implicit
       // Step 5.2
       JS::RootingContext* rcx = RootingCx();
-      JS::RootedValue rootedError(rcx, aStream->StoredError());
+      JS::Rooted<JS::Value> rootedError(rcx, aStream->StoredError());
       aReader->ClosedPromise()->MaybeReject(rootedError);
 
       // Step 5.3
@@ -221,7 +222,7 @@ void ReadableStreamDefaultReaderRead(JSContext* aCx,
     }
 
     case ReadableStream::ReaderState::Errored: {
-      JS::RootedValue storedError(aCx, stream->StoredError());
+      JS::Rooted<JS::Value> storedError(aCx, stream->StoredError());
       aRequest->ErrorSteps(aCx, storedError, aRv);
       return;
     }
@@ -238,8 +239,7 @@ void ReadableStreamDefaultReaderRead(JSContext* aCx,
 // Return a raw pointer here to avoid refcounting, but make sure it's safe
 // (the object should be kept alive by the callee).
 // https://streams.spec.whatwg.org/#default-reader-read
-already_AddRefed<Promise> ReadableStreamDefaultReader::Read(JSContext* aCx,
-                                                            ErrorResult& aRv) {
+already_AddRefed<Promise> ReadableStreamDefaultReader::Read(ErrorResult& aRv) {
   // Step 1.
   if (!mStream) {
     aRv.ThrowTypeError("Reading is not possible after calling releaseLock.");
@@ -253,7 +253,10 @@ already_AddRefed<Promise> ReadableStreamDefaultReader::Read(JSContext* aCx,
   RefPtr<ReadRequest> request = new Read_ReadRequest(promise);
 
   // Step 4.
-  ReadableStreamDefaultReaderRead(aCx, this, request, aRv);
+  AutoEntryScript aes(mGlobal, "ReadableStreamDefaultReader::Read");
+  JSContext* cx = aes.cx();
+
+  ReadableStreamDefaultReaderRead(cx, this, request, aRv);
   if (aRv.Failed()) {
     return nullptr;
   }
@@ -282,11 +285,8 @@ void ReadableStreamReaderGenericRelease(ReadableStreamGenericReader* aReader,
   } else {
     // Step 5. Otherwise, set reader.[[closedPromise]] to a promise rejected
     // with a TypeError exception.
-    RefPtr<Promise> promise = Promise::Create(aReader->GetParentObject(), aRv);
-    if (aRv.Failed()) {
-      return;
-    }
-    promise->MaybeRejectWithTypeError("Lock Released");
+    RefPtr<Promise> promise = Promise::CreateRejectedWithTypeError(
+        aReader->GetParentObject(), "Lock Released"_ns, aRv);
     aReader->SetClosedPromise(promise.forget());
   }
 
