@@ -4,11 +4,12 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "ReadableStreamPipeTo.h"
+
 #include "mozilla/dom/AbortFollower.h"
 #include "mozilla/dom/AbortSignal.h"
 #include "mozilla/dom/ReadableStream.h"
 #include "mozilla/dom/ReadableStreamDefaultReader.h"
-#include "mozilla/dom/ReadableStreamPipeTo.h"
 #include "mozilla/dom/WritableStream.h"
 #include "mozilla/dom/WritableStreamDefaultWriter.h"
 #include "mozilla/dom/Promise.h"
@@ -26,9 +27,6 @@ struct PipeToReadRequest;
 class WriteFinishedPromiseHandler;
 class ShutdownActionFinishedPromiseHandler;
 
-// TODO: Bug 1756794
-using ::ImplCycleCollectionUnlink;
-
 // https://streams.spec.whatwg.org/#readable-stream-pipe-to (Steps 14-15.)
 //
 // This class implements everything that is required to read all chunks from
@@ -649,16 +647,7 @@ struct PipeToReadRequest : public ReadRequest {
   virtual ~PipeToReadRequest() = default;
 };
 
-NS_IMPL_CYCLE_COLLECTION_CLASS(PipeToReadRequest)
-
-NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN_INHERITED(PipeToReadRequest, ReadRequest)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK(mPipeToPump)
-NS_IMPL_CYCLE_COLLECTION_UNLINK_END
-
-NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN_INHERITED(PipeToReadRequest,
-                                                  ReadRequest)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mPipeToPump)
-NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
+NS_IMPL_CYCLE_COLLECTION_INHERITED(PipeToReadRequest, ReadRequest, mPipeToPump)
 
 NS_IMPL_ADDREF_INHERITED(PipeToReadRequest, ReadRequest)
 NS_IMPL_RELEASE_INHERITED(PipeToReadRequest, ReadRequest)
