@@ -19,19 +19,28 @@ namespace mozilla::dom {
 
 class WritableStream;
 class ReadableStream;
+class UniqueMessagePortId;
+class MessagePort;
+class TransformerAlgorithmsWrapper;
 
 class TransformStream final : public nsISupports, public nsWrapperCache {
  public:
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
-  NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS(TransformStream)
+  NS_DECL_CYCLE_COLLECTION_WRAPPERCACHE_CLASS(TransformStream)
+
+  // https://streams.spec.whatwg.org/#transformstream-set-up
+  // Intended to be used by interfaces using GenericTransformStream mixin.
+  MOZ_CAN_RUN_SCRIPT static already_AddRefed<TransformStream> CreateGeneric(
+      const GlobalObject& aGlobal, TransformerAlgorithmsWrapper& aAlgorithms,
+      ErrorResult& aRv);
+
+  TransformStream(nsIGlobalObject* aGlobal, ReadableStream* aReadable,
+                  WritableStream* aWritable);
 
   // Internal slot accessors
   bool Backpressure() const { return mBackpressure; }
-  void SetBackpressure(bool aBackpressure) { mBackpressure = aBackpressure; }
   Promise* BackpressureChangePromise() { return mBackpressureChangePromise; }
-  void SetBackpressureChangePromise(Promise* aPromise) {
-    mBackpressureChangePromise = aPromise;
-  }
+  void SetBackpressure(bool aBackpressure, ErrorResult& aRv);
   MOZ_KNOWN_LIVE TransformStreamDefaultController* Controller() {
     return mController;
   }
@@ -39,8 +48,16 @@ class TransformStream final : public nsISupports, public nsWrapperCache {
     MOZ_ASSERT(!mController);
     mController = &aController;
   }
-  MOZ_KNOWN_LIVE ReadableStream* Readable() { return mReadable; }
-  MOZ_KNOWN_LIVE WritableStream* Writable() { return mWritable; }
+
+  // [Transferable]
+  // https://html.spec.whatwg.org/multipage/structured-data.html#transfer-steps
+  MOZ_CAN_RUN_SCRIPT bool Transfer(JSContext* aCx,
+                                   UniqueMessagePortId& aPortId1,
+                                   UniqueMessagePortId& aPortId2);
+  // https://html.spec.whatwg.org/multipage/structured-data.html#transfer-receiving-steps
+  static MOZ_CAN_RUN_SCRIPT bool ReceiveTransfer(
+      JSContext* aCx, nsIGlobalObject* aGlobal, MessagePort& aPort1,
+      MessagePort& aPort2, JS::MutableHandle<JSObject*> aReturnObject);
 
  protected:
   ~TransformStream();
@@ -65,8 +82,8 @@ class TransformStream final : public nsISupports, public nsWrapperCache {
               const QueuingStrategy& aWritableStrategy,
               const QueuingStrategy& aReadableStrategy, ErrorResult& aRv);
 
-  already_AddRefed<ReadableStream> GetReadable();
-  already_AddRefed<WritableStream> GetWritable();
+  ReadableStream* Readable() const { return mReadable; }
+  WritableStream* Writable() const { return mWritable; }
 
  private:
   nsCOMPtr<nsIGlobalObject> mGlobal;
@@ -75,23 +92,20 @@ class TransformStream final : public nsISupports, public nsWrapperCache {
   // MOZ_KNOWN_LIVE for slots that will never be reassigned
   bool mBackpressure = false;
   RefPtr<Promise> mBackpressureChangePromise;
-  MOZ_KNOWN_LIVE RefPtr<TransformStreamDefaultController> mController;
-  MOZ_KNOWN_LIVE RefPtr<ReadableStream> mReadable;
-  MOZ_KNOWN_LIVE RefPtr<WritableStream> mWritable;
+  RefPtr<TransformStreamDefaultController> mController;
+  RefPtr<ReadableStream> mReadable;
+  RefPtr<WritableStream> mWritable;
 };
 
 MOZ_CAN_RUN_SCRIPT void TransformStreamErrorWritableAndUnblockWrite(
-    JSContext* aCx, TransformStream* aStream, JS::HandleValue aError,
+    JSContext* aCx, TransformStream* aStream, JS::Handle<JS::Value> aError,
     ErrorResult& aRv);
 
 MOZ_CAN_RUN_SCRIPT void TransformStreamError(JSContext* aCx,
                                              TransformStream* aStream,
-                                             JS::HandleValue aError,
+                                             JS::Handle<JS::Value> aError,
                                              ErrorResult& aRv);
 
-void TransformStreamSetBackpressure(TransformStream* aStream,
-                                    bool aBackpressure, ErrorResult& aRv);
-
 }  // namespace mozilla::dom
 
 #endif  // DOM_STREAMS_TRANSFORMSTREAM_H_
