@@ -19,18 +19,18 @@
 namespace mozilla::dom {
 
 class TransformStream;
-class TransformerAlgorithms;
+class TransformerAlgorithmsBase;
 
 class TransformStreamDefaultController final : public nsISupports,
                                                public nsWrapperCache {
  public:
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
-  NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS(TransformStreamDefaultController)
+  NS_DECL_CYCLE_COLLECTION_WRAPPERCACHE_CLASS(TransformStreamDefaultController)
 
   MOZ_KNOWN_LIVE TransformStream* Stream();
   void SetStream(TransformStream& aStream);
-  TransformerAlgorithms* Algorithms();
-  void SetAlgorithms(TransformerAlgorithms* aTransformerAlgorithms);
+  TransformerAlgorithmsBase* Algorithms();
+  void SetAlgorithms(TransformerAlgorithmsBase* aTransformerAlgorithms);
 
   explicit TransformStreamDefaultController(nsIGlobalObject* aGlobal);
 
@@ -54,13 +54,18 @@ class TransformStreamDefaultController final : public nsISupports,
   nsCOMPtr<nsIGlobalObject> mGlobal;
 
   // Internal slots
-  MOZ_KNOWN_LIVE RefPtr<TransformStream> mStream;
-  RefPtr<TransformerAlgorithms> mTransformerAlgorithms;
+  RefPtr<TransformStream> mStream;
+  RefPtr<TransformerAlgorithmsBase> mTransformerAlgorithms;
 };
 
+void SetUpTransformStreamDefaultController(
+    JSContext* aCx, TransformStream& aStream,
+    TransformStreamDefaultController& aController,
+    TransformerAlgorithmsBase& aTransformerAlgorithms);
+
 void SetUpTransformStreamDefaultControllerFromTransformer(
-    JSContext* aCx, TransformStream& aStream, JS::HandleObject aTransformer,
-    Transformer& aTransformerDict);
+    JSContext* aCx, TransformStream& aStream,
+    JS::Handle<JSObject*> aTransformer, Transformer& aTransformerDict);
 
 }  // namespace mozilla::dom
 
