@@ -26,7 +26,7 @@ class WritableStreamDefaultController;
 class UnderlyingSinkAlgorithmsBase : public nsISupports {
  public:
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
-  NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS(UnderlyingSinkAlgorithmsBase)
+  NS_DECL_CYCLE_COLLECTION_CLASS(UnderlyingSinkAlgorithmsBase)
 
   MOZ_CAN_RUN_SCRIPT virtual void StartCallback(
       JSContext* aCx, WritableStreamDefaultController& aController,
@@ -43,6 +43,10 @@ class UnderlyingSinkAlgorithmsBase : public nsISupports {
       JSContext* aCx, const Optional<JS::Handle<JS::Value>>& aReason,
       ErrorResult& aRv) = 0;
 
+  // Implement this when you need to release underlying resources immediately
+  // from closed/errored(aborted) streams, without waiting for GC.
+  virtual void ReleaseObjects() {}
+
  protected:
   virtual ~UnderlyingSinkAlgorithmsBase() = default;
 };
@@ -55,7 +59,7 @@ class UnderlyingSinkAlgorithms final : public UnderlyingSinkAlgorithmsBase {
       UnderlyingSinkAlgorithms, UnderlyingSinkAlgorithmsBase)
 
   UnderlyingSinkAlgorithms(nsIGlobalObject* aGlobal,
-                           JS::HandleObject aUnderlyingSink,
+                           JS::Handle<JSObject*> aUnderlyingSink,
                            UnderlyingSink& aUnderlyingSinkDict)
       : mGlobal(aGlobal), mUnderlyingSink(aUnderlyingSink) {
     // Step 6. (implicit Step 2.)
