@@ -4,14 +4,14 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "nsSVGElement.h"
 #include "DOMSVGPointList.h"
+
+#include "nsContentUtils.h"
 #include "DOMSVGPoint.h"
 #include "nsError.h"
 #include "SVGAnimatedPointList.h"
-#include "nsCOMPtr.h"
-#include "nsSVGAttrTearoffTable.h"
-#include "nsContentUtils.h"
+#include "SVGAttrTearoffTable.h"
+#include "mozilla/dom/SVGElement.h"
 #include "mozilla/dom/SVGPointListBinding.h"
 #include <algorithm>
 
@@ -20,10 +20,9 @@
 // local helper functions
 namespace {
 
-void
-UpdateListIndicesFromIndex(FallibleTArray<mozilla::nsISVGPoint*>& aItemsArray,
-                           uint32_t aStartingIndex)
-{
+void UpdateListIndicesFromIndex(
+    FallibleTArray<mozilla::dom::DOMSVGPoint*>& aItemsArray,
+    uint32_t aStartingIndex) {
   uint32_t length = aItemsArray.Length();
 
   for (uint32_t i = aStartingIndex; i < length; ++i) {
@@ -33,16 +32,13 @@ UpdateListIndicesFromIndex(FallibleTArray<mozilla::nsISVGPoint*>& aItemsArray,
   }
 }
 
-} // namespace
+}  // namespace
 
-namespace mozilla {
+namespace mozilla::dom {
 
-  static inline
-nsSVGAttrTearoffTable<void, DOMSVGPointList>&
-SVGPointListTearoffTable()
-{
-  static nsSVGAttrTearoffTable<void, DOMSVGPointList>
-    sSVGPointListTearoffTable;
+static inline SVGAttrTearoffTable<void, DOMSVGPointList>&
+SVGPointListTearoffTable() {
+  static SVGAttrTearoffTable<void, DOMSVGPointList> sSVGPointListTearoffTable;
   return sSVGPointListTearoffTable;
 }
 
@@ -51,11 +47,11 @@ NS_IMPL_CYCLE_COLLECTION_CLASS(DOMSVGPointList)
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(DOMSVGPointList)
   // No unlinking of mElement, we'd need to null out the value pointer (the
   // object it points to is held by the element) and null-check it everywhere.
+  tmp->RemoveFromTearoffTable();
   NS_IMPL_CYCLE_COLLECTION_UNLINK_PRESERVED_WRAPPER
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(DOMSVGPointList)
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mElement)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_SCRIPT_OBJECTS
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 NS_IMPL_CYCLE_COLLECTION_TRACE_BEGIN(DOMSVGPointList)
   NS_IMPL_CYCLE_COLLECTION_TRACE_PRESERVED_WRAPPER
@@ -65,48 +61,16 @@ NS_IMPL_CYCLE_COLLECTING_ADDREF(DOMSVGPointList)
 NS_IMPL_CYCLE_COLLECTING_RELEASE(DOMSVGPointList)
 
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(DOMSVGPointList)
+  NS_INTERFACE_MAP_ENTRY_CONCRETE(DOMSVGPointList)
   NS_WRAPPERCACHE_INTERFACE_MAP_ENTRY
   NS_INTERFACE_MAP_ENTRY(nsISupports)
 NS_INTERFACE_MAP_END
 
-//----------------------------------------------------------------------
-// Helper class: AutoChangePointListNotifier
-// Stack-based helper class to pair calls to WillChangePointList and
-// DidChangePointList.
-class MOZ_RAII AutoChangePointListNotifier
-{
-public:
-  explicit AutoChangePointListNotifier(DOMSVGPointList* aPointList MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
-    : mPointList(aPointList)
-  {
-    MOZ_GUARD_OBJECT_NOTIFIER_INIT;
-    MOZ_ASSERT(mPointList, "Expecting non-null pointList");
-    mEmptyOrOldValue =
-      mPointList->Element()->WillChangePointList();
-  }
-
-  ~AutoChangePointListNotifier()
-  {
-    mPointList->Element()->DidChangePointList(mEmptyOrOldValue);
-    if (mPointList->AttrIsAnimating()) {
-      mPointList->Element()->AnimationNeedsResample();
-    }
-  }
-
-private:
-  DOMSVGPointList* const mPointList;
-  nsAttrValue      mEmptyOrOldValue;
-  MOZ_DECL_USE_GUARD_OBJECT_NOTIFIER
-};
-
-
-/* static */ already_AddRefed<DOMSVGPointList>
-DOMSVGPointList::GetDOMWrapper(void *aList,
-                               nsSVGElement *aElement,
-                               bool aIsAnimValList)
-{
+/* static */
+already_AddRefed<DOMSVGPointList> DOMSVGPointList::GetDOMWrapper(
+    void* aList, SVGElement* aElement, bool aIsAnimValList) {
   RefPtr<DOMSVGPointList> wrapper =
-    SVGPointListTearoffTable().GetTearoff(aList);
+      SVGPointListTearoffTable().GetTearoff(aList);
   if (!wrapper) {
     wrapper = new DOMSVGPointList(aElement, aIsAnimValList);
     SVGPointListTearoffTable().AddTearoff(aList, wrapper);
@@ -114,31 +78,29 @@ DOMSVGPointList::GetDOMWrapper(void *aList,
   return wrapper.forget();
 }
 
-/* static */ DOMSVGPointList*
-DOMSVGPointList::GetDOMWrapperIfExists(void *aList)
-{
+/* static */
+DOMSVGPointList* DOMSVGPointList::GetDOMWrapperIfExists(void* aList) {
   return SVGPointListTearoffTable().GetTearoff(aList);
 }
 
-DOMSVGPointList::~DOMSVGPointList()
-{
+void DOMSVGPointList::RemoveFromTearoffTable() {
+  // Called from Unlink and the destructor.
+  //
   // There are now no longer any references to us held by script or list items.
   // Note we must use GetAnimValKey/GetBaseValKey here, NOT InternalList()!
-  void *key = mIsAnimValList ?
-    InternalAList().GetAnimValKey() :
-    InternalAList().GetBaseValKey();
+  void* key = mIsAnimValList ? InternalAList().GetAnimValKey()
+                             : InternalAList().GetBaseValKey();
   SVGPointListTearoffTable().RemoveTearoff(key);
 }
 
-JSObject*
-DOMSVGPointList::WrapObject(JSContext *cx, JS::Handle<JSObject*> aGivenProto)
-{
-  return mozilla::dom::SVGPointListBinding::Wrap(cx, this, aGivenProto);
+DOMSVGPointList::~DOMSVGPointList() { RemoveFromTearoffTable(); }
+
+JSObject* DOMSVGPointList::WrapObject(JSContext* cx,
+                                      JS::Handle<JSObject*> aGivenProto) {
+  return mozilla::dom::SVGPointList_Binding::Wrap(cx, this, aGivenProto);
 }
 
-void
-DOMSVGPointList::InternalListWillChangeTo(const SVGPointList& aNewValue)
-{
+void DOMSVGPointList::InternalListWillChangeTo(const SVGPointList& aNewValue) {
   // When the number of items in our internal counterpart changes, we MUST stay
   // in sync. Everything in the scary comment in
   // DOMSVGLengthList::InternalBaseValListWillChangeTo applies here too!
@@ -146,10 +108,10 @@ DOMSVGPointList::InternalListWillChangeTo(const SVGPointList& aNewValue)
   uint32_t oldLength = mItems.Length();
 
   uint32_t newLength = aNewValue.Length();
-  if (newLength > nsISVGPoint::MaxListIndex()) {
+  if (newLength > DOMSVGPoint::MaxListIndex()) {
     // It's safe to get out of sync with our internal list as long as we have
     // FEWER items than it does.
-    newLength = nsISVGPoint::MaxListIndex();
+    newLength = DOMSVGPoint::MaxListIndex();
   }
 
   RefPtr<DOMSVGPointList> kungFuDeathGrip;
@@ -179,22 +141,22 @@ DOMSVGPointList::InternalListWillChangeTo(const SVGPointList& aNewValue)
   }
 }
 
-bool
-DOMSVGPointList::AttrIsAnimating() const
-{
+bool DOMSVGPointList::AttrIsAnimating() const {
   return InternalAList().IsAnimating();
 }
 
-SVGPointList&
-DOMSVGPointList::InternalList() const
-{
-  SVGAnimatedPointList *alist = mElement->GetAnimatedPointList();
-  return mIsAnimValList && alist->IsAnimating() ? *alist->mAnimVal : alist->mBaseVal;
+bool DOMSVGPointList::AnimListMirrorsBaseList() const {
+  return GetDOMWrapperIfExists(InternalAList().GetAnimValKey()) &&
+         !AttrIsAnimating();
 }
 
-SVGAnimatedPointList&
-DOMSVGPointList::InternalAList() const
-{
+SVGPointList& DOMSVGPointList::InternalList() const {
+  SVGAnimatedPointList* alist = mElement->GetAnimatedPointList();
+  return mIsAnimValList && alist->IsAnimating() ? *alist->mAnimVal
+                                                : alist->mBaseVal;
+}
+
+SVGAnimatedPointList& DOMSVGPointList::InternalAList() const {
   MOZ_ASSERT(mElement->GetAnimatedPointList(), "Internal error");
   return *mElement->GetAnimatedPointList();
 }
@@ -202,9 +164,7 @@ DOMSVGPointList::InternalAList() const
 // ----------------------------------------------------------------------------
 // nsIDOMSVGPointList implementation:
 
-void
-DOMSVGPointList::Clear(ErrorResult& aError)
-{
+void DOMSVGPointList::Clear(ErrorResult& aError) {
   if (IsAnimValList()) {
     aError.Throw(NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR);
     return;
@@ -216,14 +176,15 @@ DOMSVGPointList::Clear(ErrorResult& aError)
     // the internal list, otherwise they wouldn't be able to copy their
     // internal counterparts' values!
 
-    InternalListWillChangeTo(SVGPointList()); // clears mItems
+    InternalListWillChangeTo(SVGPointList());  // clears mItems
 
     if (!AttrIsAnimating()) {
       // The anim val list is in sync with the base val list
-      DOMSVGPointList *animList =
-        GetDOMWrapperIfExists(InternalAList().GetAnimValKey());
+      DOMSVGPointList* animList =
+          GetDOMWrapperIfExists(InternalAList().GetAnimValKey());
       if (animList) {
-        animList->InternalListWillChangeTo(SVGPointList()); // clears its mItems
+        animList->InternalListWillChangeTo(
+            SVGPointList());  // clears its mItems
       }
     }
 
@@ -231,9 +192,8 @@ DOMSVGPointList::Clear(ErrorResult& aError)
   }
 }
 
-already_AddRefed<nsISVGPoint>
-DOMSVGPointList::Initialize(nsISVGPoint& aNewItem, ErrorResult& aError)
-{
+already_AddRefed<DOMSVGPoint> DOMSVGPointList::Initialize(DOMSVGPoint& aNewItem,
+                                                          ErrorResult& aError) {
   if (IsAnimValList()) {
     aError.Throw(NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR);
     return nullptr;
@@ -247,10 +207,9 @@ DOMSVGPointList::Initialize(nsISVGPoint& aNewItem, ErrorResult& aError)
   // clone of aNewItem, it would actually insert aNewItem. To prevent that
   // from happening we have to do the clone here, if necessary.
 
-  nsCOMPtr<nsISVGPoint> domItem = &aNewItem;
-  if (domItem->HasOwner() || domItem->IsReadonly() ||
-      domItem->IsTranslatePoint()) {
-    domItem = domItem->Copy(); // must do this before changing anything!
+  RefPtr<DOMSVGPoint> domItem = &aNewItem;
+  if (domItem->HasOwner()) {
+    domItem = domItem->Copy();  // must do this before changing anything!
   }
 
   ErrorResult rv;
@@ -259,21 +218,18 @@ DOMSVGPointList::Initialize(nsISVGPoint& aNewItem, ErrorResult& aError)
   return InsertItemBefore(*domItem, 0, aError);
 }
 
-already_AddRefed<nsISVGPoint>
-DOMSVGPointList::GetItem(uint32_t index, ErrorResult& error)
-{
+already_AddRefed<DOMSVGPoint> DOMSVGPointList::GetItem(uint32_t index,
+                                                       ErrorResult& error) {
   bool found;
-  RefPtr<nsISVGPoint> item = IndexedGetter(index, found, error);
+  RefPtr<DOMSVGPoint> item = IndexedGetter(index, found, error);
   if (!found) {
     error.Throw(NS_ERROR_DOM_INDEX_SIZE_ERR);
   }
   return item.forget();
 }
 
-already_AddRefed<nsISVGPoint>
-DOMSVGPointList::IndexedGetter(uint32_t aIndex, bool& aFound,
-                               ErrorResult& aError)
-{
+already_AddRefed<DOMSVGPoint> DOMSVGPointList::IndexedGetter(
+    uint32_t aIndex, bool& aFound, ErrorResult& aError) {
   if (IsAnimValList()) {
     Element()->FlushAnimations();
   }
@@ -284,25 +240,22 @@ DOMSVGPointList::IndexedGetter(uint32_t aIndex, bool& aFound,
   return nullptr;
 }
 
-already_AddRefed<nsISVGPoint>
-DOMSVGPointList::InsertItemBefore(nsISVGPoint& aNewItem, uint32_t aIndex,
-                                  ErrorResult& aError)
-{
+already_AddRefed<DOMSVGPoint> DOMSVGPointList::InsertItemBefore(
+    DOMSVGPoint& aNewItem, uint32_t aIndex, ErrorResult& aError) {
   if (IsAnimValList()) {
     aError.Throw(NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR);
     return nullptr;
   }
 
   aIndex = std::min(aIndex, LengthNoFlush());
-  if (aIndex >= nsISVGPoint::MaxListIndex()) {
+  if (aIndex >= DOMSVGPoint::MaxListIndex()) {
     aError.Throw(NS_ERROR_DOM_INDEX_SIZE_ERR);
     return nullptr;
   }
 
-  nsCOMPtr<nsISVGPoint> domItem = &aNewItem;
-  if (domItem->HasOwner() || domItem->IsReadonly() ||
-      domItem->IsTranslatePoint()) {
-    domItem = domItem->Copy(); // must do this before changing anything!
+  RefPtr<DOMSVGPoint> domItem = &aNewItem;
+  if (domItem->HasOwner()) {
+    domItem = domItem->Copy();  // must do this before changing anything!
   }
 
   // Ensure we have enough memory so we can avoid complex error handling below:
@@ -311,6 +264,15 @@ DOMSVGPointList::InsertItemBefore(nsISVGPoint& aNewItem, uint32_t aIndex,
     aError.Throw(NS_ERROR_OUT_OF_MEMORY);
     return nullptr;
   }
+  if (AnimListMirrorsBaseList()) {
+    DOMSVGPointList* animVal =
+        GetDOMWrapperIfExists(InternalAList().GetAnimValKey());
+    MOZ_ASSERT(animVal, "animVal must be a valid pointer");
+    if (!animVal->mItems.SetCapacity(animVal->mItems.Length() + 1, fallible)) {
+      aError.Throw(NS_ERROR_OUT_OF_MEMORY);
+      return nullptr;
+    }
+  }
 
   AutoChangePointListNotifier notifier(this);
   // Now that we know we're inserting, keep animVal list in sync as necessary.
@@ -329,10 +291,8 @@ DOMSVGPointList::InsertItemBefore(nsISVGPoint& aNewItem, uint32_t aIndex,
   return domItem.forget();
 }
 
-already_AddRefed<nsISVGPoint>
-DOMSVGPointList::ReplaceItem(nsISVGPoint& aNewItem, uint32_t aIndex,
-                             ErrorResult& aError)
-{
+already_AddRefed<DOMSVGPoint> DOMSVGPointList::ReplaceItem(
+    DOMSVGPoint& aNewItem, uint32_t aIndex, ErrorResult& aError) {
   if (IsAnimValList()) {
     aError.Throw(NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR);
     return nullptr;
@@ -343,10 +303,9 @@ DOMSVGPointList::ReplaceItem(nsISVGPoint& aNewItem, uint32_t aIndex,
     return nullptr;
   }
 
-  nsCOMPtr<nsISVGPoint> domItem = &aNewItem;
-  if (domItem->HasOwner() || domItem->IsReadonly() ||
-      domItem->IsTranslatePoint()) {
-    domItem = domItem->Copy(); // must do this before changing anything!
+  RefPtr<DOMSVGPoint> domItem = &aNewItem;
+  if (domItem->HasOwner()) {
+    domItem = domItem->Copy();  // must do this before changing anything!
   }
 
   AutoChangePointListNotifier notifier(this);
@@ -366,9 +325,8 @@ DOMSVGPointList::ReplaceItem(nsISVGPoint& aNewItem, uint32_t aIndex,
   return domItem.forget();
 }
 
-already_AddRefed<nsISVGPoint>
-DOMSVGPointList::RemoveItem(uint32_t aIndex, ErrorResult& aError)
-{
+already_AddRefed<DOMSVGPoint> DOMSVGPointList::RemoveItem(uint32_t aIndex,
+                                                          ErrorResult& aError) {
   if (IsAnimValList()) {
     aError.Throw(NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR);
     return nullptr;
@@ -386,7 +344,7 @@ DOMSVGPointList::RemoveItem(uint32_t aIndex, ErrorResult& aError)
   MaybeRemoveItemFromAnimValListAt(aIndex);
 
   // We have to return the removed item, so get it, creating it if necessary:
-  RefPtr<nsISVGPoint> result = GetItemAt(aIndex);
+  RefPtr<DOMSVGPoint> result = GetItemAt(aIndex);
 
   // Notify the DOM item of removal *before* modifying the lists so that the
   // DOM item can copy its *old* value:
@@ -400,63 +358,48 @@ DOMSVGPointList::RemoveItem(uint32_t aIndex, ErrorResult& aError)
   return result.forget();
 }
 
-already_AddRefed<nsISVGPoint>
-DOMSVGPointList::GetItemAt(uint32_t aIndex)
-{
+already_AddRefed<DOMSVGPoint> DOMSVGPointList::GetItemAt(uint32_t aIndex) {
   MOZ_ASSERT(aIndex < mItems.Length());
 
   if (!mItems[aIndex]) {
     mItems[aIndex] = new DOMSVGPoint(this, aIndex, IsAnimValList());
   }
-  RefPtr<nsISVGPoint> result = mItems[aIndex];
+  RefPtr<DOMSVGPoint> result = mItems[aIndex];
   return result.forget();
 }
 
-void
-DOMSVGPointList::MaybeInsertNullInAnimValListAt(uint32_t aIndex)
-{
+void DOMSVGPointList::MaybeInsertNullInAnimValListAt(uint32_t aIndex) {
   MOZ_ASSERT(!IsAnimValList(), "call from baseVal to animVal");
 
-  if (AttrIsAnimating()) {
-    // animVal not a clone of baseVal
+  if (!AnimListMirrorsBaseList()) {
     return;
   }
 
   // The anim val list is in sync with the base val list
-  DOMSVGPointList *animVal =
-    GetDOMWrapperIfExists(InternalAList().GetAnimValKey());
-  if (!animVal) {
-    // No animVal list wrapper
-    return;
-  }
+  DOMSVGPointList* animVal =
+      GetDOMWrapperIfExists(InternalAList().GetAnimValKey());
 
+  MOZ_ASSERT(animVal, "AnimListMirrorsBaseList() promised a non-null animVal");
   MOZ_ASSERT(animVal->mItems.Length() == mItems.Length(),
              "animVal list not in sync!");
-
   MOZ_ALWAYS_TRUE(animVal->mItems.InsertElementAt(aIndex, nullptr, fallible));
 
   UpdateListIndicesFromIndex(animVal->mItems, aIndex + 1);
 }
 
-void
-DOMSVGPointList::MaybeRemoveItemFromAnimValListAt(uint32_t aIndex)
-{
+void DOMSVGPointList::MaybeRemoveItemFromAnimValListAt(uint32_t aIndex) {
   MOZ_ASSERT(!IsAnimValList(), "call from baseVal to animVal");
 
-  if (AttrIsAnimating()) {
-    // animVal not a clone of baseVal
+  if (!AnimListMirrorsBaseList()) {
     return;
   }
 
   // This needs to be a strong reference; otherwise, the RemovingFromList call
   // below might drop the last reference to animVal before we're done with it.
   RefPtr<DOMSVGPointList> animVal =
-    GetDOMWrapperIfExists(InternalAList().GetAnimValKey());
-  if (!animVal) {
-    // No animVal list wrapper
-    return;
-  }
+      GetDOMWrapperIfExists(InternalAList().GetAnimValKey());
 
+  MOZ_ASSERT(animVal, "AnimListMirrorsBaseList() promised a non-null animVal");
   MOZ_ASSERT(animVal->mItems.Length() == mItems.Length(),
              "animVal list not in sync!");
 
@@ -468,4 +411,4 @@ DOMSVGPointList::MaybeRemoveItemFromAnimValListAt(uint32_t aIndex)
   UpdateListIndicesFromIndex(animVal->mItems, aIndex);
 }
 
-} // namespace mozilla
+}  // namespace mozilla::dom
