@@ -20,6 +20,7 @@
 #include "mozilla/dom/Element.h"
 #include "mozilla/gfx/MatrixFwd.h"
 #include "mozilla/UniquePtr.h"
+#include "nsCSSPropertyID.h"
 #include "nsChangeHint.h"
 #include "nsCycleCollectionParticipant.h"
 #include "nsError.h"
@@ -83,8 +84,8 @@ class SVGElement : public SVGElementBase  // nsIContent
   virtual ~SVGElement();
 
  public:
-  virtual nsresult Clone(mozilla::dom::NodeInfo*,
-                         nsINode** aResult) const MOZ_MUST_OVERRIDE override;
+  nsresult Clone(mozilla::dom::NodeInfo*,
+                 nsINode** aResult) const MOZ_MUST_OVERRIDE override;
 
   // From Element
   nsresult CopyInnerTo(mozilla::dom::Element* aDest);
@@ -113,18 +114,19 @@ class SVGElement : public SVGElementBase  // nsIContent
 
   // nsIContent interface methods
 
-  virtual nsresult BindToTree(BindContext&, nsINode& aParent) override;
+  nsresult BindToTree(BindContext&, nsINode& aParent) override;
 
-  virtual nsChangeHint GetAttributeChangeHint(const nsAtom* aAttribute,
-                                              int32_t aModType) const override;
+  nsChangeHint GetAttributeChangeHint(const nsAtom* aAttribute,
+                                      int32_t aModType) const override;
 
-  virtual bool IsNodeOfType(uint32_t aFlags) const override;
+  bool IsNodeOfType(uint32_t aFlags) const override;
+  virtual bool IsSVGGraphicsElement() const { return false; }
 
   /**
    * We override the default to unschedule computation of Servo declaration
    * blocks when adopted across documents.
    */
-  virtual void NodeInfoChanged(Document* aOldDoc) override;
+  void NodeInfoChanged(Document* aOldDoc) override;
 
   NS_IMETHOD_(bool) IsAttributeMapped(const nsAtom* aAttribute) const override;
 
@@ -186,10 +188,10 @@ class SVGElement : public SVGElementBase  // nsIContent
   }
 
   bool IsStringAnimatable(uint8_t aAttrEnum) {
-    return GetStringInfo().mStringInfo[aAttrEnum].mIsAnimatable;
+    return GetStringInfo().mInfos[aAttrEnum].mIsAnimatable;
   }
   bool NumberAttrAllowsPercentage(uint8_t aAttrEnum) {
-    return GetNumberInfo().mNumberInfo[aAttrEnum].mPercentagesAllowed;
+    return GetNumberInfo().mInfos[aAttrEnum].mPercentagesAllowed;
   }
   virtual bool HasValidDimensions() const { return true; }
   void SetLength(nsAtom* aName, const SVGAnimatedLength& aLength);
@@ -199,6 +201,9 @@ class SVGElement : public SVGElementBase  // nsIContent
                                                nsCSSPropertyID aPropId,
                                                const SVGAnimatedLength& aLength,
                                                ValToUse aValToUse);
+  static bool UpdateDeclarationBlockFromPath(
+      DeclarationBlock& aBlock, const SVGAnimatedPathSegList& aPath,
+      ValToUse aValToUse);
 
   nsAttrValue WillChangeLength(uint8_t aAttrEnum,
                                const mozAutoDocUpdate& aProofOfUpdate);
@@ -321,8 +326,6 @@ class SVGElement : public SVGElementBase  // nsIContent
   void AnimationNeedsResample();
   void FlushAnimations();
 
-  virtual void RecompileScriptEventListeners() override;
-
   void GetStringBaseValue(uint8_t aAttrEnum, nsAString& aResult) const;
   void SetStringBaseValue(uint8_t aAttrEnum, const nsAString& aValue);
 
@@ -347,9 +350,17 @@ class SVGElement : public SVGElementBase  // nsIContent
   void UpdateContentDeclarationBlock();
   const mozilla::DeclarationBlock* GetContentDeclarationBlock() const;
 
+  bool Autofocus() const { return GetBoolAttr(nsGkAtoms::autofocus); }
+  void SetAutofocus(bool aAutofocus, ErrorResult& aRv) {
+    if (aAutofocus) {
+      SetAttr(nsGkAtoms::autofocus, u""_ns, aRv);
+    } else {
+      UnsetAttr(nsGkAtoms::autofocus, aRv);
+    }
+  }
+
  protected:
-  virtual JSObject* WrapNode(JSContext* cx,
-                             JS::Handle<JSObject*> aGivenProto) override;
+  JSObject* WrapNode(JSContext* cx, JS::Handle<JSObject*> aGivenProto) override;
 
   // We define BeforeSetAttr here and mark it final to ensure it is NOT used
   // by SVG elements.
@@ -358,15 +369,13 @@ class SVGElement : public SVGElementBase  // nsIContent
   // See the comment in SVGElement::WillChangeValue.
   nsresult BeforeSetAttr(int32_t aNamespaceID, nsAtom* aName,
                          const nsAttrValueOrString* aValue, bool aNotify) final;
-  virtual nsresult AfterSetAttr(int32_t aNamespaceID, nsAtom* aName,
-                                const nsAttrValue* aValue,
-                                const nsAttrValue* aOldValue,
-                                nsIPrincipal* aSubjectPrincipal,
-                                bool aNotify) override;
-  virtual bool ParseAttribute(int32_t aNamespaceID, nsAtom* aAttribute,
-                              const nsAString& aValue,
-                              nsIPrincipal* aMaybeScriptedPrincipal,
-                              nsAttrValue& aResult) override;
+  nsresult AfterSetAttr(int32_t aNamespaceID, nsAtom* aName,
+                        const nsAttrValue* aValue, const nsAttrValue* aOldValue,
+                        nsIPrincipal* aSubjectPrincipal, bool aNotify) override;
+  bool ParseAttribute(int32_t aNamespaceID, nsAtom* aAttribute,
+                      const nsAString& aValue,
+                      nsIPrincipal* aMaybeScriptedPrincipal,
+                      nsAttrValue& aResult) override;
   static nsresult ReportAttributeParseFailure(Document* aDocument,
                                               nsAtom* aAttribute,
                                               const nsAString& aValue);
@@ -380,7 +389,7 @@ class SVGElement : public SVGElementBase  // nsIContent
                       const mozAutoDocUpdate& aProofOfUpdate);
   void MaybeSerializeAttrBeforeRemoval(nsAtom* aName, bool aNotify);
 
-  nsAtom* GetEventNameForAttr(nsAtom* aAttr);
+  nsAtom* GetEventNameForAttr(nsAtom* aAttr) override;
 
   struct LengthInfo {
     nsStaticAtom* const mName;
@@ -389,39 +398,29 @@ class SVGElement : public SVGElementBase  // nsIContent
     const uint8_t mCtxType;
   };
 
-  struct LengthAttributesInfo {
-    SVGAnimatedLength* const mLengths;
-    const LengthInfo* const mLengthInfo;
-    const uint32_t mLengthCount;
+  template <typename Value, typename InfoValue>
+  struct AttributesInfo {
+    Value* const mValues;
+    const InfoValue* const mInfos;
+    const uint32_t mCount;
 
-    LengthAttributesInfo(SVGAnimatedLength* aLengths, LengthInfo* aLengthInfo,
-                         uint32_t aLengthCount)
-        : mLengths(aLengths),
-          mLengthInfo(aLengthInfo),
-          mLengthCount(aLengthCount) {}
+    AttributesInfo(Value* aValues, const InfoValue* aInfos, uint32_t aCount)
+        : mValues(aValues), mInfos(aInfos), mCount(aCount) {}
 
-    void Reset(uint8_t aAttrEnum);
+    void CopyAllFrom(const AttributesInfo&);
+    void ResetAll();
+    void Reset(uint8_t aEnum);
   };
 
+  using LengthAttributesInfo = AttributesInfo<SVGAnimatedLength, LengthInfo>;
+
   struct NumberInfo {
     nsStaticAtom* const mName;
     const float mDefaultValue;
     const bool mPercentagesAllowed;
   };
 
-  struct NumberAttributesInfo {
-    SVGAnimatedNumber* const mNumbers;
-    const NumberInfo* const mNumberInfo;
-    const uint32_t mNumberCount;
-
-    NumberAttributesInfo(SVGAnimatedNumber* aNumbers, NumberInfo* aNumberInfo,
-                         uint32_t aNumberCount)
-        : mNumbers(aNumbers),
-          mNumberInfo(aNumberInfo),
-          mNumberCount(aNumberCount) {}
-
-    void Reset(uint8_t aAttrEnum);
-  };
+  using NumberAttributesInfo = AttributesInfo<SVGAnimatedNumber, NumberInfo>;
 
   struct NumberPairInfo {
     nsStaticAtom* const mName;
@@ -429,39 +428,15 @@ class SVGElement : public SVGElementBase  // nsIContent
     const float mDefaultValue2;
   };
 
-  struct NumberPairAttributesInfo {
-    SVGAnimatedNumberPair* const mNumberPairs;
-    const NumberPairInfo* const mNumberPairInfo;
-    const uint32_t mNumberPairCount;
-
-    NumberPairAttributesInfo(SVGAnimatedNumberPair* aNumberPairs,
-                             NumberPairInfo* aNumberPairInfo,
-                             uint32_t aNumberPairCount)
-        : mNumberPairs(aNumberPairs),
-          mNumberPairInfo(aNumberPairInfo),
-          mNumberPairCount(aNumberPairCount) {}
-
-    void Reset(uint8_t aAttrEnum);
-  };
+  using NumberPairAttributesInfo =
+      AttributesInfo<SVGAnimatedNumberPair, NumberPairInfo>;
 
   struct IntegerInfo {
     nsStaticAtom* const mName;
     const int32_t mDefaultValue;
   };
 
-  struct IntegerAttributesInfo {
-    SVGAnimatedInteger* const mIntegers;
-    const IntegerInfo* const mIntegerInfo;
-    const uint32_t mIntegerCount;
-
-    IntegerAttributesInfo(SVGAnimatedInteger* aIntegers,
-                          IntegerInfo* aIntegerInfo, uint32_t aIntegerCount)
-        : mIntegers(aIntegers),
-          mIntegerInfo(aIntegerInfo),
-          mIntegerCount(aIntegerCount) {}
-
-    void Reset(uint8_t aAttrEnum);
-  };
+  using IntegerAttributesInfo = AttributesInfo<SVGAnimatedInteger, IntegerInfo>;
 
   struct IntegerPairInfo {
     nsStaticAtom* const mName;
@@ -469,39 +444,15 @@ class SVGElement : public SVGElementBase  // nsIContent
     const int32_t mDefaultValue2;
   };
 
-  struct IntegerPairAttributesInfo {
-    SVGAnimatedIntegerPair* const mIntegerPairs;
-    const IntegerPairInfo* const mIntegerPairInfo;
-    const uint32_t mIntegerPairCount;
-
-    IntegerPairAttributesInfo(SVGAnimatedIntegerPair* aIntegerPairs,
-                              IntegerPairInfo* aIntegerPairInfo,
-                              uint32_t aIntegerPairCount)
-        : mIntegerPairs(aIntegerPairs),
-          mIntegerPairInfo(aIntegerPairInfo),
-          mIntegerPairCount(aIntegerPairCount) {}
-
-    void Reset(uint8_t aAttrEnum);
-  };
+  using IntegerPairAttributesInfo =
+      AttributesInfo<SVGAnimatedIntegerPair, IntegerPairInfo>;
 
   struct BooleanInfo {
     nsStaticAtom* const mName;
     const bool mDefaultValue;
   };
 
-  struct BooleanAttributesInfo {
-    SVGAnimatedBoolean* const mBooleans;
-    const BooleanInfo* const mBooleanInfo;
-    const uint32_t mBooleanCount;
-
-    BooleanAttributesInfo(SVGAnimatedBoolean* aBooleans,
-                          BooleanInfo* aBooleanInfo, uint32_t aBooleanCount)
-        : mBooleans(aBooleans),
-          mBooleanInfo(aBooleanInfo),
-          mBooleanCount(aBooleanCount) {}
-
-    void Reset(uint8_t aAttrEnum);
-  };
+  using BooleanAttributesInfo = AttributesInfo<SVGAnimatedBoolean, BooleanInfo>;
 
   friend class mozilla::SVGAnimatedEnumeration;
 
@@ -511,37 +462,14 @@ class SVGElement : public SVGElementBase  // nsIContent
     const uint16_t mDefaultValue;
   };
 
-  struct EnumAttributesInfo {
-    SVGAnimatedEnumeration* const mEnums;
-    const EnumInfo* const mEnumInfo;
-    const uint32_t mEnumCount;
-
-    EnumAttributesInfo(SVGAnimatedEnumeration* aEnums, EnumInfo* aEnumInfo,
-                       uint32_t aEnumCount)
-        : mEnums(aEnums), mEnumInfo(aEnumInfo), mEnumCount(aEnumCount) {}
-
-    void Reset(uint8_t aAttrEnum);
-    void SetUnknownValue(uint8_t aAttrEnum);
-  };
+  using EnumAttributesInfo = AttributesInfo<SVGAnimatedEnumeration, EnumInfo>;
 
   struct NumberListInfo {
     nsStaticAtom* const mName;
   };
 
-  struct NumberListAttributesInfo {
-    SVGAnimatedNumberList* const mNumberLists;
-    const NumberListInfo* const mNumberListInfo;
-    const uint32_t mNumberListCount;
-
-    NumberListAttributesInfo(SVGAnimatedNumberList* aNumberLists,
-                             NumberListInfo* aNumberListInfo,
-                             uint32_t aNumberListCount)
-        : mNumberLists(aNumberLists),
-          mNumberListInfo(aNumberListInfo),
-          mNumberListCount(aNumberListCount) {}
-
-    void Reset(uint8_t aAttrEnum);
-  };
+  using NumberListAttributesInfo =
+      AttributesInfo<SVGAnimatedNumberList, NumberListInfo>;
 
   struct LengthListInfo {
     nsStaticAtom* const mName;
@@ -557,20 +485,8 @@ class SVGElement : public SVGElementBase  // nsIContent
     const bool mCouldZeroPadList;
   };
 
-  struct LengthListAttributesInfo {
-    SVGAnimatedLengthList* const mLengthLists;
-    const LengthListInfo* const mLengthListInfo;
-    const uint32_t mLengthListCount;
-
-    LengthListAttributesInfo(SVGAnimatedLengthList* aLengthLists,
-                             LengthListInfo* aLengthListInfo,
-                             uint32_t aLengthListCount)
-        : mLengthLists(aLengthLists),
-          mLengthListInfo(aLengthListInfo),
-          mLengthListCount(aLengthListCount) {}
-
-    void Reset(uint8_t aAttrEnum);
-  };
+  using LengthListAttributesInfo =
+      AttributesInfo<SVGAnimatedLengthList, LengthListInfo>;
 
   struct StringInfo {
     nsStaticAtom* const mName;
@@ -578,19 +494,7 @@ class SVGElement : public SVGElementBase  // nsIContent
     const bool mIsAnimatable;
   };
 
-  struct StringAttributesInfo {
-    SVGAnimatedString* const mStrings;
-    const StringInfo* const mStringInfo;
-    const uint32_t mStringCount;
-
-    StringAttributesInfo(SVGAnimatedString* aStrings, StringInfo* aStringInfo,
-                         uint32_t aStringCount)
-        : mStrings(aStrings),
-          mStringInfo(aStringInfo),
-          mStringCount(aStringCount) {}
-
-    void Reset(uint8_t aAttrEnum);
-  };
+  using StringAttributesInfo = AttributesInfo<SVGAnimatedString, StringInfo>;
 
   friend class DOMSVGStringList;
 
@@ -598,20 +502,8 @@ class SVGElement : public SVGElementBase  // nsIContent
     nsStaticAtom* const mName;
   };
 
-  struct StringListAttributesInfo {
-    SVGStringList* const mStringLists;
-    const StringListInfo* const mStringListInfo;
-    const uint32_t mStringListCount;
-
-    StringListAttributesInfo(SVGStringList* aStringLists,
-                             StringListInfo* aStringListInfo,
-                             uint32_t aStringListCount)
-        : mStringLists(aStringLists),
-          mStringListInfo(aStringListInfo),
-          mStringListCount(aStringListCount) {}
-
-    void Reset(uint8_t aAttrEnum);
-  };
+  using StringListAttributesInfo =
+      AttributesInfo<SVGStringList, StringListInfo>;
 
   virtual LengthAttributesInfo GetLengthInfo();
   virtual NumberAttributesInfo GetNumberInfo();
