@@ -6,8 +6,9 @@
 
 #include "mozilla/dom/SVGFEImageElement.h"
 
-#include "mozilla/EventStates.h"
 #include "mozilla/SVGObserverUtils.h"
+#include "mozilla/dom/Document.h"
+#include "mozilla/dom/BindContext.h"
 #include "mozilla/dom/SVGFEImageElementBinding.h"
 #include "mozilla/dom/SVGFilterElement.h"
 #include "mozilla/dom/UserActivation.h"
@@ -23,8 +24,7 @@ NS_IMPL_NS_NEW_SVG_ELEMENT(FEImage)
 
 using namespace mozilla::gfx;
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 JSObject* SVGFEImageElement::WrapNode(JSContext* aCx,
                                       JS::Handle<JSObject*> aGivenProto) {
@@ -49,7 +49,7 @@ SVGFEImageElement::SVGFEImageElement(
     already_AddRefed<mozilla::dom::NodeInfo>&& aNodeInfo)
     : SVGFEImageElementBase(std::move(aNodeInfo)), mImageAnimationMode(0) {
   // We start out broken
-  AddStatesSilently(NS_EVENT_STATE_BROKEN);
+  AddStatesSilently(ElementState::BROKEN);
 }
 
 SVGFEImageElement::~SVGFEImageElement() { nsImageLoadingContent::Destroy(); }
@@ -152,6 +152,10 @@ nsresult SVGFEImageElement::BindToTree(BindContext& aContext,
                           &SVGFEImageElement::MaybeLoadSVGImage));
   }
 
+  if (aContext.InComposedDoc()) {
+    aContext.OwnerDoc().SetUseCounter(eUseCounter_custom_feImage);
+  }
+
   return rv;
 }
 
@@ -160,7 +164,7 @@ void SVGFEImageElement::UnbindFromTree(bool aNullParent) {
   SVGFEImageElementBase::UnbindFromTree(aNullParent);
 }
 
-EventStates SVGFEImageElement::IntrinsicState() const {
+ElementState SVGFEImageElement::IntrinsicState() const {
   return SVGFEImageElementBase::IntrinsicState() |
          nsImageLoadingContent::ImageState();
 }
@@ -266,7 +270,7 @@ bool SVGFEImageElement::OutputIsTainted(const nsTArray<bool>& aInputsAreTainted,
 
   int32_t corsmode;
   if (NS_SUCCEEDED(currentRequest->GetCORSMode(&corsmode)) &&
-      corsmode != imgIRequest::CORS_NONE) {
+      corsmode != CORS_NONE) {
     // If CORS was used to load the image, the page is allowed to read from it.
     return false;
   }
@@ -353,5 +357,4 @@ void SVGFEImageElement::Notify(imgIRequest* aRequest, int32_t aType,
   }
 }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
