@@ -7,15 +7,16 @@
 #include "mozilla/dom/SVGStyleElement.h"
 
 #include "mozilla/RefPtr.h"
+#include "mozilla/dom/Document.h"
 #include "mozilla/dom/Element.h"
+#include "mozilla/dom/ReferrerInfo.h"
 #include "mozilla/dom/SVGStyleElementBinding.h"
 #include "nsCOMPtr.h"
 #include "nsContentUtils.h"
 
 NS_IMPL_NS_NEW_SVG_ELEMENT(Style)
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 JSObject* SVGStyleElement::WrapNode(JSContext* aCx,
                                     JS::Handle<JSObject*> aGivenProto) {
@@ -136,14 +137,6 @@ void SVGStyleElement::ContentChanged(nsIContent* aContent) {
 
 //----------------------------------------------------------------------
 
-void SVGStyleElement::GetXmlspace(nsAString& aXmlspace) {
-  GetAttr(kNameSpaceID_XML, nsGkAtoms::space, aXmlspace);
-}
-
-void SVGStyleElement::SetXmlspace(const nsAString& aXmlspace, ErrorResult& rv) {
-  rv = SetAttr(kNameSpaceID_XML, nsGkAtoms::space, aXmlspace, true);
-}
-
 void SVGStyleElement::GetMedia(nsAString& aMedia) {
   GetAttr(nsGkAtoms::media, aMedia);
 }
@@ -168,6 +161,17 @@ void SVGStyleElement::SetTitle(const nsAString& aTitle, ErrorResult& rv) {
   SetAttr(nsGkAtoms::title, aTitle, rv);
 }
 
+bool SVGStyleElement::Disabled() const {
+  StyleSheet* ss = GetSheet();
+  return ss && ss->Disabled();
+}
+
+void SVGStyleElement::SetDisabled(bool aDisabled) {
+  if (StyleSheet* ss = GetSheet()) {
+    ss->SetDisabled(aDisabled);
+  }
+}
+
 //----------------------------------------------------------------------
 // nsStyleLinkElement methods
 
@@ -203,5 +207,4 @@ Maybe<LinkStyle::SheetInfo> SVGStyleElement::GetStyleSheetInfo() {
   });
 }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
