@@ -4,41 +4,36 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#ifndef mozilla_dom_SVGViewportElement_h
-#define mozilla_dom_SVGViewportElement_h
+#ifndef DOM_SVG_SVGVIEWPORTELEMENT_H_
+#define DOM_SVG_SVGVIEWPORTELEMENT_H_
 
-#include "mozilla/dom/FromParser.h"
-#include "nsAutoPtr.h"
-#include "nsIContentInlines.h"
-#include "nsISVGPoint.h"
-#include "nsSVGEnum.h"
-#include "nsSVGLength2.h"
-#include "SVGGraphicsElement.h"
-#include "SVGImageContext.h"
-#include "nsSVGViewBox.h"
-#include "SVGPreserveAspectRatio.h"
-#include "SVGAnimatedPreserveAspectRatio.h"
 #include "mozilla/Attributes.h"
-
-class nsSVGOuterSVGFrame;
-class nsSVGViewportFrame;
+#include "mozilla/SVGImageContext.h"
+#include "mozilla/UniquePtr.h"
+#include "mozilla/dom/FromParser.h"
+#include "nsIContentInlines.h"
+#include "SVGAnimatedEnumeration.h"
+#include "SVGAnimatedLength.h"
+#include "SVGAnimatedPreserveAspectRatio.h"
+#include "SVGAnimatedViewBox.h"
+#include "SVGGraphicsElement.h"
+#include "SVGPoint.h"
+#include "SVGPreserveAspectRatio.h"
 
 namespace mozilla {
 class AutoPreserveAspectRatioOverride;
-class DOMSVGAnimatedPreserveAspectRatio;
+class SVGOuterSVGFrame;
+class SVGViewportFrame;
 
 namespace dom {
+class DOMSVGAnimatedPreserveAspectRatio;
 class SVGAnimatedRect;
-class SVGTransform;
 class SVGViewElement;
 class SVGViewportElement;
 
 class svgFloatSize {
-public:
-  svgFloatSize(float aWidth, float aHeight)
-    : width(aWidth)
-    , height(aHeight)
-  {}
+ public:
+  svgFloatSize(float aWidth, float aHeight) : width(aWidth), height(aHeight) {}
   bool operator!=(const svgFloatSize& rhs) {
     return width != rhs.width || height != rhs.height;
   }
@@ -46,31 +41,31 @@ public:
   float height;
 };
 
-class SVGViewportElement : public SVGGraphicsElement
-{
-  friend class ::nsSVGOuterSVGFrame;
-  friend class ::nsSVGViewportFrame;
+class SVGViewportElement : public SVGGraphicsElement {
+  friend class mozilla::SVGOuterSVGFrame;
+  friend class mozilla::SVGViewportFrame;
 
-protected:
+ protected:
+  explicit SVGViewportElement(
+      already_AddRefed<mozilla::dom::NodeInfo>&& aNodeInfo);
+  ~SVGViewportElement() = default;
 
-  SVGViewportElement(already_AddRefed<mozilla::dom::NodeInfo>& aNodeInfo);
-  ~SVGViewportElement();
-
-public:
+ public:
+  bool IsNodeOfType(uint32_t aFlags) const override;
 
   // nsIContent interface
-  NS_IMETHOD_(bool) IsAttributeMapped(const nsIAtom* aAttribute) const override;
+  NS_IMETHOD_(bool) IsAttributeMapped(const nsAtom* aAttribute) const override;
 
-  // nsSVGElement specializations:
-  virtual gfxMatrix PrependLocalTransformsTo(
-    const gfxMatrix &aMatrix,
-    SVGTransformTypes aWhich = eAllTransforms) const override;
+  // SVGElement specializations:
+  gfxMatrix PrependLocalTransformsTo(
+      const gfxMatrix& aMatrix,
+      SVGTransformTypes aWhich = eAllTransforms) const override;
 
-  virtual bool HasValidDimensions() const override;
+  bool HasValidDimensions() const override;
 
   // SVGViewportElement methods:
 
-  float GetLength(uint8_t mCtxType);
+  float GetLength(uint8_t aCtxType);
 
   // public helpers:
 
@@ -86,32 +81,26 @@ public:
    * Note also that this method does not pay attention to whether the width or
    * height values of the viewBox rect are positive!
    */
-  bool HasViewBoxRect() const {
-    return GetViewBoxInternal().HasRect();
-  }
+  bool HasViewBox() const { return GetViewBoxInternal().HasRect(); }
 
   /**
    * Returns true if we should synthesize a viewBox for ourselves (that is, if
    * we're the root element in an image document, and we're not currently being
    * painted for an <svg:image> element).
    *
-   * Only call this method if HasViewBoxRect() returns false.
+   * Only call this method if HasViewBox() returns false.
    */
   bool ShouldSynthesizeViewBox() const;
 
   bool HasViewBoxOrSyntheticViewBox() const {
-    return HasViewBoxRect() || ShouldSynthesizeViewBox();
+    return HasViewBox() || ShouldSynthesizeViewBox();
   }
 
-  bool HasChildrenOnlyTransform() const {
-    return mHasChildrenOnlyTransform;
-  }
+  bool HasChildrenOnlyTransform() const { return mHasChildrenOnlyTransform; }
 
   void UpdateHasChildrenOnlyTransform();
 
-  enum ChildrenOnlyTransformChangedFlags {
-    eDuringReflow = 1
-  };
+  enum ChildrenOnlyTransformChangedFlags { eDuringReflow = 1 };
 
   /**
    * This method notifies the style system that the overflow rects of our
@@ -132,37 +121,36 @@ public:
   }
 
   void SetViewportSize(const svgFloatSize& aSize) {
-    mViewportWidth  = aSize.width;
+    mViewportWidth = aSize.width;
     mViewportHeight = aSize.height;
   }
 
-  // WebIDL
-  already_AddRefed<SVGAnimatedRect> ViewBox();
-  already_AddRefed<DOMSVGAnimatedPreserveAspectRatio> PreserveAspectRatio();
-  virtual nsSVGViewBox* GetViewBox() override;
-
-protected:
-
-  // implementation helpers:
-
-  bool IsRoot() const {
-    NS_ASSERTION((IsInUncomposedDoc() && !GetParent()) ==
-                 (OwnerDoc() && (OwnerDoc()->GetRootElement() == this)),
-                 "Can't determine if we're root");
-    return IsInUncomposedDoc() && !GetParent();
-  }
-
   /**
    * Returns true if either this is an SVG <svg> element that is the child of
    * another non-foreignObject SVG element, or this is a SVG <symbol> element
-   * this is the root of a use-element shadow tree.
+   * that is the root of a use-element shadow tree.
    */
   bool IsInner() const {
-    const nsIContent *parent = GetFlattenedTreeParent();
+    const nsIContent* parent = GetFlattenedTreeParent();
     return parent && parent->IsSVGElement() &&
            !parent->IsSVGElement(nsGkAtoms::foreignObject);
   }
 
+  // WebIDL
+  already_AddRefed<SVGAnimatedRect> ViewBox();
+  already_AddRefed<DOMSVGAnimatedPreserveAspectRatio> PreserveAspectRatio();
+  SVGAnimatedViewBox* GetAnimatedViewBox() override;
+
+ protected:
+  // implementation helpers:
+
+  bool IsRootSVGSVGElement() const {
+    NS_ASSERTION((IsInUncomposedDoc() && !GetParent()) ==
+                     (OwnerDoc()->GetRootElement() == this),
+                 "Can't determine if we're root");
+    return !GetParent() && IsInUncomposedDoc() && IsSVGElement(nsGkAtoms::svg);
+  }
+
   /**
    * Returns the explicit or default preserveAspectRatio, unless we're
    * synthesizing a viewBox, in which case it returns the "none" value.
@@ -176,29 +164,23 @@ protected:
    * viewBox, if appropriate, or else a viewBox matching the dimensions of the
    * SVG viewport.
    */
-  nsSVGViewBoxRect GetViewBoxWithSynthesis(
-      float aViewportWidth, float aViewportHeight) const;
-
-  /**
-   * Retrieve the value of currentScale and currentTranslate.
-   */
-  virtual SVGPoint GetCurrentTranslate() const
-  { return SVGPoint(0.0f, 0.0f); }
-  virtual float GetCurrentScale() const
-  { return 1.0f; }
+  SVGViewBox GetViewBoxWithSynthesis(float aViewportWidth,
+                                     float aViewportHeight) const;
 
   enum { ATTR_X, ATTR_Y, ATTR_WIDTH, ATTR_HEIGHT };
-  nsSVGLength2 mLengthAttributes[4];
+  SVGAnimatedLength mLengthAttributes[4];
   static LengthInfo sLengthInfo[4];
-  virtual LengthAttributesInfo GetLengthInfo() override;
+  LengthAttributesInfo GetLengthInfo() override;
 
-  virtual SVGAnimatedPreserveAspectRatio *GetPreserveAspectRatio() override;
+  SVGAnimatedPreserveAspectRatio* GetAnimatedPreserveAspectRatio() override;
 
-  virtual const nsSVGViewBox& GetViewBoxInternal() const { return mViewBox; }
-  virtual nsSVGAnimatedTransformList* GetTransformInternal() const {
-    return mTransforms;
+  virtual const SVGAnimatedViewBox& GetViewBoxInternal() const {
+    return mViewBox;
   }
-  nsSVGViewBox                   mViewBox;
+  virtual SVGAnimatedTransformList* GetTransformInternal() const {
+    return mTransforms.get();
+  }
+  SVGAnimatedViewBox mViewBox;
   SVGAnimatedPreserveAspectRatio mPreserveAspectRatio;
 
   // The size of the rectangular SVG viewport into which we render. This is
@@ -211,11 +193,11 @@ protected:
   // XXXjwatt our frame should probably reset these when it's destroyed.
   float mViewportWidth, mViewportHeight;
 
-  bool     mHasChildrenOnlyTransform;
+  bool mHasChildrenOnlyTransform;
 };
 
-} // namespace dom
+}  // namespace dom
 
-} // namespace mozilla
+}  // namespace mozilla
 
-#endif // SVGViewportElement_h
+#endif  // DOM_SVG_SVGVIEWPORTELEMENT_H_
