@@ -11,12 +11,21 @@
 
 #include "prsystem.h"
 
+// Short macro to get the size of a member of a
+// given struct at compile time.
+// t is the type of struct, m the name of the
+// member:
+// DOM_SIZEOF_MEMBER(struct mystruct, myint)
+// will give you the size of the type of myint.
+#define DOM_SIZEOF_MEMBER(t, m) sizeof(((t*)0)->m)
+
 #if defined(XP_UNIX)
 #  include "unistd.h"
 #  include "dirent.h"
 #  include "poll.h"
 #  include "sys/stat.h"
 #  if defined(XP_LINUX)
+#    include <sys/prctl.h>
 #    include <sys/vfs.h>
 #    define statvfs statfs
 #    define f_frsize f_bsize
@@ -49,6 +58,8 @@
 
 #include "jsapi.h"
 #include "jsfriendapi.h"
+#include "js/friend/ErrorMessages.h"  // js::GetErrorMessage, JSMSG_*
+#include "js/PropertyAndElement.h"  // JS_DefineObject, JS_DefineProperty, JS_GetProperty, JS_SetProperty
 #include "BindingUtils.h"
 
 // Used to provide information on the OS
@@ -58,7 +69,6 @@
 #include "nsIObserver.h"
 #include "nsDirectoryServiceUtils.h"
 #include "nsIXULRuntime.h"
-#include "nsIPropertyBag2.h"
 #include "nsXPCOMCIDInternal.h"
 #include "nsServiceManagerUtils.h"
 #include "nsString.h"
@@ -66,14 +76,13 @@
 #include "nsDirectoryServiceDefs.h"
 #include "nsXULAppAPI.h"
 #include "nsAppDirectoryServiceDefs.h"
-#include "mozJSComponentLoader.h"
+#include "mozJSModuleLoader.h"
 
 #include "mozilla/ClearOnShutdown.h"
 #include "mozilla/StaticPtr.h"
 #include "mozilla/UniquePtr.h"
 
 #include "OSFileConstants.h"
-#include "nsIOSFileConstantsService.h"
 #include "nsZipArchive.h"
 
 #if defined(__DragonFly__) || defined(__FreeBSD__) || defined(__NetBSD__) || \
@@ -450,6 +459,7 @@ static const dom::ConstantSpec gLibcProperties[] = {
 #endif  // defined(EOVERFLOW)
     INT_CONSTANT(EPERM),
     INT_CONSTANT(ERANGE),
+    INT_CONSTANT(ENOSYS),
 #if defined(ETIMEDOUT)  // not defined with VC 9
     INT_CONSTANT(ETIMEDOUT),
 #endif                    // defined(ETIMEDOUT)
@@ -470,7 +480,7 @@ static const dom::ConstantSpec gLibcProperties[] = {
     INT_CONSTANT(DT_SOCK),
 #endif  // defined(DT_UNKNOWN)
 
-#if defined(S_IFIFO)
+#if defined(XP_UNIX)
     // Constants for |stat|
     INT_CONSTANT(S_IFMT),
     INT_CONSTANT(S_IFIFO),
@@ -478,12 +488,17 @@ static const dom::ConstantSpec gLibcProperties[] = {
     INT_CONSTANT(S_IFDIR),
     INT_CONSTANT(S_IFBLK),
     INT_CONSTANT(S_IFREG),
-    INT_CONSTANT(S_IFLNK),
-    INT_CONSTANT(S_IFSOCK),
-#endif  // defined(S_IFIFO)
+    INT_CONSTANT(S_IFLNK),   // not defined on minGW
+    INT_CONSTANT(S_IFSOCK),  // not defined on minGW
+#endif                       // defined(XP_UNIX)
 
     INT_CONSTANT(PATH_MAX),
 
+#if defined(XP_LINUX)
+    // prctl options
+    INT_CONSTANT(PR_CAPBSET_READ),
+#endif
+
 // Constants used to define data structures
 //
 // Many data structures have different fields/sizes/etc. on
@@ -512,14 +527,17 @@ static const dom::ConstantSpec gLibcProperties[] = {
     // The size of |posix_spawn_file_actions_t|.
     {"OSFILE_SIZEOF_POSIX_SPAWN_FILE_ACTIONS_T",
      JS::Int32Value(sizeof(posix_spawn_file_actions_t))},
+
+    // The size of |posix_spawnattr_t|.
+    {"OSFILE_SIZEOF_POSIX_SPAWNATTR_T",
+     JS::Int32Value(sizeof(posix_spawnattr_t))},
 #  endif  // !defined(ANDROID)
 
     // Defining |dirent|.
     // Size
     {"OSFILE_SIZEOF_DIRENT", JS::Int32Value(sizeof(dirent))},
 
-// Defining |flock|.
-#  if defined(XP_UNIX)
+    // Defining |flock|.
     {"OSFILE_SIZEOF_FLOCK", JS::Int32Value(sizeof(struct flock))},
     {"OSFILE_OFFSETOF_FLOCK_L_START",
      JS::Int32Value(offsetof(struct flock, l_start))},
@@ -531,7 +549,7 @@ static const dom::ConstantSpec gLibcProperties[] = {
      JS::Int32Value(offsetof(struct flock, l_type))},
     {"OSFILE_OFFSETOF_FLOCK_L_WHENCE",
      JS::Int32Value(offsetof(struct flock, l_whence))},
-#  endif  // defined(XP_UNIX)
+
     // Offset of field |d_name|.
     {"OSFILE_OFFSETOF_DIRENT_D_NAME",
      JS::Int32Value(offsetof(struct dirent, d_name))},
@@ -602,6 +620,9 @@ static const dom::ConstantSpec gLibcProperties[] = {
 
     {"OSFILE_SIZEOF_STATVFS", JS::Int32Value(sizeof(struct statvfs))},
 
+    // We have no guarantee how big "f_frsize" is, so we have to calculate that.
+    {"OSFILE_SIZEOF_STATVFS_F_FRSIZE",
+     JS::Int32Value(DOM_SIZEOF_MEMBER(struct statvfs, f_frsize))},
     {"OSFILE_OFFSETOF_STATVFS_F_FRSIZE",
      JS::Int32Value(offsetof(struct statvfs, f_frsize))},
     {"OSFILE_OFFSETOF_STATVFS_F_BAVAIL",
@@ -614,11 +635,11 @@ static const dom::ConstantSpec gLibcProperties[] = {
 // Under MacOSX, to avoid using deprecated functions that do not
 // match the constants we define in this object (including
 // |sizeof|/|offsetof| stuff, but not only), for a number of
-// functions, we need to adapt the name of the symbols we are using,
-// whenever macro _DARWIN_FEATURE_64_BIT_INODE is set. We export
-// this value to be able to do so from JavaScript.
-#if defined(_DARWIN_FEATURE_64_BIT_INODE)
-    {"_DARWIN_FEATURE_64_BIT_INODE", JS::Int32Value(1)},
+// functions, we need to use functions with a $INODE64 suffix.
+// That is true on Intel-based mac when the _DARWIN_FEATURE_64_BIT_INODE
+// macro is set. But not on Apple Silicon.
+#if defined(_DARWIN_FEATURE_64_BIT_INODE) && !defined(__aarch64__)
+    {"_DARWIN_INODE64_SYMBOLS", JS::Int32Value(1)},
 #endif  // defined(_DARWIN_FEATURE_64_BIT_INODE)
 
 // Similar feature for Linux
@@ -905,27 +926,6 @@ bool OSFileConstantsService::DefineOSFileConstants(
     return false;
   }
 
-  // sqlite3 is linked from different places depending on the platform
-  nsAutoString libsqlite3;
-#if defined(ANDROID)
-  // On Android, we use the system's libsqlite3
-  libsqlite3.AppendLiteral(MOZ_DLL_PREFIX);
-  libsqlite3.AppendLiteral("sqlite3");
-  libsqlite3.AppendLiteral(MOZ_DLL_SUFFIX);
-#elif defined(XP_WIN)
-  // On Windows, for some reason, this is part of nss3.dll
-  libsqlite3.AppendLiteral(MOZ_DLL_PREFIX);
-  libsqlite3.AppendLiteral("nss3");
-  libsqlite3.AppendLiteral(MOZ_DLL_SUFFIX);
-#else
-  // On other platforms, we link sqlite3 into libxul
-  libsqlite3 = libxul;
-#endif  // defined(ANDROID) || defined(XP_WIN)
-
-  if (!SetStringProperty(aCx, objPath, "libsqlite3", libsqlite3)) {
-    return false;
-  }
-
   return true;
 }
 
@@ -943,7 +943,7 @@ already_AddRefed<OSFileConstantsService> OSFileConstantsService::GetOrCreate() {
       return nullptr;
     }
 
-    gInstance = service.forget();
+    gInstance = std::move(service);
     ClearOnShutdown(&gInstance);
   }
 
@@ -969,7 +969,7 @@ OSFileConstantsService::Init(JSContext* aCx) {
     return rv;
   }
 
-  mozJSComponentLoader* loader = mozJSComponentLoader::Get();
+  mozJSModuleLoader* loader = mozJSModuleLoader::Get();
   JS::Rooted<JSObject*> targetObj(aCx);
   loader->FindTargetObject(aCx, &targetObj);
 
