@@ -8,46 +8,106 @@
 #define mozilla_dom_PathUtils__
 
 #include "mozilla/DataMutex.h"
-#include "mozilla/ErrorResult.h"
+#include "mozilla/EnumeratedArray.h"
 #include "mozilla/Maybe.h"
 #include "mozilla/MozPromise.h"
 #include "mozilla/Mutex.h"
-#include "mozilla/dom/DOMParser.h"
+#include "mozilla/Result.h"
+#include "mozilla/dom/PathUtilsBinding.h"
 #include "mozilla/dom/Promise.h"
+#include "nsAppDirectoryServiceDefs.h"
+#include "nsDirectoryServiceDefs.h"
 #include "nsString.h"
 #include "nsTArray.h"
 
 namespace mozilla {
+class ErrorResult;
+
 namespace dom {
 
 class PathUtils final {
  public:
+  /**
+   * Initialize the given nsIFile with the given path.
+   *
+   * This is equivalent to calling nsIFile::InitWithPath() with the caveat that
+   * on Windows debug or during Windows CI tests, we will crash if the path
+   * contains a forward slash.
+   *
+   * @param aFile The file to initialize.
+   * @param aPath The path to initialize the file with.
+   *
+   * @return The result of calling nsIFile::InitWithPath.
+   */
+  static nsresult InitFileWithPath(nsIFile* aFile, const nsAString& aPath);
+
   static void Filename(const GlobalObject&, const nsAString& aPath,
                        nsString& aResult, ErrorResult& aErr);
 
   static void Parent(const GlobalObject&, const nsAString& aPath,
-                     nsString& aResult, ErrorResult& aErr);
+                     const int32_t aDepth, nsString& aResult,
+                     ErrorResult& aErr);
 
   static void Join(const GlobalObject&, const Sequence<nsString>& aComponents,
                    nsString& aResult, ErrorResult& aErr);
 
+  /**
+   * Join a sequence of path components and return an nsIFile with the resulting
+   * path.
+   *
+   * @param aComponents  A sequence of path components. The first component must
+   *                     be an absolute path.
+   * @param aErr  The error result, if any.
+   *
+   * @return An nsIFile with the resulting path, if there were no errors.
+   * Otherwise, nullptr is returned.
+   */
+  static already_AddRefed<nsIFile> Join(const Span<const nsString>& aComponents,
+                                        ErrorResult& aErr);
+
+  static void JoinRelative(const GlobalObject&, const nsAString& aBasePath,
+                           const nsAString& aRelativePath, nsString& aResult,
+                           ErrorResult& aErr);
+
+  static void ToExtendedWindowsPath(const GlobalObject&, const nsAString& aPath,
+                                    nsString& aResult, ErrorResult& aErr);
+
   static void Normalize(const GlobalObject&, const nsAString& aPath,
                         nsString& aResult, ErrorResult& aErr);
 
   static void Split(const GlobalObject&, const nsAString& aPath,
                     nsTArray<nsString>& aResult, ErrorResult& aErr);
 
+  static void SplitRelative(const GlobalObject& aGlobal, const nsAString& aPath,
+                            const SplitRelativeOptions& aOptions,
+                            nsTArray<nsString>& aResult, ErrorResult& aErr);
+
   static void ToFileURI(const GlobalObject&, const nsAString& aPath,
                         nsCString& aResult, ErrorResult& aErr);
 
-  static already_AddRefed<Promise> GetProfileDir(const GlobalObject& aGlobal,
-                                                 ErrorResult& aErr);
+  static bool IsAbsolute(const GlobalObject&, const nsAString& aPath);
 
-  static already_AddRefed<Promise> GetLocalProfileDir(
+  static void GetProfileDirSync(const GlobalObject&, nsString& aResult,
+                                ErrorResult& aErr);
+  static void GetLocalProfileDirSync(const GlobalObject&, nsString& aResult,
+                                     ErrorResult& aErr);
+  static void GetTempDirSync(const GlobalObject&, nsString& aResult,
+                             ErrorResult& aErr);
+  static void GetOSTempDirSync(const GlobalObject&, nsString& aResult,
+                               ErrorResult& aErr);
+  static void GetXulLibraryPathSync(const GlobalObject&, nsString& aResult,
+                                    ErrorResult& aErr);
+
+  static already_AddRefed<Promise> GetProfileDirAsync(
+      const GlobalObject& aGlobal, ErrorResult& aErr);
+  static already_AddRefed<Promise> GetLocalProfileDirAsync(
+      const GlobalObject& aGlobal, ErrorResult& aErr);
+  static already_AddRefed<Promise> GetTempDirAsync(const GlobalObject& aGlobal,
+                                                   ErrorResult& aErr);
+  static already_AddRefed<Promise> GetOSTempDirAsync(
+      const GlobalObject& aGlobal, ErrorResult& aErr);
+  static already_AddRefed<Promise> GetXulLibraryPathAsync(
       const GlobalObject& aGlobal, ErrorResult& aErr);
-
-  static already_AddRefed<Promise> GetTempDir(const GlobalObject& aGlobal,
-                                              ErrorResult& aErr);
 
  private:
   class DirectoryCache;
@@ -62,7 +122,8 @@ class PathUtils final {
 class PathUtils::DirectoryCache final {
  public:
   /**
-   * A directory that can be requested via |GetDirectory|.
+   * A directory that can be requested via |GetDirectorySync| or
+   * |GetDirectoryAsync|.
    */
   enum class Directory {
     /**
@@ -77,6 +138,18 @@ class PathUtils::DirectoryCache final {
      * The temporary directory for the process.
      */
     Temp,
+    /**
+     * The OS temporary directory.
+     */
+    OSTemp,
+    /**
+     * The libxul path.
+     */
+    XulLibrary,
+    /**
+     * The number of Directory entries.
+     */
+    Count,
   };
 
   DirectoryCache();
@@ -95,6 +168,9 @@ class PathUtils::DirectoryCache final {
    */
   static DirectoryCache& Ensure(Maybe<DirectoryCache>& aCache);
 
+  void GetDirectorySync(nsString& aResult, ErrorResult& aErr,
+                        const Directory aRequestedDir);
+
   /**
    * Request the path of a specific directory.
    *
@@ -107,9 +183,9 @@ class PathUtils::DirectoryCache final {
    *
    * @return A promise that resolves to the path of the requested directory.
    */
-  already_AddRefed<Promise> GetDirectory(const GlobalObject& aGlobalObject,
-                                         ErrorResult& aErr,
-                                         const Directory aRequestedDir);
+  already_AddRefed<Promise> GetDirectoryAsync(const GlobalObject& aGlobalObject,
+                                              ErrorResult& aErr,
+                                              const Directory aRequestedDir);
 
  private:
   using PopulateDirectoriesPromise = MozPromise<Ok, nsresult, false>;
@@ -172,19 +248,17 @@ class PathUtils::DirectoryCache final {
    */
   void ResolveWithDirectory(Promise* aPromise, const Directory aRequestedDir);
 
-  /**
-   * A promise that is resolved when |mProfileDir| and |mLocalProfileDir| are
-   * populated.
-   */
-  MozPromiseHolder<PopulateDirectoriesPromise> mProfileDirsPromise;
-  nsString mProfileDir;
-  nsString mLocalProfileDir;
+  template <typename T>
+  using DirectoryArray = EnumeratedArray<Directory, Directory::Count, T>;
 
-  /**
-   * A promise that is resolved when *all* cache entries are populated.
-   */
-  MozPromiseHolder<PopulateDirectoriesPromise> mAllDirsPromise;
-  nsString mTempDir;
+  DirectoryArray<nsString> mDirectories;
+  DirectoryArray<MozPromiseHolder<PopulateDirectoriesPromise>> mPromises;
+
+  static constexpr DirectoryArray<const char*> kDirectoryNames{
+      NS_APP_USER_PROFILE_50_DIR,      NS_APP_USER_PROFILE_LOCAL_50_DIR,
+      NS_APP_CONTENT_PROCESS_TEMP_DIR, NS_OS_TEMP_DIR,
+      NS_XPCOM_LIBRARY_FILE,
+  };
 };
 
 }  // namespace dom
