@@ -9,10 +9,39 @@
 #include "mozilla/StaticPtr.h"
 
 #include "AndroidWebAuthnTokenManager.h"
-#include "GeneratedJNIWrappers.h"
 #include "JavaBuiltins.h"
+#include "JavaExceptions.h"
+#include "mozilla/java/WebAuthnTokenManagerWrappers.h"
+#include "mozilla/jni/Conversions.h"
 
 namespace mozilla {
+namespace jni {
+
+template <>
+dom::AndroidWebAuthnResult Java2Native(mozilla::jni::Object::Param aData,
+                                       JNIEnv* aEnv) {
+  // TODO:
+  // AndroidWebAuthnResult stores successful both result and failure result.
+  // We should split it into success and failure (Bug 1754157)
+  if (aData.IsInstanceOf<jni::Throwable>()) {
+    java::sdk::Throwable::LocalRef throwable(aData);
+    return dom::AndroidWebAuthnResult(throwable->GetMessage()->ToString());
+  }
+
+  if (aData
+          .IsInstanceOf<java::WebAuthnTokenManager::MakeCredentialResponse>()) {
+    java::WebAuthnTokenManager::MakeCredentialResponse::LocalRef response(
+        aData);
+    return dom::AndroidWebAuthnResult(response);
+  }
+
+  MOZ_ASSERT(
+      aData.IsInstanceOf<java::WebAuthnTokenManager::GetAssertionResponse>());
+  java::WebAuthnTokenManager::GetAssertionResponse::LocalRef response(aData);
+  return dom::AndroidWebAuthnResult(response);
+}
+}  // namespace jni
+
 namespace dom {
 
 static nsIThread* gAndroidPBackgroundThread;
@@ -58,14 +87,15 @@ void AndroidWebAuthnTokenManager::Drop() {
 }
 
 RefPtr<U2FRegisterPromise> AndroidWebAuthnTokenManager::Register(
-    const WebAuthnMakeCredentialInfo& aInfo, bool aForceNoneAttestation) {
+    const WebAuthnMakeCredentialInfo& aInfo, bool aForceNoneAttestation,
+    void _status_callback(rust_ctap2_status_update_res*)) {
   AssertIsOnOwningThread();
 
   ClearPromises();
 
   GetMainThreadEventTarget()->Dispatch(NS_NewRunnableFunction(
       "java::WebAuthnTokenManager::WebAuthnMakeCredential",
-      [aInfo, aForceNoneAttestation]() {
+      [self = RefPtr{this}, aInfo, aForceNoneAttestation]() {
         AssertIsOnMainThread();
 
         // Produce the credential exclusion list
@@ -121,19 +151,11 @@ RefPtr<U2FRegisterPromise> AndroidWebAuthnTokenManager::Register(
             attestation = AttestationConveyancePreference::None;
           }
 
-          if (static_cast<uint32_t>(attestation) <
-              static_cast<uint32_t>(
-                  AttestationConveyancePreference::EndGuard_)) {
-            // Protect the direct array reference below. Note this is still a
-            // ContiguousEnumValidator type.
-            nsString attestPref;
-            const EnumEntry& attPrefEntry =
-                AttestationConveyancePreferenceValues::strings
-                    [static_cast<uint32_t>(attestation)];
-            attestPref.AssignASCII(attPrefEntry.value, attPrefEntry.length);
-            GECKOBUNDLE_PUT(authSelBundle, "attestationPreference",
-                            jni::StringParam(attestPref));
-          }
+          nsString attestPref;
+          attestPref.AssignASCII(
+              AttestationConveyancePreferenceValues::GetString(attestation));
+          GECKOBUNDLE_PUT(authSelBundle, "attestationPreference",
+                          jni::StringParam(attestPref));
 
           const WebAuthnAuthenticatorSelection& sel =
               extra.AuthenticatorSelection();
@@ -154,6 +176,10 @@ RefPtr<U2FRegisterPromise> AndroidWebAuthnTokenManager::Register(
             if (authenticatorAttachment == AuthenticatorAttachment::Platform) {
               GECKOBUNDLE_PUT(authSelBundle, "requirePlatformAttachment",
                               java::sdk::Integer::ValueOf(1));
+            } else if (authenticatorAttachment ==
+                       AuthenticatorAttachment::Cross_platform) {
+              GECKOBUNDLE_PUT(authSelBundle, "requireCrossPlatformAttachment",
+                              java::sdk::Integer::ValueOf(1));
             }
           }
 
@@ -197,16 +223,33 @@ RefPtr<U2FRegisterPromise> AndroidWebAuthnTokenManager::Register(
             const_cast<void*>(static_cast<const void*>(uidBuf.Elements())),
             uidBuf.Length());
 
-        java::WebAuthnTokenManager::WebAuthnMakeCredential(
+        auto result = java::WebAuthnTokenManager::WebAuthnMakeCredential(
             credentialBundle, uid, challenge, idList, transportList,
             authSelBundle, extensionsBundle);
+        auto geckoResult = java::GeckoResult::LocalRef(std::move(result));
+        // This is likely running on the main thread, so we'll always dispatch
+        // to the background for state updates.
+        MozPromise<AndroidWebAuthnResult, AndroidWebAuthnResult,
+                   true>::FromGeckoResult(geckoResult)
+            ->Then(
+                GetMainThreadSerialEventTarget(), __func__,
+                [self = std::move(self)](AndroidWebAuthnResult&& aValue) {
+                  self->HandleRegisterResult(std::move(aValue));
+                },
+                [self = std::move(self)](AndroidWebAuthnResult&& aValue) {
+                  self->HandleRegisterResult(std::move(aValue));
+                });
       }));
 
   return mRegisterPromise.Ensure(__func__);
 }
 
 void AndroidWebAuthnTokenManager::HandleRegisterResult(
-    const AndroidWebAuthnResult& aResult) {
+    AndroidWebAuthnResult&& aResult) {
+  if (!gAndroidPBackgroundThread) {
+    // Promise is already rejected when shutting down background thread
+    return;
+  }
   // This is likely running on the main thread, so we'll always dispatch to the
   // background for state updates.
   if (aResult.IsError()) {
@@ -220,24 +263,28 @@ void AndroidWebAuthnTokenManager::HandleRegisterResult(
   } else {
     gAndroidPBackgroundThread->Dispatch(NS_NewRunnableFunction(
         "AndroidWebAuthnTokenManager::RegisterComplete",
-        [self = RefPtr<AndroidWebAuthnTokenManager>(this), aResult]() {
+        [self = RefPtr<AndroidWebAuthnTokenManager>(this),
+         aResult = std::move(aResult)]() {
           CryptoBuffer emptyBuffer;
-          WebAuthnMakeCredentialResult result(aResult.mClientDataJSON,
-                                              aResult.mAttObj,
-                                              aResult.mKeyHandle, emptyBuffer);
+          nsTArray<WebAuthnExtensionResult> extensions;
+          WebAuthnMakeCredentialResult result(
+              aResult.mClientDataJSON, aResult.mAttObj, aResult.mKeyHandle,
+              emptyBuffer, extensions);
           self->mRegisterPromise.Resolve(std::move(result), __func__);
         }));
   }
 }
 
 RefPtr<U2FSignPromise> AndroidWebAuthnTokenManager::Sign(
-    const WebAuthnGetAssertionInfo& aInfo) {
+    const WebAuthnGetAssertionInfo& aInfo,
+    void _status_callback(rust_ctap2_status_update_res*)) {
   AssertIsOnOwningThread();
 
   ClearPromises();
 
   GetMainThreadEventTarget()->Dispatch(NS_NewRunnableFunction(
-      "java::WebAuthnTokenManager::WebAuthnGetAssertion", [aInfo]() {
+      "java::WebAuthnTokenManager::WebAuthnGetAssertion",
+      [self = RefPtr{this}, aInfo]() {
         AssertIsOnMainThread();
 
         jni::ObjectArray::LocalRef idList =
@@ -301,16 +348,31 @@ RefPtr<U2FSignPromise> AndroidWebAuthnTokenManager::Sign(
         GECKOBUNDLE_FINISH(assertionBundle);
         GECKOBUNDLE_FINISH(extensionsBundle);
 
-        java::WebAuthnTokenManager::WebAuthnGetAssertion(
+        auto result = java::WebAuthnTokenManager::WebAuthnGetAssertion(
             challenge, idList, transportList, assertionBundle,
             extensionsBundle);
+        auto geckoResult = java::GeckoResult::LocalRef(std::move(result));
+        MozPromise<AndroidWebAuthnResult, AndroidWebAuthnResult,
+                   true>::FromGeckoResult(geckoResult)
+            ->Then(
+                GetMainThreadSerialEventTarget(), __func__,
+                [self = std::move(self)](AndroidWebAuthnResult&& aValue) {
+                  self->HandleSignResult(std::move(aValue));
+                },
+                [self = std::move(self)](AndroidWebAuthnResult&& aValue) {
+                  self->HandleSignResult(std::move(aValue));
+                });
       }));
 
   return mSignPromise.Ensure(__func__);
 }
 
 void AndroidWebAuthnTokenManager::HandleSignResult(
-    const AndroidWebAuthnResult& aResult) {
+    AndroidWebAuthnResult&& aResult) {
+  if (!gAndroidPBackgroundThread) {
+    // Promise is already rejected when shutting down background thread
+    return;
+  }
   // This is likely running on the main thread, so we'll always dispatch to the
   // background for state updates.
   if (aResult.IsError()) {
@@ -324,7 +386,8 @@ void AndroidWebAuthnTokenManager::HandleSignResult(
   } else {
     gAndroidPBackgroundThread->Dispatch(NS_NewRunnableFunction(
         "AndroidWebAuthnTokenManager::SignComplete",
-        [self = RefPtr<AndroidWebAuthnTokenManager>(this), aResult]() {
+        [self = RefPtr<AndroidWebAuthnTokenManager>(this),
+         aResult = std::move(aResult)]() {
           CryptoBuffer emptyBuffer;
 
           nsTArray<WebAuthnExtensionResult> emptyExtensions;
@@ -332,7 +395,9 @@ void AndroidWebAuthnTokenManager::HandleSignResult(
               aResult.mClientDataJSON, aResult.mKeyHandle, aResult.mSignature,
               aResult.mAuthData, emptyExtensions, emptyBuffer,
               aResult.mUserHandle);
-          self->mSignPromise.Resolve(std::move(result), __func__);
+          nsTArray<WebAuthnGetAssertionResultWrapper> results = {
+              {result, mozilla::Nothing()}};
+          self->mSignPromise.Resolve(std::move(results), __func__);
         }));
   }
 }
@@ -343,5 +408,41 @@ void AndroidWebAuthnTokenManager::Cancel() {
   ClearPromises();
 }
 
+AndroidWebAuthnResult::AndroidWebAuthnResult(
+    const java::WebAuthnTokenManager::MakeCredentialResponse::LocalRef&
+        aResponse) {
+  mClientDataJSON.Assign(
+      reinterpret_cast<const char*>(
+          aResponse->ClientDataJson()->GetElements().Elements()),
+      aResponse->ClientDataJson()->Length());
+  mKeyHandle.Assign(reinterpret_cast<uint8_t*>(
+                        aResponse->KeyHandle()->GetElements().Elements()),
+                    aResponse->KeyHandle()->Length());
+  mAttObj.Assign(reinterpret_cast<uint8_t*>(
+                     aResponse->AttestationObject()->GetElements().Elements()),
+                 aResponse->AttestationObject()->Length());
+}
+
+AndroidWebAuthnResult::AndroidWebAuthnResult(
+    const java::WebAuthnTokenManager::GetAssertionResponse::LocalRef&
+        aResponse) {
+  mClientDataJSON.Assign(
+      reinterpret_cast<const char*>(
+          aResponse->ClientDataJson()->GetElements().Elements()),
+      aResponse->ClientDataJson()->Length());
+  mKeyHandle.Assign(reinterpret_cast<uint8_t*>(
+                        aResponse->KeyHandle()->GetElements().Elements()),
+                    aResponse->KeyHandle()->Length());
+  mAuthData.Assign(reinterpret_cast<uint8_t*>(
+                       aResponse->AuthData()->GetElements().Elements()),
+                   aResponse->AuthData()->Length());
+  mSignature.Assign(reinterpret_cast<uint8_t*>(
+                        aResponse->Signature()->GetElements().Elements()),
+                    aResponse->Signature()->Length());
+  mUserHandle.Assign(reinterpret_cast<uint8_t*>(
+                         aResponse->UserHandle()->GetElements().Elements()),
+                     aResponse->UserHandle()->Length());
+}
+
 }  // namespace dom
 }  // namespace mozilla
