@@ -9,6 +9,7 @@
 #include "CryptoBuffer.h"
 #include "mozilla/Base64.h"
 #include "mozilla/Casting.h"
+#include "mozilla/Preferences.h"
 #include "nsNSSComponent.h"
 #include "nsThreadUtils.h"
 #include "pk11pub.h"
@@ -18,17 +19,15 @@
 
 #define PREF_U2F_NSSTOKEN_COUNTER "security.webauth.softtoken_counter"
 
-namespace mozilla {
-namespace dom {
+namespace mozilla::dom {
 
 using namespace mozilla;
 using mozilla::dom::CreateECParamsForCurve;
 
-const nsCString U2FSoftTokenManager::mSecretNickname =
-    NS_LITERAL_CSTRING("U2F_NSSTOKEN");
+const nsCString U2FSoftTokenManager::mSecretNickname = "U2F_NSSTOKEN"_ns;
 
 namespace {
-NS_NAMED_LITERAL_CSTRING(kAttestCertSubjectName, "CN=Firefox U2F Soft Token");
+constexpr auto kAttestCertSubjectName = "CN=Firefox U2F Soft Token"_ns;
 
 // This U2F-compatible soft token uses FIDO U2F-compatible ECDSA keypairs
 // on the SEC_OID_SECG_EC_SECP256R1 curve. When asked to Register, it will
@@ -47,7 +46,8 @@ const uint32_t kWrappedKeyBufLen = 256;
 const uint32_t kWrappingKeyByteLen = 128 / 8;
 const uint32_t kSaltByteLen = 64 / 8;
 const uint32_t kVersion1KeyHandleLen = 162;
-NS_NAMED_LITERAL_STRING(kEcAlgorithm, WEBCRYPTO_NAMED_CURVE_P256);
+constexpr auto kEcAlgorithm =
+    NS_LITERAL_STRING_FROM_CSTRING(WEBCRYPTO_NAMED_CURVE_P256);
 
 const PRTime kOneDay = PRTime(PR_USEC_PER_SEC) * PRTime(60)  // sec
                        * PRTime(60)                          // min
@@ -419,9 +419,11 @@ static UniqueSECItem KeyHandleFromPrivateKey(
 
   // It's OK to ignore the return values here because we're writing into
   // pre-allocated space
-  keyHandleBuf.AppendElement(SoftTokenHandle::Version1, mozilla::fallible);
-  keyHandleBuf.AppendElement(sizeof(saltParam), mozilla::fallible);
-  keyHandleBuf.AppendElements(saltParam, sizeof(saltParam), mozilla::fallible);
+  (void)keyHandleBuf.AppendElement(SoftTokenHandle::Version1,
+                                   mozilla::fallible);
+  (void)keyHandleBuf.AppendElement(sizeof(saltParam), mozilla::fallible);
+  (void)keyHandleBuf.AppendElements(saltParam, sizeof(saltParam),
+                                    mozilla::fallible);
   keyHandleBuf.AppendSECItem(wrappedKey.get());
 
   UniqueSECItem keyHandle(::SECITEM_AllocItem(nullptr, nullptr, 0));
@@ -566,7 +568,8 @@ nsresult U2FSoftTokenManager::IsRegistered(const nsTArray<uint8_t>& aKeyHandle,
 // *      attestation signature
 //
 RefPtr<U2FRegisterPromise> U2FSoftTokenManager::Register(
-    const WebAuthnMakeCredentialInfo& aInfo, bool aForceNoneAttestation) {
+    const WebAuthnMakeCredentialInfo& aInfo, bool aForceNoneAttestation,
+    void _ctap2_status_callback(rust_ctap2_status_update_res*)) {
   if (!mInitialized) {
     nsresult rv = Init();
     if (NS_WARN_IF(NS_FAILED(rv))) {
@@ -692,9 +695,9 @@ RefPtr<U2FRegisterPromise> U2FSoftTokenManager::Register(
 
   // // It's OK to ignore the return values here because we're writing into
   // // pre-allocated space
-  signedDataBuf.AppendElement(0x00, mozilla::fallible);
-  signedDataBuf.AppendElements(rpIdHash, mozilla::fallible);
-  signedDataBuf.AppendElements(clientDataHash, mozilla::fallible);
+  (void)signedDataBuf.AppendElement(0x00, mozilla::fallible);
+  (void)signedDataBuf.AppendElements(rpIdHash, mozilla::fallible);
+  (void)signedDataBuf.AppendElements(clientDataHash, mozilla::fallible);
   signedDataBuf.AppendSECItem(keyHandleItem.get());
   signedDataBuf.AppendSECItem(pubKey->u.ec.publicValue);
 
@@ -717,9 +720,9 @@ RefPtr<U2FRegisterPromise> U2FSoftTokenManager::Register(
     return U2FRegisterPromise::CreateAndReject(NS_ERROR_OUT_OF_MEMORY,
                                                __func__);
   }
-  registrationBuf.AppendElement(0x05, mozilla::fallible);
+  (void)registrationBuf.AppendElement(0x05, mozilla::fallible);
   registrationBuf.AppendSECItem(pubKey->u.ec.publicValue);
-  registrationBuf.AppendElement(keyHandleItem->len, mozilla::fallible);
+  (void)registrationBuf.AppendElement(keyHandleItem->len, mozilla::fallible);
   registrationBuf.AppendSECItem(keyHandleItem.get());
   registrationBuf.AppendSECItem(attestCert.get()->derCert);
   registrationBuf.AppendSECItem(signatureItem);
@@ -752,8 +755,10 @@ RefPtr<U2FRegisterPromise> U2FSoftTokenManager::Register(
     return U2FRegisterPromise::CreateAndReject(NS_ERROR_FAILURE, __func__);
   }
 
+  nsTArray<WebAuthnExtensionResult> extensions;
   WebAuthnMakeCredentialResult result(aInfo.ClientDataJSON(), attObj,
-                                      keyHandleBuf, registrationBuf);
+                                      keyHandleBuf, registrationBuf,
+                                      extensions);
   return U2FRegisterPromise::CreateAndResolve(std::move(result), __func__);
 }
 
@@ -794,7 +799,8 @@ bool U2FSoftTokenManager::FindRegisteredKeyHandle(
 //  *     Signature
 //
 RefPtr<U2FSignPromise> U2FSoftTokenManager::Sign(
-    const WebAuthnGetAssertionInfo& aInfo) {
+    const WebAuthnGetAssertionInfo& aInfo,
+    void _ctap2_status_callback(rust_ctap2_status_update_res*)) {
   if (!mInitialized) {
     nsresult rv = Init();
     if (NS_WARN_IF(NS_FAILED(rv))) {
@@ -811,7 +817,9 @@ RefPtr<U2FSignPromise> U2FSoftTokenManager::Sign(
   }
 
   nsTArray<nsTArray<uint8_t>> appIds;
-  appIds.AppendElement(rpIdHash);
+  appIds.AppendElement(std::move(rpIdHash));
+
+  Maybe<nsTArray<uint8_t>> appIdHashExt = Nothing();
 
   if (aInfo.Extra().isSome()) {
     const auto& extra = aInfo.Extra().ref();
@@ -828,7 +836,8 @@ RefPtr<U2FSignPromise> U2FSoftTokenManager::Sign(
     // Process extensions.
     for (const WebAuthnExtension& ext : extra.Extensions()) {
       if (ext.type() == WebAuthnExtension::TWebAuthnExtensionAppId) {
-        appIds.AppendElement(ext.get_WebAuthnExtensionAppId().AppId());
+        appIdHashExt = Some(ext.get_WebAuthnExtensionAppId().AppId().Clone());
+        appIds.AppendElement(appIdHashExt->Clone());
       }
     }
   }
@@ -891,12 +900,12 @@ RefPtr<U2FSignPromise> U2FSoftTokenManager::Sign(
 
   // It's OK to ignore the return values here because we're writing into
   // pre-allocated space
-  signedDataBuf.AppendElements(chosenAppId.Elements(), chosenAppId.Length(),
-                               mozilla::fallible);
-  signedDataBuf.AppendElement(0x01, mozilla::fallible);
+  (void)signedDataBuf.AppendElements(chosenAppId.Elements(),
+                                     chosenAppId.Length(), mozilla::fallible);
+  (void)signedDataBuf.AppendElement(0x01, mozilla::fallible);
   signedDataBuf.AppendSECItem(counterItem);
-  signedDataBuf.AppendElements(clientDataHash.Elements(),
-                               clientDataHash.Length(), mozilla::fallible);
+  (void)signedDataBuf.AppendElements(
+      clientDataHash.Elements(), clientDataHash.Length(), mozilla::fallible);
 
   if (MOZ_LOG_TEST(gNSSTokenLog, LogLevel::Debug)) {
     nsAutoCString base64;
@@ -930,15 +939,15 @@ RefPtr<U2FSignPromise> U2FSoftTokenManager::Sign(
 
   // It's OK to ignore the return values here because we're writing into
   // pre-allocated space
-  signatureDataBuf.AppendElement(0x01, mozilla::fallible);
+  (void)signatureDataBuf.AppendElement(0x01, mozilla::fallible);
   signatureDataBuf.AppendSECItem(counterItem);
   signatureDataBuf.AppendSECItem(signatureItem);
 
   nsTArray<WebAuthnExtensionResult> extensions;
 
-  if (chosenAppId != rpIdHash) {
-    // Indicate to the RP that we used the FIDO appId.
-    extensions.AppendElement(WebAuthnExtensionResultAppId(true));
+  if (appIdHashExt) {
+    bool usedAppId = (chosenAppId == appIdHashExt.ref());
+    extensions.AppendElement(WebAuthnExtensionResultAppId(usedAppId));
   }
 
   CryptoBuffer counterBuf;
@@ -969,12 +978,13 @@ RefPtr<U2FSignPromise> U2FSoftTokenManager::Sign(
   WebAuthnGetAssertionResult result(aInfo.ClientDataJSON(), keyHandle,
                                     signatureBuf, authenticatorData, extensions,
                                     signatureDataBuf, userHandle);
-  return U2FSignPromise::CreateAndResolve(std::move(result), __func__);
+  nsTArray<WebAuthnGetAssertionResultWrapper> results = {
+      {result, mozilla::Nothing()}};
+  return U2FSignPromise::CreateAndResolve(std::move(results), __func__);
 }
 
 void U2FSoftTokenManager::Cancel() {
   // This implementation is sync, requests can't be aborted.
 }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
