@@ -3,9 +3,10 @@
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
-
+#include "json/json.h"
 #include "mozilla/dom/U2FTokenManager.h"
 #include "mozilla/dom/U2FTokenTransport.h"
+#include "mozilla/dom/CTAPHIDTokenManager.h"
 #include "mozilla/dom/U2FHIDTokenManager.h"
 #include "mozilla/dom/U2FSoftTokenManager.h"
 #include "mozilla/dom/PWebAuthnTransactionParent.h"
@@ -16,6 +17,7 @@
 #include "mozilla/Preferences.h"
 #include "mozilla/Services.h"
 #include "mozilla/Unused.h"
+#include "nsEscape.h"
 #include "nsIObserver.h"
 #include "nsIObserverService.h"
 #include "nsIThread.h"
@@ -38,8 +40,8 @@
   "security.webauth.webauthn_testing_allow_direct_attestation"
 #define PREF_WEBAUTHN_ANDROID_FIDO2_ENABLED \
   "security.webauth.webauthn_enable_android_fido2"
-namespace mozilla {
-namespace dom {
+#define PREF_WEBAUTHN_CTAP2 "security.webauthn.ctap2"
+namespace mozilla::dom {
 
 /***********************************************************************
  * Statics
@@ -57,16 +59,27 @@ static nsIThread* gBackgroundThread;
 // Data for WebAuthn UI prompt notifications.
 static const char16_t kRegisterPromptNotifcation[] =
     u"{\"action\":\"register\",\"tid\":%llu,\"origin\":\"%s\","
-    u"\"browsingContextId\":%llu}";
+    u"\"browsingContextId\":%llu,\"is_ctap2\":%s, \"device_selected\":%s}";
 static const char16_t kRegisterDirectPromptNotifcation[] =
     u"{\"action\":\"register-direct\",\"tid\":%llu,\"origin\":\"%s\","
     u"\"browsingContextId\":%llu}";
 static const char16_t kSignPromptNotifcation[] =
     u"{\"action\":\"sign\",\"tid\":%llu,\"origin\":\"%s\","
-    u"\"browsingContextId\":%"
-    u"llu}";
+    u"\"browsingContextId\":%llu,\"is_ctap2\":%s, \"device_selected\":%s}";
 static const char16_t kCancelPromptNotifcation[] =
     u"{\"action\":\"cancel\",\"tid\":%llu}";
+static const char16_t kPinRequiredNotifcation[] =
+    u"{\"action\":\"pin-required\",\"tid\":%llu,\"origin\":\"%s\","
+    u"\"browsingContextId\":%llu,\"wasInvalid\":%s,\"retriesLeft\":%i}";
+static const char16_t kSelectDeviceNotifcation[] =
+    u"{\"action\":\"select-device\",\"tid\":%llu,\"origin\":\"%s\","
+    u"\"browsingContextId\":%llu}";
+static const char16_t kSelectSignResultNotifcation[] =
+    u"{\"action\":\"select-sign-result\",\"tid\":%llu,\"origin\":\"%s\","
+    u"\"browsingContextId\":%llu,\"usernames\":[%s]}";
+static const char16_t kPinErrorNotifications[] =
+    u"{\"action\":\"%s\",\"tid\":%llu,\"origin\":\"%s\","
+    u"\"browsingContextId\":%llu}";
 
 class U2FPrefManager final : public nsIObserver {
  private:
@@ -89,6 +102,7 @@ class U2FPrefManager final : public nsIObserver {
                                      PREF_WEBAUTHN_ANDROID_FIDO2_ENABLED);
       Preferences::AddStrongObserver(gPrefManager,
                                      PREF_WEBAUTHN_ALLOW_DIRECT_ATTESTATION);
+      Preferences::AddStrongObserver(gPrefManager, PREF_WEBAUTHN_CTAP2);
       ClearOnShutdown(&gPrefManager, ShutdownPhase::XPCOMShutdownThreads);
     }
     return gPrefManager;
@@ -121,6 +135,11 @@ class U2FPrefManager final : public nsIObserver {
     return mAllowDirectAttestation;
   }
 
+  bool GetIsCtap2() {
+    MutexAutoLock lock(mPrefMutex);
+    return mCtap2;
+  }
+
   NS_IMETHODIMP
   Observe(nsISupports* aSubject, const char* aTopic,
           const char16_t* aData) override {
@@ -139,6 +158,7 @@ class U2FPrefManager final : public nsIObserver {
         Preferences::GetBool(PREF_WEBAUTHN_ANDROID_FIDO2_ENABLED);
     mAllowDirectAttestation =
         Preferences::GetBool(PREF_WEBAUTHN_ALLOW_DIRECT_ATTESTATION);
+    mCtap2 = Preferences::GetBool(PREF_WEBAUTHN_CTAP2);
   }
 
   Mutex mPrefMutex MOZ_UNANNOTATED;
@@ -147,6 +167,7 @@ class U2FPrefManager final : public nsIObserver {
   bool mUsbTokenEnabled;
   bool mAndroidFido2Enabled;
   bool mAllowDirectAttestation;
+  bool mCtap2;
 };
 
 NS_IMPL_ISUPPORTS(U2FPrefManager, nsIObserver);
@@ -186,9 +207,10 @@ U2FTokenManager* U2FTokenManager::Get() {
 }
 
 void U2FTokenManager::AbortTransaction(const uint64_t& aTransactionId,
-                                       const nsresult& aError) {
+                                       const nsresult& aError,
+                                       bool shouldCancelActiveDialog) {
   Unused << mTransactionParent->SendAbort(aTransactionId, aError);
-  ClearTransaction();
+  ClearTransaction(shouldCancelActiveDialog);
 }
 
 void U2FTokenManager::AbortOngoingTransaction() {
@@ -197,7 +219,7 @@ void U2FTokenManager::AbortOngoingTransaction() {
     Unused << mTransactionParent->SendAbort(mLastTransactionId,
                                             NS_ERROR_DOM_ABORT_ERR);
   }
-  ClearTransaction();
+  ClearTransaction(true);
 }
 
 void U2FTokenManager::MaybeClearTransaction(
@@ -205,18 +227,21 @@ void U2FTokenManager::MaybeClearTransaction(
   // Only clear if we've been requested to do so by our current transaction
   // parent.
   if (mTransactionParent == aParent) {
-    ClearTransaction();
+    ClearTransaction(true);
   }
 }
 
-void U2FTokenManager::ClearTransaction() {
-  if (mLastTransactionId) {
+void U2FTokenManager::ClearTransaction(bool send_cancel) {
+  if (mLastTransactionId && send_cancel) {
     // Remove any prompts we might be showing for the current transaction.
     SendPromptNotification(kCancelPromptNotifcation, mLastTransactionId);
   }
 
   mTransactionParent = nullptr;
 
+  // We have to "hang up" in case auth-rs is still waiting for us to send a PIN
+  // so it can exit cleanly
+  status_update_result = nullptr;
   // Drop managers at the end of all transactions
   if (mTokenManagerImpl) {
     mTokenManagerImpl->Drop();
@@ -232,6 +257,8 @@ void U2FTokenManager::ClearTransaction() {
 
   // Forget any pending registration.
   mPendingRegisterInfo.reset();
+  mPendingSignInfo.reset();
+  mPendingSignResults.Clear();
 }
 
 template <typename... T>
@@ -250,7 +277,7 @@ void U2FTokenManager::SendPromptNotification(const char16_t* aFormat,
       GetMainThreadEventTarget()->Dispatch(r.forget(), NS_DISPATCH_NORMAL));
 }
 
-void U2FTokenManager::RunSendPromptNotification(nsString aJSON) {
+void U2FTokenManager::RunSendPromptNotification(const nsString& aJSON) {
   MOZ_ASSERT(NS_IsMainThread());
 
   nsCOMPtr<nsIObserverService> os = services::GetObserverService();
@@ -263,6 +290,111 @@ void U2FTokenManager::RunSendPromptNotification(nsString aJSON) {
       os->NotifyObservers(self, "webauthn-prompt", aJSON.get()));
 }
 
+void U2FTokenManager::StatusUpdateResFreePolicy::operator()(
+    rust_ctap2_status_update_res* p) {
+  rust_ctap2_destroy_status_update_res(p);
+}
+
+static void status_callback(rust_ctap2_status_update_res* status) {
+  if (!U2FPrefManager::Get()->GetIsCtap2() || (NS_WARN_IF(!status))) {
+    return;
+  }
+
+  // The result will be cleared automatically upon exiting this function,
+  // unless we have a Pin error, then we need it for a callback from JS.
+  // Then we move ownership of it to U2FTokenManager.
+  UniquePtr<rust_ctap2_status_update_res,
+            U2FTokenManager::StatusUpdateResFreePolicy>
+      status_result_update(status);
+  size_t len;
+  if (NS_WARN_IF(!rust_ctap2_status_update_len(status, &len))) {
+    return;
+  }
+  nsCString st;
+  if (NS_WARN_IF(!st.SetLength(len, fallible))) {
+    return;
+  }
+  if (NS_WARN_IF(!rust_ctap2_status_update_copy_json(status, st.Data()))) {
+    return;
+  }
+
+  auto* gInstance = U2FTokenManager::Get();
+
+  Json::Value jsonRoot;
+  Json::Reader reader;
+  if (NS_WARN_IF(!reader.parse(st.Data(), jsonRoot))) {
+    return;
+  }
+  if (NS_WARN_IF(!jsonRoot.isObject())) {
+    return;
+  }
+
+  nsAutoString notification_json;
+
+  uint64_t browsingCtxId = gInstance->GetCurrentBrowsingCtxId().value();
+  NS_ConvertUTF16toUTF8 origin(gInstance->GetCurrentOrigin().value());
+  if (jsonRoot.isMember("PinError")) {
+    uint64_t tid = gInstance->GetCurrentTransactionId();
+    bool pinRequired = (jsonRoot["PinError"].isString() &&
+                        jsonRoot["PinError"].asString() == "PinRequired");
+    bool pinInvalid = (jsonRoot["PinError"].isObject() &&
+                       jsonRoot["PinError"].isMember("InvalidPin"));
+    if (pinRequired || pinInvalid) {
+      bool wasInvalid = false;
+      int retries = -1;
+      if (pinInvalid) {
+        wasInvalid = true;
+        if (jsonRoot["PinError"]["InvalidPin"].isInt()) {
+          retries = jsonRoot["PinError"]["InvalidPin"].asInt();
+        }
+      }
+      gInstance->status_update_result = std::move(status_result_update);
+      nsTextFormatter::ssprintf(notification_json, kPinRequiredNotifcation, tid,
+                                origin.get(), browsingCtxId,
+                                wasInvalid ? "true" : "false", retries);
+    } else if (jsonRoot["PinError"].isString()) {
+      // Not saving the status_result, so the callback will error out and cancel
+      // the transaction, because these errors are not recoverable by
+      // user-input.
+      if (jsonRoot["PinError"].asString() == "PinAuthBlocked") {
+        // Pin authentication blocked. Device needs power cycle!
+        nsTextFormatter::ssprintf(notification_json, kPinErrorNotifications,
+                                  "pin-auth-blocked", tid, origin.get(),
+                                  browsingCtxId);
+      } else if (jsonRoot["PinError"].asString() == "PinBlocked") {
+        // No retries left. Pin blocked. Device needs reset!
+        nsTextFormatter::ssprintf(notification_json, kPinErrorNotifications,
+                                  "device-blocked", tid, origin.get(),
+                                  browsingCtxId);
+      }
+    }
+  } else if (jsonRoot.isMember("SelectDeviceNotice")) {
+    nsTextFormatter::ssprintf(notification_json, kSelectDeviceNotifcation,
+                              gInstance->GetCurrentTransactionId(),
+                              origin.get(), browsingCtxId);
+  } else if (jsonRoot.isMember("DeviceSelected")) {
+    if (gInstance->CurrentTransactionIsRegister()) {
+      nsTextFormatter::ssprintf(notification_json, kRegisterPromptNotifcation,
+                                gInstance->GetCurrentTransactionId(),
+                                origin.get(), browsingCtxId, "true", "true");
+    } else if (gInstance->CurrentTransactionIsSign()) {
+      nsTextFormatter::ssprintf(notification_json, kSignPromptNotifcation,
+                                gInstance->GetCurrentTransactionId(),
+                                origin.get(), browsingCtxId, "true", "true");
+    }
+  } else {
+    // No-op for now
+  }
+
+  if (!notification_json.IsEmpty()) {
+    nsCOMPtr<nsIRunnable> r(NewRunnableMethod<nsString>(
+        "U2FTokenManager::RunSendPromptNotification", gInstance,
+        &U2FTokenManager::RunSendPromptNotification, notification_json));
+    MOZ_ALWAYS_SUCCEEDS(
+        GetMainThreadEventTarget()->Dispatch(r.forget(), NS_DISPATCH_NORMAL));
+  }
+}
+
 RefPtr<U2FTokenTransport> U2FTokenManager::GetTokenManagerImpl() {
   MOZ_ASSERT(U2FPrefManager::Get());
   mozilla::ipc::AssertIsOnBackgroundThread();
@@ -290,7 +422,13 @@ RefPtr<U2FTokenTransport> U2FTokenManager::GetTokenManagerImpl() {
   // same time as the softtoken would always win the race to register.
   // We could support it for signing though...
   if (pm->GetUsbTokenEnabled()) {
-    return new U2FHIDTokenManager();
+    U2FTokenTransport* manager;
+    if (U2FPrefManager::Get()->GetIsCtap2()) {
+      manager = new CTAPHIDTokenManager();
+    } else {
+      manager = new U2FHIDTokenManager();
+    }
+    return manager;
   }
 
   if (pm->GetSoftTokenEnabled()) {
@@ -314,7 +452,7 @@ void U2FTokenManager::Register(
   mTokenManagerImpl = GetTokenManagerImpl();
 
   if (!mTokenManagerImpl) {
-    AbortTransaction(aTransactionId, NS_ERROR_DOM_NOT_ALLOWED_ERR);
+    AbortTransaction(aTransactionId, NS_ERROR_DOM_NOT_ALLOWED_ERR, true);
     return;
   }
 
@@ -341,6 +479,8 @@ void U2FTokenManager::Register(
   // wasn't requested or the test pref is set.
   if (noneAttestationRequested ||
       U2FPrefManager::Get()->GetAllowDirectAttestationForTesting()) {
+    MOZ_ASSERT(mPendingRegisterInfo.isNothing());
+    mPendingRegisterInfo = Some(aTransactionInfo);
     DoRegister(aTransactionInfo, noneAttestationRequested);
     return;
   }
@@ -362,13 +502,15 @@ void U2FTokenManager::DoRegister(const WebAuthnMakeCredentialInfo& aInfo,
 
   // Show a prompt that lets the user cancel the ongoing transaction.
   NS_ConvertUTF16toUTF8 origin(aInfo.Origin());
+  const char* is_ctap2 = U2FPrefManager::Get()->GetIsCtap2() ? "true" : "false";
   SendPromptNotification(kRegisterPromptNotifcation, mLastTransactionId,
-                         origin.get(), aInfo.BrowsingContextId());
+                         origin.get(), aInfo.BrowsingContextId(), is_ctap2,
+                         "false");
 
   uint64_t tid = mLastTransactionId;
   mozilla::TimeStamp startTime = mozilla::TimeStamp::Now();
 
-  mTokenManagerImpl->Register(aInfo, aForceNoneAttestation)
+  mTokenManagerImpl->Register(aInfo, aForceNoneAttestation, status_callback)
       ->Then(
           GetCurrentSerialEventTarget(), __func__,
           [tid, startTime](WebAuthnMakeCredentialResult&& aResult) {
@@ -382,7 +524,12 @@ void U2FTokenManager::DoRegister(const WebAuthnMakeCredentialInfo& aInfo,
           [tid](nsresult rv) {
             MOZ_ASSERT(NS_FAILED(rv));
             U2FTokenManager* mgr = U2FTokenManager::Get();
-            mgr->MaybeAbortRegister(tid, rv);
+            bool shouldCancelActiveDialog = true;
+            if (rv == NS_ERROR_DOM_OPERATION_ERR) {
+              // PIN-related errors. Let the dialog show to inform the user
+              shouldCancelActiveDialog = false;
+            }
+            mgr->MaybeAbortRegister(tid, rv, shouldCancelActiveDialog);
             Telemetry::ScalarAdd(Telemetry::ScalarID::SECURITY_WEBAUTHN_USED,
                                  u"U2FRegisterAbort"_ns, 1);
           })
@@ -396,14 +543,15 @@ void U2FTokenManager::MaybeConfirmRegister(
   mRegisterPromise.Complete();
 
   Unused << mTransactionParent->SendConfirmRegister(aTransactionId, aResult);
-  ClearTransaction();
+  ClearTransaction(true);
 }
 
 void U2FTokenManager::MaybeAbortRegister(const uint64_t& aTransactionId,
-                                         const nsresult& aError) {
+                                         const nsresult& aError,
+                                         bool shouldCancelActiveDialog) {
   MOZ_ASSERT(mLastTransactionId == aTransactionId);
   mRegisterPromise.Complete();
-  AbortTransaction(aTransactionId, aError);
+  AbortTransaction(aTransactionId, aError, shouldCancelActiveDialog);
 }
 
 void U2FTokenManager::Sign(PWebAuthnTransactionParent* aTransactionParent,
@@ -416,24 +564,56 @@ void U2FTokenManager::Sign(PWebAuthnTransactionParent* aTransactionParent,
   mTokenManagerImpl = GetTokenManagerImpl();
 
   if (!mTokenManagerImpl) {
-    AbortTransaction(aTransactionId, NS_ERROR_DOM_NOT_ALLOWED_ERR);
+    AbortTransaction(aTransactionId, NS_ERROR_DOM_NOT_ALLOWED_ERR, true);
     return;
   }
 
+  mLastTransactionId = aTransactionId;
+  mPendingSignInfo = Some(aTransactionInfo);
+  DoSign(aTransactionInfo);
+}
+
+void U2FTokenManager::DoSign(const WebAuthnGetAssertionInfo& aTransactionInfo) {
+  mozilla::ipc::AssertIsOnBackgroundThread();
+  MOZ_ASSERT(mLastTransactionId > 0);
+  uint64_t tid = mLastTransactionId;
+
+  NS_ConvertUTF16toUTF8 origin(aTransactionInfo.Origin());
+  uint64_t browserCtxId = aTransactionInfo.BrowsingContextId();
+
   // Show a prompt that lets the user cancel the ongoing transaction.
-  NS_ConvertUTF16toUTF8 origin(aTransactionInfo.Origin());
-  SendPromptNotification(kSignPromptNotifcation, aTransactionId, origin.get(),
-                         aTransactionInfo.BrowsingContextId());
+  const char* is_ctap2 = U2FPrefManager::Get()->GetIsCtap2() ? "true" : "false";
+  SendPromptNotification(kSignPromptNotifcation, tid, origin.get(),
+                         browserCtxId, is_ctap2, "false");
 
-  uint64_t tid = mLastTransactionId = aTransactionId;
   mozilla::TimeStamp startTime = mozilla::TimeStamp::Now();
 
-  mTokenManagerImpl->Sign(aTransactionInfo)
+  mTokenManagerImpl->Sign(aTransactionInfo, status_callback)
       ->Then(
           GetCurrentSerialEventTarget(), __func__,
-          [tid, startTime](WebAuthnGetAssertionResult&& aResult) {
+          [tid, origin, startTime, browserCtxId](
+              nsTArray<WebAuthnGetAssertionResultWrapper>&& aResult) {
             U2FTokenManager* mgr = U2FTokenManager::Get();
-            mgr->MaybeConfirmSign(tid, aResult);
+            if (aResult.Length() == 1) {
+              WebAuthnGetAssertionResult result = aResult[0].assertion;
+              mgr->MaybeConfirmSign(tid, result);
+            } else {
+              nsCString res;
+              StringJoinAppend(
+                  res, ","_ns, aResult,
+                  [](nsACString& dst,
+                     const WebAuthnGetAssertionResultWrapper& assertion) {
+                    nsCString username =
+                        assertion.username.valueOr("<Unknown username>"_ns);
+                    nsCString escaped_username;
+                    NS_Escape(username, escaped_username, url_XAlphas);
+                    dst.Append("\""_ns + escaped_username + "\""_ns);
+                  });
+              mgr->mPendingSignResults.Assign(aResult);
+              mgr->SendPromptNotification(kSelectSignResultNotifcation, tid,
+                                          origin.get(), browserCtxId,
+                                          res.get());
+            }
             Telemetry::ScalarAdd(Telemetry::ScalarID::SECURITY_WEBAUTHN_USED,
                                  u"U2FSignFinish"_ns, 1);
             Telemetry::AccumulateTimeDelta(Telemetry::WEBAUTHN_GET_ASSERTION_MS,
@@ -442,7 +622,12 @@ void U2FTokenManager::Sign(PWebAuthnTransactionParent* aTransactionParent,
           [tid](nsresult rv) {
             MOZ_ASSERT(NS_FAILED(rv));
             U2FTokenManager* mgr = U2FTokenManager::Get();
-            mgr->MaybeAbortSign(tid, rv);
+            bool shouldCancelActiveDialog = true;
+            if (rv == NS_ERROR_DOM_OPERATION_ERR) {
+              // PIN-related errors. Let the dialog show to inform the user
+              shouldCancelActiveDialog = false;
+            }
+            mgr->MaybeAbortSign(tid, rv, shouldCancelActiveDialog);
             Telemetry::ScalarAdd(Telemetry::ScalarID::SECURITY_WEBAUTHN_USED,
                                  u"U2FSignAbort"_ns, 1);
           })
@@ -455,14 +640,15 @@ void U2FTokenManager::MaybeConfirmSign(
   mSignPromise.Complete();
 
   Unused << mTransactionParent->SendConfirmSign(aTransactionId, aResult);
-  ClearTransaction();
+  ClearTransaction(true);
 }
 
 void U2FTokenManager::MaybeAbortSign(const uint64_t& aTransactionId,
-                                     const nsresult& aError) {
+                                     const nsresult& aError,
+                                     bool shouldCancelActiveDialog) {
   MOZ_ASSERT(mLastTransactionId == aTransactionId);
   mSignPromise.Complete();
-  AbortTransaction(aTransactionId, aError);
+  AbortTransaction(aTransactionId, aError, shouldCancelActiveDialog);
 }
 
 void U2FTokenManager::Cancel(PWebAuthnTransactionParent* aParent,
@@ -476,7 +662,7 @@ void U2FTokenManager::Cancel(PWebAuthnTransactionParent* aParent,
   }
 
   mTokenManagerImpl->Cancel();
-  ClearTransaction();
+  ClearTransaction(true);
 }
 
 // nsIU2FTokenManager
@@ -499,6 +685,66 @@ U2FTokenManager::ResumeRegister(uint64_t aTransactionId,
   return gBackgroundThread->Dispatch(r.forget(), NS_DISPATCH_NORMAL);
 }
 
+NS_IMETHODIMP
+U2FTokenManager::ResumeWithSelectedSignResult(uint64_t aTransactionId,
+                                              uint64_t idx) {
+  MOZ_ASSERT(XRE_IsParentProcess());
+  MOZ_ASSERT(NS_IsMainThread());
+
+  if (!gBackgroundThread) {
+    return NS_ERROR_FAILURE;
+  }
+
+  nsCOMPtr<nsIRunnable> r(NewRunnableMethod<uint64_t, uint64_t>(
+      "U2FTokenManager::RunResumeWithSelectedSignResult", this,
+      &U2FTokenManager::RunResumeWithSelectedSignResult, aTransactionId, idx));
+
+  return gBackgroundThread->Dispatch(r.forget(), NS_DISPATCH_NORMAL);
+}
+
+void U2FTokenManager::RunResumeWithSelectedSignResult(uint64_t aTransactionId,
+                                                      uint64_t idx) {
+  mozilla::ipc::AssertIsOnBackgroundThread();
+  if (NS_WARN_IF(mPendingSignResults.IsEmpty())) {
+    return;
+  }
+
+  if (NS_WARN_IF(mPendingSignResults.Length() <= idx)) {
+    return;
+  }
+
+  if (mLastTransactionId != aTransactionId) {
+    return;
+  }
+
+  WebAuthnGetAssertionResult result = mPendingSignResults[idx].assertion;
+  MaybeConfirmSign(aTransactionId, result);
+}
+
+NS_IMETHODIMP
+U2FTokenManager::PinCallback(const nsACString& aPin) {
+  if (!U2FPrefManager::Get()->GetIsCtap2()) {
+    // Not used in CTAP1
+    return NS_ERROR_FAILURE;
+  }
+  MOZ_ASSERT(XRE_IsParentProcess());
+  MOZ_ASSERT(NS_IsMainThread());
+  if (!gBackgroundThread) {
+    return NS_ERROR_FAILURE;
+  }
+  // Move the result here locally, so it will be freed either way
+  UniquePtr<rust_ctap2_status_update_res,
+            U2FTokenManager::StatusUpdateResFreePolicy>
+      result = nullptr;
+  std::swap(result, status_update_result);
+
+  if (NS_WARN_IF(
+          !rust_ctap2_status_update_send_pin(result.get(), aPin.Data()))) {
+    return NS_ERROR_FAILURE;
+  }
+  return NS_OK;
+}
+
 void U2FTokenManager::RunResumeRegister(uint64_t aTransactionId,
                                         bool aForceNoneAttestation) {
   mozilla::ipc::AssertIsOnBackgroundThread();
@@ -513,7 +759,21 @@ void U2FTokenManager::RunResumeRegister(uint64_t aTransactionId,
 
   // Resume registration and cleanup.
   DoRegister(mPendingRegisterInfo.ref(), aForceNoneAttestation);
-  mPendingRegisterInfo.reset();
+}
+
+void U2FTokenManager::RunResumeSign(uint64_t aTransactionId) {
+  mozilla::ipc::AssertIsOnBackgroundThread();
+
+  if (NS_WARN_IF(mPendingSignInfo.isNothing())) {
+    return;
+  }
+
+  if (mLastTransactionId != aTransactionId) {
+    return;
+  }
+
+  // Resume sign and cleanup.
+  DoSign(mPendingSignInfo.ref());
 }
 
 NS_IMETHODIMP
@@ -539,12 +799,14 @@ void U2FTokenManager::RunCancel(uint64_t aTransactionId) {
     return;
   }
 
+  // We have to "hang up" in case auth-rs is still waiting for us to send a PIN
+  // so it can exit cleanly
+  status_update_result = nullptr;
   // Cancel the request.
   mTokenManagerImpl->Cancel();
 
   // Reject the promise.
-  AbortTransaction(aTransactionId, NS_ERROR_DOM_ABORT_ERR);
+  AbortTransaction(aTransactionId, NS_ERROR_DOM_NOT_ALLOWED_ERR, true);
 }
 
-}  // namespace dom
-}  // namespace mozilla
+}  // namespace mozilla::dom
