@@ -6,7 +6,6 @@ const flag_TUP = 0x01;
 const flag_UV = 0x04;
 const flag_AT = 0x40;
 
-
 const cose_kty = 1;
 const cose_kty_ec2 = 2;
 const cose_alg = 3;
@@ -17,7 +16,9 @@ const cose_crv_P256 = 1;
 const cose_crv_x = -2;
 const cose_crv_y = -3;
 
-var {AppConstants} = SpecialPowers.Cu.import("resource://gre/modules/AppConstants.jsm", {});
+var { AppConstants } = SpecialPowers.ChromeUtils.import(
+  "resource://gre/modules/AppConstants.jsm"
+);
 
 function handleEventMessage(event) {
   if ("test" in event.data) {
@@ -27,12 +28,12 @@ function handleEventMessage(event) {
   } else if ("done" in event.data) {
     SimpleTest.finish();
   } else {
-    ok(false, "Unexpected message in the test harness: " + event.data)
+    ok(false, "Unexpected message in the test harness: " + event.data);
   }
 }
 
 function log(msg) {
-  console.log(msg)
+  console.log(msg);
   let logBox = document.getElementById("log");
   if (logBox) {
     logBox.textContent += "\n" + msg;
@@ -56,7 +57,7 @@ function local_isnot(value, expected, message) {
 }
 
 function local_ok(expression, message) {
-  let body = {"test": this.location.pathname, "status":expression, "msg": message}
+  let body = { test: this.location.pathname, status: expression, msg: message };
   parent.postMessage(body, "http://mochi.test:8888");
 }
 
@@ -64,34 +65,42 @@ function local_doesThrow(fn, name) {
   let gotException = false;
   try {
     fn();
-  } catch (ex) { gotException = true; }
+  } catch (ex) {
+    gotException = true;
+  }
   local_ok(gotException, name);
-};
+}
 
 function local_expectThisManyTests(count) {
   if (_expectedCompletions > 0) {
-    local_ok(false, "Error: local_expectThisManyTests should only be called once.");
+    local_ok(
+      false,
+      "Error: local_expectThisManyTests should only be called once."
+    );
   }
   _expectedCompletions = count;
 }
 
 function local_completeTest() {
-  _countCompletions += 1
+  _countCompletions += 1;
   if (_countCompletions == _expectedCompletions) {
-    log("All tests completed.")
+    log("All tests completed.");
     local_finished();
   }
   if (_countCompletions > _expectedCompletions) {
-    local_ok(false, "Error: local_completeTest called more than local_addTest.");
+    local_ok(
+      false,
+      "Error: local_completeTest called more than local_addTest."
+    );
   }
 }
 
 function local_finished() {
-  parent.postMessage({"done":true}, "http://mochi.test:8888");
+  parent.postMessage({ done: true }, "http://mochi.test:8888");
 }
 
 function string2buffer(str) {
-  return (new Uint8Array(str.length)).map((x, i) => str.charCodeAt(i));
+  return new Uint8Array(str.length).map((x, i) => str.charCodeAt(i));
 }
 
 function buffer2string(buf) {
@@ -99,11 +108,13 @@ function buffer2string(buf) {
   if (!(buf.constructor === Uint8Array)) {
     buf = new Uint8Array(buf);
   }
-  buf.map(function(x){ return str += String.fromCharCode(x) });
+  buf.map(function(x) {
+    return (str += String.fromCharCode(x));
+  });
   return str;
 }
 
-function bytesToBase64(u8a){
+function bytesToBase64(u8a) {
   let CHUNK_SZ = 0x8000;
   let c = [];
   let array = new Uint8Array(u8a);
@@ -114,16 +125,21 @@ function bytesToBase64(u8a){
 }
 
 function base64ToBytes(b64encoded) {
-  return new Uint8Array(window.atob(b64encoded).split("").map(function(c) {
-    return c.charCodeAt(0);
-  }));
+  return new Uint8Array(
+    window
+      .atob(b64encoded)
+      .split("")
+      .map(function(c) {
+        return c.charCodeAt(0);
+      })
+  );
 }
 
 function bytesToBase64UrlSafe(buf) {
   return bytesToBase64(buf)
-                 .replace(/\+/g, "-")
-                 .replace(/\//g, "_")
-                 .replace(/=/g, "");
+    .replace(/\+/g, "-")
+    .replace(/\//g, "_")
+    .replace(/=/g, "");
 }
 
 function base64ToBytesUrlSafe(str) {
@@ -140,8 +156,8 @@ function base64ToBytesUrlSafe(str) {
 
 function hexEncode(buf) {
   return Array.from(buf)
-              .map(x => ("0"+x.toString(16)).substr(-2))
-              .join("");
+    .map(x => ("0" + x.toString(16)).substr(-2))
+    .join("");
 }
 
 function hexDecode(str) {
@@ -150,8 +166,7 @@ function hexDecode(str) {
 
 function hasOnlyKeys(obj, ...keys) {
   let okeys = new Set(Object.keys(obj));
-  return keys.length == okeys.size &&
-         keys.every(k => okeys.has(k));
+  return keys.length == okeys.size && keys.every(k => okeys.has(k));
 }
 
 function webAuthnDecodeCBORAttestation(aCborAttBuf) {
@@ -163,15 +178,16 @@ function webAuthnDecodeCBORAttestation(aCborAttBuf) {
   if (attObj.fmt == "fido-u2f" && !hasOnlyKeys(attObj.attStmt, "sig", "x5c")) {
     return Promise.reject("Invalid CBOR Attestation Statement");
   }
-  if (attObj.fmt == "none" && Object.keys(attObj.attStmt).length > 0) {
+  if (attObj.fmt == "none" && Object.keys(attObj.attStmt).length) {
     return Promise.reject("Invalid CBOR Attestation Statement");
   }
 
-  return webAuthnDecodeAuthDataArray(new Uint8Array(attObj.authData))
-  .then(function (aAuthDataObj) {
-    attObj.authDataObj = aAuthDataObj;
-    return Promise.resolve(attObj);
-  });
+  return webAuthnDecodeAuthDataArray(new Uint8Array(attObj.authData)).then(
+    function(aAuthDataObj) {
+      attObj.authDataObj = aAuthDataObj;
+      return Promise.resolve(attObj);
+    }
+  );
 }
 
 function webAuthnDecodeAuthDataArray(aAuthData) {
@@ -186,14 +202,16 @@ function webAuthnDecodeAuthDataArray(aAuthData) {
   if ((flags & flag_AT) == 0x00) {
     // No Attestation Data, so we're done.
     return Promise.resolve({
-      rpIdHash: rpIdHash,
-      flags: flags,
-      counter: counter,
+      rpIdHash,
+      flags,
+      counter,
     });
   }
 
   if (aAuthData.length < 38) {
-    return Promise.reject("Authenticator Data flag was set, but not enough data passed in!");
+    return Promise.reject(
+      "Authenticator Data flag was set, but not enough data passed in!"
+    );
   }
 
   let attData = {};
@@ -206,8 +224,15 @@ function webAuthnDecodeAuthDataArray(aAuthData) {
 
   let cborPubKey = aAuthData.slice(55 + attData.credIdLen);
   var pubkeyObj = CBOR.decode(cborPubKey.buffer);
-  if (!(cose_kty in pubkeyObj && cose_alg in pubkeyObj && cose_crv in pubkeyObj
-        && cose_crv_x in pubkeyObj && cose_crv_y in pubkeyObj)) {
+  if (
+    !(
+      cose_kty in pubkeyObj &&
+      cose_alg in pubkeyObj &&
+      cose_crv in pubkeyObj &&
+      cose_crv_x in pubkeyObj &&
+      cose_crv_y in pubkeyObj
+    )
+  ) {
     throw "Invalid CBOR Public Key Object";
   }
   if (pubkeyObj[cose_kty] != cose_kty_ec2) {
@@ -220,7 +245,10 @@ function webAuthnDecodeAuthDataArray(aAuthData) {
     throw "Unexpected curve";
   }
 
-  let pubKeyBytes = assemblePublicKeyBytesData(pubkeyObj[cose_crv_x], pubkeyObj[cose_crv_y]);
+  let pubKeyBytes = assemblePublicKeyBytesData(
+    pubkeyObj[cose_crv_x],
+    pubkeyObj[cose_crv_y]
+  );
   console.log(":: CBOR Public Key Object Data ::");
   console.log("kty: " + pubkeyObj[cose_kty] + " (EC2)");
   console.log("alg: " + pubkeyObj[cose_alg] + " (ES256)");
@@ -229,12 +257,11 @@ function webAuthnDecodeAuthDataArray(aAuthData) {
   console.log("Y: " + pubkeyObj[cose_crv_y]);
   console.log("Uncompressed (hex): " + hexEncode(pubKeyBytes));
 
-  return importPublicKey(pubKeyBytes)
-  .then(function(aKeyHandle) {
+  return importPublicKey(pubKeyBytes).then(function(aKeyHandle) {
     return Promise.resolve({
-      rpIdHash: rpIdHash,
-      flags: flags,
-      counter: counter,
+      rpIdHash,
+      flags,
+      counter,
       attestationAuthData: attData,
       publicKeyBytes: pubKeyBytes,
       publicKeyHandle: aKeyHandle,
@@ -250,22 +277,27 @@ function importPublicKey(keyBytes) {
     kty: "EC",
     crv: "P-256",
     x: bytesToBase64UrlSafe(keyBytes.slice(1, 33)),
-    y: bytesToBase64UrlSafe(keyBytes.slice(33))
+    y: bytesToBase64UrlSafe(keyBytes.slice(33)),
   };
-  return crypto.subtle.importKey("jwk", jwk, {name: "ECDSA", namedCurve: "P-256"}, true, ["verify"])
+  return crypto.subtle.importKey(
+    "jwk",
+    jwk,
+    { name: "ECDSA", namedCurve: "P-256" },
+    true,
+    ["verify"]
+  );
 }
 
 function deriveAppAndChallengeParam(appId, clientData, attestation) {
   var appIdBuf = string2buffer(appId);
   return Promise.all([
     crypto.subtle.digest("SHA-256", appIdBuf),
-    crypto.subtle.digest("SHA-256", clientData)
-  ])
-  .then(function(digests) {
+    crypto.subtle.digest("SHA-256", clientData),
+  ]).then(function(digests) {
     return {
       appParam: new Uint8Array(digests[0]),
       challengeParam: new Uint8Array(digests[1]),
-      attestation: attestation
+      attestation,
     };
   });
 }
@@ -274,31 +306,38 @@ function assemblePublicKeyBytesData(xCoord, yCoord) {
   // Produce an uncompressed EC key point. These start with 0x04, and then
   // two 32-byte numbers denoting X and Y.
   if (xCoord.length != 32 || yCoord.length != 32) {
-    throw ("Coordinates must be 32 bytes long");
+    throw "Coordinates must be 32 bytes long";
   }
   let keyBytes = new Uint8Array(65);
   keyBytes[0] = 0x04;
-  xCoord.map((x, i) => keyBytes[1 + i] = x);
-  yCoord.map((x, i) => keyBytes[33 + i] = x);
+  xCoord.map((x, i) => (keyBytes[1 + i] = x));
+  yCoord.map((x, i) => (keyBytes[33 + i] = x));
   return keyBytes;
 }
 
 function assembleSignedData(appParam, flags, counter, challengeParam) {
   let signedData = new Uint8Array(32 + 1 + 4 + 32);
-  new Uint8Array(appParam).map((x, i) => signedData[0 + i] = x);
+  new Uint8Array(appParam).map((x, i) => (signedData[0 + i] = x));
   signedData[32] = new Uint8Array(flags)[0];
-  new Uint8Array(counter).map((x, i) => signedData[33 + i] = x);
-  new Uint8Array(challengeParam).map((x, i) => signedData[37 + i] = x);
+  new Uint8Array(counter).map((x, i) => (signedData[33 + i] = x));
+  new Uint8Array(challengeParam).map((x, i) => (signedData[37 + i] = x));
   return signedData;
 }
 
-function assembleRegistrationSignedData(appParam, challengeParam, keyHandle, pubKey) {
+function assembleRegistrationSignedData(
+  appParam,
+  challengeParam,
+  keyHandle,
+  pubKey
+) {
   let signedData = new Uint8Array(1 + 32 + 32 + keyHandle.length + 65);
   signedData[0] = 0x00;
-  new Uint8Array(appParam).map((x, i) => signedData[1 + i] = x);
-  new Uint8Array(challengeParam).map((x, i) => signedData[33 + i] = x);
-  new Uint8Array(keyHandle).map((x, i) => signedData[65 + i] = x);
-  new Uint8Array(pubKey).map((x, i) => signedData[65 + keyHandle.length + i] = x);
+  new Uint8Array(appParam).map((x, i) => (signedData[1 + i] = x));
+  new Uint8Array(challengeParam).map((x, i) => (signedData[33 + i] = x));
+  new Uint8Array(keyHandle).map((x, i) => (signedData[65 + i] = x));
+  new Uint8Array(pubKey).map(
+    (x, i) => (signedData[65 + keyHandle.length + i] = x)
+  );
   return signedData;
 }
 
@@ -307,7 +346,7 @@ function sanitizeSigArray(arr) {
   // this method strips leading padding bytes, if added, and also appends
   // padding zeros, if needed.
   if (arr.length > 32) {
-    arr = arr.slice(arr.length - 32)
+    arr = arr.slice(arr.length - 32);
   }
   let ret = new Uint8Array(32);
   ret.set(arr, ret.length - arr.length);
@@ -316,8 +355,12 @@ function sanitizeSigArray(arr) {
 
 function verifySignature(key, data, derSig) {
   if (derSig.byteLength < 68) {
-    return Promise.reject("Invalid signature (length=" + derSig.byteLength +
-                          "): " + hexEncode(new Uint8Array(derSig)));
+    return Promise.reject(
+      "Invalid signature (length=" +
+        derSig.byteLength +
+        "): " +
+        hexEncode(new Uint8Array(derSig))
+    );
   }
 
   // Copy signature data into the current context.
@@ -330,8 +373,12 @@ function verifySignature(key, data, derSig) {
   // internal function. The test suite doesn't like dangling globals.
   delete window.error;
 
-  let sigR = new Uint8Array(sigAsn1.result.value_block.value[0].value_block.value_hex);
-  let sigS = new Uint8Array(sigAsn1.result.value_block.value[1].value_block.value_hex);
+  let sigR = new Uint8Array(
+    sigAsn1.result.value_block.value[0].value_block.value_hex
+  );
+  let sigS = new Uint8Array(
+    sigAsn1.result.value_block.value[1].value_block.value_hex
+  );
 
   // The resulting R and S values from the ASN.1 Sequence must be fit into 32
   // bytes. Sometimes they have leading zeros, sometimes they're too short, it
@@ -345,6 +392,6 @@ function verifySignature(key, data, derSig) {
   sigData.set(R);
   sigData.set(S, R.length);
 
-  let alg = {name: "ECDSA", hash: "SHA-256"};
+  let alg = { name: "ECDSA", hash: "SHA-256" };
   return crypto.subtle.verify(alg, key, sigData, data);
 }
