@@ -7,7 +7,6 @@
 #include "js/Value.h"
 #include "mozilla/ErrorResult.h"
 #include "mozilla/Logging.h"
-#include "mozilla/ipc/Shmem.h"
 #include "mozilla/dom/Promise.h"
 #include "mozilla/dom/WebGPUBinding.h"
 #include "Device.h"
@@ -29,8 +28,7 @@
 #include "ValidationError.h"
 #include "ipc/WebGPUChild.h"
 
-namespace mozilla {
-namespace webgpu {
+namespace mozilla::webgpu {
 
 mozilla::LazyLogModule gWebGPULog("WebGPU");
 
@@ -40,21 +38,8 @@ GPU_IMPL_CYCLE_COLLECTION_WRAPPERCACHE_INHERITED(Device, DOMEventTargetHelper,
 NS_IMPL_ISUPPORTS_CYCLE_COLLECTION_INHERITED_0(Device, DOMEventTargetHelper)
 GPU_IMPL_JS_WRAP(Device)
 
-static void mapFreeCallback(void* aContents, void* aUserData) {
-  Unused << aContents;
-  Unused << aUserData;
-}
-
 RefPtr<WebGPUChild> Device::GetBridge() { return mBridge; }
 
-JSObject* Device::CreateExternalArrayBuffer(JSContext* aCx, size_t aOffset,
-                                            size_t aSize,
-                                            const ipc::Shmem& aShmem) {
-  MOZ_ASSERT(aOffset + aSize <= aShmem.Size<uint8_t>());
-  return JS::NewExternalArrayBuffer(aCx, aSize, aShmem.get<uint8_t>() + aOffset,
-                                    &mapFreeCallback, nullptr);
-}
-
 Device::Device(Adapter* const aParent, RawId aId,
                UniquePtr<ffi::WGPULimits> aRawLimits)
     : DOMEventTargetHelper(aParent->GetParentObject()),
@@ -80,7 +65,8 @@ void Device::Cleanup() {
     mBridge->UnregisterDevice(mId);
   }
 
-  if (mLostPromise) {
+  // Cycle collection may have disconnected the promise object.
+  if (mLostPromise && mLostPromise->PromiseObj() != nullptr) {
     auto info = MakeRefPtr<DeviceLostInfo>(GetParentObject(),
                                            dom::GPUDeviceLostReason::Destroyed,
                                            u"Device destroyed"_ns);
@@ -95,6 +81,17 @@ void Device::CleanupUnregisteredInParent() {
   mValid = false;
 }
 
+bool Device::IsLost() const { return !mBridge || !mBridge->CanSend(); }
+
+// Generate an error on the Device timeline for this device.
+//
+// aMessage is interpreted as UTF-8.
+void Device::GenerateError(const nsCString& aMessage) {
+  if (mBridge->CanSend()) {
+    mBridge->SendGenerateError(mId, aMessage);
+  }
+}
+
 void Device::GetLabel(nsAString& aValue) const { aValue = mLabel; }
 void Device::SetLabel(const nsAString& aLabel) { mLabel = aLabel; }
 
@@ -112,84 +109,7 @@ dom::Promise* Device::GetLost(ErrorResult& aRv) {
 
 already_AddRefed<Buffer> Device::CreateBuffer(
     const dom::GPUBufferDescriptor& aDesc, ErrorResult& aRv) {
-  if (!mBridge->CanSend()) {
-    RefPtr<Buffer> buffer = new Buffer(this, 0, aDesc.mSize, false);
-    return buffer.forget();
-  }
-
-  ipc::Shmem shmem;
-  bool hasMapFlags = aDesc.mUsage & (dom::GPUBufferUsage_Binding::MAP_WRITE |
-                                     dom::GPUBufferUsage_Binding::MAP_READ);
-  if (hasMapFlags || aDesc.mMappedAtCreation) {
-    const auto checked = CheckedInt<size_t>(aDesc.mSize);
-    if (!checked.isValid()) {
-      aRv.ThrowRangeError("Mappable size is too large");
-      return nullptr;
-    }
-    const auto& size = checked.value();
-
-    // TODO: use `ShmemPool`?
-    if (!mBridge->AllocShmem(size, ipc::Shmem::SharedMemory::TYPE_BASIC,
-                             &shmem)) {
-      aRv.ThrowAbortError(
-          nsPrintfCString("Unable to allocate shmem of size %" PRIuPTR, size));
-      return nullptr;
-    }
-
-    // zero out memory
-    memset(shmem.get<uint8_t>(), 0, size);
-  }
-
-  // If the buffer is not mapped at creation, and it has Shmem, we send it
-  // to the GPU process. Otherwise, we keep it.
-  RawId id = mBridge->DeviceCreateBuffer(mId, aDesc);
-  RefPtr<Buffer> buffer = new Buffer(this, id, aDesc.mSize, hasMapFlags);
-  if (aDesc.mMappedAtCreation) {
-    buffer->SetMapped(std::move(shmem),
-                      !(aDesc.mUsage & dom::GPUBufferUsage_Binding::MAP_READ));
-  } else if (hasMapFlags) {
-    mBridge->SendBufferReturnShmem(id, std::move(shmem));
-  }
-
-  return buffer.forget();
-}
-
-RefPtr<MappingPromise> Device::MapBufferAsync(RawId aId, uint32_t aMode,
-                                              size_t aOffset, size_t aSize,
-                                              ErrorResult& aRv) {
-  ffi::WGPUHostMap mode;
-  switch (aMode) {
-    case dom::GPUMapMode_Binding::READ:
-      mode = ffi::WGPUHostMap_Read;
-      break;
-    case dom::GPUMapMode_Binding::WRITE:
-      mode = ffi::WGPUHostMap_Write;
-      break;
-    default:
-      aRv.ThrowInvalidAccessError(
-          nsPrintfCString("Invalid map flag %u", aMode));
-      return nullptr;
-  }
-
-  const CheckedInt<uint64_t> offset(aOffset);
-  if (!offset.isValid()) {
-    aRv.ThrowRangeError("Mapped offset is too large");
-    return nullptr;
-  }
-  const CheckedInt<uint64_t> size(aSize);
-  if (!size.isValid()) {
-    aRv.ThrowRangeError("Mapped size is too large");
-    return nullptr;
-  }
-
-  return mBridge->SendBufferMap(aId, mode, offset.value(), size.value());
-}
-
-void Device::UnmapBuffer(RawId aId, ipc::Shmem&& aShmem, bool aFlush,
-                         bool aKeepShmem) {
-  if (mBridge->CanSend()) {
-    mBridge->SendBufferUnmap(aId, std::move(aShmem), aFlush, aKeepShmem);
-  }
+  return Buffer::Create(this, mId, aDesc, aRv);
 }
 
 already_AddRefed<Texture> Device::CreateTexture(
@@ -260,12 +180,18 @@ already_AddRefed<BindGroup> Device::CreateBindGroup(
 already_AddRefed<ShaderModule> Device::CreateShaderModule(
     JSContext* aCx, const dom::GPUShaderModuleDescriptor& aDesc) {
   Unused << aCx;
-  RawId id = 0;
-  if (mBridge->CanSend()) {
-    id = mBridge->DeviceCreateShaderModule(mId, aDesc);
+
+  if (!mBridge->CanSend()) {
+    return nullptr;
   }
-  RefPtr<ShaderModule> object = new ShaderModule(this, id);
-  return object.forget();
+
+  ErrorResult err;
+  RefPtr<dom::Promise> promise = dom::Promise::Create(GetParentObject(), err);
+  if (NS_WARN_IF(err.Failed())) {
+    return nullptr;
+  }
+
+  return mBridge->DeviceCreateShaderModule(this, aDesc, promise);
 }
 
 already_AddRefed<ComputePipeline> Device::CreateComputePipeline(
@@ -360,7 +286,7 @@ already_AddRefed<dom::Promise> Device::CreateRenderPipelineAsync(
 
 already_AddRefed<Texture> Device::InitSwapChain(
     const dom::GPUCanvasConfiguration& aDesc,
-    const layers::CompositableHandle& aHandle, gfx::SurfaceFormat aFormat,
+    const layers::RemoteTextureOwnerId aOwnerId, gfx::SurfaceFormat aFormat,
     gfx::IntSize* aCanvasSize) {
   if (!mBridge->CanSend()) {
     return nullptr;
@@ -387,7 +313,7 @@ already_AddRefed<Texture> Device::InitSwapChain(
   const layers::RGBDescriptor rgbDesc(size, aFormat);
   // buffer count doesn't matter much, will be created on demand
   const size_t maxBufferCount = 10;
-  mBridge->DeviceCreateSwapChain(mId, rgbDesc, maxBufferCount, aHandle);
+  mBridge->DeviceCreateSwapChain(mId, rgbDesc, maxBufferCount, aOwnerId);
 
   dom::GPUTextureDescriptor desc;
   desc.mDimension = dom::GPUTextureDimension::_2d;
@@ -456,5 +382,4 @@ already_AddRefed<dom::Promise> Device::PopErrorScope(ErrorResult& aRv) {
   return promise.forget();
 }
 
-}  // namespace webgpu
-}  // namespace mozilla
+}  // namespace mozilla::webgpu
