@@ -8,16 +8,13 @@
 #define mozilla_dom_XMLHttpRequestMainThread_h
 
 #include <bitset>
-#include "nsAutoPtr.h"
-#include "nsIXMLHttpRequest.h"
 #include "nsISupportsUtils.h"
 #include "nsIURI.h"
-#include "nsIHttpChannel.h"
-#include "nsIDocument.h"
+#include "mozilla/dom/Document.h"
 #include "nsIStreamListener.h"
-#include "nsWeakReference.h"
 #include "nsIChannelEventSink.h"
 #include "nsIAsyncVerifyRedirectCallback.h"
+#include "nsIDOMEventListener.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsIHttpHeaderVisitor.h"
 #include "nsIProgressEventSink.h"
@@ -27,18 +24,23 @@
 #include "nsIPrincipal.h"
 #include "nsIScriptObjectPrincipal.h"
 #include "nsISizeOfEventTarget.h"
-#include "nsIXPConnect.h"
 #include "nsIInputStream.h"
+#include "nsIContentSecurityPolicy.h"
 #include "mozilla/Assertions.h"
 #include "mozilla/Attributes.h"
 #include "mozilla/DOMEventTargetHelper.h"
+#include "mozilla/Maybe.h"
 #include "mozilla/MemoryReporting.h"
 #include "mozilla/NotNull.h"
 #include "mozilla/dom/MutableBlobStorage.h"
 #include "mozilla/dom/BodyExtractor.h"
+#include "mozilla/dom/ClientInfo.h"
 #include "mozilla/dom/TypedArray.h"
 #include "mozilla/dom/File.h"
 #include "mozilla/dom/FormData.h"
+#include "mozilla/dom/MimeType.h"
+#include "mozilla/dom/PerformanceStorage.h"
+#include "mozilla/dom/ServiceWorkerDescriptor.h"
 #include "mozilla/dom/URLSearchParams.h"
 #include "mozilla/dom/XMLHttpRequest.h"
 #include "mozilla/dom/XMLHttpRequestBinding.h"
@@ -49,43 +51,40 @@
 #ifdef Status
 /* Xlib headers insist on this for some reason... Nuke it because
    it'll override our member name */
-#undef Status
+typedef Status __StatusTmp;
+#  undef Status
+typedef __StatusTmp Status;
 #endif
 
+class nsIHttpChannel;
 class nsIJARChannel;
 class nsILoadGroup;
-class nsIJSID;
 
 namespace mozilla {
+class ProfileChunkedBuffer;
+
 namespace dom {
 
-class BlobSet;
 class DOMString;
 class XMLHttpRequestUpload;
+class SerializedStackHolder;
 struct OriginAttributesDictionary;
 
 // A helper for building up an ArrayBuffer object's data
 // before creating the ArrayBuffer itself.  Will do doubling
 // based reallocation, up to an optional maximum growth given.
 //
-// When all the data has been appended, call getArrayBuffer,
+// When all the data has been appended, call GetArrayBuffer,
 // passing in the JSContext* for which the ArrayBuffer object
-// is to be created.  This also implicitly resets the builder,
-// or it can be reset explicitly at any point by calling reset().
-class ArrayBufferBuilder
-{
-  uint8_t* mDataPtr;
-  uint32_t mCapacity;
-  uint32_t mLength;
-  void* mMapPtr;
-public:
+// is to be created.  This also implicitly resets the builder.
+class ArrayBufferBuilder {
+ public:
+  NS_INLINE_DECL_THREADSAFE_REFCOUNTING(ArrayBufferBuilder);
+
   ArrayBufferBuilder();
-  ~ArrayBufferBuilder();
 
-  void reset();
-
-  // Will truncate if aNewCap is < length().
-  bool setCapacity(uint32_t aNewCap);
+  // Will truncate if aNewCap is < Length().
+  bool SetCapacity(uint32_t aNewCap);
 
   // Append aDataLen bytes from data to the current buffer.  If we
   // need to grow the buffer, grow by doubling the size up to a
@@ -94,13 +93,13 @@ public:
   //
   // The data parameter must not overlap with anything beyond the
   // builder's current valid contents [0..length)
-  bool append(const uint8_t* aNewData, uint32_t aDataLen,
+  bool Append(const uint8_t* aNewData, uint32_t aDataLen,
               uint32_t aMaxGrowth = 0);
 
-  uint32_t length()   { return mLength; }
-  uint32_t capacity() { return mCapacity; }
+  uint32_t Length();
+  uint32_t Capacity();
 
-  JSObject* getArrayBuffer(JSContext* aCx);
+  JSObject* TakeArrayBuffer(JSContext* aCx);
 
   // Memory mapping to starting position of file(aFile) in the zip
   // package(aJarFile).
@@ -108,39 +107,58 @@ public:
   // The file in the zip package has to be uncompressed and the starting
   // position of the file must be aligned according to array buffer settings
   // in JS engine.
-  nsresult mapToFileInPackage(const nsCString& aFile, nsIFile* aJarFile);
+  nsresult MapToFileInPackage(const nsCString& aFile, nsIFile* aJarFile);
 
-protected:
-  static bool areOverlappingRegions(const uint8_t* aStart1, uint32_t aLength1,
+ private:
+  ~ArrayBufferBuilder();
+
+  ArrayBufferBuilder(const ArrayBufferBuilder&) = delete;
+  ArrayBufferBuilder& operator=(const ArrayBufferBuilder&) = delete;
+  ArrayBufferBuilder& operator=(const ArrayBufferBuilder&&) = delete;
+
+  bool SetCapacityInternal(uint32_t aNewCap, const MutexAutoLock& aProofOfLock)
+      MOZ_REQUIRES(mMutex);
+
+  static bool AreOverlappingRegions(const uint8_t* aStart1, uint32_t aLength1,
                                     const uint8_t* aStart2, uint32_t aLength2);
+
+  Mutex mMutex;
+
+  // All of these are protected by mMutex.
+  uint8_t* mDataPtr MOZ_GUARDED_BY(mMutex);
+  uint32_t mCapacity MOZ_GUARDED_BY(mMutex);
+  uint32_t mLength MOZ_GUARDED_BY(mMutex);
+  void* mMapPtr MOZ_GUARDED_BY(mMutex);
+
+  // This is used in assertions only.
+  bool mNeutered;
 };
 
 class nsXMLHttpRequestXPCOMifier;
 
-class RequestHeaders
-{
-  struct RequestHeader
-  {
+class RequestHeaders {
+  struct RequestHeader {
     nsCString mName;
     nsCString mValue;
   };
   nsTArray<RequestHeader> mHeaders;
   RequestHeader* Find(const nsACString& aName);
 
-public:
-  class CharsetIterator
-  {
+ public:
+  class CharsetIterator {
     bool mValid;
     int32_t mCurPos, mCurLen, mCutoff;
     nsACString& mSource;
 
-  public:
+   public:
     explicit CharsetIterator(nsACString& aSource);
-    bool Equals(const nsACString& aOther, const nsCStringComparator& aCmp) const;
+    bool Equals(const nsACString& aOther,
+                const nsCStringComparator& aCmp) const;
     void Replace(const nsACString& aReplacement);
     bool Next();
   };
 
+  bool IsEmpty() const;
   bool Has(const char* aName);
   bool Has(const nsACString& aName);
   void Get(const char* aName, nsACString& aValue);
@@ -150,28 +168,30 @@ public:
   void MergeOrSet(const char* aName, const nsACString& aValue);
   void MergeOrSet(const nsACString& aName, const nsACString& aValue);
   void Clear();
-  void ApplyToChannel(nsIHttpChannel* aChannel) const;
+  void ApplyToChannel(nsIHttpChannel* aChannel,
+                      bool aStripRequestBodyHeader) const;
   void GetCORSUnsafeHeaders(nsTArray<nsCString>& aArray) const;
 };
 
+class nsXHRParseEndListener;
+class XMLHttpRequestDoneNotifier;
+
 // Make sure that any non-DOM interfaces added here are also added to
 // nsXMLHttpRequestXPCOMifier.
 class XMLHttpRequestMainThread final : public XMLHttpRequest,
-                                       public nsIXMLHttpRequest,
                                        public nsIStreamListener,
                                        public nsIChannelEventSink,
                                        public nsIProgressEventSink,
                                        public nsIInterfaceRequestor,
-                                       public nsSupportsWeakReference,
                                        public nsITimerCallback,
                                        public nsISizeOfEventTarget,
                                        public nsINamed,
-                                       public MutableBlobStorageCallback
-{
+                                       public MutableBlobStorageCallback {
   friend class nsXHRParseEndListener;
   friend class nsXMLHttpRequestXPCOMifier;
+  friend class XMLHttpRequestDoneNotifier;
 
-public:
+ public:
   enum class ProgressEventType : uint8_t {
     loadstart,
     progress,
@@ -183,51 +203,39 @@ public:
     ENUM_MAX
   };
 
+  // Make sure that any additions done to ErrorType enum are also mirrored in
+  // XHR_ERROR_TYPE enum of TelemetrySend.sys.mjs.
   enum class ErrorType : uint16_t {
     eOK,
     eRequest,
     eUnreachable,
     eChannelOpen,
     eRedirect,
+    eTerminated,
     ENUM_MAX
   };
 
-  XMLHttpRequestMainThread();
+  explicit XMLHttpRequestMainThread(nsIGlobalObject* aGlobalObject);
 
   void Construct(nsIPrincipal* aPrincipal,
-                 nsIGlobalObject* aGlobalObject,
-                 nsIURI* aBaseURI = nullptr,
-                 nsILoadGroup* aLoadGroup = nullptr)
-  {
-    MOZ_ASSERT(aPrincipal);
-    nsCOMPtr<nsPIDOMWindowInner> win = do_QueryInterface(aGlobalObject);
-    if (win) {
-      MOZ_ASSERT(win->IsInnerWindow());
-      if (win->GetExtantDoc()) {
-        mStyleBackend = win->GetExtantDoc()->GetStyleBackendType();
-      }
-    }
-    mPrincipal = aPrincipal;
-    BindToOwner(aGlobalObject);
-    mBaseURI = aBaseURI;
-    mLoadGroup = aLoadGroup;
-  }
+                 nsICookieJarSettings* aCookieJarSettings, bool aForWorker,
+                 nsIURI* aBaseURI = nullptr, nsILoadGroup* aLoadGroup = nullptr,
+                 PerformanceStorage* aPerformanceStorage = nullptr,
+                 nsICSPEventListener* aCSPEventListener = nullptr);
 
   void InitParameters(bool aAnon, bool aSystem);
 
-  void SetParameters(bool aAnon, bool aSystem)
-  {
+  void SetParameters(bool aAnon, bool aSystem) {
     mIsAnon = aAnon || aSystem;
     mIsSystem = aSystem;
   }
 
+  void SetClientInfoAndController(
+      const ClientInfo& aClientInfo,
+      const Maybe<ServiceWorkerDescriptor>& aController);
+
   NS_DECL_ISUPPORTS_INHERITED
 
-  // nsIXMLHttpRequest
-  NS_DECL_NSIXMLHTTPREQUEST
-
-  NS_FORWARD_NSIXMLHTTPREQUESTEVENTTARGET(XMLHttpRequestEventTarget::)
-
   // nsIStreamListener
   NS_DECL_NSISTREAMLISTENER
 
@@ -250,288 +258,223 @@ public:
   NS_DECL_NSINAMED
 
   // nsISizeOfEventTarget
-  virtual size_t
-    SizeOfEventTargetIncludingThis(MallocSizeOf aMallocSizeOf) const override;
-
-  NS_REALLY_FORWARD_NSIDOMEVENTTARGET(XMLHttpRequestEventTarget)
+  virtual size_t SizeOfEventTargetIncludingThis(
+      MallocSizeOf aMallocSizeOf) const override;
 
   // states
   virtual uint16_t ReadyState() const override;
 
   // request
   nsresult CreateChannel();
-  nsresult InitiateFetch(nsIInputStream* aUploadStream,
-                         int64_t aUploadLength,
-                         nsACString& aUploadContentType);
+  nsresult InitiateFetch(already_AddRefed<nsIInputStream> aUploadStream,
+                         int64_t aUploadLength, nsACString& aUploadContentType);
 
-  virtual void
-  Open(const nsACString& aMethod, const nsAString& aUrl,
-       ErrorResult& aRv) override;
+  virtual void Open(const nsACString& aMethod, const nsAString& aUrl,
+                    ErrorResult& aRv) override;
 
-  virtual void
-  Open(const nsACString& aMethod, const nsAString& aUrl, bool aAsync,
-       const nsAString& aUsername, const nsAString& aPassword,
-       ErrorResult& aRv) override;
+  virtual void Open(const nsACString& aMethod, const nsAString& aUrl,
+                    bool aAsync, const nsAString& aUsername,
+                    const nsAString& aPassword, ErrorResult& aRv) override;
 
-  nsresult
-  Open(const nsACString& aMethod,
-       const nsACString& aUrl,
-       bool aAsync,
-       const nsAString& aUsername,
-       const nsAString& aPassword);
+  void Open(const nsACString& aMethod, const nsACString& aUrl, bool aAsync,
+            const nsAString& aUsername, const nsAString& aPassword,
+            ErrorResult& aRv);
 
-  virtual void
-  SetRequestHeader(const nsACString& aName, const nsACString& aValue,
-                   ErrorResult& aRv) override
-  {
-    aRv = SetRequestHeader(aName, aValue);
-  }
+  virtual void SetRequestHeader(const nsACString& aName,
+                                const nsACString& aValue,
+                                ErrorResult& aRv) override;
 
-  virtual uint32_t
-  Timeout() const override
-  {
-    return mTimeoutMilliseconds;
-  }
+  virtual uint32_t Timeout() const override { return mTimeoutMilliseconds; }
 
-  virtual void
-  SetTimeout(uint32_t aTimeout, ErrorResult& aRv) override;
+  virtual void SetTimeout(uint32_t aTimeout, ErrorResult& aRv) override;
 
   virtual bool WithCredentials() const override;
 
-  virtual void
-  SetWithCredentials(bool aWithCredentials, ErrorResult& aRv) override;
+  virtual void SetWithCredentials(bool aWithCredentials,
+                                  ErrorResult& aRv) override;
 
-  virtual XMLHttpRequestUpload*
-  GetUpload(ErrorResult& aRv) override;
+  virtual XMLHttpRequestUpload* GetUpload(ErrorResult& aRv) override;
 
-private:
+ private:
   virtual ~XMLHttpRequestMainThread();
 
-  nsresult SendInternal(const BodyExtractorBase* aBody);
+  nsresult MaybeSilentSendFailure(nsresult aRv);
+  void SendInternal(const BodyExtractorBase* aBody,
+                    bool aBodyIsDocumentOrString, ErrorResult& aRv);
 
   bool IsCrossSiteCORSRequest() const;
   bool IsDeniedCrossSiteCORSRequest();
 
-  // Tell our channel what network interface ID we were told to use.
-  // If it's an HTTP channel and we were told to use a non-default
-  // interface ID.
-  void PopulateNetworkInterfaceId();
-
-  void UnsuppressEventHandlingAndResume();
-
-  // Check pref "dom.mapped_arraybuffer.enabled" to make sure ArrayBuffer is
-  // supported.
-  static bool IsMappedArrayBufferEnabled();
-
-  // Check pref "dom.xhr.lowercase_header.enabled" to make sure lowercased
-  // response header is supported.
-  static bool IsLowercaseResponseHeader();
+  void ResumeTimeout();
 
   void MaybeLowerChannelPriority();
 
-public:
-  virtual void
-  Send(JSContext* aCx,
-       const Nullable<DocumentOrBlobOrArrayBufferViewOrArrayBufferOrFormDataOrURLSearchParamsOrUSVString>& aData,
-       ErrorResult& aRv) override;
+ public:
+  bool CanSend(ErrorResult& aRv);
 
-  virtual void
-  SendInputStream(nsIInputStream* aInputStream, ErrorResult& aRv) override
-  {
+  virtual void Send(
+      const Nullable<
+          DocumentOrBlobOrArrayBufferViewOrArrayBufferOrFormDataOrURLSearchParamsOrUSVString>&
+          aData,
+      ErrorResult& aRv) override;
+
+  virtual void SendInputStream(nsIInputStream* aInputStream,
+                               ErrorResult& aRv) override {
+    if (!CanSend(aRv)) {
+      return;
+    }
     BodyExtractor<nsIInputStream> body(aInputStream);
-    aRv = SendInternal(&body);
+    SendInternal(&body, false, aRv);
   }
 
-  void
-  RequestErrorSteps(const ProgressEventType aEventType,
-                    const nsresult aOptionalException,
-                    ErrorResult& aRv);
+  void RequestErrorSteps(const ProgressEventType aEventType,
+                         const nsresult aOptionalException, ErrorResult& aRv);
 
-  void
-  Abort()
-  {
-    ErrorResult rv;
-    Abort(rv);
+  void Abort() {
+    IgnoredErrorResult rv;
+    AbortInternal(rv);
     MOZ_ASSERT(!rv.Failed());
   }
 
-  virtual void
-  Abort(ErrorResult& aRv) override;
+  virtual void Abort(ErrorResult& aRv) override;
 
   // response
-  virtual void
-  GetResponseURL(nsAString& aUrl) override;
+  virtual void GetResponseURL(nsAString& aUrl) override;
 
-  virtual uint32_t
-  GetStatus(ErrorResult& aRv) override;
+  virtual uint32_t GetStatus(ErrorResult& aRv) override;
 
-  virtual void
-  GetStatusText(nsACString& aStatusText, ErrorResult& aRv) override;
+  virtual void GetStatusText(nsACString& aStatusText,
+                             ErrorResult& aRv) override;
 
-  virtual void
-  GetResponseHeader(const nsACString& aHeader, nsACString& aResult,
-                    ErrorResult& aRv) override;
+  virtual void GetResponseHeader(const nsACString& aHeader, nsACString& aResult,
+                                 ErrorResult& aRv) override;
 
-  void
-  GetResponseHeader(const nsAString& aHeader, nsAString& aResult,
-                    ErrorResult& aRv)
-  {
+  void GetResponseHeader(const nsAString& aHeader, nsAString& aResult,
+                         ErrorResult& aRv) {
     nsAutoCString result;
     GetResponseHeader(NS_ConvertUTF16toUTF8(aHeader), result, aRv);
     if (result.IsVoid()) {
       aResult.SetIsVoid(true);
-    }
-    else {
+    } else {
       // The result value should be inflated:
       CopyASCIItoUTF16(result, aResult);
     }
   }
 
-  virtual void
-  GetAllResponseHeaders(nsACString& aResponseHeaders,
-                        ErrorResult& aRv) override;
+  virtual void GetAllResponseHeaders(nsACString& aResponseHeaders,
+                                     ErrorResult& aRv) override;
 
   bool IsSafeHeader(const nsACString& aHeaderName,
                     NotNull<nsIHttpChannel*> aHttpChannel) const;
 
-  virtual void
-  OverrideMimeType(const nsAString& aMimeType, ErrorResult& aRv) override;
+  virtual void OverrideMimeType(const nsAString& aMimeType,
+                                ErrorResult& aRv) override;
 
-  virtual XMLHttpRequestResponseType
-  ResponseType() const override
-  {
+  virtual XMLHttpRequestResponseType ResponseType() const override {
     return XMLHttpRequestResponseType(mResponseType);
   }
 
-  virtual void
-  SetResponseType(XMLHttpRequestResponseType aType,
-                  ErrorResult& aRv) override;
+  virtual void SetResponseType(XMLHttpRequestResponseType aType,
+                               ErrorResult& aRv) override;
 
-  virtual void
-  GetResponse(JSContext* aCx, JS::MutableHandle<JS::Value> aResponse,
-              ErrorResult& aRv) override;
+  void SetResponseTypeRaw(XMLHttpRequestResponseType aType) {
+    mResponseType = aType;
+  }
 
-  virtual void
-  GetResponseText(DOMString& aResponseText, ErrorResult& aRv) override;
+  virtual void GetResponse(JSContext* aCx,
+                           JS::MutableHandle<JS::Value> aResponse,
+                           ErrorResult& aRv) override;
 
-  void
-  GetResponseText(XMLHttpRequestStringSnapshot& aSnapshot,
-                  ErrorResult& aRv);
+  virtual void GetResponseText(DOMString& aResponseText,
+                               ErrorResult& aRv) override;
 
-  virtual nsIDocument*
-  GetResponseXML(ErrorResult& aRv) override;
+  // GetResponse* for workers:
+  already_AddRefed<BlobImpl> GetResponseBlobImpl();
+  already_AddRefed<ArrayBufferBuilder> GetResponseArrayBufferBuilder();
+  nsresult GetResponseTextForJSON(nsAString& aString);
+  void GetResponseText(XMLHttpRequestStringSnapshot& aSnapshot,
+                       ErrorResult& aRv);
 
-  virtual bool
-  MozBackgroundRequest() const override;
+  virtual Document* GetResponseXML(ErrorResult& aRv) override;
 
-  virtual void
-  SetMozBackgroundRequest(bool aMozBackgroundRequest, ErrorResult& aRv) override;
+  virtual bool MozBackgroundRequest() const override;
 
-  virtual uint16_t
-  ErrorCode() const override
-  {
+  void SetMozBackgroundRequestExternal(bool aMozBackgroundRequest,
+                                       ErrorResult& aRv);
+
+  virtual void SetMozBackgroundRequest(bool aMozBackgroundRequest,
+                                       ErrorResult& aRv) override;
+
+  void SetOriginStack(UniquePtr<SerializedStackHolder> aOriginStack);
+
+  void SetSource(UniquePtr<ProfileChunkedBuffer> aSource);
+
+  virtual uint16_t ErrorCode() const override {
     return static_cast<uint16_t>(mErrorLoad);
   }
 
-  virtual bool
-  MozAnon() const override;
+  virtual bool MozAnon() const override;
 
-  virtual bool
-  MozSystem() const override;
+  virtual bool MozSystem() const override;
 
-  virtual nsIChannel*
-  GetChannel() const override
-  {
-    return mChannel;
-  }
-
-  virtual void
-  GetNetworkInterfaceId(nsACString& aId) const override
-  {
-    aId = mNetworkInterfaceId;
-  }
-
-  virtual void
-  SetNetworkInterfaceId(const nsACString& aId) override
-  {
-    mNetworkInterfaceId = aId;
-  }
+  virtual nsIChannel* GetChannel() const override { return mChannel; }
 
   // We need a GetInterface callable from JS for chrome JS
-  virtual void
-  GetInterface(JSContext* aCx, nsIJSID* aIID,
-               JS::MutableHandle<JS::Value> aRetval,
-               ErrorResult& aRv) override;
+  virtual void GetInterface(JSContext* aCx, JS::Handle<JS::Value> aIID,
+                            JS::MutableHandle<JS::Value> aRetval,
+                            ErrorResult& aRv) override;
 
   // This fires a trusted readystatechange event, which is not cancelable and
   // doesn't bubble.
   nsresult FireReadystatechangeEvent();
   void DispatchProgressEvent(DOMEventTargetHelper* aTarget,
-                             const ProgressEventType aType,
-                             int64_t aLoaded, int64_t aTotal);
-
-  // This is called by the factory constructor.
-  nsresult Init();
-
-  nsresult init(nsIPrincipal* principal,
-                nsPIDOMWindowInner* globalObject,
-                nsIURI* baseURI);
+                             const ProgressEventType aType, int64_t aLoaded,
+                             int64_t aTotal);
 
   void SetRequestObserver(nsIRequestObserver* aObserver);
 
-  NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS_INHERITED(XMLHttpRequestMainThread,
-                                                         XMLHttpRequest)
+  NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS_INHERITED(
+      XMLHttpRequestMainThread, XMLHttpRequest)
   virtual bool IsCertainlyAliveForCC() const override;
 
   bool AllowUploadProgress();
 
   virtual void DisconnectFromOwner() override;
 
-  static void SetDontWarnAboutSyncXHR(bool aVal)
-  {
+  static void SetDontWarnAboutSyncXHR(bool aVal) {
     sDontWarnAboutSyncXHR = aVal;
   }
-  static bool DontWarnAboutSyncXHR()
-  {
-    return sDontWarnAboutSyncXHR;
-  }
+  static bool DontWarnAboutSyncXHR() { return sDontWarnAboutSyncXHR; }
 
-  virtual void
-  SetOriginAttributes(const mozilla::dom::OriginAttributesDictionary& aAttrs) override;
+  virtual void SetOriginAttributes(
+      const mozilla::dom::OriginAttributesDictionary& aAttrs) override;
 
-  void BlobStoreCompleted(MutableBlobStorage* aBlobStorage,
-                          Blob* aBlob,
+  void BlobStoreCompleted(MutableBlobStorage* aBlobStorage, BlobImpl* aBlobImpl,
                           nsresult aResult) override;
 
-  void
-  LocalFileToBlobCompleted(Blob* aBlob);
-
-protected:
-  // XHR states are meant to mirror the XHR2 spec:
-  //   https://xhr.spec.whatwg.org/#states
-  enum class State : uint8_t {
-    unsent,           // object has been constructed.
-    opened,           // open() has been successfully invoked.
-    headers_received, // redirects followed and response headers received.
-    loading,          // response body is being received.
-    done,             // data transfer concluded, whether success or error.
-  };
+  void LocalFileToBlobCompleted(BlobImpl* aBlobImpl);
 
+ protected:
   nsresult DetectCharset();
-  nsresult AppendToResponseText(const char * aBuffer, uint32_t aBufferLen);
-  static nsresult StreamReaderFunc(nsIInputStream* in,
-                                   void* closure,
+  nsresult AppendToResponseText(Span<const uint8_t> aBuffer,
+                                bool aLast = false);
+  static nsresult StreamReaderFunc(nsIInputStream* in, void* closure,
                                    const char* fromRawSegment,
-                                   uint32_t toOffset,
-                                   uint32_t count,
-                                   uint32_t *writeCount);
+                                   uint32_t toOffset, uint32_t count,
+                                   uint32_t* writeCount);
   nsresult CreateResponseParsedJSON(JSContext* aCx);
-  void CreatePartialBlob(ErrorResult& aRv);
   // Change the state of the object with this. The broadcast argument
   // determines if the onreadystatechange listener should be called.
-  nsresult ChangeState(State aState, bool aBroadcast = true);
+  nsresult ChangeState(uint16_t aState, bool aBroadcast = true);
   already_AddRefed<nsILoadGroup> GetLoadGroup() const;
-  nsIURI *GetBaseURI();
+
+  // Finds a preload for this XHR.  If it is found it's removed from the preload
+  // table of the document and marked as used.  The called doesn't need to do
+  // any more comparative checks.
+  already_AddRefed<PreloaderBase> FindPreload();
+  // If no or unknown mime type is set on the channel this method ensures it's
+  // set to "text/xml".
+  void EnsureChannelContentType();
 
   already_AddRefed<nsIHttpChannel> GetCurrentHttpChannel();
   already_AddRefed<nsIJARChannel> GetCurrentJARChannel();
@@ -542,7 +485,8 @@ protected:
   bool InUploadPhase() const;
 
   void OnBodyParseEnd();
-  void ChangeStateToDone();
+  void ChangeStateToDone(bool aWasSync);
+  void ChangeStateToDoneInternal();
 
   void StartProgressEventTimer();
   void StopProgressEventTimer();
@@ -551,7 +495,7 @@ protected:
 
   nsresult OnRedirectVerifyCallback(nsresult result);
 
-  void SetTimerEventTarget(nsITimer* aTimer);
+  nsIEventTarget* GetTimerEventTarget();
 
   nsresult DispatchToMainThread(already_AddRefed<nsIRunnable> aRunnable);
 
@@ -562,8 +506,9 @@ protected:
   void SuspendEventDispatching();
   void ResumeEventDispatching();
 
-  struct PendingEvent
-  {
+  void AbortInternal(ErrorResult& aRv);
+
+  struct PendingEvent {
     RefPtr<DOMEventTargetHelper> mTarget;
     RefPtr<Event> mEvent;
   };
@@ -575,43 +520,58 @@ protected:
   nsCOMPtr<nsIChannel> mChannel;
   nsCString mRequestMethod;
   nsCOMPtr<nsIURI> mRequestURL;
-  nsCOMPtr<nsIDocument> mResponseXML;
+  RefPtr<Document> mResponseXML;
 
   nsCOMPtr<nsIStreamListener> mXMLParserStreamListener;
 
+  nsCOMPtr<nsICookieJarSettings> mCookieJarSettings;
+
+  RefPtr<PerformanceStorage> mPerformanceStorage;
+  nsCOMPtr<nsICSPEventListener> mCSPEventListener;
+
   // used to implement getAllResponseHeaders()
-  class nsHeaderVisitor : public nsIHttpHeaderVisitor
-  {
-    struct HeaderEntry final
-    {
+  class nsHeaderVisitor : public nsIHttpHeaderVisitor {
+    struct HeaderEntry final {
       nsCString mName;
       nsCString mValue;
 
       HeaderEntry(const nsACString& aName, const nsACString& aValue)
-        : mName(aName), mValue(aValue)
-      {}
+          : mName(aName), mValue(aValue) {}
 
-      bool
-      operator==(const HeaderEntry& aOther) const
-      {
+      bool operator==(const HeaderEntry& aOther) const {
         return mName == aOther.mName;
       }
 
-      bool
-      operator<(const HeaderEntry& aOther) const
-      {
-        return mName < aOther.mName;
+      bool operator<(const HeaderEntry& aOther) const {
+        uint32_t selfLen = mName.Length();
+        uint32_t otherLen = aOther.mName.Length();
+        uint32_t min = XPCOM_MIN(selfLen, otherLen);
+        for (uint32_t i = 0; i < min; ++i) {
+          unsigned char self = mName[i];
+          unsigned char other = aOther.mName[i];
+          MOZ_ASSERT(!(self >= 'A' && self <= 'Z'));
+          MOZ_ASSERT(!(other >= 'A' && other <= 'Z'));
+          if (self == other) {
+            continue;
+          }
+          if (self >= 'a' && self <= 'z') {
+            self -= 0x20;
+          }
+          if (other >= 'a' && other <= 'z') {
+            other -= 0x20;
+          }
+          return self < other;
+        }
+        return selfLen < otherLen;
       }
     };
 
-  public:
+   public:
     NS_DECL_ISUPPORTS
     NS_DECL_NSIHTTPHEADERVISITOR
     nsHeaderVisitor(const XMLHttpRequestMainThread& aXMLHttpRequest,
-                    NotNull<nsIHttpChannel*> aHttpChannel)
-      : mXHR(aXMLHttpRequest), mHttpChannel(aHttpChannel) {}
-    const nsACString &Headers()
-    {
+                    NotNull<nsIHttpChannel*> aHttpChannel);
+    const nsACString& Headers() {
       for (uint32_t i = 0; i < mHeaderList.Length(); i++) {
         HeaderEntry& header = mHeaderList.ElementAt(i);
 
@@ -623,8 +583,8 @@ protected:
       return mHeaders;
     }
 
-  private:
-    virtual ~nsHeaderVisitor() {}
+   private:
+    virtual ~nsHeaderVisitor();
 
     nsTArray<HeaderEntry> mHeaderList;
     nsCString mHeaders;
@@ -655,19 +615,18 @@ protected:
   // part of the surrogate.
   mozilla::UniquePtr<mozilla::Decoder> mDecoder;
 
-  const Encoding* mResponseCharset;
-
   void MatchCharsetAndDecoderToResponseDocument();
 
   XMLHttpRequestResponseType mResponseType;
 
-  // It is either a cached blob-response from the last call to GetResponse,
-  // but is also explicitly set in OnStopRequest.
+  RefPtr<BlobImpl> mResponseBlobImpl;
+
+  // This is the cached blob-response, created only at the first GetResponse()
+  // call.
   RefPtr<Blob> mResponseBlob;
+
   // We stream data to mBlobStorage when response type is "blob".
   RefPtr<MutableBlobStorage> mBlobStorage;
-  // We stream data to mBlobSet when response type is "moz-blob".
-  nsAutoPtr<BlobSet> mBlobSet;
 
   nsString mOverrideMimeType;
 
@@ -689,9 +648,13 @@ protected:
   nsCOMPtr<nsIURI> mBaseURI;
   nsCOMPtr<nsILoadGroup> mLoadGroup;
 
-  State mState;
+  Maybe<ClientInfo> mClientInfo;
+  Maybe<ServiceWorkerDescriptor> mController;
 
-  StyleBackendType mStyleBackend;
+  uint16_t mState;
+
+  // If true, this object is used by the worker's XMLHttpRequest.
+  bool mForWorker;
 
   bool mFlagSynchronous;
   bool mFlagAborted;
@@ -722,16 +685,11 @@ protected:
   void StartTimeoutTimer();
   void HandleTimeoutCallback();
 
-  nsCOMPtr<nsIDocument> mSuspendedDoc;
   nsCOMPtr<nsIRunnable> mResumeTimeoutRunnable;
 
   nsCOMPtr<nsITimer> mSyncTimeoutTimer;
 
-  enum SyncTimeoutType {
-    eErrorOrExpired,
-    eTimerStarted,
-    eNoTimerNeeded
-  };
+  enum SyncTimeoutType { eErrorOrExpired, eTimerStarted, eNoTimerNeeded };
 
   SyncTimeoutType MaybeStartSyncTimeoutTimer();
   void HandleSyncTimeoutTimer();
@@ -743,7 +701,7 @@ protected:
   bool mProgressTimerIsActive;
   bool mIsHtml;
   bool mWarnAboutSyncHtml;
-  int64_t mLoadTotal; // -1 if not known.
+  int64_t mLoadTotal;  // -1 if not known.
   // Number of HTTP message body bytes received so far. This quantity is
   // in the same units as Content-Length and mLoadTotal, and hence counts
   // compressed bytes when the channel has gzip Content-Encoding. If the
@@ -758,15 +716,13 @@ protected:
   bool mIsSystem;
   bool mIsAnon;
 
-  // A platform-specific identifer to represent the network interface
-  // that this request is associated with.
-  nsCString mNetworkInterfaceId;
-
   /**
    * Close the XMLHttpRequest's channels.
    */
   void CloseRequest();
 
+  void TerminateOngoingFetch();
+
   /**
    * Close the XMLHttpRequest's channels and dispatch appropriate progress
    * events.
@@ -783,7 +739,7 @@ protected:
 
   JS::Heap<JS::Value> mResultJSON;
 
-  ArrayBufferBuilder mArrayBufferBuilder;
+  RefPtr<ArrayBufferBuilder> mArrayBufferBuilder;
   JS::Heap<JSObject*> mResultArrayBuffer;
   bool mIsMappedArrayBuffer;
 
@@ -800,23 +756,41 @@ protected:
   // useful to change the correct state when XHR is working sync.
   bool mEventDispatchingSuspended;
 
+  // True iff mDecoder has processed the end of the stream.
+  // Used in lazy decoding to distinguish between having
+  // processed all the bytes but not the EOF and having
+  // processed all the bytes and the EOF.
+  bool mEofDecoded;
+
+  // This flag will be set in `Send()` when we successfully reuse a "fetch"
+  // preload to satisfy this XHR.
+  bool mFromPreload = false;
+
+  // Our parse-end listener, if we are parsing.
+  RefPtr<nsXHRParseEndListener> mParseEndListener;
+
+  XMLHttpRequestDoneNotifier* mDelayedDoneNotifier;
+  void DisconnectDoneNotifier();
+
+  // Any stack information for the point the XHR was opened. This is deleted
+  // after the XHR is opened, to avoid retaining references to the worker.
+  UniquePtr<SerializedStackHolder> mOriginStack;
+
   static bool sDontWarnAboutSyncXHR;
 };
 
-class MOZ_STACK_CLASS AutoDontWarnAboutSyncXHR
-{
-public:
-  AutoDontWarnAboutSyncXHR() : mOldVal(XMLHttpRequestMainThread::DontWarnAboutSyncXHR())
-  {
+class MOZ_STACK_CLASS AutoDontWarnAboutSyncXHR {
+ public:
+  AutoDontWarnAboutSyncXHR()
+      : mOldVal(XMLHttpRequestMainThread::DontWarnAboutSyncXHR()) {
     XMLHttpRequestMainThread::SetDontWarnAboutSyncXHR(true);
   }
 
-  ~AutoDontWarnAboutSyncXHR()
-  {
+  ~AutoDontWarnAboutSyncXHR() {
     XMLHttpRequestMainThread::SetDontWarnAboutSyncXHR(mOldVal);
   }
 
-private:
+ private:
   bool mOldVal;
 };
 
@@ -828,25 +802,22 @@ class nsXMLHttpRequestXPCOMifier final : public nsIStreamListener,
                                          public nsIProgressEventSink,
                                          public nsIInterfaceRequestor,
                                          public nsITimerCallback,
-                                         public nsINamed
-{
+                                         public nsINamed {
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
   NS_DECL_CYCLE_COLLECTION_CLASS_AMBIGUOUS(nsXMLHttpRequestXPCOMifier,
                                            nsIStreamListener)
 
-  explicit nsXMLHttpRequestXPCOMifier(XMLHttpRequestMainThread* aXHR) :
-    mXHR(aXHR)
-  {
-  }
+  explicit nsXMLHttpRequestXPCOMifier(XMLHttpRequestMainThread* aXHR)
+      : mXHR(aXHR) {}
 
-private:
+ private:
   ~nsXMLHttpRequestXPCOMifier() {
     if (mXHR) {
       mXHR->mXPCOMifier = nullptr;
     }
   }
 
-public:
+ public:
   NS_FORWARD_NSISTREAMLISTENER(mXHR->)
   NS_FORWARD_NSIREQUESTOBSERVER(mXHR->)
   NS_FORWARD_NSICHANNELEVENTSINK(mXHR->)
@@ -857,32 +828,53 @@ public:
 
   NS_DECL_NSIINTERFACEREQUESTOR
 
-private:
+ private:
   RefPtr<XMLHttpRequestMainThread> mXHR;
 };
 
-class nsXHRParseEndListener : public nsIDOMEventListener
-{
-public:
+class XMLHttpRequestDoneNotifier : public Runnable {
+ public:
+  explicit XMLHttpRequestDoneNotifier(XMLHttpRequestMainThread* aXHR)
+      : Runnable("XMLHttpRequestDoneNotifier"), mXHR(aXHR) {}
+
+  NS_IMETHOD Run() override {
+    if (mXHR) {
+      RefPtr<XMLHttpRequestMainThread> xhr = mXHR;
+      // ChangeStateToDoneInternal ends up calling Disconnect();
+      xhr->ChangeStateToDoneInternal();
+      MOZ_ASSERT(!mXHR);
+    }
+    return NS_OK;
+  }
+
+  void Disconnect() { mXHR = nullptr; }
+
+ private:
+  RefPtr<XMLHttpRequestMainThread> mXHR;
+};
+
+class nsXHRParseEndListener : public nsIDOMEventListener {
+ public:
   NS_DECL_ISUPPORTS
-  NS_IMETHOD HandleEvent(nsIDOMEvent *event) override
-  {
-    nsCOMPtr<nsIXMLHttpRequest> xhr = do_QueryReferent(mXHR);
-    if (xhr) {
-      static_cast<XMLHttpRequestMainThread*>(xhr.get())->OnBodyParseEnd();
+  NS_IMETHOD HandleEvent(Event* event) override {
+    if (mXHR) {
+      mXHR->OnBodyParseEnd();
     }
     mXHR = nullptr;
     return NS_OK;
   }
-  explicit nsXHRParseEndListener(nsIXMLHttpRequest* aXHR)
-    : mXHR(do_GetWeakReference(aXHR)) {}
-private:
-  virtual ~nsXHRParseEndListener() {}
 
-  nsWeakPtr mXHR;
+  explicit nsXHRParseEndListener(XMLHttpRequestMainThread* aXHR) : mXHR(aXHR) {}
+
+  void SetIsStale() { mXHR = nullptr; }
+
+ private:
+  virtual ~nsXHRParseEndListener() = default;
+
+  XMLHttpRequestMainThread* mXHR;
 };
 
-} // dom namespace
-} // mozilla namespace
+}  // namespace dom
+}  // namespace mozilla
 
-#endif // mozilla_dom_XMLHttpRequestMainThread_h
+#endif  // mozilla_dom_XMLHttpRequestMainThread_h
