@@ -92,6 +92,7 @@ static const char *gNumberElementKeys[DecimalFormatSymbols::kFormatSymbolCount]
     NULL, /* eight digit - get it from the numbering system */
     NULL, /* nine digit - get it from the numbering system */
     "superscriptingExponent", /* Multiplication (x) symbol for exponents */
+    "approximatelySign" /* Approximately sign symbol */
 };
 
 // -------------------------------------
@@ -99,7 +100,7 @@ static const char *gNumberElementKeys[DecimalFormatSymbols::kFormatSymbolCount]
 
 DecimalFormatSymbols::DecimalFormatSymbols(UErrorCode& status)
         : UObject(), locale(), currPattern(NULL) {
-    initialize(locale, status, TRUE);
+    initialize(locale, status, true);
 }
 
 // -------------------------------------
@@ -112,7 +113,7 @@ DecimalFormatSymbols::DecimalFormatSymbols(const Locale& loc, UErrorCode& status
 
 DecimalFormatSymbols::DecimalFormatSymbols(const Locale& loc, const NumberingSystem& ns, UErrorCode& status)
         : UObject(), locale(loc), currPattern(NULL) {
-    initialize(locale, status, FALSE, &ns);
+    initialize(locale, status, false, &ns);
 }
 
 DecimalFormatSymbols::DecimalFormatSymbols()
@@ -167,35 +168,36 @@ DecimalFormatSymbols::operator=(const DecimalFormatSymbols& rhs)
         fIsCustomCurrencySymbol = rhs.fIsCustomCurrencySymbol; 
         fIsCustomIntlCurrencySymbol = rhs.fIsCustomIntlCurrencySymbol; 
         fCodePointZero = rhs.fCodePointZero;
+        currPattern = rhs.currPattern;
     }
     return *this;
 }
 
 // -------------------------------------
 
-UBool
+bool
 DecimalFormatSymbols::operator==(const DecimalFormatSymbols& that) const
 {
     if (this == &that) {
-        return TRUE;
+        return true;
     }
     if (fIsCustomCurrencySymbol != that.fIsCustomCurrencySymbol) { 
-        return FALSE; 
+        return false;
     } 
     if (fIsCustomIntlCurrencySymbol != that.fIsCustomIntlCurrencySymbol) { 
-        return FALSE; 
+        return false;
     } 
     for(int32_t i = 0; i < (int32_t)kFormatSymbolCount; ++i) {
         if(fSymbols[(ENumberFormatSymbol)i] != that.fSymbols[(ENumberFormatSymbol)i]) {
-            return FALSE;
+            return false;
         }
     }
     for(int32_t i = 0; i < (int32_t)UNUM_CURRENCY_SPACING_COUNT; ++i) {
         if(currencySpcBeforeSym[i] != that.currencySpcBeforeSym[i]) {
-            return FALSE;
+            return false;
         }
         if(currencySpcAfterSym[i] != that.currencySpcAfterSym[i]) {
-            return FALSE;
+            return false;
         }
     }
     // No need to check fCodePointZero since it is based on fSymbols
@@ -220,24 +222,24 @@ struct DecFmtSymDataSink : public ResourceSink {
     // Destination for data, modified via setters.
     DecimalFormatSymbols& dfs;
     // Boolean array of whether or not we have seen a particular symbol yet.
-    // Can't simpy check fSymbols because it is pre-populated with defaults.
+    // Can't simply check fSymbols because it is pre-populated with defaults.
     UBool seenSymbol[DecimalFormatSymbols::kFormatSymbolCount];
 
     // Constructor/Destructor
     DecFmtSymDataSink(DecimalFormatSymbols& _dfs) : dfs(_dfs) {
-        uprv_memset(seenSymbol, FALSE, sizeof(seenSymbol));
+        uprv_memset(seenSymbol, false, sizeof(seenSymbol));
     }
     virtual ~DecFmtSymDataSink();
 
     virtual void put(const char *key, ResourceValue &value, UBool /*noFallback*/,
-            UErrorCode &errorCode) {
+            UErrorCode &errorCode) override {
         ResourceTable symbolsTable = value.getTable(errorCode);
         if (U_FAILURE(errorCode)) { return; }
         for (int32_t j = 0; symbolsTable.getKeyAndValue(j, key, value); ++j) {
             for (int32_t i=0; i<DecimalFormatSymbols::kFormatSymbolCount; i++) {
                 if (gNumberElementKeys[i] != NULL && uprv_strcmp(key, gNumberElementKeys[i]) == 0) {
                     if (!seenSymbol[i]) {
-                        seenSymbol[i] = TRUE;
+                        seenSymbol[i] = true;
                         dfs.setSymbol(
                             (DecimalFormatSymbols::ENumberFormatSymbol) i,
                             value.getUnicodeString(errorCode));
@@ -253,10 +255,10 @@ struct DecFmtSymDataSink : public ResourceSink {
     UBool seenAll() {
         for (int32_t i=0; i<DecimalFormatSymbols::kFormatSymbolCount; i++) {
             if (!seenSymbol[i]) {
-                return FALSE;
+                return false;
             }
         }
-        return TRUE;
+        return true;
     }
 
     // If monetary decimal or grouping were not explicitly set, then set them to be the
@@ -281,20 +283,20 @@ struct CurrencySpacingSink : public ResourceSink {
     UBool hasAfterCurrency;
 
     CurrencySpacingSink(DecimalFormatSymbols& _dfs)
-        : dfs(_dfs), hasBeforeCurrency(FALSE), hasAfterCurrency(FALSE) {}
+        : dfs(_dfs), hasBeforeCurrency(false), hasAfterCurrency(false) {}
     virtual ~CurrencySpacingSink();
 
     virtual void put(const char *key, ResourceValue &value, UBool /*noFallback*/,
-            UErrorCode &errorCode) {
+            UErrorCode &errorCode) override {
         ResourceTable spacingTypesTable = value.getTable(errorCode);
         for (int32_t i = 0; spacingTypesTable.getKeyAndValue(i, key, value); ++i) {
             UBool beforeCurrency;
             if (uprv_strcmp(key, gBeforeCurrencyTag) == 0) {
-                beforeCurrency = TRUE;
-                hasBeforeCurrency = TRUE;
+                beforeCurrency = true;
+                hasBeforeCurrency = true;
             } else if (uprv_strcmp(key, gAfterCurrencyTag) == 0) {
-                beforeCurrency = FALSE;
-                hasAfterCurrency = TRUE;
+                beforeCurrency = false;
+                hasAfterCurrency = true;
             } else {
                 continue;
             }
@@ -327,11 +329,13 @@ struct CurrencySpacingSink : public ResourceSink {
         // both beforeCurrency and afterCurrency were found in CLDR.
         static const char* defaults[] = { "[:letter:]", "[:digit:]", " " };
         if (!hasBeforeCurrency || !hasAfterCurrency) {
-            for (UBool beforeCurrency = 0; beforeCurrency <= TRUE; beforeCurrency++) {
-                for (int32_t pattern = 0; pattern < UNUM_CURRENCY_SPACING_COUNT; pattern++) {
-                    dfs.setPatternForCurrencySpacing((UCurrencySpacing)pattern,
-                        beforeCurrency, UnicodeString(defaults[pattern], -1, US_INV));
-                }
+            for (int32_t pattern = 0; pattern < UNUM_CURRENCY_SPACING_COUNT; pattern++) {
+                dfs.setPatternForCurrencySpacing((UCurrencySpacing)pattern,
+                    false, UnicodeString(defaults[pattern], -1, US_INV));
+            }
+            for (int32_t pattern = 0; pattern < UNUM_CURRENCY_SPACING_COUNT; pattern++) {
+                dfs.setPatternForCurrencySpacing((UCurrencySpacing)pattern,
+                    true, UnicodeString(defaults[pattern], -1, US_INV));
             }
         }
     }
@@ -436,7 +440,7 @@ DecimalFormatSymbols::initialize(const Locale& loc, UErrorCode& status,
     sink.resolveMissingMonetarySeparators(fSymbols);
 
     // Resolve codePointZero
-    UChar32 tempCodePointZero;
+    UChar32 tempCodePointZero = -1;
     for (int32_t i=0; i<=9; i++) {
         const UnicodeString& stringDigit = getConstDigitSymbol(i);
         if (stringDigit.countChar32() != 1) {
@@ -453,58 +457,16 @@ DecimalFormatSymbols::initialize(const Locale& loc, UErrorCode& status,
     }
     fCodePointZero = tempCodePointZero;
 
-    // Obtain currency data from the currency API.  This is strictly
-    // for backward compatibility; we don't use DecimalFormatSymbols
-    // for currency data anymore.
+    // Get the default currency from the currency API.
     UErrorCode internalStatus = U_ZERO_ERROR; // don't propagate failures out
     UChar curriso[4];
     UnicodeString tempStr;
     int32_t currisoLength = ucurr_forLocale(locStr, curriso, UPRV_LENGTHOF(curriso), &internalStatus);
     if (U_SUCCESS(internalStatus) && currisoLength == 3) {
-        uprv_getStaticCurrencyName(curriso, locStr, tempStr, internalStatus);
-        if (U_SUCCESS(internalStatus)) {
-            fSymbols[kIntlCurrencySymbol].setTo(curriso, currisoLength);
-            fSymbols[kCurrencySymbol] = tempStr;
-        }
+        setCurrency(curriso, status);
+    } else {
+        setCurrency(nullptr, status);
     }
-    /* else use the default values. */
-
-    //load the currency data
-    UChar ucc[4]={0}; //Currency Codes are always 3 chars long
-    int32_t uccLen = 4;
-    const char* locName = loc.getName();
-    UErrorCode localStatus = U_ZERO_ERROR;
-    uccLen = ucurr_forLocale(locName, ucc, uccLen, &localStatus);
-
-    // TODO: Currency pattern data loading is duplicated in number_formatimpl.cpp
-    if(U_SUCCESS(localStatus) && uccLen > 0) {
-        char cc[4]={0};
-        u_UCharsToChars(ucc, cc, uccLen);
-        /* An explicit currency was requested */
-        LocalUResourceBundlePointer currencyResource(ures_open(U_ICUDATA_CURR, locStr, &localStatus));
-        LocalUResourceBundlePointer currency(
-            ures_getByKeyWithFallback(currencyResource.getAlias(), "Currencies", NULL, &localStatus));
-        ures_getByKeyWithFallback(currency.getAlias(), cc, currency.getAlias(), &localStatus);
-        if(U_SUCCESS(localStatus) && ures_getSize(currency.getAlias())>2) { // the length is 3 if more data is present
-            ures_getByIndex(currency.getAlias(), 2, currency.getAlias(), &localStatus);
-            int32_t currPatternLen = 0;
-            currPattern =
-                ures_getStringByIndex(currency.getAlias(), (int32_t)0, &currPatternLen, &localStatus);
-            UnicodeString decimalSep =
-                ures_getUnicodeStringByIndex(currency.getAlias(), (int32_t)1, &localStatus);
-            UnicodeString groupingSep =
-                ures_getUnicodeStringByIndex(currency.getAlias(), (int32_t)2, &localStatus);
-            if(U_SUCCESS(localStatus)){
-                fSymbols[kMonetaryGroupingSeparatorSymbol] = groupingSep;
-                fSymbols[kMonetarySeparatorSymbol] = decimalSep;
-                //pattern.setTo(TRUE, currPattern, currPatternLen);
-                status = localStatus;
-            }
-        }
-        /* else An explicit currency was requested and is unknown or locale data is malformed. */
-        /* ucurr_* API will get the correct value later on. */
-    }
-        // else ignore the error if no currency
 
     // Currency Spacing.
     LocalUResourceBundlePointer currencyResource(ures_open(U_ICUDATA_CURR, locStr, &status));
@@ -539,7 +501,7 @@ DecimalFormatSymbols::initialize() {
     fSymbols[kPlusSignSymbol] = (UChar)0x002b;          // '+' plus sign
     fSymbols[kMinusSignSymbol] = (UChar)0x2d;           // '-' minus sign
     fSymbols[kCurrencySymbol] = (UChar)0xa4;            // 'OX' currency symbol
-    fSymbols[kIntlCurrencySymbol].setTo(TRUE, INTL_CURRENCY_SYMBOL_STR, 2);
+    fSymbols[kIntlCurrencySymbol].setTo(true, INTL_CURRENCY_SYMBOL_STR, 2);
     fSymbols[kMonetarySeparatorSymbol] = (UChar)0x2e;   // '.' monetary decimal separator
     fSymbols[kExponentialSymbol] = (UChar)0x45;         // 'E' exponential
     fSymbols[kPerMillSymbol] = (UChar)0x2030;           // '%o' per mill
@@ -549,13 +511,59 @@ DecimalFormatSymbols::initialize() {
     fSymbols[kSignificantDigitSymbol] = (UChar)0x0040;  // '@' significant digit
     fSymbols[kMonetaryGroupingSeparatorSymbol].remove(); // 
     fSymbols[kExponentMultiplicationSymbol] = (UChar)0xd7; // 'x' multiplication symbol for exponents
-    fIsCustomCurrencySymbol = FALSE; 
-    fIsCustomIntlCurrencySymbol = FALSE;
+    fSymbols[kApproximatelySignSymbol] = u'~';          // '~' approximately sign
+    fIsCustomCurrencySymbol = false; 
+    fIsCustomIntlCurrencySymbol = false;
     fCodePointZero = 0x30;
     U_ASSERT(fCodePointZero == fSymbols[kZeroDigitSymbol].char32At(0));
+    currPattern = nullptr;
 
 }
 
+void DecimalFormatSymbols::setCurrency(const UChar* currency, UErrorCode& status) {
+    // TODO: If this method is made public:
+    // - Adopt ICU4J behavior of not allowing currency to be null.
+    // - Also verify that the length of currency is 3.
+    if (!currency) {
+        return;
+    }
+
+    UnicodeString tempStr;
+    uprv_getStaticCurrencyName(currency, locale.getName(), tempStr, status);
+    if (U_SUCCESS(status)) {
+        fSymbols[kIntlCurrencySymbol].setTo(currency, 3);
+        fSymbols[kCurrencySymbol] = tempStr;
+    }
+
+    char cc[4]={0};
+    u_UCharsToChars(currency, cc, 3);
+
+    /* An explicit currency was requested */
+    // TODO(ICU-13297): Move this data loading logic into a centralized place
+    UErrorCode localStatus = U_ZERO_ERROR;
+    LocalUResourceBundlePointer rbTop(ures_open(U_ICUDATA_CURR, locale.getName(), &localStatus));
+    LocalUResourceBundlePointer rb(
+        ures_getByKeyWithFallback(rbTop.getAlias(), "Currencies", NULL, &localStatus));
+    ures_getByKeyWithFallback(rb.getAlias(), cc, rb.getAlias(), &localStatus);
+    if(U_SUCCESS(localStatus) && ures_getSize(rb.getAlias())>2) { // the length is 3 if more data is present
+        ures_getByIndex(rb.getAlias(), 2, rb.getAlias(), &localStatus);
+        int32_t currPatternLen = 0;
+        currPattern =
+            ures_getStringByIndex(rb.getAlias(), (int32_t)0, &currPatternLen, &localStatus);
+        UnicodeString decimalSep =
+            ures_getUnicodeStringByIndex(rb.getAlias(), (int32_t)1, &localStatus);
+        UnicodeString groupingSep =
+            ures_getUnicodeStringByIndex(rb.getAlias(), (int32_t)2, &localStatus);
+        if(U_SUCCESS(localStatus)){
+            fSymbols[kMonetaryGroupingSeparatorSymbol] = groupingSep;
+            fSymbols[kMonetarySeparatorSymbol] = decimalSep;
+            //pattern.setTo(true, currPattern, currPatternLen);
+        }
+    }
+    /* else An explicit currency was requested and is unknown or locale data is malformed. */
+    /* ucurr_* API will get the correct value later on. */
+}
+
 Locale
 DecimalFormatSymbols::getLocale(ULocDataLocaleType type, UErrorCode& status) const {
     U_LOCALE_BASED(locBased, *this);
